      MODULE assign_owner
! 27 AUGUST 2000
      USE kinds
      USE model_parameters
      USE domain_decomposition
#ifndef no_pbl
      USE domain_decomposition_sfc   ! used for flux coupler
#endif

      IMPLICIT NONE
      SAVE

      TYPE sbdmn_assign_type
         INTEGER (KIND=int_kind) :: length
         INTEGER (KIND=int_kind), POINTER :: sd(:)
      END TYPE sbdmn_assign_type
      TYPE sbdmn_owner_type
         INTEGER (KIND=int_kind) :: length
         INTEGER (KIND=int_kind), POINTER :: owner(:)
      END TYPE sbdmn_owner_type

#ifdef no_pbl
      INTEGER (KIND=int_kind), parameter :: asgn_level=max_level
#else
      INTEGER (KIND=int_kind), parameter :: asgn_level=max(sfc_level,max_level)
#endif

      INTEGER (KIND=int_kind) :: 
     &                       isdm_level(asgn_level),jsdm_level(asgn_level)

      TYPE (sbdmn_assign_type) :: sbdmn_assign(npe_atmos,asgn_level)
      TYPE (sbdmn_owner_type)  :: sbdmn_owner(asgn_level)

      CONTAINS
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE set_assign_owner (l_error)
! 27 AUGUST 2000
      IMPLICIT NONE
      LOGICAL (KIND=log_kind) :: l_error
      LOGICAL (KIND=log_kind) :: l_exist,l_reorder
      INTEGER (KIND=int_kind) :: level,read_level,task,task_temp,
     &             nsdm_temp,nsd,count,n,ierr,nsdm_sum,iteration,nsd_tmp,
     &             nsdmmax_level, npe_level, nsdm_beg

      l_error = .FALSE.

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! CHECK TO MAKE SURE THAT THE NUMBER OF SUBDOMAINS IS EVENLY DIVISIBLE
! BY THE NUMBER OF PROCESSES
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      IF (MOD (5*isdm*jsdm,npe_atmos)/=0) THEN
         IF (my_task==0) THEN
            PRINT *,' set_assign_owner : ERROR : there are ',
     &                          'a total of ',5*isdm*jsdm,' subdomains.'
            PRINT *,'          this number is not evenly divisible by ',
     &                                          'npe_atmos = ',npe_atmos
         ENDIF
         l_error = .TRUE.
      ENDIF

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! IF THE FILE sbdmn_assign EXISTS THEN READ sbdmn_assign FROM FILE
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      INQUIRE (FILE='./sbdmn_assign',EXIST=l_exist)
      IF (l_exist) THEN
         IF (my_task==0) THEN
            PRINT *,' set_assign_owner : NOTE : sbdmn_assign FILE FOUND'
         ENDIF

         OPEN (UNIT=17,FILE=trim(datapath)//'sbdmn_assign',
     &                        FORM='FORMATTED',STATUS='OLD',IOSTAT=ierr)
         DO level = 1,max_level
            count = 0
            READ (17,10,ADVANCE='YES') 
     &                    read_level,isdm_level(level),jsdm_level(level)
            IF (level/=read_level) THEN
               IF (my_task==0) THEN
                  PRINT *,' set_assign_owner : ERROR : ',
     &                             'level/=read_level at level = ',level
               ENDIF
               l_error = .TRUE.
            ENDIF
            READ (17,'()',ADVANCE='YES')
            nsdm_sum = 0
            DO task = 0,npe_atmos-1
               READ (17,20,ADVANCE='NO') task_temp,nsdm_temp
               nsdm_sum = nsdm_sum + nsdm_temp
               ALLOCATE (sbdmn_assign(task+1,level)%sd(nsdm_temp))
               sbdmn_assign(task+1,level)%length = nsdm_temp
               DO nsd = 1,nsdm_temp
                  count = count + 1
                  READ (17,30,ADVANCE='NO',IOSTAT=ierr) 
     &                                sbdmn_assign(task+1,level)%sd(nsd)
                  IF (ierr/=0) THEN
                     IF (my_task==0) THEN
                        PRINT *,' set_assign_owner : ERROR : ',
     &                    'short list at level = ',level,' task = ',task
                     ENDIF
                     l_error = .TRUE.
                  ENDIF
               ENDDO
               READ (17,'()',ADVANCE='YES')
            ENDDO
            READ (17,'()',ADVANCE='YES')
            IF (nsdm_sum/=5*isdm_level(level)*jsdm_level(level)) THEN
               IF (my_task==0) THEN
                  PRINT *,' set_assign_owner : ERROR : '
                  PRINT *,'         sbdm_sum/=5*isdm_level*jsdm_level ',
     &                                               'at level = ',level
               ENDIF
               l_error = .TRUE.
            ENDIF

            IF (count>5*isdm_level(level)*jsdm_level(level)) THEN
               IF (my_task==0) THEN
                  PRINT *,' set_assign_owner : ERROR : long list ',
     &                                               'at level = ',level
               ENDIF
               l_error = .TRUE.
            ENDIF

         ENDDO

! This code is for the flux coupler - does subdomain owner assignments on
!   grids finer than the atm (max_level) based on the atm grid

#ifndef no_pbl
#if resolution != sfc_resolution
         DO level = max_level+1,asgn_level
            isdm_level(level) = isdm_level(max_level)
            jsdm_level(level) = jsdm_level(max_level)
            nsdm_temp = 
     &              (5*isdm_level(level)*jsdm_level(level))/npe_sfc
            DO task = 0,npe_sfc-1
               ALLOCATE (sbdmn_assign(task+1,level)%sd(nsdm_temp))
               sbdmn_assign(task+1,level)%length = nsdm_temp
               DO nsd = 1,nsdm_temp
                  sbdmn_assign(task+1,level)%sd(nsd) =
     &                sbdmn_assign(task+1,max_level)%sd(nsd)
               ENDDO
            ENDDO
         ENDDO
#endif
#endif
! end flux coupler code

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! OTHERWISE GENERATE sbdmn_assign
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ELSE
         IF (my_task==0) THEN
            PRINT *,' set_assign_owner : NOTE : ',
     &                                     'sbdmn_assign FILE NOT FOUND'
         ENDIF
         DO level = 1,asgn_level
            IF ((level==1).OR.(level==2)) THEN
               isdm_level(level) = 2; jsdm_level(level) = 1
               DO task = 0,npe_atmos-1
                  IF (task==0) THEN
                     ALLOCATE (sbdmn_assign(task+1,level)%sd(10))
                     sbdmn_assign(task+1,level)%length = 10
                     DO nsd = 1,10
                        sbdmn_assign(task+1,level)%sd(nsd) = nsd
                     ENDDO
                  ELSE
                     sbdmn_assign(task+1,level)%length =  0
                  ENDIF
               ENDDO
            ELSE
               if(level == asgn_level) then
                  isdm_level(level) = isdm
                  jsdm_level(level) = jsdm
               else
                  isdm_level(level) = min( 2**(level-2), isdm )
                  jsdm_level(level) = min( 2**(level-3), jsdm )
               endif

! mod to allow for use of only some of the procs at an intermediate
!   level used in the multigrid
               nsdmmax_level = 5*isdm_level(level)*jsdm_level(level)
               nsdm_beg = max(1, nsdmmax_level / npe_atmos)
               do n = nsdm_beg, nsdmmax_level
                  nsdm_temp = n
                  if ( mod(nsdmmax_level, nsdm_temp) == 0  .and.
     *                 nsdmmax_level/nsdm_temp .le. npe_atmos ) exit
               enddo
               npe_level = nsdmmax_level / nsdm_temp
               if(npe_level .lt. npe_atmos) then
                 if(my_task == 0) print *,'At level ',level,' only ',npe_level
     *,                        ' of ',npe_atmos,' procs are used.'
               endif
               count = 0
               DO task = 0,npe_atmos-1
! only allocate sbdmn_assign%sd for procs that will be used
                  if(task .lt. npe_level) then
                     ALLOCATE (sbdmn_assign(task+1,level)%sd(nsdm_temp))
                     sbdmn_assign(task+1,level)%length = nsdm_temp
                     DO nsd = 1,nsdm_temp
                        count = count + 1
                        sbdmn_assign(task+1,level)%sd(nsd) = count
                     ENDDO
                  else
                     sbdmn_assign(task+1,level)%length =  0
                  endif
               ENDDO
            ENDIF
         ENDDO
      ENDIF

      IF ((isdm_level(max_level)/=isdm).OR.
     &                               (jsdm_level(max_level)/=jsdm)) THEN
         IF (my_task==0) THEN
            PRINT *,' set_assign_owner : ERROR : isdm_level(max_level)',
     &                             ' or jsdm_level(max_level) is wrong '
         ENDIF
         l_error = .TRUE.
      ENDIF

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! SORT sbdmn_assign LISTS INTO ASCENDING ORDER
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      l_reorder = .FALSE.
      DO level = 1,asgn_level
         DO task = 0,npe_atmos-1
            DO iteration = 1,sbdmn_assign(task+1,level)%length
               DO nsd = 1,sbdmn_assign(task+1,level)%length-1
                  IF (sbdmn_assign(task+1,level)%sd(nsd)>
     &                        sbdmn_assign(task+1,level)%sd(nsd+1)) THEN
                     l_reorder = .TRUE.
                     nsd_tmp = sbdmn_assign(task+1,level)%sd(nsd)
                     sbdmn_assign(task+1,level)%sd(nsd) = 
     &                              sbdmn_assign(task+1,level)%sd(nsd+1)
                     sbdmn_assign(task+1,level)%sd(nsd+1) = nsd_tmp
                  ENDIF
               ENDDO
            ENDDO
         ENDDO
      ENDDO
      IF ((my_task==0).AND.l_reorder) THEN
         PRINT *,' set_assign_owner : NOTE : sbdmn_assign LIST FOUND IN'
         PRINT *,'      NOT STRICTLY ASCENDING ORDER AND WAS RE-ORDERED'
      ENDIF

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! INITIALIZE sbdmn_owner
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      DO level = 1,asgn_level
         ALLOCATE (sbdmn_owner(level)%
     &                     owner(5*isdm_level(level)*jsdm_level(level)))
         sbdmn_owner(level)%owner(:) = -999
         DO task = 0,npe_atmos-1
            DO nsd = 1,sbdmn_assign(task+1,level)%length
               sbdmn_owner(level)%
     &                  owner(sbdmn_assign(task+1,level)%sd(nsd)) = task
            ENDDO
         ENDDO
         DO n = 1,5*isdm_level(level)*jsdm_level(level)
            IF (sbdmn_owner(level)%owner(n)==-999) THEN
               IF (my_task==0) THEN
                  PRINT *,' set_assign_owner : ERROR : ',
     &                    'sbdmn_owner is monkeyed up at level = ',level
                  PRINT *,'   no process was assigned to subdomain = ',n
               ENDIF
               l_error = .TRUE.
            ENDIF
         ENDDO
      ENDDO

   10 FORMAT (8X,I2,2X,I3,X,I3)
   20 FORMAT (2I5)
   30 FORMAT ( I5)

      END SUBROUTINE set_assign_owner
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      END MODULE assign_owner
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc

