!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

	module compute_mass_flux


!-----------------------------------------------------------------------
! PURPOSE: compute the mass flux at the cell walls. written for explicit
!   time integration.
!-----------------------------------------------------------------------

        use kinds
        use physical_parameters
        use model_parameters
        use grid_metrics
	use operators
	use sigma_coordinate

	implicit none

!**********************************************************************

        contains

!**********************************************************************


!=======================================================================
! START GET_MASS_FLUX
!=======================================================================
        subroutine get_mass_flux(                                     &
                ksdm,                                                 &
                pthick_c,                                             &
                u,                                                    &
                mass_flux_c,                                          &
                div_mass_flux)

!-----------------------------------------------------------------------------
! INTENT (IN)
!-----------------------------------------------------------------------------
        integer (kind=int_kind), intent(in) ::                       &
                ksdm

	real (kind=dbl_kind), intent(in) ::                           &
      		pthick_c        (ncorners,iim,jjm),                   &
                u        (nspace,ncorners,iim,jjm)

!-----------------------------------------------------------------------------
! INTENT (INOUT)
!-----------------------------------------------------------------------------
	real (kind=dbl_kind), intent(inout) ::                       &
      		mass_flux_c   (nspace,ncorners,iim,jjm),             &
      		div_mass_flux                 (iim,jjm)

!-----------------------------------------------------------------------------
! initialize
!-----------------------------------------------------------------------------
	mass_flux_c = c0
	div_mass_flux = c0

!-----------------------------------------------------------------------------
! compute flux, product of mass and velocity
!-----------------------------------------------------------------------------
	mass_flux_c(1,:,:,:) = pthick_c(:,:,:) * u(1,:,:,:)
        mass_flux_c(2,:,:,:) = pthick_c(:,:,:) * u(2,:,:,:)

!-----------------------------------------------------------------------------
! take the divergence of this flux
!-----------------------------------------------------------------------------
        call div_of_a_vector(                                        &
                ipad,                                                &
                iim,                                                 &
                jjm,                                                 &
                ndata,                                               &
                nspace,                                              &
                ncorners,                                            &
                area_inv(:,:,ksdm),                                  &
                v_weights(:,:,:,:,:,ksdm),                           &
                mass_flux_c(:,:,:,:),                                &
                div_mass_flux(:,:) )

	end subroutine get_mass_flux
!=======================================================================
! END OF GET_MASS_FLUX
!=======================================================================

	end module compute_mass_flux

!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
