!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module dissipation_operator

!-----------------------------------------------------------------------
! PURPOSE : an approximation of vector Laplacian
!-----------------------------------------------------------------------

	use kinds
        use physical_parameters
	use model_parameters
	use geodesic_grid_utilities
	use wrap_vertex_data

	implicit none
	save

	private

	integer (kind=int_kind), parameter ::
     *		ntriangles = 3,
     *		norder	   = 2			! norder controls the order of the Laplacian
						! norder = 1 is Laplacian. norder = 2 is del4, ect

!	real (kind = dbl_kind), parameter ::
!    *		visc = 5.0e14_dbl_kind	! coefficient out in front of the diffusion

        real (kind = dbl_kind) ::
     *         normal       ( nspace, 3, ntriangles, ncorners, iim, jjm, nsdm ),
     *         tangent      ( nspace, 3, ntriangles, ncorners, iim, jjm, nsdm ),
     *	       grad_normal  ( nspace,    ntriangles, ncorners, iim, jjm, nsdm )

	integer (kind=int_kind) ::
     *		idata,
     *		icorner,
     *		itriangle,
     *		ispace,
     *		i,j,k,kp1,kp2,km1

	public ::
     *	    init_diss_op,
     *      dissipation


!*************************************************************************

        contains

!*************************************************************************

!======================================================================
! BEGINNING OF INIT_DISS_OP
!======================================================================

        subroutine init_diss_op(ksdm,trans_to_X2, c)

!-----------------------------------------------------------------------
! NOTE: initialization must be called inside a do ksdm=1,nsdm loop
! NOTE: corner data (c) must be vertex wrapped
!-----------------------------------------------------------------------

	implicit none

!-----------------------------------------------------------------------
! INTENT (IN)
!-----------------------------------------------------------------------
	integer (kind = int_kind), intent(in) ::
     *          ksdm

	real (kind = dbl_kind), intent(in) :: 
     *          trans_to_X2 ( r3, r3,      ncorners, iim, jjm),
     *		c           ( r3,          ncorners, iim, jjm)

!-----------------------------------------------------------------------
! LOCAL
!-----------------------------------------------------------------------
        real (kind = dbl_kind) ::
     *          grad_tangent   ( nspace,    ntriangles, ncorners, iim, jjm),
     *          area           (            ntriangles, ncorners, iim, jjm),
     *          grad_area      (                        ncorners, iim, jjm),
     *		corner_data    ( nspace, 3, ntriangles, ncorners, iim, jjm),
     *          corner_data_r3 ( r3    , 3, ntriangles, ncorners, iim, jjm),
     *          corners        ( nspace,    ntriangles, ncorners, iim, jjm),
     *		p              ( r3    , 3                              ),
     *		m1             ( nspace ),
     *		m2             ( nspace )

!-----------------------------------------------------------------------
! init variables
!-----------------------------------------------------------------------
	normal(:,:,:,:,:,:,ksdm) = c0
	tangent(:,:,:,:,:,:,ksdm) = c0
	grad_normal(:,:,:,:,:,ksdm) = c0
	area = c0
	grad_area = c0
	grad_tangent = c0
	corner_data = c0
	corner_data_r3 = c0
	corners = c0
	p = c0
	m1 = c0
	m2 = c0

!-----------------------------------------------------------------------
! gather input corner data into new data structure
!-----------------------------------------------------------------------
 	call gather_corner_vectors(c, r3, corner_data_r3)
	call map_to_tangent_plane(corner_data_r3, trans_to_X2, corner_data)

!-----------------------------------------------------------------------
! compute the area of each triangle and "centroid"
!-----------------------------------------------------------------------
	do j=2,jjm-1
	do i=2,iim-1
	do icorner=1,ncorners
	do itriangle=1,ntriangles
	p(1:2,1) = corner_data(:,1,itriangle,icorner,i,j)
	p(1:2,2) = corner_data(:,2,itriangle,icorner,i,j)
	p(1:2,3) = corner_data(:,3,itriangle,icorner,i,j)
	call compute_triangle_area(p(:,1),p(:,2),p(:,3),
     *                area(itriangle,icorner,i,j))
	call compute_centroid(p(1:2,:),corners(:,itriangle,icorner,i,j))
	enddo
	enddo
	enddo
	enddo

!-----------------------------------------------------------------------
! compute normal and tangent vectors
!-----------------------------------------------------------------------
        do j=2,jjm-1
        do i=2,iim-1
        do icorner=1,ncorners
        do itriangle=1,ntriangles
	do k=1,3
	kp1 = k+1
	km1 = k-1
	if(kp1.eq.4) kp1=1
	if(km1.eq.0) km1=3
	m1(:) = p5*(corner_data(:,km1,itriangle,icorner,i,j) +
     *              corner_data(:,k  ,itriangle,icorner,i,j)   )
        m2(:) = p5*(corner_data(:,kp1,itriangle,icorner,i,j) +
     *              corner_data(:,k  ,itriangle,icorner,i,j)   )
	tangent(:,k,itriangle,icorner,i,j,ksdm) 
     *           = m2 - m1
	normal (1,k,itriangle,icorner,i,j,ksdm) 
     *           = +tangent(2,k,itriangle,icorner,i,j,ksdm)
	normal (2,k,itriangle,icorner,i,j,ksdm) 
     *           = -tangent(1,k,itriangle,icorner,i,j,ksdm)
	enddo
	enddo
	enddo
	enddo
	enddo

!-----------------------------------------------------------------------
! compute area used in gradient operator
!-----------------------------------------------------------------------
        do j=2,jjm-1
        do i=2,iim-1
        do icorner=1,ncorners
	p(1:2,1) = corners(:,1,icorner,i,j)
	p(1:2,2) = corners(:,2,icorner,i,j)
	p(1:2,3) = corners(:,3,icorner,i,j)
	call compute_triangle_area(p(:,1),p(:,2),p(:,3),grad_area(icorner,i,j))
	enddo
	enddo
	enddo

!-----------------------------------------------------------------------
! compute normal and tangent vectors related to gradient operator
!-----------------------------------------------------------------------
        do j=2,jjm-1
        do i=2,iim-1
        do icorner=1,ncorners
	do k=1,3
        kp1 = k+1
        km1 = k-1
        if(kp1.eq.4) kp1=1
        if(km1.eq.0) km1=3
	m1(:) = p5*(corners(:,km1,icorner,i,j)+corners(:,k,icorner,i,j))
	m2(:) = p5*(corners(:,kp1,icorner,i,j)+corners(:,k,icorner,i,j))
	grad_tangent(:,k,icorner,i,j) = m2 - m1
	grad_normal (1,k,icorner,i,j,ksdm) = +grad_tangent(2,k,icorner,i,j)
	grad_normal (2,k,icorner,i,j,ksdm) = -grad_tangent(1,k,icorner,i,j)
	enddo
	enddo
	enddo
	enddo

!-----------------------------------------------------------------------
! build the areas into normal, tangent, and grad_normal
!-----------------------------------------------------------------------
	do j=2,jjm-1
        do i=2,iim-1
        do icorner=1,ncorners
        do k=1,3
	  grad_normal(:,k,icorner,i,j,ksdm) = 
     *		grad_normal(:,k,icorner,i,j,ksdm) 
     *        / grad_area(icorner,i,j)
	do itriangle=1,ntriangles
	  normal(:,k,itriangle,icorner,i,j,ksdm) = 
     *		normal(:,k,itriangle,icorner,i,j,ksdm) 
     *        / area(itriangle,icorner,i,j)
          tangent(:,k,itriangle,icorner,i,j,ksdm) = 
     *          tangent(:,k,itriangle,icorner,i,j,ksdm) 
     *        / area(itriangle,icorner,i,j)
	enddo
	enddo
	enddo
	enddo
	enddo

	end subroutine init_diss_op
!======================================================================
! END OF INIT_DISS_OP
!======================================================================


!======================================================================
! BEGINNING OF DISSIPATION
!======================================================================
	subroutine dissipation(xin,trans_to_X1,trans_to_X2,xout)

	implicit none

!-----------------------------------------------------------------------
! INTENT (IN)
!-----------------------------------------------------------------------
        real (kind = dbl_kind), intent(in) ::
     *		xin     (nspace, ncorners, iim, jjm, nsdm),       
     *          trans_to_X2 (r3, r3, ncorners, iim, jjm, nsdm),  
     *          trans_to_X1 (r3, r3, ncorners, iim, jjm, nsdm)

!-----------------------------------------------------------------------
! INTENT (OUT)
!-----------------------------------------------------------------------
        real (kind = dbl_kind), intent(out) ::
     *          xout (nspace, ncorners, iim, jjm, nsdm)

!-----------------------------------------------------------------------
! LOCAL
!-----------------------------------------------------------------------
	integer (kind=int_kind) :: 
     *		ksdm, icorner, i, j, itriangle, idata

	real (kind = dbl_kind) ::
     *          vel(nspace, 3, ntriangles, ncorners, iim, jjm),
     *          vel_r3(r3,3, ntriangles, ncorners, iim, jjm),
     *		g(r3),h(r3),
     *		work1(ntriangles,ncorners,iim,jjm),
     *		work1_c(nspace,ncorners,iim,jjm),
     *		work2_c(nspace,ncorners,iim,jjm), 
     *          xout_global(r3, ncorners, iim, jjm, nsdm)

!----------------------------------------------------------------------------
! initialize, use xout as workspace
!----------------------------------------------------------------------------
	do ksdm=1,nsdm
        do j=1,jjm
        do i=1,iim
        do icorner=1,ncorners
        do ispace=1,nspace
          xout(ispace,icorner,i,j,ksdm) = xin(ispace,icorner,i,j,ksdm)
        enddo
        enddo
        enddo
        enddo
	enddo

	xout_global = c0
	vel_r3 = c0

!----------------------------------------------------------------------------
! loop over del2 operator norder times
!----------------------------------------------------------------------------
        do k=1,norder

!----------------------------------------------------------------------------
! transform into the global coordinate system
!----------------------------------------------------------------------------
        do ksdm=1,nsdm
        do j=1,jjm
        do i=1,iim
        do icorner=1,ncorners
          h(1:2) = xout(1:2,icorner,i,j,ksdm)
          h(3) = c0
	  g(1) =  + trans_to_X1(1,1,icorner,i,j,ksdm) * h(1)      
     *            + trans_to_X1(1,2,icorner,i,j,ksdm) * h(2)      
     *            + trans_to_X1(1,3,icorner,i,j,ksdm) * h(3)     
          g(2) =  + trans_to_X1(2,1,icorner,i,j,ksdm) * h(1)      
     *            + trans_to_X1(2,2,icorner,i,j,ksdm) * h(2)      
     *            + trans_to_X1(2,3,icorner,i,j,ksdm) * h(3)     
          g(3) =  + trans_to_X1(3,1,icorner,i,j,ksdm) * h(1)      
     *            + trans_to_X1(3,2,icorner,i,j,ksdm) * h(2)      
     *            + trans_to_X1(3,3,icorner,i,j,ksdm) * h(3)     
          xout_global(:,icorner,i,j,ksdm) = g(:)
        enddo
        enddo
        enddo
        enddo

!----------------------------------------------------------------------------
! wrap data
!----------------------------------------------------------------------------
	call wrap_vertices(r3,1,xout_global(1,1,1,1,1))

!----------------------------------------------------------------------------
! loop over subdomains
!----------------------------------------------------------------------------
	do ksdm=1,nsdm

!-----------------------------------------------------------------------
! gather vector data
!-----------------------------------------------------------------------
	call gather_corner_vectors(
     *         xout_global(:,:,:,:,ksdm),r3,vel_r3(:,:,:,:,:,:))

!-----------------------------------------------------------------------
! transform into the local coordinate system
!-----------------------------------------------------------------------
        do j=1,jjm
        do i=1,iim
        do icorner=1,ncorners
	do itriangle=1,ntriangles
	do idata=1,3
	  h(:) = vel_r3(:,idata,itriangle,icorner,i,j)
          g(1) =  + trans_to_X2(1,1,icorner,i,j,ksdm) * h(1)      
     *            + trans_to_X2(1,2,icorner,i,j,ksdm) * h(2)      
     *            + trans_to_X2(1,3,icorner,i,j,ksdm) * h(3)     
          g(2) =  + trans_to_X2(2,1,icorner,i,j,ksdm) * h(1)      
     *            + trans_to_X2(2,2,icorner,i,j,ksdm) * h(2)      
     *            + trans_to_X2(2,3,icorner,i,j,ksdm) * h(3)     
          g(3) =  + trans_to_X2(3,1,icorner,i,j,ksdm) * h(1)      
     *            + trans_to_X2(3,2,icorner,i,j,ksdm) * h(2)      
     *            + trans_to_X2(3,3,icorner,i,j,ksdm) * h(3)     
	  vel(:,idata,itriangle,icorner,i,j) = g(1:2)
	enddo
	enddo
	enddo
	enddo
	enddo

!-----------------------------------------------------------------------
! compute divergence
!-----------------------------------------------------------------------
	call divergence(ksdm, vel, work1)

!-----------------------------------------------------------------------
! compute gradient of divergence
!-----------------------------------------------------------------------
	call gradient(ksdm, work1, work1_c)

!-----------------------------------------------------------------------
! compute curl
!-----------------------------------------------------------------------
	call curl(ksdm, vel, work1)
	
!-----------------------------------------------------------------------
! compute gradient of curl
!-----------------------------------------------------------------------
	call gradient(ksdm, work1, work2_c)

!-----------------------------------------------------------------------
! find k cross grad of the curl
!-----------------------------------------------------------------------
	do j=1,jjm
        do i=1,iim
        do icorner=1,ncorners
        xout(1,icorner,i,j,ksdm) = -work2_c(2,icorner,i,j)
        xout(2,icorner,i,j,ksdm) = +work2_c(1,icorner,i,j)
        enddo
        enddo
        enddo

!----------------------------------------------------------------------------
! add this term to the intent(out) array
!----------------------------------------------------------------------------
        do j=1,jjm
        do i=1,iim
        do icorner=1,ncorners
        do ispace = 1,nspace
        xout(ispace,icorner,i,j,ksdm) = +    xout(ispace,icorner,i,j,ksdm)
     *                                  + work1_c(ispace,icorner,i,j     )
        enddo
        enddo
        enddo
        enddo

	enddo		! do ksdm=1,nsdm
        enddo           ! do k=1,norder

!----------------------------------------------------------------------------
! multiply by the viscosity
!   NOTE: based on the order of the Laplacian, the dissipation is given either
!   as positive or negative sign
!----------------------------------------------------------------------------
	if(mod(norder,2) == 0) then
	  xout = -visc * xout
	else
	  xout = +visc * xout
	endif

	end subroutine dissipation
!======================================================================
! END OF DISSIPATION
!======================================================================


!======================================================================
! BEGINNING OF GATHER_CORNER_VECTORS
!======================================================================
        subroutine gather_corner_vectors(xin, ispace, xout)

	implicit none

	integer (kind=int_kind), intent(in) ::
     *                  ispace

	real (kind=dbl_kind), intent(in) ::
     *			xin     (ispace,                ncorners, iim, jjm)

	real (kind=dbl_kind), intent(out) ::
     *			xout    (ispace, 3, ntriangles, ncorners, iim, jjm)

!-----------------------------------------------------------------------
! initialize intent out
!-----------------------------------------------------------------------
	xout = c0

!-----------------------------------------------------------------------
! loop over both corners of all interior cells
!-----------------------------------------------------------------------
	do j=2,jjm-1
	do i=2,iim-1

!-----------------------------------------------------------------------
! corner 1, triangle 1
!-----------------------------------------------------------------------
	xout(:,1,1,1,i,j) = xin(:,1,i  ,j  )
	xout(:,2,1,1,i,j) = xin(:,2,i  ,j  )
	xout(:,3,1,1,i,j) = xin(:,2,i  ,j+1)

!-----------------------------------------------------------------------
! corner 1, triangle 2
!-----------------------------------------------------------------------
        xout(:,1,2,1,i,j) = xin(:,1,i  ,j  )
        xout(:,2,2,1,i,j) = xin(:,2,i  ,j+1)
        xout(:,3,2,1,i,j) = xin(:,2,i-1,j  )

!-----------------------------------------------------------------------
! corner 1, triangle 3
!-----------------------------------------------------------------------
        xout(:,1,3,1,i,j) = xin(:,1,i  ,j  )
        xout(:,2,3,1,i,j) = xin(:,2,i-1,j  )
        xout(:,3,3,1,i,j) = xin(:,2,i  ,j  )

!-----------------------------------------------------------------------
! corner 2, triangle 1
!-----------------------------------------------------------------------
        xout(:,1,1,2,i,j) = xin(:,2,i  ,j  )
        xout(:,2,1,2,i,j) = xin(:,1,i  ,j  )
        xout(:,3,1,2,i,j) = xin(:,1,i  ,j-1)

!-----------------------------------------------------------------------
! corner 2, triangle 2
!-----------------------------------------------------------------------
        xout(:,1,2,2,i,j) = xin(:,2,i  ,j  )
        xout(:,2,2,2,i,j) = xin(:,1,i  ,j-1)
        xout(:,3,2,2,i,j) = xin(:,1,i+1,j  )

!-----------------------------------------------------------------------
! corner 2, triangle 3
!-----------------------------------------------------------------------
        xout(:,1,3,2,i,j) = xin(:,2,i  ,j  )
        xout(:,2,3,2,i,j) = xin(:,1,i+1,j  )
        xout(:,3,3,2,i,j) = xin(:,1,i  ,j  )

	enddo
	enddo

        end subroutine gather_corner_vectors
!======================================================================
! END OF GATHER_CORNER_VECTORS
!======================================================================



!======================================================================
! BEGINNING OF DIVERGENCE
!======================================================================
        subroutine divergence(ksdm, xin, xout)

        implicit none

	integer (kind=int_kind), intent(in) ::
     *		ksdm

	real (kind=dbl_kind), intent(in) :: 
     *		xin       ( nspace, 3, ntriangles, ncorners, iim, jjm)

	real (kind=dbl_kind), intent(out) ::
     *		xout      (            ntriangles, ncorners, iim, jjm)

	xout = 0.0_dbl_kind
	do j=2,jjm-1
	do i=2,iim-1
	do icorner=1,ncorners
	do itriangle=1,ntriangles

        xout(itriangle, icorner, i, j) =
     *          + ( normal    ( 1, 1, itriangle, icorner, i, j,ksdm) *
     *              xin       ( 1, 1, itriangle, icorner, i, j)   )
     *          + ( normal    ( 2, 1, itriangle, icorner, i, j,ksdm) *
     *              xin       ( 2, 1, itriangle, icorner, i, j)   )
     *          + ( normal    ( 1, 2, itriangle, icorner, i, j,ksdm) *
     *              xin       ( 1, 2, itriangle, icorner, i, j)   )
     *          + ( normal    ( 2, 2, itriangle, icorner, i, j,ksdm) *
     *              xin       ( 2, 2, itriangle, icorner, i, j)   )
     *          + ( normal    ( 1, 3, itriangle, icorner, i, j,ksdm) *
     *              xin       ( 1, 3, itriangle, icorner, i, j)   )
     *          + ( normal    ( 2, 3, itriangle, icorner, i, j,ksdm) *
     *              xin       ( 2, 3, itriangle, icorner, i, j)   )

	enddo
	enddo
	enddo
	enddo

        end subroutine divergence
!======================================================================
! END OF DIVERGENCE
!======================================================================

!======================================================================
! BEGINNING OF CURL 
!======================================================================
        subroutine curl(ksdm, xin, xout)

        implicit none

        integer (kind=int_kind), intent(in) ::
     *          ksdm

        real (kind=dbl_kind), intent(in) ::
     *          xin       ( nspace, 3, ntriangles, ncorners, iim, jjm)

        real (kind=dbl_kind), intent(out) ::
     *          xout      (            ntriangles, ncorners, iim, jjm)

	xout = 0.0_dbl_kind
        do j=2,jjm-1
        do i=2,iim-1
        do icorner=1,ncorners
        do itriangle=1,ntriangles

        xout(itriangle, icorner, i, j) = 
     *          + ( tangent   ( 1, 1, itriangle, icorner, i, j,ksdm) *
     *              xin       ( 1, 1, itriangle, icorner, i, j)   ) 
     *          + ( tangent   ( 2, 1, itriangle, icorner, i, j,ksdm) *
     *              xin       ( 2, 1, itriangle, icorner, i, j)   )
     *          + ( tangent   ( 1, 2, itriangle, icorner, i, j,ksdm) *
     *              xin       ( 1, 2, itriangle, icorner, i, j)   )
     *          + ( tangent   ( 2, 2, itriangle, icorner, i, j,ksdm) *
     *              xin       ( 2, 2, itriangle, icorner, i, j)   )
     *          + ( tangent   ( 1, 3, itriangle, icorner, i, j,ksdm) *
     *              xin       ( 1, 3, itriangle, icorner, i, j)   )
     *          + ( tangent   ( 2, 3, itriangle, icorner, i, j,ksdm) *
     *              xin       ( 2, 3, itriangle, icorner, i, j)   )

        enddo
        enddo
        enddo
        enddo

        end subroutine curl
!======================================================================
! END OF CURL
!======================================================================


!======================================================================
! BEGINNING OF GRADIENT
!======================================================================
        subroutine gradient(ksdm, xin, xout)

        implicit none

        integer (kind=int_kind), intent(in) ::
     *          ksdm

        real (kind=dbl_kind), intent(in) ::
     *          xin       (           ntriangles, ncorners, iim, jjm)

        real (kind=dbl_kind), intent(out) ::
     *          xout      ( nspace,               ncorners, iim, jjm)

	xout = 0.0_dbl_kind
        do j=2,jjm-1
        do i=2,iim-1
        do icorner=1,ncorners
	do ispace=1,nspace

        xout(ispace, icorner, i, j) = 
     *        +  ( grad_normal    (ispace, 1, icorner, i, j,ksdm) *
     *             xin            (        1, icorner, i, j)   ) 
     *        +  ( grad_normal    (ispace, 2, icorner, i, j,ksdm) *
     *             xin            (        2, icorner, i, j)   )
     *        +  ( grad_normal    (ispace, 3, icorner, i, j,ksdm) *
     *             xin            (        3, icorner, i, j)   )

        enddo
        enddo
        enddo
        enddo

        end subroutine gradient
!======================================================================
! END OF GRADIENT
!======================================================================



!======================================================================
! BEGINNING OF COMPUTE_CENTROID
!======================================================================
        subroutine compute_centroid(point,corner)

        implicit none

!----------------------------------------------------------------------------
! INTENT IN
!----------------------------------------------------------------------------
        real (kind = dbl_kind), dimension(nspace,3), intent(in) ::
     *                  point

!----------------------------------------------------------------------------
! INTENT OUT
!----------------------------------------------------------------------------
        real (kind = dbl_kind), dimension(nspace), intent(out) ::
     *                  corner

!----------------------------------------------------------------------------
! LOCAL
!----------------------------------------------------------------------------
	integer (kind=int_kind) :: k,kp1
	real (kind = dbl_kind) :: t(nspace), s(nspace), cross

!----------------------------------------------------------------------------
! compute corner location as the mean of the input data
!----------------------------------------------------------------------------
	corner(:) = (point(:,1)+point(:,2)+point(:,3))/3.0_dbl_kind

!----------------------------------------------------------------------------
! sanity check
!----------------------------------------------------------------------------
        do k=1,3
        kp1=k+1
        if(kp1.eq.4) kp1=1
        t(:) = point(:,kp1) - point(:,k)
        s(:) = corner(:)    - point(:,k)
        cross = t(1)*s(2) - t(2)*s(1)
        if(cross.lt.c0) then
          write(6,*) 'corner lies outside of triangle'
          write(6,*) 'stop in dissipation_operator:compute_centroid'
          write(6,*) point
          write(6,*) corner
          stop
        endif
        enddo

	end subroutine compute_centroid
!======================================================================
! END OF COMPUTE_CENTROID
!======================================================================



!======================================================================
! BEGINNING OF MAP_TO_TANGENT_PLANE
!======================================================================
	subroutine map_to_tangent_plane(cr3, trans_to_X2, cr2)

	implicit none

	real (kind=dbl_kind), intent(in) ::
     *          cr3         ( r3,    3, ntriangles, ncorners, iim, jjm),
     *          trans_to_X2 ( r3, r3,               ncorners, iim, jjm)

	real (kind=dbl_kind), intent(out) ::
     *          cr2         ( nspace,3, ntriangles, ncorners, iim, jjm)

	integer (kind=int_kind) :: 
     *          i,j,itriangle,icorner,idata

	real (kind=dbl_kind) ::
     *          t(r3), s(r3), g(r3), vmag


!-----------------------------------------------------------------------
! initialize
!-----------------------------------------------------------------------
	cr2 = 0.0_dbl_kind

!-----------------------------------------------------------------------
! loop over all real corners
!-----------------------------------------------------------------------
	do j=2,jjm-1
	do i=2,iim-1
	do icorner=1,ncorners
	do itriangle=1,ntriangles
	  s(:) = cr3(:,1,itriangle,icorner,i,j)			! pick the origin
          cr2(:,1,itriangle,icorner,i,j) = 0.0_dbl_kind		! this is the origin
	  do idata=2,3
	    t(:) = cr3(:,idata,itriangle,icorner,i,j)		! pick a neighbor
	    call vector_on_tangent_plane(s(:), t(:), g(:))	! find vector pointing to neighbor
            t(:) = t(:) - s(:)					! vector in r3
            vmag = ( t(1)*t(1) + t(2)*t(2) + t(3)*t(3) )**0.5	! find magnitude
	    g(:) = a * vmag * g(:)				! scale by earth radius
            t(:) = matmul(trans_to_X2(:,:,icorner,i,j),g(:))	! convert into X2
	    cr2(1,idata,itriangle,icorner,i,j) = t(1)
            cr2(2,idata,itriangle,icorner,i,j) = t(2)
	  enddo
	enddo
	enddo
	enddo
	enddo

	end subroutine map_to_tangent_plane
!======================================================================
! END OF MAP_TO_TANGENT_PLANE
!======================================================================


        end module dissipation_operator
!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
