!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module domain_decomposition_sfc


!***********************************************************************
!
!	this module defines how the horizontal domain of the 
!	model surface is decomposed.
!
!	it also contains mpi specific information
!
!	this module is developed with analogy to domain_decomposition
!
!***********************************************************************

!-------------------------------------------------------------------------
!  VARIABLE DEFINITIONS
!
!	iim_sfc		= number of cells in i-direction
!	jjm_sfc		= number of cells in j-direction
!	npe_sfc		= number of processes for AGCM
!	nsdm_sfc	= number of subdomains each process owns
!-------------------------------------------------------------------------

      use kinds
      use domain_decomposition

      implicit none
      save

#if resolution == sfc_resolution
        integer (kind=int_kind), parameter ::
     *                  npe_sfc  = npe_atmos,
     *                  nsdm_sfc = nsdm,
     *                  im_sfc  = im,
     *                  jm_sfc  = jm,
     *                  iim_sfc  = iim,
     *                  jjm_sfc  = jjm,
     *                  iimjjm_sfc = iim_sfc * jjm_sfc,
     *                  nsfc_tot = iim_sfc * jjm_sfc * nsdm_sfc,
     *                  nsfc = nsfc_tot / npe_sfc + 1,
     *                  sfc_level = max_level,
     *                  max_ig_sfc = max_ig,
     *                  im5_sfc = im_sfc/5

#else

#if sfc_resolution == 00642
 	integer (kind=int_kind), parameter :: 
     *			sfc_level = 3,
     *			im_sfc =  90,
     *			jm_sfc = 10,
     *			max_ig_sfc=00642
#endif

#if sfc_resolution == 02562
 	integer (kind=int_kind), parameter :: 
     *			sfc_level = 4,
     *			im_sfc =  170,
     *			jm_sfc = 18,
     *			max_ig_sfc=02562
#endif

#if sfc_resolution == 10242
 	integer (kind=int_kind), parameter :: 
     *			sfc_level = 5,
     *			im_sfc =  330,
     *			jm_sfc = 34,
     *			max_ig_sfc=10242
#endif

#if sfc_resolution == 40962
 	integer (kind=int_kind), parameter :: 
     *			sfc_level = 6,
     *			im_sfc =  650,
     *			jm_sfc = 66,
     *			max_ig_sfc=40962
#endif

        integer (kind=int_kind), parameter ::
     *                  npe_sfc  = npe_atmos,
     *                  nsdm_sfc = nsdm,
     *                  nsfc_tot = im_sfc * jm_sfc * nsdm_sfc,
     *                  nsfc = nsfc_tot / npe_sfc + 1,
     *                  im5_sfc = im_sfc/5,
     *                  iim_sfc = (im5_sfc - 2)/isdm + 2*ipad,
     *                  jjm_sfc = (jm_sfc  - 2)/jsdm + 2*jpad,
     *                  iimjjm_sfc = iim_sfc * jjm_sfc
  
#endif

	logical (kind=log_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) :: 
     *			grid_mask_sfc
        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *			area_sfc

	integer (kind=int_kind), dimension(iim_sfc, jjm_sfc, nsdm_sfc) ::
     *     		grid_indx_sfc
        real(kind=dbl_kind), dimension(iim_sfc, jjm_sfc, nsdm_sfc) ::
     *       grid_center_lon_sfc, grid_center_lat_sfc
        real(kind=dbl_kind), dimension(ncorners, iim_sfc, jjm_sfc, nsdm_sfc) ::
     *       grid_corner_lon_sfc, grid_corner_lat_sfc

#ifdef mpi
      integer (kind=int_kind) ::
     *                   sfc_group,
     *                   MPI_COMM_SFC,
     *			 my_task_sfc

#else
      integer (kind=int_kind), parameter :: my_task_sfc = 0
#endif


!***********************************************************************

      contains

!***********************************************************************


!======================================================================
! BEGINNING OF SPECIFY_MPI_SFC_COMMUNICATOR
!======================================================================
        subroutine specify_mpi_sfc_communicator

#ifdef mpi
!-------------------------------------------------------------------------
! LOCAL
!-------------------------------------------------------------------------
        integer i, pro_ranks(npe_sfc)

!-------------------------------------------------------------------------
! it is assumed that the module domain_decomposition has alread
!	initialized MPI
!-------------------------------------------------------------------------

!-------------------------------------------------------------------------
! define processor rank
!-------------------------------------------------------------------------
        do i = 1,npe_sfc
           pro_ranks (i) = i-1
        enddo

!-------------------------------------------------------------------------
! generate the MPI_COMM_SFC communicator
!-------------------------------------------------------------------------
        call MPI_COMM_GROUP(
     *       MPI_COMM_WORLD,
     *       MPI_GROUP_WORLD,
     *       ierr)

        call MPI_GROUP_INCL(
     *       MPI_GROUP_WORLD,
     *       npe_sfc,
     *       pro_ranks(:),
     *       sfc_group,
     *       ierr)

        call MPI_COMM_CREATE(
     *       MPI_COMM_WORLD,
     *       sfc_group,
     *       MPI_COMM_SFC,
     *       ierr)

        call MPI_COMM_RANK (MPI_COMM_SFC, my_task_sfc,ierr)
!       call MPI_COMM_SIZE (MPI_COMM_SFC, npe_sfc,ierr )
#endif
	end subroutine specify_mpi_sfc_communicator
!======================================================================
! END OF SPECIFY_MPI_SFC_COMMUNICATOR
!======================================================================


	end module domain_decomposition_sfc

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
