!------------------------------------------------------------------------------

module eau_budget

use compute_budgets
use tracer_names
use eau_global
use eau_params
use fluxcoupler_fields, elqpr => dlspr_atm , elqps => dlsps_atm , &
                        cuppr => dcupr_atm , fws   => fws_atm

implicit none

!------------------------------------------------------------------------------
!eau_budgets checks the conservation of total water, i.e. the change in total
!water is equal to the difference between precipitation and evaporation.
!Laura D. Fowler/slikrock (11-01-01).

!send comments to laura@atmos.colostate.edu

!------------------------------------------------------------------------------
contains
!------------------------------------------------------------------------------
subroutine eau_conserve_mass
!------------------------------------------------------------------------------

!argument list variables:

integer (kind=int_kind), parameter:: &
   size_2d = iim*jjm*nsdm,    &
   size_3d = iim*jjm*nlm*nsdm
     
real (kind=dbl_kind):: gravi

!precipitation budget:
real (kind=dbl_kind):: &
   tot_area = c0 , glb_cup  = c0 , glb_elqr = c0 , &
   glb_elqs = c0 , glb_prec = c0 , glb_fws  = c0 , &
   glb_trc  = c0 , glb_res  = c0

!eau_cup budget:
real (kind=dbl_kind):: &
   glbc_iq  = c0 , glbc_icw = c0 , glbc_irw = c0 , &
   glbc_ici = c0 , glbc_iri = c0 , glbc_tot = c0

!eau_liq budget:
real (kind=dbl_kind):: &
   glbl_iq  = c0 , glbl_icw = c0 , glbl_irw = c0 , &
   glbl_ici = c0 , glbl_iri = c0 , glbl_tot = c0 , &
   glb_tot  = c0

real(kind=dbl_kind), dimension(iim,jjm,nsdm) :: &
   one_2d = c1 , area_mask = c0

!------------------------------------------------------------------------------

gravi = c1/grav
area_mask = area * grid_center_mask
tot_area  = global_real_sum(area_mask(:,:,:),one_2d(:,:,:),size_2d)

!write header:

if(my_task.eq.0) write(6,100) tau, step_count

!convective, large-scale, and total precipitations:

glb_cup  = global_real_sum(cuppr(:,:,:),area_mask(:,:,:),size_2d)
glb_elqr = global_real_sum(elqpr(:,:,:),area_mask(:,:,:),size_2d)
glb_elqs = global_real_sum(elqps(:,:,:),area_mask(:,:,:),size_2d)
glb_fws  = global_real_sum(fws(:,:,:)  ,area_mask(:,:,:),size_2d)
      
glb_cup  = glb_cup /tot_area
glb_elqr = glb_elqr/tot_area
glb_elqs = glb_elqs/tot_area
glb_fws  = glb_fws /tot_area
glb_prec = glb_cup+glb_elqr+glb_elqs

glb_trcdiff = glb_trcdiff*gravi/tot_area 
glb_trc     = glb_trcdiff(iq) + glb_trcdiff(icw) + glb_trcdiff(irw) &
            + glb_trcdiff(ici)+ glb_trcdiff(iri)

glb_trcres  = glb_trcres*gravi/tot_area
glb_res     = glb_trcres(iq)  + glb_trcres(icw) + glb_trcres(irw)   &
            + glb_trcres(ici) + glb_trcres(iri)

if(my_task.eq.0) then
   write(6,110) glb_cup, glb_elqr, glb_elqs, glb_prec,glb_fws,      &
                glb_prec-glb_fws
   write(6,111) glb_trcdiff(iq)-glb_trcres(iq),                     &
                glb_trcdiff(icw)-glb_trcres(icw),                   &
                glb_trcdiff(irw)-glb_trcres(irw),                   &
                glb_trcdiff(ici)-glb_trcres(ici),                   &
                glb_trcdiff(iri)-glb_trcres(ici),                   &
                glb_trc-glb_res,glb_prec-glb_fws+glb_trc-glb_res
endif

!eau_cup budget:

glbc_iq  = global_real_sum(deauc_vtot(:,:,iq,:),area_mask(:,:,:) ,size_2d)
glbc_icw = global_real_sum(deauc_vtot(:,:,icw,:),area_mask(:,:,:),size_2d)
glbc_irw = global_real_sum(deauc_vtot(:,:,irw,:),area_mask(:,:,:),size_2d)
glbc_ici = global_real_sum(deauc_vtot(:,:,ici,:),area_mask(:,:,:),size_2d)
glbc_iri = global_real_sum(deauc_vtot(:,:,iri,:),area_mask(:,:,:),size_2d)          
     
glbc_iq  = glbc_iq*gravi/tot_area
glbc_icw = glbc_icw*gravi/tot_area
glbc_irw = glbc_irw*gravi/tot_area
glbc_ici = glbc_ici*gravi/tot_area
glbc_iri = glbc_iri*gravi/tot_area
glbc_tot = glbc_iq  + glbc_icw + glbc_irw &
         + glbc_ici + glbc_iri

if(my_task.eq.0) &
   write(6,112) glbc_iq, glbc_icw, glbc_irw, glbc_ici, glbc_iri, &
                glbc_tot ,glb_cup, glb_cup+glbc_tot 

!eau_liq budget:

glbl_iq  = global_real_sum(deaul_vtot(:,:,iq,:),area_mask(:,:,:) ,size_2d)
glbl_icw = global_real_sum(deaul_vtot(:,:,icw,:),area_mask(:,:,:),size_2d)
glbl_irw = global_real_sum(deaul_vtot(:,:,irw,:),area_mask(:,:,:),size_2d)
glbl_ici = global_real_sum(deaul_vtot(:,:,ici,:),area_mask(:,:,:),size_2d)
glbl_iri = global_real_sum(deaul_vtot(:,:,iri,:),area_mask(:,:,:),size_2d)          

glbl_iq  = glbl_iq*gravi/tot_area
glbl_icw = glbl_icw*gravi/tot_area
glbl_irw = glbl_irw*gravi/tot_area
glbl_ici = glbl_ici*gravi/tot_area
glbl_iri = glbl_iri*gravi/tot_area 

glbl_tot = glbl_iq  + glbl_icw + glbl_irw &
         + glbl_ici + glbl_iri - glbc_icw &
         - glbc_irw - glbc_ici - glbc_iri

if(my_task.eq.0) &
   write(6,113) glbl_iq, glbl_icw, glbl_irw, glbl_ici,  &
                glbl_iri, glbl_tot , glb_elqr+glb_elqs, &
                glb_elqr+glb_elqs+glbl_tot 

!total water budget:

glb_tot = glbc_tot + glbl_tot
      
if(my_task.eq.0) write(6,114) glb_tot, glb_prec, glb_tot+glb_prec

!---- formats:

100  format ( //                                                 &
               19x,' |---------------------------------------|'/ &
               19x,' |        COMPUTING WATER BUDGETS        |'/ &
               19x,' |---------------------------------------|'/ &
               15x,'      TIME       = ', e20.10, ' hours' /     &
               15x,'      STEP_COUNT = ', i8               / )

110  format ( 'GLOBALLY AVERAGED PRECIPITATION '/                         &
               ' cup precipitation             : ',e20.10, ' (kg/m2/s)' / &
               ' elq precipitation of rain     : ',e20.10, ' (kg/m2/s)' / &
               ' elq precipitation of snow     : ',e20.10, ' (kg/m2/s)' / &
               ' total precipitation           : ',e20.10, ' (kg/m2/s)' / &
               ' surface evaporation           : ',e20.10, ' (kg/m2/s)' / &
               ' precipitation - evaporation   : ',e20.10, ' (kg/m2/s)' / &
            )

111  format ( 'GLOBALLY AVERAGED CHANGE IN TOTAL WATER '/                 &
               ' water vapor                   : ',e20.10, ' (kg/m2/s)' / &
               ' cloud water                   : ',e20.10, ' (kg/m2/s)' / &
               ' rain                          : ',e20.10, ' (kg/m2/s)' / &
               ' cloud ice                     : ',e20.10, ' (kg/m2/s)' / &
               ' snow                          : ',e20.10, ' (kg/m2/s)' / &
               ' total water                   : ',e20.10, ' (kg/m2/s)' / &
               ' eau(error)                    : ',e20.10, ' (kg/m2/s)' / &
            )

112  format ( 'EAU_CUP WATER BUDGET: '/                                   &
               ' tendency of water vapor       : ',e20.10, ' (kg/m2/s)' / &
               ' tendency of cloud water       : ',e20.10, ' (kg/m2/s)' / &
               ' tendency of rain              : ',e20.10, ' (kg/m2/s)' / &
               ' tendency of cloud ice         : ',e20.10, ' (kg/m2/s)' / &
               ' tendency of snow              : ',e20.10, ' (kg/m2/s)' / &
               ' total tendency                : ',e20.10, ' (kg/m2/s)' / &
               ' precipitation                 : ',e20.10, ' (kg/m2/s)' / &
               ' total tendency-precipitation  : ',e20.10, ' (kg/m2/s)' / &
            )

113  format ( 'EAU_LIQ WATER BUDGET: '/                                   &
               ' tendency of water vapor       : ',e20.10, ' (kg/m2/s)' / &
               ' tendency of cloud water       : ',e20.10, ' (kg/m2/s)' / &
               ' tendency of rain              : ',e20.10, ' (kg/m2/s)' / &
               ' tendency of cloud ice         : ',e20.10, ' (kg/m2/s)' / &
               ' tendency of snow              : ',e20.10, ' (kg/m2/s)' / &
               ' total tendency                : ',e20.10, ' (kg/m2/s)' / &
               ' precipitation                 : ',e20.10, ' (kg/m2/s)' / &
               ' total tendency-precipitation  : ',e20.10, ' (kg/m2/s)' / &
            )

114  format ( 'TOTAL WATER BUDGET: '/                                     &
               ' total tendency                : ',e20.10, ' (kg/m2/s)' / &
               ' precipitation                 : ',e20.10, ' (kg/m2/s)' / &
               ' total tendency-precipitation  : ',e20.10, ' (kg/m2/s)' / &
            )

end subroutine eau_conserve_mass

!------------------------------------------------------------------------------

end module eau_budget

!------------------------------------------------------------------------------      
