!------------------------------------------------------------------------------
subroutine eau_cfqps &
               ( ksdm , n_phys , nagrid ,    nlm ,  pcos1 , &
                pcos2 ,  psin1 ,  psin2 ,    pl2 ,   plk2 , &
                  plk ,     tl ,    cld ,    clu ,   cldt , &
                 cldw ,   cldi ,   cldm ,  cldpc ,    ncw , &
                   cw ,     ci , effr_w , effr_i ,     pc   &
               )
      
use kinds
use eau_params
use qp_module
      
implicit none
!------------------------------------------------------------------------------

!eau_cfqps computes the 2D and 3D diagnostics related to eau_cf.
!Laura D. Fowler/slikrock (04-15-01).

!send comments to laura@atmos.colostate.edu

!modifications:
!     none.

!subroutines called:
!     qp2sum: Computation of qp2s diagnostics.
!     qp3sum: Computation of qp3s diagnostics.

!argument list variables:
!input arguments:
!----------------

integer (kind=int_kind), intent(in):: &
   ksdm,             &!loop index.
   n_phys,           &!nb of short-time steps per hour. 
   nagrid,           &!length of total domain.
   nlm                !number of layers.

real (kind=dbl_kind), intent(in):: &
   pcos1,            &!cosine of first diurnal harmonic                    (-).
   psin1,            &!sine of second diurnal harmonic                     (-).
   pcos2,            &!cosine of first diurnal harmonic                    (-).
   psin2              !sine of second diurnal harmonic                     (-).

real (kind=dbl_kind), intent(in), dimension(nagrid):: &
   cldpc,            &!incidence of PBL clouds                             (-).
   pc,               &!depth of PBL clouds                                (mb).
   ncw                !droplet concentration of water clouds          (per m3).

real (kind=dbl_kind), intent(in), dimension(nagrid,nlm):: &
   cld,              &!large-scale cloud fraction                          (-).
   clu,              &!convective cloud fraction                           (-).
   cldw,             &!fraction of water clouds                            (-).
   cldi,             &!fraction of ice and snow clouds                     (-).
   cldm,             &!fration of mixed-phase clouds                       (-).
   cldt,             &!total cloud fracton                                 (-).
   plk,              &!(pl/p0)**kapa                                       (-).
   tl,               &!layer temperature                                   (K).
   cw,               &!cloud water mixing ratio                        (kg/kg).
   ci,               &!cloud ice mixing ratio                          (kg/kg).
   effr_w,           &!effective radius of water clouds                   (mu).
   effr_i             !effective radius of ice clouds                     (mu).

real (kind=dbl_kind), intent(in), dimension(nagrid,nlm+1):: &
   pl2,              &!pressure levels                                   (hPa).
   plk2               !(pl2/p0)**kapa                                      (-).     

!local variables:

integer (kind=int_kind):: i, l, nlmm1
real (kind=dbl_kind) :: pc_loc(nagrid) ! stratus cloud base (mb)

real (kind=dbl_kind), parameter:: &
   cldh = 440.e02_dbl_kind, &
   cldl = 680.e02_dbl_kind

real (kind=dbl_kind):: &
   x_phys

real (kind=dbl_kind), dimension(nagrid):: &
   ptop,               &!top of model                                     (Pa).
   cldt2d,             &!vertically integrated cloud fraction              (-).
   cldw2d,             &!vertically integrated fraction of water clouds    (-).
   cldi2d,             &!vertically integrated fraction of ice clouds      (-).
   cldm2d,             &!vertically integrated fraction of mixed clouds    (-). 
   cldhigh,            &!high-level cloud fraction (Top<440hPa)            (-).
   cldmid,             &!middle-level cloud fraction (440hPa<Top<680hPa)   (-).
   cldlow               !low-level cloud fraction (Top>680hPa)             (-).

!------------------------------------------------------------------------------

nlmm1  = nlm-1
x_phys = n_phys

ptop = pl2(1,1)

!miscellaneous calculations needed for diagnostics:

cldt2d(1:nagrid) = c0 ; cldw2d(1:nagrid) = c0
cldi2d(1:nagrid) = c0 ; cldm2d(1:nagrid) = c0

do l = 1, nlm
   do i = 1, nagrid
      cldt2d(i) = max(cldt2d(i),cldt(i,l))
      cldw2d(i) = max(cldw2d(i),cldw(i,l))
      cldi2d(i) = max(cldi2d(i),cldi(i,l))
      cldm2d(i) = max(cldm2d(i),cldm(i,l))
   enddo
enddo
      
cldhigh(1:nagrid) = c0 ; cldmid(1:nagrid)  = c0
cldlow(1:nagrid)  = c0

do l = 1, nlm
   do i = 1, nagrid
      if(pl2(i,l).lt.cldh) cldhigh(i) = max(cldhigh(i),cldt(i,l))
      if(pl2(i,l).ge.cldl) cldlow(i) = max(cldlow(i),cldt(i,l))
      if(pl2(i,l).ge.cldh .and. pl2(i,l).lt.cldl) &
         cldmid(i) = max(cldmid(i),cldt(i,l))
   enddo
enddo

do i = 1,nagrid
   pc_loc(i) = pc(i) + pl2(i,nlm)
enddo

!qp3s diagnostics:

call qp3sum_cld(cld,pc_loc,nagrid,x_phys,iqp3_cld,ksdm,nagrid)

call qp3sum_cld(clu,pc_loc,nagrid,x_phys,iqp3_clu,ksdm,nagrid)

call qp3sum_cld(cldt,pc_loc,nagrid,x_phys,iqp3_cl,ksdm,nagrid)

call qp3sum_cld(cldw,pc_loc,nagrid,x_phys,iqp3_cldw,ksdm,nagrid)

call qp3sum_cld(cldi,pc_loc,nagrid,x_phys,iqp3_cldi,ksdm,nagrid)

call qp3sum_cld(cldm,pc_loc,nagrid,x_phys,iqp3_cldm,ksdm,nagrid)

call qp3sum(effr_w,nagrid,x_phys,iqp3_effrw,ksdm,nagrid)
    
call qp3sum(effr_i,nagrid,x_phys,iqp3_effri,ksdm,nagrid)

!qp2s diagnostics:

call qp2sum(cldhigh,100.0_dbl_kind,iqp2_cldhigh,ksdm,nagrid)
      
call qp2sum(cldmid,100.0_dbl_kind,iqp2_cldmid,ksdm,nagrid)

call qp2sum(cldlow,100.0_dbl_kind,iqp2_cldlow,ksdm,nagrid)
               
call qp2sum(cldt2d,100.0_dbl_kind,iqp2_cldt2d,ksdm,nagrid)

call qp2sum(cldw2d,100.0_dbl_kind,iqp2_cldw2d,ksdm,nagrid)
     
call qp2sum(cldi2d,100.0_dbl_kind,iqp2_cldi2d,ksdm,nagrid)
      
call qp2sum(cldm2d,100.0_dbl_kind,iqp2_cldm2d,ksdm,nagrid)
     
call qp2sum(ncw,1.0_dbl_kind,iqp2_ncw,ksdm,nagrid)     

call qp2sum(cw(:,nlmm1),1000.0_dbl_kind,iqp2_cwlm1,ksdm,nagrid)
      
call qp2sum(effr_w(:,nlmm1),1.0_dbl_kind,iqp2_effrw,ksdm,nagrid)

call qp2sum(ci(:,5),1000.0_dbl_kind,iqp2_cil5,ksdm,nagrid)
      
call qp2sum(effr_i(:,5),1.0_dbl_kind,iqp2_effri,ksdm,nagrid)

end subroutine eau_cfqps

!------------------------------------------------------------------------------
