!------------------------------------------------------------------------------
subroutine eau_cupcke &
               (nlen   , do_phys , nlm , lpbl , dt , &
                cdwork ,     cke                     &
               )
      
use kinds
use eau_params
      
implicit none
!------------------------------------------------------------------------------
!eau_cupcke computes the implicit time stepping of the vertically-integrated
!cumulus energy.
!Laura D. Fowler/slikrock (01-01-00).

!send comments to laura@atmos.colostate.edu

!references:
!Pan (1995) : Ph.D. dissertation.

!subroutines called: none.

!argument list variables:
!input arguments:
!----------------
integer (kind=int_kind), intent(in):: & 
   nlen,             &!length of total domain.
   nlm,              &!number of layers.
   lpbl               !number of layers above the PBL layer(s).
     
logical (kind=log_kind), intent(in), dimension(nlen):: &
   do_phys            !select the non-ghost cells                          (-).

real (kind=dbl_kind), intent(in):: &
   dt                 !time-step                                           (s).

real (kind=dbl_kind), intent(in), dimension(nlen,nlm,nlm):: &
   cdwork             !cloud work function                              (J/kg).

!inout arguments:
!----------------
real (kind=dbl_kind), intent(inout), dimension(nlen,nlm,nlm):: &
   cke                !cumulus kinetic energy                         (Joules).
     
! local variables:
integer (kind=int_kind):: i, l, ll

real (kind=dbl_kind), dimension(lpbl):: &
   alpha              !constant relating cke to masflux                (m4/kg).

real (kind=dbl_kind):: dtalph, temdt, temdti
      
real (kind=dbl_kind), dimension(nlen,nlm,nlm):: sqrt_cke

!------------------------------------------------------------------------------     

!alpha parameter:

alpha(:) = alpham

!time-stepping of vertically integrated cumulus kinetic energy:

temdt  = taudis/(dt+taudis)
temdti = 1./temdt
sqrt_cke(:,:,:) = c0

do i = 1, nlen
   if(do_phys(i)) then
	 
      do l = ncb, lpbl+1       !cloud-base index

         do ll = ltpcup, lpbl  !cloud-top index
            dtalph = dt/sqrt(alpha(ll))
            sqrt_cke(i,ll,l) = (dtalph*cdwork(i,ll,l)) &
                             * (dtalph*cdwork(i,ll,l)) + 4.*temdti*cke(i,ll,l)               
            sqrt_cke(i,ll,l) = dtalph*cdwork(i,ll,l) + sqrt(sqrt_cke(i,ll,l))
            sqrt_cke(i,ll,l) = 0.5*temdt*sqrt_cke(i,ll,l)
            cke(i,ll,l)      = sqrt_cke(i,ll,l)*sqrt_cke(i,ll,l)
            cke(i,ll,l)      = max(cke(i,ll,l),ckemin)
         enddo

      enddo      

   endif
enddo

!write(6,*) '--- end eau_cupcke'

end subroutine eau_cupcke

!------------------------------------------------------------------------------
