!------------------------------------------------------------------------------    
subroutine eau_cupcld &
               (     jjj ,     nlen ,      len ,      nlm ,     lpbl , &
                    dotv ,      plg ,       zl ,       tl ,       ql , &
                    qcwl ,     qrwl ,     qcil ,     qril ,       qs , &
                      hh ,      hhs ,     gaml ,    wghtl ,    plgl2 , &
                     zl2 ,      tl2 ,      ql2 ,    qcwl2 ,    qrwl2 , &
                   qcil2 ,    qril2 ,     hhl2 ,    hhsl2 ,    gaml2 , &
                   hbase ,   wvbase ,   cwbase ,   rwbase ,   cibase , &
                  ribase ,    nunst ,   ndxlst ,      tvl ,     tvl2 , &
                  hcloud ,  qvcloud ,  qlcloud ,  qrcloud ,  qicloud , &
                 qscloud , dqhcloud , dqvcloud , dqlcloud , dqrcloud , &
                dqicloud , dqscloud ,   pprodr ,   cdwork ,     eta2 , &
                  nunst2 ,    ndxl2 ,     cape ,    lamda , inc_diag , &
                     dzm                                               &
#ifdef passive_tracers
                , zzctop                                               &
#endif
               )

use kinds
use eau_params

implicit none
!------------------------------------------------------------------------------
!eau_cupcld is the cloud model for the eau_cup parameterization of convection.
!Laura D. Fowler/mcbug (03-01-01).
!send comments to laura@atmos.colostate.edu

!references:
!Fowler, L.D., and D.A. Randall, 1999: Interactions between cloud microphysics
!   and cumulus convection in a general circulation model, 2002: J. Atmos. Sci.

!modifications:
!none.

!subroutines called:
!none.

!argument list variables:
!input arguments:
!----------------
logical (kind=log_kind), intent(in):: &
   dotv               !if true, the virtual cloud temperature is computed using
                      !the total water mixing ratio. If false, the virtual tem-
                      !perature is computed using the water vapor mixing ratio.

integer (kind=int_kind), intent(in):: &
   jjj,              &!cloud base index.
   nlen,             &!length of total domain.
   len,              &!length of sub- domain.
   nlm,              &!number of layers.
   lpbl,             &!number of layers above the pbl top.
   nunst              !number of unstable cloud base.

integer (kind=int_kind), intent(in), dimension(len):: &
   ndxlst             !grid index of unstable cloud bases.
     
real (kind=dbl_kind), intent(in), dimension(nlen,nlm):: &
   plg,              &!odd-level pressure                                 (Pa).
   ql,               &!odd-level water vapor mixing ratio              (kg/kg).
   qcwl,             &!odd-level cloud water mixing ratio              (kg/kg).
   qrwl,             &!odd-level rain mixing ratio                     (kg/kg).
   qcil,             &!odd-level cloud ice mixing ratio                (kg/kg).
   qril,             &!odd-level snow mixing ratio                     (kg/kg).
   qs,               &!odd-level saturation mixing ratio               (kg/kg).
   hh,               &!odd-level moist static energy                    (J/kg).
   hhs                !odd-level moist static energy                    (J/kg).

real (kind=dbl_kind), intent(in), dimension(nlen,nlm+1):: &
   plgl2,            &!even-level pressure                                (Pa).
   ql2,              &!even-level water vapor mixing ratio             (kg/kg).
   qcwl2,            &!even-level cloud water mixing ratio             (kg/kg).
   qrwl2,            &!even-level rain mixing ratio                    (kg/kg).
   qcil2,            &!even-level cloud ice mixing ratio               (kg/kg).
   qril2,            &!even-level snow mixing ratio                    (kg/kg).
   tl2,              &!even-level temperature                              (K).
   hhl2               !even-level moist static energy                   (J/kg).

real (kind=dbl_kind), intent(in), dimension(len,nlm):: &
   tl,               &!odd-level large-scale temperature                   (K).    
   gaml,             &!odd-level (hltm/cp)*dqs/dT                          (-).
   wghtl,            &!odd-level temperature weight                        (K).
   tvl                !odd-level virtual temperature                       (K).

real (kind=dbl_kind), intent(inout), dimension(len,nlm):: &
   zl                 !odd-level height                                    (m).

real (kind=dbl_kind), intent(in), dimension(len,nlm+1):: &
   hhsl2,            &!even-level saturatio moist static energy         (J/kg).
   zl2,              &!even-level height                                   (m).
   tvl2,             &!even-level virtual temperature                      (K).     
   gaml2              !even-level (hltm/cp)*dqs/dT                         (-).

real (kind=dbl_kind), intent(in), dimension(len):: &
   hbase,            &!cloud-base moist static energy                   (J/kg).
   wvbase,           &!cloud-base water vapor mixing ratio             (kg/kg).
   cwbase,           &!cloud-base cloud water mixing ratio             (kg/kg).
   rwbase,           &!cloud-base rain mixing ratio                    (kg/kg).
   cibase,           &!cloud-base cloud ice mixing ratio               (kg/kg).
   ribase,           &!cloud-base snow mixing ratio                    (kg/kg).
   dzm                !PBL depth                                           (m).

!output arguments:
!-----------------
integer (kind=int_kind), intent(out):: &
   nunst2             !number of cumulus ensembles                         (-).
      
integer (kind=int_kind), intent(out), dimension(len*nlm,2):: &
   ndxl2              !index of cloud-top location of cumulus ensemble     (-).
     
real (kind=dbl_kind), intent(out), dimension(len,nlm-1,nlm-1):: &
   dqhcloud,         &!in-cloud tendency of moist static energy         (J/kg).
   dqvcloud,         &!in-cloud tendency of water vapor mixing ratio   (kg/kg).
   dqlcloud,         &!in-cloud tendency of cloud water mixing ratio   (kg/kg).
   dqrcloud,         &!in-cloud tendency of rain mixing ratio          (kg/kg).
   dqicloud,         &!in-cloud tendency of cloud ice mixing ratio     (kg/kg).
   dqscloud,         &!in-cloud tendency of snow mixing ratio          (kg/kg).
   pprodr             !in-cloud tendency of precipitation of rain      (kg/kg).

real (kind=dbl_kind), intent(out), dimension(len,nlm,nlm-1):: &
   hcloud,           &!in-cloud moist static energy                     (J/kg).
   qvcloud,          &!in-cloud water vapor mixing ratio               (kg/kg).
   qlcloud,          &!in-cloud cloud water mixing ratio               (kg/kg).
   qrcloud,          &!in-cloud rain mixinx ratio                      (kg/kg).
   qicloud,          &!in-cloud cloud ice mixing ratio                 (kg/kg).
   qscloud            !in-cloud snow mixing ratio                      (kg/kg).

real (kind=dbl_kind), intent(out), dimension(len,nlm,nlm):: &
   eta2               !normalized cloud-base mass flux                     (-).

real (kind=dbl_kind), intent(out), dimension(nlen):: &    
   cape               !conditional available potential energy.
      
real (kind=dbl_kind), intent(out), dimension(nlen,nlm-1):: &
   lamda              !fractional entrainment rate                     (per m).

real (kind=dbl_kind), intent(out), dimension(nlen,nlm):: &
   cdwork             !cloud work function.
     
real (kind=dbl_kind), intent(out), dimension(nlen,16):: &
   inc_diag

#ifdef passive_tracers
real (kind=dbl_kind), intent(out), dimension(len,nlm-1):: &
#else
real (kind=dbl_kind), dimension(len,nlm-1):: &
#endif
   zzctop             !height of cloud-top                                 (m).

!local variables:

logical (kind=log_kind), dimension(len,nlm):: alogic

integer (kind=int_kind):: &
   i, ilon, l, lpblp1, k, k0, knum, kpp1, mm, n, nclum1, nclum2
integer (kind=int_kind), dimension(len*nlm,2):: &
   nclust1,nclust2

real (kind=dbl_kind):: &
   alammax = 0.0015_dbl_kind, &
   cprec0  = 2.e-03_dbl_kind, &
   detsnowfac = 0.000_dbl_kind  ! 0.0 to 1.0, fraction of detsnow
                                ! converted to precip

real (kind=dbl_kind):: cpi, cprec, gkq, gkt, tem, zaa, zk, zkb1, zkb2, &
                       cimean, cwmean, hhmean, wvmean, zzmean, w_cloud

real (kind=dbl_kind), dimension(2):: all
data all /0._dbl_kind,0.00000016_dbl_kind/      

real (kind=dbl_kind), dimension(len):: aa1,zb,zzkm
real (kind=dbl_kind), dimension(len,nlm):: bb1,cc1
real (kind=dbl_kind), dimension(len,nlm,2):: h3t     

real (kind=dbl_kind), dimension(len,nlm-1,nlm-1):: &
   pprods,           &!in-cloud tendency of precipitation of snow      (kg/kg).
   dqwfcloud,        &!in-cloud tendency of cloud water due to freezing(kg/kg).
   dqimcloud          !in-cloud tendency of cloud ice due to melting   (kg/kg).
     
real (kind=dbl_kind), dimension(len,nlm,nlm-1):: &
   qsat,             &!in-cloud saturation mixing ratio                (kg/kg).
   tcloud,           &!in-cloud temperature                                (K).
   tvcloud            !in-cloud virtual temperature                        (K).

real (kind=dbl_kind), dimension(len) :: lamdamin  

!==============================================================================
!initialization of local arrays:
!==============================================================================
!write(6,*) '--- enter eau_cupcld:'

lpblp1 = lpbl+1
cpi    = c1/spec_heat_cp

dqwfcloud = c0 ; dqimcloud = c0
pprods    = c0 ; qsat      = c0
tcloud    = c0 ; tvcloud   = c0

alogic  = .false.
nclum1  = 0 ; nclum2  = 0
nclust1 = 0 ; nclust2 = 0
      
if(jjj == lpblp1) then
   lamdamin(:) = 0.2_dbl_kind / dzm(:)
else
   lamdamin(:) = 0.0002_dbl_kind
endif
!lamdamin = c0     ! comment this statement to enable lamdamin code

#ifndef passive_tracers
    zzctop  = 0.
#endif

!==============================================================================
!calculate the fractional entrainment rate (lamda): all(1) and all(2) are
!              arbitrary entrainment rates.
!==============================================================================

do ilon = 1, nunst
   n = ndxlst(ilon)
   zb(n)        = zl2(n,jjj)
   zzkm(n)      = zl(n,jjj)
   zl(n,jjj)    = zb(n)
   aa1(n)       = hbase(n)
   h3t(n,jjj,1) = hbase(n)
   h3t(n,jjj,2) = hbase(n)
enddo

do i = jjj-1, ltpcup, -1
   do mm = 1, 2
      do ilon = 1, nunst
         n = ndxlst(ilon)
         h3t(n,i,mm) = (h3t(n,i+1,mm)*(1.+all(mm)*(zl(n,i+1)- zb(n))) &
                     + all(mm)*hhl2(n,i+1)*(zl(n,i)-zl(n,i+1)))       &
                     / (1.+all(mm)*(zl(n,i)-zb(n)))
      enddo
   enddo
       
   do ilon = 1, nunst
      n = ndxlst(ilon)
      bb1(n,i) = (h3t(n,i,2)*(1.+all(2)*(zl(n,i)-zb(n)))-aa1(n)) / all(2)
   enddo
enddo

do i = jjj-1, ltpcup, -1
   do ilon = 1, nunst
      n = ndxlst(ilon)
      cc1(n,i) = hhs(n,i)-delta*tl(n,i)*(qs(n,i)-ql(n,i)) &
               * (1.+gaml(n,i))/(cpi+delta*tl(n,i)*gaml(n,i)/hltm)

      tem = bb1(n,i)-(zl(n,i)-zb(n))*cc1(n,i)
      if(tem .eq. c0) then
         lamda(n,i) = 2.*alammax
      else
         lamda(n,i) = (cc1(n,i)-aa1(n)) / tem
      endif           
   enddo
enddo

if(jjj.eq.lpblp1) then
   do ilon = 1, nunst
      n = ndxlst(ilon)
      do i = jjj-1, ltpcup, -1
         if(lamda(n,i) .ge. c0) inc_diag(n,3) = inc_diag(n,3) + c1
      enddo
   enddo
endif
do ilon = 1, nunst
   n = ndxlst(ilon)
   do i = jjj-1, ltpcup, -1
      if(lamda(n,i) .ge. c0) inc_diag(n,4) = inc_diag(n,4) + c1
   enddo
enddo

!==============================================================================
!cloud tests using the entrainment rate lamda:
!==============================================================================

do ilon = 1, nunst
      
!get the lowest level of entrainment rate (kpp1 refers to an odd-level):      
   n = ndxlst(ilon)
   kpp1 = 0
   do k = jjj-1, ltpcup, -1
      if(lamda(n,k) >= lamdamin(n)) then
         kpp1 = k
         alogic(n,k) = lamda(n,k) < alammax
         goto 661
      endif
   enddo
   if(kpp1 .eq. 0) then
      zl(n,jjj)   = zzkm(n)
      goto 662
   endif
661 continue

!check the entrainment rate decreases with cloud-top height:

   do k = kpp1-1, ltpcup, -1
!     alogic(n,k) = lamda(n,k).lt.lamda(n,k+1) .and. lamda(n,k).gt.c0
      alogic(n,k) = lamda(n,k).lt.lamda(n,k+1) .and. lamda(n,k).gt.lamdamin(n)
      if(lamda(n,k) .ge. alammax) alogic(n,k) = .false.
      do k0 = k+1, kpp1
!        if(lamda(n,k) .ge. lamda(n,k0) .and. lamda(n,k0).gt.c0) & 
         if(lamda(n,k) .ge. lamda(n,k0) .and. lamda(n,k0).gt.lamdamin(n)) &
            alogic(n,k) = .false.
      enddo
   enddo         

!entrainment equals zero:

   do k = ltpcup+1, kpp1
      if(lamda(n,k-1).le.c0 .and. lamda(n,k).gt.lamdamin(n)) then
         alogic(n,k-1) = .true.
!        if(k-1 .lt. ltpcup) alogic(n,k-1) = .false.
         goto 664
      endif
   enddo
664 continue

!reassign cloud longitude (n) and top index (k): 

   do k = kpp1, ltpcup, -1
      if(alogic(n,k)) then
         if(lamda(n,k).ge.c0) then
            nclum1 = nclum1+1
            nclust1(nclum1,1) = k
            nclust1(nclum1,2) = n
         else
            nclum2=nclum2+1
            nclust2(nclum2,1) = k
            nclust2(nclum2,2) = n
         endif
      endif
   enddo

  662 continue 
enddo   !end ilon loop

if(jjj.eq.lpblp1) then
   do ilon = 1, nunst
      n = ndxlst(ilon)
      do k = jjj-1, ltpcup, -1
         if(alogic(n,k)) inc_diag(n,5) = inc_diag(n,5) + c1
      enddo
   enddo
endif
do ilon = 1, nunst
   n = ndxlst(ilon)
   do k = jjj-1, ltpcup, -1
      if(alogic(n,k)) inc_diag(n,6) = inc_diag(n,6) + c1
   enddo
enddo

!==============================================================================
!in-cloud moist static energy:
!==============================================================================

!case when the entrainment rate is positive:

do knum = 1, nclum1
   k = nclust1(knum,1)    !cloud-top index
   n = nclust1(knum,2)    !cloud-top location index
   zzctop(n,k)  = zl(n,k) !height of the cloud-top

   hcloud(n,jjj,k)  = hbase(n)     
   do i = jjj-1, k+1, -1            
      zk   = zl2(n,i) - zl2(n,i+1)
      zkb1 = zl2(n,i) - zb(n)
      zkb2 = zl2(n,i+1) - zb(n)
            
      hcloud(n,i,k) = ((1.0+lamda(n,k)*zkb2)*hcloud(n,i+1,k) &
                    + lamda(n,k)*hh(n,i)*zk) / (1.0+lamda(n,k)*zkb1)
   enddo

   zk     = zzctop(n,k) - zl2(n,k+1)
   zkb1   = zzctop(n,k) - zb(n)
   zkb2   = zl2(n,k+1) - zb(n)
   hhmean = 0.5_dbl_kind*(hh(n,k) + hhl2(n,k+1))
   hcloud(n,k,k) = ((1.0+lamda(n,k)*zkb2)*hcloud(n,k+1,k)    &
                 + lamda(n,k)*hhmean*zk+amiu                 &
                 * lamda(n,k)*(zl(n,k)-zb(n))*hhmean)        &
                 / (1.0+lamda(n,k)*zkb1                      &
                 + amiu*lamda(n,k)*(zl(n,k)-zb(n)))
enddo !(end loop nclum1)

!case when the entrainment rate is equal to zero:

do knum = 1, nclum2
   k = nclust2(knum,1) !cloud-top index
   n = nclust2(knum,2) !cloud-top location index

   zaa = (zl(n,k)*lamda(n,k+1)-lamda(n,k)*zl(n,k+1))         &
       / (lamda(n,k+1)-lamda(n,k))
   zzctop(n,k) = zaa
   zzmean      = zl2(n,k+1)
            
   if(zaa .ge. zzmean) then
      lamda(n,k) = c0
      hcloud(n,jjj,k)  = hbase(n)
      do i = jjj-1, k, -1
         hcloud(n,i,k) = hbase(n)
      enddo
   else
      alogic(n,k) = .false.
!     if(alogic(n,k+1)) then
!        goto 222
!     else
!        alogic(n,k+1) = .true.
!        lamda(n,k+1)  = c0
!        zzctop(n,k+1) = zaa
!        do i = jjj, k+1, -1
!           hcloud(n,i,k+1) = hbase(n)
!        enddo
!     endif
      if(.not. alogic(n,k+1)) then
         alogic(n,k+1) = .true.
         lamda(n,k+1)  = c0
         zzctop(n,k+1) = zaa
         do i = jjj, k+1, -1
            hcloud(n,i,k+1) = hbase(n)
         enddo
      endif
   endif   
enddo !(end loop nclum2)    

!reassign cloud longitude (n) and top index (k):

do ilon = 1, nunst
   n = ndxlst(ilon)
   do k = ltpcup, jjj-1
      if(alogic(n,k)) then
         nunst2 = nunst2+1
         ndxl2(nunst2,1) = k
         ndxl2(nunst2,2) = n
      endif
   enddo
enddo         

if(jjj.eq.lpblp1) then
   do ilon = 1, nunst
      n = ndxlst(ilon)
      do k = jjj-1, ltpcup, -1
         if(alogic(n,k)) inc_diag(n,7) = inc_diag(n,7) + c1
      enddo
   enddo
endif
do ilon = 1, nunst
   n = ndxlst(ilon)
   do k = jjj-1, ltpcup, -1
      if(alogic(n,k)) inc_diag(n,8) = inc_diag(n,8) + c1
   enddo
enddo

!==============================================================================
!in cloud water budget:
!==============================================================================

do knum = 1, nunst2
   k = ndxl2(knum,1) !cloud-top index
   n = ndxl2(knum,2) !cloud-top location index

!cloud base:

   qvcloud(n,jjj,k) = wvbase(n)
   qlcloud(n,jjj,k) = cwbase(n)
   qicloud(n,jjj,k) = cibase(n)
   qscloud(n,jjj,k) = ribase(n)         
   tcloud(n,jjj,k)  = hcloud(n,jjj,k)-hltm*qvcloud(n,jjj,k)     &  
                    + lfus*(qicloud(n,jjj,k)+qscloud(n,jjj,k))   &
                    - grav*zl2(n,jjj)
   tcloud(n,jjj,k)  = tcloud(n,jjj,k)*cpi
!  write(6,111) jjj,k,jjj,alogic(n,k),tcloud(n,jjj,k),qvcloud(n,jjj,k), &
!               qlcloud(n,jjj,k),qicloud(n,jjj,k),qscloud(n,jjj,k)
   
!  if(tcloud(n,jjj,k) < t00) then
!     tcloud(n,jjj,k)  = tcloud(n,jjj,k) + lfus/spec_heat_cp*qlcloud(n,jjj,k)
!     qicloud(n,jjj,k) = qicloud(n,jjj,k) + qlcloud(n,jjj,k)
!     qlcloud(n,jjj,k) = c0
!  elseif(tcloud(n,jjj,k) >= t00) then
!     tcloud(n,jjj,k)  = tcloud(n,jjj,k) - lfus/spec_heat_cp*qlcloud(n,jjj,k)
!     qlcloud(n,jjj,k) = qlcloud(n,jjj,k) + qicloud(n,jjj,k)
!     qicloud(n,jjj,k) = c0
!  endif   
!  write(6,111) jjj,k,jjj,alogic(n,k),tcloud(n,jjj,k),qvcloud(n,jjj,k), &
!               qlcloud(n,jjj,k),qicloud(n,jjj,k),qscloud(n,jjj,k)
!  write(6,*)

!other cloud levels:

   do i = jjj-1, k+1, -1            
      zk   = zl2(n,i)-zl2(n,i+1)
      zkb1 = zl2(n,i)-zb(n)
      zkb2 = zl2(n,i+1)-zb(n)
            
      qvcloud(n,i,k) = ((1.0+lamda(n,k)*zkb2)*qvcloud(n,i+1,k)   &
                     + lamda(n,k)*ql(n,i)*zk)                    &
                     / (1.0+lamda(n,k)*zkb1)      
      qlcloud(n,i,k) = ((1.0+lamda(n,k)*zkb2)*qlcloud(n,i+1,k)   &
                     + lamda(n,k)*qcwl(n,i)*zk)                  &
                     / (1.0+lamda(n,k)*zkb1)
      qicloud(n,i,k) = ((1.0+lamda(n,k)*zkb2)*qicloud(n,i+1,k)   &
                     + lamda(n,k)*qcil(n,i)*zk)                  &
                     / (1.0+lamda(n,k)*zkb1)     
      qscloud(n,i,k) = ((1.0+lamda(n,k)*zkb2)*qscloud(n,i+1,k))  &
                     / (1.0+lamda(n,k)*zkb1)

!in-cloud condensation:

      gkq = gaml(n,i)/((1+gaml(n,i))*hltm)
      gkt = cpi/(1+gaml2(n,i))           
      qsat(n,i,k) = qs(n,i) + gkq*(hcloud(n,i,k)-hhs(n,i)        &
                  + lfus*(qicloud(n,i,k)+qscloud(n,i,k)))
            
      dqvcloud(n,i,k) = (qvcloud(n,i,k)-qsat(n,i,k))/(1+gaml(n,i))
      if(dqvcloud(n,i,k).ge.c0) then
         tcloud(n,i,k)  = tl2(n,i)                               &
                        + gkt*(hcloud(n,i,k)-hhsl2(n,i)          &
                        + lfus*(qicloud(n,i,k)+qscloud(n,i,k)))
      else   
         tcloud(n,i,k)  = hcloud(n,i,k)-hltm*qvcloud(n,i,k)     &
                        - grav*zl2(n,i)                          &
                        + lfus*(qicloud(n,i,k)+qscloud(n,i,k)) 
         tcloud(n,i,k)  = tcloud(n,i,k)*cpi         
         alogic(n,k) = .false.
      endif   
!     write(6,111) jjj,k,i,alogic(n,k),tcloud(n,i,k),qvcloud(n,i,k), &
!                  qlcloud(n,i,k),qicloud(n,i,k),qscloud(n,i,k)

      if(alogic(n,k)) then
!in-cloud melting/freezing:
         if(tcloud(n,i,k) < t00) then
            w_cloud = c0
            tcloud(n,i,k)    = tcloud(n,i,k) +lfus/spec_heat_cp*qlcloud(n,i,k)
            dqwfcloud(n,i,k) = qlcloud(n,i,k)
         elseif(tcloud(n,i,k) >= t00) then
            w_cloud = c1
            tcloud(n,i,k)    = tcloud(n,i,k) -lfus/spec_heat_cp*qicloud(n,i,k)
            dqimcloud(n,i,k) = qicloud(n,i,k)
         endif 

         dqlcloud(n,i,k) = w_cloud*dqvcloud(n,i,k)           &
                         - dqwfcloud(n,i,k)+dqimcloud(n,i,k)
         dqicloud(n,i,k) = (c1-w_cloud)*dqvcloud(n,i,k)   & 
                         + dqwfcloud(n,i,k)-dqimcloud(n,i,k)
         qlcloud(n,i,k)  = qlcloud(n,i,k) + dqlcloud(n,i,k)
         qicloud(n,i,k)  = qicloud(n,i,k) + dqicloud(n,i,k)
         qvcloud(n,i,k)  = qvcloud(n,i,k) - dqvcloud(n,i,k)

!precipitation:
         cprec = cprec0*zk/(1+cprec0*zk)
         pprodr(n,i,k)   = cprec*qlcloud(n,i,k)
         pprods(n,i,k)   = cprec*qicloud(n,i,k)
         dqlcloud(n,i,k) = dqlcloud(n,i,k) - cprec*qlcloud(n,i,k)
         dqicloud(n,i,k) = dqicloud(n,i,k) - cprec*qicloud(n,i,k)
         dqscloud(n,i,k) = pprods(n,i,k)

!reduce the detrainment of snow by some fraction, detsnowfac, 0-1
         pprodr(n,i,k)   = pprodr(n,i,k) + detsnowfac*dqscloud(n,i,k)
         dqscloud(n,i,k) = (c1-detsnowfac)*dqscloud(n,i,k)

         qlcloud(n,i,k)  = qlcloud(n,i,k) - cprec*qlcloud(n,i,k)
         qicloud(n,i,k)  = qicloud(n,i,k) - cprec*qicloud(n,i,k)
         qscloud(n,i,k)  = qscloud(n,i,k) + dqscloud(n,i,k)
      endif
!     write(6,111) jjj,k,i,alogic(n,k),tcloud(n,i,k),qvcloud(n,i,k), &
!                  qlcloud(n,i,k),qicloud(n,i,k),qscloud(n,i,k)
!     write(6,*)
!     tem = dqlcloud(n,i,k)+pprodr(n,i,k)+dqicloud(n,i,k)+dqscloud(n,i,k)
!     write(6,111) jjj,k,i,alogic(n,k),dqvcloud(n,i,k),tem, &
!                  dqvcloud(n,i,k)-tem

   enddo

!cloud top water budget controled by amiu:

   zk     = zzctop(n,k)-zl2(n,k+1)
   zkb1   = zzctop(n,k)-zb(n)
   zkb2   = zl2(n,k+1)-zb(n)

   wvmean = 0.5_dbl_kind*(ql(n,k) + ql2(n,k+1))
   cwmean = 0.5_dbl_kind*(qcwl(n,k) + qcwl2(n,k+1))
   cimean = 0.5_dbl_kind*(qcil(n,k) + qcil2(n,k+1))

   qvcloud(n,k,k) = ((1.0+lamda(n,k)*zkb2)*qvcloud(n,k+1,k)   &
                  + lamda(n,k)*wvmean*zk+amiu                 &
                  * lamda(n,k)*(zl(n,k)-zb(n))*wvmean)        &
                  / (1.0+lamda(n,k)*zkb1                      &
                  + amiu*lamda(n,k)*(zl(n,k)-zb(n)))
   qlcloud(n,k,k) = ((1.0+lamda(n,k)*zkb2)*qlcloud(n,k+1,k)   &
                  + lamda(n,k)*cwmean*zk+amiu                 &
                  * lamda(n,k)*(zl(n,k)-zb(n))*cwmean)        &
                  / (1.0+lamda(n,k)*zkb1                      &
                  + amiu*lamda(n,k)*(zl(n,k)-zb(n)))
   qicloud(n,k,k) = ((1.0+lamda(n,k)*zkb2)*qicloud(n,k+1,k)   &
                  + lamda(n,k)*cimean*zk+amiu                 &
                  * lamda(n,k)*(zl(n,k)-zb(n))*cimean)        &
                  / (1.0+lamda(n,k)*zkb1                      &
                  + amiu*lamda(n,k)*(zl(n,k)-zb(n)))
   qscloud(n,k,k) = ((1.0+lamda(n,k)*zkb2)*qscloud(n,k+1,k))  &
                  / (1.0+lamda(n,k)*zkb1                      &
                  + amiu*lamda(n,k)*(zl(n,k)-zb(n)))

!in-cloud condensation:

   gkq = gaml(n,k)/((1+gaml(n,k))*hltm)
   gkt = cpi/(1+gaml(n,k))                        
   qsat(n,k,k) = qs(n,k) + gkq*(hcloud(n,k,k)-hhs(n,k)        &
               + lfus*(qicloud(n,k,k)+qscloud(n,k,k)))

   dqvcloud(n,k,k) = (qvcloud(n,k,k)-qsat(n,k,k))/(1+gaml(n,k))
   if(dqvcloud(n,k,k).ge.c0) then
      tcloud(n,k,k)  = tl(n,k)                                &
                     + gkt*(hcloud(n,k,k)-hhs(n,k)            &
                     + lfus*(qicloud(n,k,k)+qscloud(n,k,k)))
   else
      tcloud(n,k,k)  = hcloud(n,k,k)-hltm*qvcloud(n,k,k)     &
                     - grav*zl(n,k)                           &
                     + lfus*(qicloud(n,k,k)+qscloud(n,k,k)) 
      tcloud(n,k,k)  = tcloud(n,k,k)*cpi           
      alogic(n,k) = .false.
   endif   
!  write(6,111) jjj,k,k,alogic(n,k),tcloud(n,k,k),qvcloud(n,k,k), &
!                qlcloud(n,k,k),qicloud(n,k,k),qscloud(n,k,k)

   if(alogic(n,k)) then
!in-cloud melting/freezing:
      if(tcloud(n,k,k) < t00) then
         w_cloud = c0
         tcloud(n,k,k)    = tcloud(n,k,k) +lfus/spec_heat_cp*qlcloud(n,k,k)
         dqwfcloud(n,k,k) = qlcloud(n,k,k)
      elseif(tcloud(n,k,k) >= t00) then
         w_cloud = c1
         tcloud(n,k,k)    = tcloud(n,k,k) -lfus/spec_heat_cp*qicloud(n,k,k)
         dqimcloud(n,k,k) = qicloud(n,k,k)
      endif 

      dqlcloud(n,k,k) = w_cloud*dqvcloud(n,k,k)           &
                      - dqwfcloud(n,k,k)+dqimcloud(n,k,k)
      dqicloud(n,k,k) = (c1-w_cloud)*dqvcloud(n,k,k)   & 
                      + dqwfcloud(n,k,k)-dqimcloud(n,k,k)
      qlcloud(n,k,k)  = qlcloud(n,k,k) + dqlcloud(n,k,k)
      qicloud(n,k,k)  = qicloud(n,k,k) + dqicloud(n,k,k)
      qvcloud(n,k,k)  = qvcloud(n,k,k) - dqvcloud(n,k,k)

!precipitation:
         cprec = cprec0*zk/(1+cprec0*zk)
         pprodr(n,k,k)   = cprec*qlcloud(n,k,k)
         pprods(n,k,k)   = cprec*qicloud(n,k,k)
         dqlcloud(n,k,k) = dqlcloud(n,k,k) - cprec*qlcloud(n,k,k)
         dqicloud(n,k,k) = dqicloud(n,k,k) - cprec*qicloud(n,k,k)
         dqscloud(n,k,k) = pprods(n,k,k)

!reduce the detrainment of snow by some fraction, detsnowfac, 0-1
         pprodr(n,k,k)   = pprodr(n,k,k) + detsnowfac*dqscloud(n,k,k)
         dqscloud(n,k,k) = (c1-detsnowfac)*dqscloud(n,k,k)

         qlcloud(n,k,k)  = qlcloud(n,k,k) - cprec*qlcloud(n,k,k)
         qicloud(n,k,k)  = qicloud(n,k,k) - cprec*qicloud(n,k,k)
         qscloud(n,k,k)  = qscloud(n,k,k) + dqscloud(n,k,k)
   endif
!   write(6,111) jjj,k,k,alogic(n,k),tcloud(n,k,k),qvcloud(n,k,k), &
!                qlcloud(n,k,k),qicloud(n,k,k),qscloud(n,k,k)
!   write(6,*)
!   tem = dqlcloud(n,k,k)+pprodr(n,k,k)+dqicloud(n,k,k)+dqscloud(n,k,k)
!   write(6,111) jjj,k,i,alogic(n,k),dqvcloud(n,k,k),tem, &
!                dqvcloud(n,k,k)-tem

!virtual temperature
    if(dotv) then
       do i = k, jjj
          tvcloud(n,i,k) = tcloud(n,i,k)*(1.+delta*qvcloud(n,i,k) &
                         - qlcloud(n,i,k)-qicloud(n,i,k)          &
                         - qscloud(n,i,k))
       enddo
    else
       do i = k, jjj
          tvcloud(n,i,k) = tcloud(n,i,k)*(1.+delta*qvcloud(n,i,k)) 
       enddo   
    endif
         
!   if(alogic(n,k)) then
!      do i = k, jjj
!         write(6,111) jjj,k,i,alogic(n,k),tcloud(n,i,k),qlcloud(n,i,k), &
!                      qicloud(n,i,k)
!      enddo
!      write(6,*)
!   endif
!111 format(3(1x,i3),l3,f10.5,5(1x,e15.8))
111 format(3(1x,i3),l3,8(1x,e15.8))

enddo !(end knum loop)

if(jjj.eq.lpblp1) then
   do ilon = 1, nunst
      n = ndxlst(ilon)
      do k = jjj-1, ltpcup, -1
         if(alogic(n,k)) inc_diag(n,9) = inc_diag(n,9) + c1
      enddo
   enddo
endif
do ilon = 1, nunst
   n = ndxlst(ilon)
   do k = jjj-1, ltpcup, -1
      if(alogic(n,k)) inc_diag(n,10) = inc_diag(n,10) + c1
   enddo
enddo

!==============================================================================
!calculate the normalized mass flux and cloud work function: 
!==============================================================================

!normalized mass flux:

do knum = 1, nunst2
   k = ndxl2(knum,1) !cloud-top index
   n = ndxl2(knum,2) !cloud-top location index

   do i = jjj, k+1, -1
      eta2(n,i,k) = c1 + lamda(n,k)*(zl2(n,i)-zb(n))
   enddo
   eta2(n,k,k) = c1 + lamda(n,k)*(zzctop(n,k)-zb(n)) &
               + amiu*lamda(n,k)*(zzctop(n,k)-zb(n))
   if(eta2(n,k,k).gt.20.0_dbl_kind) alogic(n,k) = .false.
enddo

if(jjj.eq.lpblp1) then
   do ilon = 1, nunst
      n = ndxlst(ilon)
      do k = jjj-1, ltpcup, -1
         if(alogic(n,k)) inc_diag(n,11) = inc_diag(n,11) + c1
      enddo
   enddo
endif
do ilon = 1, nunst
   n = ndxlst(ilon)
   do k = jjj-1, ltpcup, -1
      if(alogic(n,k)) inc_diag(n,12) = inc_diag(n,12) + c1
   enddo
enddo

!cloud work function:

do knum = 1, nunst2
   k = ndxl2(knum,1) !cloud-top index
   n = ndxl2(knum,2) !cloud-top location index 

   do i = k+1, jjj-1
          cdwork(n,k) = cdwork(n,k) + 0.5*grav                      &
        * ((tvcloud(n,i,k)-tvl2(n,i))/tvl2(n,i)*eta2(n,i,k)         &
        + (tvcloud(n,i+1,k)-tvl2(n,i+1))/tvl2(n,i+1)*eta2(n,i+1,k)) &
        * (zl2(n,i)-zl2(n,i+1))
   enddo
   cdwork(n,k) = cdwork(n,k) + 0.5*grav                                   &
               * (tvcloud(n,k+1,k)-tvl2(n,k+1))/tvl2(n,k+1)*eta2(n,k+1,k) &
               * (zl(n,k)-zl2(n,k+1))     
   if(cdwork(n,k) .lt. c0) then
      cdwork(n,k)  = c0
      alogic(n,k) = .false.
   endif

enddo

if(jjj.eq.lpblp1) then
   do ilon = 1, nunst
      n = ndxlst(ilon)
      do k = jjj-1, ltpcup, -1
         if(alogic(n,k)) inc_diag(n,13) = inc_diag(n,13) + c1
      enddo
   enddo
endif
do ilon = 1, nunst
   n = ndxlst(ilon)
   do k = jjj-1, ltpcup, -1
      if(alogic(n,k)) inc_diag(n,14) = inc_diag(n,14) + c1
   enddo
enddo

!==============================================================================
!reset nunst2 and ndxl2:
!==============================================================================

nunst2 = 0 ; ndxl2  = 0

do ilon= 1, nunst
   n = ndxlst(ilon)
   do k = ltpcup, jjj-1
      if(alogic(n,k)) then
         nunst2 = nunst2+1
         ndxl2(nunst2,1) = k !cloud-top index
         ndxl2(nunst2,2) = n !cloud-top location
      else
         cdwork(n,k)  = c0
         lamda(n,k) = c0
         do l = 1, lpblp1
            eta2(n,l,k)    = c0
            hcloud(n,l,k)  = c0
            qvcloud(n,l,k) = c0
            qlcloud(n,l,k) = c0
            qrcloud(n,l,k) = c0
            qicloud(n,l,k) = c0
            qscloud(n,l,k) = c0
         enddo
         do l = 1, lpbl
	        pprodr(n,l,k)   = c0
            pprods(n,l,k)   = c0
            dqvcloud(n,l,k) = c0
            dqlcloud(n,l,k) = c0
            dqrcloud(n,l,k) = c0
            dqicloud(n,l,k) = c0
            dqscloud(n,l,k) = c0
         enddo
      endif
   enddo
enddo

!cape: cloud work function when lamda=0:

do knum = 1, nunst2
   k = ndxl2(knum,1)! cloud-top index
   n = ndxl2(knum,2)! cloud-top location index
   if(alogic(n,k) .and. lamda(n,k).eq.c0) cape(n) = cdwork(n,k)
enddo                

!write(6,*) '--- end eau_cupcld'
end subroutine eau_cupcld

!------------------------------------------------------------------------------      
