!------------------------------------------------------------------------------
subroutine eau_cupinit &
               (      nlen ,     len ,     nlm ,    lpbl ,     pl , &
                        bp ,     phl ,     thl ,      ql ,     cw , &
                        rw ,      ci ,      ri ,   hbase , qlbase , &
                    cwbase ,  rwbase ,  cibase ,  ribase            &
#ifdef passive_tracers
                , npassive , tracerl , trcbase                      &
#endif
               )

use kinds
use eau_params

implicit none
!------------------------------------------------------------------------------
!eau_cupinit sets the cloud base conditions for the convective updrafts in
!eau_cup.
!Laura D. Fowler/mcbug (03-01-01). 

!send comments to laura@atmos.colostate.edu.

!modifications:
!    - added the calculation of cloud base values for passive tracers.
!      Laura D. Fowler/mcbug (02-01-02).

!subroutines called:
!none.

!argument list variables:
!input arguments:
!----------------
integer (kind=int_kind), intent(in):: &
  nlen,              &!length of horizontal domain.
  len,               &!length of sub-domain.
  nlm,               &!number of layers.
  lpbl                !index of layer above the PBL top.

real (kind=dbl_kind), intent(in), dimension(nlen,nlm):: &
  pl,                &!pressure                                           (Pa).
  bp,                &!(pl/p0)**kapa                                       (-).
  phl,               &!geopotential                                        (m).
  thl,               &!potential temperature                               (K).
  ql,                &!water vapor mixing ratio                        (kg/kg).
  cw,                &!cloud water mixing ratio                        (kg/kg).
  rw,                &!rain mixing ratio                               (kg/kg).
  ci,                &!cloud ice mixing ratio                          (kg/kg).
  ri                  !snow mixing ratio                               (kg/kg).

#ifdef passive_tracers
integer (kind=int_kind), intent(in):: &
   npassive           !number of passive tracers.

real (kind=dbl_kind), intent(in), dimension(nlen,nlm,npassive):: &
   tracerl            !mixing ratio of passive tracers                 (kg/kg).
#endif

!output arguments:
!-----------------
real (kind=dbl_kind), intent(out), dimension(nlen,lpbl+1):: &
   hbase,            &!cloud-base moist static energy                   (J/kg).
   qlbase,           &!cloud-base water vapor mixing ratio             (kg/kg).
   cwbase,           &!cloud-base cloud water mixing ratio             (kg/kg).
   rwbase,           &!cloud-base rain mixing ratio                    (kg/kg).
   cibase,           &!cloud-base cloud ice mixing ratio               (kg/kg).
   ribase             !cloud-base snow mixing ratio                    (kg/kg).

#ifdef passive_tracers
real (kind=dbl_kind),intent(out),dimension(nlen,lpbl+1,npassive):: &
   trcbase            !cloud base passive tracer mixing ratio          (kg/kg).
#endif

!local variables:

integer (kind=int_kind):: lpblp1

real (kind=dbl_kind), dimension(len,nlm):: &
   tl,               &!temperature                                         (K).
   ssl,              &!dry static energy                                (J/kg).
   hhl                !moist static energy                              (J/kg).

!------------------------------------------------------------------------------

lpblp1 = lpbl+1

tl(:,:)  = thl(1:len,:)*bp(1:len,:)
ssl(:,:) = spec_heat_cp*tl(:,:) + phl(1:len,:)
hhl(:,:) = ssl(:,:) + hltm*ql(1:len,:) - lfus*ci(1:len,:)

!cloud-base values for clouds starting in layers above the PBL:

hbase(1:len,1:lpbl)  = hhl(:,1:lpbl)
qlbase(1:len,1:lpbl) = ql(1:len,1:lpbl)
cwbase(1:len,1:lpbl) = cw(1:len,1:lpbl)
cibase(1:len,1:lpbl) = ci(1:len,1:lpbl)
!ribase(1:len,1:lpbl) = ri(1:len,1:lpbl)
rwbase(1:len,1:lpbl) = c0
ribase(1:len,1:lpbl) = c0

!cloud-base values for clouds starting at the top of the PBL. for now, the
!cloud-base properties are those of the top-layer of the multu-layer PBL.

hbase(1:len,lpblp1)  = hhl(:,lpblp1)
qlbase(1:len,lpblp1) = ql(1:len,lpblp1)
cwbase(1:len,lpblp1) = cw(1:len,lpblp1)
cibase(1:len,lpblp1) = ci(1:len,lpblp1)
!ribase(1:len,lpblp1) = ri(1:len,lpblp1)
rwbase(1:len,lpblp1) = c0
ribase(1:len,lpblp1) = c0

#ifdef passive_tracers
!cloud base values for passive tracers:

   trcbase(1:len,1:lpbl,:) = tracerl(1:len,1:lpbl,:)
   trcbase(1:len,lpblp1,:) = tracerl(1:len,lpblp1,:)       

#endif

end subroutine eau_cupinit
!------------------------------------------------------------------------------
