!------------------------------------------------------------------------------
subroutine eau_cuplsng &
               (   nlen ,     len ,     nlm ,    lpbl ,        rat , &
                 nunst2 ,   ndxl2 ,  imassl ,      hh ,       hhl2 , &
                     ql ,     ql2 ,    qcwl ,   qcwl2 ,       qrwl , &
                  qrwl2 ,    qcil ,   qcil2 ,    qril ,      qril2 , &
                   eta2 ,  hcloud , qvcloud , qlcloud ,    qrcloud , &
                qicloud , qscloud ,  pprodr ,  cuprec , cuprec_pbl , &
                  dhcup ,   dtcup ,   dvcup ,   dlcup ,      drcup , &
                  dicup ,   dscup ,   tcuph ,   tcupv ,      tcupl , &
                  tcupr ,   tcupi ,   tcups ,   fcuph ,      fcupv , &
                  fcupl ,   fcupr ,   fcupi ,   fcups                & 
               )

use kinds
use eau_params
      
implicit none
!------------------------------------------------------------------------------
!eau_cuplsng calculates the cloud-top detrainment and large-scale compensating
!subsidence of the moist static energy, water vapor, cloud water, cloud ice, 
!rain, and snow in the convective updrafts.
! Laura D. Fowler/mcbug (01-21-02).

!send comments to laura@atmos.colostate.edu

!modifications:
!    none.

!subroutines called:
!    eau_cupsubs: calculates the compensating subsidence of the large-scale
!                 variables.

!argument list variables:
!input arguments:
!----------------
integer (kind=int_kind), intent(in):: &
   nlen,             &!length of total domain.
   len,              &!length of sub domain.
   nlm,              &!number of layers.
   lpbl               !number of layers above the pbl top.

integer (kind=int_kind), intent(in), dimension(nlm):: &
   nunst2             !total number of cloud-tops per grid-point.

integer (kind=int_kind), intent(in), dimension(len*nlm,2,nlm):: &
   ndxl2              !indexes for cloud-top and cloud-top location.

real (kind=dbl_kind), intent(in), dimension(len):: &
   rat                !scale the convective tendencies when the mass flux is
                      !greater than the PBL depth.
 
real (kind=dbl_kind), intent(in), dimension(len,nlm):: &
   imassl             !inverse mass of layer                           (m2/kg).

real (kind=dbl_kind), intent(in), dimension(len,nlm-1,nlm-1,nlm):: &
   pprodr             !in-cloud tendency of precipitation of rain      (kg/kg).

real (kind=dbl_kind), intent(in), dimension(len,nlm,nlm-1,nlm):: &
   hcloud,           &!in-cloud moist static energy                     (J/kg).
   qvcloud,          &!in-cloud water vapor mixing ratio               (kg/kg).
   qlcloud,          &!in-cloud cloud water mixing ratio               (kg/kg).
   qrcloud,          &!in-cloud rain mixing ratio                      (kg/kg).
   qicloud,          &!in-cloud cloud ice mixing ratio                 (kg/kg).
   qscloud            !in-cloud snow mixing ratio                      (kg/kg).

real (kind=dbl_kind), intent(in), dimension(len,nlm,nlm,nlm):: &
   eta2               !cumulus mass flux                               (kg/m2).

real (kind=dbl_kind), intent(in), dimension(nlen,nlm):: &
   hh,               &!odd-level moist static energy                    (J/kg).
   ql,               &!odd-level water vapor mixing ratio              (kg/kg).
   qcwl,             &!odd-level cloud water mixing ratio              (kg/kg).
   qrwl,             &!odd-level rain mixing ratio                     (kg/kg). 
   qcil,             &!odd-level cloud ice mixing ratio                (kg/kg).
   qril               !odd-level snow mixing ratio                     (kg/kg).

real (kind=dbl_kind), intent(in), dimension(nlen,nlm+1):: &
   hhl2,             &!even-level layer edge moist static energy        (J/kg).
   ql2,              &!even-level layer edge water vapor mixing ratio  (kg/kg).
   qcwl2,            &!even-level layer edge cloud water mixing ratio  (kg/kg).
   qrwl2,            &!even-level layer edge rain mixing ratio         (kg/kg).
   qcil2,            &!even-level layer edge cloud ice mixing ratio    (kg/kg).
   qril2              !even-level layer edge snow mixing ratio         (kg/kg).
     
!output arguments:
!-----------------
!ls. subs: large-scale compensating subsidence.
real (kind=dbl_kind), intent(out), dimension(nlen):: &
   cuprec,           &!cumulus precipitation rate                    (kg/m2/s).
   cuprec_pbl         !cumulus precipitation rate from PBL base      (kg/m2/s).

real (kind=dbl_kind), intent(inout), dimension(nlen,nlm):: &
   tcups              !odd-level ls. subs. of snow                   (kg/kg/s). 
real (kind=dbl_kind), intent(out), dimension(nlen,nlm):: &
   dtcup,            &!odd-level heating rate                            (K/s).
   dhcup,            &!odd-level detrainment of moist static energy   (J/kg/s).
   dvcup,            &!odd-level detrainment of water vapor          (kg/kg/s).
   dlcup,            &!odd-level detrainment of cloud water          (kg/kg/s).
   drcup,            &!odd-level detrainment of rain                 (kg/kg/s).
   dicup,            &!odd-level detrainment of cloud ice            (kg/kg/s).
   dscup,            &!odd-level detrainment of snow                 (kg/kg/s).
   tcuph,            &!odd-level ls. subs. of moist static energy     (J/kg/s).
   tcupv,            &!odd-level ls. subs. of water vapor            (kg/kg/s).
   tcupl,            &!odd-level ls. subs. of cloud water            (kg/kg/s).
   tcupr,            &!odd-level ls. subs. of rain                   (kg/kg/s).
   tcupi              !odd-level ls. subs. of cloud ice              (kg/kg/s).

real (kind=dbl_kind), intent(out), dimension(nlen,nlm+1):: &
   fcuph,            &!odd-level flux of moist static energy          (J/m2/s).
   fcupv,            &!odd-level flux of water vapor                 (kg/m2/s).
   fcupl,            &!odd-level flux of cloud water                 (kg/m2/s).
   fcupr,            &!odd-level flux of rain                        (kg/m2/s).
   fcupi,            &!odd-level flux of cloud ice                   (kg/m2/s).
   fcups              !odd-level flux of snow                        (kg/m2/s).

!local variables:
integer (kind=int_kind):: ic, il, ll, lpblp1, kn, j, nlmp1

real (kind=dbl_kind):: cpi, tem, hhmean, qvmean, qlmean, qimean, qsmean   

real (kind=dbl_kind), dimension(len,nlm):: pprec, pprec_pbl

!------------------------------------------------------------------------------
!write(6,*) '--- enter eau_cuplsng:'

lpblp1 = lpbl+1
nlmp1  = nlm+1
cpi    = c1/spec_heat_cp

pprec(:,:) = c0 ; pprec_pbl(:,:)  = c0

!tendencies due to cloud-top detrainment:

do j = ncb, lpblp1
   if(nunst2(j) .ne. 0) then

      do kn = 1, nunst2(j)
         ic = ndxl2(kn,1,j) !cloud top index
         il = ndxl2(kn,2,j) !cloud top location

         hhmean = 0.5*(hh(il,ic)+hhl2(il,ic+1))
         qvmean = 0.5*(ql(il,ic)+ql2(il,ic+1))
         qlmean = 0.5*(qcwl(il,ic)+qcwl2(il,ic+1))
         qimean = 0.5*(qcil(il,ic)+qcil2(il,ic+1))
         qsmean = c0

         tem = imassl(il,ic)*eta2(il,ic,ic,j)
         dhcup(il,ic) = dhcup(il,ic) + tem*(hcloud(il,ic,ic,j)-hhmean)
         dvcup(il,ic) = dvcup(il,ic) + tem*(qvcloud(il,ic,ic,j)-qvmean)
         dlcup(il,ic) = dlcup(il,ic) + tem*(qlcloud(il,ic,ic,j)-qlmean)
         dicup(il,ic) = dicup(il,ic) + tem*(qicloud(il,ic,ic,j)-qimean)
         dscup(il,ic) = dscup(il,ic) + tem*(qscloud(il,ic,ic,j)-qsmean)

         do ll = ic, j-1               
            tem = imassl(il,ll)*eta2(il,ll,ic,j)
            pprec(il,ll) = pprec(il,ll) + tem*pprodr(il,ll,ic,j) 
         enddo
      enddo

   endif
enddo

do il = 1, len
   do ll = 1, lpbl
      cuprec(il) = cuprec(il) + pprec(il,ll)/imassl(il,ll)
   enddo
enddo

!convective precipitation from PBL cloud base:

if(nunst2(lpblp1) .ne. 0) then
   do kn = 1, nunst2(lpblp1)
      ic = ndxl2(kn,1,lpblp1) !cloud top index
      il = ndxl2(kn,2,lpblp1) !cloud top location
      do ll = ic, lpbl               
         tem = imassl(il,ll)*eta2(il,ll,ic,lpblp1)
	   pprec_pbl(il,ll) = pprec_pbl(il,ll) + tem*pprodr(il,ll,ic,lpblp1)
      enddo 
   enddo
endif
do il = 1, len
   do ll = 1, lpbl
      cuprec_pbl(il) = cuprec_pbl(il) + pprec_pbl(il,ll)/imassl(il,ll)
   enddo
enddo

!large-scale compensating subsidence:

!write(6,*) '--- moist static energy:'
 call eau_cupsubs &
         (  nlen ,  len , nlen ,  nlm ,   lpbl , nunst2 , ndxl2 ,  &
          imassl , eta2 ,   hh , hhl2 , .true. ,  tcuph , fcuph    &
         )
!write(6,*) '--- water vapor:'
 call eau_cupsubs &
        (  nlen ,  len , nlen ,  nlm ,    lpbl , nunst2 , ndxl2 , &
          imassl , eta2 ,   ql ,  ql2 , .true. ,  tcupv , fcupv   &
         )
!write(6,*) '--- cloud water:'
 call eau_cupsubs &
         (  nlen ,  len , nlen ,   nlm ,   lpbl , nunst2 , ndxl2 , &
          imassl , eta2 , qcwl , qcwl2 , .true. ,  tcupl , fcupl   &
         )
!write(6,*) '--- rain:'
!call eau_cupsubs &
!        (  nlen ,  len , nlen ,   nlm ,   lpbl , nunst2 , ndxl2 , &
!         imassl , eta2 , qrwl , qrwl2 , .true. ,  tcupr , fcupr   &
!        )
!write(6,*) '--- cloud ice:'
 call eau_cupsubs &
         (  nlen ,  len , nlen ,   nlm ,   lpbl , nunst2 , ndxl2 , &
          imassl , eta2 , qcil , qcil2 , .true. ,  tcupi , fcupi   &
         )
!write(6,*) '--- snow:'
!call eau_cupsubs &
!        (  nlen ,  len , nlen ,   nlm ,    lpbl , nunst2 , ndxl2 , &
!         imassl , eta2 , qril , qril2 , .false. ,  tcups , fcups   &
!        )

!convective heating rate:

do il = 1, len
   do ll = 1, nlm
      dtcup(il,ll) = tcuph(il,ll)+dhcup(il,ll)         &
                   - hltm*(dvcup(il,ll)+tcupv(il,ll)) &
                   + lfus*(dicup(il,ll)+tcupi(il,ll))  &
                   + lfus*(dscup(il,ll)+tcups(il,ll))
      dtcup(il,ll) = cpi*dtcup(il,ll)
   enddo
enddo

!scaling of the convective precipitation rates and tendencies when the
!mass flux is greater than the PBL depth:

cuprec(1:len)     = cuprec(1:len)*rat(:)
cuprec_pbl(1:len) = cuprec_pbl(1:len)*rat(:)

do ll = 1, nlm
   dhcup(1:len,ll) = dhcup(1:len,ll)*rat(:)
   dtcup(1:len,ll) = dtcup(1:len,ll)*rat(:)
   dvcup(1:len,ll) = dvcup(1:len,ll)*rat(:)
   dlcup(1:len,ll) = dlcup(1:len,ll)*rat(:)
   drcup(1:len,ll) = drcup(1:len,ll)*rat(:)
   dicup(1:len,ll) = dicup(1:len,ll)*rat(:)
   dscup(1:len,ll) = dscup(1:len,ll)*rat(:)
   tcuph(1:len,ll) = tcuph(1:len,ll)*rat(:)
   tcupv(1:len,ll) = tcupv(1:len,ll)*rat(:)
   tcupl(1:len,ll) = tcupl(1:len,ll)*rat(:)
   tcupr(1:len,ll) = tcupr(1:len,ll)*rat(:)
   tcupi(1:len,ll) = tcupi(1:len,ll)*rat(:)
   tcups(1:len,ll) = tcups(1:len,ll)*rat(:)
enddo

do ll = 1, nlmp1
   fcuph(1:len,ll) = fcuph(1:len,ll)*rat(:)
   fcupv(1:len,ll) = fcupv(1:len,ll)*rat(:)
   fcupl(1:len,ll) = fcupl(1:len,ll)*rat(:)
   fcupr(1:len,ll) = fcupr(1:len,ll)*rat(:)
   fcupi(1:len,ll) = fcupi(1:len,ll)*rat(:)
   fcups(1:len,ll) = fcups(1:len,ll)*rat(:)
enddo

!write(6,*) '--- end subroutine eau_cupls:'
end subroutine eau_cuplsng

!------------------------------------------------------------------------------
