!------------------------------------------------------------------------------
subroutine eau_cuppbps &
               (      ksdm , n_phys , n_pbp , nagrid ,    nlm , &
                       hhl ,   hhls ,   ssl ,   ssvl , cuprec , &
                cuprec_pbl ,   cape ,    mb ,    cke , cdwork , &
                    masflx ,  dtcup , dhcup ,  dvcup ,  dlcup , &
                     drcup ,  dicup , dscup ,  tcuph ,  tcupv , &
                     tcupl ,  tcupr , tcupi ,  tcups , inc_diag &
               )

use kinds
use eau_params
use pbp_module

implicit none
!------------------------------------------------------------------------------
!eau_cuppbps computes the point by point diagnostics that are related to the
!eau_cup parameterization of convection.
!Laura D. Fowler/slikrock (11-12-01).

!send comments to laura@atmos.colostate.edu.

!modifications:
!    none.

!subroutines called:
!    pbp1sum: saves diagnostics for pbp1s diagnostics.
!    pbp2sum: saves diagnostics for pbp2s diagnostics.

!argument list variables:
!input arguments:
!----------------
integer(kind=int_kind), intent(in):: &
   ksdm,             &!loop index.
   n_phys,           &!nb of short-time steps per hour.
   n_pbp,            &!nb of pbp time steps per hour. 
   nagrid,           &!length of horizontal domain with ghost cells.
   nlm                !number of layers.

real(kind=dbl_kind), intent(in), dimension(nagrid):: &
   cuprec,           &!cumulus precitation rate                      (kg/m2/s).
   cuprec_pbl         !precipitation rate from PBL cloud base        (kg/m2/s).

real (kind=dbl_kind), intent(in), dimension(nagrid,nlm):: &
   mb,               &!cloud base mass flux at PBP top               (kg/m2/s).
   cape,             &!conditional available potential energy           (J/kg).
   hhl,              &!odd-level moist static energy                    (J/kg).
   hhls,             &!odd-level saturation moist static energy         (J/kg).
   ssl,              &!odd-level dry static energy                      (J/kg).
   ssvl,             &!odd-level virtual dry static energy              (J/kg).
   dtcup,            &!odd-level cumulus heating rate                    (K/s).
   dhcup,            &!odd-level microp. tendency of moist stat. ener.(J/kg/s).
   dvcup,            &!odd-level microp. tendency of water vapor     (kg/kg/s).
   dlcup,            &!odd-level microp. tendency of cloud water     (kg/kg/s).
   drcup,            &!odd-level microp. tendency of rain            (kg/kg/s).
   dicup,            &!odd-level microp. tendency of cloud ice       (kg/kg/s).
   dscup,            &!odd-level microp. tendency of snow            (kg/kg/s).
   tcuph,            &!odd-level conv. transport of moist stat. ener. (J/kg/s).
   tcupv,            &!odd-level conv. transport of water vapor      (kg/kg/s).
   tcupl,            &!odd-level conv. transport of cloud water      (kg/kg/s).
   tcupr,            &!odd-level conv. transport of rain             (kg/kg/s).
   tcupi,            &!odd-level conv. transport cloud ice           (kg/kg/s).
   tcups              !odd-level conv. transport of snow             (kg/kg/s).

real(kind=dbl_kind), intent(in), dimension(nagrid,16):: inc_diag

real (kind=dbl_kind), intent(in), dimension(nagrid,nlm,nlm):: &
   cke,              &!cumulus kinetic energy                           (J/kg).
   cdwork,           &!cloud work function                              (J/kg).
   masflx             !cloud base convective mass flux               (kg/kg/s).

! local variables:
real (kind=dbl_kind):: x_pbp

real(kind=dbl_kind), dimension(nagrid,nlm):: &
   dqcup              !convective tendency of total water            (kg/kg/s).

!------------------------------------------------------------------------------

x_pbp = min(n_phys,n_pbp)
      
dqcup(:,:) = c0
dqcup(:,:) = dvcup(:,:) + dlcup(:,:) + drcup(:,:) + dicup(:,:) + dscup(:,:) &
           + tcupv(:,:) + tcupl(:,:) + tcupr(:,:) + tcupi(:,:) + tcups(:,:)

!pbp1 diagnostics:

call pbp1sum(cuprec(1),x_pbp,ipbp1_cuppr,ksdm,nagrid)
call pbp1sum(cuprec_pbl(1),x_pbp,ipbp1_cupprpbl,ksdm,nagrid)
call pbp1sum(cuprec(1),x_pbp,ipbp1_totpr,ksdm,nagrid)
call pbp1sum(mb(1,nlm),x_pbp,ipbp1_cupmb,ksdm,nagrid)
call pbp1sum(cape(1,nlm),x_pbp,ipbp1_cape,ksdm,nagrid)

call pbp1sum(inc_diag(1,1),x_pbp,ipbp1_cupinc1,ksdm,nagrid)
call pbp1sum(inc_diag(1,2),x_pbp,ipbp1_cupinc2,ksdm,nagrid)
call pbp1sum(inc_diag(1,3),x_pbp,ipbp1_cupinc3,ksdm,nagrid)
call pbp1sum(inc_diag(1,4),x_pbp,ipbp1_cupinc4,ksdm,nagrid)
call pbp1sum(inc_diag(1,5),x_pbp,ipbp1_cupinc5,ksdm,nagrid)
call pbp1sum(inc_diag(1,6),x_pbp,ipbp1_cupinc6,ksdm,nagrid)
call pbp1sum(inc_diag(1,7),x_pbp,ipbp1_cupinc7,ksdm,nagrid)
call pbp1sum(inc_diag(1,8),x_pbp,ipbp1_cupinc8,ksdm,nagrid)
call pbp1sum(inc_diag(1,9),x_pbp,ipbp1_cupinc9,ksdm,nagrid)
call pbp1sum(inc_diag(1,10),x_pbp,ipbp1_cupinc10,ksdm,nagrid)
call pbp1sum(inc_diag(1,11),x_pbp,ipbp1_cupinc11,ksdm,nagrid)
call pbp1sum(inc_diag(1,12),x_pbp,ipbp1_cupinc12,ksdm,nagrid)
call pbp1sum(inc_diag(1,13),x_pbp,ipbp1_cupinc13,ksdm,nagrid)
call pbp1sum(inc_diag(1,14),x_pbp,ipbp1_cupinc14,ksdm,nagrid)
call pbp1sum(inc_diag(1,15),x_pbp,ipbp1_cupinc15,ksdm,nagrid)
call pbp1sum(inc_diag(1,16),x_pbp,ipbp1_cupinc16,ksdm,nagrid)

!pbp2 diagnostics :
      
call pbp2sum(cke(1,1,nlm),nagrid,x_pbp,ipbp2_cke,ksdm,nagrid)
call pbp2sum(cdwork(1,1,nlm),nagrid,x_pbp,ipbp2_cdwork,ksdm,nagrid)
call pbp2sum(masflx(1,1,nlm),nagrid,x_pbp,ipbp2_masflx,ksdm,nagrid)
call pbp2sum(hhl(1,1),nagrid,x_pbp,ipbp2_hhl,ksdm,nagrid)
call pbp2sum(hhls(1,1),nagrid,x_pbp,ipbp2_hhls,ksdm,nagrid)
call pbp2sum(ssl(1,1),nagrid,x_pbp,ipbp2_ssl,ksdm,nagrid)
call pbp2sum(ssvl(1,1),nagrid,x_pbp,ipbp2_ssvl,ksdm,nagrid)
call pbp2sum(dtcup(1,1),nagrid,x_pbp,ipbp2_dtcup,ksdm,nagrid)
call pbp2sum(dqcup(1,1),nagrid,x_pbp,ipbp2_dqcup,ksdm,nagrid)
call pbp2sum(dhcup(1,1),nagrid,x_pbp,ipbp2_dhcup,ksdm,nagrid)
call pbp2sum(dvcup(1,1),nagrid,x_pbp,ipbp2_dvcup,ksdm,nagrid)
call pbp2sum(dlcup(1,1),nagrid,x_pbp,ipbp2_dlcup,ksdm,nagrid)
call pbp2sum(drcup(1,1),nagrid,x_pbp,ipbp2_drcup,ksdm,nagrid)
call pbp2sum(dicup(1,1),nagrid,x_pbp,ipbp2_dicup,ksdm,nagrid)
call pbp2sum(dscup(1,1),nagrid,x_pbp,ipbp2_dscup,ksdm,nagrid)
call pbp2sum(tcuph(1,1),nagrid,x_pbp,ipbp2_tcuph,ksdm,nagrid)
call pbp2sum(tcupv(1,1),nagrid,x_pbp,ipbp2_tcupv,ksdm,nagrid)
call pbp2sum(tcupl(1,1),nagrid,x_pbp,ipbp2_tcupl,ksdm,nagrid)
call pbp2sum(tcupr(1,1),nagrid,x_pbp,ipbp2_tcupr,ksdm,nagrid)
call pbp2sum(tcupi(1,1),nagrid,x_pbp,ipbp2_tcupi,ksdm,nagrid)
call pbp2sum(tcups(1,1),nagrid,x_pbp,ipbp2_tcups,ksdm,nagrid)

end subroutine eau_cuppbps

!------------------------------------------------------------------------------

