!------------------------------------------------------------------------------
subroutine eau_cuptran &
               (  nlen ,      len ,   lene ,   nlm , lpbl , &
                nunst2 ,    ndxl2 , imassl ,  eta2 ,   xc , &
                  xile ,  entrain ,  xflux , dxcup          &              
               )

use kinds
use eau_params

implicit none
!------------------------------------------------------------------------------
!eau_cuptran calculates the cumulus transport of a conservative variables using
!a flux divergence formulation.
!Laura D. Fowler/slikrock (11-06-97).

!send comments to laura@atmos.colostate.edu

!modifications:
!none.

!subroutines called:
!none.

!argument list variables:
!input arguments:
!----------------
logical (kind=log_kind), intent(in):: &
   entrain            !if true, large-scale variable is entrained inside the
                      !convective updraft.

integer (kind=int_kind), intent(in):: &
   nlen,             &!length of total domain.
   len,              &!length of sub-domain.
   lene,             &!additional length of sub-domain.
   nlm,              &!number of layers.
   lpbl               !number of layers above the pbl top.
integer (kind=int_kind), intent(in), dimension(nlm):: &
   nunst2             !location of given cumulus cloud base.
integer (kind=int_kind), intent(in), dimension(len*nlm,2,nlm):: &
   ndxl2              !index for cloud-top and cloud-top location.    

real (kind=dbl_kind), intent(in), dimension(len,nlm):: &
   imassl             !inverse mass of a layer                         (m2/kg).
real (kind=dbl_kind), intent(in), dimension(lene,nlm+1):: &
   xile               !even-level large-scale variable                     (?).
real (kind=dbl_kind), intent(in), dimension(len,nlm,nlm-1,nlm):: &
   xc                 !even-level in-cloud variable                        (?).
real (kind=dbl_kind), intent(in), dimension(len,nlm,nlm,nlm):: &
   eta2               !cumulus mass flux                               (kg/m2).                                        
     
!output arguments:
!-----------------
real (kind=dbl_kind), intent(out), dimension(nlen,nlm):: &
   dxcup  !odd-level divergence of the flux of variable xc due to 
          !cumulus transport                                               (?).
real (kind=dbl_kind), intent(out), dimension(nlen,nlm+1):: & 
   xflux  !even-level flux of variable xc due to cumulus transport         (?).
     
!local variables:
integer (kind=int_kind):: ic, il, j, kn, ll, lpblp1

real (kind=dbl_kind), dimension(len,nlm+1):: flux

!------------------------------------------------------------------------------

!initialize output arrays:

lpblp1 = lpbl+1

dxcup(1:len,1:nlm) = c0 ; xflux(1:len,1:nlm+1) = c0

!cumulus transport:

do j = ncb, lpblp1
   if(nunst2(j) .ne. 0) then

      do kn = 1, nunst2(j)
         ic = ndxl2(kn,1,j) ! cloud top index.
         il = ndxl2(kn,2,j) ! cloud top location.

         flux(:,:) = c0
         do ll = ic+1, j
            if(entrain) then
               flux(il,ll)  = (xc(il,ll,ic,j)-xile(il,ll)) * eta2(il,ll,ic,j)
            else
               flux(il,ll)  = xc(il,ll,ic,j)*eta2(il,ll,ic,j)
            endif
            xflux(il,ll) = xflux(il,ll) + flux(il,ll) 
         enddo
         do ll = ic, j
            dxcup(il,ll) = dxcup(il,ll) + (flux(il,ll+1)-flux(il,ll)) &
                         * imassl(il,ll)
         enddo
               
      enddo

   endif   
enddo

!write(6,*)
!write(6,*) '--- subroutine eau_cuptran:'
!xflux(:,lpblp1) = c0
!if(nunst2(lpblp1) > 0) then
!  do kn = 1, nunst2(lpblp1)
!     ic = ndxl2(kn,1,lpblp1) !cloud top index.
!     il = ndxl2(kn,2,lpblp1) !cloud top location.
!     xflux(il,lpblp1) = xflux(il,lpblp1)               &
!           + (xc(il,lpblp1,ic,lpblp1)-xile(il,lpblp1)) &
!           * eta2(il,lpblp1,ic,lpblp1)
!     write(6,100) nunst2(lpblp1),il,ic,xc(il,lpblp1,ic,lpblp1),xile(il,lpblp1)
!  enddo
!endif
!write(6,*)
!do il = 1, len
!  write(6,101) il,lpblp1,xflux(il,lpblp1)
!enddo
100 format(3(1x,i3),3(1x,e15.8))
101 format(2(1x,i3),1x,e15.8)

end subroutine eau_cuptran

!------------------------------------------------------------------------------
