!-----------------------------------------------------------------------

 subroutine eau_ener             &
            ( len ,  nlm , lpbl, &
               tl , qwvl , qcil, &
             qril                &
                   )
      
 use kinds
 use eau_params

 implicit none
      
!-----------------------------------------------------------------------
! eau_ener checks the conservation of moist static energy in eau_liq.
! Laura D. Fowler/slikrock (05-15-00).

! send comments to laura@atmos.colostate.edu.

! MODIFICATIONS:
! * Modified declarations to adapt the code from BUGS4 to BUGS5.
!   Laura D. Fowler/slikrock (02-01-01).

! SUBROUTINES CALLED:
!     none.

! FUNCTIONS CALLED:
!     none.

! INCLUDED COMMONS:
!     none.

! ARGUMENT LIST VARIABLES:
!     INPUT ARGUMENTS:
!     ----------------

 integer (kind=int_kind), intent(in) :: &
     len, &    ! Length of sub domain.
     nlm, &    ! Number of layers.
     lpbl      ! Number of layers above the PBL top.

 real (kind=dbl_kind), intent(in), dimension(len,nlm) :: &
     tl, &     ! Temperature                                         (K).
     qwvl, &   ! Water vapor mixing ratio                        (kg/kg).
     qcil, &   ! Cloud ice mixing ratio                          (kg/kg).
     qril      ! Snow mixing ration                              (kg/kg).

! LOCAL VARIABLES:

 integer (kind=int_kind) :: i, l
     
 real (kind=dbl_kind) :: ener_tot

 real (kind=dbl_kind), dimension(len,nlm) :: &
     tl_ener, qv_ener, qw_ener, mstener

!-----------------------------------------------------------------------

 ener_tot     = c0
 tl_ener(:,:) = c0
 qv_ener(:,:) = c0
 qw_ener(:,:) = c0
 mstener(:,:) = c0

 do l = 1, lpbl
    tl_ener(:,l) = spec_heat_cp*tl(:,l)
    qv_ener(:,l) = hltm*qwvl(:,l)
    qw_ener(:,l) = lfus*(qcil(:,l)+qril(:,l))
    mstener(:,l) = tl_ener(:,l)+qv_ener(:,l)-qw_ener(:,l)
 enddo

 do l = 1, lpbl
    do i = 1, len
       ener_tot = ener_tot + mstener(i,l)
    enddo
 enddo
 write(6,*) '          total energy  =',ener_tot

 end subroutine eau_ener

!-----------------------------------------------------------------------
