!-----------------------------------------------------------------------

 subroutine eau_gather &
                    (    len ,   nlm ,   lpbl, &
                          pl ,    tl ,   qwvl, &
                        qcwl ,  qrwl ,   qcil, &
                        qril , wghtl , qsatsl, &
                     dqsatsl , satsl ,    nn1, &
                         nn2 ,   nn3 ,    np1, &
                         np2 ,   np3 ,   inn1, &
                        inn2 ,  inn3 ,   inp1, &
                        inp2 ,   inp3          &
                    )
      
 use kinds
 use eau_params
 use eau_util
      
 implicit none
      
!-----------------------------------------------------------------------

! eau_gather "gathers" all the grid-points into groups of supersaturated
! and subsaturated grid-points. Each group is further divided to form
! sub-groups of ice phase, mixed phase, and liquid phase.
! Laura D. Fowler/slikrock (03-09-00).

! modifications:
! * Modified declarations to adapt the code from bugs4 to bugs5.
!   Laura D. Fowler/slikrock (02-01-01).

! subroutines called:
!    eau_sat: Computes saturation conditions.

! argument list variables:
!     input arguments:
!     ----------------

 integer (kind=int_kind), intent(in) :: & 
      len, &    ! length of sub domain.
      nlm, &    ! number of layers.
      lpbl      ! number of layers above the PBL top.

 real (kind=dbl_kind), intent(in), dimension(len,nlm) :: &
      pl, &     ! pressure                                         (Pa).
      qcwl, &   ! cloud water mixing ratio                      (kg/kg).
      qrwl, &   ! rain mixing ratio                             (kg/kg).
      qcil, &   ! cloud ice mixing ratio                        (kg/kg).
      qril      ! snow mixing ratio                             (kg/kg).

 real (kind=dbl_kind), intent(inout), dimension(len,nlm) :: &
      tl, &     ! temperature                                       (K).
      qwvl      ! water vapor mixing ratio                      (kg/kg).

! output arguments:
! -----------------

 integer (kind=int_kind), intent(out) :: &
      nn1, &    ! number of sub-saturated grid-points: ice phase.
      nn2, &    ! number of sub-saturated grid-points: mixed phase.
      nn3, &    ! number of sub-saturated grid-points: liquid phase.
      np1, &    ! number of super-saturated grid-points: ice phase.
      np2, &    ! number of super-saturated grid-points: mixed phase.
      np3       ! number of super-saturated grid-points: liquid phase.

 integer (kind=int_kind), intent(out), dimension(len*nlm) :: &
      inn1, &   ! location of nn1 grid-points.
      inn2, &   ! location of nn2 grid-points.
      inn3, &   ! location of nn3 grid-points.
      inp1, &   ! location of np1 grid-points.
      inp2, &   ! location of np2 grid-points.
      inp3      ! location of np3 grid-points.
      
 real (kind=dbl_kind), intent(out), dimension(len,nlm):: &
      qsatsl, & ! saturation mixing ratio                       (kg/kg).
      dqsatsl, &! d(qsatsl) with respect to temperature       (kg/kg/K).
      satsl, &  ! saturation                                        (-).
      wghtl     ! partitioning between cloud water and cloud ice
                !  for mixed-phase clouds.

! local variables:

 integer (kind=int_kind) :: i, l, n0

 integer (kind=int_kind), dimension(len*nlm) :: in0

 real (kind=dbl_kind), dimension(len,nlm) :: essl,dtessl

!-----------------------------------------------------------------------

!     write(6,*) '--- enter eau_gather'

 n0   = 0
 nn1  = 0
 np1  = 0
 nn2  = 0
 np2  = 0
 nn3  = 0
 np3  = 0
 in0  = 0
 inn1 = 0
 inp1 = 0
 inn2 = 0
 inp2 = 0
 inn3 = 0
 inp3 = 0

 essl(:,:)   = c0
 dtessl(:,:) = c0

 do l = 1, lpbl

    call eau_sat &
             (              len ,             pl(1,l), &
                        tl(1,l) ,      qwvl=qwvl(1,l), &
              qsats=qsatsl(1,l) , dqsats=dqsatsl(1,l), &
                  ess=essl(1,l) ,   dtess=dtessl(1,l), &
                sats=satsl(1,l) ,     wght=wghtl(1,l)  &
             )

 enddo

 do l = 1, lpbl
    do i = 1, len
       if (tl(i,l) < tbgmin) then
          if (satsl(i,l) < c1) then
             if (qcil(i,l) == c0 .and. qril(i,l) == c0) then
                n0 = n0 + 1
                in0(n0) = i+(l-1)*len
             else
                nn1 = nn1 + 1
                inn1(nn1) = i+(l-1)*len
             endif
          else
             np1 = np1 + 1
             inp1(np1) = i+(l-1)*len
          endif
       endif
    enddo
 enddo

 do l = 1, lpbl
    do i = 1, len
       if (tl(i,l) >= tbgmin .and. tl(i,l) < tbgmax) then
          if (satsl(i,l).lt.1.) then
             if (qcwl(i,l) == c0 .and. &
                 qcil(i,l) == c0 .and. &
                 qril(i,l) == c0) then
                n0 = n0 + 1
                in0(n0) = i+(l-1)*len
             else
                nn2 = nn2 + 1
                inn2(nn2) = i+(l-1)*len
             endif
          else 
             np2 = np2 + 1
             inp2(np2) = i+(l-1)*len
          endif
       endif
    enddo
 enddo

 do l = 1, lpbl
    do i = 1, len
       if (tl(i,l) >= tbgmax) then
          if (satsl(i,l) < c1) then
             if (qcwl(i,l) == c0 .and. &
                 qrwl(i,l) == c0) then
                n0 = n0 + 1
                in0(n0) = i+(l-1)*len
             else
                nn3 = nn3 + 1
                inn3(nn3) = i+(l-1)*len
             endif
          else
             np3 = np3 + 1
             inp3(np3) = i+(l-1)*len
          endif
       endif
    enddo
 enddo

! write(6,*)
! write(6,*) 'nb of dry grid-points                       =', n0
! write(6,*) 'nb of sub-sat. grid-points - ice phase      =', nn1
! write(6,*) 'nb of sub-sat. grid-points - mixed phase    =', nn2
! write(6,*) 'nb of sub-sat. grid-points - liquid phase   =', nn3
! write(6,*) 'nb of super-sat. grid-points - ice phase    =', np1
! write(6,*) 'nb of super-sat. grid-points - mixed phase  =', np2
! write(6,*) 'nb of super-sat. grid-points - liquid phase =', np3

! write(6,*) 'end eau_gather'

 end subroutine eau_gather
      
!-----------------------------------------------------------------------
