!-----------------------------------------------------------------------

 subroutine eau_l2 &
                ( len ,  nlm ,  lpbl, &
                  bpp ,   pp ,    tt, &
                  qwv ,  qrw ,   qri, &
                 bpp2 ,  pp2 ,  rho2, &
                 qrw2 , qri2          &
                )
     
 use kinds
 use eau_params
      
 implicit none

!-----------------------------------------------------------------------

! eau_l2 computes the air density, and rain and snow mixing ratios at
! the edges between layers.
! Laura D. Fowler/slikrock (02-28-00).

! send comments to laura@atmos.colostate.edu.

! modifications:
! * Modified declarations to adapt the code from bugs4 to bugs5.
!   Laura D. Fowler/slikrock (02-01-01).

! argument list variables:
! input arguments:
! ----------------

 integer (kind=int_kind), intent(in) :: &
   len, &  ! length of sub-domain.
   nlm, &  ! number of layers.
   lpbl    ! number of layers above the PBL top.

 real (kind=dbl_kind), intent(in), dimension(len,nlm) :: &
   pp, &   ! layer pressure                                    (Pa).
   bpp, &  ! (pp/p0)**kapa                                      (-).
   tt, &   ! temperature                                        (K).
   qwv, &  ! water vapor mixing ratio                       (kg/kg).
   qrw, &  ! rain mixing ratio                              (kg/kg).
   qri     ! snow mixing ratio                              (kg/kg).

 real (kind=dbl_kind), intent(in), dimension(len,nlm+1) :: &
   pp2, &  ! level pressure                                    (Pa).
   bpp2    ! (pp2/p0)**kapa                                     (-).


! output arguments:
! -----------------

 real (kind=dbl_kind), dimension(len,nlm+1) :: &
   rho2, & ! level density          (kg/m^3).
   qrw2, & ! level rain mixing ratio (kg/kg).
   qri2    ! level snow mixing ratio (kg/kg).

! local variables:

 integer (kind=int_kind) :: i,l

 real (kind=dbl_kind) :: acoef, bcoef

 real (kind=dbl_kind), dimension(len,nlm) :: aile, bile, th

 real (kind=dbl_kind), dimension(len,nlm+1) :: th2, tt2, qwv2
     
!-----------------------------------------------------------------------

 rho2(:,:) = c0
 qrw2(:,:) = c0
 qri2(:,:) = c0
 qwv2(:,:) = c0
 th2(:,:)  = c0
 tt2(:,:)  = c0

!---- computes variables at layer edges:

 th(:,:)  = tt(:,:)/bpp(:,:)
 th2(:,1) = th(:,1)
 tt2(:,1) = th2(:,1)*bpp2(:,1)
 qwv2(:,1) = 0.
 qrw2(:,1) = 0.
 qri2(:,1) = 0.

 do l = 2, nlm
    aile(:,l) = (bpp2(:,l)-bpp(:,l-1))/(bpp(:,l)-bpp(:,l-1))
    bile(:,l) = 1.-aile(:,l)
         
    th2(:,l)  = aile(:,l)*th(:,l-1) + bile(:,l)*th(:,l)
    tt2(:,l)  = th2(:,l)*bpp2(:,l)
         
    qwv2(:,l) = (qwv(:,l-1)*(pp2(:,l)-pp(:,l-1))  &
               + qwv(:,l)*(pp(:,l)-pp2(:,l)))     &
               /(pp(:,l)-pp(:,l-1)) 
    qrw2(:,l) = qrw(:,l-1)
    qri2(:,l) = qri(:,l-1)
 enddo                  

! th2(:,nlmp1)  = th2(:,nlm)+(th(:,nlm)-th2(:,nlm))  &
!               * (bpp2(:,nlmp1)-bpp2(:,nlm))        &
!               / (bpp(:,nlm)-bpp2(:,nlm))
! tt2(:,nlmp1)  = th2(:,nlmp1)*bpp2(:,nlmp1)
! qwv2(:,nlmp1) = qwv2(:,nlm)                        &
!               + (qwv(:,nlm)-qwv2(:,nlm))           &
!               * (pp2(:,nlmp1)-pp2(:,nlm))          &
!               / (pp(:,nlm)- pp2(:,nlm))
! qrw2(:,nlmp1) = qrw(:,nlm)
! qri2(:,nlmp1) = qri(:,nlm)

 do l = 1, nlm
    do i = 1, len
       acoef     = qwv2(i,l)/(1.+qwv2(i,l))
       bcoef     = (1.+0.61*acoef)*tt2(i,l)
       rho2(i,l) = pp2(i,l)/(rv*bcoef)
    enddo
 enddo

 end subroutine eau_l2
      
!-----------------------------------------------------------------------
