!-----------------------------------------------------------------------

 subroutine eau_liqpbps &
                ( ksdm , nagrid ,  nlm ,  elqr, &
                  elqs ,   d_th , d_wv ,  d_cw, &
                  d_rw ,   d_ci , d_ri , d_rwf, &
                 d_rif &
                )
      
 use kinds
 use eau_params
 use pbp_module
      
 implicit none
      
!-----------------------------------------------------------------------

! eau_liqpbps computes the point by point diagnostics that are related
! to the eau_liq parameterization of cloud microphysics.
! Laura D. Fowler/slikrock (06-09-00).

! send comments to laura@atmos.colostate.edu.

! modifications:
!     none.

! subroutines called:
!     pbp1sum: saves diagnostics for pbp1s diagnostics.
!     pbp2sum: saves diagnostics for pbp2s diagnostics.

! argument list variables:
!     input arguments:
!     ----------------

 integer(kind=int_kind), intent(in) :: &
   ksdm, &   ! loop index.
   nagrid, & ! length of horizontal domain with ghost cells.
   nlm       ! number of layers.

 real (kind=dbl_kind), intent(in), dimension(nagrid) :: &
   elqr, &      ! Precipitation of rain                        (kg/m2).
   elqs         ! Precipitation of snow                        (kg/m2).

 real(kind=dbl_kind),dimension(nagrid,nlm) :: &
   d_th, &   ! tendency of temperature due to eau_liqng         (K/s).
   d_wv, &   ! tendency of water vapor due to eau_liqng     (kg/kg/s).
   d_cw, &   ! tendency of cloud water due to eau_liqng     (kg/kg/s).
   d_rw, &   ! tendency of rain due to eau_liqng            (kg/kg/s).
   d_ci, &   ! tendency of cloud ice due to eau_liqng       (kg/kg/s).
   d_ri, &   ! tendency of snow due to eau_liqng            (kg/kg/s).
   d_rwf, &  ! tendency of rain due to precipitation        (kg/kg/s).
   d_rif     ! tendency of snow due to precipitation        (kg/kg/s).

! local variables:

 real(kind=dbl_kind), dimension(nagrid) :: elqprec
     
 real(kind=dbl_kind), dimension(nagrid,nlm) :: &
   d_qt   !tendency of total water                          (kg/kg/s).

!-----------------------------------------------------------------------

 elqprec(:) = elqr(:) + elqs(:)

 d_qt(:,:) = d_wv(:,:) + d_cw(:,:) + d_rw(:,:) + d_ci(:,:) + d_ri(:,:)

!---- pbp1 diagnostics:

 call pbp1sum(elqr,c1,ipbp1_elqprr,ksdm,nagrid)
      
 call pbp1sum(elqs,c1,ipbp1_elqprs,ksdm,nagrid)
      
 call pbp1sum(elqprec,c1,ipbp1_elqpr,ksdm,nagrid)
      
 call pbp1sum(elqprec,c1,ipbp1_totpr,ksdm,nagrid)

!---- pbp2 diagnostics:

 call pbp2sum(d_th(1,1),nagrid,c1,ipbp2_dtelq,ksdm,nagrid)
      
 call pbp2sum(d_wv(1,1),nagrid,c1,ipbp2_dvelq,ksdm,nagrid)
      
 call pbp2sum(d_cw(1,1),nagrid,c1,ipbp2_dlelq,ksdm,nagrid)
      
 call pbp2sum(d_rw(1,1),nagrid,c1,ipbp2_drelq,ksdm,nagrid)
      
 call pbp2sum(d_ci(1,1),nagrid,c1,ipbp2_dielq,ksdm,nagrid)
      
 call pbp2sum(d_ri(1,1),nagrid,c1,ipbp2_dselq,ksdm,nagrid)

 call pbp2sum(d_qt(1,1),nagrid,c1,ipbp2_dqelq,ksdm,nagrid)

 call pbp2sum(d_rwf(1,1),nagrid,c1,ipbp2_drwfelq,ksdm,nagrid)

 call pbp2sum(d_rif(1,1),nagrid,c1,ipbp2_drifelq,ksdm,nagrid)

 end subroutine eau_liqpbps
      
!-----------------------------------------------------------------------
