!-----------------------------------------------------------------------

 subroutine eau_liqqps &
                ( ksdm , nagrid ,   nlm , pcos1, &
                 pcos2 ,  psin1 , psin2 ,  elqr, &
                  elqs ,   d_th ,  d_wv ,  d_cw, &
                  d_rw ,   d_ci ,  d_ri , d_rwf, &
                 d_rif, &
                 ddzl_tp, ddzl_iq, dzlsprec, dzlrprec, dtdyni &
                )
     
 use kinds
 use eau_params
 use qp_module
 use qp_ml
      
 implicit none

!-----------------------------------------------------------------------

! eau_liqqps computes the 2D and 3D diagnostics that are related to the
! eau_liq parameterization of cloud microphysics.
! Laura D. Fowler/slikrock (04-15-01).

! send comments to laura@atmos.colostate.edu.

! modifications:
!     none.

! subroutines called:
!     qp2sum: saves diagnostics for QP2s diagnostics.
!     qp3sum: saves diagnostics for QP3s diagnostics.
!     hf2sum: saves high-frequency diagnostics.

! argument list variables:
! input arguments:
! ----------------

 integer(kind=int_kind), intent(in) :: &
   ksdm, &   ! loop index.
   nagrid, & ! length of horizontal domain with ghost cells.
   nlm       ! number of layers.

 real(kind=dbl_kind), intent(in) :: &
   pcos1, &  ! cosine of first diurnal harmonic                   (-).
   pcos2, &  ! cosine of first diurnal harmonic                   (-).
   psin1, &  ! sine of second diurnal harmonic                    (-).
   psin2     ! sine of second diurnal harmonic                    (-).

 real(kind=dbl_kind), intent(in), dimension(nagrid) :: &
   elqr, &   ! large-scale precipitation of rain            (kg/m2/s).
   elqs      ! large-scale precipitation of snow            (kg/m2/s).
     
 real(kind=dbl_kind), intent(in) :: dtdyni ! inverse timestep
 real(kind=dbl_kind), intent(in), dimension(nagrid) :: &
   ddzl_tp,  &    ! Heating due to drizzle                       (K)
   ddzl_iq,  &    ! moistening due to drizzle                    (kg/kg)
   dzlsprec, &    ! large-scale precipitation of frozen drizzle  (kg/m2)
   dzlrprec       ! large-scale precipitation of liquid drizzle  (kg/m2).
     
 real(kind=dbl_kind),dimension(nagrid,nlm) :: &
   d_th, &   ! tendency of temperature due to eau_liq           (K/s).
   d_wv, &   ! tendency of water vapor due to eau_liq       (kg/kg/s).
   d_cw, &   ! tendency of cloud water due to eau_liq       (kg/kg/s).
   d_rw, &   ! tendency of rain due to eau_liq              (kg/kg/s).
   d_ci, &   ! tendency of cloud ice due to eau_liq         (kg/kg/s).
   d_ri, &   ! tendency of snow due to eau_liq              (kg/kg/s).
   d_rwf, &  ! tendency of rain due to precipitation        (kg/kg/s).
   d_rif     ! tendency of snow due to precipitation        (kg/kg/s).

! local variables:

 real(kind=dbl_kind), dimension(nagrid) :: elqprec
     
 real(kind=dbl_kind), dimension(nagrid,nlm) :: &
   d_qt   !tendency of total water                          (kg/kg/s).

!-----------------------------------------------------------------------

!---- initialization of local variables:

 elqprec(:) = elqr(:) + elqs(:)

 d_qt(:,:) = d_wv(:,:) + d_cw(:,:) + d_rw(:,:) + d_ci(:,:) + d_ri(:,:)

!---- qp2 diagnostics:

 call qp2sum(elqr,c1,iqp2_elqprr,ksdm,nagrid)
      
 call qp2sum(elqs,c1,iqp2_prsn,ksdm,nagrid)
      
 call qp2sum(elqprec,c1,iqp2_elqpr,ksdm,nagrid)

 call qp2sum(elqprec,pcos1,iqp2_elqcos1,ksdm,nagrid)
      
 call qp2sum(elqprec,pcos2,iqp2_elqcos2,ksdm,nagrid)
      
 call qp2sum(elqprec,psin1,iqp2_elqsin1,ksdm,nagrid)
      
 call qp2sum(elqprec,psin2,iqp2_elqsin2,ksdm,nagrid)

 call qp2sum(elqprec,c1,iqp2_pr,ksdm,nagrid)
      
 call qp2sum(elqprec,pcos1,iqp2_prcos1,ksdm,nagrid)

 call qp2sum(elqprec,pcos2,iqp2_prcos2,ksdm,nagrid)

 call qp2sum(elqprec,psin1,iqp2_prsin1,ksdm,nagrid)

 call qp2sum(elqprec,psin2,iqp2_prsin2,ksdm,nagrid)

      call qp2sum(ddzl_tp, dtdyni,iqp2_deaultp, ksdm, nagrid)
      call qp2sum(ddzl_iq, dtdyni,iqp2_deauliq, ksdm, nagrid)
      call qp2sum(dzlsprec, dtdyni,iqp2_dzlsprec, ksdm, nagrid)
      call qp2sum(dzlrprec, dtdyni,iqp2_dzlrprec, ksdm, nagrid)

!---- qp3 diagnostics:

 call qp3sum(d_th(1,1),nagrid,c1,iqp3_tntlsp,ksdm,nagrid)
      
 call qp3sum(d_wv(1,1),nagrid,c1,iqp3_dvelq,ksdm,nagrid)
      
 call qp3sum(d_cw(1,1),nagrid,c1,iqp3_dlelq,ksdm,nagrid)
      
 call qp3sum(d_rw(1,1),nagrid,c1,iqp3_drelq,ksdm,nagrid)
      
 call qp3sum(d_ci(1,1),nagrid,c1,iqp3_dielq,ksdm,nagrid)
      
 call qp3sum(d_ri(1,1),nagrid,c1,iqp3_dselq,ksdm,nagrid)

 call qp3sum(d_qt(1,1),nagrid,c1,iqp3_dqelq,ksdm,nagrid)
 call qp3mlsum(d_qt(1,1),nagrid,c1,iqp3ml_dqelq,ksdm,nagrid)

 call qp3sum(d_rwf(1,1),nagrid,c1,iqp3_drwfelq,ksdm,nagrid)

 call qp3sum(d_rif(1,1),nagrid,c1,iqp3_drifelq,ksdm,nagrid)

!---- high-frequency hf2 diagnostics:

 call hf2sum(elqprec,c1,ihf2_elqpr,ksdm,nagrid)

 call hf2sum(elqprec,c1,ihf2_pr,ksdm,nagrid)

 end subroutine eau_liqqps

!-----------------------------------------------------------------------
