!-----------------------------------------------------------------------

 subroutine eau_mass                           &
                (    len ,  nlm , lpbl ,  dpl, &
                    qwvl , qcwl , qrwl , qcil, &
                    qril , elqr , elqs         &
                )
      
 use kinds
 use eau_params

 implicit none
      
!-----------------------------------------------------------------------

! eau_mass checks the conservation of mass in eau_liq.
! Laura D. Fowler/slikrock (05-12-00).

! send comments to laura@atmos.colostate.edu.

! modifications:
!   none.

! subroutines called:
!     none.

! argument list variables:
! input arguments:
! ----------------

 integer (kind=int_kind), intent(in) :: & 
   len, &    ! length of sub domain.
   nlm, &    ! number of layers.
   lpbl      ! number of layers above the PBL top.

 real (kind=dbl_kind), intent(in), dimension(len) :: &
   elqr, &   ! precipitation of rain                           (kg/m2).
   elqs      ! precipitation of snow                           (kg/m2). 
 
 real (kind=dbl_kind), intent(in), dimension(len,nlm) :: &
   dpl, &    ! layer thickness                                    (Pa).
   qwvl, &   ! water vapor mixing ratio                        (kg/kg).
   qcwl, &   ! cloud water mixing ratio                        (kg/kg).
   qrwl, &   ! rain mixing ratio                               (kg/kg).
   qcil, &   ! cloud ice mixing ratio                          (kg/kg).
   qril      ! snow mixing ratio                               (kg/kg).

! local variables:

 integer (kind=int_kind) :: i, l, inegwv, inegcw, inegrw, inegci, inegri

 integer (kind=int_kind), dimension(len*nlm) :: &
   loc_negwv, loc_negcw, loc_negrw, loc_negci, loc_negri
     
 real (kind=dbl_kind), dimension(len) :: &
   qwv_tot , qcw_tot , qrw_tot, &
   qci_tot , qri_tot ,  qm_tot
     
!-----------------------------------------------------------------------

 inegwv = 0; inegcw = 0; inegrw = 0
 inegci = 0; inegri = 0
 loc_negwv = 0; loc_negcw = 0; loc_negrw = 0
 loc_negci = 0; loc_negri = 0

 do i = 1, len
    do l = 1, lpbl
       if (qwvl(i,l) < c0) then
          inegwv = inegwv + 1
          loc_negwv(inegwv) = i+(l-1)*len 
       endif
       if (qcwl(i,l) < c0) then
          inegcw = inegcw + 1
          loc_negcw(inegcw) = i+(l-1)*len 
       endif
       if (qrwl(i,l) < c0) then
          inegrw = inegrw + 1
          loc_negrw(inegrw) = i+(l-1)*len 
       endif
       if (qcil(i,l) < c0) then
          inegci = inegci + 1
          loc_negci(inegci) = i+(l-1)*len 
       endif
       if (qril(i,l) < c0) then
          inegri = inegri + 1
          loc_negri(inegri) = i+(l-1)*len 
       endif
    enddo
 enddo

 qwv_tot = c0
 qcw_tot = c0
 qrw_tot = c0
 qci_tot = c0
 qri_tot = c0
 qm_tot  = c0

 do l = 1, lpbl
    do i = 1, len
       qwv_tot(i)  = qwv_tot(i) + qwvl(i,l)*dpl(i,l)
       qcw_tot(i)  = qcw_tot(i) + qcwl(i,l)*dpl(i,l)
       qrw_tot(i)  = qrw_tot(i) + qrwl(i,l)*dpl(i,l)
       qci_tot(i)  = qci_tot(i) + qcil(i,l)*dpl(i,l)
       qri_tot(i)  = qri_tot(i) + qril(i,l)*dpl(i,l)
    enddo
 enddo

 qwv_tot(:) = qwv_tot(:)/grav
 qcw_tot(:) = qcw_tot(:)/grav
 qrw_tot(:) = qrw_tot(:)/grav
 qci_tot(:) = qci_tot(:)/grav
 qri_tot(:) = qri_tot(:)/grav
 qm_tot(:) =  qwv_tot(:) + qcw_tot(:) + qrw_tot(:)  &
           +  qci_tot(:) + qri_tot(:) + elqr(:)     &
           +  elqs(:)

 do i = 1, len
    write(6,*) 'check for negative water species:'
    write(6,*) 'inegwv=',inegwv,'      wv_total =',qwv_tot(i)
    write(6,*) 'inegcw=',inegcw,'      cw_total =',qcw_tot(i)
    write(6,*) 'inegrw=',inegrw,'      rw_total =',qrw_tot(i)
    write(6,*) 'inegci=',inegci,'      ci_total =',qci_tot(i)
    write(6,*) 'inegri=',inegri,'      ri_total =',qri_tot(i)
    write(6,*) '   precipitation of rain =',elqr(i)
	write(6,*) '   precipitation of snow =',elqs(i)
    write(6,*) '              total mass =',qm_tot(i)
 enddo

 end subroutine eau_mass
      
!-----------------------------------------------------------------------
