!-----------------------------------------------------------------------

 subroutine eau_melt                 &
                ( len ,  nlm , lpbl, &
                   tl , qcwl , qrwl, &
                 qcil , qril         &
                )
      
 use kinds
 use eau_params
      
 implicit none
      
!-----------------------------------------------------------------------

! eau_melt computes the instantaneous rates of melting/freezing of cloud
! ice and snow/cloud water and rain.
! Laura D. Fowler/slikrock (03-15-00).

! send comments to laura@atmos.colostate.edu.

! references:
! Fowler, L.D., D.A. Randall, and S.A. Rutledge, 1996: Liquid and Ice
! Cloud Microphysics in the CSU General Circulation Model.
! Part I: Model description and simulated Microphysical processes.
! Journal of Climate, 9, 489-529.

! modifications:
!   none.

! subroutines called:
!     none.
      
! argument list variables:
! input arguments:
! ----------------

 integer (kind=int_kind), intent(in) :: & 
   len, &    ! length of sub domain.
   nlm, &    ! number of layers.
   lpbl      ! number of layers above the PBL top.
     
 real (kind=dbl_kind), intent(inout), dimension(len,nlm) :: &
   tl, &   ! Temperature                                          (K).
   qcwl, & ! Cloud water mixing ratio                         (kg/kg).
   qrwl, & ! Rain mixing ratio                                (kg/kg).
   qcil, & ! Cloud ice mixing ratio                           (kg/kg).
   qril    ! Snow mixing ratio                                (kg/kg).      

! local variables:

 integer (kind=int_kind) :: i,l

 real (kind=dbl_kind) :: fac

 real (kind=dbl_kind), dimension(len,nlm) :: &
   dt_cw, dt_rw, dt_ci, dt_ri

!-----------------------------------------------------------------------

 fac = lfus/spec_heat_cp
 dt_cw(:,:) = c0
 dt_rw(:,:) = c0
 dt_ci(:,:) = c0
 dt_ri(:,:) = c0

!---- freezing of cloud water: t < tbgmin.

 do l = 1, lpbl
    do i = 1, len
       if (tl(i,l) < tbgmin .and. qcwl(i,l) > c0) then
          qcil(i,l)  = qcil(i,l) + qcwl(i,l)
!          tl(i,l)    = tl(i,l) + fac*qcwl(i,l)
          dt_cw(i,l) = fac*qcwl(i,l) 
          qcwl(i,l)  = c0
       endif
    enddo
 enddo

!---- freezing of rain: t < t00.

 do l = 1, lpbl
    do i = 1, len
       if (tl(i,l) < t00 .and. qrwl(i,l) > c0) then
          qril(i,l)  = qril(i,l) + qrwl(i,l)
!          tl(i,l)    = tl(i,l) + fac*qrwl(i,l)
          dt_rw(i,l) = fac*qrwl(i,l)
          qrwl(i,l)  = c0
       endif
    enddo
 enddo

!---- melting of cloud ice: t > t00.

 do l = 1, lpbl
    do i = 1, len
       if (tl(i,l) >= t00 .and. qcil(i,l) > c0) then
          qcwl(i,l)  = qcwl(i,l) + qcil(i,l)
!          tl(i,l)    = tl(i,l) - fac*qcil(i,l)
          dt_ci(i,l) = -fac*qcil(i,l) 
          qcil(i,l) = c0
       endif
    enddo
 enddo

!---- melting of snow: t > t00.

 do l = 1, lpbl
    do i = 1, len
       if (tl(i,l) >= t00 .and. qril(i,l) > c0) then
          qrwl(i,l)  = qrwl(i,l) + qril(i,l)
!          tl(i,l)    = tl(i,l) - fac*qril(i,l)
          dt_ri(i,l) = -fac*qril(i,l) 
          qril(i,l)  = c0
       endif
    enddo
 enddo

!---- temperature budget:
 tl(:,1:lpbl) = tl(:,1:lpbl)                      &
              + dt_cw(:,1:lpbl) + dt_rw(:,1:lpbl) &
              + dt_ci(:,1:lpbl) + dt_ri(:,1:lpbl)

 end subroutine eau_melt

!-----------------------------------------------------------------------
