!-----------------------------------------------------------------------

 subroutine eau_prec                                  &
                (   len ,     nlm ,    lpbl ,  dtime, &
                    bpp ,      pp ,      dp ,     tt, &
                    qwv ,     qrw ,     qri ,   bpp2, &
                    pp2 , qrwfall , qrifall , dqelqr, &
                 dqelqs                               &
                )
      
 use kinds
 use eau_params
         
 implicit none

!-----------------------------------------------------------------------

! eau_prec computes the falling rates of rain and snow, and rates of
! precipitation of rain and snow at the surface.
! Laura D. Fowler/slikrock (02-28-00).

! send comments to laura@atmos.colostate.edu.

! modifications:
!     none.

! subroutines called:
!     eau_l2: computes variables needed at layer edges.

! argument list variables:
! input arguments:
! ----------------

 integer (kind=int_kind), intent(in) :: & 
   len, &    ! length of sub domain.
   nlm, &    ! number of layers.
   lpbl      ! number of layers above the PBL top.

 real (kind=dbl_kind), intent(in) :: &
   dtime     ! time-step                                      (s).

 real (kind=dbl_kind), intent(in), dimension(len,nlm) :: &
   pp, &      ! layer pressure                                (mb).
   bpp, &     ! (pp/p0)**kapa                                  (-).
   dp, &      ! pressure thickness                            (Pa).
   tt, &      ! temperature                                    (K).
   qwv, &     ! water vapor mixing ratio                   (kg/kg).
   qrw, &     ! rain mixing ratio                          (kg/kg).
   qri        ! snow mixing ratio                          (kg/kg).

 real (kind=dbl_kind), intent(in), dimension(len,nlm+1) :: &
   pp2, &     ! level pressure                                (mb).
   bpp2       ! (pp2/p0)**kapa                                 (-).

! output arguments:
! -----------------

 real (kind=dbl_kind), intent(out), dimension(len) :: &
   dqelqr, &  ! precipitation rate of rain               (kg/m2/s).
   dqelqs     ! precipitation rate of snow               (kg/m2/s).

 real (kind=dbl_kind), intent(out), dimension(len,nlm) :: &
   qrwfall, & ! falling rate of rain                     (kg/kg/s).
   qrifall    ! falling rate of snow                     (kg/kg/s).

! other variables:

 integer (kind=int_kind) :: i, l

 real (kind=dbl_kind), dimension(len,nlm+1) :: &
   qrw2, rwslp2, rwv2, rwfall2, &
   qri2, rislp2, riv2, rifall2, &
   rho2

!-----------------------------------------------------------------------

 rwslp2(:,:) = c0
 rislp2(:,:) = c0
 rwv2(:,:)   = c0
 riv2(:,:)   = c0
 qrw2(:,:)   = c0
 qri2(:,:)   = c0
      
!---- computes variables at layer edges:

 call eau_l2                    &
          ( len ,  nlm ,  lpbl, &
            bpp ,   pp ,    tt, &
            qwv ,  qrw ,   qri, &
           bpp2 ,  pp2 ,  rho2, &
           qrw2 , qri2          &
          )

!---- fallspeed of rain:

 do l = 1, lpbl+1
    do i = 1, len
       if (qrw2(i,l) >= 1.e-09) then
          rwslp2(i,l) = pielq*rhor*nzeror/(rho2(i,l)*qrw2(i,l))
          rwslp2(i,l) = rwslp2(i,l)**0.25
       endif
    enddo
 enddo      

 do l = 1, lpbl+1
    do i = 1, len
       if (rwslp2(i,l) > c0) then
!         rwv2(i,l) = (-.267+206.e02/rwslp2(i,l)
!                   - 2.045e+07/(rwslp2(i,l)**2)
!                   + 9.06e09/(rwslp2(i,l)**3))
!                   * (pzero/pp2(i,l))**0.4
          rwv2(i,l) = (rwslp2(i,l)**(-3)) *                    &
                    (((206.e02 - .267*rwslp2(i,l))*rwslp2(i,l) &
                    - 2.045e+07)*rwslp2(i,l) + 9.06e+09 )      &
                    * (pzero/pp2(i,l))**0.4
          rwv2(i,l) = max(0.0_dbl_kind,rwv2(i,l))
!          rwv2(i,l) = min(5.0_dbl_kind,rwv2(i,l))
       endif
    enddo
 enddo      

!---- fallspeed of snow:

 do l = 1, lpbl+1
    do i = 1, len
       if (qri2(i,l) >= 1.e-09) then
         rislp2(i,l) = pielq*rhos*nzeros/(rho2(i,l)*qri2(i,l))
         rislp2(i,l) = rislp2(i,l)**0.25
       endif
    enddo
 enddo      

 do l = 1, lpbl+1
    do i = 1, len
       if (rislp2(i,l) >= 1.e-09) then
          riv2(i,l) = asecond*gams3          &
                    / (6.*rislp2(i,l)**belq) &
                    * (pzero/pp2(i,l))**0.4
          riv2(i,l) = max(0.0_dbl_kind,riv2(i,l))
!          riv2(i,l) = min(5.0_dbl_kind,riv2(i,l))
       endif
    enddo
 enddo

!---- falling rates and precipitation:

 rwfall2(:,:) = rho2(:,:)*qrw2(:,:)*rwv2(:,:)
 rifall2(:,:) = rho2(:,:)*qri2(:,:)*riv2(:,:)
      
 do l = 1, lpbl
    qrwfall(:,l) = (rwfall2(:,l)-rwfall2(:,l+1))/(dp(:,l)/grav)
    qrifall(:,l) = (rifall2(:,l)-rifall2(:,l+1))/(dp(:,l)/grav)
 enddo   

 dqelqr(:) = rwfall2(:,lpbl+1)*dtime
 dqelqs(:) = rifall2(:,lpbl+1)*dtime

 end subroutine eau_prec
      
!-----------------------------------------------------------------------
