
!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

	module global_reductions

!---------------------------------------------------------------------
! PURPOSE: a set of function calls to compute global means, etc
!       adopted from the LANL POP model, tdr 2-2-98
!---------------------------------------------------------------------

	use kinds
	use domain_decomposition
	
	implicit none
	save

!---------------------------------------------------------------------
! VARIABLES
!	X	intent(in) array
!	MASK	real (kind=dbl_kind), elements with 0 are masked out
!	size	length of array
!---------------------------------------------------------------------

!*********************************************************************

      contains

!*********************************************************************

      integer function global_int_sum (X,MASK,size)

      implicit none
      integer size
      integer X(size), local_sum
      real (kind=dbl_kind) MASK(size)

      local_sum = sum(X*MASK)

#ifdef mpi
      call MPI_ALLREDUCE(local_sum, global_int_sum, 1,
     &                   mpi_int_type, MPI_SUM, MPI_COMM_ATMOS, ierr)
#else
      global_int_sum = local_sum
#endif

      end function global_int_sum

!---------------------------------------------------------------------
 
      real (kind=dbl_kind) function global_real_sum (X,MASK,size)

      implicit none
      integer size
      real (kind=dbl_kind) X(size), MASK(size), local_sum

      local_sum = sum(X*MASK)

#ifdef mpi
      call MPI_ALLREDUCE(local_sum, global_real_sum, 1,
     &                   mpi_float_type, MPI_SUM, MPI_COMM_ATMOS, ierr)
#else
      global_real_sum = local_sum
#endif

      end function global_real_sum

!---------------------------------------------------------------------

      integer function global_count (MASK,size)

      implicit none
      integer size, local_count
      real (kind=dbl_kind) MASK(size)

      local_count = sum(MASK)

#ifdef mpi
      call MPI_ALLREDUCE(local_count, global_count, 1,
     &                   mpi_int_type, MPI_SUM, MPI_COMM_ATMOS, ierr)
#else
      global_count = local_count
#endif

      end function global_count

!---------------------------------------------------------------------
 
      integer function global_int_maxval(X,size)

      implicit none
      integer size, local_int_maxval
      integer X(size)

      local_int_maxval = maxval(X)

#ifdef mpi
      call MPI_ALLREDUCE(local_int_maxval, global_int_maxval, 1,
     &                   mpi_int_type, MPI_MAX, MPI_COMM_ATMOS, ierr)
#else
      global_int_maxval = local_int_maxval
#endif

      end function global_int_maxval


!---------------------------------------------------------------------
 
      real (kind=dbl_kind) function global_real_maxval(X,size)

      implicit none
      integer size
      real (kind=dbl_kind) X(size), local_real_maxval

      local_real_maxval = maxval(X)

#ifdef mpi
      call MPI_ALLREDUCE(local_real_maxval, global_real_maxval, 1,
     &                   mpi_float_type, MPI_MAX, MPI_COMM_ATMOS, ierr)
#else
      global_real_maxval = local_real_maxval
#endif

      end function global_real_maxval

!---------------------------------------------------------------------
 
      integer function global_int_minval(X,size)

      implicit none
      integer size, local_int_minval
      integer X(size)

      local_int_minval = minval(X)

#ifdef mpi
      call MPI_ALLREDUCE(local_int_minval, global_int_minval, 1,
     &                   mpi_int_type, MPI_MIN, MPI_COMM_ATMOS, ierr)
#else
      global_int_minval = local_int_minval
#endif

      end function global_int_minval

!---------------------------------------------------------------------
 
      real (kind=dbl_kind) function global_real_minval(X,size)

      implicit none
      integer size
      real (kind=dbl_kind) X(size), local_real_minval

      local_real_minval = minval(X)

#ifdef mpi
      call MPI_ALLREDUCE(local_real_minval, global_real_minval, 1,
     &                   mpi_float_type, MPI_MIN, MPI_COMM_ATMOS, ierr)
#else
      global_real_minval = local_real_minval
#endif

      end function global_real_minval

!---------------------------------------------------------------------
 
      subroutine global_real_maxloc(WORK,idim,jdim,i,j,val)

      implicit none
      integer idim,jdim,i,j
      real (kind=dbl_kind) WORK(idim,jdim),IJ(2),val

      val = maxval(WORK)
      IJ = maxloc(WORK)
      i = IJ(1)
      j = IJ(2)

#ifdef mpi
      if(my_task.eq.0) write(6,*) ' real_maxloc not mpi'
      call MPI_FINALIZE(ierr )
      stop
#endif

      end subroutine global_real_maxloc

!---------------------------------------------------------------------

      end module global_reductions
