!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module held_suarez_forcing_module

!-----------------------------------------------------------------------
! PURPOSE: compute idealized forcing proposed by Held and Suarez
!
! NOTE : this option should not be used when GCM physics is used
!-----------------------------------------------------------------------


        use kinds
        use physical_parameters
        use model_parameters
	use grid_metrics
        use prognostics

        implicit none

	private

	integer (kind=int_kind) :: k, ispace, icorner

	real (kind=dbl_kind), dimension(iim,jjm,nlm,nsdm) ::
     *		sig,
     *		work1,
     *		work2,
     *		work3

	real (kind=dbl_kind), parameter ::
     *		t1=315.0, 
     *		t2=200.0, 
     *		delty=60.0, 
     *		deltz=10.0,
     *		ka=2.8935e-7, 
     *		ks=2.8935e-6,
     *		sigb=0.7,
     *		kv = 1.157407e-5


	public ::
     *	   held_suarez_forcing

!**********************************************************************

        contains

!**********************************************************************


!======================================================================
! BEGINNING OF HELD_SUAREZ_FORCING
!======================================================================
	subroutine held_suarez_forcing

	implicit none

!-----------------------------------------------------------------------
! sanity check
!-----------------------------------------------------------------------
#ifdef link_to_gcm_physics
	write(6,*) ' turn off held_suarez_forcing'
	stop
#endif

!-----------------------------------------------------------------------
! compute the equivalent Phillips Sigma coordinate value
!-----------------------------------------------------------------------
	do k=1,nlm
          sig(:,:,k,:) = (plg(:,:,k,:) - p_top)/(ps(:,:,n3,:) - p_top)
        enddo

!-----------------------------------------------------------------------
! compute the "radiative equilibrium" temperature
!-----------------------------------------------------------------------
	do k=1,nlm
          work1(:,:,k,:) = delty*(sin(grid_center_lat(:,:,:)))**2
          work2(:,:,k,:) = deltz*log(bpg(:,:,k,:))/kappa *
     *				(cos(grid_center_lat(:,:,:)))**2
          work3(:,:,k,:) = (t1-work1(:,:,k,:)-work2(:,:,k,:))*bpg(:,:,k,:)
        enddo
        where(work3.lt.t2) work3=t2

!-----------------------------------------------------------------------
! compute the restoring rate and forcing on potential temperature
!-----------------------------------------------------------------------
         work1 = (sig - sigb) / (c1 - sigb)
        where(sig.lt.sigb) work1=c0
       do k=1,nlm
          work2(:,:,k,:) =  ka + (ks-ka)*work1(:,:,k,:)*
     *               (cos(grid_center_lat(:,:,:)))**4
        enddo
	    th_res = th_res - ka  * (th(:,:,:,n3,:)*bpg-work3)/bpg
        th_src = th_src - work2*(th(:,:,:,n3,:)*bpg-work3)/bpg 
     *               * pthick(:,:,:,n3,:)

!-----------------------------------------------------------------------
! compute forcing on vorticty and divergence
!-----------------------------------------------------------------------
        work2 = kv*work1
	do k=1,nlm
        do icorner=1,ncorners
	do ispace=1,nspace
          u_src(ispace,icorner,:,:,k,:) = u_src(ispace,icorner,:,:,k,:) 
     *       - work2(:,:,k,:)*u(ispace,icorner,:,:,k,n3,:)
	enddo
	enddo
	enddo

	end subroutine held_suarez_forcing
!======================================================================
! END OF HELD_SUAREZ_FORCING
!======================================================================

	end module held_suarez_forcing_module
!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
