

	implicit none

!************************************************************************
! important variables
!	im = # of points in x direction
!	jm = # of points in y direction
!	km = # of points in vertical
!	nt = # of tracers (only used in i/o interface)
!	nvar = # of variables that will be transformed
!	tmpqsx = temporary array holding data before transform
!	qsx = spectral transform of tmpqsx
!	tmpqsxv = vertically summed amplitude of qsx
!************************************************************************	

	integer, parameter :: im=320, jm=160, km=17, nt=5
	integer, parameter :: nvar = 7

	integer  i,j,k,n,m,ibeg,iend,irec,ivar
	real, dimension(km+1) :: sigl2
	real, dimension(km) :: delsig,sig,weight
	real, dimension(im,jm) :: ps, f, pb
	real, dimension(im,jm,km) :: eta,div,psi,chi,th,ke
	real, dimension(im,jm,km) :: ur,vr,ud,vd,t
	real, dimension(im,jm,km) :: pressure
	real, dimension(im,jm,km,nt) :: tr,trb

	logical psc, gsc, gsc_ns

	real, dimension(im,jm,km,nvar) :: tmpqsx,qsx
	real, dimension(im,jm,nvar) :: tmpqsxv
	real  ptop,pinterface,sigmin,sigmax

        real, dimension(4*im+15) ::  wsavet
        complex, dimension(im) :: temp


	tmpqsxv = 0.0
	tmpqsx = 0.0
	qsx = 0.0
        sig=0.0
        sigl2=0.0
        ptop=0.0
        pinterface=0.0
	weight = 0


!************************************************************************
! read in sigl2 data
! compute weights for each vertical level
!************************************************************************
        open(14,file='data/sigma.defined',form='formatted',status='old')
        do k=1,km+1
        read(14,*) i,sigl2(k)
        write(6,*) i,sigl2(k)
        enddo
        read(14,*) ptop
        do k=1,km
        sig(k) = (sigl2(k)+sigl2(k+1))/2
        write(6,*) ' sig ',k,sig(k)
        enddo
        sigmin=minval(sigl2)
        sigmax=maxval(sigl2)
        if (sigmin.lt.0) then
        read(14,*) pinterface
        endif
        if(sigmax.gt.1.0) then
          if(sigmin.lt.0.0) then
           psc=.false.
           gsc_ns=.false.
           gsc=.true.
          else
           psc=.false.
           gsc_ns=.true.
           gsc=.false.
          endif
        else
          psc=.true.
          gsc_ns=.false.
          gsc=.false.
        endif
	close(14)
        write(6,*) psc,gsc_ns,gsc


	if(psc) then
	  do k=1,km
	   weight(k) = sigl2(k+1)-sigl2(k)
	  enddo
	endif

	if(gsc) then
	  do k=1,km
	   if(sigl2(k+1).le.0) then
	     weight(k) = (sigl2(k+1)-sigl2(k))*(pinterface-ptop)
	   elseif(sigl2(k+1).gt.1.0) then
	     weight(k) = (sigl2(k+1)-sigl2(k))*(1e5-9e4)
	   else
	     weight(k) = (sigl2(k+1)-sigl2(k))*(9e4-pinterface)
	   endif
	  enddo
	  weight=weight/(1e5-ptop)
	endif

        if(gsc_ns) then
          do k=1,km
           if(sigl2(k+1).gt.1.0) then
             weight(k) = (sigl2(k+1)-sigl2(k))*(1e5-9e4)
           else
             weight(k) = (sigl2(k+1)-sigl2(k))*(9e4-ptop)
           endif
          enddo
          weight=weight/(1e5-ptop)
        endif


	write(6,*) 'weight : ',sum(weight)
	do k=1,km
	write(6,*) k,weight(k)
	enddo
	write(6,*)


!************************************************************************
! initiialize fft
!************************************************************************
	call cffti(im,wsavet)

!************************************************************************
! input the records to be averaged over
!************************************************************************
	write(6,*) ' enter beginning and ending record'
	read(5,*) ibeg
	read(5,*) iend

!************************************************************************
! skip records 1 to ibeg-1
! get_data is the i/o driver
!************************************************************************
	open(10,file='fields.out',form='unformatted',status='old')
	do irec=1,ibeg-1
	write(6,*) ' skipping record ',irec
        call       get_data(10,                 &
                        ps,                     &
                        pb,                     &
                        eta,                    &
                        div,                    &
                        psi,                    &
                        chi,                    &
                        th,                     &
                        ke,                     &
                        tr,                     &
                        im,                     &
                        jm,                     &
                        km,                     &
                        nt,psc)
	enddo



!************************************************************************
! read in data and compute spectra
!************************************************************************
	do irec=ibeg,iend
	write(6,*) ' working on record ',irec
        call       get_data(10,                 &
                        ps,                     &
                        pb,                     &
                        eta,                    &
                        div,                    &
                        psi,                    &
                        chi,                    &
                        th,                     &
                        ke,                     &
                        tr,                     &
                        im,                     &
                        jm,                     &
                        km,                     &
                        nt,psc)

!************************************************************************
! compute pressure at layer center (can probably delete)
!************************************************************************
        call get_pressure(ps,pb,sig,psc,gsc_ns,gsc,ptop,    &
                pinterface,pressure,im,jm,km)

!************************************************************************
! compute velocity
!************************************************************************
	call get_vel_lat_lon(psi,chi,ur,vr,ud,vd,im,jm,km)

!************************************************************************
! compute static temperatue
!************************************************************************
! 	call convert_to_static_temp(th,pressure,t,im,jm,km)
	write(6,*) ' not converting'
	t = th
	
!************************************************************************
! put data into a single matrix
!************************************************************************
	tmpqsx(:,:,:,1) = ud+ur
	tmpqsx(:,:,:,2) = vd+vr
	tmpqsx(:,:,:,3) = t
	tmpqsx(:,:,:,4) = tmpqsx(:,:,:,1)*tmpqsx(:,:,:,2)
	tmpqsx(:,:,:,5) = tmpqsx(:,:,:,2)*tmpqsx(:,:,:,3)
	tmpqsx(:,:,:,6) = eta
	tmpqsx(:,:,:,7) = div


!************************************************************************
! transform data
!************************************************************************
	do ivar=1,nvar
	do k=1,km
	do j=1,jm
	temp(:)=cmplx(tmpqsx(:,j,k,ivar),0)
	call cfftf(im,temp,wsavet)
	qsx(:,j,k,ivar) = qsx(:,j,k,ivar) + temp*conjg(temp)/(im**2)*weight(k)
	enddo
	enddo
	enddo

!************************************************************************
! normalized and zero the zonal mean
!************************************************************************
	tmpqsxv = 2*sum(qsx,dim=3)/(iend-ibeg)
	tmpqsxv(1,:,:) = 0
	
	enddo		! do irec=ibeg,iend

	close(10)


!************************************************************************
! write out data to disk
!************************************************************************
	open(13,file='specx.data',form='unformatted',status='unknown')
	do ivar=1,nvar
	write(13) ((tmpqsxv(i,j,ivar),i=1,im/2+1),j=1,jm)
	enddo
	close(13)
	

	stop
	end
