
	program stats

	implicit none

! define the lat/lon/sig grid
	integer, parameter :: im=160, jm=80, km=17, nt=5, nfield=3
	real, dimension(km) :: p_in

! define the number of layers to interpolate onto
	integer, parameter :: kint = 39
	integer, dimension(im,jm,kint) :: mask, mask_tmp
	real, dimension(kint) :: p_out

!specify undefined
	real, parameter :: undefined = -9.999e30

!specify work arrays to determine sigma and pressure
	real, dimension(km+1) :: sigl2
	real, dimension(km) :: sig
	real  ptop, pinterface,sigmin,sigmax
	logical gsc, gsc_ns, psc

	integer  i,j,k,nsamples,n,m,ibeg,iend,ktmp,counter,l
	real, dimension(im,jm) :: ps, f, phis, area, pb
	real, dimension(im,jm,km) :: eta,div,psi,chi,th,ke
	real, dimension(im,jm,km) :: ur,vr,ud,vd,utot,vtot
	real, dimension(im,jm,km,nt) :: tr
	real, dimension(im,jm,km) :: pressure
	real, dimension(im,jm,kint,nfield,2) :: field
	real, dimension(1,jm,kint,nfield,2) :: field_za
	real, dimension(im,jm,kint,nfield,nfield) :: correl
	real, dimension(1,jm,kint,nfield,nfield) :: correl_za

	real, dimension(im,jm) :: psb,pbb
	real, dimension(im,jm,kint) :: ub,vb,tb,thb,relb,		&
				     etab,divb,psib,chib,keb,		&
      				     uu,vv,tt,thth,			&
      				     vt,uv,f_3d,t
	real, dimension(im,jm,kint,nt) :: trb

        real, dimension(im,jm,kint) :: utotp,vtotp,tp,thp,work
	real, dimension(im,jm,kint) :: etap,divp,psip,chip,kep
	real, dimension(im,jm,kint,nt) :: trp
	real, dimension(1,jm,kint) :: umzm, vmzm
	integer izm

	field=0.0
	correl=0.0
	field_za=0.0
	correl_za=0.0
	mask = 0
	mask_tmp = 0
	sig=0.0
	sigl2=0.0
	ptop=0.0
	pinterface=0.0

	psb=0.0
	pbb=0.0
        ub=0.0
	vb=0.0
	tb=0.0
	thb=0.0
	relb=0.0
	trb=0.0
        etab=0.0
	divb=0.0
	psib=0.0
	chib=0.0
	keb=0.0

! define p_out
	do k=1,kint
	  p_out(k) = 1.0e5 - 2.5e3*(k-1)
	  write(6,*) k,p_out(k)
	enddo

! read in sigl2 data
	open(14,file='data/sigma.defined',form='formatted',status='old')
	do k=1,km+1
	read(14,*) i,sigl2(k)
	write(6,*) i,sigl2(k)
	enddo
	read(14,*) ptop
	do k=1,km
	sig(k) = (sigl2(k)+sigl2(k+1))/2
	write(6,*) ' sig ',k,sig(k)
	enddo
	sigmin=minval(sigl2)
	sigmax=maxval(sigl2)
	if (sigmin.lt.0) then
	read(14,*) pinterface
	endif
	if(sigmax.gt.1.0) then
	  if(sigmin.lt.0.0) then
	   psc=.false.
	   gsc_ns=.false.
	   gsc=.true.
	  else
	   psc=.false.
	   gsc_ns=.true.
	   gsc=.false.
	  endif
	else
	  psc=.true.
	  gsc_ns=.false.
	  gsc=.false.
	endif
	write(6,*) psc,gsc_ns,gsc, ptop,pinterface
	

	do k=1,kint
	f_3d(:,:,k)=f(:,:)
	enddo


	open(13,file='bcdata.out',form='unformatted',status='old')
	call get_bcdata(13,phis,f,area,im,jm,km)
	close(13)


	read(5,*) ibeg
	read(5,*) iend

	open(13,file='fields.out',form='unformatted',status='old')
	do k=1,ibeg-1
	write(6,*) ' skipping record ',k
	call get_data(13,ps,pb,eta,div,psi,chi,th,ke,tr,im,jm,km,nt,psc)
	enddo


	do k=ibeg,iend
	write(6,*) ' averaging record ',k,ibeg,iend
	call get_data(13,ps,pb,eta,div,psi,chi,th,ke,tr,im,jm,km,nt,psc)
	write(6,*) '********'
	write(6,*) minval(ps),maxval(ps)
	write(6,*) minval(tr),maxval(tr)
        write(6,*) minval(psi),maxval(psi)
	write(6,*) minval(chi),maxval(chi)
	write(6,*) minval(th),maxval(th)
	write(6,*) '********'
	call get_vel_lat_lon(psi,chi,ur,vr,ud,vd,im,jm,km)
	call get_pressure(ps,pb,sig,psc,gsc_ns,gsc,ptop,pinterface,pressure,im,jm,km)
 	call convert_to_static_temp(th,pressure,t,im,jm,km)

	do i=1,km
	write(6,*) pressure(10,10,i),ur(10,10,i)+ud(10,10,i),t(10,10,i)
	enddo


	utot = ur + ud
	vtot = vr + vd

	call interp_to_pressure(						&
	      pressure, eta, div, psi, chi, t, ke, tr, utot, vtot, th,  	&
		im,jm,km,nt,							&
	        etap,divp,psip,chip,tp,kep,trp,utotp,vtotp,thp, 		&
	        kint,p_out,mask_tmp )

	tp = tp

	psb  = psb  + ps
	pbb  = pbb  + pb
	ub   = ub   + utotp
	vb   = vb   + vtotp
	etab = etab + etap
	divb = divb + divp
	psib = psib + psip
	chib = chib + chip
	thb  = thb  + tp
	keb  = keb  + kep
	trb  = trb  + trp
	relb = relb + (etap-f_3d)
	tb   = tb   + tp
	mask = mask + mask_tmp

        field(:,:,:,1,1) = utotp(:,:,:)
        field(:,:,:,2,1) = vtotp(:,:,:)
        field(:,:,:,3,1) = tp(:,:,:)
        field(:,:,:,:,2) = field(:,:,:,:,2) + field(:,:,:,:,1)
        do m=1,nfield
        do n=1,m
        correl(:,:,:,m,n) = correl(:,:,:,m,n) +                 &
                field(:,:,:,m,1)*field(:,:,:,n,1)
        enddo
        enddo

	write(6,*) ' end of loop ',k,ibeg,iend
	enddo		! do k=ibeg,iend

	close(13)
	write(6,*) ' out of loop'
	write(6,*) maxval(field(:,:,:,1,2)),maxval(ub(:,:,:))


	n=maxval(mask)
	where(mask.lt.0.2*n) mask=-1
	write(6,*) n,0.2*n,maxval(mask)

	do i=1,nfield
	do j=1,2
	field(:,:,:,i,j)=field(:,:,:,i,j)/mask(:,:,:)
	enddo
	enddo

	do m=1,nfield
	do n=1,m
	correl(:,:,:,m,n)=correl(:,:,:,m,n)/mask(:,:,:)
	enddo
	enddo

	do m=1,nfield
	do n=1,m
	correl(:,:,:,m,n) = correl(:,:,:,m,n) - 		&
		field(:,:,:,m,2)*field(:,:,:,n,2)
	enddo
	enddo

!set undefined
	do i=1,nfield
        do j=1,2
	where(mask.eq.-1) field(:,:,:,i,j) = undefined
	enddo
	enddo

	do i=1,nfield
	do j=1,i
	where(mask.eq.-1) correl(:,:,:,i,j) = undefined
	enddo
	enddo

	where(mask.eq.-1)
	  etab = undefined
	  divb = undefined
	  psib = undefined
	  chib = undefined
	  thb  = undefined
	  keb  = undefined
	  ub   = undefined
	  vb   = undefined
	  tb   = undefined
	elsewhere
          etab = etab/mask
          divb = divb/mask
          psib = psib/mask
          chib = chib/mask
          thb  = thb/mask
          keb  = keb/mask
          ub   = ub/mask
          vb   = vb/mask
          tb   = tb/mask
	endwhere
	psb = psb/maxval(mask)
	
	do k=1,nt
	where(mask.eq.-1)
	 trb(:,:,:,k)  = undefined
	elsewhere
	 trb(:,:,:,k)  = trb(:,:,:,k)/mask
	endwhere
	enddo

!**********************************************************************************
!write out correlation data
	open(13,file='correl.data',form='unformatted',status='unknown')
	do m=1,nfield
	call write_data_3d(13,field(1,1,1,m,2),im,jm,kint)
	enddo
	do m=1,nfield
	do n=1,m
	call write_data_3d(13,correl(1,1,1,m,n),im,jm,kint)
	enddo
	enddo
	where(mask.le.0) mask=0
	work=mask
	call write_data_3d(13,work(1,1,1),im,jm,kint)
	close(13)

!compute and write out zonally averaged data
	do j=1,jm
	do k=1,kint
	do m=1,nfield
	  counter = 0
	  do i=1,im	
	  field_za(1,j,k,m,2) = field_za(1,j,k,m,2) 			&
			+ field(i,j,k,m,2)*mask(i,j,k)
	  counter = counter + mask(i,j,k)
	  enddo
	  if(counter.ne.0) then
	    field_za(1,j,k,m,2) = field_za(1,j,k,m,2)/counter
	  else
	    field_za(1,j,k,m,2) = undefined
	  endif
	enddo
	enddo
	enddo

	do j=1,jm       
        do k=1,kint    
        do m=1,nfield  
	do n=1,m
          counter = 0
          do i=1,im    
          correl_za(1,j,k,m,n) = correl_za(1,j,k,m,n) 			&
			+ correl(i,j,k,m,n)*mask(i,j,k)
          counter = counter + mask(i,j,k)
          enddo
          if(counter.ne.0) then
            correl_za(1,j,k,m,n) = correl_za(1,j,k,m,n)/counter
          else
            correl_za(1,j,k,m,n) = undefined
          endif
	enddo
        enddo
        enddo
        enddo

	open(13,file='correl_za.data',form='unformatted',status='unknown')
        do m=1,nfield
        call write_data_3d(13,field_za(1,1,1,m,2),1,jm,kint)
        enddo
        do m=1,nfield
        do n=1,m
        call write_data_3d(13,correl_za(1,1,1,m,n),1,jm,kint)
        enddo
        enddo
        close(13)
!**********************************************************************************


!write out avg data
	open(13,file='avg.data',form='unformatted',status='unknown')
	call write_data_2d(13, psb,  im, jm)
	if(.not.psc) call write_data_2d(13, pbb,  im, jm)
	call write_data_3d(13, etab(1,1,1), im, jm, kint)
        call write_data_3d(13, divb(1,1,1), im, jm, kint)
        call write_data_3d(13, psib(1,1,1), im, jm, kint)
        call write_data_3d(13, chib(1,1,1), im, jm, kint)
        call write_data_3d(13, thb(1,1,1),  im, jm, kint)
        call write_data_3d(13, keb(1,1,1),  im, jm, kint)
        call write_data_3d(13, ub(1,1,1),   im, jm, kint)
        call write_data_3d(13, vb(1,1,1),   im, jm, kint)
        call write_data_3d(13, relb(1,1,1), im, jm, kint)
        call write_data_3d(13, tb(1,1,1),   im, jm, kint)
	do k=1,nt
        call write_data_3d(13, trb(1,1,1,k),  im, jm, kint)
        enddo
	close(13)
	
	end
