        subroutine interp_to_pressure(					
     *         pressure, eta, div, psi, chi, t, ke, tr, utot, vtot, th,	
     *         im,jm,km,nt,
     *         etap,divp,psip,chip,tp,kep,trp,utotp,vtotp,thp,
     *	       kint, p_out, mask_tmp) 

	implicit none
	integer, parameter :: nvar = 14
	integer im,jm,km,kint,i,j,k,nt
	integer, dimension(im,jm,kint):: mask_tmp
	integer, dimension(kint) :: flag
	real, dimension(kint)       :: p_out
	real, dimension(kint,nvar)  :: x_out
	real, dimension(km)         :: p_in
	real, dimension(km,nvar)    :: x_in
	real, dimension(im,jm,km)   :: pressure, th
	real, dimension(im,jm,km)   :: eta,div,psi,chi,t,ke,utot,vtot
	real, dimension(im,jm,km,nt):: tr
	real, dimension(im,jm,kint) :: etap,divp,psip,chip,tp,kep
        real, dimension(im,jm,kint) :: utotp,vtotp,thp
	real, dimension(im,jm,kint,nt) :: trp
	

	etap = 0
	divp = 0
	psip = 0
	chip = 0
	tp = 0
	kep = 0
	trp = 0
	utotp = 0
	vtotp = 0
	thp = 0
	mask_tmp = 0

	do i=1,im
	do j=1,jm
	  x_in(:, 1) = eta(i,j,:)
          x_in(:, 2) = div(i,j,:)
          x_in(:, 3) = psi(i,j,:)
          x_in(:, 4) = chi(i,j,:)
          x_in(:, 5) = t(i,j,:)
          x_in(:, 6) = ke(i,j,:)
          x_in(:, 7) = utot(i,j,:)
          x_in(:, 8) = vtot(i,j,:)
          x_in(:, 9) = th(i,j,:)
	  x_in(:,10) = tr(i,j,:,1)
	  x_in(:,11) = tr(i,j,:,2)
	  x_in(:,12) = tr(i,j,:,3)
	  x_in(:,13) = tr(i,j,:,4)
	  x_in(:,14) = tr(i,j,:,5)
	  p_in(:) = pressure(i,j,:)
	  call vert_interp(nvar, x_in, p_in, km, x_out, p_out, kint, flag)
	  etap(i,j,:)  = x_out(:, 1)
          divp(i,j,:)  = x_out(:, 2)
          psip(i,j,:)  = x_out(:, 3)
          chip(i,j,:)  = x_out(:, 4)
	  tp(i,j,:)    = x_out(:, 5)
          kep(i,j,:)   = x_out(:, 6)
          utotp(i,j,:) = x_out(:, 7)
          vtotp(i,j,:) = x_out(:, 8)
          thp(i,j,:)   = x_out(:, 9)
	  trp(i,j,:,1) = x_out(:,10)
          trp(i,j,:,2) = x_out(:,11)
          trp(i,j,:,3) = x_out(:,12)
          trp(i,j,:,4) = x_out(:,13)
          trp(i,j,:,5) = x_out(:,14)
	  mask_tmp(i,j,:) = flag(:)
	enddo
	enddo

	return
	end


	subroutine vert_interp(nvar, x_in, p_in, nlm_in, 
     *		x_out, p_out, nlm_out, mask)
!
! a subroutine to interpolate from one set of pressure surfaces to another
!	nvar    : number of variables in the input data
!  	x_in 	: input data
!	p_in	: pressure surface which data resides
!	nlm_in	: number of p_in surfaces
!	x_out	: output data
!	p_out	: pressure surfaces to interpolate onto
!	nlm_out	: number of p_out surfaces
!	mask	: true is output surface exists
 
	implicit none
	integer, intent(in) :: nlm_in, nlm_out, nvar
	real,  dimension(nlm_in,nvar), intent(in)  :: x_in
	real,  dimension(nlm_in) :: p_in
	real,  dimension(nlm_out), intent(in)  :: p_out
	real,  dimension(nlm_out,nvar), intent(out) :: x_out
	integer, dimension(nlm_out), intent(out) :: mask
	integer ktmp, k, kp1, km1
	real, dimension(nvar) ::  increment

!	if(p_in(nlm_in).gt.p_out(nlm_out)) stop 'vert_interp: error 1'
	mask = 0

	do ktmp = 1,nlm_out
	  do k = 2,nlm_in
	    if(p_in(k).le.p_out(ktmp)) then
	      if(p_in(k-1).gt.p_out(ktmp)) then
		kp1 = k-1
		km1 = k
		mask(ktmp) = 1
		goto 10
	      endif
	    endif
	  enddo


 10 	  if(mask(ktmp).eq.1) then
	     if(p_in(k).gt.p_out(ktmp).or.p_in(k-1).lt.p_out(ktmp)) then
                write(6,*) ' vert_interp: error 2'
                stop
             endif
	     increment(:) = (x_in(kp1,:)-x_in(km1,:))*		
     * 			(p_out(ktmp)-p_in(km1))/(p_in(kp1)-p_in(km1))
 	     x_out(ktmp,:) = x_in(km1,:) + increment
	  else
	     x_out(ktmp,:) = 0
	  endif
	
	enddo

	return
	end
