!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

	module model_control

!-----------------------------------------------------------------
! 	this module hold variables which are specified at runtime
!	  or vary as the integration proceeds.does most of the i/o

!---- modifications:
!

!-----------------------------------------------------------------

	use kinds
	use physical_parameters
	use domain_decomposition
	use prognostics
	use grid_mask
	use operators_hl
	use operators
	use elliptic_solver
 	use grid_metrics
	use interpolate_to_latlon
        use gather_scatter
	use time_manager_module
	use multigrid_solver

	implicit none
	save

	integer (kind=int_kind) :: 
     *			icase, 
     *			ksdm,
     *			ioerr

	character*40 simulation_tag

	integer (kind = int_kind), parameter :: interp_file = 47

!-----------------------------------------------------------------
!  VARIABLE DEFINITION
!-----------------------------------------------------------------
!
!  character strings
!       simulation_tag = name of test case
!
!  integer scalar
!       icase = case number
!
!-----------------------------------------------------------------


!****************************************************************

	contains

!****************************************************************


!======================================================================
! BEGINNING OF INIT_MODEL_CONTROL
!======================================================================
	subroutine init_model_control

	implicit none

	icase = 0
	simulation_tag = ''

	end subroutine init_model_control
!======================================================================
! END OF INIT_MODEL_CONTROL
!======================================================================


!======================================================================
! BEGINNING OF DIAGNOSTICS
!======================================================================
        subroutine diagnostics
        implicit none
	end subroutine diagnostics
!======================================================================
! END OF DIAGNOSTICS
!======================================================================

	
!======================================================================
! BEGINNING OF WRITE_RESTART
!======================================================================
	subroutine write_restart
	implicit none

        integer lrecl, i, j, ksdm, k, nt, np, iend, ibeg
        real (kind=dbl_kind), allocatable, dimension(:) :: xrec
        real (kind=real_kind) :: vers = 5.1
        character*8, time_label
        integer (kind=int_kind) :: monthofyear

!  do master processor restart io for serial io
#ifndef parallel_io
!  global variables
!  global rectangular grid workarray
        real (kind=dbl_kind), dimension(im,jm) :: work_glob
!  max_ig vectors
        real (kind=dbl_kind), dimension(:,:), allocatable ::
     *              ps_vec, pb_vec
        real (kind=dbl_kind), dimension(:,:,:), allocatable ::
     *              eta_vec, div_vec, th_vec
        real (kind=dbl_kind), dimension(:,:,:,:), allocatable ::
     *              tracer_vec
#endif

!-------------------------------------------------------------------------
!  label the restart with a time stamp 
!-------------------------------------------------------------------------
        if(n_restart.gt.0) then
          write(unit=time_label,fmt='(i7)')
     *                  1000000+nsecond_dyn/86400
          time_label(1:1) = 'd'
        else
          monthofyear = mod(nmonth-1,12)+1
          write(unit=time_label,fmt='(i7)')
     &          1000000+100*(nyear+nyear0)+monthofyear 
          time_label(1:1) = 'y'
        endif

#ifdef eau_ng
        lrecl = kind(xrec)*ntprog*(2+nlm*(3+nwater+nelqng)+nlmp1*nelqngp1)
#else
        lrecl = kind(xrec)*ntprog*(2+nlm*(3+nwater))
#endif

#ifndef parallel_io

!-------------------------------------------------------------------------
!  gather the global arrays and reduce to vector
!-------------------------------------------------------------------------
        if(my_task.eq.0) then
           allocate(ps_vec(max_ig,ntprog))
#ifndef no_pbl
           allocate(pb_vec(max_ig,ntprog))
#endif
           allocate(eta_vec(max_ig,nlm,ntprog))
           allocate(div_vec(max_ig,nlm,ntprog))
           allocate(th_vec(max_ig,nlm,ntprog))
           allocate(tracer_vec(max_ig,nlm,ntracer,ntprog))
        endif
        do np = 1,ntprog
           call gather_fine(ps(:,:,np,:),work_glob(:,:))
           if(my_task.eq.0)
     *        call reduce_to_vector_global(work_glob(:,:),ps_vec(:,np))
#ifndef no_pbl
           call gather_fine(pb(:,:,np,:),work_glob(:,:))
           if(my_task.eq.0)
     *        call reduce_to_vector_global(work_glob(:,:),pb_vec(:,np))
#endif
           do k = 1,nlm
              call gather_fine(eta(:,:,k,np,:),work_glob(:,:))
              if(my_task.eq.0)
     *           call reduce_to_vector_global(work_glob(:,:),eta_vec(:,k,np))
              call gather_fine(div(:,:,k,np,:),work_glob(:,:))
              if(my_task.eq.0)
     *           call reduce_to_vector_global(work_glob(:,:),div_vec(:,k,np))
              call gather_fine(th(:,:,k,np,:),work_glob(:,:))
              if(my_task.eq.0)
     *           call reduce_to_vector_global(work_glob(:,:),th_vec(:,k,np))
              do nt = 1,ntracer
                 call gather_fine(tracer(:,:,k,nt,np,:),work_glob(:,:))
                 if(my_task.eq.0)
     *              call reduce_to_vector_global(
     *                   work_glob(:,:),tracer_vec(:,k,nt,np))
              enddo
           enddo
        enddo

        if(my_task.eq.0) then
           open(4,
     *     file=trim(datapath)//'restarts/model_restart_atm.'//trim(time_label)
     *,      form='unformatted', access='DIRECT', recl=lrecl)

!-------------------------------------------------------------------------
!  get the header record
!-------------------------------------------------------------------------
           write(4,rec=1,iostat=ioerr)nsecond_dyn,step_count,tau,max_ig,nlm
     *,              nwater,ntprog,lrecl
           if(ioerr.ne.0) 
     * print *,'model restart header ioerr, mytask ',ioerr,my_task

!-------------------------------------------------------------------------
! write out array data
!-------------------------------------------------------------------------
           allocate( xrec( lrecl/kind(xrec) ) )
           do i=1,max_ig
              xrec = 0.0

              ibeg = 1
              iend = ibeg+ntprog-1
              xrec(ibeg:iend) = ps_vec(i,:)

#ifndef no_pbl
              ibeg = iend+1
              iend = ibeg+ntprog-1
              xrec(ibeg:iend) = pb_vec(i,:)
#endif

              do np = 1,ntprog
                 ibeg = iend+1
                 iend = ibeg+nlm-1
                 xrec(ibeg:iend) = eta_vec(i,:,np)
              enddo
 
              do np = 1,ntprog
                 ibeg = iend+1
                 iend = ibeg+nlm-1
                 xrec(ibeg:iend) = div_vec(i,:,np)
              enddo

              do np = 1,ntprog
                 ibeg = iend+1
                 iend = ibeg+nlm-1
                 xrec(ibeg:iend) = th_vec(i,:,np)
              enddo

              do nt = 1,nwater
                 do np = 1,ntprog
                    ibeg = iend+1
                    iend = ibeg+nlm-1
                    xrec(ibeg:iend) = tracer_vec(i,:,nt,np)
                 enddo
              enddo

              write(4,rec=1+i,iostat=ioerr)xrec
              if(ioerr.ne.0) 
     *           print *,'model restart ioerr, mytask, rec '
     *,          ioerr,my_task,i+1

           enddo
           deallocate(xrec)

           close(4)

!-----------------------------------------------------------------------
! write out passive tracers (if any)
!-----------------------------------------------------------------------
	   if(ntracer.gt.nwater) then
              lrecl = kind(xrec)*ntprog*(nlm*(ntracer-nwater))
              open(4,
     *  file=trim(datapath)//'restarts/model_restart_tracer.'//trim(time_label)
     *,            form='unformatted', access='DIRECT', recl=lrecl)

!-------------------------------------------------------------------------
!  write the header record
!-------------------------------------------------------------------------
              write(4,rec=1,iostat=ioerr)nsecond_dyn,step_count,tau,max_ig,nlm
     *,                  ntracer-nwater,ntprog,lrecl, vers
              if(ioerr.ne.0) 
     *           print *,'model restart header ioerr, mytask ',ioerr,my_task

!-------------------------------------------------------------------------
! write out array data
!-------------------------------------------------------------------------
              allocate( xrec( lrecl/kind(xrec) ) )
              do i=1,max_ig
                 xrec = 0.0
                 iend = 0
                 do nt = nwater+1,ntracer
                    do np = 1,ntprog
                       ibeg = iend+1
                       iend = ibeg+nlm-1
                       xrec(ibeg:iend) = tracer_vec(i,:,nt,np)
                    enddo
                 enddo

                 write(4,rec=1+i,iostat=ioerr)xrec
        if(ioerr.ne.0) 
     * print *,'model restart ioerr, mytask, rec '
     *,ioerr,my_task,grid_center_index(i,j,ksdm)+1

              enddo
              deallocate(xrec)

              close(4)
           endif   !  if(ntracer.gt.nwater)

           deallocate(ps_vec)
#ifndef no_pbl
           deallocate(pb_vec)
#endif
           deallocate(eta_vec)
           deallocate(div_vec)
           deallocate(th_vec)
           deallocate(tracer_vec)

        endif  ! if(my_task.eq.0)

#else
        open(4,
     *     file=trim(datapath)//'restarts/model_restart_atm.'//trim(time_label)
     *,      form='unformatted', access='DIRECT', recl=lrecl)

!-------------------------------------------------------------------------
!  get the header record
!-------------------------------------------------------------------------
        if(my_task.eq.0) then
#ifdef eau_ng
        write(4,rec=1,iostat=ioerr) nsecond_dyn,step_count,tau,max_ig
     *,       nlm,nwater,nelqng,nelqngp1,ntprog,lrecl,vers
#else
        write(4,rec=1,iostat=ioerr)nsecond_dyn,step_count,tau,max_ig,nlm
     *,              nwater,ntprog,lrecl, vers
#endif
        if(ioerr.ne.0) 
     * print *,'model restart header ioerr, mytask ',ioerr,my_task
        write(6,*) nsecond_dyn,step_count,tau,max_ig,nlm
     *,              nwater,ntprog,lrecl, vers
        endif

!-------------------------------------------------------------------------
! write out array data
!-------------------------------------------------------------------------
        allocate( xrec( lrecl/kind(xrec) ) )
        do j = 1,jjm
          do i = 1,iim
            do ksdm = 1,nsdm
              if(grid_center_mask(i,j,ksdm).gt.0.5_dbl_kind) then
                xrec = 0.0

                ibeg = 1
                iend = ibeg+ntprog-1
                xrec(ibeg:iend) = ps(i,j,:,ksdm)

#ifndef no_pbl
                ibeg = iend+1
                iend = ibeg+ntprog-1
                xrec(ibeg:iend) = pb(i,j,:,ksdm)
#endif

                do np = 1,ntprog
                  ibeg = iend+1
                  iend = ibeg+nlm-1
                  xrec(ibeg:iend) = eta(i,j,:,np,ksdm)
                enddo

                do np = 1,ntprog
                  ibeg = iend+1
                  iend = ibeg+nlm-1
                  xrec(ibeg:iend) = div(i,j,:,np,ksdm)
                enddo

                do np = 1,ntprog
                  ibeg = iend+1
                  iend = ibeg+nlm-1
                  xrec(ibeg:iend) = th(i,j,:,np,ksdm)
                enddo

                do nt = 1,nwater
                  do np = 1,ntprog
                    ibeg = iend+1
                    iend = ibeg+nlm-1
                    xrec(ibeg:iend) = tracer(i,j,:,nt,np,ksdm)
                  enddo
                enddo
#ifdef eau_ng
                do nt = 1, nelqng
                   do np = 1, ntprog
                      ibeg = iend+1
                      iend = ibeg+nlm-1
                      xrec(ibeg:iend) = elqng(i,j,:,nt,np,ksdm)
                   enddo
                enddo
                do nt = 1, nelqngp1
                   do np = 1, ntprog
                      ibeg = iend + 1
                      iend = ibeg+nlmp1-1
                      xrec(ibeg:iend) = elqngp1(i,j,:,nt,np,ksdm)
                   enddo
                enddo
#endif
                write(4,rec=1+grid_center_index(i,j,ksdm),iostat=ioerr)xrec
        if(ioerr.ne.0) 
     * print *,'model restart ioerr, mytask, rec '
     *,ioerr,my_task,grid_center_index(i,j,ksdm)+1

              endif
            enddo
          enddo
        enddo
        deallocate(xrec)

        close(4)

!-----------------------------------------------------------------------
! write out passive tracers (if any)
!-----------------------------------------------------------------------
	if(ntracer.gt.nwater) then
          lrecl = kind(xrec)*ntprog*(nlm*(ntracer-nwater))
          open(4,
     *  file=trim(datapath)//'restarts/model_restart_tracer.'//trim(time_label)
     *,        form='unformatted', access='DIRECT', recl=lrecl)

!-------------------------------------------------------------------------
!  write the header record
!-------------------------------------------------------------------------
          if(my_task.eq.0) then
          write(4,rec=1,iostat=ioerr)nsecond_dyn,step_count,tau,max_ig,nlm
     *,              ntracer-nwater,ntprog,lrecl, vers
          if(ioerr.ne.0) 
     *       print *,'model restart header ioerr, mytask ',ioerr,my_task
          endif

!-------------------------------------------------------------------------
! write out array data
!-------------------------------------------------------------------------
          allocate( xrec( lrecl/kind(xrec) ) )
          do j = 1,jjm
            do i = 1,iim
              do ksdm = 1,nsdm
                if(grid_center_mask(i,j,ksdm).gt.0.5_dbl_kind) then
                  xrec = 0.0
                  iend = 0
                  do nt = nwater+1,ntracer
                    do np = 1,ntprog
                      ibeg = iend+1
                      iend = ibeg+nlm-1
                      xrec(ibeg:iend) = tracer(i,j,:,nt,np,ksdm)
                    enddo
                  enddo

                  write(4,rec=1+grid_center_index(i,j,ksdm),iostat=ioerr)xrec
        if(ioerr.ne.0) 
     * print *,'model restart ioerr, mytask, rec '
     *,ioerr,my_task,grid_center_index(i,j,ksdm)+1

                endif
              enddo
            enddo
          enddo
          deallocate(xrec)

          close(4)
        endif

#endif

!-------------------------------------------------------------------------
!  Adjust for restartability...
!  reset step_count to zero, reset advective tendencies to zero
!-------------------------------------------------------------------------
 	step_count=0
        ps_f = c0
#ifndef no_pbl
        pb_f = c0
#endif
	u_f = c0
        eta_f = c0
        div_f = c0
        th_f = c0
        tracer_f = c0
        mass_flux_c = c0
        div_mass_flux = c0

        psi = c0
        chi = c0
!-----------------------------------------------------------------------
! compute relative vorticity
!-----------------------------------------------------------------------
        do k=1,nlm
          relative(:,:,k,:) = eta(:,:,k,n4,:) - f(:,:,:)
        enddo

!-------------------------------------------------------------------------
! solve for psi and chi
!-------------------------------------------------------------------------
	do k=1,nlm
          call elliptic(ipad,iim,jjm,1,nsdm,nweights,grid_center_mask(:,:,:),
     *       l_weights(:,:,:,:),relative(:,:,k,:),psi(:,:,k,:))
          call elliptic(ipad,iim,jjm,1,nsdm,nweights,grid_center_mask(:,:,:),
     *       l_weights(:,:,:,:),div(:,:,k,n4,:),chi(:,:,k,:))
        enddo

        call wrap( max_level,
     *             iim,
     *             jjm,
     *             nlm,
     *             nsdm,
     *             psi(:,:,:,:))
        call wrap( max_level,
     *             iim,
     *             jjm,
     *             nlm,
     *             nsdm,
     *             chi(:,:,:,:))

!-------------------------------------------------------------------------
! compute kinetic energy
!-------------------------------------------------------------------------
        call calculate_u_and_ke(nlm,psi(:,:,:,:),chi(:,:,:,:),
     *           u(:,:,:,:,:,n4,:),ke(:,:,:,:))

	end subroutine write_restart
!======================================================================
! END OF WRITE_RESTART
!======================================================================


!======================================================================
! BEGINNING OF READ_RESTART
!======================================================================
	subroutine read_restart
	implicit none

	integer (kind=int_kind) ::
     *       i,j,k,nt,np,ibeg, iend
        integer (kind=int_kind) ::
     *       max_ig_tem, nlm_tem, ntprog_tem, ntracer_tem, lrecl
     *,      nsecond_tem, step_count_tem, lrecl4, lrecl8, lreclw
#ifdef eau_ng
      integer (kind=int_kind)::
     *  nelqng_tem, nelqngp1_tem
#endif	
	logical (kind=log_kind), parameter :: psc_to_gsc = .false.

        real (kind=selected_real_kind(6)), dimension(:), allocatable :: xrec4
        real (kind=selected_real_kind(13)), dimension(:), allocatable :: xrec8
        real (kind=selected_real_kind(6)) :: tau_tem4
        real (kind=selected_real_kind(13)) :: tau_tem8

#ifdef eau_ng
      open(4,file=trim(datapath)//'data/ic_atm',form='unformatted',
     *      action='read'
     *,    access='DIRECT', recl=44, status='old', iostat=ioerr )
#else        
        open(4,file=trim(datapath)//'data/ic_atm',form='unformatted',
     *       action='read'
     *,      access='DIRECT', recl=36, status='old', iostat=ioerr )
#endif
        if (ioerr /= 0) then
          write(*,*) '*** read_restart:  ic_atm file error ',ioerr
          stop
        endif

!-------------------------------------------------------------------------
!  get the header record
!-------------------------------------------------------------------------
#ifdef eau_ng
        read(4,rec=1)nsecond_dyn,step_count,tau_tem4,max_ig_tem,nlm_tem
     *,              ntracer_tem,nelqng_tem,nelqngp1_tem,ntprog_tem,lrecl4
        read(4,rec=1)nsecond_dyn,step_count,tau_tem8,max_ig_tem,nlm_tem
     *,              ntracer_tem,nelqng_tem,nelqngp1_tem,ntprog_tem,lrecl8
        lreclw = ntprog*(1+ nlm*(3+nwater+nelqng)+nlmp1*nelqngp1)
#else
        read(4,rec=1)nsecond_dyn,step_count,tau_tem4,max_ig_tem,nlm_tem
     *,              ntracer_tem,ntprog_tem,lrecl4
        read(4,rec=1)nsecond_dyn,step_count,tau_tem8,max_ig_tem,nlm_tem
     *,              ntracer_tem,ntprog_tem,lrecl8
        lreclw = ntprog*(1+ nlm*(3+nwater))
#endif
	write(6,*) nsecond_dyn,step_count,tau_tem8,max_ig_tem,nlm_tem
     *,              ntracer_tem,ntprog_tem,lrecl8,lrecl4
        if(.not.psc_to_gsc) lreclw = lreclw + ntprog

        close(4)

!-------------------------------------------------------------------------
!  get the data records
!-------------------------------------------------------------------------

        if (nlm_tem .ne. nlm) then
          write(*,*) '>>> nlm_tem does not equal nlm!!!'
          write(*,*) '>>> nlm_tem = ',nlm_tem,' nlm = ',nlm
          write(*,*) '>>> i.e., Number of layers in restart file does ',
     *      'not match nlm in sigma_coordinate.F'
          write(*,*) '>>> EXITING!!!'
          stop
        endif
        write(6,*) lreclw, lrecl4, lrecl8
        if(4*lreclw.eq.lrecl4) then

        lrecl = lrecl4
        tau = tau_tem4
        open(4,file=trim(datapath)//'data/ic_atm',form='unformatted'
     *,      access='DIRECT', recl=lrecl)
        allocate( xrec4( lrecl/kind(xrec4) ) )
        do j = 1,jjm
          do i = 1,iim
            do ksdm = 1,nsdm
              read(4,rec=1+grid_center_index(i,j,ksdm)) xrec4

!-------------------------------------------------------------------------
!  load the data from one grid point into the model arrays
!-------------------------------------------------------------------------
              ibeg=1
              iend=1+ntprog-1
              ps(i,j,1:ntprog,ksdm) = xrec4(ibeg:iend)
#ifndef no_pbl
              if(.not.psc_to_gsc) then
                ibeg=iend+1
                iend = ibeg+ntprog-1
                pb(i,j,1:ntprog,ksdm) = xrec4(ibeg:iend)
              endif
#endif
              do np = 1,ntprog
                ibeg=iend+1
                iend = ibeg+nlm-1
                eta(i,j,1:nlm,np,ksdm) = xrec4(ibeg:iend)
              enddo
              do np = 1,ntprog
                ibeg=iend+1
                iend = ibeg+nlm-1
                div(i,j,1:nlm,np,ksdm) = xrec4(ibeg:iend)
              enddo
              do np = 1,ntprog
                ibeg=iend+1
                iend = ibeg+nlm-1
                th(i,j,1:nlm,np,ksdm) = xrec4(ibeg:iend)
              enddo
              do nt = 1,nwater
                do np=1,ntprog
                  ibeg=iend+1
                  iend = ibeg+nlm-1
                  tracer(i,j,1:nlm,nt,np,ksdm) = xrec4(ibeg:iend)
                enddo
              enddo
#ifdef eau_ng
              do nt = 1, nelqng
                 do np = 1, ntprog
                    ibeg = iend+1
                    iend = ibeg+nlm-1
                    elqng(i,j,1:nlm,nt,np,ksdm) = xrec4(ibeg:iend)
                 enddo
              enddo
              do nt = 1, nelqngp1
                 do np = 1, ntprog
                    ibeg = iend+1
                    iend = ibeg+nlmp1-1
                    elqngp1(i,j,1:nlmp1,nt,np,ksdm) = xrec4(ibeg:iend)                    
                 enddo
              enddo
#endif
            enddo
          enddo
        enddo
        deallocate(xrec4)
        
        else if(8*lreclw.eq.lrecl8) then

        lrecl = lrecl8
        tau = tau_tem8
        open(4,file=trim(datapath)//'data/ic_atm',form='unformatted'
     *,      access='DIRECT', recl=lrecl)
        allocate( xrec8( lrecl/kind(xrec8) ) )
        do j = 1,jjm
          do i = 1,iim
            do ksdm = 1,nsdm
              read(4,rec=1+grid_center_index(i,j,ksdm)) xrec8

!-------------------------------------------------------------------------
!  load the data from one grid point into the model arrays
!-------------------------------------------------------------------------
              ibeg=1
              iend=1+ntprog-1
              ps(i,j,1:ntprog,ksdm) = xrec8(ibeg:iend)
#ifndef no_pbl
              if(.not.psc_to_gsc) then
                ibeg=iend+1
                iend = ibeg+ntprog-1
                pb(i,j,1:ntprog,ksdm) = xrec8(ibeg:iend)
              endif
#endif
              do np = 1,ntprog
                ibeg=iend+1
                iend = ibeg+nlm-1
                eta(i,j,1:nlm,np,ksdm) = xrec8(ibeg:iend)
              enddo
              do np = 1,ntprog
                ibeg=iend+1
                iend = ibeg+nlm-1
                div(i,j,1:nlm,np,ksdm) = xrec8(ibeg:iend)
              enddo
              do np = 1,ntprog
                ibeg=iend+1
                iend = ibeg+nlm-1
                th(i,j,1:nlm,np,ksdm) = xrec8(ibeg:iend)
              enddo
              do nt = 1,nwater
                do np=1,ntprog
                  ibeg=iend+1
                  iend = ibeg+nlm-1
                  tracer(i,j,1:nlm,nt,np,ksdm) = xrec8(ibeg:iend)
                enddo
              enddo
#ifdef eau_ng
              do nt = 1, nelqng
                 do np = 1, ntprog
                    ibeg = iend+1
                    iend = ibeg+nlm-1
                    elqng(i,j,1:nlm,nt,np,ksdm) = xrec8(ibeg:iend)
                 enddo
              enddo
              do nt = 1, nelqngp1
                 do np = 1, ntprog
                    ibeg = iend+1
                    iend = ibeg+nlmp1-1
                    elqngp1(i,j,1:nlmp1,nt,np,ksdm) = xrec8(ibeg:iend)                    
                 enddo
              enddo
#endif
            enddo
          enddo
        enddo
        deallocate(xrec8)

        else
          if(my_task.eq.0)print*,'unable to determine type of ic_atm - stopping'
          stop
        endif
        
        close(4)
        
!-- August, 2007:  Ensure that the PBL water vapor mixing ratio is positive
!--   and non-zero
        where(tracer(:,:,nlm,:,:,:) <= c0)
          tracer(:,:,nlm,:,:,:) = 1.e-10_dbl_kind
        endwhere

!-------------------------------------------------------------------------
! read passive tracers (if any)
!-------------------------------------------------------------------------
        if(ntracer.gt.nwater) then
          open(4,file=trim(datapath)//'data/ic_tracer',form='unformatted'
     *,        access='DIRECT', recl=36, iostat=ioerr, status='OLD')

!-------------------------------------------------------------------------
!  test the error code for opening the ic_tracer file
!-------------------------------------------------------------------------
          if(ioerr.eq.0) then
!-------------------------------------------------------------------------
!  get the header record
!-------------------------------------------------------------------------
            read(4,rec=1)nsecond_tem,step_count_tem,tau_tem4,max_ig_tem,nlm_tem
     *,                ntracer_tem,ntprog_tem,lrecl4
            read(4,rec=1)nsecond_tem,step_count_tem,tau_tem8,max_ig_tem,nlm_tem
     *,                ntracer_tem,ntprog_tem,lrecl8
            lreclw = ntprog*( nlm*(ntracer-nwater))
            close(4)

!-------------------------------------------------------------------------
!  get the data records
!-------------------------------------------------------------------------
            if(4*lreclw.eq.lrecl4) then

            lrecl = lrecl4
            open(4,file=trim(datapath)//'data/ic_tracer',form='unformatted'
     *,          access='DIRECT', recl=lrecl, iostat=ioerr)
            allocate( xrec4( lrecl/kind(xrec4) ) )
            do j = 1,jjm
              do i = 1,iim
                do ksdm = 1,nsdm
                  read(4,rec=1+grid_center_index(i,j,ksdm)) xrec4

!-------------------------------------------------------------------------
!  load the data from one grid point into the model arrays
!-------------------------------------------------------------------------
                  iend=0
                  do nt = nwater+1,ntracer
                    do np=1,ntprog
                      ibeg=iend+1
                      iend = ibeg+nlm-1
                      tracer(i,j,1:nlm,nt,np,ksdm) = xrec4(ibeg:iend)
                    enddo
                  enddo
                enddo
              enddo
            enddo
            deallocate(xrec4)
            close(4)

            elseif(8*lreclw.eq.lrecl8) then

            lrecl = lrecl8

            open(4,file=trim(datapath)//'data/ic_tracer',form='unformatted'
     *,          access='DIRECT', recl=lrecl, iostat=ioerr)
            allocate( xrec8( lrecl/kind(xrec8) ) )
            do j = 1,jjm
              do i = 1,iim
                do ksdm = 1,nsdm
                  read(4,rec=1+grid_center_index(i,j,ksdm)) xrec8

!-------------------------------------------------------------------------
!  load the data from one grid point into the model arrays
!-------------------------------------------------------------------------
                  iend=0
                  do nt = nwater+1,ntracer
                    do np=1,ntprog
                      ibeg=iend+1
                      iend = ibeg+nlm-1
                      tracer(i,j,1:nlm,nt,np,ksdm) = xrec8(ibeg:iend)
                    enddo
                  enddo
                enddo
              enddo
            enddo
            deallocate(xrec8)
            close(4)

            else
              if(my_task.eq.0) print *,'cant identify ic_tracer file type'
              if(my_task.eq.0) print *,'initializing tracer to constant'
              do nt = nwater+1,ntracer
                tracer(:,:,:,nt,:,:) = 12.*0.000350/29.
              enddo
            endif
          else
!-------------------------------------------------------------------------
!   initialize tracer to 350 ppmv carbon if there is no ic_tracer file
!-------------------------------------------------------------------------
            if(my_task.eq.0) print *,'error opening ic_tracer file'
            if(my_task.eq.0) print *,'initializing tracer to constant'
            do nt = nwater+1,ntracer
              tracer(:,:,:,nt,:,:) = 12.*0.000350/29.
            enddo
          endif
        
	endif

!-------------------------------------------------------------------------
! set step_count equal to zero, so the model begins with an Euler Forward
!-------------------------------------------------------------------------
        step_count = 0

#ifndef no_pbl
!-------------------------------------------------------------------------
! reset pb if converting from a Phillips Sigma coordinate to Generalized
!-------------------------------------------------------------------------
	if(psc_to_gsc) pb = 0.9*ps
#endif

!-------------------------------------------------------------------------
! get pointers
!-------------------------------------------------------------------------
	call get_pointers

!-----------------------------------------------------------------------
! compute relative vorticity
!-----------------------------------------------------------------------
        do k=1,nlm
          relative(:,:,k,:) = eta(:,:,k,n4,:) - f(:,:,:)
        enddo

!-------------------------------------------------------------------------
! solve for psi and chi
!-------------------------------------------------------------------------
        do k=1,nlm
          call elliptic(ipad,iim,jjm,1,nsdm,nweights,grid_center_mask(:,:,:),
     *       l_weights(:,:,:,:),relative(:,:,k,:),psi(:,:,k,:))
          call elliptic(ipad,iim,jjm,1,nsdm,nweights,grid_center_mask(:,:,:),
     *       l_weights(:,:,:,:),div(:,:,k,n4,:),chi(:,:,k,:))
        enddo

        call wrap( max_level,
     *             iim,
     *             jjm,
     *             nlm,
     *             nsdm,
     *             psi(:,:,:,:))
        call wrap( max_level,
     *             iim,
     *             jjm,
     *             nlm,
     *             nsdm,
     *             chi(:,:,:,:))

!---------------------------------------------------------------------------
! compute velocity and kinetic energy
!---------------------------------------------------------------------------
	do k=1,nlm
	do ksdm=1,nsdm
        call calculate_velocity(                                
     *          ipad,                                                 
     *          iim,                                                   
     *          jjm,                                                   
     *          ndata,                                                 
     *          nspace,                                                
     *          ncorners,                                              
     *          corner_area_inv(:,:,:,ksdm),                                       
     *          v_weights(:,:,:,:,:,ksdm),                                             
     *          psi(:,:,k,ksdm),                                                   
     *          chi(:,:,k,ksdm),                                                   
     *          u  (:,:,:,:,k,n4,ksdm) )
	enddo
	enddo

        call calculate_u_and_ke(
     *            nlm                   ,
     *            psi   (    :,:,:,   :),
     *            chi   (    :,:,:,   :),
     *            u     (:,:,:,:,:,n4,:),
     *            ke    (    :,:,:,   :) )


        end subroutine read_restart
!======================================================================
! END OF READ_RESTART
!======================================================================


!===========================================================================
! BEGINNING OF GET_POINTERS
!===========================================================================
        subroutine get_pointers

!----------------------------------------------------------------------------
! pointers hardwired for Adams Bashforth 3rd Order integration
!----------------------------------------------------------------------------
        n4   = MOD (step_count+1,2) + 1 ! {2,1}
        n3   = MOD (step_count+0,2) + 1 ! {1,2}
        n2   = 0
        n3_f = MOD (step_count+2,3) + 1 ! {3,1,2}
        n2_f = MOD (step_count+1,3) + 1 ! {2,3,1}
        n1_f = MOD (step_count+0,3) + 1 ! {1,2,3}

        end subroutine get_pointers
!===========================================================================
! END OF GET_POINTERS
!===========================================================================



!===========================================================================
! BEGINNING OF GET_WEIGHTS
!===========================================================================
        subroutine get_weights

        implicit none

        real (kind=dbl_kind), parameter ::
     *                  w3_ab3 = 1.91666666666666666666666666666666,
     *                  w2_ab3 =-1.33333333333333333333333333333333,
     *                  w1_ab3 = 0.41666666666666666666666666666666

        if (step_count.LE.1) then ! Euler forward time step
          w3 =  1.0
          w2 =  0.0
          w1 =  0.0
        endif
        if (step_count.EQ.2) then ! AB 2nd order time step
          w3 =  1.5
          w2 = -0.5
          w1 =  0.0
        endif
        if (step_count.GE.3) then ! AB 3rd order time step
          w3 = w3_ab3
          w2 = w2_ab3
          w1 = w1_ab3
        endif

        end subroutine get_weights
!===========================================================================
! END OF GET_WEIGHTS
!===========================================================================

!===========================================================================
! BEGINNING OF GET_WEIGHTS_SCM
!===========================================================================
        subroutine get_weights_scm

        implicit none

          w3 =  1.0
          w2 =  0.0
          w1 =  0.0

        end subroutine get_weights_scm
!===========================================================================
! END OF GET_WEIGHTS_SCM
!===========================================================================


!======================================================================
! BEGINNING OF READ_ATM_BC
!======================================================================
        subroutine read_atm_bc(iunit)

        implicit none

        integer (kind = int_kind) :: iunit,i,j, max_ig_tem, lrecl, wrecl, n
        character(char_len) filename
        real (kind=real_kind), allocatable, dimension(:) :: xrec

	phis = c0
	f = c0
	tpgvar = c0

!-----------------------------------------------------------------------
! open correct file based on grid size
!-----------------------------------------------------------------------
        filename=' '
        if(max_ig.eq.02562) then
           filename=trim(datapath)//'data/bc_atm_02562'
         elseif(max_ig.eq.10242) then
           filename=trim(datapath)//'data/bc_atm_10242'
         elseif(max_ig.eq.40962) then
           filename=trim(datapath)//'data/bc_atm_40962'
         elseif(max_ig.eq.00001) then
           filename=trim(datapath)//'data/bc_atm_00001'
         else
           stop 'read_atm_bc stop'
        endif
        filename=trim(filename)

!-----------------------------------------------------------------------
! open file, check error status
!-----------------------------------------------------------------------

          open(unit=iunit,file=filename,form='unformatted',
     *          status='old',iostat=ioerr,access='DIRECT',recl=8)
          if(ioerr.ne.0) then
            write(6,*) ' read_atm_bc, error open ',ioerr
            stop
          endif

!-------------------------------------------------------------------------
!  get the header record
!-------------------------------------------------------------------------
          read(iunit,rec=1)max_ig_tem,lrecl

          close(iunit)

          open(unit=iunit,file=filename,form='unformatted',
     *          status='old',iostat=ioerr,access='DIRECT',recl=lrecl)

          wrecl = lrecl/kind(xrec)
          allocate(xrec(wrecl))

        do j = 1,jjm
          do i = 1,iim
            do ksdm = 1,nsdm
              read(iunit,rec=1+grid_center_index(i,j,ksdm)) xrec
              
!-----------------------------------------------------------------------
! read in Coriolis parameter
!-----------------------------------------------------------------------
              f(i,j,ksdm) = xrec(2)

#ifndef held_suarez
!-----------------------------------------------------------------------
! read in surface geopotentials
!-----------------------------------------------------------------------
              phis(i,j,ksdm) = xrec(1)

!-----------------------------------------------------------------------
! read in topographic variance
!-----------------------------------------------------------------------
              if(wrecl.ge.3) tpgvar(i,j,ksdm) = xrec(3)
#endif

            enddo
          enddo
        enddo

          write(6,*) ' bc, proc ',my_task,minval(phis),maxval(phis)
          write(6,*) ' bc, proc ',my_task,minval(f),maxval(f)
          write(6,*) ' bc, proc ',my_task,minval(tpgvar),maxval(tpgvar)
#ifdef gravity_wave_drag
!-------------------------------------------------------------------------
! average to cell walls
!-------------------------------------------------------------------------
	do ksdm = 1,nsdm
        call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  tpgvar(:,:,ksdm),
     *                  tpgvar_c(:,:,:,ksdm))
	enddo

#endif

          deallocate(xrec)

!-----------------------------------------------------------------------
! close file
!-----------------------------------------------------------------------
          close(iunit)


!-------------------------------------------------------------------------
! Get tracer boundary conditions
!-------------------------------------------------------------------------

#ifdef passive_tracers
!-----------------------------------------------------------------------
! open file, check error status
!-----------------------------------------------------------------------

          open(unit=3,file=trim(datapath)//'data/bc_co2',form='unformatted',
     *          status='old',iostat=ioerr,access='DIRECT',recl=8)
          if(ioerr.ne.0) then
            write(6,*) ' bc_co2, error open ',ioerr
            tracer_dat = c0
          else

!-------------------------------------------------------------------------
!  get the header record
!-------------------------------------------------------------------------
            read(3,rec=1)max_ig_tem,lrecl

            close(3)

            open(unit=3,file=trim(datapath)//'data/bc_co2',form='unformatted',
     *          status='old',iostat=ioerr,access='DIRECT',recl=lrecl)
            wrecl = lrecl/kind(xrec)
            allocate(xrec(wrecl))
            tracer_dat = c0

            do j = 1,jjm
              do i = 1,iim
                do ksdm = 1,nsdm
                   read(3,rec=1+grid_center_index(i,j,ksdm)) xrec
                   do n = 1,min(npassive,wrecl/12)
                     tracer_dat(i,j,n,1:12,ksdm) = xrec(12*(n-1)+1:12*n)
                   enddo
                enddo
              enddo
            enddo

            deallocate(xrec)

!-----------------------------------------------------------------------
! close file
!-----------------------------------------------------------------------
          endif
          close(3)
#endif


        end subroutine read_atm_bc

!======================================================================
! END OF READ_ATM_BC
!======================================================================

!===========================================================================
! BEGINNING OF WRITE_BC
!===========================================================================
        subroutine write_bc

        implicit none
        real (kind = real_kind), dimension(6) :: work
        integer lrecl, i, j, ksdm

        lrecl = 6 * kind(work)
	open(unit=3, file = trim(datapath)//'bcdata.out', form='unformatted'
     *,       access='DIRECT', recl = lrecl)

        write(3,rec=1)max_ig,lrecl

        do j = 1,jjm
          do i = 1,iim
            do ksdm = 1,nsdm
              if(grid_center_mask(i,j,ksdm).gt.0.5_dbl_kind) then
                work = c0
                work(1) = phis(i,j,ksdm)
                work(2) = f(i,j,ksdm)
                work(3) = tpgvar(i,j,ksdm)
                work(4) = c1/area_inv(i,j,ksdm)
                work(5) = grid_center_lon(i,j,ksdm)
                work(6) = grid_center_lat(i,j,ksdm)
                write(3,rec=1+grid_center_index(i,j,ksdm))work
              endif
            enddo
          enddo
        enddo


	close(3)

	end subroutine write_bc
!===========================================================================
! END OF WRITE_BC
!===========================================================================


!===========================================================================
! BEGINNING OF WRITE_PROGNOSTICS
!===========================================================================
	subroutine write_prognostics

        implicit none

	real (kind = dbl_kind), dimension(im,jm) :: work, workt
	real (kind = dbl_kind), dimension(im,jm,nlm) :: work1
	real (kind = dbl_kind), dimension(iim,jjm,1,nlm) :: work2=c0
	real (kind = dbl_kind), dimension(iim,jjm,nlm,nsdm) :: work_del
        integer (kind=int_kind) ::  k,nt
	integer (kind=int_kind), save :: irecord=0

	if(my_task.eq.0) then
	  irecord = irecord + 1
	  write(6,*)
	  write(6,*) ' writing prognostics at time : ',tau,irecord
	  write(6,*)
	endif


	work = c0
	call gather_fine(ps(:,:,n4,:),work(:,:))
	if(my_task.eq.0) call interp_driver_2d(work, interp_file)

#ifndef no_pbl
	work = c0
        call gather_fine(pb(:,:,n4,:),work(:,:))
	if(my_task.eq.0) call interp_driver_2d(work, interp_file)
#endif
        work1 = c0
	do k=1,nlm
        call gather_fine(relative(:,:,k,:),work1(:,:,k))
	enddo
        if(my_task.eq.0) call interp_driver_3d(work1(1,1,1), nlm, interp_file)

        work1 = c0
        do k=1,nlm
        call gather_fine(div(:,:,k,n4,:),work1(:,:,k))
        enddo
        if(my_task.eq.0) call interp_driver_3d(work1(1,1,1), nlm, interp_file)

        work1 = c0
        do k=1,nlm
        call gather_fine(th(:,:,k,n4,:),work1(:,:,k))
        enddo
        if(my_task.eq.0) call interp_driver_3d(work1(1,1,1), nlm, interp_file)

        work1 = c0
	do nt=1,ntracer
        do k=1,nlm
        call gather_fine(tracer(:,:,k,nt,n4,:),work1(:,:,k))
        enddo
        if(my_task.eq.0) call interp_driver_3d(work1(1,1,1), nlm, interp_file)
	enddo

        work = c0
        workt = c0
        do k = 1,nlm
          call gather_fine(pthick(:,:,k,n4,:),work1(:,:,k))
        enddo
        do nt = 1,ntracer
          workt = c0
          do k = 1,nlm
            call gather_fine(tracer(:,:,k,nt,n4,:),workt(:,:))
            work(:,:) = work(:,:) + workt(:,:) * work1(:,:,k) * dsig(k)/grav
          enddo
        enddo
        if(my_task.eq.0) call interp_driver_2d(work, interp_file)

        work1 = c0
        do k=1,nlm
        call gather_fine(ke(:,:,k,:),work1(:,:,k))
        enddo
        if(my_task.eq.0) call interp_driver_3d(work1(1,1,1), nlm, interp_file)

        work1 = c0
        do k=1,nlm
        call gather_fine(dsigdt(:,:,k,:),work1(:,:,k))
        enddo
        if(my_task.eq.0) call interp_driver_3d(work1(1,1,1), nlm, interp_file)

        work1 = c0
        do k=1,nlm
        call gather_fine(psi(:,:,k,:),work1(:,:,k))
        enddo
        if(my_task.eq.0) call interp_driver_3d(work1(1,1,1), nlm, interp_file)

        work1 = c0
        do k=1,nlm
        call gather_fine(chi(:,:,k,:),work1(:,:,k))
        enddo
        if(my_task.eq.0) call interp_driver_3d(work1(1,1,1), nlm, interp_file)

	do k=1,nlm
	do ksdm=1,nsdm
!         call laplacian(
!    *       psi      (:,:  ,k, ksdm),
!    *       ldl_fine (:,:,:,   ksdm),
!    *       areai    (:,:,     ksdm),
!    *       work_del (:,:  ,k, ksdm) )
	enddo
	enddo
        work1 = c0
	work_del = c0
	do k=1,nlm
        call gather_fine(work_del(:,:,k,:),work1(:,:,k))
	enddo
        if(my_task.eq.0) call interp_driver_3d(work1(1,1,1), nlm, interp_file)

	do k=1,nlm
	do ksdm=1,nsdm
!         call laplacian(
!    *       chi      (:,:  ,k, ksdm),
!    *       ldl_fine (:,:,:,   ksdm),
!    *       areai    (:,:,     ksdm),
!    *       work_del (:,:  ,k, ksdm) )
	enddo
	enddo
        work1 = c0
	work_del = c0
	do k=1,nlm
        call gather_fine(work_del(:,:,k,:),work1(:,:,k))
	enddo
        if(my_task.eq.0) call interp_driver_3d(work1(1,1,1), nlm, interp_file)

        work = c0
        call gather_fine(ps_f(:,:,n3_f,:),work(:,:))
        if(my_task.eq.0) call interp_driver_2d(work, interp_file)

#ifndef no_pbl
        work = c0
        call gather_fine(pb_f(:,:,n3_f,:),work(:,:))
        if(my_task.eq.0) call interp_driver_2d(work, interp_file)
#endif

        work1 = c0
        do k=1,nlm
        call gather_fine(div_mass_flux(:,:,k,n3_f,:),work1(:,:,k))
        enddo
        if(my_task.eq.0) call interp_driver_3d(work1(1,1,1), nlm, interp_file)

        work1 = c0
        do k=1,nlm
        call gather_fine(pthick(:,:,k,n4,:),work1(:,:,k))
        enddo
        if(my_task.eq.0) call interp_driver_3d(work1(1,1,1), nlm, interp_file)

        work1 = c0
        do k=1,nlm
        work_del(:,:,k,:) = th_src(:,:,k,:) * pthick(:,:,k,n3,:)
        call gather_fine(work_del(:,:,k,:),work1(:,:,k))
        enddo
        if(my_task.eq.0) call interp_driver_3d(work1(1,1,1), nlm, interp_file)

        work1 = c0
        do k=1,nlm
        call gather_fine(th_res(:,:,k,:),work1(:,:,k))
        enddo
        if(my_task.eq.0) call interp_driver_3d(work1(1,1,1), nlm, interp_file)

        do nt=1,ntracer
	work1 = c0
	do k=1,nlm
	call gather_fine(tracer_src(:,:,k,nt,:),work1(:,:,k))
	enddo
        if(my_task.eq.0) call interp_driver_3d(work1(1,1,1), nlm, interp_file)
        enddo

#ifndef no_pbl
        work = c0
        call gather_fine(entrainment(:,:,:),work(:,:))
        if(my_task.eq.0) call interp_driver_2d(work(1,1), interp_file)

        work = c0
        call gather_fine(cumulusmf(:,:,:),work(:,:))
        if(my_task.eq.0) call interp_driver_2d(work(1,1), interp_file)

        work = c0
        call gather_fine(pleak(:,:,:),work(:,:))
        if(my_task.eq.0) call interp_driver_2d(work(1,1), interp_file)

        work = c0
        call gather_fine(drag(:,:,:),work(:,:))
        if(my_task.eq.0) call interp_driver_2d(work(1,1), interp_file)
#endif
#ifdef mpi
        call MPI_BARRIER(MPI_COMM_ATMOS,ierr)
#endif

	end subroutine write_prognostics

!===========================================================================
! END OF WRITE_PROGNOSTICS
!===========================================================================



!===========================================================================
! BEGINNING OF OPEN_FILES
!===========================================================================
        subroutine open_files
	if(my_task.eq.0) then
          OPEN (UNIT= interp_file, FILE=trim(datapath)//'fields.out',
     &         FORM='UNFORMATTED',STATUS='UNKNOWN')
	endif
	end subroutine open_files
!===========================================================================
! END OF OPEN_FILES
!===========================================================================


!===========================================================================
! BEGINNING OF INPUT
!===========================================================================
      SUBROUTINE input
 
! shallow water version
! written by Ross Hiekes, CSU
 
! called from
!	dcontrol (currently main)
 
! load run time parameter settings
 
! variable definitions
 
!  character strings
!	test case = simulation_tag
 
!  integer scalar
!	icase = case number
!	step_count_end = step at which to halt execution
!	count = integer counter of grid cells
!	panel = ranges 0 to 4, used in summing processes to avoid including 'wrap' cells
!	i = dummy index
!	j = dummy index
 
!  real scalar
!	tau_end = time to end execution, if set equal to 0.0, tau_end=24.0*tau_end_day+tau_end_hour
!	tau_end_day = time in days to end execution
!	tau_end_hour = time in hours to end execution
!	output{1,2,3}_freq = write output at various frequencies {1,2,3} (hours)
!	restart_freq = frequence which to write restarts (hours)
 
!  logicals
!	l_restart : T implies to BEGIN MODEL READING FROM RESTART FILE call "restart"
 
 
	implicit none

! these need to be cleaned up and removed
	logical l_tau_timer
	integer step_count_end

!-----------------------------------------------------------------
! READ NAMELIST
! namelist is opened as unit 2

      if(my_task.eq.0) then
        OPEN (UNIT=2,FILE=trim(datapath)//'data/namelist',
     *        FORM='FORMATTED',STATUS='OLD')
        READ (2,01) simulation_tag
        READ (2,*) icase
        READ (2,02) tau_end
        READ (2,02) tau_end_day
        READ (2,02) tau_end_hour
        READ (2,02) dt
        READ (2,*) nvcycle
        READ (2,02) visc
        READ (2,04) l_restart
	READ (2,*) n_restart
        READ (2,*) n_output
        READ (2,*) n_diagnostics
        READ (2,*) n_timer
        READ (2,*) n_renorm
        READ (2,*) n_qp
        READ (2,*) n_pbp
#ifdef link_to_gcm_physics
        READ (2,*) n_physics
#else
        n_physics = 1
#endif
        READ (2,02) diff_coeff_pb

        CLOSE (UNIT=2)
   01   FORMAT (A40 )
   02   FORMAT (F7.2)
   03   FORMAT (I4  )
   04   FORMAT (L1  )

!----------------------------------------------------------------------
! compute time as the sum of days and hours if tau_end == 0.0
!----------------------------------------------------------------------
        IF (tau_end.EQ.0.0) tau_end = 24.*tau_end_day+tau_end_hour-0.001
 
! echo data to standard out
        PRINT *, ' input : simulation_tag = ',simulation_tag
        PRINT *, ' input : icase          = ',icase
        PRINT *, ' input : tau_end        = ',tau_end
        PRINT *, ' input : tau_end_day    = ',tau_end_day
        PRINT *, ' input : tau_end_hour   = ',tau_end_hour
        PRINT *, ' input : time step      = ',dt
        PRINT *, ' input : nvcycle        = ',nvcycle
        PRINT *, ' input : visc           = ',visc
        PRINT *, ' input : l_restart      = ',l_restart      

	invdt = c1/dt
#ifdef link_to_gcm_physics
        dtphy = dt * n_physics
#endif

      endif 	! if(my_task.eq.0)

#ifdef mpi
        call MPI_BCAST(tau_end,1,mpi_float_type,
     *  0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(tau_end_day,1,mpi_float_type,
     *  0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(tau_end_hour,1,mpi_float_type,
     *  0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(dt,1,mpi_float_type,
     *  0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(invdt,1,mpi_float_type,
     *  0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(nvcycle,1,mpi_int_type,
     *  0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(visc,1,mpi_float_type,
     *  0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(step_count_end,1,mpi_int_type,
     *  0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(l_tau_timer,1,MPI_LOGICAL,
     *  0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(l_restart,1,MPI_LOGICAL,
     *  0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(n_restart,1,mpi_int_type,
     *  0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(n_diagnostics,1,mpi_int_type,
     *  0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(n_timer,1,mpi_int_type,
     *  0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(n_output,1,mpi_int_type,
     *  0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(n_renorm,1,mpi_int_type,
     *  0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(n_qp,1,mpi_int_type,
     *  0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(n_pbp,1,mpi_int_type,
     *  0,MPI_COMM_ATMOS,ierr)

#ifdef link_to_gcm_physics
        call MPI_BCAST(n_physics,1,mpi_int_type,
     *  0,MPI_COMM_ATMOS,ierr)

        call MPI_BCAST(dtphy,1,mpi_float_type,
     *  0,MPI_COMM_ATMOS,ierr)
#endif

#endif

	end subroutine input
!===========================================================================
! END OF INPUT
!===========================================================================

!===========================================================================
! BEGINNING OF ECHO_DIRECTIVES
!===========================================================================
        subroutine echo_directives

	if(my_task.eq.0) then

        write(6,*) ' parameter settings for this run'
        write(6,*)

#if resolution == 02562
        write(6,*) '   resolution is 02562'
#endif

#if resolution == 10242
        write(6,*) '   resolution is 10242'
#endif

        write(6,*) '   time stepping is explicit'

#ifndef no_pbl
        write(6,*) '   generalized sigma coordinate is used'
#else
        write(6,*) '   phillips sigma coordinate is used'
#endif

#ifdef no_stratosphere
        write(6,*) '   there is no stratosphere zone'
#endif

#ifdef vertical_upwind_pbl
        write(6,*) '   the pbl tracers is vertically upwinded'
#endif

#ifdef vertical_upwind_pbl
        write(6,*) '   the pbl momentum is vertically upwinded'
#endif

#ifndef no_pbl
#ifdef pbl_depth_limiter
        write(6,*) '   the pbl is depth limited'
#else
        write(6,*) '   the pbl is linearly restored'
#endif
#endif

#ifdef held_suarez
        write(6,*) '   Held-Saurez thermal forcing'
#endif

#ifdef held_suarez
        write(6,*) '   Held-Saurez momentum forcing'
#endif

#ifdef csu_momentum
        write(6,*) '   CSU momentum forcing'
#endif

#ifndef no_pbl
#ifdef link_to_gcm_physics
        write(6,*) '   code is linked to BUGS gcm'
#endif
#endif

#ifdef del4
        write(6,*) '   del4 is turned on'
#endif

        write(6,*) '   dry convective adjustment is turned on'

#ifdef fluxcorrectedtransport
        write(6,*) '   tracer flux_corrected_transport is turned on'
#endif

#ifndef no_pbl
#ifdef gravity_wave_drag
        write(6,*) '   gravity wave drag is turned on'
#endif
#endif   

#ifdef passive_tracers
        write(6,*) '   transport of passive tracers is turned on'
#endif

#ifndef no_pbl
#ifdef eau_ng
        write(6,*) '   eau_ng is turned on'
        write(6,*) '   eau_ng variables are vertically upwinded'
#endif   
#endif   
	endif 	! if(my_task.eq.0)

#ifdef mpi
	call MPI_BARRIER(MPI_COMM_ATMOS,ierr)
#endif

	end subroutine echo_directives
!===========================================================================
! END ECHO_DIRECTIVES
!===========================================================================

	end module model_control

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
