!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||rameters.F


	module model_parameters

!-----------------------------------------------------------------
!
!	this modules specifies parameters related to the model
!	  some of these parameters are specifed at compile time,
!	  while others specified at run time.
!
!---- modifications:
!     added the parameters elqng and elqngp1 when the parameteri
!     zation of fractional cloudiness is turned on.
!     Laura D. Fowler/mcbug (09-06-02).

!-----------------------------------------------------------------

	use kinds
	use physical_parameters
	
	implicit none
	save

        logical, parameter ::                          &
                        l_velocity    = .false. ,      & ! use momentum formulation
                        l_filter      = .false. ,      & ! compute filter (Z-grid) velocity
                        l_replace     = .false.          ! replace velocity w/ filtered velocity

	real (kind=dbl_kind) ::                       &
      			dt,                           & ! the time step in seconds
      			invdt,                        & ! 1/dt
      			dtphy,                        &
      			visc,                         &
      			diff_coeff_pb,                &
      			total_area                      ! the total area of the sphere

        real (kind=dbl_kind) ::                      &
                        w3,                          &  ! the AB3 weight for the time level n   tendency
                        w2,                          &  ! the AB3 weight for the time level n-1 tendency
                        w1                              ! the AB3 weight for the time level n-2 tendency
                                                        ! AB3 == Adams Bashforth 3rd order time stepping scheme

        integer (kind=int_kind) ::                   &
                        n4,                          &  ! pointer to the time level n+1 slot (prognostic quantities)
                        n3,                          &  ! pointer to the time level n   slot (prognostic quantities)
                        n2,                          &  ! pointer to the time level n-1 slot (prognostic quantities)
                        n3_f,                        &  ! pointer to the time level n   slot (tendency quantities)
                        n2_f,                        &  ! pointer to the time level n-1 slot (tendency quantities)
                        n1_f                            ! pointer to the time level n-2 slot (tendency quantities)

        integer (kind=int_kind) ::                   &
                        nvcycle                         ! number of v-cycles per time step


	logical (kind=log_kind), parameter ::        &
      			l_grid_even=.true.              ! true for grids generated by sucessive bisection

	integer (kind=int_kind), parameter ::        &
      			ntracer = 7,                 &  ! place holder to the number of tracer fields to integrate
      			ntprog = 2,                  &  ! number of time slots needed for prognostic variables
      			nttend = 3                      ! number of time slots needed for tendency variables

#ifdef eau_ng
        integer (kind=int_kind), parameter :: &
                        nelqng   = 13
        integer (kind=int_kind), parameter :: &
                        nelqngp1 = 3
#endif

#if resolution == 00642
 	integer (kind=int_kind), parameter ::        &
      			max_level = 3,               & ! level of recursion
      			im =  90,                    & ! total number of points in the i-direction
      			jm = 10,                     & ! total number of points in the j-direction
                        grid_max1 =   420,           & ! multigrid info
      		        grid_max2 =  1320,           & ! multigrid info
      			max_ig=00642                   ! total number of grid cells
#endif

#if resolution == 02562
	integer (kind=int_kind), parameter ::        &
                        max_level = 4,               &
                        im =  170,                   &
                        jm = 18,                     &
                        grid_max1 =  1320,           &
                        grid_max2 =  4380,           &
                        max_ig=02562              
#endif

#if resolution == 10242
        integer (kind=int_kind), parameter ::        &
                        max_level = 5,               &
                        im =  330,                   &
                        jm = 34,                     &
                        grid_max1 =  4380,           &
                        grid_max2 =  15600,          &
                        max_ig=10242      
#endif

#if resolution == 40962
        integer (kind=int_kind), parameter ::        &
                        max_level = 6,               &
                        im =  650,                   &
                        jm = 66,                     &
                        grid_max1 =  15600,          &
                        grid_max2 =  58500,          &
                        max_ig=40962
#endif

	integer (kind=int_kind), parameter ::          &
                        ncorners = 2,                  &! number of corners each grid cell owns
                        nspace = 2,                    &! number of spacial dimensions
                        ndata = 3,                     &! amount of data incorporated in stencils
                        nweights = 7,                  &! amount of data incorporated in Laplacian
                        nweights_zm = 4,               &! amount of data in the "corner" Laplacian
                        max_ig_corners = 2*(max_ig-2), &! number of real corners
                        r3 = 3,                        &! 3-space
                        r2 = 2                          ! 2-space

        integer (kind=int_kind), parameter ::        &
                        im5 = im/5,                  &  ! number of cells in i-direction per panel
                        im_jm=im*jm,                 &  ! total number of cells, including ghost cells
                        nwalls = 3                      ! number of walls owned per grid cell
 
!-----------------------------------------------------------------------
!   VARIABLE DEFINITION
!-----------------------------------------------------------------------
!
!	dt  		:  time step (seconds)
!	dtphy		:  physics time step (seconds)
!	visc		:  value of del4 viscosity/diffusion (m4/s)
!	diff_coeff_pb   :  value of del4 diffusion of pb (m4/s)
!	nvcycle		:  number of V cycles to do each time step
!	total_area	:  sum of area_mask
!	n{4,3,2}	:  pointers to time level {n+1,n,n-1} for prognostics
!	n{3,2,1}_f	:  pointers to time level {n,n-1,n-2} for time tendency arrays
!-----------------------------------------------------------------------

!***********************************************************************

	contains

!***********************************************************************


!======================================================================
! BEGINNING OF INIT_MODEL_PARAMETERS
!======================================================================

	subroutine init_model_parameters

	implicit none

	dt = c0
	dtphy = c0
	invdt = c0
	visc = c0
	diff_coeff_pb = 1.e16_dbl_kind
	total_area = c0
	w3 = c0
	w2 = c0
	w1 = c0
	n4 = 0
	n3 = 0
	n2 = 0
	n3_f = 0
	n2_f = 0
	n1_f = 0
	nvcycle = 0

	end subroutine init_model_parameters

!======================================================================
! BEGINNING OF INIT_MODEL_PARAMETERS
!======================================================================

	end module model_parameters

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
