!-----------------------------------------------------------------------

	MODULE netcdf_geodesic

!-----------------------------------------------------------------------
!
! FIXME - add documentation
!
! History:
!    Note: These version numbers are intended to reflect changes to the
!          actual output, not logic or input changes.  When output is
!          changed, also increment module parameter "VERSION" below.
!
!    version 0.4: 
!       12/14/99: added land,sea,icefrac definition routines, 
!                 define_fracs and set_fracs.   
!                 fracs written for atm and co2 files. - brian
!
!    version 0.5: 
!       12/10/00: atmosphere routines for BUGS5 (mpi) model 
!                 3d fields are interpolated to isobaric (or isentropic) levels 
!                 - don
!
!-----------------------------------------------------------------------

       use kinds
       use netcdf_general_tools

       implicit none
       private

       character(len=8), parameter :: GRIDTYPE = "geodesic"
       character(len=3), parameter :: VERSION = "0.5"


       type(meta_type), save :: gatm3, gatm2, gsfc2, gatmhf, gsfchf, gatm3ml


       integer (kind=int_kind) :: status, n

       public :: ginit_atm_nc, ginit_atm_nc_2, ginit_sfc_nc
     *,          gwrt_atm_nc, gwrt_atm_nc_2, gwrt_sfc_nc, gwrt_hf_nc
     *,          gwrt_hf_sfc_nc


	CONTAINS


!-----------------------------------------------------------------------

	SUBROUTINE ginit_atm_nc (run, outfile2, 
     *                           listatm2, nameatm2, unitsatm2, typeatm2,
     *                           doatm2, multfac2, contmin2, contint2,
     *                           altunits2, 
     *                           points, levels, pm, outfile3,
     *                           listatm3, nameatm3, unitsatm3, typeatm3,
     *                           doatm3,
     *                           outfilehf, listatmhf, nameatmhf, unitsatmhf,
     *                           typeatmhf, doatmhf,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners, max_ig, qpgridarea)

!-----------------------------------------------------------------------
! Purpose:
!    Public interface to geodesic grid initialization routine.
!-----------------------------------------------------------------------

      character(len=*), intent(in) :: run
      character(len=*), intent(in) :: outfile2
      character(len=*), dimension(:), intent(in) :: listatm2
      character(len=*), dimension(:), intent(in) :: nameatm2
      character(len=*), dimension(:), intent(in) :: unitsatm2
      character(len=*), dimension(:), intent(in) :: typeatm2
      real (kind=dbl_kind), dimension(:), intent(in) :: 
     *    multfac2, contmin2, contint2
      character(len=*), dimension(:), intent(in) :: altunits2
      logical (kind=log_kind), dimension(:), intent(in) :: doatm2
      integer (kind=int_kind), intent(in) :: points
      integer (kind=int_kind), intent(in) :: levels
      integer (kind=int_kind), intent(in) :: corners
      integer (kind=int_kind), intent(in) :: max_ig
      character(len=*), intent(in) :: outfile3
      character(len=*), dimension(:), intent(in) :: listatm3
      character(len=*), dimension(:), intent(in) :: nameatm3
	  character(len=*), dimension(:), intent(in) :: unitsatm3
	  character(len=*), dimension(:), intent(in) :: typeatm3
	  logical (kind=log_kind), dimension(:), intent(in) :: doatm3
	  character(len=*), intent(in) :: outfilehf
	  character(len=*), dimension(:), intent(in) :: listatmhf
	  character(len=*), dimension(:), intent(in) :: nameatmhf
	  character(len=*), dimension(:), intent(in) :: unitsatmhf
	  character(len=*), dimension(:), intent(in) :: typeatmhf
	  logical (kind=log_kind), dimension(:), intent(in) :: doatmhf

        real (kind=dbl_kind), dimension(levels), intent(in) :: pm
        real (kind=selected_real_kind(13)), dimension(corners,points),
     * intent(in) :: qpcornerlat, qpcornerlon
        real (kind=selected_real_kind(13)), dimension(points),
     * intent(in) :: qpcenterlat, qpcenterlon
        real (kind=selected_real_kind(13)), dimension(points),
     * intent(in), optional ::  qpgridarea
        logical (kind=log_kind), dimension(points) :: dumhr
        logical (kind=log_kind) :: dogridarea
        integer (kind=int_kind) :: ncfid
        real (kind=real_kind), dimension(levels) :: levelvals

        dumhr = .true.
        levelvals(1:levels) = pm(levels:1:-1)
        dogridarea = present(qpgridarea)

        gatm2 = set_meta (outfile2, "gatm", points, 0, 0)
        status = nf_create (trim(datapath)//outfile2, NF_CLOBBER, ncfid)
        call nc_init_geo (ncfid, run, gatm2, corners, max_ig, 
     *                    listatm2, nameatm2, unitsatm2, typeatm2, 
     *                    doatm2, dogridarea, 
     *                    multfac2, contmin2, contint2, altunits2)
        call define_fracs (ncfid)
        status = nf_enddef (ncfid)
        if(dogridarea) then
          call init_geo_coord (ncfid, gatm2, points,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners, qpgridarea=qpgridarea)
        else
          call init_geo_coord (ncfid, gatm2, points,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners)
        endif
        status = nf_close (ncfid)


        gatm3 = set_meta (outfile3, "gatm", points, 0, 0, levels)
        status = nf_create (trim(datapath)//outfile3, NF_CLOBBER, ncfid)
        call nc_init_geo (ncfid, run, gatm3, corners, max_ig, 
     *                    listatm3, nameatm3, unitsatm3, 
     *                    typeatm3, doatm3, dogridarea)
        call define_fracs (ncfid)
        status = nf_enddef (ncfid)
        if(dogridarea) then
          call init_geo_coord (ncfid, gatm3, points,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners, levelvals, qpgridarea)
        else
          call init_geo_coord (ncfid, gatm3, points,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners, levelvals)
        endif
        status = nf_close (ncfid)

        gatmhf = set_meta (outfilehf, "gatm", points, 0, 0)
        status = nf_create (trim(datapath)//outfilehf, NF_CLOBBER, ncfid)
        call nc_init_geo (ncfid, run, gatmhf, corners, max_ig, 
     *                    listatmhf, nameatmhf, unitsatmhf, 
     *                    typeatmhf, doatmhf, dogridarea)
        call define_fracs (ncfid)
        status = nf_enddef (ncfid)
        if(dogridarea) then
          call init_geo_coord (ncfid, gatmhf, points,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners, qpgridarea=qpgridarea)
        else
          call init_geo_coord (ncfid, gatmhf, points,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners)
        endif
        status = nf_close (ncfid)

	END SUBROUTINE ginit_atm_nc



!-----------------------------------------------------------------------

	SUBROUTINE ginit_atm_nc_2 (run, points, levels, 
     *                           outfile3,
     *                           listatm3, nameatm3, unitsatm3, typeatm3,
     *                           doatm3,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners, max_ig, qpgridarea)

!-----------------------------------------------------------------------
! Purpose:
!    Public interface to geodesic grid initialization routine.
!-----------------------------------------------------------------------

	character(len=*), intent(in) :: run
	integer (kind=int_kind), intent(in) :: points
	integer (kind=int_kind), intent(in) :: levels
	integer (kind=int_kind), intent(in) :: corners
	integer (kind=int_kind), intent(in) :: max_ig
	character(len=*), intent(in) :: outfile3
	character(len=*), dimension(:), intent(in) :: listatm3
	character(len=*), dimension(:), intent(in) :: nameatm3
	character(len=*), dimension(:), intent(in) :: unitsatm3
	character(len=*), dimension(:), intent(in) :: typeatm3
	logical (kind=log_kind), dimension(:), intent(in) :: doatm3

        real (kind=selected_real_kind(13)), dimension(corners,points),
     * intent(in) :: qpcornerlat, qpcornerlon
        real (kind=selected_real_kind(13)), dimension(points),
     * intent(in) :: qpcenterlat, qpcenterlon
        real (kind=selected_real_kind(13)), dimension(points),
     * intent(in), optional ::  qpgridarea
        logical (kind=log_kind), dimension(points) :: dumhr
        logical (kind=log_kind) :: dogridarea
        integer (kind=int_kind) :: ncfid,k
        real (kind=real_kind), dimension(levels) :: levelvals

        dumhr = .true.
        do k = 1,levels
          levelvals(k) = k
        enddo
        dogridarea = present(qpgridarea)

        gatm3ml = set_meta (outfile3, "gatm", points, 0, 0, levels)
        status = nf_create (trim(datapath)//outfile3, NF_CLOBBER, ncfid)
        call nc_init_geo (ncfid, run, gatm3ml, corners, max_ig, 
     *                    listatm3, nameatm3, unitsatm3, 
     *                    typeatm3, doatm3, dogridarea)
        call define_fracs (ncfid)
        status = nf_enddef (ncfid)
        if(dogridarea) then
          call init_geo_coord (ncfid, gatm3ml, points,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners, levelvals, qpgridarea)
        else
          call init_geo_coord (ncfid, gatm3ml, points,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners, levelvals)
        endif
        status = nf_close (ncfid)

	END SUBROUTINE ginit_atm_nc_2




!-----------------------------------------------------------------------

	SUBROUTINE ginit_sfc_nc (run, outfile2, 
     *                           listsfc2, namesfc2, unitssfc2, 
     *                           typesfc2, dosfc2,
     *                           points,
     *                           outfilehf, 
     *                           listsfchf, namesfchf, unitssfchf, 
     *                           typesfchf, dosfchf,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners, max_ig, qpgridarea)

!-----------------------------------------------------------------------
! Purpose:
!    Public interface to geodesic grid initialization routine.
!-----------------------------------------------------------------------

	character(len=*), intent(in) :: run
	character(len=*), intent(in) :: outfile2
	character(len=*), dimension(:), intent(in) :: listsfc2
	character(len=*), dimension(:), intent(in) :: namesfc2
	character(len=*), dimension(:), intent(in) :: unitssfc2
	character(len=*), dimension(:), intent(in) :: typesfc2
	logical (kind=log_kind), dimension(:), intent(in) :: dosfc2
	character(len=*), intent(in) :: outfilehf
	character(len=*), dimension(:), intent(in) :: listsfchf
	character(len=*), dimension(:), intent(in) :: namesfchf
	character(len=*), dimension(:), intent(in) :: unitssfchf
	character(len=*), dimension(:), intent(in) :: typesfchf
	logical (kind=log_kind), dimension(:), intent(in) :: dosfchf
	integer (kind=int_kind), intent(in) :: points
	integer (kind=int_kind), intent(in) :: corners
	integer (kind=int_kind), intent(in) :: max_ig

        real (kind=selected_real_kind(13)), dimension(corners,points),
     * intent(in) :: qpcornerlat, qpcornerlon
        real (kind=selected_real_kind(13)), dimension(points),
     * intent(in) :: qpcenterlat, qpcenterlon
        real (kind=selected_real_kind(13)), dimension(points),
     * intent(in), optional :: qpgridarea
        logical (kind=log_kind), dimension(points) :: dumhr
        logical (kind=log_kind) :: dogridarea
        integer (kind=int_kind) :: ncfid

        dumhr = .true.

        gsfc2 = set_meta (outfile2, "gsfc", points, 0, 0)
        status = nf_create (trim(datapath)//outfile2, NF_CLOBBER, ncfid)
        dogridarea = present(qpgridarea)
        call nc_init_geo (ncfid, run, gsfc2, corners, max_ig, 
     *                    listsfc2, namesfc2, unitssfc2, 
     *                    typesfc2, dosfc2, dogridarea )
        call define_indx (ncfid)
        status = nf_enddef (ncfid)
        if(dogridarea) then
           call init_geo_coord (ncfid, gsfc2, points,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners, qpgridarea=qpgridarea)
        else
           call init_geo_coord (ncfid, gsfc2, points,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners)
        endif
        status = nf_close (ncfid)

        gsfchf = set_meta (outfilehf, "gsfc", points, 0, 0)
        status = nf_create (trim(datapath)//outfilehf, NF_CLOBBER, ncfid)
        dogridarea = present(qpgridarea)
        call nc_init_geo (ncfid, run, gsfchf, corners, max_ig, 
     *                  listsfchf, namesfchf, unitssfchf, 
     *                  typesfchf, dosfchf, dogridarea )
        call define_indx (ncfid)
        status = nf_enddef (ncfid)
        if(dogridarea) then
           call init_geo_coord (ncfid, gsfchf, points,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners, qpgridarea=qpgridarea)
        else
           call init_geo_coord (ncfid, gsfchf, points,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners)
        endif
        status = nf_close (ncfid)

	END SUBROUTINE ginit_sfc_nc




!-----------------------------------------------------------------------

       SUBROUTINE gwrt_atm_nc (nsecond, nsecinit, nyear0, n_points, l_grid_mask,
     *                         geo2, namegeo2, dogeo2, indxgeo2,
     *			       landfrac, seafrac, icefrac, interp_indx,
     *                         geo3, namegeo3, dogeo3, indxgeo3 )

!-----------------------------------------------------------------------
! Purpose:
!
! Bugs:
!    * setting fracs here is somewhat hackish (it only needs to be done 
!      once - this should ideally happen in the init routines)
!----------------------------------------------------------------------

       integer (kind=int_kind), intent(in) :: nsecond
       integer (kind=int_kind), intent(in) :: nsecinit
       integer (kind=int_kind), intent(in) :: nyear0
       integer (kind=int_kind), intent(in) :: n_points
       real (kind=real_kind), dimension(:,:,:), intent(in) ::
     *           landfrac, seafrac, icefrac
       logical (kind=log_kind), dimension(:,:,:), intent(in) ::
     *           l_grid_mask
       integer (kind=int_kind), dimension(:,:,:), intent(in) ::
     *           interp_indx
       real (kind=real_kind), dimension(:,:,:,:), intent(in) :: geo2
       character(len=*), dimension(:), intent(in) :: namegeo2
       logical (kind=log_kind), dimension(:), intent(in) :: dogeo2
       integer (kind=int_kind), dimension(:), intent(in) :: indxgeo2
       real (kind=real_kind), dimension(:,:,:,:,:), intent(in) :: geo3
       character(len=*), dimension(:), intent(in) :: namegeo3
       logical (kind=log_kind), dimension(:), intent(in) :: dogeo3
       integer (kind=int_kind), dimension(:), intent(in) :: indxgeo3

       integer (kind=int_kind) :: fileID

       print *, "gwrt_atm_nc: writing geodesic atm data."

       call next_time_step(gatm2)
       status = nf_open(trim(datapath)//file_name(gatm2), NF_WRITE, fileID)
       if (get_time_step(gatm2) == 1) then   ! first time called
          call set_fracs(fileID, landfrac, seafrac, icefrac,
     *                   l_grid_mask, n_points)
          call set_interp_indx(fileID, interp_indx, l_grid_mask, n_points)
       end if
       call set_time(fileID, nsecond, get_time_step(gatm2), 
     *               nyear0, nsecinit)
       call write_geo_2d(fileID, gatm2, geo2, namegeo2, dogeo2, indxgeo2,
     *                   l_grid_mask, n_points)
       status = nf_close(fileID)

       call next_time_step(gatm3)
       status = nf_open(trim(datapath)//file_name(gatm3), NF_WRITE, fileID)
       if (get_time_step(gatm3) == 1) then   ! first time called
          call set_fracs(fileID, landfrac, seafrac, icefrac,
     *                   l_grid_mask, n_points)
          call set_interp_indx(fileID, interp_indx, l_grid_mask, n_points)
       end if
       call set_time(fileID, nsecond, get_time_step(gatm3),
     *               nyear0, nsecinit)
       call write_geo_3d(fileID, gatm3, geo3, namegeo3, dogeo3, indxgeo3,
     *                   l_grid_mask, n_points )
       status = nf_close(fileID)


       END SUBROUTINE gwrt_atm_nc


!-----------------------------------------------------------------------

       SUBROUTINE gwrt_atm_nc_2 (nsecond, nsecinit, nyear0, n_points, 
     *                         l_grid_mask,
     *                         landfrac, seafrac, icefrac, interp_indx,
     *                         geo3, namegeo3, dogeo3, indxgeo3 )

!-----------------------------------------------------------------------
! Purpose:
!
! Bugs:
!    * setting fracs here is somewhat hackish (it only needs to be done
!      once - this should ideally happen in the init routines)
!----------------------------------------------------------------------

       integer (kind=int_kind), intent(in) :: nsecond
       integer (kind=int_kind), intent(in) :: nsecinit
       integer (kind=int_kind), intent(in) :: nyear0
       integer (kind=int_kind), intent(in) :: n_points
       real (kind=real_kind), dimension(:,:,:), intent(in) ::
     *           landfrac, seafrac, icefrac
       logical (kind=log_kind), dimension(:,:,:), intent(in) ::
     *           l_grid_mask
       integer (kind=int_kind), dimension(:,:,:), intent(in) ::
     *           interp_indx
       real (kind=real_kind), dimension(:,:,:,:,:), intent(in) :: geo3
       character(len=*), dimension(:), intent(in) :: namegeo3
       logical (kind=log_kind), dimension(:), intent(in) :: dogeo3
       integer (kind=int_kind), dimension(:), intent(in) :: indxgeo3

       integer (kind=int_kind) :: fileID

       print *, "gwrt_atm_nc_2: writing geodesic atm data."
!       write(*,*) 'gatm3ml = ',gatm3ml,' file_name(gatm3ml) = ',file_name(gatm3ml)
!       read(*,*)

       call next_time_step(gatm3ml)
       status = nf_open(trim(datapath)//file_name(gatm3ml), NF_WRITE, fileID)
       if (get_time_step(gatm3ml) == 1) then   ! first time called
          call set_fracs(fileID, landfrac, seafrac, icefrac,
     *                   l_grid_mask, n_points)
          call set_interp_indx(fileID, interp_indx, l_grid_mask, n_points)
       end if
       call set_time(fileID, nsecond, get_time_step(gatm3ml),
     *               nyear0, nsecinit)
       call write_geo_3d(fileID, gatm3ml, geo3, namegeo3, dogeo3, indxgeo3,
     *                   l_grid_mask, n_points )
       status = nf_close(fileID)


       END SUBROUTINE gwrt_atm_nc_2



!-----------------------------------------------------------------------

       SUBROUTINE gwrt_sfc_nc (nsecond, nsecinit, nyear0, n_points, l_grid_mask,
     *                         geo2, namegeo2, dogeo2, indxgeo2,
     *			       interp_indx )

!-----------------------------------------------------------------------
! Purpose:
!
! Bugs:
!    * setting fracs here is somewhat hackish (it only needs to be done 
!      once - this should ideally happen in the init routines)
!----------------------------------------------------------------------

       integer (kind=int_kind), intent(in) :: nsecond
       integer (kind=int_kind), intent(in) :: nsecinit
       integer (kind=int_kind), intent(in) :: nyear0
       integer (kind=int_kind), intent(in) :: n_points
       logical (kind=log_kind), dimension(:,:,:), intent(in) ::
     *           l_grid_mask
       integer (kind=int_kind), dimension(:,:,:), intent(in) ::
     *           interp_indx
       real (kind=real_kind), dimension(:,:,:,:), intent(in) :: geo2
       character(len=*), dimension(:), intent(in) :: namegeo2
       logical (kind=log_kind), dimension(:), intent(in) :: dogeo2
       integer (kind=int_kind), dimension(:), intent(in) :: indxgeo2

       integer (kind=int_kind) :: fileID

       print *, "gwrt_sfc_nc: writing geodesic atm data."

       call next_time_step(gsfc2)
       status = nf_open(trim(datapath)//file_name(gsfc2), NF_WRITE, fileID)
       if (get_time_step(gsfc2) == 1) then   ! first time called
          call set_interp_indx(fileID, interp_indx, l_grid_mask, n_points)
       end if
       call set_time(fileID, nsecond, get_time_step(gsfc2), 
     *               nyear0, nsecinit)
       call write_geo_2d(fileID, gsfc2, geo2, namegeo2, dogeo2, indxgeo2,
     *                   l_grid_mask, n_points)
       status = nf_close(fileID)


       END SUBROUTINE gwrt_sfc_nc





!-----------------------------------------------------------------------

       SUBROUTINE gwrt_hf_nc (nsecond, nyear0, n_points, l_grid_mask,
     *                         geohf, namegeohf, dogeohf, indxgeohf,
     *			       landfrac, seafrac, icefrac, interp_indx  )

!-----------------------------------------------------------------------
! Purpose:
!
! Bugs:
!    * setting fracs here is somewhat hackish (it only needs to be done 
!      once - this should ideally happen in the init routines)
!----------------------------------------------------------------------

       integer (kind=int_kind), intent(in) :: nsecond
       integer (kind=int_kind), intent(in) :: nyear0
       integer (kind=int_kind), intent(in) :: n_points
       real (kind=real_kind), dimension(:,:,:), intent(in) ::
     *           landfrac, seafrac, icefrac
       logical (kind=log_kind), dimension(:,:,:), intent(in) ::
     *           l_grid_mask
       integer (kind=int_kind), dimension(:,:,:), intent(in) ::
     *           interp_indx
       real (kind=real_kind), dimension(:,:,:,:), intent(in) :: geohf
       character(len=*), dimension(:), intent(in) :: namegeohf
       logical (kind=log_kind), dimension(:), intent(in) :: dogeohf
       integer (kind=int_kind), dimension(:), intent(in) :: indxgeohf

       integer (kind=int_kind) :: fileID, nsecondtem

       print *, "gwrt_hf_nc: writing geodesic atm data."

       call next_time_step(gatmhf)
       status = nf_open(trim(datapath)//file_name(gatmhf), NF_WRITE, fileID)
       if (get_time_step(gatmhf) == 1) then   ! first time called
          call set_fracs(fileID, landfrac, seafrac, icefrac,
     *                   l_grid_mask, n_points)
          call set_interp_indx(fileID, interp_indx, l_grid_mask, n_points)
       end if
       nsecondtem = nsecond - 21600
       call set_time(fileID, nsecond, get_time_step(gatmhf), 
     *               nyear0, nsecondtem)
       call write_geo_2d(fileID, gatmhf, geohf, namegeohf, dogeohf, indxgeohf,
     *                   l_grid_mask, n_points)
       status = nf_close(fileID)


       END SUBROUTINE gwrt_hf_nc





!-----------------------------------------------------------------------

       SUBROUTINE gwrt_hf_sfc_nc (nsecond, nsecinit, nyear0, n_points,
     *                         l_grid_mask,
     *                         geo2, namegeo2, dogeo2, indxgeo2,
     *			       interp_indx )

!-----------------------------------------------------------------------
! Purpose:
!
! Bugs:
!    * setting fracs here is somewhat hackish (it only needs to be done 
!      once - this should ideally happen in the init routines)
!----------------------------------------------------------------------

       integer (kind=int_kind), intent(in) :: nsecond
       integer (kind=int_kind), intent(in) :: nsecinit
       integer (kind=int_kind), intent(in) :: nyear0
       integer (kind=int_kind), intent(in) :: n_points
       logical (kind=log_kind), dimension(:,:,:), intent(in) ::
     *           l_grid_mask
       integer (kind=int_kind), dimension(:,:,:), intent(in) ::
     *           interp_indx
       real (kind=real_kind), dimension(:,:,:,:), intent(in) :: geo2
       character(len=*), dimension(:), intent(in) :: namegeo2
       logical (kind=log_kind), dimension(:), intent(in) :: dogeo2
       integer (kind=int_kind), dimension(:), intent(in) :: indxgeo2

       integer (kind=int_kind) :: fileID

       print *, "gwrt_hf_sfc_nc: writing geodesic atm data."

       call next_time_step(gsfchf)
       status = nf_open(trim(datapath)//file_name(gsfchf), NF_WRITE, fileID)
       if (get_time_step(gsfchf) == 1) then   ! first time called
          call set_interp_indx(fileID, interp_indx, l_grid_mask, n_points)
       end if
       call set_time(fileID, nsecond, get_time_step(gsfchf), 
     *               nyear0, nsecinit)
       call write_geo_2d(fileID, gsfchf, geo2, namegeo2, dogeo2, indxgeo2,
     *                   l_grid_mask, n_points)
       status = nf_close(fileID)


       END SUBROUTINE gwrt_hf_sfc_nc





!-----------------------------------------------------------------------

	SUBROUTINE nc_init_geo (qpfid, run_name, geo_meta, corners,
     *				        max_ig, listgeo, namegeo, unitsgeo, 
     *                      typegeo, dogeo, dogridarea, 
     *                      multfac, contmin, contint, altunits)	
     
!-----------------------------------------------------------------------
!  Purpose:
!	initialize netcdf files
!       defines all standard geodesic dimensions and variables.
!
!  Scope:
!    Module parameters used:
!   	   STRLEN, MAXNAME, MAXUNITS, MAXLONGN, 
!	   time, char_time
!    Module variables used:
!
!    Module procedures called:
!      define_geo_grid - adds variables that need to be defined for
!                        the geodesic output grid.
!    netcdf_general_tools procedures:
!      define_time - defines the time variables (time, char_time, 
!                    period_start, period_end, period_length)
!      get_units(list, longname, units) - subroutine - removes the units from 
!  	      string list, returns longname(len=MAXLONGN) (string without units)
!  	      and units(len=MAXUNITS)
!
!  Assumptions:
!	- qpfid is connected to an open netcdf file in define mode
!	- namegeo is an array of unique variable names 
!	  only of letters, numbers and underscore (technically,
!	  the variables actually used need unique names.)
!	- listgeo is an array of variable descriptions  
!	   with units optionally included in parentheses.
!-----------------------------------------------------------------------
!   argument declarations
      integer (kind=int_kind), intent(in) :: qpfid
      character(len=*), intent(in) :: run_name
      type(meta_type), intent(in) :: geo_meta
      integer (kind=int_kind), intent(in) :: corners
      integer (kind=int_kind), intent(in) :: max_ig
      character(len=*), dimension(:), intent(in) :: unitsgeo
      character(len=*), dimension(:), intent(in) :: namegeo
      character(len=*), dimension(:), intent(in) :: listgeo
      character(len=*), dimension(:), intent(in) :: typegeo
      logical (kind=log_kind), dimension(:), intent(in) :: dogeo
      logical (kind=log_kind), intent(in) :: dogridarea
      real (kind=dbl_kind), dimension(:), intent(in), optional :: 
     *  multfac, contmin, contint
      character(len=*), dimension(:), intent(in), optional :: altunits
     	
!   variables to hold longname and units 
	integer (kind=int_kind) :: l_len, u_len, t_len=3

!   geo2 dimension id numbers
	integer (kind=int_kind) :: celldim, leveldim, numdim
	integer (kind=int_kind) :: timedim
	   
! array to hold dimension ids
	integer (kind=int_kind), allocatable, dimension(:) :: qpdims

! general variable id - used to define variables
	integer (kind=int_kind) :: qpvid
	
! add standard global attributes
        call add_global_atts (qpfid, run_name, GRIDTYPE, VERSION, max_ig)

! define dimensions 
        call define_time (qpfid, timedim, .true.)
        call define_geo_grid (qpfid, geo_meta%npoints, corners, celldim,
     *                        dogridarea)


!   If data is three dimensional, add level dimension and coordinate variable.
!   Define dimensionality of coordinate variables.
	if (geo_meta%nlevels < 1) then 
	   numdim = 2
	   allocate (qpdims(numdim))
	   qpdims = (/celldim, timedim/)
        else
	   status = nf_def_dim (qpfid,"level",geo_meta%nlevels,leveldim)	
	   status = nf_def_var (qpfid,"level",NF_FLOAT,1,leveldim,qpvid)
	   numdim = 3
	   allocate (qpdims(numdim))
	   qpdims = (/celldim, leveldim, timedim/)
	endif

!   loop to define variables and add attributes
      do n = 1, size(dogeo)
         if (dogeo(n)) then   
!	print *, n, " Netcdf Geodesic Define: ", namegeo(n)
            status = nf_def_var (qpfid, namegeo(n), NF_FLOAT, numdim,
     *				   qpdims, qpvid)
	        call get_units (listgeo(n), unitsgeo(n), l_len, u_len)
      	    status = nf_put_att_text (qpfid, qpvid, 'long_name', 
     *				        l_len, listgeo(n))
      	    status = nf_put_att_text (qpfid, qpvid, 'title', 
     *				        l_len, listgeo(n))
	        status = nf_put_att_text (qpfid, qpvid, 'units', 
     *	       			        u_len, unitsgeo(n))
	        status = nf_put_att_text (qpfid, qpvid, 'type', 
     *	      			        t_len, typegeo(n))
! FIXME - positions may not be needed
	        status = nf_put_att_text (qpfid, qpvid, 'positions',
     *				        6, "center")
	        status = nf_put_att_real (qpfid, qpvid, 'missing_value',
     *				        NF_FLOAT, 1, MISSING)
            if (present(multfac))
     *         status = nf_put_att_double (qpfid, qpvid, 'mult_factor',
     *           NF_DOUBLE, 1, multfac(n))
            if (present(contmin))
     *         status = nf_put_att_double (qpfid, qpvid, 'contour_min',
     *           NF_DOUBLE, 1, contmin(n))
            if (present(contint))
     *         status = nf_put_att_double (qpfid, qpvid, 'contour_int',
     *           NF_DOUBLE, 1, contint(n))
            if (present(altunits))
     *         status = nf_put_att_text (qpfid, qpvid, 'alt_units', 
     *                                    len_trim(altunits(n)), altunits(n)) 
      	 endif
      enddo
      	
	deallocate (qpdims)


	END SUBROUTINE nc_init_geo   



!-----------------------------------------------------------------------

	SUBROUTINE define_geo_grid (fileID, ncells, corners, cell_dim,
     *                              dogridarea )
	
!-----------------------------------------------------------------------
! Purpose:
!     Defines the geodesic grid - copies data from appropriate grid file
!
! Scope: private subroutine
!   Module variables used:
!       status
!   Parameters:
!     In:
!       fileID - netcdf file ID 
!       ncells - number of cells used on grid (length of variable arrays)
!       nhr - grid size
!     Out:
!       cell_dim - netcdf dimension ID (number of grid cells - returned
!                  for easy use by calling routine.)
!   Local variables:
!       gridcellsID - netcdf dimension ID (number of grid cells)
!       gridsizeID - netcdf dim. ID (grid size - this is the size of
!                    the whole grid, of which a certain subset of cells
!                    may be written (this subset is gridcellsID))
!       gridcornID - netcdf dim. ID (number of corners per grid cell -
!                    note that this is 6 even though)
!       gridrankID - netcdf dim. ID (dimensionality of grid)
!
! Bugs:
!   * hardcoded file name for grid definition (C*.global.nc)
!-----------------------------------------------------------------------

        integer (kind=int_kind), intent(in) :: fileID
        integer (kind=int_kind), intent(in) :: ncells
        integer (kind=int_kind), intent(in) :: corners
        integer (kind=int_kind), intent(out) :: cell_dim
        logical (kind=log_kind), intent(in) :: dogridarea

        integer (kind=int_kind) :: gridcellsID, gridsizeID
     *,                            gridcornID, gridrankID

        integer (kind=int_kind) :: varid

        integer (kind=int_kind), dimension(:), allocatable :: tmpdim



! define grid dimension on destination grid
        status = nf_def_dim (fileID, "grid_cells", ncells, gridcellsID)

        status = nf_def_dim (fileID, "grid_size", ncells, gridsizeID)

        status = nf_def_dim(fileID, "grid_corners", corners, gridcornID)

        status = nf_def_dim (fileID, "grid_rank", 1, gridrankID)

! define grid variables on destination grid
        allocate (tmpdim(1))
        tmpdim = (/gridcellsID/)

        status = nf_def_var (fileID, "grid_center_lat", NF_DOUBLE,
     *                       size(tmpdim), tmpdim, varid)
        status = nf_put_att_text (fileID, varid, "units", 7, "radians")
        status = nf_def_var (fileID, "grid_center_lon", NF_DOUBLE,
     *                       size(tmpdim), tmpdim, varid)
        status = nf_put_att_text (fileID, varid, "units", 7, "radians")
        if(dogridarea) then
         status = nf_def_var (fileID, "grid_area", NF_DOUBLE,
     *                        size(tmpdim), tmpdim, varid)
         status = nf_put_att_text (fileID, varid, "units", 14, "square radians")
        endif

        tmpdim = (/gridrankID/)

        status = nf_def_var (fileID, "grid_dims", NF_INT,
     *                       size(tmpdim), tmpdim, varid)

        deallocate (tmpdim)
        allocate (tmpdim(2))
        tmpdim = (/gridcornID, gridcellsID/)

        status = nf_def_var (fileID, "grid_corner_lat", NF_DOUBLE,
     *                       size(tmpdim), tmpdim, varid)
        status = nf_put_att_text (fileID, varid, "units", 7, "radians")
        status = nf_def_var (fileID, "grid_corner_lon", NF_DOUBLE,
     *                       size(tmpdim), tmpdim, varid)
        status = nf_put_att_text (fileID, varid, "units", 7, "radians")

        deallocate (tmpdim)

        cell_dim = gridcellsID

        END SUBROUTINE define_geo_grid



!-----------------------------------------------------------------------

	SUBROUTINE init_geo_coord (qpfid, geo_meta, points,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners, levelvals, qpgridarea)
	
!-----------------------------------------------------------------------
!  Purpose:
!   sets values of coordinate variables (center_lat, center_lon, etc.)
! 
!  Scope:
!   Module variables used:
!       status
!   Module routines called:
!       copy_geo_grid
!
!  Assumptions: 
!	1) qpfid is the file id of a netcdf file that is open and in data mode
!	2) levelvals (if present) is a real array with geo_meta%nlevels elements
!	3) dimensions have already been defined
!
!  Bugs:
!       1) A good method for finding position data files is needed.
!-----------------------------------------------------------------------

	integer (kind=int_kind), intent(in) :: qpfid
	integer (kind=int_kind), intent(in) :: corners
	integer (kind=int_kind), intent(in) :: points
	type(meta_type), intent(in) :: geo_meta
	real (kind=real_kind), dimension(:), intent(in), optional ::
     *        levelvals
        real (kind=selected_real_kind(13)), dimension(corners,points),
     * intent(in) :: qpcornerlat, qpcornerlon
        real (kind=selected_real_kind(13)), dimension(points),
     * intent(in) :: qpcenterlat, qpcenterlon
        real (kind=selected_real_kind(13)), dimension(points),
     * intent(in), optional :: qpgridarea

        integer (kind=int_kind) :: temp_id


        if(present(qpgridarea)) then
           call copy_geo_grid (qpfid, geo_meta%npoints,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners, qpgridarea)
        else
           call copy_geo_grid (qpfid, geo_meta%npoints,
     *                   qpcornerlat, qpcornerlon, qpcenterlat, qpcenterlon,
     *                   corners)
	endif

	if (present(levelvals)) then
	   status = nf_inq_varid (qpfid, "level", temp_id)
	   status = nf_put_var_real (qpfid, temp_id, levelvals) 
	endif


	END SUBROUTINE init_geo_coord 



!-----------------------------------------------------------------------

	SUBROUTINE copy_geo_grid (fileID, points, 
     *             cornerlatvals, cornerlonvals, centerlatvals, centerlonvals,
     *             corners, areavals)

!-----------------------------------------------------------------------
!  Purpose:
!     Copies coordinate variable information to the netcdf output file
!
!  Scope:
!    Module variables used:
!       status
! 
!-----------------------------------------------------------------------

        integer (kind=int_kind), intent(in) :: fileID
        integer (kind=int_kind), intent(in) :: points
        integer (kind=int_kind), intent(in) :: corners
        real (kind=selected_real_kind(13)), intent(in),
     *           dimension(points) :: centerlatvals, centerlonvals
        real (kind=selected_real_kind(13)), intent(in),
     *           dimension(points), optional :: areavals
        real (kind=selected_real_kind(13)), intent(in),
     *           dimension(corners,points) :: cornerlatvals, cornerlonvals

        integer (kind=int_kind) :: latID, lonID, cornlatID, cornlonID, areaID



        status = nf_inq_varid (fileID, "grid_center_lat", latID)
        status = nf_inq_varid (fileID, "grid_center_lon", lonID)
        status = nf_inq_varid (fileID, "grid_corner_lat", cornlatID)
        status = nf_inq_varid (fileID, "grid_corner_lon", cornlonID)
        if(present(areavals)) 
     *     status = nf_inq_varid (fileID, "grid_area", areaID)

        status = nf_put_var_double (fileID, latID, centerlatvals)
        status = nf_put_var_double (fileID, lonID, centerlonvals)
        status = nf_put_var_double (fileID, cornlatID, cornerlatvals)
        status = nf_put_var_double (fileID, cornlonID, cornerlonvals)
        if(present(areavals)) 
     *     status = nf_put_var_double (fileID, areaID, areavals)

        END SUBROUTINE copy_geo_grid 
        


!-----------------------------------------------------------------------

	SUBROUTINE define_fracs (fileID)

!-----------------------------------------------------------------------
!  Purpose:
!    Define the fractional variables (landfrac, seafrac, icefrac)
!  for the given file.
!
!  Additionally, define interpolation index variable
!
!  Scope: private subroutine
!    Module variables used:
!      status
!    Parameters:
!     In:
!      fileID: netcdf file ID, open and in define mode.
!    Local variables:
!      sizedim - netcdf dimension ID (grid size - stored in a 
!                single valued array for type consistency)
!      dumid - netcdf variable ID (dummy parameter for function call)
!
!  Bugs:
!-----------------------------------------------------------------------

        integer (kind=int_kind), intent(in) :: fileID

        integer (kind=int_kind), dimension(1) :: sizedim
        integer (kind=int_kind) :: dumid

!  get the dimension id needed (the grid size dimension)
        status = nf_inq_dimid (fileID, "grid_size", sizedim(1))
        call handle_error(status, "couldn't get grid_size")

!  define frac variables
     	status = nf_def_var (fileID, "landfrac", NF_FLOAT, 1, sizedim,
     *			     dumid)
        call handle_error(status, "couldn't define landfrac")

     	status = nf_def_var (fileID, "seafrac", NF_FLOAT, 1, sizedim,
     *			     dumid)
        call handle_error(status, "couldn't define seafrac")

     	status = nf_def_var (fileID, "icefrac", NF_FLOAT, 1, sizedim,
     *			     dumid)
        call handle_error(status, "couldn't define icefrac")

     	status = nf_def_var (fileID, "interp_indx", NF_INT, 1, sizedim,
     *			     dumid)
        call handle_error(status, "couldn't define interp_indx")


	END SUBROUTINE define_fracs



	SUBROUTINE define_indx (fileID)

!-----------------------------------------------------------------------
!  Purpose:
!    Define the interpolation index variable
!
!  Scope: private subroutine
!    Module variables used:
!      status
!    Parameters:
!     In:
!      fileID: netcdf file ID, open and in define mode.
!    Local variables:
!      sizedim - netcdf dimension ID (grid size - stored in a 
!                single valued array for type consistency)
!      dumid - netcdf variable ID (dummy parameter for function call)
!
!  Bugs:
!-----------------------------------------------------------------------

        integer (kind=int_kind), intent(in) :: fileID

        integer (kind=int_kind), dimension(1) :: sizedim
        integer (kind=int_kind) :: dumid

!  get the dimension id needed (the grid size dimension)
        status = nf_inq_dimid (fileID, "grid_size", sizedim(1))
        call handle_error(status, "couldn't get grid_size")

!  define frac variables
     	status = nf_def_var (fileID, "interp_indx", NF_INT, 1, sizedim,
     *			     dumid)
        call handle_error(status, "couldn't define interp_indx")


	END SUBROUTINE define_indx



!-----------------------------------------------------------------------

	SUBROUTINE set_fracs (fid, landfrac, seafrac, icefrac,
     *                        l_grid_mask, n_points)  
	
!-----------------------------------------------------------------------
!  Purpose: writes arrays of values representing the fraction of land, 
!           sea and ice at each point of the grid
!
!  Scope: private subroutine
!   Module variables used:
!	status
!   Parameters:
!     In:
!       fid - netcdf file ID, open and in data mode.
!       landfrac, seafrac, icefrac - fractional values to be written
!       l_grid_mask - logical mask to pack the input fractional values
!       n_points - number of unique data points in the arrays
!                  (number of true points in l_grid_mask)
!   Local variables:
!       fracvid - netcdf variable id (current frac variable)
!       temfrac - packed vector of unique data points
!   Intrinsics - fortran 90 function PACK
!
!  Bugs:
!     * no error checking
!     * relies on netcdf's type conversion if REALX4 /= real*4
!-----------------------------------------------------------------------
		
	integer (kind=int_kind), intent(in) :: fid
	integer (kind=int_kind), intent(in) :: n_points
	real (kind=real_kind), dimension(:,:,:), intent(in) :: landfrac
	real (kind=real_kind), dimension(:,:,:), intent(in) :: seafrac
	real (kind=real_kind), dimension(:,:,:), intent(in) :: icefrac
	logical (kind=log_kind), dimension(:,:,:), intent(in) :: l_grid_mask
	
	integer (kind=int_kind) :: fracvid
        real (kind=real_kind), dimension(n_points) :: temfrac

        temfrac = pack(landfrac, l_grid_mask)
	status = nf_inq_varid (fid, "landfrac", fracvid)
	status = nf_put_var_real (fid, fracvid, temfrac)

        temfrac = pack(seafrac, l_grid_mask)
	status = nf_inq_varid (fid, "seafrac", fracvid)
	status = nf_put_var_real (fid, fracvid, temfrac)

        temfrac = pack(icefrac, l_grid_mask)
	status = nf_inq_varid (fid, "icefrac", fracvid)
	status = nf_put_var_real (fid, fracvid, temfrac)


	END SUBROUTINE set_fracs


!-----------------------------------------------------------------------

	SUBROUTINE set_interp_indx (fid, interp_indx, l_grid_mask, n_points)  
	
!-----------------------------------------------------------------------
!  Purpose: writes array of values representing the sequential index 
!           of each point of the grid for the purpose of interpolation
!           to a latlon grid consistent with indices in interp_sgg_*
!           Modeled on set_fracs
!
!  Scope: private subroutine
!   Module variables used:
!	status
!   Parameters:
!     In:
!       fid - netcdf file ID, open and in data mode.
!       interp_indx - interpolation indices to be written
!       l_grid_mask - logical mask to pack the input indices
!       n_points - number of unique data points in the indx array
!                  (number of true points in l_grid_mask)
!   Local variables:
!       terpvid - netcdf variable id (current terp variable)
!       tem_indx - packed vector of unique data points
!   Intrinsics - fortran 90 function PACK
!
!  Bugs:
!     * no error checking
!-----------------------------------------------------------------------
		
	integer (kind=int_kind), intent(in) :: fid
	integer (kind=int_kind), dimension(:,:,:), intent(in) :: interp_indx
	integer (kind=int_kind), intent(in) :: n_points
	logical (kind=log_kind), dimension(:,:,:), intent(in) :: l_grid_mask
	
	integer (kind=int_kind) :: terpvid
        integer (kind=int_kind), dimension(n_points) :: tem_indx

        tem_indx = pack(interp_indx, l_grid_mask)
	status = nf_inq_varid (fid, "interp_indx", terpvid)
	status = nf_put_var_int (fid, terpvid, tem_indx)


	END SUBROUTINE set_interp_indx



!-----------------------------------------------------------------------

	SUBROUTINE write_geo_2d(fid, geo_meta, geo2, namegeo2, dogeo2,
     *                          indxgeo2, l_grid_mask, n_points )
	
!-----------------------------------------------------------------------
!  Purpose:
!	writes an entire time record worth of two dimensional data 
!	to the netcdf file connected to fid
!
!  Scope:
!   Module parameters used:
!   Module variables used:
!   Intrinsics used: PACK
!
!  Assumptions:
!	-  netcdf file has the 2 dimensional variables with names 
!          in the array namegeo2 already defined
!-----------------------------------------------------------------------

        integer (kind=int_kind), intent(in) :: n_points
        logical (kind=log_kind), dimension(:,:,:), intent(in) :: l_grid_mask
        integer (kind=int_kind), intent(in) :: fid
	type(meta_type), intent(in) :: geo_meta
	real (kind=real_kind), dimension(:,:,:,:), intent(in) :: geo2
	character(len=*), dimension(:), intent(in) :: namegeo2
	logical (kind=log_kind), dimension(:), intent(in) :: dogeo2
	integer (kind=int_kind), dimension(:), intent(in) :: indxgeo2

	integer (kind=int_kind) :: ivar
	integer (kind=int_kind) :: qpvid
	integer (kind=int_kind), dimension(2) :: start, count
        real (kind=real_kind), dimension(n_points) :: tem_geo2

	start = (/1, geo_meta%timerec/)
	count = (/geo_meta%npoints, 1/)

!  Expand variables and write them.
!    - ivar keeps track of which variable (name) is being used
      	ivar = 0
      	do n = 1, size(dogeo2)
      	   if (dogeo2(n)) then
!      	  print *, "Writing geodesic data to variable ", namegeo2(n) 
	      ivar = ivar + 1
      	      status = nf_inq_varid (fid, namegeo2(n), qpvid)
      	      call handle_error(status, namegeo2(n))
              tem_geo2 = pack(geo2(:,:,indxgeo2(n),:),l_grid_mask )
      	      status = nf_put_vara_real (fid, qpvid, start, 
     *		  			 count, tem_geo2)
              call handle_error(status, namegeo2(n))
      	   end if
      	end do

	
	END SUBROUTINE write_geo_2d	



!-----------------------------------------------------------------------

	SUBROUTINE write_geo_3d(fid3, geo_meta, geo3, namegeo3, 
     *                          dogeo3, indxgeo3, l_grid_mask, n_points)
	
!-----------------------------------------------------------------------
!  Purpose:
!	writes an entire time record worth of three dimensional data 
!	to the netcdf file connected to fid3
!
!  Scope:
!   Module variables used:
!   Intrinsics used: PACK
!
!  Assumptions:
!	- fid3 is connected to a netcdf file that is open and in data mode
!	  and has the 3 dimensional variables with names in the array namegeo2
!	  already defined
!-----------------------------------------------------------------------

        integer (kind=int_kind), intent(in) :: n_points
        logical (kind=log_kind), dimension(:,:,:), intent(in) :: l_grid_mask
	integer (kind=int_kind), intent(in) :: fid3
	type(meta_type), intent(in) :: geo_meta
	logical (kind=log_kind), dimension(:), intent(in) :: dogeo3
	real (kind=real_kind), dimension(:,:,:,:,:), intent(in) :: geo3
	character(len=*), dimension(:), intent(in) :: namegeo3
	integer (kind=int_kind), dimension(:), intent(in) :: indxgeo3

	integer (kind=int_kind) :: ivar, k, i, m
	integer (kind=int_kind) :: qpvid
	integer (kind=int_kind), dimension(3) :: start, count
     	real (kind=real_kind) :: qp3vals(n_points*size(geo3,3))
        real (kind=real_kind), dimension(n_points) :: tem_geo3

	start = (/1, 1, geo_meta%timerec/)
	count = (/geo_meta%npoints, geo_meta%nlevels, 1/)

!  Expand variables and write
!    - ivar keeps track of which variable (name) is being used
      	ivar = 0

      	do n = 1, size(dogeo3)
      	   if (dogeo3(n)) then
!      	  print *, "Writing geodesic data to variable ", namegeo3(n) 
	      ivar = ivar + 1
              k = 0
              do m = size(geo3,3),1,-1
                 tem_geo3(:) = pack(geo3(:,:,m,indxgeo3(n),:), l_grid_mask)
                 do i = 1,size(tem_geo3,1)
                    k = k + 1
                    qp3vals(k) = tem_geo3(i)
                 enddo
              enddo
      	      status = nf_inq_varid (fid3, namegeo3(n), qpvid)
      	      status = nf_put_vara_real (fid3, qpvid, start, 
     *		  			 count, qp3vals)
      	   end if
      	end do

	END SUBROUTINE write_geo_3d



        END MODULE netcdf_geodesic

