!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      module qp_pbp_diagnostics_module

!-----------------------------------------------------------------------
! PURPOSE: accumlate qp data in time, store pbp data
!          compute derived quantities,
!-----------------------------------------------------------------------

      use prognostics
      use physical_parameters
      use pbp_module
#ifndef no_pbl
      use pbp_module_sfc
#endif
      use tracer_names
      use utility
      use time_manager_module
      use adjustments
#ifndef scm
      use grid_metrics
      use qp_module
      use qp_ml
      use gmeans_module
      use operators
      use multigrid_solver
      use fluxcoupler_fields
#else
      use scm_module
#endif

#ifndef no_pbl
#ifdef gravity_wave_drag
	use gravity_wave_drag_module
#endif

#ifdef eau_ng
      use eau_names
#endif
#endif

      implicit none
      private

      public ::
     *     qp_pbp_diagnostics 

!**********************************************************************

        contains

!**********************************************************************


!======================================================================
! BEGINNING OF QP_DIAGNOSTICS
!======================================================================
      subroutine qp_pbp_diagnostics(nsecond_dyn)
      implicit none

      integer (kind=int_kind), intent(in) :: nsecond_dyn

      integer (kind=int_kind) :: ksdm, k, n
      real (kind=dbl_kind), dimension(iim,jjm) :: 
     *		tas = 0.0, 
     *		psl = 0.0,
     *		pcloc = 0.0,
     *          work_2d = 0.0,
     *          work_north = 0.0,
     *          work_east = 0.0,
     *          work_inc,
     *          work_fill
      real (kind=dbl_kind), dimension(iim,jjm,nsdm) :: 
     *          div_200 = 0.0,
     *          chi_200 = 0.0

      real (kind=dbl_kind), dimension(3,iim,jjm) :: 
     *          work_n = 0.0,
     *          work_t = 0.0

      real (kind=dbl_kind), dimension(iim,jjm,nlm) ::
     *		cwtem = 0.0,
     *		work_3d = 0.0,
     *		work_3db = 0.0

      real (kind=dbl_kind), dimension(iim,jjm,nlm+1) ::
     *            work_3dl2 = 0.0

      real (kind=dbl_kind), dimension(iim,jjm,nsdm), save ::
     *		tasmin = 999.0,
     *		tasmax = 0.0,
     *		tasmin6 = 999.0,
     *		tasmax6 = 0.0
#ifndef no_pbl
     *,		psbmin = 99999.0,
     *		psbmax = 0.0
#endif

      real (kind=dbl_kind), dimension(iim,jjm,nlm,nsdm) ::
     *		workq1 = 0.0, workq2 = 0.0

#ifndef scm
! work arrays for computing products
      real (kind=dbl_kind), dimension(iim,jjm,mm) ::
     *    ua = 0.0,
     *    va = 0.0,
     *    wap = 0.0,
     *    ta = 0.0,
     *    zeta_a = 0.0,
     *    diva = 0.0,
     *    qa = 0.0,
     *    zg = 0.0
#endif
      logical (kind=log_kind), save :: first

      real (kind=dbl_kind) :: pmkappa

!----------------------------------------------------------------------------
! update the pbp sample counter
!----------------------------------------------------------------------------
      ipbp_sample = ipbp_sample + 1
#ifndef no_pbl
      ipbp_sample_sfc = ipbp_sample_sfc + 1
#endif

!----------------------------------------------------------------------------
! loop over the sub-domains
!----------------------------------------------------------------------------
      do ksdm = 1,nsdm
!----------------------------------------------------------------------------
!  vorticity and divergence and winds, and vertical motion
!   here, work_3d is zonal wind, work_3db is meriodional wind
!----------------------------------------------------------------------------
#ifndef scm
! the qp3 call does the winds as well
      call qp3sum_divvort(u(1,1,1,1,1,n3,ksdm)
     *,           pthick_c(1,1,1,1,ksdm)
     *,           plgl2(1,1,1,ksdm)
     *,           1.0_dbl_kind,ksdm,div_200(1,1,ksdm)
     *,           diva, zeta_a, ua, va, wap )
      workq1(:,:,:,ksdm) = dsigdt(:,:,2:nlmp1,ksdm)
      call qp3mlsum(workq1, iim*jjm, c1, iqp3ml_dsigdt,
     *            ksdm, iim*jjm)
#endif
      call pbp2sum(div(1,1,1,n4,ksdm), iim*jjm, 1.0_dbl_kind
     *,           ipbp2_divergence, ksdm, iim*jjm)
      call pbp2sum(relative(1,1,1,ksdm), iim*jjm, 1.0_dbl_kind
     *,           ipbp2_vorticity, ksdm, iim*jjm)

!----------------------------------------------------------------------------
! wind related diagnostic : pbp winds
!----------------------------------------------------------------------------
#ifdef scm
!-- write out the scm winds from the prescribed data as interpolated
!-- to the model sigma levels
      call pbp2sum(u_terp(1,1,1,ksdm), iim*jjm, 1.0_dbl_kind
     *,           ipbp2_ua, ksdm, iim*jjm)
      call pbp2sum(v_terp(1,1,1,ksdm), iim*jjm, 1.0_dbl_kind
     *,           ipbp2_va, ksdm, iim*jjm) 
#else
!-- zonal and meridional winds on sigma surfaces
      do k = 1,nlm
        work_3d(:,:,k) = u(1,1,:,:,k,n4,ksdm)
      enddo
      call pbp2sum(work_3d, iim*jjm, 1.0_dbl_kind
     *,           ipbp2_ua, ksdm, iim*jjm)
      do k = 1,nlm
        work_3d(:,:,k) = u(2,1,:,:,k,n4,ksdm)
      enddo
      call pbp2sum(work_3d, iim*jjm, 1.0_dbl_kind
     *,           ipbp2_va, ksdm, iim*jjm)
#endif  

#ifndef scm
!----------------------------------------------------------------------------
! wind related diagnostic : global mean total relative angular momentum
!----------------------------------------------------------------------------
!get zonal wind from corner 1 - later we will properly avg
      work_2d = 0.0
      do k = 1,nlm
         work_2d(:,:) = work_2d(:,:) + cos(grid_center_lat(:,:,ksdm)) 
     *		* u(1,1,:,:,k,n4,ksdm) * pthick(:,:,k,n4,ksdm) * dsig(k)
      enddo
      work_2d = work_2d * gravi * a
      call gmean_sum_atm(work_2d, iim*jjm, c1, iglob_moa, ksdm )

!----------------------------------------------------------------------------
! wind related diagnostic : vertically integrated ke
!----------------------------------------------------------------------------
      work_2d(:,:) = c0
      work_3d(:,:,:) = ke(:,:,:,ksdm)
      do k = 1,nlm
        work_2d(:,:) = work_2d(:,:) + work_3d(:,:,k)
     *		 * pthick(:,:,k,n4,ksdm) * dsig(k)
      enddo
      work_2d(:,:) = work_2d(:,:) * gravi
      call qp2sum(work_2d, c1,iqp2_ke, ksdm, iim*jjm)
      if(mod(nsecond_dyn,21600*nhf_freq).eq.0)
     *          call hf2sum(work_2d, c1,ihf2_ke, ksdm, iim*jjm)
      call qp3sum(ke(1,1,1,ksdm), iim*jjm, 1.0_dbl_kind
     *,           iqp3_ke, ksdm, iim*jjm)

!----------------------------------------------------------------------------
! wind related diagnostic : vertically integrated enstrophy
!----------------------------------------------------------------------------
      work_3d(:,:,:) = relative(:,:,:,ksdm)**2
      work_2d(:,:) = c0
      do k = 1,nlm
        work_2d(:,:) = work_2d(:,:) + work_3d(:,:,k)
     *		 * pthick(:,:,k,n4,ksdm) * dsig(k)
      enddo
      work_2d(:,:) = work_2d(:,:) * gravi
      call qp2sum(work_2d, c1,iqp2_en, ksdm, iim*jjm)
      if(mod(nsecond_dyn,21600*nhf_freq).eq.0)
     *          call hf2sum(work_2d, c1,ihf2_en, ksdm, iim*jjm)
      call qp3sum(work_3d(1,1,1), iim*jjm, 1.0_dbl_kind
     *,           iqp3_en, ksdm, iim*jjm)
#endif

!----------------------------------------------------------------------------
!  geopotential
!----------------------------------------------------------------------------
#ifndef scm
      call qp3sum_zg(phi(1,1,1,ksdm), th(1,1,1,n4,ksdm), iim*jjm
     *,           bpgl2(1,1,1,ksdm), bpg(1,1,1,ksdm), 1.0_dbl_kind
     *,           iqp3_zg, ksdm, iim*jjm, parray=zg)
      if(mod(nsecond_dyn,21600*nhf_freq).eq.0) then
         do k = 1,mm
            if(abs(pm(k)-50000._dbl_kind).lt.1._dbl_kind) 
     *           call hf2sum(zg(1,1,k), c1,ihf2_zg500, ksdm, iim*jjm)
         enddo
      endif
#endif
      call pbp2sum(phi(1,1,1,ksdm), iim*jjm, 1.0_dbl_kind
     *,           ipbp2_zg, ksdm, iim*jjm)

!----------------------------------------------------------------------------
!  this will be temperature, the conversion from potential temperature
!    will be done after normalization
!----------------------------------------------------------------------------
! get interpolated temperature to do a product further down 
#ifndef scm
      call qp3sum(th(1,1,1,n4,ksdm), iim*jjm, 1.0_dbl_kind
     *,           iqp3_ta, ksdm, iim*jjm, parray=ta)
      call qp3mlsum(th(1,1,1,n4,ksdm), iim*jjm, 1.0_dbl_kind
     *,           iqp3ml_th, ksdm, iim*jjm)
      do k = 1,mm
         pmkappa = (pm(k)*inv_p0_sfc)**kappa
         ta(:,:,k) = ta(:,:,k) * pmkappa
      enddo
#endif
      call pbp1sum(th(1,1,1,n4,ksdm), c1, ipbp1_th1, ksdm, iim*jjm)
      work_3d(:,:,:) = th(:,:,:,n4,ksdm) * bpg(:,:,:,ksdm)
      call pbp2sum(work_3d, iim*jjm, 1.0_dbl_kind
     *,            ipbp2_ta, ksdm, iim*jjm)
#ifndef scm
      call qp3mlsum(work_3d, iim*jjm, 1.0_dbl_kind
     *,            iqp3ml_ta, ksdm, iim*jjm)
#endif

#ifdef scm
      work_3d(:,:,:) = (th(:,:,:,n4,ksdm) - t_terp(:,:,:,ksdm)) * 
     *  bpg(:,:,:,ksdm)
      where (t_terp(:,:,:,ksdm) == c0) work_3d(:,:,:) = c0
      call pbp2sum(work_3d, iim*jjm, 1.0_dbl_kind
     *,            ipbp2_tadiff, ksdm, iim*jjm)

      work_3d(:,:,:) = (tracer(:,:,:,iq,n4,ksdm) - q_terp(:,:,:,ksdm))
      where (q_terp(:,:,:,ksdm) == c0) work_3d(:,:,:) = c0
      call pbp2sum(work_3d, iim*jjm, 1.0_dbl_kind
     *,            ipbp2_qdiff, ksdm, iim*jjm)
#endif

#ifndef scm
      if(mod(nsecond_dyn,21600*nhf_freq).eq.0) then
         do k = 1,mm
            if(abs(pm(k)-85000._dbl_kind).lt.1._dbl_kind) 
     *           call hf2sum(ta(1,1,k), c1,ihf2_ta850, ksdm, iim*jjm)
            if(abs(pm(k)-50000._dbl_kind).lt.1._dbl_kind) 
     *           call hf2sum(ta(1,1,k), c1,ihf2_ta500, ksdm, iim*jjm)
            if(abs(pm(k)-5000._dbl_kind).lt.1._dbl_kind) 
     *           call hf2sum(ta(1,1,k), c1,ihf2_ta50, ksdm, iim*jjm)
         enddo
      endif

! vertical integral of temperature
      work_2d = 0.0
      do k = 1,nlm
         work_2d(:,:) = work_2d(:,:) + work_3d(:,:,k)
     *		 * pthick(:,:,k,n4,ksdm) * dsig(k)
      enddo
      work_2d(:,:) = work_2d(:,:) / (ps(:,:,n4,ksdm)-p_top)
      call gmean_sum_atm(work_2d, iim*jjm, c1, iglob_ta, ksdm )

      if(mod(nsecond_dyn,21600).eq.0) then
        do k = 1,mm
          ta(:,:,k) = ta(:,:,k) * (pm(k)/p0_sfc)**kappa
        enddo
      endif

!----------------------------------------------------------------------------
!  this will be pressure, the conversion from p**kappa
!    will be done after normalization
!----------------------------------------------------------------------------
      call qp3sum(bpg(1,1,1,ksdm), iim*jjm, 1.0_dbl_kind
     *,           iqp3_pressure, ksdm, iim*jjm)
#endif
      call pbp2sum(plg(1,1,1,ksdm), iim*jjm, 0.010_dbl_kind
     *,           ipbp2_pl, ksdm, iim*jjm)
      work_3d(:,:,1:nlm) = plgl2(:,:,2:nlm+1,ksdm)
      call pbp2sum(work_3d(1,1,1), iim*jjm, 0.01_dbl_kind
     *,           ipbp2_pl2, ksdm, iim*jjm)

!----------------------------------------------------------------------------
!  total diabatic tendencies
!----------------------------------------------------------------------------
#ifndef scm
      work_3d(:,:,1:nlm) = th_src(:,:,:,ksdm) * invpthick(:,:,:,n3,ksdm)
      call qp3sum(work_3d(1,1,1), iim*jjm, 1.0_dbl_kind
     *,           iqp3_tnt, ksdm, iim*jjm)
      work_3d(:,:,1:nlm) = tracer_src(:,:,:,iq,ksdm) * invpthick(:,:,:,n3,ksdm)
      call qp3sum(work_3d(1,1,1), iim*jjm, 1.0_dbl_kind
     *,           iqp3_tnmrd, ksdm, iim*jjm)
      call qp3mlsum(work_3d(1,1,1), iim*jjm, c1, iqp3ml_tnmrd,
     *            ksdm, iim*jjm)

#ifndef no_pbl
!----------------------------------------------------------------------------
! pbl wind speed
!----------------------------------------------------------------------------
      call qp2sum(spdm(1,1,ksdm), c1,iqp2_spdm, ksdm, iim*jjm)
#endif
#endif
#ifndef no_pbl
      call pbp1sum(spdm(1,1,ksdm), c1,ipbp1_spdm, ksdm, iim*jjm)
#endif

#ifdef scm
!-- Q1 and Q2 for scm only at this point
      if ((scm_mode /= 'rad') .and. (scm_mode /= 'RAD')) then
        workq1(:,:,:,ksdm) =  
     *   (th_src(:,:,:,ksdm) 
     *    - tadvt_terp(:,:,:,ksdm)*pthick(:,:,:,n3,ksdm)  
     *    - delt_rad(:,:,:,ksdm)/bpg(:,:,:,ksdm))  
     *    * bpg(:,:,:,ksdm)*invpthick(:,:,:,n3,ksdm)
      else
        workq1(:,:,:,ksdm) =  
     *   (th_src(:,:,:,ksdm) 
     *    - delt_rad(:,:,:,ksdm)/bpg(:,:,:,ksdm))  
     *    * bpg(:,:,:,ksdm)*invpthick(:,:,:,n3,ksdm)
      endif

      if ((scm_mode /= 'rad') .and. (scm_mode /= 'RAD')) then
        workq2(:,:,:,ksdm) =  
     *   (tracer_src(:,:,:,iq,ksdm) 
     *    - qadvt_terp(:,:,:,ksdm)*pthick(:,:,:,n3,ksdm))  
     *    * (-hltm/spec_heat_cp)*invpthick(:,:,:,n3,ksdm)
      else
        workq2(:,:,:,ksdm) =  
     *   tracer_src(:,:,:,iq,ksdm)
     *     * (-hltm/spec_heat_cp)*invpthick(:,:,:,n3,ksdm)
      endif

      call pbp2sum(workq1(1,1,1,ksdm), iim*jjm, 86400.0_dbl_kind,  
     *            ipbp2_q1, ksdm, iim*jjm)
      call pbp2sum(workq2(1,1,1,ksdm), iim*jjm, 86400.0_dbl_kind,  
     *            ipbp2_q2, ksdm, iim*jjm)

      workq1(:,:,:,ksdm) = tadvt_terp(:,:,:,ksdm)*bpg(:,:,:,ksdm)
      call pbp2sum(workq1(1,1,1,ksdm), iim*jjm, 86400.0_dbl_kind,  
     *            ipbp2_scmvdelt, ksdm, iim*jjm)
      workq1(:,:,:,ksdm) = qadvt_terp(:,:,:,ksdm)
      call pbp2sum(workq1(1,1,1,ksdm), iim*jjm, 86400000.0_dbl_kind,  
     *            ipbp2_scmvdelq, ksdm, iim*jjm)

      call pbp2sum(trlxterm(1,1,1,ksdm), iim*jjm, 86400.0_dbl_kind,  
     *            ipbp2_trlxterm, ksdm, iim*jjm)
      call pbp2sum(qrlxterm(1,1,1,ksdm), iim*jjm, 86400000.0_dbl_kind,  
     *            ipbp2_qrlxterm, ksdm, iim*jjm)
      call pbp2sum(trevterm(1,1,1,ksdm), iim*jjm, 86400.0_dbl_kind,  
     *            ipbp2_trevterm, ksdm, iim*jjm)
      call pbp2sum(qrevterm(1,1,1,ksdm), iim*jjm, 86400000.0_dbl_kind,  
     *            ipbp2_qrevterm, ksdm, iim*jjm)
#endif

!----------------------------------------------------------------------------
! surface pressure
!----------------------------------------------------------------------------
#ifndef scm
      call qp2sum(ps(1,1,n4,ksdm), c1,iqp2_ps, ksdm, iim*jjm)
      if(mod(nsecond_dyn,21600*nhf_freq).eq.0)
     *   call hf2sum(ps(1,1,n4,ksdm),c1,ihf2_ps,ksdm,iim*jjm)

      call gmean_sum_atm(ps(1,1,n4,ksdm), iim*jjm, c1, iglob_ps, ksdm )
#endif
      call pbp1sum(ps(1,1,n4,ksdm), c1,ipbp1_ps, ksdm, iim*jjm)

#ifndef no_pbl
!----------------------------------------------------------------------------
! pbl specific humidity
!----------------------------------------------------------------------------
#ifndef scm
      call qp2sum(tracer(1,1,nlm,iq,n4,ksdm),c1,iqp2_huss,ksdm,iim*jjm)
      if(mod(nsecond_dyn,21600*nhf_freq).eq.0)
     *   call hf2sum(tracer(1,1,nlm,iq,n4,ksdm),c1,ihf2_huss,ksdm,iim*jjm)
#endif
      call pbp1sum(tracer(1,1,nlm,iq,n4,ksdm), c1, ipbp1_huss, ksdm, iim*jjm)

!----------------------------------------------------------------------------
! pbl depth
!----------------------------------------------------------------------------
#ifdef generalized_sigma
#ifndef scm
      call qp2sum(ps(1,1,n4,ksdm),  c1,iqp2_blp, ksdm, iim*jjm)
      call qp2sum(pb(1,1,n4,ksdm), -c1,iqp2_blp, ksdm, iim*jjm, step_sample=0)
      if(mod(nsecond_dyn,21600*nhf_freq).eq.0) then
         call hf2sum(phil2(1,1,lpbl+1,ksdm), c1,ihf2_zblh, ksdm, iim*jjm)
         call hf2sum(phis(1,1,ksdm), -c1,ihf2_zblh, ksdm, iim*jjm,
     *               step_sample=0)
      endif
#endif
      call pbp1sum(ps(1,1,n4,ksdm), c1, ipbp1_blp, ksdm, iim*jjm)
      call pbp1sum(pb(1,1,n4,ksdm), -c1, ipbp1_blp, ksdm, iim*jjm)
#endif
#endif

!----------------------------------------------------------------------------
! surface air temperature
!----------------------------------------------------------------------------
      tas(:,:) = th(:,:,nlm,n4,ksdm) * bpgl2(:,:,nlm+1,ksdm)
#ifndef scm
      call qp2sum(tas, c1,iqp2_tas, ksdm, iim*jjm)
      if(mod(nsecond_dyn,21600*nhf_freq).eq.0)
     *   call hf2sum(tas,c1,ihf2_tas,ksdm,iim*jjm)
#endif
      call pbp1sum(tas, c1, ipbp1_tas, ksdm, iim*jjm)

!----------------------------------------------------------------------------
! surface relative humidity
!----------------------------------------------------------------------------
#ifndef scm
      work_2d(:,:) = tracer(:,:,nlm,iq,n4,ksdm) / qsl2(:,:,nlm+1,ksdm)
      call qp2sum(work_2d, c1,iqp2_hurs, ksdm, iim*jjm)
#endif
      
!----------------------------------------------------------------------------
! sea level pressure
!----------------------------------------------------------------------------
      call cslp(psl, ps(1,1,n4,ksdm), th(1,1,nlm,n4,ksdm), phis(1,1,ksdm)
     *,         iim*jjm )
#ifndef scm
      call qp2sum(psl, c1,iqp2_psl, ksdm, iim*jjm)
      if(mod(nsecond_dyn,21600*nhf_freq).eq.0)
     *       call hf2sum(psl, c1,ihf2_psl, ksdm, iim*jjm)
#endif
      call pbp1sum(psl, c1, ipbp1_psl, ksdm, iim*jjm)

!----------------------------------------------------------------------------
! tropopause temperature (minimum temperature above the mixed layer
!----------------------------------------------------------------------------
#ifndef scm
      work_2d(:,:) = th(:,:,nlm-npbl,n4,ksdm) * bpg(:,:,nlm-npbl,ksdm)
      do k = 1,nlm-(npbl+1)
        where( th(:,:,k,n4,ksdm) * bpg(:,:,k,ksdm).lt.work_2d(:,:))
     &     work_2d(:,:) = th(:,:,k,n4,ksdm) * bpg(:,:,k,ksdm)
      enddo
      call qp2sum(work_2d, c1,iqp2_ttrop, ksdm, iim*jjm)

!----------------------------------------------------------------------------
! pressure level for the 0C temperature (melting level), -20C, and -40C:
!----------------------------------------------------------------------------
      work_3dl2(:,:,:) = thl2(:,:,:,ksdm)*bpgl2(:,:,:,ksdm)

      work_2d(:,:) = c0
      where (work_3dl2(:,:,nlm+1)<273.15_dbl_kind) 
     *   work_2d(:,:) = plgl2(:,:,nlm+1,ksdm) 
      do k = nlm, 1, -1
         where(work_3dl2(:,:,k+1) >= 273.15_dbl_kind .and.
     *         work_3dl2(:,:,k) < 273.15_dbl_kind .and.
     *         work_2d(:,:) == c0 )
     *      work_2d(:,:) = work_2d(:,:) + plg(:,:,k,ksdm)
      enddo
      call qp2sum(work_2d,c1,iqp2_meltlev,ksdm,iim*jjm)

      work_2d(:,:) = c0
      do k = nlm, 1, -1
         where(work_3dl2(:,:,k+1) >= 253.15_dbl_kind .and.
     *         work_3dl2(:,:,k) < 253.15_dbl_kind .and.
     *         work_2d(:,:) == c0)
     *      work_2d(:,:) = work_2d(:,:) + plg(:,:,k,ksdm)
      enddo
      call qp2sum(work_2d,c1,iqp2_press20C,ksdm,iim*jjm)

      do k = nlm, 1, -1
         work_2d(:,:) = c0
         where(work_3dl2(:,:,k+1) >= 233.15_dbl_kind .and.
     *         work_3dl2(:,:,k) < 233.15_dbl_kind .and.
     *         work_2d(:,:) == c0)
     *      work_2d(:,:) = work_2d(:,:) + plg(:,:,k,ksdm)
      enddo
      call qp2sum(work_2d,c1,iqp2_press40C,ksdm,iim*jjm)

!----------------------------------------------------------------------------
! 3d specific humidity, cloud water, rain, cloud ice, snow, respectively
!----------------------------------------------------------------------------
      call qp3sum(tracer(1,1,1,iq,n4,ksdm),iim*jjm,c1,iqp3_hus
     +,           ksdm,iim*jjm, parray=qa)
      call qp3mlsum(tracer(1,1,1,iq,n4,ksdm),iim*jjm,c1,iqp3ml_hus
     +,           ksdm,iim*jjm)
      if(mod(nsecond_dyn,21600*nhf_freq).eq.0) then
         do k = 1,mm
            if(abs(pm(k)-85000._dbl_kind).lt.1._dbl_kind) 
     *           call hf2sum(qa(1,1,k), c1,ihf2_hus850, ksdm, iim*jjm)
            if(abs(pm(k)-50000._dbl_kind).lt.1._dbl_kind) 
     *           call hf2sum(qa(1,1,k), c1,ihf2_hus500, ksdm, iim*jjm)
         enddo
      endif
      cwtem(:,:,:) = tracer(:,:,:,icw,n4,ksdm)
	  where(pc_atm(:,:,ksdm) > 0.01_dbl_kind)
     *  cwtem(:,:,nlm) = pblcw_atm(:,:,1,ksdm) *
     *  (plgl2(:,:,nlm+1,ksdm) - plgl2(:,:,nlm,ksdm)) / (100.*pc_atm(:,:,ksdm))
	  pcloc(:,:) = pc_atm(:,:,ksdm) + plgl2(:,:,nlm,ksdm) * 0.01
      call qp3sum_cld(cwtem(1,1,1), pcloc, iim*jjm,c1,iqp3_clw
     +,           ksdm,iim*jjm)
      call qp3sum(tracer(1,1,1,irw,n4,ksdm),iim*jjm,c1,iqp3_rwelq
     +,           ksdm,iim*jjm)
      call qp3sum_cld(tracer(1,1,1,ici,n4,ksdm),pcloc,iim*jjm,c1,iqp3_cli
     +,           ksdm,iim*jjm)
      call qp3sum(tracer(1,1,1,iri,n4,ksdm),iim*jjm,c1,iqp3_rielq
     +,           ksdm,iim*jjm)
#endif

      call pbp2sum(tracer(1,1,1,iq ,n4,ksdm),iim*jjm,c1,ipbp2_wvelq
     +,            ksdm,iim*jjm)
      call pbp2sum(tracer(1,1,1,icw,n4,ksdm),iim*jjm,c1,ipbp2_cwelq
     +,            ksdm,iim*jjm)
      call pbp2sum(tracer(1,1,1,irw,n4,ksdm),iim*jjm,c1,ipbp2_rwelq
     +,            ksdm,iim*jjm)
      call pbp2sum(tracer(1,1,1,ici,n4,ksdm),iim*jjm,c1,ipbp2_cielq
     +,            ksdm,iim*jjm)
      call pbp2sum(tracer(1,1,1,iri,n4,ksdm),iim*jjm,c1,ipbp2_rielq
     +,            ksdm,iim*jjm)

!----------------------------------------------------------------------------
! elqng variables:
!----------------------------------------------------------------------------
#ifndef no_pbl
#ifndef scm
#ifdef eau_ng
      call qp3sum(elqng(1,1,1,iacld,n4,ksdm),iim*jjm,c1
     +,           iqp3_acldelq,ksdm,iim*jjm)
      call qp3sum(elqng(1,1,1,ithcldmclr,n4,ksdm),iim*jjm,c1
     +,           iqp3_cctpelq,ksdm,iim*jjm)
      call qp3sum(elqng(1,1,1,iwvcldmclr,n4,ksdm),iim*jjm,c1
     +,           iqp3_ccwvelq,ksdm,iim*jjm)
      call qp3sum(elqng(1,1,1,irwcldmclr,n4,ksdm),iim*jjm,c1
     +,           iqp3_ccrwelq,ksdm,iim*jjm)
      call qp3sum(elqng(1,1,1,iricldmclr,n4,ksdm),iim*jjm,c1
     +,           iqp3_ccrielq,ksdm,iim*jjm)

      work_2d(:,:)    = c0
      work_3d(:,:,:)  = c0
      work_3db(:,:,:) = elqng(:,:,:,iacld,n4,ksdm)

      work_3d(:,:,1) = work_3db(:,:,1)
      do k = 2, nlm
         work_2d(:,:)   = (c1-max(work_3db(:,:,k-1),work_3db(:,:,k)))
     +                  / (c1-min(work_3db(:,:,k-1),c1-1.e-15_dbl_kind))
         work_3d(:,:,k) = c1-work_2d(:,:)*(c1-work_3d(:,:,k-1))
      enddo
      call qp2sum(work_3d(1,1,nlm),c1,iqp2_elqacld,ksdm,iim*jjm)
      call qp3sum(work_3d(1,1,1),iim*jjm,c1,iqp3_acldtelq,ksdm,iim*jjm)

#endif
#endif
#endif

!----------------------------------------------------------------------------
! saturation mixing ratio and relative humidity
!----------------------------------------------------------------------------
      call pbp2sum(qs(1,1,1,ksdm), iim*jjm, c1, ipbp2_qsat, ksdm, iim*jjm)
      work_3d(:,:,:) = tracer(:,:,:,iq,n4,ksdm)/(qs(:,:,:,ksdm) + 1.0e-10)
      call pbp2sum(work_3d(1,1,1), iim*jjm, c1, ipbp2_hur, ksdm, iim*jjm)
#ifndef scm
      call qp3sum(qs(1,1,1,ksdm),iim*jjm,c1,iqp3_qsat ,ksdm,iim*jjm)
      call qp3sum(work_3d(1,1,1),iim*jjm,c1,iqp3_hur,ksdm,iim*jjm)
      call qp3mlsum(work_3d(1,1,1),iim*jjm,c1,iqp3ml_hur,ksdm,iim*jjm)
#endif

!----------------------------------------------------------------------------
! passive tracers
!----------------------------------------------------------------------------
#ifndef scm
#ifdef passive_tracers
      do n = 1,npassive
         call qp3sum(tracer(1,1,1,ipassive(n),n4,ksdm),iim*jjm,c1
     +,               iqp3_passive(n) ,ksdm,iim*jjm)
!vertically integrated tracer:
         work_2d = c0
         do k = 1, nlm
            work_2d = work_2d + tracer(:,:,k,ipassive(n),n4,ksdm)
     +              * pthick(:,:,k,n4,ksdm)*dsig(k)*gravi
         enddo
         call qp2sum(work_2d(1,1),c1,iqp2_passive(n),ksdm,iim*jjm)

!pbl value:
         call qp2sum(tracer(1,1,nlm,ipassive(n),n4,ksdm),c1
     +,              iqp2_passivepbl(n),ksdm,iim*jjm)

!sfc flux:
         call qp2sum(tracer_flux(1,1,ipassive(n)-nwater,ksdm),c1
     +,              iqp2_tracersrc(n),ksdm,iim*jjm)
      enddo
#endif
#endif
!----------------------------------------------------------------------------
! cloud water
!----------------------------------------------------------------------------
      work_2d = c0
      do k=1,nlm
       work_2d = work_2d + 
     *      tracer(:,:,k,icw,n4,ksdm)*pthick(:,:,k,n4,ksdm)*dsig(k)*gravi
      enddo
#ifndef scm
      call qp2sum(work_2d(1,1),c1,iqp2_clwvi,ksdm,iim*jjm)
#endif
      call pbp1sum(work_2d(1,1),c1,ipbp1_elqcwt,ksdm,iim*jjm)

#ifndef scm
      work_2d = c0
      do k=lpbl+1,nlm
       work_2d = work_2d + 
     *      tracer(:,:,k,icw,n4,ksdm)*pthick(:,:,k,n4,ksdm)*dsig(k)*gravi
      enddo
      call qp2sum(work_2d(1,1),c1,iqp2_pblclwvi,ksdm,iim*jjm)

      work_2d = c0
      work_2d = tracer(:,:,lpbl,icw,n4,ksdm)
     *        * pthick(:,:,lpbl,n4,ksdm)*dsig(lpbl)*gravi
      call qp2sum(work_2d(1,1),c1,iqp2_lpblclwvi,ksdm,iim*jjm)
#endif
     
!----------------------------------------------------------------------------
! rain
!----------------------------------------------------------------------------
      work_2d = c0
      do k=1,nlm
       work_2d = work_2d + 
     *      tracer(:,:,k,irw,n4,ksdm)*pthick(:,:,k,n4,ksdm)*dsig(k)*gravi
      enddo
#ifndef scm
      call qp2sum(work_2d(1,1),c1,iqp2_elqrwt,ksdm,iim*jjm)
#endif
      call pbp1sum(work_2d(1,1),c1,ipbp1_elqrwt,ksdm,iim*jjm)

#ifndef scm
      work_2d = c0
      do k=lpbl+1,nlm
       work_2d = work_2d + 
     *      tracer(:,:,k,irw,n4,ksdm)*pthick(:,:,k,n4,ksdm)*dsig(k)*gravi
      enddo
      call qp2sum(work_2d(1,1),c1,iqp2_pblelqrwt,ksdm,iim*jjm)

      work_2d = c0
      work_2d = tracer(:,:,lpbl,irw,n4,ksdm)
     *        * pthick(:,:,lpbl,n4,ksdm)*dsig(lpbl)*gravi
      call qp2sum(work_2d(1,1),c1,iqp2_lpblelqrwt,ksdm,iim*jjm)
#endif
     
!----------------------------------------------------------------------------
! cloud ice
!----------------------------------------------------------------------------
      work_2d = c0
      do k=1,nlm
       work_2d = work_2d + 
     *      tracer(:,:,k,ici,n4,ksdm)*pthick(:,:,k,n4,ksdm)*dsig(k)*gravi
      enddo
#ifndef scm
      call qp2sum(work_2d(1,1),c1,iqp2_clivi,ksdm,iim*jjm)
#endif
      call pbp1sum(work_2d(1,1),c1,ipbp1_elqcit,ksdm,iim*jjm)

#ifndef scm
      work_2d = c0
      do k=lpbl+1,nlm
       work_2d = work_2d + 
     *      tracer(:,:,k,ici,n4,ksdm)*pthick(:,:,k,n4,ksdm)*dsig(k)*gravi
      enddo
      call qp2sum(work_2d(1,1),c1,iqp2_pblclivi,ksdm,iim*jjm)
     
      work_2d = c0
      work_2d = tracer(:,:,lpbl,ici,n4,ksdm)
     *        * pthick(:,:,lpbl,n4,ksdm)*dsig(lpbl)*gravi
      call qp2sum(work_2d(1,1),c1,iqp2_lpblclivi,ksdm,iim*jjm)
#endif
     
!----------------------------------------------------------------------------
! snow
!----------------------------------------------------------------------------
      work_2d = c0
      do k=1,nlm
       work_2d = work_2d + 
     *      tracer(:,:,k,iri,n4,ksdm)*pthick(:,:,k,n4,ksdm)*dsig(k)*gravi
      enddo
#ifndef scm
      call qp2sum(work_2d(1,1),c1,iqp2_elqrit,ksdm,iim*jjm)
#endif
      call pbp1sum(work_2d(1,1),c1,ipbp1_elqrit,ksdm,iim*jjm)

#ifndef scm
      work_2d = c0
      do k=lpbl+1,nlm
       work_2d = work_2d + 
     *      tracer(:,:,k,iri,n4,ksdm)*pthick(:,:,k,n4,ksdm)*dsig(k)*gravi
      call qp2sum(work_2d(1,1),c1,iqp2_pblelqrit,ksdm,iim*jjm)

      work_2d = c0
      work_2d = tracer(:,:,lpbl,iri,n4,ksdm)
     *        * pthick(:,:,lpbl,n4,ksdm)*dsig(lpbl)*gravi
      enddo
      call qp2sum(work_2d(1,1),c1,iqp2_lpblelqrit,ksdm,iim*jjm)
#endif
     
!----------------------------------------------------------------------------
! water vapor
!----------------------------------------------------------------------------
      work_2d = c0
      do k=1,nlm
       work_2d = work_2d + 
     *      tracer(:,:,k,iq,n4,ksdm)*pthick(:,:,k,n4,ksdm)*dsig(k)*gravi
      enddo
#ifndef scm
      call qp2sum(work_2d(1,1),c1,iqp2_prw,ksdm,iim*jjm)
      if(mod(nsecond_dyn,21600*nhf_freq).eq.0)
     *   call hf2sum(work_2d(1,1),c1,ihf2_prw,ksdm,iim*jjm)
#endif
      call pbp1sum(work_2d(1,1),c1,ipbp1_elqwvt,ksdm,iim*jjm)

#ifndef scm
! water vapor in the pbl only:
      work_2d = c0
      do k=lpbl+1,nlm
       work_2d = work_2d + 
     *      tracer(:,:,k,iq,n4,ksdm)*pthick(:,:,k,n4,ksdm)*dsig(k)*gravi
      enddo
      call qp2sum(work_2d(1,1),c1,iqp2_pblprw,ksdm,iim*jjm)
#endif

!----------------------------------------------------------------------------
! sum
!----------------------------------------------------------------------------
      work_2d = c0
      do k=1,nlm
       work_2d = work_2d + 
     *      tracer(:,:,k,icw,n4,ksdm)*pthick(:,:,k,n4,ksdm)*dsig(k)*gravi +
     *      tracer(:,:,k,irw,n4,ksdm)*pthick(:,:,k,n4,ksdm)*dsig(k)*gravi +
     *      tracer(:,:,k,ici,n4,ksdm)*pthick(:,:,k,n4,ksdm)*dsig(k)*gravi +
     *      tracer(:,:,k,iri,n4,ksdm)*pthick(:,:,k,n4,ksdm)*dsig(k)*gravi +
     *      tracer(:,:,k,iq,n4,ksdm) *pthick(:,:,k,n4,ksdm)*dsig(k)*gravi
      enddo
      call pbp1sum(work_2d(1,1),c1,ipbp1_sumelq,ksdm,iim*jjm)

!----------------------------------------------------------------------------
! surface momentum stresses
!----------------------------------------------------------------------------
#ifndef scm
#ifndef no_pbl
! use corner 1 wind at cell center - later will replace with a proper average
      work_east(:,:) = drag(:,:,ksdm) * u(1,1,:,:,nlm,n4,ksdm)
      work_north(:,:) = drag(:,:,ksdm) * u(2,1,:,:,nlm,n4,ksdm)
      call qp2sum(work_east(1,1),c1,iqp2_tauu,ksdm,iim*jjm)
      call qp2sum(work_north(1,1),c1,iqp2_tauv,ksdm,iim*jjm)
      if(mod(nsecond_dyn,21600*nhf_freq).eq.0) then
         call hf2sum(work_east(1,1),c1,ihf2_tauu,ksdm,iim*jjm)
         call hf2sum(work_north(1,1),c1,ihf2_tauv,ksdm,iim*jjm)
      endif

! contribution to surface torque
      work_2d = a * cos(grid_center_lat(:,:,ksdm)) * work_east
      call gmean_sum_atm(work_2d, iim*jjm, c1, iglob_torts, ksdm, step_sample=0)
#endif

!----------------------------------------------------------------------------
! surface winds
!----------------------------------------------------------------------------
! use corner 1 wind at cell center - later will replace with a proper average
      work_east(:,:) = u(1,1,:,:,nlm,n4,ksdm)
      work_north(:,:) = u(2,1,:,:,nlm,n4,ksdm)
      call qp2sum(work_east(1,1),c1,iqp2_uas,ksdm,iim*jjm)
      call qp2sum(work_north(1,1),c1,iqp2_vas,ksdm,iim*jjm)
      if(mod(nsecond_dyn,21600*nhf_freq).eq.0) then
         call hf2sum(work_east(1,1),c1,ihf2_uas,ksdm,iim*jjm)
         call hf2sum(work_north(1,1),c1,ihf2_vas,ksdm,iim*jjm)
      endif
     
#ifndef no_pbl
#ifdef gravity_wave_drag
!----------------------------------------------------------------------------
! gravity wave drag diagnostics
!----------------------------------------------------------------------------
! use corner 1 wind at cell center - later will replace with a proper average
      work_east(:,:) = u_forcing_gwd(1,1,:,:,ksdm)
      work_north(:,:) = u_forcing_gwd(2,1,:,:,ksdm)
      call qp2sum(work_east(1,1),c1,iqp2_tauugwd,ksdm,iim*jjm)
      call qp2sum(work_north(1,1),c1,iqp2_tauvgwd,ksdm,iim*jjm)

! contribution to surface torque
      work_2d = a * cos(grid_center_lat(:,:,ksdm)) * work_east
      call gmean_sum_atm(work_2d, iim*jjm, c1, iglob_torts, ksdm, step_sample=0)
#endif
#endif

!----------------------------------------------------------------------------
! daily min and max surface air temperature, pbl depth
!----------------------------------------------------------------------------
      tasmin(:,:,ksdm) = min(tasmin(:,:,ksdm), tas(:,:) )
      tasmax(:,:,ksdm) = max(tasmax(:,:,ksdm), tas(:,:) )
#ifndef no_pbl
      psbmin(:,:,ksdm) = min(psbmin(:,:,ksdm), ps(:,:,n4,ksdm)-pb(:,:,n4,ksdm) )
      psbmax(:,:,ksdm) = max(psbmax(:,:,ksdm), ps(:,:,n4,ksdm)-pb(:,:,n4,ksdm) )
#endif
      if ( mod(nsecond_dyn,86400).eq.0) then
        call qp2sum(tasmin(1,1,ksdm),c1,iqp2_tasmin,ksdm,iim*jjm)
        call qp2sum(tasmax(1,1,ksdm),c1,iqp2_tasmax,ksdm,iim*jjm)
#ifndef no_pbl
        call qp2sum(psbmin(1,1,ksdm),c1,iqp2_psbmin,ksdm,iim*jjm)
        call qp2sum(psbmax(1,1,ksdm),c1,iqp2_psbmax,ksdm,iim*jjm)
#endif
        tasmin(:,:,ksdm) = 999.
        tasmax(:,:,ksdm) = 0.0
#ifndef no_pbl
        psbmax(:,:,ksdm) = 0.0
        psbmin(:,:,ksdm) = 1.e+10
#endif
      endif

      tasmin6(:,:,ksdm) = min(tasmin6(:,:,ksdm), tas(:,:) )
      tasmax6(:,:,ksdm) = max(tasmax6(:,:,ksdm), tas(:,:) )
      if ( mod(nsecond_dyn,21600*nhf_freq).eq.0) then
        call hf2sum(tasmin6(1,1,ksdm),c1,ihf2_tasmin,ksdm,iim*jjm)
        call hf2sum(tasmax6(1,1,ksdm),c1,ihf2_tasmax,ksdm,iim*jjm)
        tasmin6(:,:,ksdm) = 999.
        tasmax6(:,:,ksdm) = 0.0
      endif
#endif

#ifndef no_pbl
!----------------------------------------------------------------------------
! pbl top leak diagnostics
!----------------------------------------------------------------------------
      work_2d = c0
      where(pleak(:,:,ksdm).gt.c0) work_2d(:,:) = pleak(:,:,ksdm)
#ifndef scm
      call qp2sum(work_2d(1,1),c1,iqp2_leakp,ksdm,iim*jjm)
#endif
      call pbp1sum(work_2d(1,1), c1, ipbp1_leakp, ksdm, iim*jjm)

      work_2d = c0
      where(pleak(:,:,ksdm).lt.c0) work_2d(:,:) = pleak(:,:,ksdm)
#ifndef scm
      call qp2sum(work_2d(1,1),c1,iqp2_leakn,ksdm,iim*jjm)
#endif
      call pbp1sum(work_2d(1,1), c1, ipbp1_leakn, ksdm, iim*jjm)

      work_2d = c0
      where(pleak(:,:,ksdm).gt.c0) work_2d(:,:) = c1
#ifndef scm
      call qp2sum(work_2d(1,1),c1,iqp2_leakpinc,ksdm,iim*jjm)
#endif
      call pbp1sum(work_2d(1,1), c1, ipbp1_leakpinc, ksdm, iim*jjm)

      work_2d = c0
      where(pleak(:,:,ksdm).lt.c0) work_2d(:,:) = c1
#ifndef scm
      call qp2sum(work_2d(1,1),c1,iqp2_leakninc,ksdm,iim*jjm)
#endif
      call pbp1sum(work_2d(1,1), c1, ipbp1_leakninc, ksdm, iim*jjm)

#ifndef scm
      call qp2sum(psb_smth(1,1,ksdm),c1,iqp2_psbsmth,ksdm,iim*jjm)
#endif

      work_2d = sqrt(psb_lapl(:,:,ksdm)**2)
#ifndef scm
      call qp2sum(work_2d(1,1),c1,iqp2_psblapl,ksdm,iim*jjm)

      call qp2sum(div_mass_flux(1,1,nlm,n3_f,ksdm),gravi,iqp2_pbldmf,
     *            ksdm,iim*jjm)
#endif
      call pbp1sum(div_mass_flux(1,1,nlm,n3_f,ksdm), gravi, ipbp1_pbldmf,
     *             ksdm, iim*jjm)
#endif

!----------------------------------------------------------------------------
! water fill diagnostics
!----------------------------------------------------------------------------

!   water vapor
#ifndef scm
      call qp3sum(fill_rate(1,1,1,iq,ksdm), iim*jjm, c1, iqp3_fill3q,
     *            ksdm, iim*jjm)
      call qp3mlsum(fill_rate(1,1,1,iq,ksdm), iim*jjm, c1, iqp3ml_fill3q,
     *            ksdm, iim*jjm)
#endif
      call pbp2sum(fill_rate(1,1,1,iq,ksdm), iim*jjm, c1, ipbp2_fill3q,
     *            ksdm, iim*jjm)
      work_2d = c0
      work_inc = c0
      work_fill = c0
      do k = 1,nlm
        work_2d(:,:) = work_2d(:,:) + fill_rate(:,:,k,iq,ksdm)
     *                              * pthick(:,:,k,n4,ksdm) * dsig(k)
        where(fill_rate(:,:,k,iq,ksdm).gt.1.e-20_dbl_kind)
     *    work_fill(:,:) = work_fill(:,:) + c1
      enddo
      where(work_fill.gt.c1)work_inc = c1
#ifndef scm
      call qp2sum(work_2d(1,1), gravi, iqp2_fillvq, ksdm, iim*jjm)
      call qp2sum(work_inc(1,1), c1, iqp2_filliq, ksdm, iim*jjm)
      call qp2sum(work_fill(1,1), c1, iqp2_filllq, ksdm, iim*jjm)
#endif
      call pbp1sum(work_2d(1,1), gravi, ipbp1_fillvq, ksdm, iim*jjm)
      call pbp1sum(work_inc(1,1), c1, ipbp1_filliq, ksdm, iim*jjm)
      call pbp1sum(work_fill(1,1), c1, ipbp1_filllq, ksdm, iim*jjm)

!   cloud water
#ifndef scm
      call qp3sum(fill_rate(1,1,1,icw,ksdm), iim*jjm, c1, iqp3_fill3cw,
     *            ksdm, iim*jjm)
      call qp3mlsum(fill_rate(1,1,1,icw,ksdm), iim*jjm, c1, iqp3ml_fill3cw,
     *            ksdm, iim*jjm)
#endif
      call pbp2sum(fill_rate(1,1,1,icw,ksdm), iim*jjm, c1, ipbp2_fill3cw,
     *            ksdm, iim*jjm)
      work_2d = c0
      work_inc = c0
      work_fill = c0
      do k = 1,nlm
        work_2d(:,:) = work_2d(:,:) + fill_rate(:,:,k,icw,ksdm)
     *                              * pthick(:,:,k,n4,ksdm) * dsig(k)
        where(fill_rate(:,:,k,icw,ksdm).gt.1.e-20_dbl_kind)
     *    work_fill(:,:) = work_fill(:,:) + c1
      enddo
      where(work_fill.gt.c1)work_inc = c1
#ifndef scm
      call qp2sum(work_2d(1,1), gravi, iqp2_fillvcw, ksdm, iim*jjm)
      call qp2sum(work_inc(1,1), c1, iqp2_fillicw, ksdm, iim*jjm)
      call qp2sum(work_fill(1,1), c1, iqp2_filllcw, ksdm, iim*jjm)
#endif
      call pbp1sum(work_2d(1,1), gravi, ipbp1_fillvcw, ksdm, iim*jjm)
      call pbp1sum(work_inc(1,1), c1, ipbp1_fillicw, ksdm, iim*jjm)
      call pbp1sum(work_fill(1,1), c1, ipbp1_filllcw, ksdm, iim*jjm)

!   cloud ice
#ifndef scm
      call qp3sum(fill_rate(1,1,1,ici,ksdm), iim*jjm, c1, iqp3_fill3ci,
     *            ksdm, iim*jjm)
      call qp3mlsum(fill_rate(1,1,1,ici,ksdm), iim*jjm, c1, iqp3ml_fill3ci,
     *            ksdm, iim*jjm)
#endif
      call pbp2sum(fill_rate(1,1,1,ici,ksdm), iim*jjm, c1, ipbp2_fill3ci,
     *            ksdm, iim*jjm)
      work_2d = c0
      work_inc = c0
      work_fill = c0
      do k = 1,nlm
        work_2d(:,:) = work_2d(:,:) + fill_rate(:,:,k,ici,ksdm)
     *                              * pthick(:,:,k,n4,ksdm) * dsig(k)
        where(fill_rate(:,:,k,ici,ksdm).gt.1.e-20_dbl_kind)
     *    work_fill(:,:) = work_fill(:,:) + c1
      enddo
      where(work_fill.gt.c1)work_inc = c1
#ifndef scm
      call qp2sum(work_2d(1,1), gravi, iqp2_fillvci, ksdm, iim*jjm)
      call qp2sum(work_inc(1,1), c1, iqp2_fillici, ksdm, iim*jjm)
      call qp2sum(work_fill(1,1), c1, iqp2_filllci, ksdm, iim*jjm)
#endif
      call pbp1sum(work_2d(1,1), gravi, ipbp1_fillvci, ksdm, iim*jjm)
      call pbp1sum(work_inc(1,1), c1, ipbp1_fillici, ksdm, iim*jjm)
      call pbp1sum(work_fill(1,1), c1, ipbp1_filllci, ksdm, iim*jjm)

!   rain
#ifndef scm
      call qp3sum(fill_rate(1,1,1,irw,ksdm), iim*jjm, c1, iqp3_fill3rw,
     *            ksdm, iim*jjm)
      call qp3mlsum(fill_rate(1,1,1,irw,ksdm), iim*jjm, c1, iqp3ml_fill3rw,
     *            ksdm, iim*jjm)
#endif
      call pbp2sum(fill_rate(1,1,1,irw,ksdm), iim*jjm, c1, ipbp2_fill3rw,
     *            ksdm, iim*jjm)
      work_2d = c0
      work_inc = c0
      work_fill = c0
      do k = 1,nlm
        work_2d(:,:) = work_2d(:,:) + fill_rate(:,:,k,irw,ksdm)
     *                              * pthick(:,:,k,n4,ksdm) * dsig(k)
        where(fill_rate(:,:,k,irw,ksdm).gt.1.e-20_dbl_kind)
     *    work_fill(:,:) = work_fill(:,:) + c1
      enddo
      where(work_fill.gt.c1)work_inc = c1
#ifndef scm
      call qp2sum(work_2d(1,1), gravi, iqp2_fillvrw, ksdm, iim*jjm)
      call qp2sum(work_inc(1,1), c1, iqp2_fillirw, ksdm, iim*jjm)
      call qp2sum(work_fill(1,1), c1, iqp2_filllrw, ksdm, iim*jjm)
#endif
      call pbp1sum(work_2d(1,1), gravi, ipbp1_fillvrw, ksdm, iim*jjm)
      call pbp1sum(work_inc(1,1), c1, ipbp1_fillirw, ksdm, iim*jjm)
      call pbp1sum(work_fill(1,1), c1, ipbp1_filllrw, ksdm, iim*jjm)

!   snow
#ifndef scm
      call qp3sum(fill_rate(1,1,1,iri,ksdm), iim*jjm, c1, iqp3_fill3ri,
     *            ksdm, iim*jjm)
      call qp3mlsum(fill_rate(1,1,1,iri,ksdm), iim*jjm, c1, iqp3ml_fill3ri,
     *            ksdm, iim*jjm)
#endif
      call pbp2sum(fill_rate(1,1,1,iri,ksdm), iim*jjm, c1, ipbp2_fill3ri,
     *            ksdm, iim*jjm)
      work_2d = c0
      work_inc = c0
      work_fill = c0
      do k = 1,nlm
        work_2d(:,:) = work_2d(:,:) + fill_rate(:,:,k,iri,ksdm)
     *                              * pthick(:,:,k,n4,ksdm) * dsig(k)
        where(fill_rate(:,:,k,iri,ksdm).gt.1.e-20_dbl_kind)
     *    work_fill(:,:) = work_fill(:,:) + c1
      enddo
      where(work_fill.gt.c1)work_inc = c1
#ifndef scm
      call qp2sum(work_2d(1,1), gravi, iqp2_fillvri, ksdm, iim*jjm)
      call qp2sum(work_inc(1,1), c1, iqp2_filliri, ksdm, iim*jjm)
      call qp2sum(work_fill(1,1), c1, iqp2_filllri, ksdm, iim*jjm)
#endif
      call pbp1sum(work_2d(1,1), gravi, ipbp1_fillvri, ksdm, iim*jjm)
      call pbp1sum(work_inc(1,1), c1, ipbp1_filliri, ksdm, iim*jjm)
      call pbp1sum(work_fill(1,1), c1, ipbp1_filllri, ksdm, iim*jjm)

!----------------------------------------------------------------------------
! products of 3d fields
!----------------------------------------------------------------------------
#ifndef scm
       call qp3_product_sum(ua, va, iqp3_cvuva, c1, ksdm)
       call qp3_product_sum(ua, ta, iqp3_cvuta, c1, ksdm)
       call qp3_product_sum(va, ta, iqp3_cvvta, c1, ksdm)
       call qp3_product_sum(wap, ta, iqp3_cvwapta, c1, ksdm)
       call qp3_product_sum(va, zeta_a, iqp3_cvvzeta, c1, ksdm)
       call qp3_product_sum(ua, qa, iqp3_cvuhusa, c1, ksdm)
       call qp3_product_sum(va, qa, iqp3_cvvhusa, c1, ksdm)
       call qp3_product_sum(wap, qa, iqp3_cvwhusa, c1, ksdm)
       call qp3_product_sum(ua, ua, iqp3_cvuua, c1, ksdm)
       call qp3_product_sum(va, va, iqp3_cvvva, c1, ksdm)
       call qp3_product_sum(ta, ta, iqp3_cvtta, c1, ksdm)
       call qp3_product_sum(va, zg, iqp3_cvvzga, c1, ksdm)

!----------------------------------------------------------------------------
! more global means : total kinetic energy
!----------------------------------------------------------------------------
      work_2d = 0.0
      do k = 1,nlm
         work_2d(:,:) = work_2d(:,:) + ke(:,:,k,ksdm)
     *		 * pthick(:,:,k,n4,ksdm) * dsig(k)
      enddo
      work_2d = work_2d * gravi
      call gmean_sum_atm(work_2d, iim*jjm, c1, iglob_enek, ksdm )

!----------------------------------------------------------------------------
! more global means : total surface torque (mountain torque component)
!----------------------------------------------------------------------------

!  if this is the first time step, get the gradient of the surface topography
      if(first) then

! psuedocode
!        call gradient(....,xin,xout)
!        transform to X1 (x,y,z)
!	 average xout to center (the 1 component is work_east)
!        transform to X2 (e,n) using corner 1

! get normal and tangential derivatives
!tdrmod
!	call NORMAL_DERIVATIVE(
!    *			phis,
!    *			inv_p_dist,
!    *			inv_c_dist,
!    *			work_n)
!call TANGENTIAL_DERIVATIVE(
!    *			phis,
!    *			inv_p_dist,
!    *			inv_c_dist,
!    *			work_t)
!
! get gradient from normal and tangential derivatives
!       call vect_rot_and_avg_to_cell_center
!    *        ( work_n(:,:,:), work_t(:,:,:),
!    *          work_east(:,:), work_north(:,:),
!    *          c_dist(:,:,:,ksdm),
!    *          cos_rot_angle(:,:,:,ksdm), sin_rot_angle(:,:,:,ksdm) )
!endtdrmod

! end psuedocode

      endif
      work_2d = ps(:,:,n4,ksdm) * work_east(:,:) * gravi
     *		 * cos(grid_center_lat(:,:,ksdm)) * a
      call gmean_sum_atm(work_2d, iim*jjm, c1, iglob_torts, ksdm )
     
#endif
      enddo	! do ksdm=1,nsdm

#ifndef scm
!  get the mjo2 diagnostic chi200
      if(mjo2_log(imjo2_chi200)) then
         if(mod(nsecond_dyn,21600).eq.0.or..not.amip_sampling) then
!  do multigrid solution for chi200
            do k = 1,2
               call multigrid (1,div_200(:,:,:),chi_200(:,:,:))
            enddo
            imjo2_sample(imjo2_chi200) = imjo2_sample(imjo2_chi200) + 1
            mjo2(:,:,mjo2_indx(imjo2_chi200),:) =
     *                mjo2(:,:,mjo2_indx(imjo2_chi200),:) + chi_200(:,:,:)
         endif
      endif
#endif

      first = .false.

c-------------------------------------


      end subroutine qp_pbp_diagnostics




      end module qp_pbp_diagnostics_module
