      module buoyshear_module

      use kinds
      use physical_parameters
      use pbl_parameters

      implicit none
      private

      public :: buoyshear
      
      contains

!---------------------------------------------------------------------------
      Subroutine BUOYSHEAR(len, negprod, shear, shear0, strts,     &
                           epslnb, entran, dhb, drb, dwb, bync,    &
                           psbloc, buoy0, fsvs, ps, pb, diss, pc,  &
                           ts, wm, fhs, fws, alphab, krash2)
     
      integer(kind=int_kind), intent(in) :: len
      real(kind=dbl_kind), intent(in) :: krash2
      real(kind=dbl_kind), dimension(len), intent(in) ::           &
          psbloc, dwb, drb, dhb, entran, epslnb, shear0, fsvs,     &
          ps, pb, diss, pc, ts, wm, fhs, fws, alphab
      real(kind=dbl_kind), dimension(len), intent(inout) ::        &
          bync, buoy0, shear

      logical(kind=log_kind), dimension(len), intent(in) ::        &
          strts
      logical(kind=log_kind), dimension(len), intent(out) ::        &
          negprod

!-- Compute fsvb for all non-stratus points which
!-- allows computation of B for new cubic TKE equation

      real (kind=dbl_kind) :: &
         hltfwb, &
         fhc, &
         hltfwc, &
         fullsum, &
         zerosum, &
         dissfrac
      real (kind=dbl_kind), dimension(len) ::  &
         fsvb, &
         betab, &
         fhb, &
         fwb, &
         tce, &
         gammac, &
         clrfrc, &
         cldfrc, &
         alphac, &
         betac, &
         epslnc, &
         wscs, &
         wscc, &
         wcbc, &
         wcbb, &
         fsvcm, &
         fsvcp

      integer (kind=int_kind) :: &
         i


      do i = 1,len
        negprod(i) = .false.           ! default value
!-- for now, ignore shear1 contribution
!       shear(i) = shear0(i)+entran(i)*shear1(i)
        shear(i) = shear0(i)
        if (.not.strts(i)) then
          betab(i) = 1.0-delta*epslnb(i)
          fhb(i) = -entran(i)*dhb(i)+drb(i)
          fwb(i) = -entran(i)*dwb(i)
          fsvb(i) = fhb(i)-betab(i)*hltm*fwb(i)
          bync(i) = kb2*psbloc(i)*((fsvs(i)/ps(i))+  &
                    (fsvb(i)/pb(i)))
          buoy0(i) = kb2*psbloc(i)*fsvs(i)/ps(i)

          fullsum = bync(i)+shear(i)
          zerosum = buoy0(i)+shear0(i)
          dissfrac = krash2*diss(i)

          if ((fullsum < dissfrac).and.(zerosum < dissfrac))  &
             negprod(i) = .true.
        endif
      enddo

      do i = 1,len
        if (strts(i)) then     ! do stratus case

!-- for now:  compute buoyancy (bync) as done in grossp and old ntrain
!-- i've currently set this up as done in the SOB model (bblmolm.f),
!-- whereby we use the positive E formulation for the stratus case
!-- all the time!

          wscs(i)=kb2*(ps(i)-pc(i))/ps(i)                                
          wscc(i)=kb2*(ps(i)-pc(i))/pc(i)                                
          wcbc(i)=kb2*(pc(i)-pb(i))/pc(i)                                
          wcbb(i)=kb2*(pc(i)-pb(i))/pb(i)

          tce   (i)=ts(i)*(pc(i)/ps(i))**kappa                        
          epslnc(i)=tce(i)*cpbl                                      
          gammac(i)=gamfac*wm(i)/(tce(i)*tce(i))                     
          alphac(i)=(c1+deltap1*gammac(i)*epslnc(i))  &
                    /(c1+gammac(i))
          betac (i)=c1-delta*epslnc(i)                           
          cldfrc(i)=(pc(i)-pb(i))/psbloc(i)                          
          clrfrc(i)=c1-cldfrc(i)                                 

          fhb(i)=-entran(i)     *dhb(i)+drb(i)                       
          hltfwb=-entran(i)*hltm*dwb(i)                              
          fhc   =fhb(i)*clrfrc(i)+fhs(i)*cldfrc(i)                   
          hltfwc=hltfwb*clrfrc(i)+fws(i)*cldfrc(i)                   
          fsvcm(i)=          fhc- betac(i)*hltfwc                    
          fsvcp(i)=alphac(i)*fhc-epslnc(i)*hltfwc                    
          fsvb (i)=alphab(i)*fhb(i)-epslnb(i)*hltfwb                 
          bync(i)=wscs(i)*fsvs(i) +wscc(i)*fsvcm(i)     &             
                   +wcbc(i)*fsvcp(i)+wcbb(i)*fsvb(i)                 

        endif
      enddo

      end subroutine buoyshear


      end module buoyshear_module
