!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module chunk_parameters


!***********************************************************************
!
!	this module defines the number of chunks a subdomain may be
!            broken into (nchunks_*), the offset of the starting memory
!            position of a chunk relative to the first chunk (ioffset_*)
!            and the size of the chunks (lenchunk_*)
!
!       Chunking is defined for these operations:
!            convective parameterization = cup
!
!***********************************************************************

      use kinds
      use domain_decomposition

      implicit none

      integer (kind=int_kind), parameter :: 
     *   nchunks_cup = 1
     *,  nchunks_rad  = 1
     *,  nchunks_eau  = 1  ! currently doesn't work for other values
     *,  nchunks_ocn  = 1

      integer (kind=int_kind), dimension(nchunks_cup) ::
     *   ioffset_cup,   lenchunk_cup
      integer (kind=int_kind), dimension(nchunks_rad) ::
     *   ioffset_rad,   lenchunk_rad
      integer (kind=int_kind), dimension(nchunks_eau) ::
     *   ioffset_eau,   lenchunk_eau
      integer (kind=int_kind), dimension(nchunks_ocn) ::
     *   ioffset_ocn,   lenchunk_ocn


!***********************************************************************

      contains

!***********************************************************************


        subroutine init_chunks

        real (kind=dbl_kind) :: temlen
        integer (kind=int_kind) :: n

!  test that number of chunks is less than or equal to grid cells
!    in subdomain
        if(nchunks_cup.gt.iim*jjm) stop 'nchunks_cup too large'
        if(nchunks_rad.gt.iim*jjm) stop 'nchunks_rad too large'
        if(nchunks_eau.gt.iim*jjm) stop 'nchunks_eau too large'
        if(nchunks_ocn.gt.iim*jjm) stop 'nchunks_ocn too large'

!  calculate chunk sizes and offsets
         temlen = float(iim*jjm)/nchunks_cup
         ioffset_cup(1) = 0
         do n = 2,nchunks_cup
            ioffset_cup(n) = (n-1)*temlen + 0.1
            lenchunk_cup(n-1) = ioffset_cup(n) - ioffset_cup(n-1)
         enddo
         lenchunk_cup(nchunks_cup) = iim*jjm - ioffset_cup(nchunks_cup)

         temlen = float(iim*jjm)/nchunks_rad
         ioffset_rad(1) = 0
         do n = 2,nchunks_rad
            ioffset_rad(n) = (n-1)*temlen + 0.1
            lenchunk_rad(n-1) = ioffset_rad(n) - ioffset_rad(n-1)
         enddo
         lenchunk_rad(nchunks_rad) = iim*jjm - ioffset_rad(nchunks_rad)

         temlen = float(iim*jjm)/nchunks_eau
         ioffset_eau(1) = 0
         do n = 2,nchunks_eau
            ioffset_eau(n) = (n-1)*temlen + 0.1
            lenchunk_eau(n-1) = ioffset_eau(n) - ioffset_eau(n-1)
         enddo
         lenchunk_eau(nchunks_eau) = iim*jjm - ioffset_eau(nchunks_eau)

         temlen = float(iim*jjm)/nchunks_ocn
         ioffset_ocn(1) = 0
         do n = 2,nchunks_ocn
            ioffset_ocn(n) = (n-1)*temlen + 0.1
            lenchunk_ocn(n-1) = ioffset_ocn(n) - ioffset_ocn(n-1)
         enddo
         lenchunk_ocn(nchunks_ocn) = iim*jjm - ioffset_ocn(nchunks_ocn)


	end subroutine init_chunks

	end module chunk_parameters






