C--------------------

      SUBROUTINE CLOUDY(sqrt3,ggv,omcld
     *,                 AGVCD,RSVCD,CSMCLD
     *,                 CMUC,SCOSC,SC,ACLD
     *,                 TAUC,RCO,RCG,OZCD,SWILC
     *,                 RVBC,RVDC,RNBC,RNDC
     *,                 RC2,TR2,TR3,RSNCD,AGNCD,DSCLD,dscldtom
     *,                 NSOL,NCLD,NCLMP1,NCLDNP,LITD,dooz
     *,                 len, imnnp, imlmp1, nlmp1)

c	Calculates atmospheric heating rate due to cloudy sky.  Performed
c	only at cloudy points where sun is up.

c	Modifications:
c		- replaced gcmlib calls with inlined code. kwitt 1/27/92
c               - replaced common block work with argument list
c                         d.d.11/09/92
c		- cloud optical depth, single scatter albedo, and asymmetry
c		  constant now passed in as arguments instead of being 
c		  prescribed from within the radiation code
c		     dd 02/24/92

C--------------------

      use kinds

      IMPLICIT  none

C    Declare argument list arrays
      integer len, imnnp, imlmp1, nlmp1
      REAL (kind=dbl_kind) ::
     *        sqrt3,ggv(imlmp1),omcld(imlmp1)
     *,       AGVCD(len),RSVCD(len)
     *,       CSMCLD(len)
     *,       CMUC(len),SCOSC(len)
     *,       SC(len),ACLD(IMLMP1)
     *,       TAUC(IMLMP1),RCO(len),RCG(len)
     *,       OZCD(IMNNP),SWILC(IMLMP1),dscldtom(len)
     *,       RVBC(len),RVDC(len),RNBC(len),RNDC(len)
     *,       RC2(len),TR2(len),TR3(len)
     *,       RSNCD(len),AGNCD(len),DSCLD(len)
      integer (kind=int_kind) ::
     *      NSOL,NCLD,NCLMP1,NCLDNP,LITD(IMNNP)
      logical (kind=log_kind) ::
     *      dooz

C    Declare local arrays

      REAL (kind=dbl_kind) ::
     *     FK(5),XK(5),F2(IMLMP1),CR(IMNNP),E2(IMLMP1),U1(IMLMP1),
     *     AK(IMLMP1),DA(IMLMP1),DB(IMLMP1),ALF1(IMLMP1),
     *     ALF2(IMLMP1),RVNBC(len),UL(IMNNP),AVBC(len)
     *,    PU(IMNNP),TA(IMLMP1),WA(IMLMP1),OA(IMLMP1),eo(IMLMP1)
     *,    DPC(IMLMP1),W(IMNNP),DN(IMNNP),UP(IMNNP),VM(IMNNP)
     *,    CSS(IMLMP1),CCU(IMLMP1),VP(IMNNP),TAUT(len),CM(IMNNP)
     *,    DL(IMNNP),SWIL(IMLMP1),SWALE(IMNNP),DEN(IMLMP1)
     *,    TR1(len),SOL(IMNNP),ANBC(len),ggpv(imlmp1)
     *,    ggsqv(imlmp1),gsqm1v(imlmp1),ggpp1v(imlmp1)
     *,    ggpm1v(imlmp1)
      REAL (kind=dbl_kind) ::
     *     sctem(len),acldtem(imlmp1),rnbctem(len)
     *,    rndctem(len)
      integer (kind=int_kind) ::
     *      lp, nnp, i, l, l0, l1, ik, k

      DATA FK/0.107, 0.104, 0.073, 0.044, 0.025/
      DATA XK/0.005, 0.041, 0.416, 4.752, 72.459/

      LP = nLMP1 * NCLD
      nnp = nlmp1+1

      DO I=1,NCLD
         DN(I)    = 0.
         TR1(I)   = 0.
         RVNBC(I) = 0.
      ENDDO
      do i = 1,nclmp1
        acldtem(i) = 0.0
            ggsqv(i)  = ggv(i)**2
            ggpv(i)   = ggv(i)/(1.+ggv(i)) * 0.75
            ggpp1v(i) = 1. + ggpv(i)
            ggpm1v(i) = ggpv(i) - 1.
            gsqm1v(i) = 1. - ggsqv(i) 
  
      enddo

      IF (DOOZ) THEN
         DO L=2,nNP
            L0 = (L-2) * NCLD
            L1 = (L-1) * NCLD
            DO I=1,NCLD
               W(L0+I) = OZCD(L1+I) * CSMCLD(I)
            ENDDO
         ENDDO

         DO I=1,NCLMP1
            eo(I) = 103.63 * W(I)
            eo(I) = eo(I) * eo(I) * eo(I)
            VP(I) = 1. + 138.57 * W(I)

	    cm(i) = vp(i) ** .805
            DN(NCLD+I) = 0.02118 * W(I) / (1. + 0.042 * W(I) +
     *         0.000323 * W(I) * W(I)) + 1.08173 * W(I) /
     *         CM(I) + 0.0658 * W(I) / (1.   + eo(I))

C     COMPUTE OZONE HEATING ON THE WAY DOWN

            acldtem(I) = DN(NCLD+I) - DN(I)
         ENDDO

         DO L=1,nLMP1
            L0 = (L-1) * NCLD
            DO I=1,NCLD
               acldtem(L0+I) = acldtem(L0+I) * SCOSC(I)
            ENDDO
         ENDDO
      ELSE
         DO I=1,NCLD
            DN(LP+I) = 0.
         ENDDO
      ENDIF

C     DOWNFLUX AND GROUND ABSORPTION FOR LAMBDA UNDER 0.7 MICRONS       
C     AS ABOVE BUT FOR LAMDA BETWEEN 0.7 AND 0.9 MICRONS

      DO I=1,NCLD
         sctem(I) = 0.5 - DN(LP+I)
         TAUT(I) = sctem(I) * TR3(I) * SCOSC(I)
         AVBC(I) = TAUT(I) * (1. - RSVCD(I))
      ENDDO

      do i = 1,ncld 
         rvbc(litd(i)) = taut(i)
      enddo

      DO I=1,NCLD
         sctem(I) = sctem(I) * (1. - RCG(I)) * SCOSC(I)
         TAUT(I) = (sctem(I) - AVBC(I)) / (1. - AGVCD(I))
      ENDDO

      do i = 1,ncld 
         rvdc(litd(i)) = taut(i)
      enddo

      DO I=1,NCLD
         rnbctem(I) = 0.147 * TR2(I) * SCOSC(I)
         ANBC(I) = (1. - RSNCD(I)) * rnbctem(I)
         rndctem(I) = (0.147 * (1. - RC2(I)) * SCOSC(I)
     *        - ANBC(I)) / (1. - AGNCD(I))
         sctem(I) = sctem(I) + (1. - RC2(I)) * SCOSC(I) * 0.147
      ENDDO

      IF (DOOZ) THEN
         DO I=1,NCLD
            UP(LP+I) = DN(LP+I)
            W(LP+I)  = OZCD(LP+I) * (1.9 e 0 + CSMCLD(I))
         ENDDO

         DO I=1,NCLMP1
            W(I) = 1.9 * OZCD(I)
         ENDDO

         DO L=1,nLMP1
            L0 = (L-1) * NCLD
            DO I=1,NCLD
               W(L0+I) = W(LP+I) - W(L0+I)
            ENDDO
         ENDDO

         DO I=1,NCLMP1
            eo(I) = 103.63 * W(I)
            eo(I) = eo(I) * eo(I) * eo(I)
            VP(I) = 1. + 138.57 * W(I)

	    cm(i) = vp(i) ** .805
            UP(I) = 0.02118 * W(I) / (1. + 0.042 * W(I) +
     *          0.000323 * W(I) * W(I)) + 1.08173 * W(I) /
     *          CM(I) + 0.0658 * W(I) / (1. + eo(I))
         ENDDO

C        COMPUTE OZONE HEATING ON THE WAY UP                            

         DO I=1,NCLMP1
            W(I) = UP(I) - UP(NCLD+I)
         ENDDO

         DO L=1,nLMP1
            L0 = (L-1) * NCLD
            DO I=1,NCLD
              acldtem(L0+I)=acldtem(L0+I)+W(L0+I) * RCO(I) * SCOSC(I)
            ENDDO
         ENDDO

         do i=1,nclmp1
            acldtem(i) = max(0.0_dbl_kind, acldtem(i))
         enddo
      ENDIF

      DO L=1,nLMP1
         L0 = (L-1) * NCLD
         DO I=1,NCLD
            OA(L0+I)  = CMUC(I)
         ENDDO
      ENDDO

      DO I=1,NCLMP1
         TA(I) = 0.5 - GGPv(i) * OA(I)
         WA(I) = 1. - ta(i)
      ENDDO

C     CYCLE OVER FIVE BANDS

      DO IK=1,5

C     COMPUTE TOTAL OPTICAL DEPTH AND SINGLE SCATTERING ALBEDO
C     PIZERO OF CLOUDS IS 0.99

         DO I=1,NCLMP1
            CCU(I) = TAUC(I) + SWILC(I) * XK(IK)
            CR(I)  = TAUC(I) / CCU(I) * omcld(i)

C     COMPUTE REFLECTION AND TRANSMISSION FOR DIFFUSE INCIDENCE        

            vp(i) = sqrt((1. - cr(i) * ggv(i)))
            vm(i) = sqrt((1. - cr(i)))
            U1(I) = VP(I) / VM(I)
            DPC(I) = SQRT3 * VP(I) * VM(I) * CCU(I)
            E2(I) = -DPC(I)
         ENDDO

         DO I=1,NCLMP1
            e2(i) = exp(e2(i))
            W(I) = U1(I) + 1.
            CM(I) = U1(I) - 1.
            W(I) = W(I) * W(I)
            CM(I) = W(I) - (cm(i) * cm(i)) * E2(I) * E2(I)
            PU(I) = (U1(I) * U1(I) - 1.) * (1. - E2(I) * E2(I)) / CM(I)
            CSS(I) = 4. * U1(I) * E2(I) / CM(I)

C     DELTA-SCALING OF ALL PERTINENT QUANTITIES

            VP(I) = 1. - GGSQv(i) * CR(I)
            CCU(I) = CCU(I) * VP(I)
            CR(I) = CR(I) * gsqm1v(i) / VP(I)
         ENDDO

C     COMPUTE DIRECT TRANSMISSION

         DO I=1,NCLD
            SOL(I) = SCOSC(I) * FK(IK)
         ENDDO

         DO L=1,nLMP1
            L0 = (L-1) * NCLD
            DO I=1,NCLD
               DPC(L0+I) = CCU(L0+I) * CSMCLD(I)
            ENDDO
         ENDDO

         DO I=1,NCLMP1
c           eo(I)  = -DPC(I)
            eo(i) = exp(-dpc(i))
         ENDDO

C     COMPUTE DIFFUSE REFLECTION AND TRANSMISSION FOR DIRECT BEAM
C     SET UP DELTA-EDDINGTON PARAMETERS

         DO L=1,nLMP1
            L0 = (L-1) * NCLD
            L1 = L * NCLD
            DO I=1,NCLD
               SOL(L1+I) = SOL(L0+I) * eo(L0+I)
            ENDDO
         ENDDO

         DO I=1,NCLD
            RVNBC(I) = RVNBC(I) + SOL(LP+I)
         ENDDO

         DO I=1,NCLMP1
            SWIL(I) = 1.75 - CR(I) * GGPP1v(i)
            SWALE(I) = -0.25 - CR(I) * GGPM1v(i)

            ak(i) = sqrt(SWIL(I) * SWIL(I) - SWALE(I) * SWALE(I))
            ALF1(I) = SWIL(I) * WA(I) + SWALE(I) * TA(I)
            ALF2(I) = SWIL(I) * TA(I) + SWALE(I) * WA(I)

            VP(I) = 1. - AK(I) * OA(I)
            VM(I) = 1. + AK(I) * OA(I)
            UL(I) = TA(I) - ALF2(I) * OA(I)
            U1(I) = WA(I) + ALF1(I) * OA(I)
            CM(I) = ALF2(I) + AK(I) * TA(I)
            DL(I) = ALF2(I) - AK(I) * TA(I)
            E2(I) = ALF1(I) - AK(I) * WA(I)
            F2(I) = ALF1(I) + AK(I) * WA(I)

            DEN(I) = 1. - AK(I) * AK(I) * OA(I) * OA(I)
            den(i) = max(.000001_dbl_kind, den(i))

            DPC(I) = AK(I) * CCU(I)
            da(i) = exp(-dpc(i))
            DPC(I) = 2. * DPC(I)

            db(i) = exp(-dpc(i))
            DEN(I) = DEN(I) * (AK(I) + SWIL(I) +
     *         (AK(I) - SWIL(I)) * DB(I))

C     COMPUTE UPWARD AND DOWNWARD FLUXES AT LAYER BOUNDARIES

            UP(I) = (CR(I) * (VP(I) * CM(I) - VM(I) * DL(I) * DB(I) -
     *         2. * AK(I) * UL(I) * eo(I) * DA(I)) / DEN(I)) * sol(i)

            DN(NCLD+I) = sol(i) * (-CR(I) * (VM(I) * F2(I) * eo(I) -
     *         VP(I) * E2(I) * eo(I) * DB(I) - 2. * AK(I) * U1(I) *
     *         DA(I)) / DEN(I))
         ENDDO

C     FILL UP BOUNDARY TERMS

         DO I=1,NCLD
            UP(LP+I) = RSNCD(I) * SOL(LP+I)
            PU(LP+I) = AGNCD(I)

            DN(I) = 0.
            CR(I) = 0.
            CM(I) = 1.
            VP(I) = 0.
            VM(I) = UP(I)
         ENDDO

C     SAFETY NET

         do i=1,ncldnp
            up(i) = max(0.0_dbl_kind, up(i))
            dn(i) = max(0.0_dbl_kind, dn(i))
            sol(i) = max(0.0_dbl_kind, sol(i))
         enddo

         do i=1,nclmp1
            pu(i) = max(0.0_dbl_kind, pu(i))
            css(i) = max(0.0_dbl_kind, css(i))

C     FLUX ADDING OF LAYERS
C     COMPUTE MAGNIFICATION FACTORS

            eo(I) = CSS(I) * CSS(I)
         ENDDO

         DO L=1,nLMP1
            L0 = (L-1) * NCLD
            L1 = L * NCLD

            DO I=1,NCLD
               CR(L1+I) = PU(L0+I) + eo(L0+I) * CR(L0+I) * CM(L0+I)
               CM(L1+I) = 1. / (1. - CR(L1+I) * PU(L1+I))
            ENDDO
         ENDDO

C     COMPUTE FLUXES ON THE WAY DOWN

         DO I=1,NCLMP1
            DB(I) = CR(I) * UP(I)
            eo(I) = CSS(I) * CM(I)
         ENDDO

         DO L=1,nLMP1
            L0 = (L-1) * NCLD
            L1 = L * NCLD
            DO I=1,NCLD
               VP(L1+I) = eo(L0+I) * (VP(L0+I) + DB(L0+I)) + DN(L1+I)
            ENDDO
         ENDDO

         DO I=1,NCLMP1
            VM(NCLD+I) = CM(NCLD+I) * (VP(NCLD+I) * PU(NCLD+I) +
     *         UP(NCLD+I))
         ENDDO

         DO I=1,NCLD
            UL(LP+I) = VM(LP+I)
            DL(LP+I) = VP(LP+I) + UL(LP+I) * CR(LP+I) + SOL(LP+I)
            TR1(I) = TR1(I) + DL(LP+I) - UL(LP+I)
         ENDDO

C     COMPUTE FLUXES ON THE WAY UP

         DO I=1,NCLMP1
            eo(I) = CSS(I) * CM(I)
         ENDDO

         DO K=1,nLMP1
            L=nNP-K
            L0 = (L-1) * NCLD
            L1 = L * NCLD
            DO I=1,NCLD
               UL(L0+I) = VM(L0+I) + UL(L1+I) * eo(L0+I)
            ENDDO
         ENDDO

         DO I=1,NCLMP1
            DL(I) = VP(I) + UL(I) * CR(I) + SOL(I)
         ENDDO

         DO I=1,NCLMP1
            acldtem(I) = acldtem(I) + DL(I) - UL(I) - (DL(NCLD+I) -
     *         UL(NCLD+I))
         ENDDO
      ENDDO

      DO I=1,NCLD
         TAUT(I) = rnbctem(I) + RVNBC(I)
      ENDDO

      do i = 1,ncld 
         rnbc(litd(i)) = taut(i)
      enddo

      DO I=1,NCLD
        TAUT(I)=rndctem(I) + (TR1(I) - RVNBC(I) * (1. - RSNCD(I))) /
     *      (1. - AGNCD(I))
      ENDDO

      do i = 1,ncld 
         rndc(litd(i)) = taut(i)
      enddo

      DO I=1,NCLD
         TAUT(I) = sctem(I) + TR1(I)
      ENDDO

      do i = 1,ncld 
         sc(litd(i)) = taut(i)
      enddo

C     ADD UP ABSORPTION IN COLUMN

      DO I=1,NCLMP1
         eo(I) = acldtem(I)
      ENDDO

      do i = 1,nclmp1 
         acld(litd(i)) = eo(i)
      enddo

      DO I=1,NSOL
         DSCLD(I) = SC(I)
         DSCLDtom(I) = SC(I)
      ENDDO

      DO L=1,nLMP1
         L0 = (L-1) * NSOL
         DO I=1,NSOL
            DSCLD(I) = DSCLD(I) + ACLD(L0+I)
         ENDDO
      ENDDO

      DO L=2,nLMP1
         L0 = (L-1) * NSOL
         DO I=1,NSOL
            DSCLDtom(I) = DSCLDtom(I) + ACLD(L0+I)
         ENDDO
      ENDDO

      RETURN
      END
