      module comp_tke_module

      use kinds
      use physical_parameters
      use pbl_parameters
      use cpl_cubic_module
      use buoyshear_module
      use negtke_module

      implicit none
      private

      public :: comp_tke, compe, comp_tke_classic, compri
      
      contains

!======================================================================
! BEGINNING OF COMPRI
!======================================================================
      Subroutine CompRi(len,dzm,em,dsv,safe,  &
                        strts,tb,rib)

!---------------------------------------------------------------------------
! PURPOSE
!---------------------------------------------------------------------------
!-- compute the PBL top Richardson number

      implicit none

!---------------------------------------------------------------------------
! INTENT IN
!---------------------------------------------------------------------------
      integer (kind=int_kind), intent(in) ::  &
         len
      real (kind=dbl_kind), intent(in), dimension(len) ::  &        
         dzm, &
         em, &
         dsv, &
         safe, &
         tb
      logical (kind=log_kind), intent(in), dimension(len) :: &
         strts

!---------------------------------------------------------------------------
! INTENT OUT
!---------------------------------------------------------------------------
      real (kind=dbl_kind), intent(out), dimension(len) ::  &     
         rib

!---------------------------------------------------------------------------
! LOCAL
!---------------------------------------------------------------------------
      integer (kind=int_kind) :: &
         i

      do i = 1,len
        if (strts(i)) then
          rib(i) = grav*safe(i)*dzm(i)/(spec_heat_cp*tb(i)*em(i))
        else
          rib(i) = grav*dsv(i)*dzm(i)/(spec_heat_cp*tb(i)*em(i))
        endif
        rib(i) = max(0.0_dbl_kind, rib(i))
      enddo

      end subroutine compri
!======================================================================
! END OF COMPRI
!======================================================================

!======================================================================
! BEGINNING OF COMPE
!======================================================================
      subroutine compe(len,bone,btwo,rob,tke, &
                       rib,zb,alphab,drb,tb, &
                       strts,dpent)

!---------------------------------------------------------------------------
! PURPOSE
!---------------------------------------------------------------------------
!-- compute the PBL top entrainment rate
!--   test modified formula by C. Konor if konor=T

      implicit none

!---------------------------------------------------------------------------
! INTENT IN
!---------------------------------------------------------------------------
      integer (kind=int_kind), intent(in) :: &
         len
      real (kind=dbl_kind), intent(in) :: &
         bone, &
         btwo
      real (kind=dbl_kind), dimension(len), intent(in) :: &
         rob, &
         tke, &
         rib, &
         zb, &
         alphab, &
         drb, &
         tb
      logical (kind=log_kind), dimension(len), intent(in) :: &
         strts

!---------------------------------------------------------------------------
! INTENT OUT
!---------------------------------------------------------------------------
      real (kind=dbl_kind), dimension(len), intent(out) :: &
         dpent

!---------------------------------------------------------------------------
! LOCAL
!---------------------------------------------------------------------------
      integer (kind=int_kind) :: &        
         i          
      real (kind=dbl_kind) :: &         
         gde, &
         bkone=0.4, &
         bktwo=0.8, &
         bk2tilda, &
         earg

      if (konor) then
        do i = 1,len
          if (strts(i)) then
            gde = grav*zb(i)/tke(i)
            earg = -0.1*(tke(i)-tkemin)/tkemin
            bk2tilda = bktwo*(1.-exp(earg))
            dpent(i) = (rob(i)*bkone*sqrt(tke(i)-tkemin) +  &
               bktwo*alphab(i)*gde*(drb(i)/(spec_heat_cp*tb(i))))  &
               /(1.+(bktwo*rib(i)))
          else
            dpent(i) = rob(i)*bkone*sqrt(tke(i)-tkemin)/  &
                       (1.+(bktwo*rib(i)))
          endif
        enddo
      else
        do i = 1,len
          if (strts(i)) then
            gde = grav*zb(i)/tke(i)
            dpent(i) = (rob(i)*bone*sqrt(tke(i)) +  &
               btwo*alphab(i)*gde*(drb(i)/(spec_heat_cp*tb(i))))  &
               /(1.+(btwo*rib(i)))
          else
            dpent(i) = rob(i)*bone*sqrt(tke(i))/(1.+(btwo*rib(i)))
          endif
        enddo
      endif

      end subroutine compe
!======================================================================
! END OF COMPE
!======================================================================

!======================================================================
! BEGINNING OF COMP_TKE_CLASSIC
!======================================================================
      subroutine comp_tke_classic(len,ps,pb,pc,dhb, &
                                  dwb,drb,wm,nentr,speedm, &
                                  shear1,psbloc,tke,epslnb,alphab, &
                                  entran,fws,fhs,fsvs,ros, &
                                  zb,cu,ts,tb,strts, &
                                  rob,dt, &
                                  shear, gprod, cons)

!---------------------------------------------------------------------------
! PURPOSE
!---------------------------------------------------------------------------       
!-- ENTRAINMENT PARAMETERIZATION WITH PROGNOSTIC TKE. 
!-- This is the pre-c221/c228 parameterization of E/TKE.  The details
!-- are actually given in Randall, Sellers and Dazlich (1989):  Applications
!-- of a prognostic turbulence kinetic energy in a bulk boundary-layer 
!-- model.  This was an update to the closure described in Suarez, Arakawa
!-- and Randall (1983).  

!-- safe = delsv - jump in virtual dry static energy across the PBL top

      implicit none

!---------------------------------------------------------------------------
! INTENT IN
!---------------------------------------------------------------------------
      integer (kind=int_kind), intent(in) :: &
         len
      real (kind=dbl_kind), intent(in), dimension(len) ::  &         
         ps, &
         pb, &
         pc, &
         dhb, &
         dwb, &
         drb, &
         wm, &
         speedm, &
         shear1, &
         psbloc, &
         epslnb, &
         alphab, &
         fws, &
         fhs, &
         fsvs, &
         ros, &
         zb, &
         cu, &
         ts, &
         tb, &
         rob
      real (kind=dbl_kind), intent(in) :: &
         dt
      logical (kind=log_kind), dimension(len), intent(in) :: &
         nentr, &
         strts

!---------------------------------------------------------------------------
! INTENT INOUT
!---------------------------------------------------------------------------
      real (kind=dbl_kind), intent(inout), dimension(len) ::  &          
         entran, &
         tke

!---------------------------------------------------------------------------
! INTENT OUT
!---------------------------------------------------------------------------
      real (kind=dbl_kind), intent(out), dimension(len) ::  &         
         shear, &
         gprod, &
         cons

!---------------------------------------------------------------------------
! LOCAL
!---------------------------------------------------------------------------
      real (kind=dbl_kind), parameter :: &
         a1=0.162, &
         a2=0.96
      real (kind=dbl_kind) ::  &      
         zoom, &
         s0flr, &
         ia1, &
         a2fac, &
         athird, &
         hkapa, &
         aquad, &
         bquad, &
         cquad, &
         bsq, &
         enum, &
         edenom, &
         diffx, &
         adiffl, &
         adiffr, &
         a1a2fc
      real (kind=dbl_kind), dimension(len) ::  &       
         cons0, &
         dtilda, &
         enocns, &
         wtb, &
         wts, &
         safe, &
         shear0, &
         rom, &
         saftem, &
         etem, &
         es, &
         netprd, &
         gprod0, &
         tce, &
         gammac, & 
         clrfrc, &
         cldfrc, &
         alphac, &
         betac, &
         epslnc, &
         entl, &
         entr, &
         consl, &
         consr, &
         diffl, &
         diffr, &
         gdtbms, &
         temv

!-- added by mb for cubic tke solver
      real (kind=dbl_kind), dimension(len) :: &
         aapcoeff, &
         invcoeff
      real (kind=dbl_kind) :: &
         a0coeff, &
         a1coeff, &
         a2coeff, &
         root1, &
         sumcoeff, &
         a1fac, &
         a1a2fcinv
      complex (kind=dbl_kind) :: &
         root2, &
         root3
      logical :: &
         threereal
      integer (kind=int_kind), dimension(len) :: &
         itno

      integer (kind=int_kind), parameter :: &
         mlen=1    ! added for call to grossp
      real (kind=dbl_kind), parameter :: &
         critrn=0.1_dbl_kind
      integer (kind=int_kind), parameter :: &
         itnomo=40

      integer (kind=int_kind) :: &
         i
      logical (kind=log_kind) :: &
         done

!---------------------------------------------------------------------------
! DEFINE SOME CONSTANTS
!---------------------------------------------------------------------------
         zoom   = 0.9                                                     
         s0flr  = 8.0                                                     
         ia1    = 1.0/a1                                                  
         a2fac  = (1.0-a2)/a2                                             
         athird = 1.0/3.0                                                 
         hkapa  = 0.5*kappa                                                 

         a1a2fc = 1.0/(a2*a1**1.50)                                      
         a1a2fcinv = a2*a1**1.50   
         a1fac = a1**1.50                                   

      do i = 1,len                                                  
        cons0(i) = 0.0
!-- epslnb and safe are passed in                                       
!        epslnb(i) = tb(i)*cpbl       
        safe  (i) = 1.0-delta*epslnb(i)                                     
        safe  (i) = dhb(i)-safe(i)*hltm*dwb(i)                              
        rom   (i) = grav2*zb(i)
        rom   (i) = psbloc(i)/rom(i)                                   
        dtilda(i) = tke(i)*ia1                                              
      enddo

      do i = 1,len                                                  
        dtilda(i) = a2fac*rom(i)*dtilda(i)**1.5                          
        enocns(i) = dtilda(i)/tke(i)                                        
!---------------------------------------------------------------------------
!-- IMPOSE A LOWER LIMIT ON THE SPEEDM USED TO COMPUTE SHEAR0         
!---------------------------------------------------------------------------
        shear0(i) = speedm(i)*speedm(i)*speedm(i)
        shear0(i) = max(s0flr,shear0(i))
        shear0(i) = shear0(i)*ros(i)*cu(i)*cu(i)                      
 
        itno(i) = 0                                                        
        wtb(i) = hkapa*psbloc(i)/pb(i)                                       
        wts(i) = hkapa*psbloc(i)/ps(i)                                       
        saftem(i) = safe(i) 
	saftem(i) = max(0.0_dbl_kind,saftem(i))                                            
      enddo

      do i = 1,len
        if ((.not.nentr(i)).and.(.not.strts(i))) then
          if (fsvs(i).ge.0.0_dbl_kind) then
 
           aquad = saftem(i)*(tke(i)+wtb(i)*saftem(i))                      
           bquad = tke(i)*fsvs(i)-dtilda(i)*saftem(i)                        
           cquad = -dtilda(i)*fsvs(i)                                        
                                                                       
           if (aquad.ne.c0) then                                           
             bsq       = bquad*bquad                                        
             entran(i) = (-bquad+sqrt(bsq-4.0_dbl_kind*aquad*cquad)) &
                         /(2.0*aquad)     
           else                                                           
             entran(i) = -cquad/bquad                                      
           endif                                                          
                                                                       
           etem(i) = entran(i)                                              
           es  (i) = entran(i)*saftem(i)                                    
           cons(i) = wtb(i)*es(i)*es(i)/(fsvs(i)+es(i))
          else 

           enum = dtilda(i)+wts(i)*fsvs(i)                                  
           edenom = tke(i)                                   
           if (enum.ge.c0) edenom = edenom+wtb(i)*saftem(i)                 
                                                                       
           entran(i) = enum/edenom                                          
           etem  (i) = max(entran(i),0.0_dbl_kind)   
           es    (i) = etem(i)*saftem(i)                                    
           cons  (i) = wtb(i)*es(i)-wts(i)*fsvs(i)                          

          endif

           netprd(i) = wts(i)*fsvs(i)-wtb(i)*es(i)  &                       
                       +shear0(i)+etem(i)*shear1(i)                         
           gprod (i) = netprd(i)+cons(i)                                    
           gprod0(i) = wts(i)*max(fsvs(i),0.0_dbl_kind)+shear0(i)           

        elseif ((nentr(i)).and.(.not.strts(i))) then
           gprod0(i) = shear0(i) + wts(i) * max(fsvs(i),0.0_dbl_kind)     
           gprod (i) = gprod0(i)                                            
           cons0 (i) = wts(i) * max(-fsvs(i),0._dbl_kind)   
           cons  (i) = cons0 (i)                                            
           entran(i) = 0.0                                                  

        elseif (strts(i)) then
          tce   (i) = ts(i)*(pc(i)/ps(i))**kappa
!-- these are passed in                               
!          gammab(i) = gamfac*qsb(i)/(tb(i)*tb(i))                             
!          alphab(i) = (c1+deltap1*gammab(i)*epslnb(i))
!     *             /(c1+gammab(i))         
          epslnc(i) = tce(i)*cpbl                                             
          gammac(i) = gamfac*wm(i)/(tce(i)*tce(i))                            
          alphac(i) = (c1+deltap1*gammac(i)*epslnc(i))  &
                      /(c1+gammac(i))         
          betac (i) = c1-delta*epslnc(i)                                     
                                                                       
          cldfrc(i) = (pc(i)-pb(i))/psbloc(i)                                 
          clrfrc(i) = c1-cldfrc(i)                                           
                                                                       
!          safe  (i) = alphab(i)*dhb(i)-epslnb(i)*hltm*dwb(i)                  
          entran(i) = c0                                                     

        call grossp(mlen,dhb(i),dwb(i),drb(i),ps(i), &
                    pc(i),pb(i),shear1(i),gprod0(i),epslnb(i), &
                    cons0(i),clrfrc(i),cldfrc(i),alphac(i),betac(i), &
                    epslnc(i),fsvs(i),shear0(i),entran(i),alphab(i), &
                    fhs(i),fws(i))  
       
          if (nentr(i)) then
            gprod(i) = gprod0(i)                                             
            cons (i) = cons0 (i) 
          else
            if (cons0(i).gt.dtilda(i)) then
              gprod (i) = gprod0(i)                                            
              cons  (i) = cons0 (i)                                            
              entran(i) = -(cons0(i)-dtilda(i))/tke(i)   
            else
              entran(i) = enocns(i) 

              call grossp(mlen,dhb(i),dwb(i),drb(i),ps(i), &
                          pc(i),pb(i), shear1(i),gprod(i),epslnb(i), &
                          cons(i),clrfrc(i),cldfrc(i),alphac(i),betac(i), &
                          epslnc(i),fsvs(i),shear0(i),entran(i),alphab(i),&
                          fhs(i),fws(i))  

              if (cons(i).ne.0.0_dbl_kind) then
!---------------------------------------------------------------------------
!   PREPARE TO ITERATE.        
!---------------------------------------------------------------------------
                entl (i) = 0.0
                consl(i) = cons0(i)
                diffl(i) = cons0 (i)-dtilda(i)                                   
                entr (i) = enocns(i)                                             
                consr(i) = cons  (i)                                             
                diffr(i) = consr (i)-dtilda(i)+entr(i)*tke(i)                    
!---------------------------------------------------------------------------
!   FIRST GUESS BY INTERPOLATION BETWEEN "R" AND "L"                  
!---------------------------------------------------------------------------
                entran(i)= entl(i)-(entr(i)-entl(i))*diffl(i)  &
                	   /(diffr(i)-diffl(i))  


!-- top of iteration loop
                done = .false.
                do while (.not.done)

              call grossp(mlen,dhb(i),dwb(i),drb(i),ps(i), &
                          pc(i),pb(i),shear1(i),gprod(i),epslnb(i), &
                          cons(i),clrfrc(i),cldfrc(i),alphac(i),betac(i), &
                          epslnc(i),fsvs(i),shear0(i),entran(i),alphab(i), &
                          fhs(i),fws(i))  

                  diffx=cons(i)+entran(i)*tke(i)-dtilda(i)                     
!---------------------------------------------------------------------------
!     WHICH SIDE ARE WE ON?                                             
!---------------------------------------------------------------------------
                  if (diffx < c0) then
                    entl(i) = entran(i)
                    consl(i) = cons(i)
                    diffl(i) = diffx
                  else
                    entr(i) = entran(i)
                    consr(i) = cons(i)
                    diffr(i) = diffx
                  endif
                  itno(i) = itno(i)+1
                  adiffl = abs(diffl(i))
                  adiffr = abs(diffr(i))
                    
                  done=(min(adiffl,adiffr) <= critrn*dtilda(i))  & 
                    .or.itno(i) > itnomo  

                  if (.not.done) then                                      
                   if(adiffl > adiffr)then
                     entran(i) = entl(i)-(entr(i)-entl(i))  &
                                 *zoom*diffl(i)/(diffr(i)-diffl(i))        
                   else
                     entran(i) = entr(i)-(entl(i)-entr(i))  &
                                 *zoom*diffr(i)/(diffl(i)-diffr(i))         
                   endif
                  endif
                enddo
              endif 
            endif                      
          endif
        endif
!---------------------------------------------------------------------------
!     IF GPROD GT GPROD0, REDUCE TO GPROD0                              
!---------------------------------------------------------------------------      
        gprod(i) = min(gprod0(i),gprod(i))

!---------------------------------------------------------------------------
!     LIMIT ABS(ENTRAN) TO ENTMAX                                       
!---------------------------------------------------------------------------
	entran(i) = min(entmax,entran(i))
	entran(i) = max((-entmax),entran(i))

!---------------------------------------------------------------------------
!     PREDICT NEW TKE                                                   
!---------------------------------------------------------------------------
        gdtbms(i) = grav2*dt/psbloc(i)                                      
        shear(i) = shear0(i) + entran(i)*shear1(i)
!        rom   (i) = 100.*psb(i,j)/spanew(i,j)                               
 
        temv(i) = sqrt(tke(i))    
        temv(i) = temv(i)*a1a2fc*rom(i) + (gprod0(i)-gprod(i))/tke(i)
!        tke(i) = (tke(i)+gdtbms(i)*gprod0(i))/(1.0+gdtbms(i)*temv(i))
!        tke(i) = max(tkemin,tke(i))

!----------------------------------------------------------------------------
! test for now:
! form cubic equation for sqrt(tke) by making the dissipation term
! fully implicit.  then use cubic solver as done for "old" scheme.
! the coefficients for the solver are almost the same as before.
        invcoeff(i) = c1/GDTBMS(i)
        aapcoeff(i) = a1a2fcinv/rom(i)
!        a2coeff = aapcoeff(i)*(invcoeff(i)+
!     +             (gprod0(i)-gprod(i))/tke(i))
!        a1coeff = 0.0
!        a0coeff = -aapcoeff(i)*(tke(i)*invcoeff(i)+gprod0(i))

!-- use these 
        a2coeff = (invcoeff(i)+entran(i))*a1fac/rom(i)
        a1coeff = 0.0
        a0coeff = tke(i)*invcoeff(i) + gprod(i)+cons(i)
        a0coeff = -a0coeff*a1fac/rom(i)
        
        call cpl_cubic(a0coeff, a1coeff, a2coeff, root1, root2, &
                   root3,sumcoeff, threereal)
        tke(i) = max(0.0_dbl_kind, ROOT1)**2
        tke(i) = max(tkemin,tke(i))
!----------------------------------------------------------------------------

      enddo

      end subroutine comp_tke_classic
!======================================================================
! END OF COMP_TKE_CLASSIC
!======================================================================


!======================================================================
! BEGINNING OF GROSSP
!======================================================================
      subroutine grossp(mlen,dhb,dwb,drb,ps, &
                        pc,pb,shear1,gprod,epslnb, &
                        cons,clrfrc,cldfrc,alphac,betac, &
                        epslnc,fsvs,shear0,entran,alphab, &
                        fhs,fws)   

!---------------------------------------------------------------------------
! PURPOSE
!---------------------------------------------------------------------------       
!--  FOR GIVEN ENTRAN, FIND THE GROSS PRODUCTION RATE.                 
!--  THIS ROUTINE IS USED FOR THE STRATUS CASE ONLY.                   
                                                                       
      implicit none

!---------------------------------------------------------------------------
! INTENT IN
!---------------------------------------------------------------------------
      integer (kind=int_kind), intent(in) :: &
         mlen
      real (kind=dbl_kind), dimension(mlen), intent(in) ::  & 
         dhb, &
         dwb, &
         drb, &
         ps, &
         pc, &
         pb, &
         shear1, &      
         epslnb, &
         clrfrc, &
         cldfrc, &
         alphac, &
         betac, &
         fsvs, &
         entran, &
         alphab, &
         fhs, &
         fws, &
         epslnc, &
         shear0

!---------------------------------------------------------------------------
! INTENT OUT
!---------------------------------------------------------------------------
      real (kind=dbl_kind), dimension(mlen), intent(out) ::   &    
         gprod, &
         cons

!---------------------------------------------------------------------------
! LOCAL
!---------------------------------------------------------------------------
      real (kind=dbl_kind), dimension(mlen) ::   &      
         fsvb, &
         fsvcm, &
         fsvcp, &
         wscs, &
         wscc, &      
         wcbc, &
         wcbb, &
         posboy, &
         bync

      real (kind=dbl_kind) :: &
         fhb, &
         hltfwb, &
         fhc, &
         hltfwc, &
         zeta0
      integer (kind=int_kind) :: &
         i

!---------------------------------------------------------------------------
! DEFINE SOME CONSTANTS
!---------------------------------------------------------------------------
                  
      do i = 1,mlen
         wscs(i) = kb2*(ps(i)-pc(i))/ps(i)                                
         wscc(i) = kb2*(ps(i)-pc(i))/pc(i)                                
         wcbc(i) = kb2*(pc(i)-pb(i))/pc(i)                                
         wcbb(i) = kb2*(pc(i)-pb(i))/pb(i)
                                
!---------------------------------------------------------------------------
!     BUOYANCY FLUXES ABOVE AND BELOW CLOUD BASE                        
!---------------------------------------------------------------------------
        fhb    = -entran(i)     *dhb(i)+drb(i)                              
        hltfwb = -entran(i)*hltm*dwb(i)                                     
        fhc    = fhb   *clrfrc(i)+fhs(i)*cldfrc(i)                          
        hltfwc = hltfwb*clrfrc(i)+fws(i)*cldfrc(i)                          
                                                                       
        fsvcm(i) =           fhc- betac(i)*hltfwc                           
        fsvcp(i) = alphac(i)*fhc-epslnc(i)*hltfwc                           
        fsvb (i) = alphab(i)*fhb-epslnb(i)*hltfwb 
                          
!---------------------------------------------------------------------------
!     NET BUOYANT PRODUCTION                                            
!---------------------------------------------------------------------------
        bync(i) = wscs(i)*fsvs (i)+wscc(i)*fsvcm(i)  &                       
                  +wcbc(i)*fsvcp(i)+wcbb(i)*fsvb(i)        
                 
!---------------------------------------------------------------------------
!     BUOYANT CONSUMPTION, STARTING WITH THE SUBCLOUD LAYER             
!---------------------------------------------------------------------------
        cons(i)=0.0                                                       
                                                                       
        if (fsvs(i) < c0.or.fsvcm(i) < c0) then                       
          if (fsvs(i) <= c0 .and. fsvcm(i) <= c0) then                   
            cons(i) = -wscs(i)*fsvs(i)-wscc(i)*fsvcm(i)                   
          else                                                           
            zeta0 = fsvs(i)/(fsvs(i)-fsvcm(i))                            
                                                                       
            if (fsvs(i) < c0) then                                    
               cons(i) = -wscs(i)*     zeta0 *fsvs (i)                    
            else                                                        
               cons(i) = -wscc(i)*(c1-zeta0)*fsvcm(i)                    
            endif                                                       
          endif                                                          
        endif   
                                                          
!---------------------------------------------------------------------------     !     NOW, THE CLOUD LAYER                                              
!---------------------------------------------------------------------------

        if (fsvcp(i) < c0 .or. fsvb(i) < c0) then                       
          if (fsvcp(i) <= c0 .and. fsvb(i) <= c0) then                   
            cons(i) = cons(i)-wcbc(i)*fsvcp(i)-wcbb(i)*fsvb(i)            
          else                                                           
            zeta0 = fsvcp(i)/(fsvcp(i)-fsvb(i))                           
                                                                       
            if (fsvcp(i) < c0) then                                    
               cons(i) = cons(i)-wcbc(i)*     zeta0 *fsvcp(i)             
            else                                                        
               cons(i) = cons(i)-wcbb(i)*(1.0-zeta0)*fsvb (i)             
            endif                                                       
          endif                                                          
        endif                                                             

!---------------------------------------------------------------------------
!     CONS IS POSITIVE, SO POSBOY IS GREATER THAN BYNC.                 
!---------------------------------------------------------------------------
        posboy(i) = bync(i)+cons(i)                                         
        gprod (i) = posboy(i)+shear0(i)+entran(i)*shear1(i) 

      enddo                

      end subroutine grossp
!======================================================================
! END OF GROSSP
!======================================================================
!======================================================================
! BEGINNING OF COMP_TKE
!======================================================================
      subroutine comp_tke(len,ps,pb,pc,dhb, &
                          dwb,drb,wm,nentr,speedm, &
                          shear1,psbloc,tke,epslnb,alphab, &
                          entran,enodrb,eold,fws,fhs, &
                          fsvs,ros,zb,cu,ts, &
                          tb,strts,rob,rib,diss, &
                          fss,safe,sumcoeff, &
                          dt, negprod, bync, shear )

!---------------------------------------------------------------------------
! PURPOSE
!---------------------------------------------------------------------------       
!  ENTRAINMENT PARAMETERIZATION WITH PROGNOSTIC TKE.                 
!     Computes the PBL top entrainment rate (E) diagnostically
!     and prognoses the PBL turbulent kinetic energy (TKE).

!     REFERENCES: Krasner, R.D. (1993) "Further Development and Testing
!        of a Second-Order Bulk Boundary Layer Model",  Masters Thesis
!        Colorado State University, Dept. of Atmospheric Science.

!     MODIFICATIONS:
!        - originally added as subroutine newentrain back in 199?
!        - add comments, modified argument list -- M. Branson, Fall 1998
!        - pass in gammab variable, remove calc of qsb and gammab

!     SUBROUTINES CALLED:
!               CUBIC

!        mlcom variables used (passed through argument):
!               grav - gravitational acceleration constant (m/s^2)
!               grav2 - grav/100.
!               kapa = R/cp = 0.286
!               cp - specific heat capacity
!               dt - time step
!               hltm - latent heat of vapourisation
!               delta - 0.608
!               gamfac - C*(L/Cp)  (C=5417.98)

!     ARGUMENT LIST VARIABLES
!        Input variables:
!               PS - surface pressure
!               PB - PBL top pressure
!               PC - cloud base pressure (computed in subroutine cloud) 
!               DHB - moist static energy jump across the PBL top
!               DWB - total water mixing ratio jump across the PBL top
!               DRB - longwave radiative flux jump across the PBL top
!               WM - PBL water vapor mixing ratio
!               NENTR - Logical to indicate where both the PBL and the layer
!                 above are saturated.  In this case, E is set to zero.
!               SPEEDM - PBL wind speed magnitude
!               SHEAR1 - PBL top shear
!               PSB - PBL pressure depth (ps-pb)
!               TKE - PBL turbulent kinetic energy (m^2/s^2)
!               ENTRAN - PBL top entrainment rate
!               FWS - water vapor mixing ratio flux at the surface
!               FHS - moist static energy flux at the surface
!               FSVS - virtual dry static energy flux at the surface
!               ROS - surface density
!               ZB - PBL top height (m)
!               CU - surface momentum transfer coefficient
!               TS - surface temperature
!               TB - PBL top temperature
!               STRTS - logical to indicate if stratus cloud is present
!                 in the PBL
!               ROB - PBL top density
!               RIB - bulk Richardson number
!               DISS - TKE dissipation rate
!               FSS - surface sensible heat flux
!               TKEMIN - minimum allowed value for TKE
!               C - TKE constant
!               LEN - number of grid points in current chunk

!        Output variables:
!               SAFE - layer cloud instability criteria value

!     LOCAL VARIABLES:
!               krash - controls partioning of E/tke in neg production mode
!               krash2 - multiplies dissipation in test of buoyancy and
!                 and shear sums vs D to determine positive/negative E mode
!               bone - entrainment constant -- dependent on value of c 
!               btwo - entrainment constant -- dependent on value of c

!-- modified version of comp_tke to use the UCLA scheme when konor logical
!-- is set to .true.
!-- main difference is how negative entrainment handled.  
!--  1) condition is when tke < tkemin.  
!--  2) when this occurs, the "detrainment" rate is given by eq. 8.14
!--     in the konor paper.  

      use kinds

      implicit none

!---------------------------------------------------------------------------
! INTENT IN
!---------------------------------------------------------------------------
      integer (kind=int_kind), intent(in) :: &
         len
      real (kind=dbl_kind), intent(in), dimension(len) :: &   
         ps, &
         pb, &
         pc, &
         dhb, &
         dwb, &
         drb, &
         wm, &
         speedm, &
         shear1, &
         psbloc, &
         epslnb, &
         alphab, &
         fws, &
         fhs, &
         fsvs, &
         ros, &
         zb, &
         cu, &
         ts, &
         tb, &
         rob, &
         rib, &
         fss, &
         safe
      real (kind=dbl_kind), intent(in) :: &
         dt
      real (kind=dbl_kind), intent(out), dimension(len) ::  &      
         tke, &
         sumcoeff, &
         bync, &
         shear
      real (kind=dbl_kind), intent(inout), dimension(len) ::  &        
         entran, &
         diss
      logical (kind=log_kind), dimension(len), intent(in) ::  &
         nentr, &
         strts
      logical (kind=log_kind), dimension(len), intent(out) :: &
         negprod

!-- local variables
      real (kind=dbl_kind) ::       &     
         krash, &
         krash2, &
         bone, &
         btwo, &
         s0flr, &
         gde, &
         a0coeff, &
         a1coeff, &
         a2coeff, &
         root1, &
         kpcinv
      real (kind=dbl_kind), dimension(len) ::  &        
         buoy0, &
         coeff, &
         invcoeff, &
         enodrb, &
         eold, &
         shear0, &
         rom
      complex (kind=dbl_kind) :: &
         root2, &
         root3
      logical (kind=log_kind) :: &
         threereal
      integer (kind=int_kind) :: &          
         i          
      real (kind=dbl_kind) :: &
         psbmax=250., &
         taucollapse=10800.      

!-- added by mb:  assign values to constants krash and krash2
      data krash  / 0.9 /
      data krash2 / 0.9 /

      bone = 0.252 * (c**(1./3.))
      btwo = 0.504 * (c**(-2./3.))
!
!     DEFINE SOME CONSTANTS
!
      S0FLR  = 8.0                                                
!
      do i = 1,len                                                  
        rom   (i) = grav2*zb(i)
        rom   (i) = psbloc(i)/rom(i)                                   

!                                                                       
!     IMPOSE A LOWER LIMIT ON THE SPEEDM USED TO COMPUTE SHEAR0         
!                                                                       
        shear0(i) = speedm(i)*speedm(i)*speedm(i)
        shear0(i) = max(s0flr,shear0(i))
        shear0(i) = shear0(i)*ros(i)*cu(i)*cu(i)                      

        coeff(i) = grav2*dt/psbloc(i)                                

        invcoeff(i) = 1.0/coeff(i)                                

      enddo

!-- compute buoyancy and shear terms (total and surface), and for non-stratus
!-- points compare B+S to kD to determine negative or positive entrainment mode
      call BUOYSHEAR(len, negprod, shear, shear0, strts,     &
                           epslnb, entran, dhb, drb, dwb, bync,    &
                           psbloc, buoy0, fsvs, ps, pb, diss, pc,  &
                           ts, wm, fhs, fws, alphab, krash2)

      do i = 1,len
        if ((konor).and.(tke(i) < tkemin).and.(.not.(nentr(i)))) then
!        if ((tke(i) < tkemin).and.(.not.(nentr(i)))) then

          tke(i) = tkemin
          diss(i) = rom(i)*tke(i)*sqrt(tke(i))*c
          entran(i) = -psbmax/(grav*taucollapse)

          eold(i) = entran(i)
          enodrb(i) = entran(i)

        elseif ((.not.(konor)).and.(negprod(i)).and.  &
         (.not.(nentr(i)))) then
          call NEGETKE(i, len, rom, krash, kpcinv, a0coeff, a1coeff,   &
                         a2coeff, invcoeff, buoy0, shear0, tke,            &
                         root1, root2, root3, sumcoeff, threereal)

          tke(i) = max(0.0_dbl_kind, ROOT1)**2
          tke(i) = max(tkemin,tke(i))
          diss(i) = rom(i)*tke(i)*sqrt(tke(i))*c
          entran(i) = (1.-krash)*(buoy0(i)+shear0(i)-diss(i))/tke(i)

          eold(i) = entran(i)
          enodrb(i) = entran(i)

        else
          if (nentr(i)) then         ! all points with E=0
            entran(i) = 0.0
            eold(i) = entran(i)
            enodrb(i) = entran(i)
            shear(i) = shear0(i)
            if (.not. strts(i)) then
              bync(i) = buoy0(i)   ! can't do this for all points
            endif
          else
            if (strts(i)) then         ! stratus points
              gde = grav*zb(i)/tke(i)

!-- ** for no LCI test only: set safe = 0 if it is negative
!              if (safe(i).lt.0.) safe(i) = 0.

!-- safeguard
!              if (safe(i).lt.0.) then
!                write(*,*) 'SAFE < 0 and strts = T?!?',i,safe(i)/cp
!                read(*,*)
!              endif

              call compE(1,bone,btwo,rob(i),tke(i),  &
                         rib(i),zb(i),alphab(i),drb(i),tb(i),  &
                         strts(i),entran(i))

              enodrb(i) = (rob(i)*bone*sqrt(tke(i)))  &
                           /(1.+(btwo*rib(i)))

!-- ** no DRB formula for entran
!              entran(i) = (rob(i)*bone*sqrt(tke(i)))  &
!                          /(1.+(btwo*rib(i)))
            else                       ! clear sky points
              call compE(1,bone,btwo,rob(i),tke(i),  &
                         rib(i),zb(i),alphab(i),drb(i),tb(i), &
                         strts(i),entran(i))
              enodrb(i) = rob(i)*bone*sqrt(tke(i))/(1.+(btwo*rib(i)))
            endif
            eold(i) = rob(i)*bone*sqrt(tke(i))/(1.+(btwo*rib(i)))
          endif
          a2coeff = invcoeff(i)+entran(i)
          a2coeff = a2coeff/(c*rom(i))
          a1coeff = 0.0
          a0coeff = tke(i)*invcoeff(i)
          a0coeff = a0coeff+bync(i)+shear(i)
          a0coeff = -a0coeff/(c*rom(i))

          call cpl_cubic(a0coeff, a1coeff, a2coeff, root1, root2, &
                     root3,sumcoeff(1), threereal)

!-- new 2/99: if root1 < 0,
          if (root1 < 0.) then
             if (strts(i)) buoy0(i) = bync(i)
             call NEGETKE(i, len, rom, krash, kpcinv, a0coeff, a1coeff,   &
                         a2coeff, invcoeff, buoy0, shear0, tke,            &
                         root1, root2, root3, sumcoeff, threereal)
          endif

          tke(i) = max(0.0_dbl_kind, ROOT1)**2
          tke(i) = max(tkemin,tke(i))
          diss(i) = rom(i)*tke(i)*sqrt(tke(i))*c

        endif

        entran(i) = min(entmax,entran(i))
        entran(i) = max((-entmax),entran(i))
        enodrb(i) = min(entmax,enodrb(i))
        enodrb(i) = max((-entmax),enodrb(i))
        eold(i) = min(entmax,eold(i))
        eold(i) = max((-entmax),eold(i))
      enddo


      end subroutine comp_tke
!======================================================================
! END OF COMP_TKE
!======================================================================


      end module comp_tke_module
