c------------------------------------------------------------------------------
      subroutine elqclop(dpl,psb,tl,cw0,cw1,ci0,ci1,ri0,ri1,ctot0,ctot1
!     *,             dopbl,pblcw,pblci,cld,clu,tice,grav,emisrnd,emismax
     *,             dopbl,lwp,cld,clu,tice,grav,emisrnd,emismax
     *,             codrnd,codmax,omega,asmfct,nlen,nlm,npbl,lpbl,nls
     *,             len,nocld,pcb)

      use kinds

      implicit none
c------------------------------------------------------------------------------
c     computes the cloud optical depth and cloud infrared emissivity as a 
c     function of the cloud liquid and ice water paths for use in the short-
c     wave (swrad) and longwave (lwrad) radiation codes. The single scat-
c     tering albedo and asymmetry factor are prescribed.

c     input variables:
c     ----------------
c     dpl : pressure layer-thickness         (mb).
c     tl  : layer temperature                 (K).
c     qcw  : cloud water content           (kg/kg).
c     qci  : cloud ice water content       (kg/kg).
c     cld : large-scale saturation cloud fraction.
c     tice: freezing temperature of water     (K).
c     grav: acceleration due to gravity  (m/s**2).
c     pcb : boundary layer cloud depth (mb)

c     output variables:
c     -----------------
c     emisrnd: infrared emissivity of large-scale condensation
c              clouds (random overlap).
c     emismax: infrared emissivity of anvil clouds (maximum overlap).
c     codrnd : optical thickness of large-scale condensation clouds 
c              (random overlap).
c     codmax : optical thickness of anvil clouds (maximum overlap)  .
c     omega  : single scattering albedo.
c     asmfct : asymmetry factor.

c     Laura D. Fowler/ stratus (03-24-93)

c------------------------------------------------------------------------------

      integer (kind=int_kind) ::
     *     nlen, nlm, npbl, lpbl, nls, len
      logical (kind=log_kind) ::
     *     dopbl,bitc(nlen), nocld

      REAL (kind=dbl_kind) ::
     *     tice
      REAL (kind=dbl_kind) ::
!     *     psb(nlen) ,pblcw(len),pblci(len), pcb(len)
     *     psb(nlen) , lwp(len,npbl), pcb(len)
     *,    temc(len),tv1(len)  ,tv2(len)
      REAL (kind=dbl_kind) ::
     *     dpl(nlen,nlm),tl(nlen,nlm)   ,cld(nlen,nlm)
     *,    clu(nlen,nlm),cw0(nlen,nlm)  ,cw1(nlen,nlm)
     *,    ci0(nlen,nlm),ci1(nlen,nlm)  ,ri0(nlen,nlm)
     *,    ri1(nlen,nlm),ctot0(nlen,nlm),ctot1(nlen,nlm)
      REAL (kind=dbl_kind) ::
     *     wcw(len,nlm+1),wci(len,nlm+1)
     *,    wri(len,nlm+1)   
      REAL (kind=dbl_kind) ::
     *     codrnd(nlen,nlm+1)  ,codmax(nlen,nlm+1)
     *,    codrndw(nlen,nlm+1) ,codrndi(nlen,nlm+1)
     *,    codrndri(nlen,nlm+1),codmaxw(nlen,nlm+1)
     *,    codmaxi(nlen,nlm+1) ,codmaxri(nlen,nlm+1)
     *,    omega(nlen,nlm+1)   ,asmfct(nlen,nlm+1)
      REAL (kind=dbl_kind) ::
     *     emisrnd(nlen,nlm+2)   ,emismax(nlen,nlm+2)
      integer (kind=int_kind) ::
     *     nlsp1, nlmp1, nnp, nlsp2, i, l, k, kk
      REAL (kind=dbl_kind) ::
     *    
     *   ecritl, tcrit, cod3, cod2, cod1, grav, ecrits, reri, om0ri
     *,  gri, gci, om0ci, reci, gcw, om0cw, recw, abslw
      
      data abslw         /0.75/
      data recw,om0cw,gcw/10.  ,0.99,0.85/
      data reci,om0ci,gci/30.  ,0.98,0.75/
      data reri,om0ri,gri/1000.,0.98,0.75/

      nnp = nlm+2
      nlmp1 = nlm+1
      nlsp1 = nls+1
      nlsp2 = nlsp1+1

c---- 0. Initialization:
      do l = 1, nlmp1
         do i = 1, len
            codrnd(i,l)   = 0.
            codrndw(i,l)  = 0.
            codrndi(i,l)  = 0.
            codrndri(i,l) = 0.
            codmax(i,l)   = 0.
            codmaxw(i,l)  = 0.
            codmaxi(i,l)  = 0.
            codmaxri(i,l) = 0.
            omega(i,l)    = 0.
            asmfct(i,l)   = 0.
            wcw(i,l)      = 0.
            wci(i,l)      = 0.
            wri(i,l)      = 0.
         enddo
      enddo
      do l = 1, nnp
         do i = 1, len
            emisrnd(i,l)  = 0.
            emismax(i,l)  = 0.
         enddo
      enddo
      
      if(nocld) return

c---- 1. cloud optical properties in the free troposphere:
!      do l = nlsp1, nlm
      do l = nlsp1, lpbl
         do i = 1, len

c---- 1.1 large-scale saturation clouds:
            if(cld(i,l).ne.0. .and. ctot1(i,l).ne.0.) then
!               if (l.eq.nlm) write(*,*) 'elqclop: i = ',i
               wcw(i,l+1)      = cw1(i,l)*dpl(i,l)*1.e02/grav
               wcw(i,l+1)      = wcw(i,l+1)*1.e03
               codrndw(i,l+1)  = 1.5*wcw(i,l+1)/recw

               wci(i,l+1)      = ci1(i,l)*dpl(i,l)*1.e02/grav
               wci(i,l+1)      = wci(i,l+1)*1.e03
               codrndi(i,l+1)  = 1.5*wci(i,l+1)/reci

               wri(i,l+1)      = ri1(i,l)*dpl(i,l)*1.e02/grav
               wri(i,l+1)      = wri(i,l+1)*1.e03
               codrndri(i,l+1) = 1.5*wri(i,l+1)/reri

               codrnd(i,l+1)   = codrndw(i,l+1)+codrndi(i,l+1)
     +                          +codrndri(i,l+1)
               emisrnd(i,l+1)  = 1.-exp(-abslw*codrnd(i,l+1))

               cod1 = om0cw*codrndw(i,l+1)
               cod2 = om0ci*codrndi(i,l+1)
               cod3 = om0ri*codrndri(i,l+1)
               omega(i,l+1)    = (cod1+cod2+cod3)/codrnd(i,l+1) 
               asmfct(i,l+1)   = (gcw*cod1+gci*cod2+gri*cod3) 
     +                          /(cod1+cod2+cod3)
            endif
         enddo
      enddo

c---- 1.2 convective clouds:
      do l = nlsp1, nlm
         do i = 1, len
            if(clu(i,l).ne.0. .and. ctot0(i,l).ne.0.) then
               wcw(i,l+1)      = cw0(i,l)*dpl(i,l)*1.e02/grav
               wcw(i,l+1)      = wcw(i,l+1) *1.e03
               codmaxw(i,l+1)  = 1.5*wcw(i,l+1)/recw

               wci(i,l+1)      = ci0(i,l)*dpl(i,l)*1.e02/grav
               wci(i,l+1)      = wci(i,l+1) *1.e03
               codmaxi(i,l+1)  = 1.5*wci(i,l+1)/reci

               wri(i,l+1)      = ri0(i,l)*dpl(i,l)*1.e02/grav
               wri(i,l+1)      = wri(i,l+1)*1.e03
               codmaxri(i,l+1) = 1.5*wri(i,l+1)/reri

               codmax(i,l+1)   = codmaxw(i,l+1)+codmaxi(i,l+1)
     +                          +codmaxri(i,l+1)
               emismax(i,l+1)  = 1.-exp(-abslw*codmax(i,l+1))

               cod1 = om0cw*codmaxw(i,l+1)
               cod2 = om0ci*codmaxi(i,l+1)
               cod3 = om0ri*codmaxri(i,l+1)
               omega(i,l+1)    = (cod1+cod2+cod3)/codmax(i,l+1) 
               asmfct(i,l+1)   = (gcw*cod1+gci*cod2+gri*cod3)
     +                          /(cod1+cod2+cod3)
            endif
         enddo
      enddo

c---- 2. cloud optical properties of PBL clouds (as in clop):
      if(dopbl) then
        kk = 1
        do k = lpbl+1,nlm
         do i = 1, len
!            if(pblcw(i,kk)+pblci(i,kk).le.0.) then
            if(lwp(i,kk).le.0.) then
               emisrnd(i,k+1) = 0.
               codrnd(i,k+1)  = 0.
               omega(i,k+1)   = 0.
               asmfct(i,k+1)  = 0.
               cld(i,k) = 0.0
            else
c              cld(i,k) = 1.0
c the above statement is commented since cld is already equal to 
c   cldpc which can be a fraction.
               if (tl(i,k).ge.tice) then
                 wcw(i,k+1)      = lwp(i,kk)*1.e03
                 codrndw(i,k+1)  = 1.5*wcw(i,k+1)/recw
               else
                 wci(i,k+1)      = lwp(i,kk)*1.e03
                 codrndi(i,k+1)  = 1.5*wci(i,k+1)/reci
               endif

               codrnd(i,k+1)   = codrndw(i,k+1)+codrndi(i,k+1)
               emisrnd(i,k+1)  = 1.-exp(-abslw*codrnd(i,k+1))

               cod1 = om0cw*codrndw(i,k+1)
               cod2 = om0ci*codrndi(i,k+1)
               omega(i,k+1)    = (cod1+cod2)/codrnd(i,k+1) 
               asmfct(i,k+1)   = (gcw*cod1+gci*cod2)/(cod1+cod2)
 
!               write(*,*) i,k+1,codrnd(i,k+1)
!               read(*,*)
            endif
         enddo
         kk = kk+1
        enddo
      else
         tcrit = tice - 82.5
         ecrits = 0.0105125
         ECRITl = 0.007884375
         do i = 1, len
            if(cld(i,nlm).lt.1.e-6_dbl_kind) then
               emisrnd(i,nlmp1) = 0.
               codrnd(i,nlmp1)  = 0.
               omega(i,nlmp1)   = 0.
               asmfct(i,nlmp1)  = 0.
            else
               temc(i) = max(0.0_dbl_kind,(tl(i,nlm)-tcrit))
               temc(i) = .000002*temc(i)*temc(i)
               temc(i) = min(ecrits,temc(i))
               codrnd(i,nlmp1) = .00694875*(tl(i,nlm)-tice) + 0.08
               codrnd(i,nlmp1) = max(ecrits,codrnd(i,nlmp1))
               codrnd(i,nlmp1) = min(0.08_dbl_kind,codrnd(i,nlmp1))
               bitc(i) = codrnd(i,nlmp1).eq.ecrits
               if(bitc(i)) codrnd(i,nlmp1) = temc(i)
               codrnd(i,nlmp1) = codrnd(i,nlmp1) * pcb(I) * cld(i,nlm)
               omega(i,nlmp1)  = 0.99
               asmfct(i,nlmp1) = 0.85

               tv2(i)  = tl(i,nlm)-tcrit
               tv2(i)  = max(0.0_dbl_kind,tv2(i))
               tv1(i)  = 1.5e-06*tv2(i)*tv2(i)
               tv1(i)  = min(ecritl,tv1(i))
               tv2(i)  = tl(i,nlm)-tice
               tv2(i)  = 5.2115625e-03*tv2(i) + 0.06
               tv2(i)  = max(ecritl,tv2(i))
               tv2(i)  = min(0.06_dbl_kind,tv2(i))
               bitc(i) = tv1(i).eq.ecritl
               if(bitc(i)) tv1(i) = tv2(i)
               emisrnd(i,nlmp1) = 1.-exp(-tv1(i)*pcb(i)*cld(i,nlm)) 
            endif
         enddo
      endif

      return
      end


