!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
	
 	module fluxcoupler_fields

!-----------------------------------------------------------------
!	this module specifies fields that the flux coupler will 
!          interpolate to the surface grid, or average from the 
!          surface grid. All fields here are on the atmospheric grid.
!-----------------------------------------------------------------

	use kinds
        use domain_decomposition
        use sigma_coordinate
	use physical_parameters

 	implicit none
 	save

!  fields to be interpolated from the atmospheric grid to the surface grid
	real (kind=dbl_kind), dimension(iim,jjm,nsdm) ::
     *     radvdc_atm,     radndc_atm,     radvbc_atm,     radnbc_atm
     *,    dlwbot_atm,     fluxd_atm,      dcupr_atm,      dlspr_atm
     *,    dlsps_atm,      spdm_atm,       shear1_atm,     ps_atm
     *,    psb_atm,        tpgvar_atm

	real (kind=dbl_kind), dimension(iim,jjm,npbl+2,nsdm) ::
     *     t_atm, sh_atm

!  fields to be averaged from the surface grid to the atmospheric grid

	real (kind=dbl_kind), dimension(iim,jjm,nsdm) ::
     *     alvdf_atm,      alndf_atm,      alvdr_atm,      alndr_atm
     *,    tgeff4_atm,     pc_atm,         cldpc_atm      

	real (kind=dbl_kind), dimension(iim,jjm,npbl,nsdm) ::
     *     lwp_atm, pblcw_atm, pblci_atm

	real (kind=dbl_kind), dimension(iim,jjm,nsdm) ::
     *     fws_atm, fss_atm, dpent_atm, co2flx_atm

	real (kind=dbl_kind), dimension(iim,jjm,2,nsdm) ::
     *     dqlci_atm, dtlci_atm, drag_atm

!-------------------------------------------------------------------
!   VARIABLE DEFINITION
!-------------------------------------------------------------------
!   alxyy_atm = surface albedo (two bands(x) - v=visible n=near ir,
!               two beams(yy) - df=diffuse dr-direct)
!   tgeff4_atm = effective radiative surface temperature to the fourth power
!                    (K^4)
!   pc_atm = pbl cloud base pressure (mb)
!   cldpc_atm = pbl cloud fraction
!   lwp_atm - pbl cloud liquid water path
!   pblcw_atm = pbl cloud liquid water mixing ratio (kg/kg)
!   pblci_atm = pbl cloud ice mixing ratio (kg/kg)
!   radxyc_atm = incident surface solar radiation (two bands(x) - 
!              v=visible n=near ir, two beams(y) - d=diffuse b-direct) (W/m^2)
!   dlwbot_atm = incident surface longwave radiation (W/m^2)
!   fluxd_atm = downward longwave flux at pbl top (W/m^2)
!   dcupr_atm = convective precipitation (kg/m^2/s)
!   dlspr_atm = large scale stratiform rain precipitation (kg/m^2/s)
!   dlsps_atm = large scale stratiform snow precipitation (kg/m^2/s)
!   spdm_atm = pbl wind speed (m/s)
!   shear1_atm = pbl top wind shear (m^2/s^2)
!   ps_atm = surface pressure (mb) if atmos and sfc grids are same;
!                                  else sealevel pressure
!   psb_atm = pbl depth (mb)
!   t_atm = lowest levels potential temperature (K)
!   sh_atm = lowest levels water vapor mixing ratio (kg/kg)
!   fws_atm = surface evaporation (kg/m^2/s)
!   fss_atm = surface sensible heat flux (W/m^2)
!   dpent_atm = pbl top entrainment rate (kg/m^2/s)
!   dqlci_atm = LCI water vapor flux into layers above and below pbl top
!              (kg/m^2/s) 
!   dtlci_atm = LCI heat flux  into layers above and below pbl top (W/m^2)
!   drag_atm = surface drag coefficient for actual and oceanic roughness lengths
!   co2flx_atm = surface flux of co2 from land 
!               
!
!-------------------------------------------------------------------

 	end module fluxcoupler_fields

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
