!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
	
 	module fluxcoupler_fields_sfc

!-----------------------------------------------------------------
!	this module specifies fields that the flux coupler will 
!          interpolate to the surface grid, or average from the 
!          surface grid. All fields here are on the surface grid.
!-----------------------------------------------------------------

	use kinds
        use domain_decomposition_sfc
        use sigma_coordinate
	use physical_parameters

 	implicit none
 	save

!  fields to be interpolated from the atmospheric grid to the surface grid
	real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *     radvdc_sfc,     radndc_sfc,     radvbc_sfc,     radnbc_sfc
     *,    dlwbot_sfc,     fluxd_sfc,      dcupr_sfc,      dlspr_sfc
     *,    dlsps_sfc,      spdm_sfc,       shear1_sfc,     ps_sfc
     *,    psb_sfc,        tpgvar_sfc

	real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,npbl+2,nsdm_sfc) ::
     *     t_sfc, sh_sfc

!  global surface grid descriptions (lat and lon in degrees and zs in gpm)
	real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *	   lon_sfc, lat_sfc, zs_sfc

!  fields to be averaged from the surface grid to the atmospheric grid

	real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *     alvdf_sfc,      alndf_sfc,      alvdr_sfc,      alndr_sfc
     *,    tgeff4_sfc,     pc_sfc,         cldpc_sfc      

	real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,npbl,nsdm_sfc) ::
     *     lwp_sfc, pblcw_sfc, pblci_sfc

	real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *     fws_sfc, fss_sfc, dpent_sfc, co2flx_sfc, tke_sfc

	real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,2,nsdm_sfc) ::
     *     dqlci_sfc, dtlci_sfc, drag_sfc


!-------------------------------------------------------------------
!   VARIABLE DEFINITION
!-------------------------------------------------------------------
!   alxyy_sfc = surface albedo (two bands(x) - v=visible n=near ir,
!               two beams(yy) - df=diffuse dr-direct)
!   tgeff4_sfc = effective radiative surface temperature to the fourth power
!                    (K^4)
!   pc_sfc = pbl cloud base pressure (mb)
!   cldpc_sfc = pbl cloud fraction
!   lwp_sfc - pbl cloud liquid water path
!   pblcw_sfc = pbl cloud liquid water mixing ratio (kg/kg)
!   pblci_sfc = pbl cloud ice mixing ratio (kg/kg)
!   radxyc_sfc = incident surface solar radiation (two bands(x) - 
!              v=visible n=near ir, two beams(y) - d=diffuse b-direct) (W/m^2)
!   dlwbot_sfc = incident surface longwave radiation (W/m^2)
!   fluxd_sfc = downward longwave flux at pbl top (W/m^2)
!   dcupr_sfc = convective precipitation (kg/m^2/s)
!   dlspr_sfc = large scale stratiform rain precipitation (kg/m^2/s)
!   dlsps_sfc = large scale stratiform snow precipitation (kg/m^2/s)
!   spdm_sfc = pbl wind speed (m/s)
!   shear1_sfc = pbl top wind shear (m^2/s^2)
!   ps_sfc = surface pressure (mb) if atmos and sfc grids are same;
!                                  else sealevel pressure
!   psb_sfc = pbl depth (mb)
!   t_sfc = lowest levels potential temperature (K)
!   sh_sfc = lowest levels water vapor mixing ratio (kg/kg)
!   fws_sfc = surface evaporation (kg/m^2/s)
!   fss_sfc = surface sensible heat flux (W/m^2)
!   dpent_sfc = pbl top entrainment rate (kg/m^2/s)
!   dqlci_sfc = LCI water vapor flux into layers above and below pbl top
!              (kg/m^2/s) 
!   dtlci_sfc = LCI heat flux  into layers above and below pbl top (W/m^2)
!   drag_sfc = surface drag coefficient for actual and oceanic roughness lengths
!   co2flx_sfc = surface flux of co2 from land 
!               
!
!-------------------------------------------------------------------

 	end module fluxcoupler_fields_sfc

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
