!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

	module initialize_pcontrol

!-----------------------------------------------------------------
! 	this module initializes fields and parameters used
!          by the physics portion of the model (pcontrol)
!-----------------------------------------------------------------

	use kinds
	use model_parameters
	use sigma_coordinate
	use physical_parameters
	use domain_decomposition
	use prognostics
	use chunk_parameters
	use time_manager_module
	use atmos_ozone_utilities
        use fluxcoupler_fields
#ifndef scm
	use gather_scatter
        use qp_module
        use grid_metrics
#endif
        use eau_global
#ifdef bugs_rad
        use rad_o3params
        use rad_params
#else
        use radgcm_parameters
#endif
#ifdef rad_isccp
        use isccp_tauinit
#endif

	implicit none

!*************************************************************************

	contains

!*************************************************************************


!======================================================================
! BEGINNING OF INIT_PCONTROL
!======================================================================

	subroutine init_pcontrol
!-------------------------------------------------------------------------
!  This routine reads the physics initial and boundary conditions files
!       (cke and climatological ozone)
!-------------------------------------------------------------------------

	integer (kind=int_kind)::
     *  i, nsecond_tem, max_ig_tem, nlm_tem, lrecl, j, ksdm,
     *  ioerr
	
      real (kind=selected_real_kind(6)), allocatable, dimension(:,:) :: xrec4
      real (kind=selected_real_kind(13)), allocatable, dimension(:,:) :: xrec8

        cke = c0
!-------------------------------------------------------------------------
!  get the header record
!-------------------------------------------------------------------------
 
          open(4,file=trim(datapath)//'data/ic_physics',form='unformatted'
     *,        access='DIRECT', recl=16,status='old',action='read'
     *,        iostat=ioerr)
          if (ioerr /= 0) then
            write(*,*) '*** init_pcontrol: ic_physics file error '
     *,       ioerr
            stop
          endif
          read(4,rec=1)nsecond_tem,max_ig_tem,nlm_tem,lrecl

          close(4)

!-------------------------------------------------------------------------
! read in physics prognostic arrays
!-------------------------------------------------------------------------
          open(4,file=trim(datapath)//'data/ic_physics',form='unformatted'
     *,        access='DIRECT', recl=lrecl)

          if(lrecl.eq.4*nlm*nlm) then
            allocate(xrec4(nlm,nlm))
            if(my_task.eq.0) print *,'reading real*4 ic_physics'
            do j = 1,jjm
              do i = 1,iim
                do ksdm = 1,nsdm
#ifndef scm
                  read(4,rec=1+grid_center_index(i,j,ksdm)) xrec4
#else
                  read(4,rec=1+1) xrec4
#endif
                  cke(i,j,:,:,ksdm) = xrec4(:,:)
                enddo
              enddo
            enddo
            deallocate(xrec4)
          else if (lrecl.eq.8*nlm*nlm) then
            allocate(xrec8(nlm,nlm))
            if(my_task.eq.0) print *,'reading real*8 ic_physics'
            do j = 1,jjm
              do i = 1,iim
                do ksdm = 1,nsdm
#ifndef scm
                  read(4,rec=1+grid_center_index(i,j,ksdm)) xrec8
#else
                  read(4,rec=1+1) xrec8
#endif
                  cke(i,j,:,:,ksdm) = xrec8(:,:)
                enddo
              enddo
            enddo
            deallocate(xrec8)
          else
            if(my_task.eq.0) print *,'improper ic_physics recl, cke set to 0'
          endif

          close(4)

!-------------------------------------------------------------------------
! compute subdomain chunks for physics parameterizations
!-------------------------------------------------------------------------
        call init_chunks

!-------------------------------------------------------------------------
! read in the climatological ozone
!-------------------------------------------------------------------------

        o3dat = c0
        if(clizon) then
!THE OLD WAY OF GETTING OZONE - READING THE PRE-INTERPOLATED (BOTH HORIZONTALLY
! AND VERTICALLY) FILE
!!-------------------------------------------------------------------------
!!  get the header record
!!-------------------------------------------------------------------------
! 
!          open(3,file=trim(datapath)//'data/bc_ozone',form='unformatted'
!     *,        access='DIRECT', recl=12,status='old',action='read'
!     *,        iostat=ioerr)
!          if (ioerr /= 0) then
!            write(*,*) '*** init_pcontrol: bc_ozone file error ',ioerr
!            stop
!          endif
!          read(3,rec=1)max_ig_tem,nlm_tem,lrecl
!
!          close(3)
!
!!-------------------------------------------------------------------------
!! read in physics prognostic arrays
!!-------------------------------------------------------------------------
!          open(3,file=trim(datapath)//'data/bc_ozone',form='unformatted'
!     *,        access='DIRECT', recl=lrecl)
!
!          allocate(xrec(nlm+1,12))
!          do j = 1,jjm
!            do i = 1,iim
!              do ksdm = 1,nsdm
!#ifndef scm
!                read(3,rec=1+grid_center_index(i,j,ksdm)) xrec
!#else
!                read(3,rec=1+1) xrec
!#endif
!                o3dat(i,j,:,:,ksdm) = xrec(:,:)
!              enddo
!            enddo
!          enddo
!          deallocate(xrec)
!          close(3)
!          where(o3dat(:,:,:,:,:).lt.1.e-9_dbl_kind) o3dat(:,:,:,:,:) = 1.e-9

!THE NEW WAY OF GETTING OZONE - READING THE STANDARD FILE AND DOING
! ALL INTERPOLATIONS HERE - BORROWING FROM BUGS6
           	call initialize_ozone(o3dat, plgl2)
        endif

!-------------------------------------------------------------------------
! initialize flux_coupler fields
!-------------------------------------------------------------------------
        alvdr_atm = c0
        alndf_atm = c0
        alvdr_atm = c0
        alndr_atm = c0
        tgeff4_atm = c0
        pc_atm = c0
        cldpc_atm = c0
        lwp_atm = c0
        pblcw_atm = c0
        pblci_atm = c0
        radvdc_atm = c0
        radndc_atm = c0
        radvbc_atm = c0
        radnbc_atm = c0
        dlwbot_atm = c0
        fluxd_atm = c0
        dcupr_atm = c0
        dlspr_atm = c0
        dlsps_atm = c0
        ps_atm = c0
        psb_atm = c0
        dpent_atm = c0
        co2flx_atm = c0

!----------------------------------------------------------------------
!     initialize eau_cup and eau_liq global variables:
!----------------------------------------------------------------------

      deauc_vtot(:,:,:,:) = c0
      deaul_vtot(:,:,:,:) = c0


!----------------------------------------------------------------------
!     read input data for the ISCCP cloud simulator
!----------------------------------------------------------------------
#ifdef rad_isccp
      call init_tau
#endif

	end subroutine init_pcontrol

#ifndef scm
!======================================================================
! BEGINNING OF WRITE_RESTART_PHYSICS
!======================================================================
	subroutine write_restart_physics
	implicit none

        integer (kind=int_kind) :: lrecl, i, j, ioerr
        real (kind=real_kind) :: vers = 5.1

        character*8, time_label
        integer (kind=int_kind) :: monthofyear

!  do master processor restart io for serial io
#ifndef parallel_io
!  global variables
!  global rectangular grid workarray
        real (kind=dbl_kind), dimension(im,jm) :: work_glob
!  max_ig vectors
        real (kind=dbl_kind), dimension(:,:,:), allocatable :: cke_vec
#endif

!-------------------------------------------------------------------------
!  label the restart with a time stamp 
!-------------------------------------------------------------------------
        if(n_restart.gt.0) then
          write(unit=time_label,fmt='(i7)')
     *                  1000000+nsecond_dyn/86400
          time_label(1:1) = 'd'
        else
          monthofyear = mod(nmonth-1,12)+1
          write(unit=time_label,fmt='(i7)')
     &          1000000+100*(nyear+nyear0)+monthofyear 
          time_label(1:1) = 'y'
        endif

        lrecl = kind(cke) * nlm * nlm

#ifndef parallel_io

!-------------------------------------------------------------------------
!  gather the global arrays and reduce to vector
!-------------------------------------------------------------------------
        if(my_task.eq.0) then
           allocate(cke_vec(max_ig,nlm,nlm))
        endif
        do i = 1,nlm
           do j = 1,nlm
              call gather_fine(cke(:,:,i,j,:),work_glob(:,:))
              if(my_task.eq.0)
     *           call reduce_to_vector_global(work_glob(:,:),cke_vec(:,i,j))
           enddo
        enddo

        if(my_task.eq.0) then
           open(4,
     * file=trim(datapath)//'restarts/model_restart_physics.'//trim(time_label)
     *,         form='unformatted', access='DIRECT', recl=lrecl)

!-------------------------------------------------------------------------
! write out scalar data
!-------------------------------------------------------------------------
           write(4,rec=1,iostat=ioerr) nsecond_dyn, max_ig, nlm, lrecl, vers
           if(ioerr.ne.0) 
     *        print *,'phys restart header ioerr, mytask ',ioerr,my_task

!-------------------------------------------------------------------------
! write out array data
!-------------------------------------------------------------------------
           do i = 1,max_ig
              write(4,rec=1+i,iostat=ioerr) cke_vec(i,:,:)
              if(ioerr.ne.0) 
     *           print *,'phys restart ioerr, mytask, rec '
     *,          ioerr,my_task,i+1
           enddo

           close(4)
           deallocate(cke_vec)

        endif

#else

        open(4,
     *  file=trim(datapath)//'restarts/model_restart_physics.'//trim(time_label)
     *,      form='unformatted', access='DIRECT', recl=lrecl)

!-------------------------------------------------------------------------
! write out scalar data
!-------------------------------------------------------------------------
        if(my_task.eq.0) then
           write(4,rec=1,iostat=ioerr) nsecond_dyn, max_ig, nlm, lrecl, vers
           if(ioerr.ne.0) 
     * print *,'phys restart header ioerr, mytask ',ioerr,my_task
        endif

!-------------------------------------------------------------------------
! write out array data
!-------------------------------------------------------------------------
        do j = 1,jjm
          do i = 1,iim
            do ksdm = 1,nsdm
              if(grid_center_mask(i,j,ksdm).gt.0.5_dbl_kind) then
           write(4,rec=1+grid_center_index(i,j,ksdm),iostat=ioerr) 
     *                    cke(i,j,:,:,ksdm)
        if(ioerr.ne.0) 
     * print *,'phys restart ioerr, mytask, rec '
     *,ioerr,my_task,grid_center_index(i,j,ksdm)+1
              endif
            enddo
          enddo
        enddo

        close(4)

#endif


	end subroutine write_restart_physics
!======================================================================
! END OF WRITE_RESTART_PHYSICS
!======================================================================
#endif


	end module initialize_pcontrol

!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
