c-------------------------------------------------------------------

      SUBROUTINE landiceflx( len,
     *     grav, dt, cp, hltm, delta,
     *     pr0, ribc, vkrmn, grav2, gamfac, tice,
     *     tg, bps, thm, ps, sh, 
     *     spdm, drag, ventmf, cu, ct, ustar,
     *     psb, fws, fss, smr, ros, ts, zb,
     *     dotkef, tke, thvgm, louis, zwind, ztemp,
     *     z0 )

c     subroutine landice computes the surface fluxes for a land point with 
c        permanent ice cover.

      use kinds
      use eau_util

      IMPLICIT none

c-------------------------------------------------------------------

c     SUBROUTINES CALLED:  EAU_SAT, VMFCAL

c     FUNCTIONS CALLED:
c          none
                       
c     argument list variables
c        len             gridpoints in this subdomain
c        dotkef          if true, use changan formulation rather than deardorff
      integer (kind=int_kind) ::
     *   len
 

c        dotkef          if true, use changan formulation rather than deardorff
      logical (kind=log_kind) ::
     *   dotkef, louis


c        grav            gravitational acceleration (m/s**2)
c        dt              time step (s)
c        cp              specific heat of dry air (J/kg)
c        delta
c        ribc            critical richardson number
c        pr0
c        vkrmn           von karman's constant
c        grav2           grav *0.01 mb/Pa
c        gamfac
c        tice            freezing temperature (K)
c        BPS            (surface pressure/p00)**kapa
c        PSB             PBL depth (hPa)
c        PS              surface presure (hPa)  
c        SMR             net surface radiation (W/m**2)
c        SPDM            PBL wind speed (m/s) 
c        ros             surface air density (kg/m**3)
c        ts              surface air temperature (K)
c        zb              boundary layer geometric depth (m)
c        tke             pbl turbulent kinetic energy
      real (kind=dbl_kind), intent(in) ::
     *     grav, dt, cp, hltm, delta, ribc,
     *     pr0, vkrmn, grav2, gamfac, tice, zwind, ztemp
c     input arrays
      real (kind=dbl_kind), intent(in), dimension(len) ::
     *     bps, ps, 
     *     spdm, psb,
     *     smr, ros, ts, zb,
     *     tke
c     output arrays
C        DRAG            SHEAR STRESS (PA)
c        FWS             surface evaporation (MM) 
c        FSS             surface sensible heat flux (w/m**2) 
c        ventmf
c        cu
c        ct
c        ustar
      real (kind=dbl_kind), intent(out), dimension(len) ::
     *     fws, fss, cu, ventmf
     *,    ct, ustar, thvgm
      real (kind=dbl_kind), intent(out), dimension(len,2) ::
     *     drag

c     in/out arrays (modified)
c        tg              ice surface temperature (K)
C        THM            MIXED LAYER POTENTIAL TEMPERATURE (K)
c        SH              PBL water vapor mixing ratio (kg/kg)
      real (kind=dbl_kind), intent(inout), dimension(len) ::
     *     tg, thm, sh, z0

c     local variables
      real (kind=dbl_kind), dimension(len) ::
     *     tha, sha, www,
     *     cni, cuni, ctni,
     *     ctni3, cti, cui,
     *     rib, thmtem, shtem
     *,    ustaro  ! friction velocity (m/s) ( for oceanic z0)
     *,    cuo     ! ( for oceanic z0)
     *,    zzwind, zztemp  ! for louis
     *,    cun, ctn
     *,    z1z0Urt, z1z0Trt

c     local variables from flxslv
      real (kind=dbl_kind) ::
     *     ig2, hlti, qgi, psbg2, czh
      real (kind=dbl_kind), dimension(len) ::
     *     GTR, QG, GAM  
     *,    TEMV       
     *,    VDT, CPGBL        
     *,    thgeff, shgeff, ptem
      real (kind=dbl_kind), dimension(len,3) ::
     *     b
      real (kind=dbl_kind), dimension(len,7) ::
     *     a

      integer (kind=int_kind) :: i
             
c-------------------------------------------------------------------

c     initialization
      DO I=1,len
         www(i) = 1.0     !  wet surface
         THA(I) = Tg(I) / BPS(I)
         z0(i) = 0.02
      enddo
      ptem(1:len) = ps(1:len) * 100.
      call eau_sat(len, ptem, tg, qsats=sha )
      do i = 1,len
         thgeff(i) = tha(i)
         shgeff(i) = sha(i)
      enddo

c     get wind stress (drag) and ventilation mass flux (ventmf)
      if(louis) then
          do i = 1,len
             zzwind(i) = zwind
             zztemp(i) = ztemp
          enddo
         call VMFCALZ(PR0,RIBC,VKRMN,DELTA,GRAV
     *,               SH,PS,z0           
     *,               SPDM,SHA,ROS,CU,ct,THVGM,RIB     
     *,               USTAR,VENTMF,THM, tha, zzwind, zztemp
     *,               cuni, cun, ctn, z1z0Urt, z1z0Trt, len ) 
         call VMFCALZO(PR0,RIBC,VKRMN,DELTA,GRAV
     *,                PS,tha          
     *,                SPDM,ROS,CUo,THVGM,RIB     
     *,                USTARo,zzwind,zztemp
     *,                len ) 
      else
         CALL VMFCAL(PR0,RIBC,VKRMN,DELTA,GRAV
     *,              SH,PS,Z0,SPDM   
     *,              SHA,ZB,ROS,CU,THVGM,RIB,USTAR     
     *,              VENTMF,THM, tha, cni, cuni, ctni
     *,              ctni3, cti, cui, len
     *,              thgeff, shgeff, tke, ct, dotkef )  
C   this call for ustar, cu for oceanic value of z0                                   
         CALL VMFCALo(PR0,RIBC,VKRMN,GRAV,SPDM   
     *,              ZB,ROS,CUo,THVGM,USTARo     
     *,              tha, len, thgeff, tke, dotkef ) 
      endif                                    
      DO I=1,len
         DRAG(I,1) = ROS(I) * USTAR(I)**2/spdm(i)
         DRAG(I,2) = ROS(I) * USTARo(I)**2/spdm(i)
      enddo

c     get latent and sensible heat fluxes, new ice surface temperature
c        (old subroutine flxslv inlined)
C                                                                       
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                       
C      THIS ROUTINE PERFORMS THE IMPLICIT TIME DIFFERENCING FOR THE     
C           BOUNDARY TEMPERATURE, AND THE PBL TEMPERATURE AND             
C           MOISTURE                                                    
C                                                                                                                                              
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                       
      czh = 4.2 e 4 * 5.1     !  ice surface heat capacity
      IG2   =1.0/GRAV2                                               
      HLTI  =1.0/HLTM                                                
      do i = 1,len 
         gtr(i) = tg(i)
      enddo
                                             
      call eau_sat(len, ptem, gtr, qsats=qg)                                         
C                                                                       
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                       
C    MATRIX SYSTEM TO SOLVE IS:                                         
C                                                                       
C              (  A1   A2    0  )      (  T  )       (  B1  )           
C              (  A3   A4   A5  )  *   ( tg  )   =   (  B2  )           
C              (   0   A6   A7  )      ( SH  )       (  B3  )           
C                                                                       
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      DO I=1,LEN                                                   
         TEMV (I)=PSB(i)*IG2                                               
         GAM  (I)=QG(I)*GAMFAC/(tg(i)*tg(i))                               
         VDT  (I)=ventmf(i)*dt                                                 
         CPGBL(I)=CP*GAM(I)*HLTI                                           
C                                                                       
         A(I,1)=bps(i)*(TEMV(I)+VDT(I))                                   
         A(I,2)=-VDT(I)                                                    
         A(I,3)=-VDT(I)*CP*bps(i)                                      
         A(I,4)= CZH+dt*( ventmf(i)*(cp+CPGBL(I)*hltm) )                                       
         A(I,5)=-VDT(I)*HLTM                                      
         A(I,6)=-VDT(I)*CPGBL(I)                                     
         A(I,7)=TEMV(I)+VDT(I)                                       
C                                                                       
         B(I,1)=TEMV(I)*Thm(i)*bps(i)                                       
         B(I,2)=(CZH*tg(i)+dt*(SMR(i)                                  
     1          +ventmf(i)*HLTM*(CPGBL(I)*tg(i) -QG(I))))                 
         B(I,3)=TEMV(I)*SH(i)+VDT(I)* (QG(I)-CPGBL(I)*tg(i))          
C                                                                       
C     SOLVE SYSTEM BY ELIMINATION AND BACK SUBSTITUTION                 
C                                                                       
         TEMV(I  )=A(I,3)/A(I,1)                                           
         A   (I,4)=A(I,4)-A(I,2)*TEMV(I)                                   
         B   (I,2)=B(I,2)-B(I,1)*TEMV(I)                                   
         TEMV(I  )=A(I,6)/A(I,4)                                           
         A   (I,7)=A(I,7)-A(I,5)*TEMV(I)                                   
         B   (I,3)=B(I,3)-B(I,2)*TEMV(I)                                   
         B   (I,3)=B(I,3)/A(I,7)                                           
         B   (I,2)=(B(I,2)-A(I,5)*B(I,3))/A(I,4)                           
         B   (I,1)=(B(I,1)-A(I,2)*B(I,2))/A(I,1)                           
      enddo                                                          
C                                                                       
C     CHECK ICE POINTS WITH tg > TICE AND CORRECT                
C                                                                       
      DO I = 1,len 
         if(b(i,2).gt.tice) then                                             
            GTR(I)=TICE                                                 
            QGI   =QG(I)+CPGBL(I)*(GTR(I)-tg(i))                        
            PSBG2 =PSB(i)*IG2                                           
            A(I,1)=bps(i)*(PSBG2+VDT(I))                               
            A(I,7)=PSBG2+VDT(I)                                   
            B(I,1)=(PSBG2*Thm(i)*bps(i)+VDT(I)*GTR(I))/A(I,1)            
            B(I,2)=TICE                                                 
            B(I,3)=(PSBG2*SH(i)+VDT(I)*QGI)/A(I,7)     
         endif           
      enddo                                                       
C                                                                       
C     UPDATE PROGNOSTIC VARIABLES                                       
C                                                                       
      DO I=1,LEN                                                   
         Thmtem(i)=B(I,1)                                                     
         GTR(I)=B(I,2)                                                     
         SHtem (i)=B(I,3)                                                     
         QG (I)=QG(I)+CPGBL(I)*(GTR(I)-tg(i))                              
         tg (i)=GTR(I)                                                     
C                                                                       
C     SURFACE SENSIBLE AND LATENT HEAT FLUXES                           
C                                                                       
         FSS(i)=ventmf(i)*CP*(tg(i)-bps(i) * Thmtem(i))                             
         FWS(i)=ventmf(i)*(QG(I)-SHtem(i))                              
      enddo 

c     END INLINED FLXSLV                                                         
C                

      RETURN
      END
