      module negtke_module

      use kinds
      use physical_parameters
      use pbl_parameters
      use cpl_cubic_module

      implicit none
      private

      public :: negetke
      
      contains

!---------------------------------------------------------------------------
      Subroutine NEGETKE(ipt, len, rom, krash, kpcinv, a0coeff, a1coeff,   &
                         a2coeff, invcoeff, buoy0, shear0, tke,            &
                         root1, root2, root3, sumcoeff, threereal)
!-- compute solution to tke equation when "E<0" mode is encountered.
!-- new way to do this is to solve the cubic equation, so the dissipation
!-- term in treated in a fully-implicit manner (not semi-implicit)

      integer(kind=int_kind), intent(in) :: len, ipt
      real(kind=dbl_kind), dimension(len), intent(in) ::       &
        rom, invcoeff, buoy0, shear0, tke
      real(kind=dbl_kind), dimension(len), intent(inout) ::       &
        sumcoeff
      real(kind=dbl_kind), intent(in) ::       &
        krash
      complex (kind=dbl_kind), intent(inout) :: &
         root2, &
         root3
      logical (kind=log_kind), intent(inout) :: &
         threereal
      real(kind=dbl_kind), intent(inout) ::       &
        root1
      real(kind=dbl_kind), intent(out) ::       &
        kpcinv, a2coeff, a1coeff, a0coeff
      

      kpcinv = c1 /(krash*rom(ipt)*c)
      a2coeff = invcoeff(ipt)*kpcinv
      a1coeff = 0.0
      a0coeff = krash*(buoy0(ipt)+shear0(ipt)) + invcoeff(ipt)*tke(ipt)
      a0coeff = -a0coeff*kpcinv

      call cpl_cubic(a0coeff, a1coeff, a2coeff, root1, root2, &
                 root3,sumcoeff(1), threereal)
!      write(*,'(i4,4f10.5,2l2)') i,root1,root1**2,
!     +  max(0.0_dbl_kind, ROOT1)**2,tke(i),strts(i), threereal

      end subroutine negetke

!======================================================================
! END OF COMP_TKE
!======================================================================


      end module negtke_module
