!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
	
 	module ocean_parameters

!-----------------------------------------------------------------
!	this module specifies parameters and options
!           for the ocean parameterization
!       default values are provided below
!-----------------------------------------------------------------

	use kinds
        use domain_decomposition_sfc

 	implicit none
 	save

	logical (kind=log_kind), parameter ::
     *		dopop   =  .false.
     *,		dotoms   = .false.
     *,		lqflux   = .false.
     *,         e2prog   = .true.

      real (kind=dbl_kind) ::
     *          sstanom = 0.0

!   ocean domain definition
        integer (kind=int_kind) :: nocean=0
        logical (kind=log_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *          hr_sea=.false.
        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *          lon_sea=0, lat_sea=0
        integer (kind=int_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *          link_hr_sea=0

!   prognostic variables
        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *      sst = 0    !  sea (or seaice) surface temperature (K)
     *,     iz = 0     !  sea ice thickness (m)
     *,     ti = 0     !  sea ice interior temperature (K)
     *,     sn = 0     !  snow depth on seaice (m)
     *,     e2 = 0     !  qflux ocean mixed layer energy (J/m^2)
     *,     tkeo = 0   !  turbulence kinetic energy

!   boundary conditions
        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *      hmax = 0   !  qflux ocean maximum mixed layer depth (m)
     *,     zs_sea = 0   !  ocean surface geopotential (m2/s2)
     *,     qf = 0     !  interpolated qflux upper ocean heat convergence (W/m^2)

        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,12,nsdm_sfc) ::
     *      hmix = 0   !  monthly qflux ocean upper mixed layer depth (m)    
     *,     qfm = 0    !  monthly qflux upper ocean heat convergence (W/m^2)    
     *,     e2m = 0    !  monthly qflux ocean prescribed mixed layer energy (J/m^2)

        real (kind=dbl_kind), dimension(:,:,:,:), allocatable ::
     *      sstm    !  monthly prescribed SST (K)
     *,     izm     !  monthly prescribed sea ice thickness (m)    
        integer (kind=int_kind) :: nmonth_bc

 

!-------------------------------------------------------------------
!   VARIABLE DEFINITION
!-------------------------------------------------------------------
!
!	dopop = if true, use POP dynamic ocean model
!	dotoms = if true, use TOMS dynamic ocean model
!	lqflux = if true, use QFLUX mixed layer ocean model
!       e2prog = if true, prognose e2 for qflux ocean
!
!       sstanom = a spatially uniform constant perturbation to the sst field (K)
!
!       nmonth_bc = number of months data for sst and iz in bc_ocean file
!
!-------------------------------------------------------------------

 	end module ocean_parameters

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
