!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
	
 	module pbl_parameters

!-----------------------------------------------------------------
!	this module specifies parameters and options
!           for the pbl and surface flux parameterizations  
!       default values are provided below
!-----------------------------------------------------------------

	use kinds
        use physical_parameters

 	implicit none
 	save


!  FOR PBL.F

	real (kind=dbl_kind), parameter :: 
     *		pr0 = 0.74
     *,         ribc = 3.05
     *,         vkrmn = 0.35
     *,         entmax = 0.1416
     *,         tkemin = 0.01
     *,         c = 1.0
     *,         hltinv = c1/hltm                
     *,         cpbl   = spec_heat_cp*hltinv                       
     *,         kappap1 = kappa + c1     
     *,         grav2 = grav * 0.01                 
     *,         deltap1 = delta + c1    
     *,         kb2 = p5*kappa 


	logical (kind=log_kind), parameter ::
     *		dotkef     = .false.
     *,         pblclassic = .true.
     *,         konor      = .false.
     *,         frac_strat = .true.
!     *,         extrapt    = .true.
!     *,         extrapq    = .true.

        real (kind=dbl_kind), parameter ::
     *          fac_extrapt = 1.0 
     *,         fac_extrapq = 1.0

!-------------------------------------------------------------------
!   VARIABLE DEFINITION
!-------------------------------------------------------------------
!	pr0 = turbulent Prandtl number at neutral stability
!	ribc = critical bulk Richardson number 
!	vkrmn = Von Karman's constant
!	entmax = absolute entraiment rate limit (kg/m^2/s)
!	tkemin = minimum limit for tke
!	c = TKE dissipation constant
!
!	dotkef = if true, use tke to compute surface transfer coefficients
!       pblclassic = if true, use older pbl entrainment and tke solution
!       frac_strat = if true, use a fractional pbl stratus parameterization
!                    based on sub-grid-scale topographic variance
!       extrapt = if true, get pb+ values of theta and t by extrapolation
!       extrapq = if true, get pb+ values of q by extrapolation
!       fac_extrapt = a factor that determines the degree to which pb+ values
!                       of theta are determined by extrapolation of the
!                       values of the two layers above: 0.0 is just a copy of the
!                       layer above; 1.0 is a linear extrapolation of both layers
!       fac_extrapq = like fac_extrapt, but for q
!
!-------------------------------------------------------------------

!  FOR VMFCAL*

	logical (kind=log_kind), parameter ::
     *		louis   = .false.

	real (kind=dbl_kind), parameter :: 
     *		zwind = 10.0
     *,         ztemp = 2.0

!-------------------------------------------------------------------
!   VARIABLE DEFINITION
!-------------------------------------------------------------------
!	louis = if true, use louis adaption to deardorff
!
!       zwind = reference height for wind (m)
!       ztemp = reference height for temperature (m)
!
!-------------------------------------------------------------------

 	end module pbl_parameters

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
