!------------------------------------------------------------------------------

subroutine pcontrol

use kinds
use domain_decomposition
use model_parameters
use sigma_coordinate
use tracer_names
use prognostics
use fluxcoupler_fields
use time_manager_module
use timer
use eau_global
use cumulus_friction
#ifndef scm
  use eau_budget
  use grid_metrics
#endif
#ifdef bugs_rad
   use landice_parameters
   use ocean_parameters
   use sib_parameters
   use rad_o3params
#endif
#ifdef scm
   use scm_module
   use pbp_module
#endif

implicit none

!------------------------------------------------------------------------------

!local variables:

logical (kind=log_kind), dimension(iim,jjm,nsdm):: &
   do_phys            !true if non-ghost grid-cell.

integer (kind=int_kind):: &
   ksdm, nt, nagrid

real (kind=dbl_kind), dimension(iim,jjm):: &
   psb                !thickness of the PBL layer                         (Pa). 

!local variables needed by the parameterization of convection:

real (kind=dbl_kind), dimension(iim,jjm,nlm,nsdm), save:: &
   delt_cup           !eau_cup pressure weighted temperature tendency (Pa.K/s).
      
real (kind=dbl_kind), dimension(iim,jjm,nlm,nwater,nsdm), save:: &
   delq_cup           !eau_cup pressure weighted tracer tendency  (Pa.kg/kg/s). 

!local variables needed by the parameterizarion of cloud microphysics:

real (kind=dbl_kind), dimension(iim,jjm,nlm):: &
   delt_elq           !eau_liq temperature tendency                      (K/s).
      
real (kind=dbl_kind), dimension(iim,jjm,nlm,nwater):: &
   delq_elq,         &!eau_liq tracer tendency                       (kg/kg/s).
   delqdw_cup         !eau_cup tracer tendency needed in eau_liq     (kg/kg/s).

!local variables needed by the flux coupler:

#ifdef bugs_rad
   real (kind=dbl_kind), dimension(iim,jjm,npbl+1,nsdm):: &
      pblfulw,      &!all-sky longwave upward fluxes in PBL            (W/m^2).
      pblfdlw        !all-sky longwave downward fluxes in PBL          (W/m^2).
#endif

!local variables needed for the passive tracers:

#ifdef passive_tracers
   real (kind=dbl_kind), dimension(iim,jjm,nlm,npassive,nsdm), save:: &
      dt_tracer     !tendency of passive tracers due to convective
                    !transport                                       (kg/kg/s).
#endif

!------------------------------------------------------------------------------

nagrid = iim*jjm
do_phys(:,:,:)   = .false.

#ifdef bugs_rad
   pblfulw(:,:,:,:) = 0.0_dbl_kind ; pblfdlw(:,:,:,:) = 0.0_dbl_kind
#endif

!loop over the individual domains starts here:

do ksdm = 1, nsdm

   psb(:,:) = ps(:,:,n3,ksdm) - pb(:,:,n3,ksdm)

#ifdef scm
   do_phys(:,:,ksdm) = .true.
#else
   where(grid_center_mask(:,:,ksdm).eq.1) do_phys(:,:,ksdm) = .true.
#endif

   if(l_physics) then

      delt_cup(:,:,:,ksdm)    = 0.0_dbl_kind
      delq_cup(:,:,:,:,ksdm)  = 0.0_dbl_kind
      delt_rad(:,:,:,ksdm)    = 0.0_dbl_kind

#ifdef passive_tracers
      dt_tracer(:,:,:,:,ksdm) = 0.0_dbl_kind
#endif

!call to the eau_cup parameterization of convection:

      call timer_start('eau_cup')

      call eau_cup_control &
               (                   nagrid ,                   ksdm , &
                        do_phys(1,1,ksdm) ,                    nlm , &
                                     lpbl ,                     dt , &
                          plg(1,1,1,ksdm) ,      plgl2(1,1,1,ksdm) , &
                          bpg(1,1,1,ksdm) ,      bpgl2(1,1,1,ksdm) , &
                          phi(1,1,1,ksdm) ,      phil2(1,1,1,ksdm) , &
                        th(1,1,1,n3,ksdm) ,        tl2(1,1,1,ksdm) , &
                          ssl(1,1,1,ksdm) ,      ssll2(1,1,1,ksdm) , &
                  tracer(1,1,1,1,n3,ksdm) , tracerl2(1,1,1,1,ksdm) , &
                           qs(1,1,1,ksdm) ,       qsl2(1,1,1,ksdm) , &
                        cke(1,1,1,1,ksdm) ,               psb(1,1) , &
                     cdwork(1,1,1,1,ksdm) ,    cumulusmf(1,1,ksdm) , &
                      dcupr_atm(1,1,ksdm) ,    lamda(1,1,1,1,ksdm) , &
                     masflx(1,1,1,1,ksdm) ,   delt_cup(1,1,1,ksdm) , &
                   delq_cup(1,1,1,1,ksdm) , deauc_vtot(1,1,1,ksdm)   &
#ifdef passive_tracers
                , dt_tracer(1,1,1,1,ksdm)                            &
#endif
               )   

      dlcup(:,:,:,ksdm) = delq_cup(:,:,:,icw,ksdm)
      dicup(:,:,:,ksdm) = delq_cup(:,:,:,ici,ksdm)

      delt_cup(:,:,:,ksdm) = delt_cup(:,:,:,ksdm)*pthick(:,:,:,n3,ksdm)
      do nt = 1, nwater
         delq_cup(:,:,:,nt,ksdm) = delq_cup(:,:,:,nt,ksdm) &
                                 * pthick(:,:,:,n3,ksdm)
      enddo

      call timer_stop('eau_cup')

!call to the parameterizations of radiation:

      call timer_start('rad')

#ifdef bugs_rad

      call rad_control &
               (                     ksdm ,   n_physics , n_pbp,        &
                        do_phys(1,1,ksdm) ,                    tofday , &
#ifdef scm
                                       dt ,                   lon_scm , &
                                  lat_scm ,         plgl2(1,1,1,ksdm) , &
#else
                                       dt , grid_center_lon(1,1,ksdm) , &
                grid_center_lat(1,1,ksdm) ,         plgl2(1,1,1,ksdm) , &
#endif
                        bpgl2(1,1,1,ksdm) ,           tl2(1,1,1,ksdm) , &         
                          plg(1,1,1,ksdm) ,           bpg(1,1,1,ksdm) , &
                        th(1,1,1,n3,ksdm) ,   tracer(1,1,1,1,n3,ksdm) , &
                    pblcw_atm(1,1,1,ksdm) ,     pblci_atm(1,1,1,ksdm) , &
                       climo3(1,1,1,ksdm) ,         dlcup(1,1,1,ksdm) , &
                        dicup(1,1,1,ksdm) ,       alvdr_atm(1,1,ksdm) , &
                      alvdf_atm(1,1,ksdm) ,       alndr_atm(1,1,ksdm) , &
                      alndf_atm(1,1,ksdm) ,      tgeff4_atm(1,1,ksdm) , &
                         pc_atm(1,1,ksdm) ,                             &
                      cldpc_atm(1,1,ksdm) ,          hr_sea(1,1,ksdm) , &
                         hr_sib(1,1,ksdm) ,          hr_ice(1,1,ksdm) , &
                     radvbc_atm(1,1,ksdm) ,      radvdc_atm(1,1,ksdm) , &
                     radnbc_atm(1,1,ksdm) ,      radndc_atm(1,1,ksdm) , &
                      pblfulw(1,1,1,ksdm) ,       pblfdlw(1,1,1,ksdm) , &
                     delt_rad(1,1,1,ksdm)                               &
               ) 

      fluxd_atm(:,:,ksdm)  = pblfdlw(:,:,1,ksdm)
      dlwbot_atm(:,:,ksdm) = pblfdlw(:,:,npbl+1,ksdm) 

#else

      call phys_rad &
                (                   nagrid ,                      ksdm , &
                         do_phys(1,1,ksdm) ,           plg(1,1,1,ksdm) , &
                           bpg(1,1,1,ksdm) ,         plgl2(1,1,1,ksdm) , &
                         bpgl2(1,1,1,ksdm) ,         th(1,1,1,n3,ksdm) , &
                   tracer(1,1,1,1,n3,ksdm) ,            qs(1,1,1,ksdm) , &
                         dlcup(1,1,1,ksdm) ,         dicup(1,1,1,ksdm) , &
#ifdef scm
                                  psb(1,1) ,                   lat_scm , &
                 		           lon_scm ,       alvdf_atm(1,1,ksdm) , &
#else
                                  psb(1,1) , grid_center_lat(1,1,ksdm) , &
                 grid_center_lon(1,1,ksdm) ,       alvdf_atm(1,1,ksdm) , &
#endif
                       alndf_atm(1,1,ksdm) ,       alvdr_atm(1,1,ksdm) , &
                       alndr_atm(1,1,ksdm) ,      tgeff4_atm(1,1,ksdm) , &
                          pc_atm(1,1,ksdm) ,       cldpc_atm(1,1,ksdm) , &
                       lwp_atm(1,1,1,ksdm) ,      radvdc_atm(1,1,ksdm) , &
                      radndc_atm(1,1,ksdm) ,      radvbc_atm(1,1,ksdm) , &
                      radnbc_atm(1,1,ksdm) ,      dlwbot_atm(1,1,ksdm) , &
                       fluxd_atm(1,1,ksdm) ,      delt_rad(1,1,1,ksdm)   &
                )

#endif

      delt_rad(:,:,:,ksdm) = delt_rad(:,:,:,ksdm)*pthick(:,:,:,n3,ksdm)

      call timer_stop('rad')

   endif ! (end l_physics)

   th_src(:,:,:,ksdm) = th_src(:,:,:,ksdm)                          &
                      + (delt_cup(:,:,:,ksdm)+delt_rad(:,:,:,ksdm)) & 
                      / bpg(:,:,:,ksdm)
   tracer_src(:,:,:,iq,ksdm) = tracer_src(:,:,:,iq,ksdm)            &
                             + delq_cup(:,:,:,iq,ksdm)

#ifdef passive_tracers
   do nt = 1, npassive
	tracer_src(:,:,:,ipassive(nt),ksdm) =                         &
             tracer_src(:,:,:,ipassive(nt),ksdm)                    &
            + dt_tracer(:,:,:,nt,ksdm)*pthick(:,:,:,n3,ksdm)
   enddo
#endif

!call to  the eau_liq parameterization of cloud microphysics: 

   call timer_start('eau_liq')

   delt_elq(:,:,:) = 0.0_dbl_kind ; delq_elq(:,:,:,:) = 0.0_dbl_kind
   do nt = 1, nwater
      delqdw_cup(:,:,:,nt) = delq_cup(:,:,:,nt,ksdm)  &
                           * invpthick(:,:,:,n3,ksdm) 
   enddo

   call eau_liq_control &
                (                 nagrid ,                ksdm , &
                       do_phys(1,1,ksdm) ,                 nlm , &
                                    lpbl ,              nstrat , &
                                      dt ,   plgl2(1,1,1,ksdm) , &
                       bpgl2(1,1,1,ksdm) ,   phil2(1,1,1,ksdm) , &
                         plg(1,1,1,ksdm) ,     bpg(1,1,1,ksdm) , &
                         phi(1,1,1,ksdm) ,   th(1,1,1,n3,ksdm) , &
                 tracer(1,1,1,1,n3,ksdm) , delqdw_cup(1,1,1,1) , &
                     dlspr_atm(1,1,ksdm) , dlsps_atm(1,1,ksdm) , &
                                delt_elq ,            delq_elq , &
                  deaul_vtot(1,1,1,ksdm)                         &
                )   
    
   th_src(:,:,:,ksdm) = th_src(:,:,:,ksdm) &
                      + delt_elq(:,:,:)*pthick(:,:,:,n3,ksdm)/bpg(:,:,:,ksdm)
   do nt = 1, nwater
      tracer_src(:,:,:,nt,ksdm) = tracer_src(:,:,:,nt,ksdm) &
                 + delq_elq(:,:,:,nt)*pthick(:,:,:,n3,ksdm)
   enddo

!call to eau_cke to update the cumulus kinetic energy:

   call eau_cupcke &
            (           nagrid , do_phys(1,1,ksdm) ,                  nlm , &
                          lpbl ,                dt , cdwork(1,1,1,1,ksdm) , &
             cke(1,1,1,1,ksdm)                                              &
            )

   call timer_stop('eau_liq')

enddo !(end ksdm loop)

! prepare the convective mass flux for cumulus friction:
#ifdef cup_friction
   if(l_physics) then
      call timer_start('cu fric')
      call cumulus_friction_init
      call timer_stop('cu fric')
   endif
#endif

!---- for SCM runs:
#ifdef scm
   if ((scm_mode == 'rev') .or. (scm_mode == 'REV') .or.  &
       (scm_mode == 'hrz') .or. (scm_mode == 'HRZ') .or.  &
       (scm_mode == 'rvx') .or. (scm_mode == 'RVX') .or.  &
       (scm_mode == 'rlx') .or. (scm_mode == 'RLX'))  then
      th_src(:,:,:,:) = th_src(:,:,:,:) &
                      + tadvt_terp(:,:,:,:)*pthick(:,:,:,n3,:)
      tracer_src(:,:,:,iq,:) = tracer_src(:,:,:,iq,:) &
                      + qadvt_terp(:,:,:,:)*pthick(:,:,:,n3,:)
  endif

!   if ((scm_mode == 'rvx') .or. (scm_mode == 'RVX'))  then
!      th_src(:,:,:,:) = th_src(:,:,:,:) &
!                      + vtadvt_terp(:,:,:,:)*pthick(:,:,:,n3,:)
!      tracer_src(:,:,:,iq,:) = tracer_src(:,:,:,iq,:) &
!                             + vqadvt_terp(:,:,:,:)*pthick(:,:,:,n3,:)
!   endif

#endif

end subroutine pcontrol

!------------------------------------------------------------------------------
