!-----------------------------------------------------------------------

      subroutine phys_rad
     *              (    nagrid ,       ksdm
     *,                 do_phys ,         pl
     *,                     bpg ,        pl2
     *,                    bpeg ,          t
     *,                  tracer ,        qls
     *,                   dlcup ,      dicup
     *,                     psb ,        lat
     *,                     lon ,  alvdf_atm
     *,               alndf_atm ,  alvdr_atm
     *,               alndr_atm , tgeff4_atm
     *,                  pc_atm ,  cldpc_atm
     *,                 lwp_atm , radvdc_atm
     *,              radndc_atm , radvbc_atm
     *,              radnbc_atm , dlwbot_atm
     *,               fluxd_atm ,       delt
     *              )

      use kinds
      use eau_util
      use time_manager_module
      use physical_parameters
      use model_parameters
      use sigma_coordinate
      use tracer_names
      use pcontrol_parameters
      use radgcm_parameters
#ifdef scm
      use scm_module
#else
      use qp_module
#endif

      implicit none

!-----------------------------------------------------------------------

! phys_rad is as the "old" subroutine physics, but without the call to
! convection. phys_rad is the driver for radgcm, the parameterization of
! long- and short-wave radiation.
! Laura D. Fowler/slikrock (03-01-01).

! send comments to laura@atmos.colostate.edu

! modifications:
!     none.

! subroutines called:
!     coszen: calculation of the solar zenith angle.
!     radgcm: Harshvardhan's parameterization of radiation.

! argument list variables:
!     input arguments:
!     ----------------

      integer (kind=int_kind) ::
     &  nagrid     !length of horizontal domain with ghost cells.
     &, ksdm       !loop index.

      logical (kind=log_kind), intent(in), dimension(nagrid)::
     &  do_phys    !select the non-ghost cells

!     flux coupler variables:
!     -----------------------

      real (kind=dbl_kind), intent(in), dimension(nagrid)::
     &  alvdf_atm  !
     &, alndf_atm  !
     &, alvdr_atm  !
     &, alndr_atm  !
     &, tgeff4_atm !sfc emitting temperature to 4th power
     &, pc_atm     !pbl cloud depth
     &, cldpc_atm  !pbl cloud fraction
      real (kind=dbl_kind), intent(in), dimension(nagrid,npbl)::
     &  lwp_atm    !pbl liquid water path

      real (kind=dbl_kind), intent(in), dimension(nagrid)::
     &  psb        !PBL pressure                                  (Pa).
     &, lon        !grid cell longitudes                     (radians).
     &, lat        !grid cell latitudes                      (radians).

      real (kind=dbl_kind), intent(in), dimension(nagrid,nlm)::
     &  pl         !mid layer pressure                            (Pa).
     &, bpg        !(plg/1000)**kapa                               (-).
     &, t          !potential temperature                          (K).
     &, qls        !saturation mixing ratio                    (kg/kg).
     &, dlcup      !convective detrainment of cloud water      (kg/kg).
     &, dicup      !convective detrainment of cloud ice        (kg/kg).

      real (kind=dbl_kind), intent(in), dimension(nagrid,nlm,ntracer)::
     &  tracer     !tracer mixing ratio                        (kg/kg).

      real (kind=dbl_kind), intent(in), dimension(nagrid,nlmp1)::
     &  pl2        !layer edge pressure                           (Pa).
     &, bpeg       !(pl2g/1000)**kapa                              (-).

!     output arguments:
!     -----------------

      real (kind=dbl_kind), intent(out), dimension(nagrid)::
     &  radndc_atm !sfc incident solar flux component           (W/m2).
     &, radvdc_atm !sfc incident solar flux component           (W/m2).
     &, radnbc_atm !sfc incident solar flux component           (W/m2).
     &, radvbc_atm !sfc incident solar flux component           (W/m2).
     &, dlwbot_atm !downward lw flux at sfc                     (W/m2).
     &, fluxd_atm  !downward lw flux at pbl top                 (W/m2).

      real (kind=dbl_kind), intent(out), dimension(nagrid,nlm)::
     &  delt       !radiative tendency of temperature              (K).

! local variables:

      logical (kind=log_kind), dimension(nagrid)::
     &  bombit

      integer (kind=int_kind)::
     &  i, k, nbomb, nc1c3, j, n
     
      real (kind=dbl_kind), dimension(nagrid) ::
     &  tgeff      !surface effective emitting temperature         (K).
     &, pc         !pbl cloud base pressure
     &, ptem       !
     &, cldpc      !pbl cloud fraction
     &, alvdf      !
     &, alvdr      !
     &, alndf      !
     &, alndr      !
     &, o3top      !ozone mixing ratio above model top         (kg/kg).
     &, tbmns      !
     &, tb         !
     &, gamb       !
     &, qob        !
     &, ts         !surface air temperature                        (K).
     &, cosz       !cos(solar zenith angle)
     &, sfac       !adjustment factor to solar const for
                   !no diurnal cycle

      real (kind=dbl_kind), dimension(nagrid,npbl) ::
     *   lwp         ! pbl cloud liquid water path

      real (kind=dbl_kind), dimension(nagrid,nlm) ::
     &  plg        !copy of pl                                   (hPa).
     &, dplg       !pressure thickness                           (hPa).
     &, tl         !temperature                                    (K).
     &, ql         !water vapor mixing ratio                   (kg/kg).
     &, cw         !vloud water mixing ratio                   (kg/kg).
     &, ci         !cloud ice mixing ratio                     (kg/kg).
     &, ri         !snow mixing ratio                          (kg/kg).
     &, o3l        !ozone mixing ratio                         (kg/kg).

      real (kind=dbl_kind), dimension(nagrid,nlmp1) ::
     &  pl2g       !copy of pl2                                  (hPa).
      real (kind=dbl_kind), dimension(nagrid) ::
     &  psbg       !copy of psb                                  (hPa).

!---- local radiation outputs:

      real (kind=dbl_kind), dimension(nagrid) ::
     &  dlwbot     !incident lw at surface                      (W/m2).
     &, fluxd_pb   !downward lw flux at pbl top                 (W/m2).
     &, radndc     !sfc incident solar flux component           (W/m2).
     &, radvdc     !sfc incident solar flux component           (W/m2).
     &, radnbc     !sfc incident solar flux component           (W/m2).
     &, radvbc     !sfc incident solar flux component           (W/m2).

      real (kind=dbl_kind), dimension(nagrid,nlm) ::
     &  atl        !lw all-sky heating rate profile              (K/s).
     &, asl        !sw all-sky heating rate profile              (K/s).

!----------------------------------------------------------------
 
      nc1c3 = 86400 / dtphy

      bombit(:) = .false.

      psbg          = psb*0.01_dbl_kind
      plg           = pl*0.01_dbl_kind
      pl2g          = pl2*0.01_dbl_kind
      dplg(:,1:nlm) = pl2g(:,2:nlmp1)-pl2g(:,1:nlm)

      tl(:,:) = t(:,:)*bpg(:,:)
      ql(:,:) = tracer(:,:,iq)   
      cw(:,:) = tracer(:,:,icw)
      ci(:,:) = tracer(:,:,ici)
      ri(:,:) = tracer(:,:,iri)

!---- copy flux coupler variables to local variables:

      pc(:)    = min(pc_atm(:)+pl2g(:,lpbl+1),pl2g(:,nlmp1))
      cldpc(:) = max(cldpc_atm(:),0.0_dbl_kind)
      cldpc(:) = min(cldpc(:),1.0_dbl_kind)
      alvdf(:) = alvdf_atm(:)
      alndf(:) = alndf_atm(:)
      alvdr(:) = alvdr_atm(:)
      alndr(:) = alndr_atm(:)
      tgeff(:) = tgeff4_atm(:)**0.25

      lwp(:,:) = 0.0_dbl_kind
      do i = 1, nagrid
         if(do_phys(i)) then
            do k = 1, npbl
               lwp(i,k) = max(lwp_atm(i,k),0.0_dbl_kind)
            enddo
         endif
      enddo

      o3top(:) = 0.0_dbl_kind
      o3l(:,:) = 0.0_dbl_kind
      n = 0
      do j = 1, jjm
         do i = 1, iim
            n = n + 1
            if(do_phys(n)) then         
               o3top(n)     = climo3(i,j,1,ksdm)
               o3l(n,1:nlm) = climo3(i,j,2:nlm+1,ksdm)
            endif
         enddo
      enddo

!---- mdb: modified for npbl>1

      tb(:)    = tl(:,nlm) * bpeg(:,nlm) / bpg(:,nlm)
      ptem(:)  = pl2g(:,nlm)*100.
      call eau_sat(nagrid, ptem, tb, qsats=qob)
      gamb(:)  = gamfac*qob(:) / (tb(:) * tb(:))
      tbmns(:) = (ql(:,nlm) - qob(:)) * hltm
      tbmns(:) = tbmns(:) / ((1.+gamb(:))*spec_heat_cp)
      tbmns(:) = max(0.0_dbl_kind, tbmns(:))
      tbmns(:) = tbmns(:) + tb(:)

      ts(:)    = tl(:,nlm)*bpeg(:,nlmp1)/bpg(:,nlm)

!---- calculates solar zenith angle:

      s0 = solcon / rsdist
      call coszen
     +        (       nagrid ,    .false.
     +,               ndiurn ,      cosd
     +,                 sind ,    tofday
     +,        24.0_dbl_kind ,        pi
     +,                  lat ,       lon
     +,                 cosz ,      sfac
     +,                nc1c3
     +        )

#ifndef scm
      call qp2sum(cosz,1._dbl_kind,iqp2_swcosz,ksdm,nagrid)
      call qp2sum(sfac,1._dbl_kind,iqp2_swsfac,ksdm,nagrid)
#endif

      call radgcm
     +        (        ksdm , n_physics ,  do_phys
     +,              nagrid ,       nlm ,     npbl
     +,              nstrat ,     n_pbp ,      plg
     +,                 bpg ,      dplg ,     pl2g
     +,                bpeg ,        tl ,       ql
     +,                 qls ,        cw ,       ci
     +,                  ri ,       o3l ,    dlcup
     +,               dicup ,     o3top ,       ts
     +,               tgeff ,     tbmns ,       pc
     +,                psbg ,     cldpc ,      lwp
     +,               alvdr ,     alvdf ,    alndr
     +,               alndf ,      cosz ,     sfac
     +,                  s0 ,    co2fac ,     grav
     +,        spec_heat_cp ,      tice ,   nozone
     +,              clizon ,     dopbl ,    nocld
     +,              radvbc ,    radvdc ,   radnbc
     +,              radndc ,    dlwbot , fluxd_pb
     +,                 atl ,       asl
     +        )

#ifdef scm
      if(scm_rad) then
         delt(1,:) = delt(1,:) + rad_terp(1,1,:,ksdm)
      else
         delt(:,:) = delt(:,:) + atl(:,:) + asl(:,:)
      endif
#else
      delt(:,:) = delt(:,:) + atl(:,:) + asl(:,:)
#endif

!---- copy local output variables back to flux coupler variables:

      fluxd_atm(:)  = fluxd_pb(:)
      dlwbot_atm(:) = dlwbot(:)
      radvbc_atm(:) = radvbc(:)
      radnbc_atm(:) = radnbc(:)
      radvdc_atm(:) = radvdc(:)
      radndc_atm(:) = radndc(:)

      if (.not. dsarmd) then
         nbomb = 0
         do i = 1, nagrid
            if(bombit(i)) then
               nbomb = nbomb + 1
               write(6,17062)i,pl2g(i,nlmp1)-pl2g(i,nlm)
               write(6,27062)(delt(i,k),k=1,nlm)
               write(6,47062)(tl(i,k),k=1,nlm)
               write(6,57062)(ql(i,k),k=1,nlm)
            endif
         enddo
      endif
17062 format('  *********** bomb at i=',i6,' ps-pb=',f7.2)
27062 format('  delt  ',9f6.1)
47062 format('    tl  ',9f6.1)
57062 format('    ql  ',9e10.4)      

      end subroutine phys_rad
      
!-----------------------------------------------------------------------
