!------------------------------------------------------------------------------

       subroutine radgcm
     *               (  ksdm , n_physics ,  do_phys
     *,               nagrid ,       nlm ,     npbl
     *,                  nls ,     n_pbp ,      plg
     *,                  bpg ,      dplg ,     pl2g
     *,                 bpeg ,        tl ,       ql
     *,                  qls ,        cw ,       ci
     *,                   ri ,       o3l ,    dlcup
     *,                dicup ,     o3top ,       ts
     *,                tgeff ,     tbmns ,       pc
     *,                  psb ,     cldpc ,      lwp
     *,                alvdr ,     alvdf ,    alndr
     *,                alndf ,      cosz ,     sfac
     *,                   s0 ,    CO2fac ,     grav
     *,                   cp ,      tice ,   nozone
     *,               clizon ,     dopbl ,    nocld
     *,               radvbc ,    radvdc ,   radnbc
     *,               radndc ,    dlwbot , fluxd_pb
     *,                  atl ,       asl
     *               )               

      use kinds
#ifndef scm
      use qp_module
      use gmeans_module
#endif
      use pbp_module

      implicit none

!------------------------------------------------------------------------------

!     subroutines called:
!     clcf    : parameterization of cloud fraction.
!     elqclop : parameterization of cloud optical properties.
!     lwrad   : parameterization of longwave radiation transfer.
!     swrad   : parameterization of shortwave radiation transfer.

!     argument list variables
!     input arguments:
!     ----------------

      logical (kind=log_kind), intent(in) ::
     *  nozone
     *, clizon
     *, dopbl
     *, nocld

      integer (kind=int_kind), intent(in)::
     &  ksdm         !loop index.
     &, n_physics    !nb of short-time steps per hour.
     &, n_pbp        !number of time-steps between pbp outputs.
     &, nagrid       !length of horizontal domain with ghost cells.
     &, nlm          !number of layers.
     &, nls          !number of layers in stratosphere.
     &, npbl         !number of PBL layers.

      logical (kind=log_kind), intent(in), dimension(nagrid)::
     &  do_phys   !select the non-ghost cells                      (-).

      real (kind=dbl_kind), intent(in)::
     &  s0 , CO2fac, cp, grav, tice

      real (kind=dbl_kind), intent(in), dimension(nagrid)::
     *  alvdr , alvdf , alndr , alndf
     *,    ts , tgeff , tbmns ,    pc
     *,   psb ,  cosz ,  sfac , cldpc
     *, o3top

      real (kind=dbl_kind), intent(in), dimension(nagrid,nlm)::
     *    plg ,   bpg ,  dplg ,    tl
     *,    ql ,   qls ,    cw ,    ci
     *,    ri ,   o3l , dlcup , dicup

      real (kind=dbl_kind), intent(in), dimension(nagrid,npbl)::
     *    lwp

      real (kind=dbl_kind), intent(in), dimension(nagrid,nlm+1)::
     *   pl2g ,  bpeg

!     output arguments:
!     -----------------

      real (kind=dbl_kind), intent(out), dimension(nagrid) ::
     *    radvbc ,   radvdc , radnbc , radndc
     *,   dlwbot , fluxd_pb

      real (kind=dbl_kind), intent(out), dimension(nagrid,nlm)::
     * atl, asl

! local variables:

      integer (kind=int_kind)::
     *  i, ibeg, l, len, lpbl, nlmm1, nlmp1, nnp
     
      real (kind=dbl_kind)::
     *  cld_wght, grav2, grav2i, x_physics, x_pbp

      real (kind=dbl_kind), dimension(nagrid)::
     *  pcb, cldt2d, tskin

      real (kind=dbl_kind), dimension(nagrid,nlm)::
     *     cw0 ,   cw1 ,    ci0 ,   ci1
     *,    ri0 ,   ri1 ,  ctot0 , ctot1
     *, temtot ,   cld ,    clu , cldcw
     *,  cldci ,  cld2 ,   clu2 ,  cldt
     *,  qltem

      real (kind=dbl_kind), dimension(nagrid,nlm+1)::
     *  codanv , codnoanv, om0, gwip

      real (kind=dbl_kind), dimension(nagrid,nlm+2)::
     *  emitrnd , emitmax, emitrnd2 , emitmax2

! local variables used for diagnostics only:

      real (kind=dbl_kind), dimension(nagrid)::
     *  radvbcc , radvdcc , radnbcc , radndcc
     *,  ulwbot ,  ulwtop ,      rs ,   rsclr
     *,  ultclr ,  olrtom , olrctom ,   swinc
     *,  dswbot ,  dswclr ,  dswtop ,  dswbcl
     *,  dswtom , dswctom ,      ss ,   ssclr

      real (kind=dbl_kind), dimension(nagrid,nlm)::
     *  atlclr  ,  aslclr

!------------------------------------------------------------------------------

!     write(6,*) '---- enter radgcm: ksdm=', ksdm

      lpbl  = nlm-npbl 
      nnp   = nlm+2
      nlmp1 = nlm + 1
      nlmm1 = nlm - 1

      grav2  = grav * 0.01
      grav2i = 1. / grav2

      x_physics = n_physics
      x_pbp     = min(n_physics, n_pbp)
      cld_wght  = n_physics*100.0_dbl_kind

!---- initialization of output arrays:

      radvbc(:)     = 0.0_dbl_kind
      radvdc(:)     = 0.0_dbl_kind
      radnbc(:)     = 0.0_dbl_kind
      radndc(:)     = 0.0_dbl_kind
      
      dlwbot(:)     = 0.0_dbl_kind
      fluxd_pb(:)   = 0.0_dbl_kind
      
      atl(:,:)      = 0.0_dbl_kind
      asl(:,:)      = 0.0_dbl_kind

!---- initilization of local arrays:

      radvbcc(:)    = 0.0_dbl_kind
      radvdcc(:)    = 0.0_dbl_kind
      radnbcc(:)    = 0.0_dbl_kind
      radndcc(:)    = 0.0_dbl_kind

      ulwbot(:)     = 0.0_dbl_kind
      ulwtop(:)     = 0.0_dbl_kind
      rs(:)         = 0.0_dbl_kind
      rsclr(:)      = 0.0_dbl_kind
      ultclr(:)     = 0.0_dbl_kind
      olrtom(:)     = 0.0_dbl_kind
      olrctom(:)    = 0.0_dbl_kind
      swinc(:)      = 0.0_dbl_kind
      dswbot(:)     = 0.0_dbl_kind
      dswclr(:)     = 0.0_dbl_kind
      dswtop(:)     = 0.0_dbl_kind
      dswbcl(:)     = 0.0_dbl_kind
      dswtom(:)     = 0.0_dbl_kind
      dswctom(:)    = 0.0_dbl_kind
      ss(:)         = 0.0_dbl_kind
      ssclr(:)      = 0.0_dbl_kind

      pcb(:)        = 0.0_dbl_kind
      cldt2d(:)     = 0.0_dbl_kind
      tskin(:)      = 0.0_dbl_kind

      cld(:,:)      = 0.0_dbl_kind
      clu(:,:)      = 0.0_dbl_kind
      cld2(:,:)     = 0.0_dbl_kind
      clu2(:,:)     = 0.0_dbl_kind
      cldcw(:,:)    = 0.0_dbl_kind
      cldci(:,:)    = 0.0_dbl_kind
      cldt(:,:)     = 0.0_dbl_kind
      
      codanv(:,:)   = 0.0_dbl_kind
      codnoanv(:,:) = 0.0_dbl_kind
      om0(:,:)      = 0.0_dbl_kind
      gwip(:,:)     = 0.0_dbl_kind
      emitrnd(:,:)  = 0.0_dbl_kind
      emitmax(:,:)  = 0.0_dbl_kind
      emitrnd(:,:)  = 0.0_dbl_kind
      emitmax(:,:)  = 0.0_dbl_kind      

      atlclr(:,:)   = 0.0_dbl_kind
      aslclr(:,:)   = 0.0_dbl_kind

!---- loop over the nagrid grid-points starts here:

      len = 1

      do i = 1, nagrid
         if(do_phys(i)) then
         
            ibeg = i

            cw0(i,:)     = 0.0_dbl_kind
            cw1(i,:)     = 0.0_dbl_kind
            ci0(i,:)     = 0.0_dbl_kind
            ci1(i,:)     = 0.0_dbl_kind
            ri0(i,:)     = 0.0_dbl_kind
            ri1(i,:)     = 0.0_dbl_kind
            ctot0(i,:)   = 0.0_dbl_kind
            ctot1(i,:)   = 0.0_dbl_kind
            temtot(i:,:) = 0.0_dbl_kind

            do l = 1,nlmm1
               temtot(i,l) = cw(i,l)+ci(i,l)+ri(i,l)
               if(temtot(i,l).gt.1.e-05) then
                  if(dlcup(i,l).ne.0. or. dicup(i,l).ne.0.) then
                     cw0(i,l) = cw(i,l)
                     ci0(i,l) = ci(i,l)
                     ri0(i,l) = ri(i,l)
                  else
                     cw1(i,l) = cw(i,l)
                     ci1(i,l) = ci(i,l)
                     ri1(i,l) = ri(i,l)
                  endif
               endif
            enddo
            cw0(i,nlm) = 0.0
            ci0(i,nlm) = 0.0
            ri0(i,nlm) = 0.0
            cw1(i,nlm) = cw(i,nlm)
            ci1(i,nlm) = ci(i,nlm)
            ri1(i,nlm) = ri(i,nlm)

           do l = 1, nlm
               ctot0(i,l) = cw0(i,l)+ci0(i,l)+ri0(i,l)
               ctot1(i,l) = cw1(i,l)+ci1(i,l)+ri1(i,l)
           enddo

            pcb(i) = pc(i) - pl2g(i,lpbl)

!---- cloud fraction:

            call clcf
     +              (  plg(ibeg,1) ,  pl2g(ibeg,1)
     +,                 tl(ibeg,1) ,   cw0(ibeg,1)
     +,                cw1(ibeg,1) ,   ci0(ibeg,1)
     +,                ci1(ibeg,1) ,   ri0(ibeg,1)
     +,                ri1(ibeg,1) , ctot0(ibeg,1)
     +,              ctot1(ibeg,1) ,      pc(ibeg)
     +,                cldpc(ibeg) ,   cld(ibeg,1)
     +,                clu(ibeg,1) , cldcw(ibeg,1)
     +,              cldci(ibeg,1) ,        nagrid
     +,                        nlm ,           nls
     +,                        len
     +              )

!---- cloud optical properties:

            call elqclop
     +              (   dplg(ibeg,1) ,        psb(ibeg)
     +,                   tl(ibeg,1) ,      cw0(ibeg,1)
     +,                  cw1(ibeg,1) ,      ci0(ibeg,1)
     +,                  ci1(ibeg,1) ,      ri0(ibeg,1)
     +,                  ri1(ibeg,1) ,    ctot0(ibeg,1)
     +,                ctot1(ibeg,1) ,            dopbl
     +,                  lwp(ibeg,1) ,      cld(ibeg,1)
     +,                  clu(ibeg,1) ,             tice
     +,                         grav ,  emitrnd(ibeg,1)
     +,              emitmax(ibeg,1) , codnoanv(ibeg,1)
     +,               codanv(ibeg,1) ,      om0(ibeg,1)
     +,                 gwip(ibeg,1) ,           nagrid
     +,                         nlm  ,             npbl
     +,                         lpbl ,              nls
     +,                          len ,            nocld
     +,                    pcb(ibeg)
     +              )

            do l = 1, nlm
               cld2(i,l) = 0.0
               clu2(i,l) = max(clu(i,l),cld(i,l))
            enddo
      
            do l = 1, nlmp1
               codanv(i,l)   = codanv(i,l) + codnoanv(i,l)
               codnoanv(i,l) = 0.
            enddo

            do l = 1, nnp
               emitmax2(i,l) = max(emitmax(i,l),emitrnd(i,l))
               emitrnd2(i,l) = 0.0
            enddo

!---- limit the water vapor mixing ratio:

            qltem(i,1:nlm) = ql(i,1:nlm)
            where (plg(i,1:nlm).le.100.0_dbl_kind)
     +           qltem(i,1:nlm) = 2.0e-6

!---- longwave radiation:

            call lwrad
     +              (           grav2 ,               cp
     +,                        nozone ,           clizon
     +,                   plg(ibeg,1) ,     pl2g(ibeg,1)
     +,                    tl(ibeg,1) ,      tgeff(ibeg)
     +,                      ts(ibeg) ,      tbmns(ibeg)
     +,                 qltem(ibeg,1) ,      o3l(ibeg,1)
     +,                   o3top(ibeg) ,     cld2(ibeg,1)
     +,                  clu2(ibeg,1) ,      atl(ibeg,1)
     +,                atlclr(ibeg,1) ,         rs(ibeg)
     +,                fluxd_pb(ibeg) ,     ulwbot(ibeg)
     +,                  dlwbot(ibeg) ,     ulwtop(ibeg)
     +,                   rsclr(ibeg) ,     ultclr(ibeg)
     +,                        co2fac , emitmax2(ibeg,1)
     +,              emitrnd2(ibeg,1) ,     olrtom(ibeg)
     +,                 olrctom(ibeg) ,           nagrid
     +,                          nlm  ,             lpbl
     +,                          nls  ,              len
     +,                           nnp ,            nocld
     +              )

!---- shortwave radiation:

            call swrad
     +              (           grav2 ,               cp
     +,                        nozone ,           clizon
     +,                            s0 ,   codanv(ibeg,1)
     +,              codnoanv(ibeg,1) ,      om0(ibeg,1)
     +,                  gwip(ibeg,1) ,      alvdf(ibeg)
     +,                   alndf(ibeg) ,      alvdr(ibeg)
     +,                   alndr(ibeg) ,       cosz(ibeg)
     +,                  pl2g(ibeg,1) ,     dplg(ibeg,1)
     +,                    tl(ibeg,1) ,    qltem(ibeg,1)
     +,                   o3l(ibeg,1) ,      o3top(ibeg)
     +,                   cld(ibeg,1) ,      clu(ibeg,1)
     +,                   swinc(ibeg) ,     dswclr(ibeg)
     +,                  dswtop(ibeg) ,      ssclr(ibeg)
     +,                      ss(ibeg) ,     dswbot(ibeg)
     +,                   asl(ibeg,1) ,   aslclr(ibeg,1)
     +,                  radvbc(ibeg) ,     radvdc(ibeg)
     +,                  radnbc(ibeg) ,     radndc(ibeg)
     +,                          sfac ,     dswbcl(ibeg)
     +,                       .false. ,    radvbcc(ibeg)
     +,                 radvdcc(ibeg) ,    radnbcc(ibeg)
     +,                 radndcc(ibeg) ,     dswtom(ibeg)
     +,                 dswctom(ibeg) ,           nagrid
     +,                           nlm ,              nls
     +,                           len ,            nocld
     +              )


!---- total cloudiness:
            do l = 1, nlm
               if(cld(i,l).gt.1.0e-3_dbl_kind .or. 
     +            clu(i,l).gt.1.0e-3_dbl_kind)
     +            cldt(i,l) = 1.0_dbl_kind
            enddo
      
            do l = nls+1, nlm
               cldt2d(i) = max(cldt2d(i),cldt(i,l))
            enddo

        endif ! end do_phys

      enddo ! end loop over nagrid grid-points

!---- radiation qp3s diagnostics:

#ifndef scm
      call qp3sum(o3l(1,1)   ,nagrid,x_physics,iqp3_o3     ,ksdm,nagrid)
      call qp3sum(asl(1,1)   ,nagrid,x_physics,iqp3_tntsw  ,ksdm,nagrid)
      call qp3sum(atl(1,1)   ,nagrid,x_physics,iqp3_tntlw  ,ksdm,nagrid)
      call qp3sum(aslclr(1,1),nagrid,x_physics,iqp3_tntswcs,ksdm,nagrid)
      call qp3sum(atlclr(1,1),nagrid,x_physics,iqp3_tntlwcs,ksdm,nagrid)
#endif

!---- radiation pbp2s diagnostics:

      call pbp2sum(o3l(1,1)   ,nagrid,x_pbp,ipbp2_o3     ,ksdm,nagrid)
      call pbp2sum(asl(1,1)   ,nagrid,x_pbp,ipbp2_tntsw  ,ksdm,nagrid)
      call pbp2sum(atl(1,1)   ,nagrid,x_pbp,ipbp2_tntlw  ,ksdm,nagrid)
      call pbp2sum(aslclr(1,1),nagrid,x_pbp,ipbp2_tntswcs,ksdm,nagrid)
      call pbp2sum(atlclr(1,1),nagrid,x_pbp,ipbp2_tntlwcs,ksdm,nagrid)
      call pbp2sum(cldt(1,1)  ,nagrid,x_pbp,ipbp2_cl     ,ksdm,nagrid)

!---- radiation qp2s diagnostics:

#ifndef scm
      call qp2sum(ss(1)    , 1._dbl_kind,iqp2_rss   ,ksdm,nagrid)
      call qp2sum(rs(1)    ,-1._dbl_kind,iqp2_rls   ,ksdm,nagrid)
      call qp2sum(ulwtop(1),-1._dbl_kind,iqp2_rlut  ,ksdm,nagrid)
      call qp2sum(dswtop(1), 1._dbl_kind,iqp2_rst   ,ksdm,nagrid)
      call qp2sum(swinc(1) , 1._dbl_kind,iqp2_rsdt  ,ksdm,nagrid)
      call qp2sum(dlwbot(1), 1._dbl_kind,iqp2_rlds  ,ksdm,nagrid)
      call qp2sum(dswbot(1), 1._dbl_kind,iqp2_rsds  ,ksdm,nagrid)
      call qp2sum(dlwbot(1),-1._dbl_kind,iqp2_rlus  ,ksdm,nagrid)
      call qp2sum(dswbot(1),-1._dbl_kind,iqp2_rsus  ,ksdm,nagrid)
      call qp2sum(swinc(1) ,-1._dbl_kind,iqp2_rsut  ,ksdm,nagrid)
      call qp2sum(dswbcl(1), 1._dbl_kind,iqp2_rsdscs,ksdm,nagrid)
      call qp2sum(dswbcl(1),-1._dbl_kind,iqp2_rsuscs,ksdm,nagrid)
      call qp2sum(ulwbot(1), 1._dbl_kind,iqp2_rldscs,ksdm,nagrid)
      call qp2sum(ultclr(1),-1._dbl_kind,iqp2_rlutcs,ksdm,nagrid)
      call qp2sum(swinc(1) ,-1._dbl_kind,iqp2_rsutcs,ksdm,nagrid)
      call qp2sum(dswtom(1), 1._dbl_kind,iqp2_rtmt  ,ksdm,nagrid)

      call qp2sum(rs(1)    ,-1._dbl_kind,iqp2_rlus  ,ksdm,nagrid
     +,           step_sample=0 )
      call qp2sum(ss(1)    , 1._dbl_kind,iqp2_rsus  ,ksdm,nagrid
     +,           step_sample=0)
      call qp2sum(dswtop(1), 1._dbl_kind,iqp2_rsut  ,ksdm,nagrid
     +,           step_sample=0 )
      call qp2sum(ssclr(1) , 1._dbl_kind,iqp2_rsuscs,ksdm,nagrid
     +,           step_sample=0 )
      call qp2sum(rsclr(1) ,-1._dbl_kind,iqp2_rldscs,ksdm,nagrid
     +,           step_sample=0 )
      call qp2sum(dswclr(1), 1._dbl_kind,iqp2_rsutcs,ksdm,nagrid
     +,           step_sample=0 )
      call qp2sum(olrtom(1),-1._dbl_kind,iqp2_rtmt  ,ksdm,nagrid
     +,           step_sample=0 )
#endif

!---- radiation pbp1s diagnostics:

      call pbp1sum(ss(1)    , x_pbp,ipbp1_rss   ,ksdm,nagrid)
      call pbp1sum(rs(1)    ,-x_pbp,ipbp1_rls   ,ksdm,nagrid)
      call pbp1sum(ulwtop(1),-x_pbp,ipbp1_rlt   ,ksdm,nagrid)
      call pbp1sum(dswtop(1), x_pbp,ipbp1_rst   ,ksdm,nagrid)
      call pbp1sum(swinc(1) , x_pbp,ipbp1_rsdt  ,ksdm,nagrid)
      call pbp1sum(dlwbot(1), x_pbp,ipbp1_rlds  ,ksdm,nagrid)
      call pbp1sum(dswbot(1), x_pbp,ipbp1_rsds  ,ksdm,nagrid)
!      call pbp1sum(dlwbot(1),-x_pbp,ipbp1_rlus  ,ksdm,nagrid)
!      call pbp1sum(rs(1)    ,-x_pbp,ipbp1_rlus  ,ksdm,nagrid)
      call pbp1sum(dlwbot(1), x_pbp,ipbp1_rlus  ,ksdm,nagrid)
      call pbp1sum(rs(1)    , x_pbp,ipbp1_rlus  ,ksdm,nagrid)
      call pbp1sum(dswbot(1),-x_pbp,ipbp1_rsus  ,ksdm,nagrid)
      call pbp1sum(ss(1)    , x_pbp,ipbp1_rsus  ,ksdm,nagrid)
      call pbp1sum(swinc(1) ,-x_pbp,ipbp1_rsut  ,ksdm,nagrid)
      call pbp1sum(dswtop(1), x_pbp,ipbp1_rsut  ,ksdm,nagrid)
      call pbp1sum(dswbcl(1), x_pbp,ipbp1_rsdscs,ksdm,nagrid)
      call pbp1sum(dswbcl(1),-x_pbp,ipbp1_rsuscs,ksdm,nagrid)
      call pbp1sum(ssclr(1) , x_pbp,ipbp1_rsuscs,ksdm,nagrid)
      call pbp1sum(ulwbot(1), x_pbp,ipbp1_rldscs,ksdm,nagrid)
      call pbp1sum(rsclr(1) ,-x_pbp,ipbp1_rldscs,ksdm,nagrid)
      call pbp1sum(ultclr(1),-x_pbp,ipbp1_rlutcs,ksdm,nagrid)
      call pbp1sum(swinc(1) ,-x_pbp,ipbp1_rsutcs,ksdm,nagrid)
      call pbp1sum(dswclr(1), x_pbp,ipbp1_rsutcs,ksdm,nagrid)

      tskin(:) = ((dlwbot(:) - 0.02*(rs(:)-dlwbot(:)))/(0.98*5.67e-8))**0.25
      call pbp1sum(tskin(1),x_pbp,ipbp1_tskin,ksdm,nagrid)

!---- radiation high-frequency diagnostics:

#ifndef scm
      call hf2sum(ulwtop(1),-1._dbl_kind,ihf2_rlut,ksdm,nagrid)
      call hf2sum(dswbot(1), 1._dbl_kind,ihf2_rsds,ksdm,nagrid)
      call hf2sum(dswbot(1), 1._dbl_kind,ihf2_rsus,ksdm,nagrid)
      call hf2sum(ss(1),    -1._dbl_kind,ihf2_rsus,ksdm,nagrid, step_sample=0)
      call hf2sum(dlwbot(1), 1._dbl_kind,ihf2_rlds,ksdm,nagrid)
      call hf2sum(dlwbot(1), 1._dbl_kind,ihf2_rlus,ksdm,nagrid)
      call hf2sum(rs(1),    -1._dbl_kind,ihf2_rlus,ksdm,nagrid, step_sample=0)
      call hf2sum(swinc(1) , 1._dbl_kind,ihf2_rsdt,ksdm,nagrid)
      call hf2sum(swinc(1), -1._dbl_kind,ihf2_rsut,ksdm,nagrid)
      call hf2sum(dswtop(1), 1._dbl_kind,ihf2_rsut,ksdm,nagrid, step_sample=0)

!---- cloud diagnostics:

      call qp3sum_cld(cldt(1,1),pc(1),nagrid,cld_wght,iqp3_cl,ksdm
     +,               nagrid)
      call qp2sum(cldt2d(1),100._dbl_kind,iqp2_clt,ksdm, nagrid )
      call hf2sum(cldt2d(1), 100._dbl_kind,ihf2_clt,ksdm,nagrid)
#endif
      call pbp1sum(cldt2d(1),x_pbp*100._dbl_kind,ipbp1_clt,ksdm,nagrid)

#ifndef scm
      call qp3sum(cld  ,nagrid ,x_physics,iqp3_cld,ksdm,nagrid)
      call qp3sum(clu  ,nagrid ,x_physics,iqp3_clu,ksdm,nagrid)
      call qp3sum(cldcw,nagrid,x_physics,iqp3_cldw,ksdm,nagrid)
      call qp3sum(cldci,nagrid,x_physics,iqp3_cldi,ksdm,nagrid)

!---- global means:

      call gmean_sum_atm(dswtom,nagrid, 1._dbl_kind,iglob_rmt,ksdm)
      call gmean_sum_atm(olrtom,nagrid,-1._dbl_kind,iglob_rmt,ksdm
     +,                  step_sample=0)

      call gmean_sum_atm(ss,nagrid, x_physics,iglob_hfns,ksdm)
      call gmean_sum_atm(rs,nagrid,-x_physics,iglob_hfns,ksdm
     +,                  step_sample=0)
#endif

!     write(6,*) '---- exit radgcm:'

      return
      end subroutine radgcm

!-----------------------------------------------------------------------

