!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
	
 	module radgcm_parameters

!-----------------------------------------------------------------
!	this module specifies parameters and options
!           for the cumulus parameterization
!       default values are provided below
!-----------------------------------------------------------------

	use kinds
        use model_parameters
        use sigma_coordinate
        use physical_parameters

 	implicit none
 	save

	real (kind=dbl_kind), parameter :: 
     *		CO2fac = 1.0
     *,         solcon = 1365.
     *,         perhl = 102.7
     *,         decmax = 23.441
     *,         eccn = 0.016715
     *,         PERHLR = PERHL * (pi/180.)
     *,         RECCN  = (1./(1.-ECCN*ECCN)**3)
     *,         H = PI2/365.

        integer (kind=int_kind), parameter ::
     *          eqnx = 80

	logical (kind=log_kind), parameter ::
     *		clizon  = .true.
     *,         CO2for  = .false.
     *,         nocld = .false.
     *,         dopbl = .false.
     *,         ndiurn = .false.

!  computed variables (time varying)
        real (kind=dbl_kind) ::
     *          s0
     *,         rsdist
     *,         sind
     *,         cosd

!  climatological ozone
        real (kind=dbl_kind), save, dimension(iim,jjm,nlm+1,12,nsdm) ::
     *          o3dat  ! monthly ozone mixing ratios
        real (kind=dbl_kind), save, dimension(iim,jjm,nlm+1,nsdm) ::
     *          climo3  ! interpolated ozone mixing ratios

!-------------------------------------------------------------------
!   VARIABLE DEFINITION
!-------------------------------------------------------------------
!	CO2fac = scaling factor * 330ppm to get co2 concentration in lwrad
!       solcon = solar constant at mean orbital radius (W/m^2)
!       perhl = time of perihelion relative to equinox (days)
!       decmax = inclination of axis (degrees)
!       eccn = orbital eccentricity 
!
!       eqnx = day of year of vernal equinox
!
!	clizon = if true, use climatological ozone
!       CO2for = if true, do a double CO2 lw forcing calculation
!       nocld = if true, clouds are ignored and treated as invisible
!       dopbl = if true, pbl cloud water is diagnosed from cloud depth,
!               else cloud properties are function of T and P
!       ndiurn = if true, the diurnal cycle is time averaged
!
!       s0 = Actual solar constant adjusted for orbital distance (W/m^2)
!       rsdist = square of ratio of orbital distance to mean
!       sind = sin(solar declination angle)
!       cosd = cos(solar declination angle)
!
!-------------------------------------------------------------------

 	end module radgcm_parameters

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
