      module sfc_control_module

      use kinds
      use physical_parameters
      use domain_decomposition_sfc
      use fluxcoupler_fields_sfc
      use time_manager_module
      use bc_update
      use cpl_pbl
      use utility

      use hstatc
      use eau_util

      implicit none

      contains


      subroutine sfc_control( init_fc, dtx, ksdm, n_flux_coupler )

c    This subroutine calls the ocean surface flux parameterization
c      and includes the prognostic treatment of the seasea temperature.
c    It also includes the prognostic q-flux mixed layer ocean as an
c      option.

!-- modified by M. Branson, 2/2001 - for npbl>1, compute mixed layer
!-- average of pbl layers so that pbl_state will still receive three 
!-- layers, the lowest of which will be the mixed layer

      use pbl_parameters
      use ocean_parameters
      use landice_parameters
      use sib_parameters
      use pbp_module_sfc

#ifndef scm
      use qp_module_sfc
      use gmeans_module
#else
      use scm_module
#endif

#ifdef bugs_rad
      use rad_params
#else
      use radgcm_parameters
#endif

! mdb 
      use sigma_coordinate

      implicit none


c    Argument list variables
      real (kind=dbl_kind), intent(in) ::
     *     dtx
      integer (kind=int_kind), intent(in) ::
     *     ksdm
     *,    n_flux_coupler
      logical (kind=log_kind), intent(in) ::
     *     init_fc

c    local variables
      REAL (kind=dbl_kind) ::
     *     rgfac, cpdgrv, cpinv, dtinv, grav2i
     *,    p00
c    pbl work variables
      REAL (kind=dbl_kind), dimension(iim_sfc,jjm_sfc) ::
     *     bync=0, shear=0, sumcoeff=0, altent1=0, altent2=0
     *,    ribn=0, diss=0, drb=0, fsvs=0, fhs=0
     *,    deltasv=0, eta=0, dsvcrit=0, pcbnolci=0
     *,    safe=0, wbp=0, tbp=0, tb=0, dwb=0, dhb=0
     *,    zb=0, ros=0, ts=0, spdm=0, toposd=0
     *,    liqb=0, tbmns=0, qsatb=0
     *,    gprod=0, cons=0, shacu=0, negeinc=0, lciinc=0
      logical (kind=log_kind), dimension(iim_sfc,jjm_sfc) ::
     *     negprod

c    surface flux work variables
      REAL (kind=dbl_kind), dimension(iim_sfc,jjm_sfc) ::
     *     thvgm=0, ct=0, cu=0, ventmf=0, smr=0, ss=0, rs=0, ustar=0
     *,    buoyf=0, obukhovl=0, spd10=0, t2=0

c    ocean surface work variables
      REAL (kind=dbl_kind), dimension(iim_sfc,jjm_sfc) ::
     *     flxsea, zmelt, sic

c    sib surface work and diagnostic variables
      REAL (kind=dbl_kind), dimension(iim_sfc,jjm_sfc) ::
     *     xgpp, runoff, mrfso, mrsos, mrso, mrros, snc, snm, assimn, respg
     *,    eci, ect, egi, egs, q3l, q3o, exo, tcas, qcas, rhcas
     *,    ra, rb, rd, chf, shf, radc31, radc32
      REAL (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsoil+1) ::
     *     soilscale
      REAL (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,4) ::
     *     rstfac_diag

C    work scalars
      REAL (kind=dbl_kind) :: frozfrac, dlsp

      REAL (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,2) ::
     *     dplg

C   do loop indices
      integer (kind=int_kind) ::
     *     i, j, l, ioffset, n, c3day

!-- mdb:  new variables
      real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nlm) :: 
     +     bp, pl
      real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nlmp1) :: 
     +     pl2, bpe
      real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,npbl+2) ::
     +     phi, ssl, tl, tvl, thvl, qs, bptem, pltem
      real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,npbl+3) ::
     +     phil2, thl2, ssll2, qsl2, tl2, ql2
      integer (kind=int_kind) :: k, kk, ilev, lindx, nelements            
      real (kind=dbl_kind) :: ptop, dp, sump, sumth, sumq, 
     +     thpbl, qpbl, invkappap1, dkappa, work1,
     +     work2, aa,
     +     x1,x2,xx1,xx2,xx3

      real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,4) :: pl2mix, bpemix
      real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,3) :: thmix, qmix
      real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,2) :: 
     +     dtlci_sfc_copy=0., dqlci_sfc_copy=0.
      real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc)   :: 
     +     lwpmix, pb, pblcw_mix

c   surface albedo variables
      real (kind=dbl_kind), parameter :: oceald=0.06
      real (kind=dbl_kind) ::
     *    fmelt, ocealb, tvec, scov, c_flux_coupler
      real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc) :: cosz, sfac
      real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,2,2) :: salb

!   qp work variable
      real (kind=dbl_kind), dimension(iim_sfc, jjm_sfc) :: qtem


      ptop = p_top * 0.01
      invkappap1 = 1./(kappa+1.)
      dkappa = 1. / kappa
      c_flux_coupler = n_flux_coupler


c  entrainment variables

      RGFAC  = 100.0/ gas_const_R
      grav2i = 100.  / GRAV
      CPDGRV = spec_heat_CP  / GRAV
      dtinv = 1. / dtx
      cpinv = 1. / spec_heat_CP
      p00 = p0_sfc * 0.01
      ioffset = 0.0
      runoff = 0.0
      ptop = p_top * 0.01
      invkappap1 = 1./(kappa+1.)
      dkappa = 1. / kappa
      c3day = nint(86400/dtx)

!  initialize some fields that will be written to qp
      ventmf = c0
      mrso = c0
      mrros = c0
      mrsos = c0
      mrfso = c0
      snc = c0
      snm = c0
      sic = c0
      assimn = c0
      respg = c0
      soilscale = c0
      eci = c0
      ect = c0
      egi = c0
      egs = c0
      q3l = c0
      q3o = c0
      exo = c0
      tcas = c0
      qcas = c0
      rhcas = c0
      rstfac_diag = c0
      ra = c0
      rb = c0
      rd = c0
      chf = c0
      shf = c0
      radc31 = c0
      radc32 = c0

!-- call press to get pressure variables
      pb(:,:) = ps_sfc(:,:,ksdm)-psb_sfc(:,:,ksdm)
      call press(
     *          pl2, 
     *          pl,
     *          bpe,
     *          bp,
     *          ps_sfc(:,:,ksdm)*100.,
     *          pb*100.,
     *          iim_sfc, jjm_sfc)

      call get_tracerl2(
     *          bp(:,:,nlm-(npbl+1):nlm),
     *          bpe(:,:,nlm-(npbl+1):nlmp1),
     *          t_sfc(:,:,:,ksdm),
     *          thl2(:,:,:),
     *          iim, jjm, npbl+2, npbl, 'THETA' )

      call get_geopotential(
     *          bpe(:,:,nlm-(npbl+1):nlmp1),
     *          bp(:,:,nlm-(npbl+1):nlm),
     *          zs_sfc(:,:,ksdm),
     *          t_sfc(:,:,:,ksdm),
     *          thl2(:,:,:),
     *          sigl2(nlm-(npbl+1):nlmp1),
     *          phi(:,:,:),
     *          phil2(:,:,:),
     *          iim, jjm, npbl+2  )

      bptem = bp(:,:,nlm-(npbl+1):nlm)
      pltem = pl(:,:,nlm-(npbl+1):nlm)
      tl(:,:,:) = t_sfc(:,:,:,ksdm) * bptem(:,:,:)
      tvl(:,:,:) = tl(:,:,:)*(1.+delta*sh_sfc(:,:,:,ksdm))
      thvl = tvl / bptem
      nelements = iim_sfc*jjm_sfc*(npbl+2)
      call eau_sat( nelements, pltem, tl, qsats=qs )

      call get_waterl2(
     *          pl2(:,:,nlm-(npbl+1):nlmp1),
     *          bp(:,:,nlm-(npbl+1):nlm),
     *          bpe(:,:,nlm-(npbl+1):nlmp1),
     *          ssl(:,:,:),
     *          ssll2(:,:,:),
     *          qs(:,:,:),
     *          qsl2(:,:,:),
     *          phi(:,:,:),
     *          phil2(:,:,:),
     *          t_sfc(:,:,:,ksdm),
     *          thl2(:,:,:),
     *          sh_sfc(:,:,:,ksdm),
     *          tl2(:,:,:),
     *          ql2(:,:,:),
     *          iim, jjm, npbl+2, npbl )

!-- convert from Pascals to mb 
      pl = pl*0.01_dbl_kind
      pl2 = pl2*0.01_dbl_kind

      do j = 1,jjm_sfc
        do i = 1,iim_sfc

	   if(grid_mask_sfc(i,j,ksdm)) then

           spdm(i,j) = max(1.0_dbl_kind, spdm_sfc(i,j,ksdm))
           toposd(i,j) = sqrt(tpgvar_sfc(i,j,ksdm))

            ts(i,j) = t_sfc(i,j,npbl+2,ksdm) * bpe(i,j,nlmp1)
            ros(i,j) = RGFAC * pl2(i,j,nlm) / TS(i,j)
            zb(i,j) = (phil2(i,j,3)-phil2(i,j,npbl+3))/grav

! for npbl>1, compute the mass-weighted pbl potential temperature
           if (npbl > 1) then
             sump = 0.; sumth = 0.; sumq = 0.
	     do k = 3,npbl+2
               kk = k + (nlm - (npbl+2))
	       dp    = pl2(i,j,kk+1) - pl2(i,j,kk)
	       sump  = sump  + dp
	       sumth = sumth + t_sfc(i,j,k,ksdm) * dp
	       sumq  = sumq  + sh_sfc(i,j,k,ksdm) * dp
             enddo
             thpbl = sumth / sump
             qpbl = sumq / sump

!-- set surface pressure and pfunc of mixed layer
             pl2mix(i,j,4) = pl2(i,j,nlmp1)
             bpemix(i,j,4) = bpe(i,j,nlmp1)

!-- set pressure and pfunc of pbl top and two layer edges above 
             do k = 1,3
               kk = k + (nlm - (npbl+2))
               pl2mix(i,j,k) = pl2(i,j,kk)
               bpemix(i,j,k) = bpe(i,j,kk)
             enddo

             thmix(i,j,3) = thpbl
             qmix(i,j,3) = qpbl
             do k = 1,2
               thmix(i,j,k) = t_sfc(i,j,k,ksdm)
               qmix(i,j,k) = sh_sfc(i,j,k,ksdm)
             enddo
           else   ! npbl = 1, so just copy arrays
             do k = 1,4
               kk = k + (nlm - (npbl+2))
               pl2mix(i,j,k) = pl2(i,j,kk)
               bpemix(i,j,k) = bpe(i,j,kk)
             enddo
             do k = 1,3
               thmix(i,j,k) = t_sfc(i,j,k,ksdm)
               qmix(i,j,k) = sh_sfc(i,j,k,ksdm)
             enddo
           endif

!-- set B+ values that pbl_state needs here
           tbp(i,j) = tl2(i,j,3)
           wbp(i,j) = ql2(i,j,3)

!           write(*,*) 'thmix(i,j,:) = ',thmix(i,j,:)
!           write(*,*) 'qmix(i,j,:) = ',qmix(i,j,:)*1000.
!           write(*,*) 'bpemix(i,j,:) = ',bpemix(i,j,:)
!           write(*,*) 'pl2mix(i,j,:) = ',pl2mix(i,j,:)
!           read(*,*)

           if(.not.init_fc) then

            ss(i,j) = radvdc_sfc(i,j,ksdm)*(1.-alvdf_sfc(i,j,ksdm)) +
     *                radndc_sfc(i,j,ksdm)*(1.-alndf_sfc(i,j,ksdm)) + 
     *                radvbc_sfc(i,j,ksdm)*(1.-alvdr_sfc(i,j,ksdm)) +
     *                radnbc_sfc(i,j,ksdm)*(1.-alndr_sfc(i,j,ksdm))

c  total large scale precip
            dlsp = dlspr_sfc(i,j,ksdm) + dlsps_sfc(i,j,ksdm)

c  do ocean points
        if(hr_sea(i,j,ksdm)) then 

#ifdef scm
         if(scm_presfluxes) sst(:,:,ksdm) = tg_scm(:,:,ksdm)
#endif

            rs(i,j) = stefan*sst(i,j,ksdm)**4 - dlwbot_sfc(i,j,ksdm)
            smr(i,j) = ss(i,j) - rs(i,j)

         call oceanflx( 1,
     *     grav, dtx, spec_heat_cp, hltm, delta, stefan,
     *     pr0, ribc, vkrmn, grav2, gamfac, tice, 
     *     sst(i,j,ksdm), bpemix(i,j,4), thmix(i,j,3),
     *     pl2mix(i,j,4),
     *     qmix(i,j,3), spdm(i,j), drag_sfc(i,j,1:2,ksdm),
     *     ventmf(i,j), cu(i,j), ct(i,j), ustar(i,j),
     *     zmelt(i,j),
     *     flxsea(i,j),
     *     psb_sfc(i,j,ksdm), fws_sfc(i,j,ksdm), fss_sfc(i,j,ksdm),
     *     smr(i,j), ros(i,j), ts(i,j), zb(i,j),
     *     ti(i,j,ksdm), iz(i,j,ksdm), sn(i,j,ksdm), dlsp,
     *     dcupr_sfc(i,j,ksdm), ss(i,j), lqflux, 86400._dbl_kind,
     *     dtpg,
     *     e2prog, tofday, hmix(i,j,k1,ksdm), hmix(i,j,k2,ksdm), hmax(i,j,ksdm),
     *     e2m(i,j,k1,ksdm), e2m(i,j,k2,ksdm), e2(i,j,ksdm), qf(i,j,ksdm), 
     *     tke_sfc(i,j,ksdm), dotkef, tpgf1, thvgm(i,j),
     *     louis, ztemp, zwind,
     *     z0d(i,j,ksdm) ) 

           if(iz(i,j,ksdm).gt.1.e-4_dbl_kind) sic(i,j) = 1.0 

        else if(hr_ice(i,j,ksdm)) then
#ifdef scm
         if(scm_presfluxes) tgi(:,:,ksdm) = tg_scm(:,:,ksdm)
#endif

            rs(i,j) = stefan*tgi(i,j,ksdm)**4 - dlwbot_sfc(i,j,ksdm)
            smr(i,j) = ss(i,j) - rs(i,j)
         call landiceflx( 1,
     *     grav, dtx, spec_heat_cp, hltm, delta,
     *     pr0, ribc, vkrmn, grav2, gamfac, tice, 
     *     tgi(i,j,ksdm), bpemix(i,j,4), thmix(i,j,3),
     *     pl2mix(i,j,4), qmix(i,j,3),
     *     spdm(i,j), drag_sfc(i,j,1:2,ksdm), ventmf(i,j), cu(i,j),
     *     ct(i,j), ustar(i,j), psb_sfc(i,j,ksdm), fws_sfc(i,j,ksdm),
     *     fss_sfc(i,j,ksdm), smr(i,j),
     *     ros(i,j), ts(i,j), zb(i,j),
     *     dotkef, tke_sfc(i,j,ksdm), thvgm(i,j), louis, zwind, ztemp,
     *     z0d(i,j,ksdm) )

        else if(hr_sib(i,j,ksdm)) then

#ifdef scm
         if(scm_presfluxes) tg(:,:,ksdm) = tg_scm(:,:,ksdm)
#endif
           call SIB
     *   ( iim_sfc*jjm_sfc, 1, nsoil                ! array and loop bounds
     *,    0.0_dbl_kind                     ! time independent variable
     *,    tcan(i,j,ksdm), tg(i,j,ksdm)       ! prognostic variables
     *,    deept(i,j,1,ksdm)                  ! prognostic variables
     *,    www(i,j,1,ksdm), snow(i,j,1,ksdm)  ! prognostic variables
     *,    capac(i,j,1,ksdm)                  ! prognostic variables
     *,    gs(i,j,ksdm), tke_sfc(i,j,ksdm)       ! prognostic variables
     *,    t_sfc(i,j,3,ksdm), sh_sfc(i,j,3,ksdm)      ! atmospheric forcing
!     *,    pl2(i,j,4)                                ! atmospheric forcing
!     *,    bpe(i,j,4), ros(i,j), ts(i,j)             ! atmospheric forcing
     *,    pl2(i,j,nlmp1)                             ! atmospheric forcing
     *,    bpe(i,j,nlmp1), ros(i,j), ts(i,j)          ! atmospheric forcing
     *,    psb_sfc(i,j,ksdm)                          ! atmospheric forcing
     *,    dcupr_sfc(i,j,ksdm), dlsp                  ! atmospheric forcing
     *,    radvbc_sfc(i,j,ksdm), radnbc_sfc(i,j,ksdm) ! atmospheric forcing
     *,    radvdc_sfc(i,j,ksdm), radndc_sfc(i,j,ksdm) ! atmospheric forcing
     *,    zb(i,j), spdm(i,j)                         ! atmospheric forcing
     *,    dlwbot_sfc(i,j,ksdm)                       ! atmospheric forcing
     *,    z0d(i,j,ksdm), zlt(i,j,ksdm), z1(i,j,ksdm)  ! surface parameters
     *,    z2(i,j,ksdm)                                ! surface parameters
     *,    rbc(i,j,ksdm), rdc(i,j,ksdm), dd(i,j,ksdm)  ! surface parameters
     *,    zdepth(i,j,1,ksdm), poros(i,j,ksdm)         ! surface parameters
     *,    phsat(i,j,ksdm), bee(i,j,ksdm)              ! surface parameters
     *,    respcp(i,j,ksdm)                            ! surface parameters
     *,    vmax0(i,j,ksdm), green(i,j,ksdm)            ! surface parameters
     *,    tran(i,j,1,1,ksdm)                          ! surface parameters
     *,    ref(i,j,1,1,ksdm)                           ! surface parameters
     *,    gmudmu(i,j,ksdm), trop(i,j,ksdm)            ! surface parameters
     *,    phc(i,j,ksdm), trda(i,j,ksdm)               ! surface parameters
     *,    trdm(i,j,ksdm), slti(i,j,ksdm)              ! surface parameters
     *,    shti(i,j,ksdm)                              ! surface parameters
     *,    hltii(i,j,ksdm), hhti(i,j,ksdm)             ! surface parameters
     *,    effcon(i,j,ksdm)                            ! surface parameters
     *,    binter(i,j,ksdm), gradm(i,j,ksdm)           ! surface parameters
     *,    atheta(i,j,ksdm)                            ! surface parameters
     *,    btheta(i,j,ksdm), aparc(i,j,ksdm)           ! surface parameters
     *,    wopt(i,j,ksdm), zm(i,j,ksdm)                ! surface parameters
     *,    wsat(i,j,ksdm), vcover(i,j,ksdm)            ! surface parameters
     *,    radfac(i,j,1,1,1,ksdm)                      ! surface parameters
     *,    thermk(i,j,ksdm), satco(i,j,ksdm)           ! surface parameters
     *,    slope(i,j,ksdm)                             ! surface parameters
     *,    chil(i,j,ksdm), ztdep(i,j,1,ksdm)           ! surface parameters
     *,    sandfrac(i,j,ksdm)                          ! surface parameters
     *,    pi, grav, dtx, spec_heat_cp, gas_const_R, hltm   ! constants
     *,    delta, asnow, kappa, snomel, clai, cww, pr0      ! constants
     *,    ribc, vkrmn, pco2m, po2m, stefan, grav2          ! constants
     *,    gamfac, tice, snofac                             ! constants
     *,    fss_sfc(i,j,ksdm), fws_sfc(i,j,ksdm)           ! output
     *,    drag_sfc(i,j,1,ksdm)                           ! output
     *,    co2flx_sfc(i,j,ksdm), cu(i,j), ct(i,j)         ! output
     *,    ustar(i,j)                                     ! output
     *,    ioffset
     *,    .false., forcerestore, doSiBco2, fixday, dotkef  ! logical switches
     *,    ventmf(i,j), thvgm(i,j), louis, zwind, ztemp
     *,    respfactor(i,j,1,ksdm)
     *,    xgpp(i,j), prescribed_gw, runoff(i,j), mrros(i,j)
     *,    snc(i,j), snm(i,j), assimn(i,j), soilscale(i,j,1), respg(i,j)
     *,    eci(i,j), ect(i,j), egi(i,j), egs(i,j)
     *,    exo(i,j), q3o(i,j), q3l(i,j), tcas(i,j), qcas(i,j), rhcas(i,j)
     *,    rstfac_diag(i,j,1) 
     *,    ra(i,j), rb(i,j), rd(i,j), chf(i,j), shf(i,j)
     *,    radc31(i,j), radc32(i,j) )

!  calculate some diagnostics
           mrso(i,j) = 0.0  ! total soil water content
           do k = 1,3
              mrso(i,j) = mrso(i,j) + zdepth(i,j,k, ksdm) * www(i,j,k,ksdm)
           enddo
           mrso(i,j) = mrso(i,j) 

           mrsos(i,j) = zdepth(i,j,1,ksdm) * www(i,j,1,ksdm) 
           mrsos(i,j) = mrsos(i,j) + (poros(i,j,ksdm)*0.1 - zdepth(i,j,1,ksdm))
     *                    * www(i,j,2,ksdm)

           frozfrac = 0.5 - (tg(i,j,ksdm)-tice)
           frozfrac = max(0.0_dbl_kind, frozfrac)
           frozfrac = min(1.0_dbl_kind, frozfrac) * 1000.
           mrfso(i,j) = zdepth(i,j,1,ksdm)*www(i,j,1,ksdm) * frozfrac
           frozfrac = 0.5 -(deept(i,j,2,ksdm)-tice)
           frozfrac = max(0.0_dbl_kind, frozfrac)
           frozfrac = min(1.0_dbl_kind, frozfrac) * 1000.
           mrfso(i,j) = mrfso(i,j)
     *            + zdepth(i,j,3,ksdm)*www(i,j,3,ksdm) * frozfrac
           do n = 3,nsoil
              frozfrac = 0.5 -(deept(i,j,n,ksdm)-tice)
              frozfrac = max(0.0_dbl_kind, frozfrac)
              frozfrac = min(1.0_dbl_kind, frozfrac) * 1000.
              mrfso(i,j) = mrfso(i,j) + 
     *          ztdep(i,j,n,ksdm)*www(i,j,2,ksdm)*poros(i,j,ksdm) * frozfrac
           enddo

        endif

#ifdef scm
         if (scm_presfluxes) then
           fss_sfc(:,:,ksdm) = fss_scm(:,:,ksdm)
           fws_sfc(:,:,ksdm) = fws_scm(:,:,ksdm) / hltm
         endif
#endif

c     get pbl entrainment

C     PBL THERMODYNAMIC VARIABLES - modified code from comp3 (mb)

         if(hr_sea(i,j,ksdm).or.hr_ice(i,j,ksdm).or.hr_sib(i,j,ksdm) ) then

         call cpl_PBL_STATE(iim_sfc*jjm_sfc,1,
     +     bpemix(i,j,2),pl2mix(i,j,2),
     +     thmix(i,j,2),qmix(i,j,2),
     +     zs_sfc(i,j,ksdm),ts(i,j),psb_sfc(i,j,ksdm),dhb(i,j),
     +     dwb(i,j),dplg(i,j,1),
     +     tb(i,j),qsatb(i,j),tbp(i,j),wbp(i,j),dtx,
     +     dtlci_sfc(i,j,1,ksdm),dqlci_sfc(i,j,1,ksdm),
     +     safe(i,j),pc_sfc(i,j,ksdm),
     +     pcbnolci(i,j), 
     +     dsvcrit(i,j), zb(i,j), toposd(i,j), eta(i,j),
     +     cldpc_sfc(i,j,ksdm), lwpmix(i,j), pblcw_mix(i,j),
     +     liqb(i,j), tbmns(i,j), shacu(i,j), lciinc(i,j),
     +     fss_sfc(i,j,ksdm),fws_sfc(i,j,ksdm),ventmf(i,j),
     +     thvgm(i,j),deltasv(i,j),fhs(i,j),fsvs(i,j),
     +     tke_sfc(i,j,ksdm),spdm(i,j),shear1_sfc(i,j,ksdm),ros(i,j),
     +     cu(i,j),fluxd_sfc(i,j,ksdm),drb(i,j),
     +     dpent_sfc(i,j,ksdm),diss(i,j),ribn(i,j),
     +     altent1(i,j),altent2(i,j), sumcoeff(i,j),
     +     negprod(i,j),
     +     bync(i,j), shear(i,j),
     +     gprod(i,j), cons(i,j), negeinc(i,j))

c     convert the heating and moistening rates to fluxes             
         do l=1,2
           dtlci_sfc_copy(i,j,l) = dtlci_sfc(i,j,l,ksdm) * 
     +       86400._dbl_kind * dtinv
           dqlci_sfc_copy(i,j,l) = dqlci_sfc(i,j,l,ksdm) *
     +       86400000._dbl_kind * dtinv
           dtlci_sfc(i,j,l,ksdm) = dtlci_sfc(i,j,l,ksdm) * spec_heat_cp 
     *                     * dplg(i,j,l) * grav2i * dtinv
           dqlci_sfc(i,j,l,ksdm) = dqlci_sfc(i,j,l,ksdm) * dplg(i,j,l)
     *                     * grav2i * dtinv
         enddo

         endif

         endif  ! if (.not.init_fc)

	endif	! if(grid_mask_sfc))

        enddo   ! i do loop
      enddo   ! j do loop

      lwp_sfc(:,:,:,ksdm)   = 0.
      pblcw_sfc(:,:,:,ksdm) = 0.
      pblci_sfc(:,:,:,ksdm) = 0.
      do j = 1,jjm_sfc
         do i = 1,iim_sfc
         if (grid_mask_sfc(i,j,ksdm)) then
         if (init_fc)  then

           call cpl_PBL_STATE(iim_sfc*jjm_sfc, 1,
     +       bpemix(i,j,2), pl2mix(i,j,2),
     +       thmix(i,j,2), qmix(i,j,2),
     +       zs_sfc(i,j,ksdm), ts(i,j), psb_sfc(1,1,ksdm), dhb(i,j),
     +       dwb(i,j), dplg(i,j,1),
     +       tb(i,j), qsatb(i,j), tbp(i,j), wbp(i,j), dt,
     +       dtlci_sfc(i,j,1,ksdm), dqlci_sfc(i,j,1,ksdm), safe(i,j),
     +       pc_sfc(i,j,ksdm), pcbnolci(i,j), dsvcrit(i,j),
     +       zb(i,j), toposd(i,j), eta(i,j),
     +       cldpc_sfc(i,j,ksdm), lwpmix(i,j), pblcw_mix(i,j), liqb(i,j), 
     +       tbmns(i,j), shacu(i,j), lciinc(i,j) )

         endif

! THIS IS BOGUS
!   CLD (CLDPC) IS ALREADY SET TO 1 OR 0 INSIDE PBL_STATE

! stratus clouds either 0/1 right now
! distribute liquid water from mixed layer back to multipbl
!            if(pc_sfc(i,j,ksdm).gt.pl2(i,j,3)) then
!               cldpc_sfc(i,j,ksdm) = 1.0

            if(cldpc_sfc(i,j,ksdm) > 0) then
               if (npbl > 1) then
                 aa = liqb(i,j)/(pc_sfc(i,j,ksdm)-pl2(i,j,3))
                 ilev = 4
                 lindx = 1
! do the fully saturated pbl layers
                 do while (pc_sfc(i,j,ksdm) > pl2(i,j,ilev))
                   work2 = (pc_sfc(i,j,ksdm)*pl2(i,j,ilev)) - 
     *               (0.5*(pl2(i,j,ilev)**2.))
                   work1 = (pc_sfc(i,j,ksdm)*pl2(i,j,ilev-1)) - 
     *               (0.5*(pl2(i,j,ilev-1)**2.))
                   lwp_sfc(i,j,lindx,ksdm) = aa*(work2-work1)*grav2i
                   ilev = ilev+1
                   lindx = lindx+1
                 enddo
! do the lowest layer that contains pc 
                 work1 = (pc_sfc(i,j,ksdm)*pl2(i,j,ilev-1)) - 
     *             (0.5*(pl2(i,j,ilev-1)**2.))
                 work2 = 0.5*(pc_sfc(i,j,ksdm)**2.) 
                 lwp_sfc(i,j,lindx,ksdm) = aa*(work2-work1)*grav2i
               else
                 lwp_sfc(i,j,1,ksdm)   = lwpmix(i,j)
                 pblcw_sfc(i,j,1,ksdm) = pblcw_mix(i,j)
               endif
!            else
!               cldpc_sfc(i,j,ksdm) = 0.0
!               if (lwpmix(i,j) .ne. 0.0) then
!                 write(*,*) 'cld = 0: lwpmix(i,j) = ',lwpmix(i,j)
!               endif
            endif
            pc_sfc(i,j,ksdm) = pc_sfc(i,j,ksdm) - pl2mix(i,j,3)

           endif
         enddo
      enddo


      do j=1,jjm_sfc
        do i = 1,iim_sfc
	
	  if(grid_mask_sfc(i,j,ksdm)) then

!-- compute 10-meter wind speed and 2-meter air temperature from Deardorff (1972) 
!-- formulas.

            if(.not.init_fc) then
              buoyf(i,j) = (fss_sfc(i,j,ksdm)/(spec_heat_cp*ros(i,j)) 
     *          + delta*t_sfc(i,j,3,ksdm)*fws_sfc(i,j,ksdm)/ros(i,j))

              if (buoyf(i,j) /= c0) then
                obukhovl(i,j) = -thvl(i,j,npbl+2) * ustar(i,j)**3 / 
     *            (grav * vkrmn * buoyf(i,j))
              else
                obukhovl(i,j) = c0
              endif

              if (buoyf(i,j) >= c0) then
                x1 = abs(c1 - 15._dbl_kind*10._dbl_kind/obukhovl(i,j))**0.25
                x2 = abs(c1 - 9._dbl_kind*2._dbl_kind/obukhovl(i,j))**0.25
                xx1 = (c1+x1**2)/c2
                xx2 = (c1+x1)/c2
                xx3 = (c1+x2**2)/c2

                spd10(i,j) = (ustar(i,j)/vkrmn)* (log(10._dbl_kind/z0d(i,j,ksdm)) - 
     *            (log(xx1)+2._dbl_kind*log(xx2)-2._dbl_kind*atan(x1)+pi/c2))

                t2(i,j) = (log(2._dbl_kind/z0d(i,j,ksdm)) - 2._dbl_kind*log(xx3))
                t2(i,j) = thvgm(i,j)+t_sfc(i,j,3,ksdm) - 
     *            buoyf(i,j)*pr0*t2(i,j)/(vkrmn*ustar(i,j))

              else
                spd10(i,j) = (ustar(i,j)/vkrmn)*(log(10._dbl_kind/z0d(i,j,ksdm)) +
     *            (4.7_dbl_kind*10._dbl_kind/obukhovl(i,j)))
                t2(i,j) = (ustar(i,j)/vkrmn)*(log(2._dbl_kind/z0d(i,j,ksdm)) +
     *            (4.7_dbl_kind*10._dbl_kind/(pr0*obukhovl(i,j))))
                t2(i,j) = thvgm(i,j)+t_sfc(i,j,3,ksdm) -
     *            buoyf(i,j)*pr0*t2(i,j)/(vkrmn*ustar(i,j))

              endif
              buoyf(i,j) = buoyf(i,j) * (spec_heat_cp*ros(i,j)) ! this is in W/m^2
              t2(i,j) = t2(i,j) * bpe(i,j,nlmp1)   ! theta to temperature
           endif
	
c   cos(zenith angle)

            call coszen(1, .false., ndiurn, cosd, sind, tofday,
     *        24.0_dbl_kind, pi, lat_sfc(i,j,ksdm), lon_sfc(i,j,ksdm), 
     *        cosz(i,j), sfac, c3day )
              cosz(i,j) = max( 0.01746_dbl_kind,cosz(i,j) ) 

          endif
          if(hr_sea(i,j,ksdm)) then
C                                                                       
C        OCEAN ALBEDO                       
C                                                                       
c     direct beam ocean albedo
         OCEALB = 0.026 / (cosz(i,j)**1.7 + 0.065)
     *              + 0.15                 
     *             * (cosz(i,j) - 0.1) * (cosz(i,j) - 0.5) 
     *              * (cosz(i,j) - 1.0) 
c     temperature ramp for snow and ice albedos near freezing
c     fmelt = 1 at -5C, 0.82 at 0C
            FMELT = 0.82 + .036*(tice-sst(i,j,ksdm) )
            fmelt = max(fmelt,0.82_dbl_kind)
            fmelt = min(fmelt,1.00_dbl_kind)
            tvec = ocealb+(.7 * FMELT-ocealb)*iz(i,j,ksdm)*20.0
            tvec = min(tvec,.7*fmelt)
c        snow on the ice   ! cold snow albedo is .8
            scov =  min(1.0_dbl_kind,sn(i,j,ksdm)*0.0005) 
            tvec = (1.-scov)*tvec + scov*.8*fmelt 
            alvdr_sfc(i,j,ksdm) = tvec
            alndr_sfc(i,j,ksdm) = tvec
c       diffuse beam
            tvec = oceald+(.7 * FMELT-oceald)*iz(i,j,ksdm)*20.0
            tvec = min(tvec,.7*fmelt)
            tvec = (1.-scov)*tvec + scov*.8*fmelt 
            alvdf_sfc(i,j,ksdm) = tvec
            alndf_sfc(i,j,ksdm) = tvec
            tgeff4_sfc(i,j,ksdm) = sst(i,j,ksdm)**4

         elseif(hr_ice(i,j,ksdm)) then

           alvdf_sfc(i,j,ksdm) = 0.8       
           alvdr_sfc(i,j,ksdm) = 0.8       
           alndf_sfc(i,j,ksdm) = 0.5   
           alndr_sfc(i,j,ksdm) = 0.5   
           tgeff4_sfc(i,j,ksdm) = tgi(i,j,ksdm)**4    

         elseif(hr_sib(i,j,ksdm)) then
c   GET Vegetated LAND SURFACE ALBEDOS
           call rada2(snow(i,j,1,ksdm), zlt(i,j,ksdm), z1(i,j,ksdm)
     *,               z2(i,j,ksdm)
     *,               asnow, tg(i,j,ksdm), cosz(i,j), tice
     *,               ref(i,j,1,1,ksdm), tran(i,j,1,1,ksdm), chil(i,j,ksdm)
     *,               green(i,j,ksdm), vcover(i,j,ksdm), soref(i,j,1,ksdm)
     *,               radfac(i,j,1,1,1,ksdm), salb(i,j,1,1)
     *,               thermk(i,j,ksdm)
     *,               tgeff4_sfc(i,j,ksdm), tcan(i,j,ksdm), 1, iim_sfc*jjm_sfc)

            alvdf_sfc(i,j,ksdm) = salb(i,j,1,2)
            alvdr_sfc(i,j,ksdm) = salb(i,j,1,1)
            alndf_sfc(i,j,ksdm) = salb(i,j,2,2)
            alndr_sfc(i,j,ksdm) = salb(i,j,2,1)

         endif

        enddo
      enddo
#ifdef scm
         if(scm_presfluxes) tgeff4_sfc(:,:,ksdm) = tg_scm(:,:,ksdm)**4
         if(scm_alb) then
            alvdf_sfc(:,:,ksdm) = alb_scm(:,:,ksdm)
            alvdr_sfc(:,:,ksdm) = alb_scm(:,:,ksdm)
            alndf_sfc(:,:,ksdm) = alb_scm(:,:,ksdm)
            alndr_sfc(:,:,ksdm) = alb_scm(:,:,ksdm)
         endif
#endif

!--------------------------------------------------------------------------
! fill qps
!--------------------------------------------------------------------------
      if(.not.init_fc) then
#ifndef scm
        call qp2sum_sfc(dtlci_sfc_copy(1,1,1),c_flux_coupler,iqp2_sfc_dtlci1,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(dtlci_sfc_copy(1,1,2),c_flux_coupler,iqp2_sfc_dtlci2,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(dqlci_sfc_copy(1,1,1),c_flux_coupler,iqp2_sfc_dqlci1,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(dqlci_sfc_copy(1,1,2),c_flux_coupler,iqp2_sfc_dqlci2,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(ventmf(1,1),c_flux_coupler,iqp2_sfc_ventmf,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(obukhovl(1,1),c_flux_coupler,iqp2_sfc_mol,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(ustar(1,1),c_flux_coupler,iqp2_sfc_ustar,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(z0d(1,1,ksdm),c_flux_coupler,iqp2_sfc_z0,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(buoyf(1,1),c_flux_coupler,iqp2_sfc_buoyf,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(spd10(1,1),c_flux_coupler,iqp2_sfc_spd10,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(t2(1,1),c_flux_coupler,iqp2_sfc_t2,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(tke_sfc(1,1,ksdm),c_flux_coupler,iqp2_sfc_tke,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(fss_sfc(1,1,ksdm), c_flux_coupler, iqp2_sfc_hfss,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(fws_sfc(1,1,ksdm), hltm*c_flux_coupler, iqp2_sfc_hfls,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(dpent_sfc(1,1,ksdm), c_flux_coupler, iqp2_sfc_blent, 
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(pc_sfc(1,1,ksdm),100.0_dbl_kind*c_flux_coupler,
     *                  iqp2_sfc_clblp, ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(cldpc_sfc(1,1,ksdm), c_flux_coupler, iqp2_sfc_clbl, 
     *                  ksdm, iim_sfc*jjm_sfc)
        qtem(:,:) = tgeff4_sfc(:,:,ksdm) **0.25
        call qp2sum_sfc(qtem(1,1), c_flux_coupler, iqp2_sfc_tradgrdeff, 
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(ps_sfc(1,1,ksdm), c_flux_coupler, iqp2_sfc_ps,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(psb_sfc(1,1,ksdm), c_flux_coupler, iqp2_sfc_psb,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(dlwbot_sfc(1,1,ksdm), c_flux_coupler, iqp2_sfc_dlwbot,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(dcupr_sfc(1,1,ksdm), c_flux_coupler, iqp2_sfc_dcupr,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(dlspr_sfc(1,1,ksdm), c_flux_coupler, iqp2_sfc_dlspr,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(dlsps_sfc(1,1,ksdm), c_flux_coupler, iqp2_sfc_dlsps,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(spdm_sfc(1,1,ksdm), c_flux_coupler, iqp2_sfc_spdm,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(shear1_sfc(1,1,ksdm), c_flux_coupler, iqp2_sfc_shear1,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(t_sfc(1,1,1,ksdm), c_flux_coupler, iqp2_sfc_tpblp2,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(t_sfc(1,1,2,ksdm), c_flux_coupler, iqp2_sfc_tpblp1,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(t_sfc(1,1,3,ksdm), c_flux_coupler, iqp2_sfc_tpbl,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(sh_sfc(1,1,1,ksdm), c_flux_coupler, iqp2_sfc_shpblp2,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(sh_sfc(1,1,2,ksdm), c_flux_coupler, iqp2_sfc_shpblp1,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(sh_sfc(1,1,3,ksdm), c_flux_coupler, iqp2_sfc_shpbl,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(zs_sfc(1,1,ksdm), c_flux_coupler, iqp2_sfc_zs,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(co2flx_sfc(1,1,ksdm), c_flux_coupler, iqp2_sfc_co2flx,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(shacu(1,1), c_flux_coupler, iqp2_sfc_shacu,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(lciinc(1,1), c_flux_coupler, iqp2_sfc_lciinc,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(safe(1,1), c_flux_coupler, iqp2_sfc_safe,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(eta(1,1), c_flux_coupler, iqp2_sfc_eta,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(negeinc(1,1), c_flux_coupler, iqp2_sfc_negeinc,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(tbp(1,1), c_flux_coupler, iqp2_sfc_tbp,
     *                  ksdm, iim_sfc*jjm_sfc)
        call qp2sum_sfc(wbp(1,1), c_flux_coupler, iqp2_sfc_wbp,
     *                  ksdm, iim_sfc*jjm_sfc)

!  Sib only diagnostics
        call qp2sum_sfc(www(1,1,1,ksdm),c_flux_coupler,iqp2_sfc_www1,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(www(1,1,2,ksdm),c_flux_coupler,iqp2_sfc_www2,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(runoff(1,1),c_flux_coupler,iqp2_sfc_mrro,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(mrros(1,1),c_flux_coupler,iqp2_sfc_mrros,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(mrso(1,1),c_flux_coupler*1000._dbl_kind,iqp2_sfc_mrso,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(mrfso(1,1),c_flux_coupler,iqp2_sfc_mrfso,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(mrsos(1,1),c_flux_coupler*1000._dbl_kind,iqp2_sfc_mrsos,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(snow(1,1,1,ksdm),c_flux_coupler*1000._dbl_kind,
     *                  iqp2_sfc_snw,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(snow(1,1,2,ksdm),c_flux_coupler*1000._dbl_kind,
     *                  iqp2_sfc_snw,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(www(1,1,3,ksdm),c_flux_coupler,iqp2_sfc_www3,
     *                  ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(snc(1,1),c_flux_coupler*100._dbl_kind,
     *                  iqp2_sfc_snc,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(snm(1,1),c_flux_coupler,
     *                  iqp2_sfc_snm,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(sic(1,1),c_flux_coupler*100._dbl_kind,
     *                  iqp2_sfc_sic,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(assimn(1,1),c_flux_coupler,
     *                  iqp2_sfc_assimn,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(respg(1,1),c_flux_coupler,
     *                  iqp2_sfc_respg,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(q3l(1,1),c_flux_coupler,
     *                  iqp2_sfc_q3l,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(q3o(1,1),c_flux_coupler,
     *                  iqp2_sfc_q3o,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(exo(1,1),c_flux_coupler,
     *                  iqp2_sfc_exo,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(eci(1,1),c_flux_coupler,
     *                  iqp2_sfc_eci,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(ect(1,1),c_flux_coupler,
     *                  iqp2_sfc_ect,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(egi(1,1),c_flux_coupler,
     *                  iqp2_sfc_egi,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(egs(1,1),c_flux_coupler,
     *                  iqp2_sfc_egs,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(tcas(1,1),c_flux_coupler,
     *                  iqp2_sfc_tcas,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(qcas(1,1),c_flux_coupler,
     *                  iqp2_sfc_qcas,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(rhcas(1,1),c_flux_coupler,
     *                  iqp2_sfc_rhcas,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(capac(1,1,1,ksdm),c_flux_coupler,
     *                  iqp2_sfc_capac1,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(capac(1,1,2,ksdm),c_flux_coupler,
     *                  iqp2_sfc_capac2,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(tcan(1,1,ksdm),c_flux_coupler,
     *                  iqp2_sfc_tcan,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(tg(1,1,ksdm),c_flux_coupler,
     *                  iqp2_sfc_tg,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(rstfac_diag(1,1,1),c_flux_coupler,
     *                  iqp2_sfc_rstfac1,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(rstfac_diag(1,1,2),c_flux_coupler,
     *                  iqp2_sfc_rstfac2,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(rstfac_diag(1,1,3),c_flux_coupler,
     *                  iqp2_sfc_rstfac3,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(rstfac_diag(1,1,4),c_flux_coupler,
     *                  iqp2_sfc_rstfac4,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(zlt(1,1,ksdm),c_flux_coupler,
     *                  iqp2_sfc_zlt,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(green(1,1,ksdm),c_flux_coupler,
     *                  iqp2_sfc_green,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(vcover(1,1,ksdm),c_flux_coupler,
     *                  iqp2_sfc_vcover,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(ra(1,1),c_flux_coupler,
     *                  iqp2_sfc_ra,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(rb(1,1),c_flux_coupler,
     *                  iqp2_sfc_rb,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(rd(1,1),c_flux_coupler,
     *                  iqp2_sfc_rd,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(gs(1,1,ksdm),c_flux_coupler,
     *                  iqp2_sfc_rst,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(chf(1,1),c_flux_coupler,
     *                  iqp2_sfc_chf,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(shf(1,1),c_flux_coupler,
     *                  iqp2_sfc_shf,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(radc31(1,1),c_flux_coupler,
     *                  iqp2_sfc_radc31,ksdm,iim_sfc*jjm_sfc)
        call qp2sum_sfc(radc32(1,1),c_flux_coupler,
     *                  iqp2_sfc_radc32,ksdm,iim_sfc*jjm_sfc)
        do n = 1,nsoil+1
          call qp2sum_sfc(soilscale(1,1,n),c_flux_coupler,
     *                  iqp2_sfc_soilscale(n),ksdm,iim_sfc*jjm_sfc)
        enddo

        call hf2sum_sfc(runoff(1,1),c_flux_coupler,ihf2_sfc_mrro,
     *                  ksdm,iim_sfc*jjm_sfc)
        call hf2sum_sfc(mrso(1,1),c_flux_coupler*1000._dbl_kind,ihf2_sfc_mrso,
     *                  ksdm,iim_sfc*jjm_sfc)
        call hf2sum_sfc(snow(1,1,1,ksdm),c_flux_coupler*1000._dbl_kind,
     *                  ihf2_sfc_snw,
     *                  ksdm,iim_sfc*jjm_sfc)
        call hf2sum_sfc(snow(1,1,2,ksdm),c_flux_coupler*1000._dbl_kind,
     *                  ihf2_sfc_snw,
     *                  ksdm,iim_sfc*jjm_sfc)

        call gmean_sum_sfc(snow(1,1,1,ksdm), iim_sfc*jjm_sfc, 1000._dbl_kind,
     *                  iglob_snd, ksdm )
        call gmean_sum_sfc(snow(1,1,2,ksdm), iim_sfc*jjm_sfc, 1000._dbl_kind,
     *                  iglob_snd, ksdm, step_sample=0 )
        call gmean_sum_sfc(snc(1,1), iim_sfc*jjm_sfc, 100._dbl_kind,
     *		iglob_snc, ksdm )
        call gmean_sum_sfc(sst(1,1,ksdm), iim_sfc*jjm_sfc, 1._dbl_kind,
     *		 	iglob_tso, ksdm, iz=iz(1,1,ksdm) )
#endif

! pbp diags
 	call pbp1sum_sfc(dtlci_sfc_copy(1,1,1),c_flux_coupler,ipbp1_sfc_dtlci1,
     *                  ksdm,iim_sfc*jjm_sfc)
 	call pbp1sum_sfc(dtlci_sfc_copy(1,1,2),c_flux_coupler,ipbp1_sfc_dtlci2,
     *                  ksdm,iim_sfc*jjm_sfc)
	call pbp1sum_sfc(dqlci_sfc_copy(1,1,1),c_flux_coupler,ipbp1_sfc_dqlci1,
     *                  ksdm,iim_sfc*jjm_sfc)
	call pbp1sum_sfc(dqlci_sfc_copy(1,1,2),c_flux_coupler,ipbp1_sfc_dqlci2,
     *                  ksdm,iim_sfc*jjm_sfc)
	call pbp1sum_sfc(ventmf(1,1),c_flux_coupler,ipbp1_sfc_ventmf,
     *                  ksdm,iim_sfc*jjm_sfc)
        call pbp1sum_sfc(obukhovl(1,1), c_flux_coupler, ipbp1_sfc_mol, 
     *                  ksdm, iim_sfc*jjm_sfc)
	call pbp1sum_sfc(ustar(1,1),c_flux_coupler,ipbp1_sfc_ustar,
     *                  ksdm,iim_sfc*jjm_sfc)
	call pbp1sum_sfc(z0d(1,1,ksdm),c_flux_coupler,ipbp1_sfc_z0,
     *                  ksdm,iim_sfc*jjm_sfc)
        call pbp1sum_sfc(buoyf(1,1), c_flux_coupler, ipbp1_sfc_buoyf, 
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(spd10(1,1), c_flux_coupler, ipbp1_sfc_spd10, 
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(t2(1,1), c_flux_coupler, ipbp1_sfc_t2, 
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(tke_sfc(1,1,ksdm),c_flux_coupler,ipbp1_sfc_tke,
     *                  ksdm,iim_sfc*jjm_sfc)
        call pbp1sum_sfc(fss_sfc(1,1,ksdm), c_flux_coupler, ipbp1_sfc_hfss, 
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(fws_sfc(1,1,ksdm), hltm*c_flux_coupler, ipbp1_sfc_hfls,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(dpent_sfc(1,1,ksdm), c_flux_coupler, ipbp1_sfc_blent, 
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(pc_sfc(1,1,ksdm),100.0_dbl_kind*c_flux_coupler,
     *                  ipbp1_sfc_clblp, ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(cldpc_sfc(1,1,ksdm), c_flux_coupler, ipbp1_sfc_clbl, 
     *                  ksdm, iim_sfc*jjm_sfc)
        qtem(:,:) = tgeff4_sfc(:,:,ksdm) **0.25
        call pbp1sum_sfc(qtem(1,1), c_flux_coupler, ipbp1_sfc_tradgrdeff, 
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(ps_sfc(1,1,ksdm), c_flux_coupler, ipbp1_sfc_ps,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(psb_sfc(1,1,ksdm), c_flux_coupler, ipbp1_sfc_psb,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(dlwbot_sfc(1,1,ksdm), c_flux_coupler, ipbp1_sfc_dlwbot,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(dcupr_sfc(1,1,ksdm), c_flux_coupler, ipbp1_sfc_dcupr,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(dlspr_sfc(1,1,ksdm), c_flux_coupler, ipbp1_sfc_dlspr,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(dlsps_sfc(1,1,ksdm), c_flux_coupler, ipbp1_sfc_dlsps,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(spdm_sfc(1,1,ksdm), c_flux_coupler, ipbp1_sfc_spdm,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(shear1_sfc(1,1,ksdm), c_flux_coupler, ipbp1_sfc_shear1,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(t_sfc(1,1,1,ksdm), c_flux_coupler, ipbp1_sfc_tpblp2,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(t_sfc(1,1,2,ksdm), c_flux_coupler, ipbp1_sfc_tpblp1,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(t_sfc(1,1,3,ksdm), c_flux_coupler, ipbp1_sfc_tpbl,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(sh_sfc(1,1,1,ksdm), c_flux_coupler, ipbp1_sfc_shpblp2,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(sh_sfc(1,1,2,ksdm), c_flux_coupler, ipbp1_sfc_shpblp1,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(sh_sfc(1,1,3,ksdm), c_flux_coupler, ipbp1_sfc_shpbl,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(zs_sfc(1,1,ksdm), c_flux_coupler, ipbp1_sfc_zs,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(co2flx_sfc(1,1,ksdm), c_flux_coupler, ipbp1_sfc_co2flx,
     *                  ksdm, iim_sfc*jjm_sfc)

!-- added by mb
        call pbp1sum_sfc(gprod(1,1), c_flux_coupler, ipbp1_sfc_gprod,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(cons(1,1), c_flux_coupler, ipbp1_sfc_cons,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(bync(1,1), c_flux_coupler, ipbp1_sfc_bync,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(shear(1,1), c_flux_coupler, ipbp1_sfc_totshear,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(lciinc(1,1), c_flux_coupler, ipbp1_sfc_lciinc,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(safe(1,1), c_flux_coupler, ipbp1_sfc_safe,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(tbp(1,1), c_flux_coupler, ipbp1_sfc_tbp,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(wbp(1,1), c_flux_coupler, ipbp1_sfc_wbp,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(thvgm(1,1), c_flux_coupler, ipbp1_sfc_thvgm,
     *                  ksdm, iim_sfc*jjm_sfc)

!  Sib only diagnostics
        call pbp1sum_sfc(www(1,1,1,ksdm),c_flux_coupler,ipbp1_sfc_www1,
     *                  ksdm,iim_sfc*jjm_sfc)
        call pbp1sum_sfc(www(1,1,2,ksdm),c_flux_coupler,ipbp1_sfc_www2,
     *                  ksdm,iim_sfc*jjm_sfc)
        call pbp1sum_sfc(runoff(1,1), c_flux_coupler,ipbp1_sfc_mrro,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(mrso(1,1), c_flux_coupler*1000._dbl_kind,
     *                  ipbp1_sfc_mrso,ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(snow(1,1,1,ksdm), c_flux_coupler*1000._dbl_kind,
     *                  ipbp1_sfc_snw,ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(snow(1,1,2,ksdm), c_flux_coupler*1000._dbl_kind,
     *                  ipbp1_sfc_snw,ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(www(1,1,3,ksdm),c_flux_coupler,ipbp1_sfc_www3,
     *                  ksdm,iim_sfc*jjm_sfc)
        call pbp1sum_sfc(assimn(1,1), c_flux_coupler, ipbp1_sfc_assimn,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(respg(1,1), c_flux_coupler, ipbp1_sfc_respg,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(q3l(1,1), c_flux_coupler, ipbp1_sfc_q3l,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(q3o(1,1), c_flux_coupler, ipbp1_sfc_q3o,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(exo(1,1), c_flux_coupler, ipbp1_sfc_exo,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(eci(1,1), c_flux_coupler, ipbp1_sfc_eci,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(ect(1,1), c_flux_coupler, ipbp1_sfc_ect,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(egi(1,1), c_flux_coupler, ipbp1_sfc_egi,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(egs(1,1), c_flux_coupler, ipbp1_sfc_egs,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(tcas(1,1), c_flux_coupler, ipbp1_sfc_tcas,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(qcas(1,1), c_flux_coupler, ipbp1_sfc_qcas,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(rhcas(1,1), c_flux_coupler, ipbp1_sfc_rhcas,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(capac(1,1,1,ksdm), c_flux_coupler*1000._dbl_kind,
     *                   ipbp1_sfc_capac1,ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(capac(1,1,2,ksdm), c_flux_coupler*1000._dbl_kind,
     *                   ipbp1_sfc_capac2,ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(tcan(1,1,ksdm), c_flux_coupler, ipbp1_sfc_tcan,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(tg(1,1,ksdm), c_flux_coupler, ipbp1_sfc_tg,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(rstfac_diag(1,1,1), c_flux_coupler,
     *                  ipbp1_sfc_rstfac1, ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(rstfac_diag(1,1,2), c_flux_coupler,
     *                  ipbp1_sfc_rstfac2, ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(rstfac_diag(1,1,3), c_flux_coupler,
     *                  ipbp1_sfc_rstfac3, ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(rstfac_diag(1,1,4), c_flux_coupler,
     *                  ipbp1_sfc_rstfac4, ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(zlt(1,1,ksdm), c_flux_coupler, ipbp1_sfc_zlt,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(green(1,1,ksdm), c_flux_coupler, ipbp1_sfc_green,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(vcover(1,1,ksdm), c_flux_coupler, ipbp1_sfc_vcover,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(ra(1,1), c_flux_coupler, ipbp1_sfc_ra,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(rb(1,1), c_flux_coupler, ipbp1_sfc_rb,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(rd(1,1), c_flux_coupler, ipbp1_sfc_rd,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(gs(1,1,ksdm), c_flux_coupler, ipbp1_sfc_rst,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(chf(1,1), c_flux_coupler, ipbp1_sfc_chf,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(shf(1,1), c_flux_coupler, ipbp1_sfc_shf,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(radc31(1,1), c_flux_coupler, ipbp1_sfc_radc31,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(radc32(1,1), c_flux_coupler, ipbp1_sfc_radc32,
     *                  ksdm, iim_sfc*jjm_sfc)
        call pbp1sum_sfc(zb(1,1), c_flux_coupler, ipbp1_sfc_dzm,
     *                  ksdm, iim_sfc*jjm_sfc)

      endif



      end subroutine sfc_control

      end module sfc_control_module
