!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module sfc_init

!-----------------------------------------------------------------------
! PURPOSE:
! This file initializes the fields for the ocean,landice,and sib
! It reads in both initial conditions and boundary conditions
!-----------------------------------------------------------------------

      use kinds
      use model_parameters
      use pbl_parameters
      use domain_decomposition_sfc
      use fluxcoupler_fields_sfc
      use time_manager_module

      implicit none
      
      private

      real (kind=real_kind) :: lat_tem, lon_tem
      integer (kind=int_kind) :: ioerr
      real (kind=selected_real_kind(6)), dimension(:), allocatable ::  xrec4
      real (kind=selected_real_kind(13)), dimension(:), allocatable ::  xrec8

!-----------------------------------------------------------------------
! contains
!	init_sfc_variables
!	init_ocnflx
!	init_landice
!	init_sib
!-----------------------------------------------------------------------

      public :: init_sfc_variables, init_ocnflx, init_landice, init_sib,
     *          init_coupler

!***********************************************************************

      contains

!***********************************************************************


!======================================================================
! BEGIN SUBROUTINE INIT_SFC_VARIABLES
!======================================================================
      subroutine init_sfc_variables

      implicit none

!-----------------------------------------------------------------------
! initialize flux surface coupler fields
!-----------------------------------------------------------------------
        alvdr_sfc = c0
        alndf_sfc = c0
        alvdr_sfc = c0
        alndr_sfc = c0
        tgeff4_sfc = c0
        pc_sfc = c0
        cldpc_sfc = c0
        lwp_sfc = c0
        pblcw_sfc = c0
        pblci_sfc = c0
        radvdc_sfc = c0
        radndc_sfc = c0
        radvbc_sfc = c0
        radnbc_sfc = c0
        dlwbot_sfc = c0
        fluxd_sfc = c0
        dcupr_sfc = c0
        dlspr_sfc = c0
        dlsps_sfc = c0
        spdm_sfc = c0
        shear1_sfc = c0
        ps_sfc = c0
        psb_sfc = c0
        dpent_sfc = c0
        co2flx_sfc = c0

      end subroutine init_sfc_variables
!======================================================================
! END SUBROUTINE INIT_SFC_VARIABLES
!======================================================================


!======================================================================
! BEGIN SUBROUTINE INIT_OCNFLX
!======================================================================
      subroutine init_ocnflx(frac_sfc)

! This subroutine initializes the fields for the ocean surface flux
!    parameterization

      use ocean_parameters

      implicit none

      real (kind=real_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc)
     *,    intent(out) :: frac_sfc

!-----------------------------------------------------------------------
! local variables
!-----------------------------------------------------------------------
      integer (kind=int_kind) :: i, j, ksdm
     *,     max_ig_tem, lrecl, nocean_tem, nsecond_tem
      logical (kind=log_kind) :: region_flag=.false.

!-----------------------------------------------------------------------
!   ocean grid fields
!-----------------------------------------------------------------------
      real (kind=real_kind) ::  area_tem, wght_tem
      real (kind=real_kind), dimension(:), allocatable ::  xrec

!-----------------------------------------------------------------------
!    read in the gridmap file header
!-----------------------------------------------------------------------
        open(unit=3,file=trim(datapath)//'data/gridmap_ocean',form='unformatted'
     *,      access='DIRECT',recl=12,status='old',action='read'
     *,      iostat=ioerr)
        if (ioerr /= 0) then
          write(*,*) '*** init_ocnflx: gridmap_ocean file error ',ioerr
          stop
        endif
        read(3,rec=1)nocean,max_ig_tem,lrecl
        close(3)

       if(my_task_sfc.eq.0) then
	write(6,*)
	write(6,*) ' READING IN OCEAN COMPONENT DATA'
	write(6,*) ' number of ocean points ',nocean,max_ig_tem
       endif

        frac_sfc = 0.0
!-----------------------------------------------------------------------
! if there are no ocean points, return
!-----------------------------------------------------------------------
        if(nocean.eq.0) then
         hr_sea = .false.
         return
        endif

!-----------------------------------------------------------------------
!    read in the gridmap file data
!-----------------------------------------------------------------------
        lon_sea = c0
        lat_sea = c0
        open(unit=3,file=trim(datapath)//'data/gridmap_ocean',form='unformatted'
     *,      access='DIRECT',recl=lrecl)
        do ksdm = 1,nsdm_sfc
          do j = 1,jjm_sfc
            do i = 1,iim_sfc
	      if(grid_mask_sfc(i,j,ksdm)) then
                read(3,rec=1+grid_indx_sfc(i,j,ksdm))lon_tem,
     *             lat_tem, area_tem
     *,            wght_tem, link_hr_sea(i,j,ksdm), hr_sea(i,j,ksdm)
	        if(hr_sea(i,j,ksdm)) then
  		  region_flag = .true.
                  lon_sea(i,j,ksdm) = lon_tem
                  lat_sea(i,j,ksdm) = lat_tem
                  area_sfc(i,j,ksdm) = area_tem
	        else
		  link_hr_sea(i,j,ksdm) = -10
	        endif
	      endif
            enddo
          enddo
        enddo

        close(3)

	if(maxval(lon_sea).gt.20.and.maxval(lat_sea).gt.20) then
	  write(6,*) ' wrong units'
	  write(6,*) maxval(lon_sea),maxval(lat_sea)
	  stop
	endif

!-----------------------------------------------------------------------
!     mask out wrapped ocean points
!-----------------------------------------------------------------------
      hr_sea(:,:,:) = hr_sea(:,:,:).and.grid_mask_sfc(:,:,:)

!-----------------------------------------------------------------------
!    Read in the initial condition file
!      These fields are defined only at ocean points, and will be 
!      scattered to the global grid
!-----------------------------------------------------------------------
        open(unit=4,file=trim(datapath)//'data/ic_ocean',form='unformatted'
     *,      access='DIRECT', recl=16,status='old',action='read'
     *,      iostat=ioerr)
        if (ioerr /= 0) then
          write(*,*) '*** init_ocnflx: ic_ocean file error ',ioerr
          stop
        endif
        read(4,rec=1)nsecond_tem,nocean_tem,max_ig_tem,lrecl
        close(4)

        open(unit=4,file=trim(datapath)//'data/ic_ocean',form='unformatted'
     *,      access='DIRECT', recl=lrecl)
!-----------------------------------------------------------------------
! read in data at ocean points
!-----------------------------------------------------------------------
        if(lrecl.eq.24) then
          if(my_task.eq.0)print *,'reading real*4 ic_ocean'
          allocate(xrec4(6))
          do ksdm = 1,nsdm_sfc
            do j = 1,jjm_sfc
              do i = 1,iim_sfc
                if(hr_sea(i,j,ksdm)) then
                   read(4,rec=1+link_hr_sea( i,j,ksdm) ) xrec4
                   sst(i,j,ksdm)  = xrec4(1)
                   ti(i,j,ksdm)   = xrec4(2)
                   sn(i,j,ksdm)   = xrec4(3)
                   iz(i,j,ksdm)   = xrec4(4)
                   e2(i,j,ksdm)   = xrec4(5)
                   tkeo(i,j,ksdm) = xrec4(6)
	        endif
              enddo
            enddo
          enddo
          deallocate(xrec4)
        elseif(lrecl.eq.48) then
          if(my_task.eq.0)print *,'reading real*8 ic_ocean'
          allocate(xrec8(6))
          do ksdm = 1,nsdm_sfc
            do j = 1,jjm_sfc
              do i = 1,iim_sfc
                if(hr_sea(i,j,ksdm)) then
                   read(4,rec=1+link_hr_sea( i,j,ksdm) ) xrec8
                   sst(i,j,ksdm)  = xrec8(1)
                   ti(i,j,ksdm)   = xrec8(2)
                   sn(i,j,ksdm)   = xrec8(3)
                   iz(i,j,ksdm)   = xrec8(4)
                   e2(i,j,ksdm)   = xrec8(5)
                   tkeo(i,j,ksdm) = xrec8(6)
	        endif
              enddo
            enddo
          enddo
          deallocate(xrec8)
        endif

!-----------------------------------------------------------------------
! close file containing initial conditions
!-----------------------------------------------------------------------
      close(4)
      if(my_task_sfc.eq.0) write(6,*) ' ocean ic read in'

!-----------------------------------------------------------------------
! INITIAL CONDITIONS HAVE BEEN READ IN
!-----------------------------------------------------------------------


!-----------------------------------------------------------------------
!   read in the boundary conditions
!      These fields are defined only at ocean points, and will be 
!      scattered to the global grid
!-----------------------------------------------------------------------
        open(unit=3,file=trim(datapath)//'data/bc_ocean',form='unformatted'
     *,      access='DIRECT', recl=12,status='old',action='read'
     *,      iostat=ioerr)
        if (ioerr /= 0) then
          write(*,*) '*** init_ocnflx: bc_ocean file error ',ioerr
          stop
        endif
        read(3,rec=1)nocean_tem,max_ig_tem,lrecl
        close(3)

        open(unit=3,file=trim(datapath)//'data/bc_ocean',form='unformatted'
     *,      access='DIRECT', recl = lrecl)

        nmonth_bc = (lrecl/kind(xrec) - (2+12*3) ) / 2
        allocate(xrec(2+12*3+2*nmonth_bc))
        allocate(sstm(iim_sfc, jjm_sfc, nmonth_bc, nsdm_sfc))
        allocate(izm(iim_sfc, jjm_sfc, nmonth_bc, nsdm_sfc))
        sstm = 0
        izm = 0
        if(nmonth_bc.gt.12) read(3,rec=1)nocean_tem,max_ig_tem,lrecl,nyear0
        do ksdm = 1,nsdm_sfc
          do j = 1,jjm_sfc
            do i = 1,iim_sfc
              if(hr_sea(i,j,ksdm)) then
                 read(3,rec=1+link_hr_sea( i,j,ksdm ) ) xrec
                    zs_sea(i,j,ksdm) = xrec(1)
                    hmax(i,j,ksdm) = xrec(2)
                    sstm(i,j,:,ksdm) = xrec(3:2+nmonth_bc)
                    izm(i,j,:,ksdm) = xrec(3+nmonth_bc:2+2*nmonth_bc)
                    hmix(i,j,:,ksdm) = xrec(3+2*nmonth_bc:14+2*nmonth_bc)
                    qfm(i,j,:,ksdm) = xrec(15+2*nmonth_bc:26+2*nmonth_bc)
                    e2m(i,j,:,ksdm) = xrec(27+2*nmonth_bc:38+2*nmonth_bc)
               endif
            enddo
          enddo
        enddo
        deallocate(xrec)

!-----------------------------------------------------------------------
! close boundary conditions file
!-----------------------------------------------------------------------
       close(3)
      if(my_task_sfc.eq.0) write(6,*) ' ocean bc read in'

!-----------------------------------------------------------------------
! BOUNDARY CONDITIONS HAVE BEEN READ IN
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! compute lat,lon, and surface height (gpm)
!-----------------------------------------------------------------------
        where(hr_sea)
              !zs_sfc(:,:,:)  = zs_sea(:,:,:)
              lon_sfc(:,:,:) = lon_sea(:,:,:)
              lat_sfc(:,:,:) = lat_sea(:,:,:)
              frac_sfc(:,:,:) = 1.0
        endwhere

!-----------------------------------------------------------------------
! dump some diagnostic info to screen as a sanity check
!-----------------------------------------------------------------------
      do i=1,npe_sfc
      write(6,*)
      if(my_task_sfc.eq.i-1) then
      write(6,*) ' SUMMARY OF OCEAN INIT AND BC for task ',my_task_sfc
      if(region_flag) then
        write(6,*) '    my_task   : ', my_task_sfc
        write(6,*) '    sst       : ', minval(sst,mask=hr_sea),
     *                                 maxval(sst,mask=hr_sea)
        write(6,*) '    tke       : ', minval(tkeo,mask=hr_sea),
     *                                 maxval(tkeo,mask=hr_sea)
        write(6,*) '    ice temp  : ', minval(ti,mask=hr_sea),
     *                                 maxval(ti,mask=hr_sea)
        write(6,*) '    ice thick : ', minval(iz,mask=hr_sea),
     *                                 maxval(iz,mask=hr_sea)
        write(6,*) '    zs        : ', minval(zs_sea,mask=hr_sea),
     *                                 maxval(zs_sea,mask=hr_sea)
        write(6,*) '    link      : ', minval(link_hr_sea,mask=hr_sea),
     *                                 maxval(link_hr_sea,mask=hr_sea)
      else
        write(6,*) '   no ocean points are owned by this process'
      endif
      endif
      enddo
#ifdef mpi
!     CALL MPI_BARRIER (MPI_COMM_SFC,ierr)
#endif

      end subroutine init_ocnflx
!======================================================================
! END OF INIT_OCNFLX
!======================================================================


!======================================================================
! BEGINNING OF INIT_LANDICE
!======================================================================
      subroutine init_landice(frac_sfc)

! This subroutine initializes the fields for the land ice surface flux
!    parameterization

      use landice_parameters

      implicit none

      real (kind=real_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc)
     *,    intent(out) :: frac_sfc

!-----------------------------------------------------------------------
! local variables
!-----------------------------------------------------------------------
      integer (kind=int_kind) :: i, j, ksdm
     *,     max_ig_tem, lrecl, nice_tem, nsecond_tem
      logical (kind=log_kind) :: region_flag=.false.

!-----------------------------------------------------------------------
!   ice grid fields
!-----------------------------------------------------------------------
      real (kind=real_kind) :: area_tem, wght_tem, zs_tem

!-----------------------------------------------------------------------
!    read in the gridmap file header
!-----------------------------------------------------------------------
        open(unit=3,file=trim(datapath)//'data/gridmap_landice',
     *       form='unformatted'
     *,      access='DIRECT',recl=12, status='old', action='read'
     *,      iostat=ioerr)
        if (ioerr /= 0) then
          write(*,*) '*** init_landice: gridmap_landice file error '
     *,     ioerr
          stop
        endif
        read(3,rec=1)nlandice,max_ig_tem,lrecl
        close(3)

       if(my_task_sfc.eq.0) then
	write(6,*)
	write(6,*) ' READING IN ICE COMPONENT DATA'
	write(6,*) ' number of ice points ',nlandice
       endif

        frac_sfc = 0.0
!-----------------------------------------------------------------------
! if there are no ice points, return
!-----------------------------------------------------------------------
        if(nlandice.eq.0) then
         hr_ice = .false.
         return
        endif

!-----------------------------------------------------------------------
!    read in the gridmap file data
!-----------------------------------------------------------------------
        lon_ice = c0
        lat_ice = c0
        open(unit=3,file=trim(datapath)//'data/gridmap_landice',
     *       form='unformatted'
     *,      access='DIRECT',recl=lrecl)
        do ksdm = 1,nsdm_sfc
          do j = 1,jjm_sfc
            do i = 1,iim_sfc
	      if(grid_mask_sfc(i,j,ksdm)) then
              read(3,rec=1+grid_indx_sfc(i,j,ksdm))lon_tem,
     *             lat_tem, area_tem
     *,            wght_tem, link_hr_ice(i,j,ksdm), hr_ice(i,j,ksdm)
	      if(hr_ice(i,j,ksdm)) then
	 	region_flag = .true.
                lon_ice(i,j,ksdm) = lon_tem
                lat_ice(i,j,ksdm) = lat_tem
                  area_sfc(i,j,ksdm) = area_tem
	      else
	        link_hr_ice(i,j,ksdm) = -10
	      endif
	      endif
            enddo
          enddo
        enddo

        close(3)

        if(maxval(lon_ice).gt.20.and.maxval(lat_ice).gt.20) then
          write(6,*) ' wrong units'
          stop
        endif

!-----------------------------------------------------------------------
!     mask out wrapped ice points
!-----------------------------------------------------------------------
      hr_ice(:,:,:) = hr_ice(:,:,:).and.grid_mask_sfc(:,:,:)

!-----------------------------------------------------------------------
!    Read in the initial condition file
!      These fields are defined only at ice points, and will be 
!      scattered to the global grid
!-----------------------------------------------------------------------
        open(unit=4,file=trim(datapath)//'data/ic_landice',form='unformatted'
     *,      access='DIRECT', recl=16,status='old',action='read'
     *,      iostat=ioerr)
        if (ioerr /= 0) then
          write(*,*) '*** init_landice: ic_landice file error ',ioerr
          stop
        endif
        read(4,rec=1)nsecond_tem,nice_tem,max_ig_tem,lrecl
        close(4)

        open(unit=4,file=trim(datapath)//'data/ic_landice',form='unformatted'
     *,      access='DIRECT', recl=lrecl)
!-----------------------------------------------------------------------
! read in data at ice points
!-----------------------------------------------------------------------
        allocate(xrec4(2))
        allocate(xrec8(2))
        read(4,rec=2)xrec4
        read(4,rec=2)xrec8
        if(xrec4(1).gt.1.e02 .and. xrec4(1).lt.3.e02) then
          if(my_task.eq.0)print *,'reading real*4 ic_landice'
          do ksdm = 1,nsdm_sfc
            do j = 1,jjm_sfc
              do i = 1,iim_sfc
                if(hr_ice(i,j,ksdm)) then
                   read(4,rec=1+link_hr_ice( i,j,ksdm) ) xrec4
                   tgi(i,j,ksdm)   = xrec4(1)
                   tkei(i,j,ksdm) = xrec4(2)
	        endif
              enddo
            enddo
          enddo
        elseif(xrec8(1).gt.1.d02 .and. xrec8(1).lt.3.d02) then
          if(my_task.eq.0)print *,'reading real*8 ic_landice'
          do ksdm = 1,nsdm_sfc
            do j = 1,jjm_sfc
              do i = 1,iim_sfc
                if(hr_ice(i,j,ksdm)) then
                   read(4,rec=1+link_hr_ice( i,j,ksdm) ) xrec8
                   tgi(i,j,ksdm)   = xrec8(1)
                   tkei(i,j,ksdm) = xrec8(2)
	        endif
              enddo
            enddo
          enddo
        else
          if(my_task.eq.0)print *,'cant identify type of ic_landice - stopping'
          stop
        endif
        deallocate(xrec8)
        deallocate(xrec4)

!-----------------------------------------------------------------------
! close file containing initial conditions
!-----------------------------------------------------------------------
      close(4)
      if(my_task_sfc.eq.0) write(6,*) ' landice ic read in'

!-----------------------------------------------------------------------
! INITIAL CONDITIONS HAVE BEEN READ IN
!-----------------------------------------------------------------------


!-----------------------------------------------------------------------
!   read in the boundary conditions
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!    Read in the boundary condition file
!      These fields are defined only at ice points, and will be 
!      scattered to the global grid
!-----------------------------------------------------------------------
        open(unit=3,file=trim(datapath)//'data/bc_landice',form='unformatted'
     *,      access='DIRECT', recl=12,status='old',action='read'
     *,      iostat=ioerr)
        if (ioerr /= 0) then
          write(*,*) '*** init_landice: bc_landice file error ',ioerr
          stop
        endif
        read(3,rec=1)nice_tem,max_ig_tem,lrecl
        close(3)

        open(unit=3,file=trim(datapath)//'data/bc_landice',form='unformatted'
     *,      access='DIRECT', recl=lrecl)
!-----------------------------------------------------------------------
! read in data at ice points
!-----------------------------------------------------------------------
        do ksdm = 1,nsdm_sfc
          do j = 1,jjm_sfc
            do i = 1,iim_sfc
              if(hr_ice(i,j,ksdm)) then
                 read(3,rec=1+link_hr_ice( i,j,ksdm) )zs_tem
                 zs_ice(i,j,ksdm) = zs_tem
              endif
            enddo
          enddo
        enddo

!-----------------------------------------------------------------------
! close file containing boundary conditions
!-----------------------------------------------------------------------
      close(3)
      if(my_task_sfc.eq.0) write(6,*) ' landice bc read in'

!-----------------------------------------------------------------------
! compute lat,lon, and surface height (gpm)
!-----------------------------------------------------------------------
        where(hr_ice)
              !zs_sfc(:,:,:) = zs_ice(:,:,:)
              lon_sfc(:,:,:) = lon_ice(:,:,:)
              lat_sfc(:,:,:) = lat_ice(:,:,:)
              frac_sfc(:,:,:) = 1.0
        endwhere

!-----------------------------------------------------------------------
! dump some diagnostic info to screen as a sanity check
!-----------------------------------------------------------------------
      do i=1,npe_sfc
      write(6,*)
      if(my_task_sfc.eq.i-1) then
      write(6,*) ' SUMMARY OF ICE INIT AND BC for task ',my_task_sfc
      if(region_flag) then
        write(6,*) '    my_task: ', my_task_sfc
        write(6,*) '    tgi    : ', minval(tgi,mask=hr_ice),
     *                            maxval(tgi,mask=hr_ice)
        write(6,*) '    tke    : ', minval(tkei,mask=hr_ice),
     *                            maxval(tkei,mask=hr_ice)
        write(6,*) '    zs     : ', minval(zs_ice,mask=hr_ice),
     *                            maxval(zs_ice,mask=hr_ice)
        write(6,*) '    link   : ', minval(link_hr_ice,mask=hr_ice),
     *                            maxval(link_hr_ice,mask=hr_ice)
      else
        write(6,*) '   no ice points are owned by this process'
      endif
      endif
      enddo
#ifdef mpi
!     CALL MPI_BARRIER (MPI_COMM_SFC,ierr)
#endif

!-----------------------------------------------------------------------
! BOUNDARY CONDITIONS HAVE BEEN READ IN
!-----------------------------------------------------------------------

      end subroutine init_landice
!======================================================================
! END OF INIT_LANDICE
!======================================================================


!======================================================================
! BEGINNING OF INIT_SIB
!======================================================================
      subroutine init_sib(frac_sfc)

! This subroutine initializes the fields for the SIB surface flux
!    parameterization

      use sib_parameters

      implicit none

      real (kind=real_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc)
     *,    intent(out) :: frac_sfc

!-----------------------------------------------------------------------
! local variables
!-----------------------------------------------------------------------
      integer (kind=int_kind) :: i, j, ksdm, k, m
     *,     max_ig_tem, lrecl, nsib_tem, nsecond_tem, nsoil_tem
     *,     npt_tem
      logical (kind=log_kind) :: region_flag=.false.

!-----------------------------------------------------------------------
!   sib grid fields
!-----------------------------------------------------------------------
      real (kind=real_kind) :: area_tem, wght_tem
      real (kind=real_kind), dimension(:), allocatable :: xrec

!-----------------------------------------------------------------------
!    read in the gridmap file header
!-----------------------------------------------------------------------
        lon_sib = c0
        lat_sib = c0
        open(unit=3,file=trim(datapath)//'data/gridmap_sib',form='unformatted'
     *,      access='DIRECT',recl=12,status='old',action='read'
     *,      iostat=ioerr)
        if (ioerr /= 0) then
          write(*,*) '*** init_sib: gridmap_sib file error ',ioerr
          stop
        endif
        read(3,rec=1)nsib,max_ig_tem,lrecl
        close(3)

       if(my_task_sfc.eq.0) then
	write(6,*)
	write(6,*) ' READING IN SIB COMPONENT DATA'
	write(6,*) ' number of sib points ',nsib
       endif

        frac_sfc = 0.0
!-----------------------------------------------------------------------
! if there are no vegetated land points, return
!-----------------------------------------------------------------------
        if(nsib.eq.0) then
         hr_sib = .false.
         return
        endif

!-----------------------------------------------------------------------
!    read in the gridmap file data
!-----------------------------------------------------------------------
        open(unit=3,file=trim(datapath)//'data/gridmap_sib',form='unformatted'
     *,      access='DIRECT',recl=lrecl)
        do ksdm = 1,nsdm_sfc
          do j = 1,jjm_sfc
            do i = 1,iim_sfc
	      if(grid_mask_sfc(i,j,ksdm)) then
              read(3,rec=1+grid_indx_sfc(i,j,ksdm))lon_tem,
     *             lat_tem, area_tem
     *,            wght_tem, link_hr_sib(i,j,ksdm), hr_sib(i,j,ksdm)
	      if(hr_sib(i,j,ksdm)) then
		region_flag = .true.
                lon_sib(i,j,ksdm) = lon_tem
                lat_sib(i,j,ksdm) = lat_tem
                  area_sfc(i,j,ksdm) = area_tem
	      else
		link_hr_sib(i,j,ksdm) = -10
	      endif
	      endif
            enddo
          enddo
        enddo

        close(3)

        if(maxval(lon_sib).gt.20.and.maxval(lat_sib).gt.20) then
          write(6,*) ' wrong units',maxval(lon_sib),maxval(lat_sib)
          stop
        endif

!-----------------------------------------------------------------------
!     mask out wrapped sib points
!-----------------------------------------------------------------------
      hr_sib(:,:,:) = hr_sib(:,:,:).and.grid_mask_sfc(:,:,:)

!-----------------------------------------------------------------------
!    Read in the initial condition file
!      These fields are defined only at sib points, and will be 
!      scattered to the global grid
!-----------------------------------------------------------------------
        open(unit=4,file=trim(datapath)//'data/ic_sib',form='unformatted'
     *,      access='DIRECT', recl=20,status='old',action='read'
     *,      iostat=ioerr)
        if (ioerr /= 0) then
          write(*,*) '*** init_sib: ic_sib file error ',ioerr
          stop
        endif
        read(4,rec=1)nsecond_tem,nsib_tem,nsoil_tem,max_ig_tem,lrecl
        close(4)

        open(unit=4,file=trim(datapath)//'data/ic_sib',form='unformatted'
     *,      access='DIRECT', recl=lrecl)
!-----------------------------------------------------------------------
! read in data at sib points
!-----------------------------------------------------------------------
        if(lrecl.eq.4*(11+nsoil)) then
          if(my_task.eq.0)print *,'reading real*4 ic_sib'
          allocate(xrec4((11+nsoil)))
          do ksdm = 1,nsdm_sfc
            do j = 1,jjm_sfc
              do i = 1,iim_sfc
                if(hr_sib(i,j,ksdm)) then
                   read(4,rec=1+link_hr_sib( i,j,ksdm) ) xrec4
                   tcan(i,j,ksdm)    = xrec4(1)
                   tg(i,j,ksdm)      = xrec4(2)
                   deept(i,j,:,ksdm) = xrec4(3:2+nsoil)
                   capac(i,j,:,ksdm) = xrec4(3+nsoil:4+nsoil)
                   snow(i,j,:,ksdm)  = xrec4(5+nsoil:6+nsoil)
                   www(i,j,:,ksdm)   = xrec4(7+nsoil:9+nsoil)
                   gs(i,j,ksdm)      = xrec4(10+nsoil)
                   tkes(i,j,ksdm)    = xrec4(11+nsoil)
	        endif
              enddo
            enddo
          enddo
          deallocate(xrec4)
        elseif(lrecl.eq.8*(11+nsoil)) then
          if(my_task.eq.0)print *,'reading real*8 ic_sib'
          allocate(xrec8((11+nsoil)))
          do ksdm = 1,nsdm_sfc
            do j = 1,jjm_sfc
              do i = 1,iim_sfc
                if(hr_sib(i,j,ksdm)) then
                   read(4,rec=1+link_hr_sib( i,j,ksdm) ) xrec8
                   tcan(i,j,ksdm)    = xrec8(1)
                   tg(i,j,ksdm)      = xrec8(2)
                   deept(i,j,:,ksdm) = xrec8(3:2+nsoil)
                   capac(i,j,:,ksdm) = xrec8(3+nsoil:4+nsoil)
                   snow(i,j,:,ksdm)  = xrec8(5+nsoil:6+nsoil)
                   www(i,j,:,ksdm)   = xrec8(7+nsoil:9+nsoil)
                   gs(i,j,ksdm)      = xrec8(10+nsoil)
                   tkes(i,j,ksdm)    = xrec8(11+nsoil)
	        endif
              enddo
            enddo
          enddo
          deallocate(xrec8)
        endif

!-----------------------------------------------------------------------
! close file containing initial conditions
!-----------------------------------------------------------------------
      close(4)
      if(my_task_sfc.eq.0) write(6,*) ' sib ic read in'

!-----------------------------------------------------------------------
! INITIAL CONDITIONS HAVE BEEN READ IN
!-----------------------------------------------------------------------


!-----------------------------------------------------------------------
!   read in the boundary conditions
!      These fields are defined only at sib points, and will be 
!      scattered to the global grid
!-----------------------------------------------------------------------
        open(unit=3,file=trim(datapath)//'data/bc_sib',form='unformatted'
     *,      access='DIRECT', recl=16,status='old',action='read'
     *,      iostat=ioerr)
        if (ioerr /= 0) then
          write(*,*) '*** init_sib: bc_sib file error ',ioerr
          stop
        endif
        read(3,rec=1)nsib_tem,nsoil_tem,max_ig_tem,lrecl
        close(3)

        open(unit=3,file=trim(datapath)//'data/bc_sib',form='unformatted'
     *,      access='DIRECT', recl = lrecl)

        allocate(xrec(40+8*12))
        do ksdm = 1,nsdm_sfc
          do j = 1,jjm_sfc
            do i = 1,iim_sfc
              if(hr_sib(i,j,ksdm)) then
                 read(3,rec=1+link_hr_sib( i,j,ksdm ) ) xrec
                 z2	(i,j,ksdm) = xrec(1)
                 z1	(i,j,ksdm) = xrec(2)
                 vcover	(i,j,ksdm) = xrec(3)
                 chil	(i,j,ksdm) = xrec(4)
                 sodep	(i,j,ksdm) = xrec(5)
                 rootd	(i,j,ksdm) = xrec(6)
                 phc	(i,j,ksdm) = xrec(7)
                 tran	(i,j,1,1,ksdm) = xrec(8)
                 tran	(i,j,2,1,ksdm) = xrec(9)
                 tran	(i,j,1,2,ksdm) = xrec(10)
                 tran	(i,j,2,2,ksdm) = xrec(11)
                 ref	(i,j,1,1,ksdm) = xrec(12)
                 ref	(i,j,2,1,ksdm) = xrec(13)
                 ref	(i,j,1,2,ksdm) = xrec(14)
                 ref	(i,j,2,2,ksdm) = xrec(15)
                 vmax0	(i,j,ksdm) = xrec(16)
                 effcon	(i,j,ksdm) = xrec(17)
                 gradm	(i,j,ksdm) = xrec(18)
                 binter	(i,j,ksdm) = xrec(19)
                 atheta	(i,j,ksdm) = xrec(20)
                 btheta	(i,j,ksdm) = xrec(21)
                 trda	(i,j,ksdm) = xrec(22)
                 trdm	(i,j,ksdm) = xrec(23)
                 trop	(i,j,ksdm) = xrec(24)
                 respcp	(i,j,ksdm) = xrec(25)
                 slti	(i,j,ksdm) = xrec(26)
                 shti	(i,j,ksdm) = xrec(27)
                 hltii	(i,j,ksdm) = xrec(28)
                 hhti	(i,j,ksdm) = xrec(29)
                 soref	(i,j,:,ksdm) = xrec(30:31)
                 bee	(i,j,ksdm) = xrec(32)
                 phsat	(i,j,ksdm) = xrec(33)
                 satco	(i,j,ksdm) = xrec(34)
                 poros	(i,j,ksdm) = xrec(35)
                 slope	(i,j,ksdm) = xrec(36)
                 wopt	(i,j,ksdm) = xrec(37)
                 wsat	(i,j,ksdm) = xrec(38)
                 zm	(i,j,ksdm) = xrec(39)
                 zs_sib	(i,j,ksdm) = xrec(40)
                 aparc_m	(i,j,:,ksdm) = xrec(41:52)
                 zlt_m	(i,j,:,ksdm) = xrec(53:64)
                 green_m	(i,j,:,ksdm) = xrec(65:76)
                 z0d_m	(i,j,:,ksdm) = xrec(77:88)
                 dd_m	(i,j,:,ksdm) = xrec(89:100)
                 rbc_m	(i,j,:,ksdm) = xrec(101:112)
                 rdc_m	(i,j,:,ksdm) = xrec(113:124)
                 gmudmu_m	(i,j,:,ksdm) = xrec(125:136)
               endif
            enddo
          enddo
        enddo
        deallocate(xrec)

!-----------------------------------------------------------------------
! close boundary conditions file
!-----------------------------------------------------------------------
       close(3)
      if(my_task_sfc.eq.0) write(6,*) ' sib bc read in'

!-----------------------------------------------------------------------
!   prescribed soil moisture
!-----------------------------------------------------------------------
      if(prescribed_gw) then
          open(unit=3,file=trim(datapath)//'data/sib_www',form='unformatted'
     *,        access='DIRECT',recl=12,status='old',action='read'
     *,        iostat=ioerr)
          if (ioerr /= 0) then
            write(*,*) '*** init_sib: sib_www file error ',ioerr
            stop
          endif
          read(3,rec=1)nsib_tem, max_ig_tem,lrecl
          close(3)

!read the time varying fields
          open(unit=3,file=trim(datapath)//'data/sib_www',form='unformatted'
     *,        access='DIRECT',recl=lrecl)
          do ksdm = 1,nsdm_sfc
            do j = 1,jjm_sfc
              do i = 1,iim_sfc
                if(hr_sib(i,j,ksdm)) 
     *              read(3,rec=1+link_hr_sib( i,j,ksdm) )
     *                  ((www_m(i,j,k,m,ksdm),k=1,3),m=1,12)
              enddo
            enddo
          enddo
          close(3)
      endif  ! if prescribed_gw


!-----------------------------------------------------------------------
!   co2 respfactor
!-----------------------------------------------------------------------
      if(doSiBCO2) then
          open(unit=3,file=trim(datapath)//'data/CO2_respfactor',
     *         form='unformatted'
     *,        access='DIRECT',recl=16,status='old',action='read'
     *,        iostat=ioerr)
          if (ioerr /= 0) then
            write(*,*) '*** init_sib: CO2_respfactorfile error ',ioerr
            stop
          endif
          read(3,rec=1)nsib_tem, max_ig_tem, nsoil_tem, lrecl
          close(3)

!read the time varying fields
          allocate ( xrec(nsoil+1))
          open(unit=3,file=trim(datapath)//'data/CO2_respfactor',
     *         form='unformatted'
     *,        access='DIRECT',recl=lrecl)
          do ksdm = 1,nsdm_sfc
            do j = 1,jjm_sfc
              do i = 1,iim_sfc
                if(hr_sib(i,j,ksdm)) then
                   read(3,rec=1+link_hr_sib( i,j,ksdm) ) xrec
                   respfactor(i,j,:,ksdm) = xrec(:)
                endif
              enddo
            enddo
          enddo
          deallocate(xrec)
          close(3)
      endif  ! if dosibco2


      if(bgcyc) then
          open(unit=3,file=trim(datapath)//'data/sib_biome',form='unformatted'
     *,        access='DIRECT',recl=12,status='old',action='read'
     *,        iostat=ioerr)
          if (ioerr /= 0) then
            write(*,*) '*** init_sib: sib_biome file error ',ioerr
            stop
          endif
          read(3)nsib_tem,npt_tem,max_ig_tem,lrecl
          close(3) 

          open(unit=3,file=trim(datapath)//'data/sib_biome',form='unformatted'
     *,        access='DIRECT',recl=lrecl)
          do ksdm = 1,nsdm_sfc
            do j = 1,jjm_sfc
              do i = 1,iim_sfc
                if(hr_sib(i,j,ksdm)) 
     *              read(4,rec=1+link_hr_sib( i,j,ksdm) )
     *                   msibbiome(i,j,:,ksdm)
              enddo
            enddo
          enddo
          close(3)

      endif  ! if bgcyc

!-----------------------------------------------------------------------
! BOUNDARY CONDITIONS HAVE BEEN READ IN
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! compute lat,lon, and surface height (gpm)
!-----------------------------------------------------------------------
        where(hr_sib)
              !zs_sfc(:,:,:) = zs_sib(:,:,:)
              lon_sfc(:,:,:) = lon_sib(:,:,:)
              lat_sfc(:,:,:) = lat_sib(:,:,:)
              frac_sfc(:,:,:) = 1.0
        endwhere

!-----------------------------------------------------------------------
! dump some diagnostic info to screen as a sanity check
!-----------------------------------------------------------------------
      do i=1,npe_sfc
      write(6,*)
      if(my_task_sfc.eq.i-1) then
      write(6,*) ' SUMMARY OF SIB INIT AND BC for task ',my_task_sfc
      if(region_flag) then
        write(6,*) '    my_task: ', my_task_sfc
        write(6,*) '    tg     : ', minval(tg,mask=hr_sib),
     *                            maxval(tg,mask=hr_sib)
        write(6,*) '    tcan   : ', minval(tcan,mask=hr_sib),
     *                            maxval(tcan,mask=hr_sib)
        write(6,*) '    tke    : ', minval(tkes,mask=hr_sib),
     *                            maxval(tkes,mask=hr_sib)
        write(6,*) '    zs     : ', minval(zs_sib,mask=hr_sib),
     *                            maxval(zs_sib,mask=hr_sib)
        write(6,*) '    link   : ', minval(link_hr_sib,mask=hr_sib),
     *                            maxval(link_hr_sib,mask=hr_sib)
      else
        write(6,*) '   no sib points are owned by this process'
      endif
      endif
      enddo
#ifdef mpi
!     CALL MPI_BARRIER (MPI_COMM_SFC,ierr)
#endif

      end subroutine init_sib
!======================================================================
! END OF INIT_SIB
!======================================================================

!======================================================================
! BEGINNING OF INIT_COUPLER
!======================================================================
      subroutine init_coupler

!-- June, 2007:  Read in PBL turbulent kinetic energy (TKE) and surface
!--  geopotential the way the coupled model does.

      integer (kind=int_kind) :: i, j, ksdm, ioerr, max_ig_tem, lrecl
      character(char_len) filename
      real (kind=real_kind) :: xrec
      real (kind=dbl_kind) :: xrec2
      logical (kind=log_kind) :: l_exist
  
!-- 1) sfc geopotential
!-----------------------------------------------------------------------
! open correct file based on grid size
!-----------------------------------------------------------------------
      filename=' '
      if (max_ig_sfc == 02562) then
        filename = 'data/bc_atm_02562'
      elseif(max_ig_sfc == 10242) then
        filename = 'data/bc_atm_10242'
      elseif(max_ig_sfc == 40962) then
        filename = 'data/bc_atm_40962'
      elseif(max_ig_sfc == 00001) then
        filename = 'data/bc_atm'
      else
        stop 'init_coupler stop'
      endif
      filename = trim(filename)

#ifdef scm
      open(unit=3,file=filename,form='formatted', 
     *     status='old',iostat=ioerr)
      if (ioerr .ne. 0) then
        write(6,*) ' read_atm_bc, error open ',ioerr
        stop
      endif

      read(3,*) xrec
      zs_sfc(:,:,:) = xrec
      
      close(3)
#else
      open(unit=3,file=filename,form='unformatted',
     *     status='old',iostat=ioerr,access='DIRECT',recl=8)
      if (ioerr /= 0) then
        write(6,*) ' init_coupler, error open ',ioerr
        stop
      endif

!-------------------------------------------------------------------------
!  get the header record
!-------------------------------------------------------------------------
      read(3,rec=1) max_ig_tem,lrecl
      close(3)

      open(unit=3,file=filename,form='unformatted',
     *     status='old',iostat=ioerr,access='DIRECT',recl=lrecl)

!-----------------------------------------------------------------------
! read in surface geopotentials
!-----------------------------------------------------------------------
      do j = 1,jjm_sfc
        do i = 1,iim_sfc
          do ksdm = 1,nsdm_sfc
            read(3,rec=1+grid_indx_sfc(i,j,ksdm)) xrec
            zs_sfc(i,j,ksdm) = xrec
          enddo
        enddo
      enddo
      close(3)
#endif

!-- 2) PBL TKE
!-----------------------------------------------------------------------
! read in PBL turbulent kinetic energy
!-----------------------------------------------------------------------
      filename = 'data/model_restart_pbl'
      inquire(file=trim(filename),exist=l_exist)
      if (l_exist) then
        !allocate(xrec8(1))
        open(unit=3,file=filename,form='unformatted',
     *       status='old',iostat=ioerr,access='DIRECT',recl=8)
        read(3,rec=1) max_ig_tem,lrecl
        close(3)
        open(unit=3,file=filename,form='unformatted',
     *       status='old',iostat=ioerr,access='DIRECT',recl=lrecl)
        do j = 1,jjm_sfc
          do i = 1,iim_sfc
            do ksdm = 1,nsdm_sfc
              read(3,rec=1+grid_indx_sfc(i,j,ksdm)) xrec2
              tke_sfc(i,j,ksdm) = xrec2
            enddo
          enddo
        enddo
        close(3)
      else
        tke_sfc = tkemin
      endif
      
      end subroutine init_coupler
!======================================================================
! END OF INIT_COUPLER
!======================================================================

!      subroutine write_sib_prognostics
!        use domain_decomposition
!        use sib_parameters
!	use grid_metrics_old
!	use interpolate_to_latlon
!      implicit none
!
!      real (kind=dbl_kind), dimension(im,jm) :: work, work1, work2
!      integer (kind=int_kind) :: k
!
!        work = c0
!        work1 = c0
!        call gather_fine(capac(:,:,1,:),work1(:,:))
!        work = work + work1
!        work1 = c0
!        call gather_fine(capac(:,:,2,:),work1(:,:))
!        work = work + work1
!        work1 = c0
!        call gather_fine(snow(:,:,1,:),work1(:,:))
!        work = work + work1
!        work1 = c0
!        call gather_fine(snow(:,:,2,:),work1(:,:))
!        work = work + work1
!        do k = 1,3
!           work1 = c0
!           call gather_fine(www(:,:,k,:),work1(:,:))
!           work2 = c0
!           call gather_fine(zdepth(:,:,k,:),work2(:,:))
!           work = work + work1 * work2
!        enddo
!        work2 = c0
!        call gather_fine(grid_mask(:,:,:),work2(:,:))
!        work = work * 1000.
!        where(work2.le.0.001_dbl_kind) work = 1.e36
!        
!        if(my_task_sfc.eq.0) call interp_driver_2d(work, 47)
!
!      end subroutine write_sib_prognostics

      end module sfc_init
