        module sfc_restart

	use kinds
	use model_parameters
	use time_manager_module
	use domain_decomposition_sfc
	use gather_scatter

        implicit none

        real (kind=real_kind) :: vers = 5.1

#ifndef parallel_io
        real (kind=dbl_kind), dimension(iim_sfc, jjm_sfc, nsdm_sfc) :: work_loc
        real (kind=dbl_kind), dimension(im_sfc,jm_sfc) :: 
     *       work_glob, hr_glob, link_glob
        real (kind=dbl_kind), dimension(:), allocatable ::
     *       hr_vr, link_vr
        integer (kind=dbl_kind), dimension(:), allocatable :: link_vec
#endif

        contains

!======================================================================
! BEGINNING OF WRITE_RESTART_OCEAN
!======================================================================
	subroutine write_restart_ocean
	use ocean_parameters

	implicit none

        integer (kind=int_kind) :: i, lrecl, ioerr, n

        character*8, time_label
        integer (kind=int_kind) :: monthofyear

!  do master processor restart io for serial io
#ifndef parallel_io
!  global variables
!  max_ig vectors
        real (kind=dbl_kind), dimension(:), allocatable :: 
     *     sst_vec, ti_vec, sn_vec, iz_vec, e2_vec, tkeo_vec
#endif

!-------------------------------------------------------------------------
!  label the restart with a time stamp 
!-------------------------------------------------------------------------
        if(n_restart.gt.0) then
          write(unit=time_label,fmt='(i7)')
     *                  1000000+nsecond_dyn/86400
          time_label(1:1) = 'd'
        else
          monthofyear = mod(nmonth-1,12)+1
          write(unit=time_label,fmt='(i7)')
     &          1000000+100*(nyear+nyear0)+monthofyear 
          time_label(1:1) = 'y'
        endif

        lrecl = kind(sst)*6

#ifndef parallel_io

!-------------------------------------------------------------------------
!  gather the global arrays and reduce to vector
!-------------------------------------------------------------------------
        if(my_task.eq.0) then
           allocate(sst_vec(max_ig_sfc))
           allocate(ti_vec(max_ig_sfc))
           allocate(sn_vec(max_ig_sfc))
           allocate(iz_vec(max_ig_sfc))
           allocate(e2_vec(max_ig_sfc))
           allocate(tkeo_vec(max_ig_sfc))
           allocate(link_vr(max_ig_sfc))
           allocate(hr_vr(max_ig_sfc))
           allocate(link_vec(max_ig_sfc))
        endif
        call gather_fine_sfc(sst, work_glob )
        if(my_task.eq.0)
     *     call reduce_to_vector_global_sfc(work_glob(:,:),sst_vec(:))
        call gather_fine_sfc(ti, work_glob )
        if(my_task.eq.0)
     *     call reduce_to_vector_global_sfc(work_glob(:,:),ti_vec(:))
        call gather_fine_sfc(sn, work_glob )
        if(my_task.eq.0)
     *     call reduce_to_vector_global_sfc(work_glob(:,:),sn_vec(:))
        call gather_fine_sfc(iz, work_glob )
        if(my_task.eq.0)
     *     call reduce_to_vector_global_sfc(work_glob(:,:),iz_vec(:))
        call gather_fine_sfc(e2, work_glob )
        if(my_task.eq.0)
     *     call reduce_to_vector_global_sfc(work_glob(:,:),e2_vec(:))
        call gather_fine_sfc(tkeo, work_glob )
        if(my_task.eq.0)
     *     call reduce_to_vector_global_sfc(work_glob(:,:),tkeo_vec(:))

        work_loc = 0._dbl_kind
        where(hr_sea(:,:,:)) work_loc(:,:,:) = 1._dbl_kind
        call gather_fine_sfc(work_loc, work_glob )
        if(my_task.eq.0) 
     *     call reduce_to_vector_global_sfc(work_glob(:,:),hr_vr(:))

        work_loc(:,:,:) = link_hr_sea(:,:,:)
        call gather_fine_sfc(work_loc, work_glob )
        if(my_task.eq.0) then
           call reduce_to_vector_global_sfc(work_glob(:,:),link_vr(:))
           link_vec(:) = nint(link_vr(:))
        endif
        

        if(my_task.eq.0) then
           open(4,
     *   file=trim(datapath)//'restarts/model_restart_ocean.'//trim(time_label)
     *,         form='unformatted', access='DIRECT', recl=lrecl)

!-------------------------------------------------------------------------
! write out scalar data
!-------------------------------------------------------------------------
           write(4,rec=1,iostat=ioerr)
     *        nsecond_dyn, nocean, max_ig_sfc, lrecl, vers

!-------------------------------------------------------------------------
! write out array data
!-------------------------------------------------------------------------
           do i = 1,max_ig_sfc
              if(hr_vr(i).gt.0.5_dbl_kind) then
                 n = link_vec( i)
                 write(4,rec=1+n, iostat=ioerr )
     *              sst_vec(i), ti_vec(i), sn_vec(i)
     *,             iz_vec(i), e2_vec(i), tkeo_vec(i)
              endif
           enddo

           close(4)
           deallocate(sst_vec)
           deallocate(ti_vec)
           deallocate(sn_vec)
           deallocate(iz_vec)
           deallocate(e2_vec)
           deallocate(tkeo_vec)
           deallocate(link_vec)
           deallocate(link_vr)
           deallocate(hr_vr)

        endif

#else

        open(4,
     *    file=trim(datapath)//'restarts/model_restart_ocean.'//trim(time_label)
     *,      form='unformatted', access='DIRECT', recl=lrecl)

!-------------------------------------------------------------------------
! write out scalar data
!-------------------------------------------------------------------------
        if(my_task.eq.0) then
           write(4,rec=1,iostat=ioerr)
     *       nsecond_dyn, nocean, max_ig_sfc, lrecl, vers
        endif

!-------------------------------------------------------------------------
! write out array data
!-------------------------------------------------------------------------
        do ksdm = 1,nsdm_sfc
          do j = 1,jjm_sfc
            do i = 1,iim_sfc
              if(hr_sea(i,j,ksdm)) then
                 write(4,rec=1+link_hr_sea( i,j,ksdm), iostat=ioerr )
     *              sst(i,j,ksdm), ti(i,j,ksdm), sn(i,j,ksdm)
     *,             iz(i,j,ksdm), e2(i,j,ksdm), tkeo(i,j,ksdm)
              endif
            enddo
          enddo
        enddo

        close(4)
#endif

	end subroutine write_restart_ocean
!======================================================================
! END OF WRITE_RESTART_OCEAN
!======================================================================



!======================================================================
! BEGINNING OF WRITE_RESTART_LANDICE
!======================================================================
	subroutine write_restart_landice
	use landice_parameters

	implicit none

        integer (kind=int_kind) :: i, lrecl, ioerr, n

        character*8, time_label
        integer (kind=int_kind) :: monthofyear
!  do master processor restart io for serial io
#ifndef parallel_io
!  global variables
!  max_ig vectors
        real (kind=dbl_kind), dimension(:), allocatable :: 
     *     tgi_vec, tkei_vec
#endif

!-------------------------------------------------------------------------
!  label the restart with a time stamp 
!-------------------------------------------------------------------------
        if(n_restart.gt.0) then
          write(unit=time_label,fmt='(i7)')
     *                  1000000+nsecond_dyn/86400
          time_label(1:1) = 'd'
        else
          monthofyear = mod(nmonth-1,12)+1
          write(unit=time_label,fmt='(i7)')
     &          1000000+100*(nyear+nyear0)+monthofyear 
          time_label(1:1) = 'y'
        endif


        lrecl = max(20, kind(tgi)*2)

#ifndef parallel_io

!-------------------------------------------------------------------------
!  gather the global arrays and reduce to vector
!-------------------------------------------------------------------------
        if(my_task.eq.0) then
           allocate(tgi_vec(max_ig_sfc))
           allocate(tkei_vec(max_ig_sfc))
           allocate(link_vr(max_ig_sfc))
           allocate(hr_vr(max_ig_sfc))
           allocate(link_vec(max_ig_sfc))
        endif
        call gather_fine_sfc(tgi, work_glob )
        if(my_task.eq.0)
     *     call reduce_to_vector_global_sfc(work_glob(:,:),tgi_vec(:))
        call gather_fine_sfc(tkei, work_glob )
        if(my_task.eq.0)
     *     call reduce_to_vector_global_sfc(work_glob(:,:),tkei_vec(:))

        work_loc = 0._dbl_kind
        where(hr_ice(:,:,:)) work_loc(:,:,:) = 1._dbl_kind
        call gather_fine_sfc(work_loc, work_glob )
        if(my_task.eq.0) 
     *     call reduce_to_vector_global_sfc(work_glob(:,:),hr_vr(:))

        work_loc(:,:,:) = link_hr_ice(:,:,:)
        call gather_fine_sfc(work_loc, work_glob )
        if(my_task.eq.0) then
           call reduce_to_vector_global_sfc(work_glob(:,:),link_vr(:))
           link_vec(:) = nint(link_vr(:))
        endif
        

        if(my_task.eq.0) then
           open(4,
     * file=trim(datapath)//'restarts/model_restart_landice.'//trim(time_label)
     *,         form='unformatted', access='DIRECT', recl=lrecl)

!-------------------------------------------------------------------------
! write out scalar data
!-------------------------------------------------------------------------
           write(4,rec=1,iostat=ioerr)
     *        nsecond_dyn, nlandice, max_ig_sfc, lrecl, vers

!-------------------------------------------------------------------------
! write out array data
!-------------------------------------------------------------------------
           do i = 1,max_ig_sfc
              if(hr_vr(i).gt.0.5_dbl_kind) then
                 n = link_vec( i)
                 write(4,rec=1+n, iostat=ioerr )
     *              tgi_vec(i), tkei_vec(i)
              endif
           enddo

           close(4)
           deallocate(tgi_vec)
           deallocate(tkei_vec)
           deallocate(link_vec)
           deallocate(link_vr)
           deallocate(hr_vr)

        endif

#else
        open(4,
     *  file=trim(datapath)//'restarts/model_restart_landice.'//trim(time_label)
     *,      form='unformatted', access='DIRECT', recl=lrecl)

!-------------------------------------------------------------------------
! write out scalar data
!-------------------------------------------------------------------------
        if(my_task.eq.0) then
           write(4,rec=1,iostat=ioerr)
     *         nsecond_dyn, nlandice, max_ig_sfc, lrecl, vers
        endif

!-------------------------------------------------------------------------
! write out array data
!-------------------------------------------------------------------------
        do ksdm = 1,nsdm_sfc
          do j = 1,jjm_sfc
            do i = 1,iim_sfc
              if(hr_ice(i,j,ksdm)) then
                 write(4,rec=1+link_hr_ice(i,j,ksdm), iostat=ioerr )
     *              tgi(i,j,ksdm), tkei(i,j,ksdm)
              endif
            enddo
          enddo
        enddo

        close(4)
#endif

	end subroutine write_restart_landice
!======================================================================
! END OF WRITE_RESTART_LANDICE
!======================================================================



!======================================================================
! BEGINNING OF WRITE_RESTART_SIB
!======================================================================
	subroutine write_restart_sib
	use sib_parameters

	implicit none

        integer (kind=int_kind) :: i, j, lrecl, ioerr, n

        character*8, time_label
        integer (kind=int_kind) :: monthofyear

#ifndef parallel_io
!  global variables
!  max_ig vectors
        real (kind=dbl_kind), dimension(:), allocatable :: 
     *     tg_vec, tkes_vec, tcan_vec, gs_vec
        real (kind=dbl_kind), dimension(:,:), allocatable :: 
     *     snow_vec, capac_vec, www_vec, deept_vec
#endif

!-------------------------------------------------------------------------
!  label the restart with a time stamp 
!-------------------------------------------------------------------------
        if(n_restart.gt.0) then
          write(unit=time_label,fmt='(i7)')
     *                  1000000+nsecond_dyn/86400
          time_label(1:1) = 'd'
        else
          monthofyear = mod(nmonth-1,12)+1
          write(unit=time_label,fmt='(i7)')
     &          1000000+100*(nyear+nyear0)+monthofyear 
          time_label(1:1) = 'y'
        endif

        lrecl = kind(tg)*(4+2*2+3+nsoil)

#ifndef parallel_io

!-------------------------------------------------------------------------
!  gather the global arrays and reduce to vector
!-------------------------------------------------------------------------
        if(my_task.eq.0) then
           allocate(tcan_vec(max_ig_sfc))
           allocate(tg_vec(max_ig_sfc))
           allocate(gs_vec(max_ig_sfc))
           allocate(tkes_vec(max_ig_sfc))
           allocate(snow_vec(max_ig_sfc,2))
           allocate(deept_vec(max_ig_sfc,nsoil))
           allocate(capac_vec(max_ig_sfc,2))
           allocate(www_vec(max_ig_sfc,3))
           allocate(link_vr(max_ig_sfc))
           allocate(hr_vr(max_ig_sfc))
           allocate(link_vec(max_ig_sfc))
        endif
        call gather_fine_sfc(tg, work_glob )
        if(my_task.eq.0)
     *     call reduce_to_vector_global_sfc(work_glob(:,:),tg_vec(:))
        call gather_fine_sfc(tkes, work_glob )
        if(my_task.eq.0)
     *     call reduce_to_vector_global_sfc(work_glob(:,:),tkes_vec(:))
        call gather_fine_sfc(tcan, work_glob )
        if(my_task.eq.0)
     *     call reduce_to_vector_global_sfc(work_glob(:,:),tcan_vec(:))
        call gather_fine_sfc(gs, work_glob )
        if(my_task.eq.0)
     *     call reduce_to_vector_global_sfc(work_glob(:,:),gs_vec(:))
        do j = 1, 2
           call gather_fine_sfc(snow(:,:,j,:), work_glob )
           if(my_task.eq.0)
     *        call reduce_to_vector_global_sfc(work_glob(:,:),snow_vec(:,j))
           call gather_fine_sfc(capac(:,:,j,:), work_glob )
           if(my_task.eq.0)
     *        call reduce_to_vector_global_sfc(work_glob(:,:),capac_vec(:,j))
        enddo
        do j = 1, 3
           call gather_fine_sfc(www(:,:,j,:), work_glob )
           if(my_task.eq.0)
     *        call reduce_to_vector_global_sfc(work_glob(:,:),www_vec(:,j))
        enddo
        do j = 1, nsoil
           call gather_fine_sfc(deept(:,:,j,:), work_glob )
           if(my_task.eq.0)
     *        call reduce_to_vector_global_sfc(work_glob(:,:),deept_vec(:,j))
        enddo

        work_loc = 0._dbl_kind
        where(hr_sib(:,:,:)) work_loc(:,:,:) = 1._dbl_kind
        call gather_fine_sfc(work_loc, work_glob )
        if(my_task.eq.0) 
     *     call reduce_to_vector_global_sfc(work_glob(:,:),hr_vr(:))

        work_loc(:,:,:) = link_hr_sib(:,:,:)
        call gather_fine_sfc(work_loc, work_glob )
        if(my_task.eq.0) then
           call reduce_to_vector_global_sfc(work_glob(:,:),link_vr(:))
           link_vec(:) = nint(link_vr(:))
        endif
        

        if(my_task.eq.0) then
           open(4,
     *   file=trim(datapath)//'restarts/model_restart_sib.'//trim(time_label)
     *,         form='unformatted', access='DIRECT', recl=lrecl)

!-------------------------------------------------------------------------
! write out scalar data
!-------------------------------------------------------------------------
           write(4,rec=1,iostat=ioerr)
     *        nsecond_dyn, nsib, nsoil, max_ig_sfc, lrecl, vers

!-------------------------------------------------------------------------
! write out array data
!-------------------------------------------------------------------------
           do i = 1,max_ig_sfc
              if(hr_vr(i).gt.0.5_dbl_kind) then
                 n = link_vec( i)
                 write(4,rec=1+n, iostat=ioerr )
     *              tcan_vec(i), tg_vec(i), deept_vec(i,:),
     *              capac_vec(i,:), snow_vec(i,:), www_vec(i,:),
     *              gs_vec(i), tkes_vec(i)
              endif
           enddo

           close(4)
           deallocate(tg_vec)
           deallocate(tcan_vec)
           deallocate(deept_vec)
           deallocate(snow_vec)
           deallocate(capac_vec)
           deallocate(www_vec)
           deallocate(gs_vec)
           deallocate(tkes_vec)
           deallocate(link_vec)
           deallocate(link_vr)
           deallocate(hr_vr)

        endif

#else
        open(4,
     *     file=trim(datapath)//'restarts/model_restart_sib.'//trim(time_label)
     *,      form='unformatted', access='DIRECT', recl=lrecl)

!-------------------------------------------------------------------------
! write out scalar data
!-------------------------------------------------------------------------
        if(my_task.eq.0) then
           write(4,rec=1,iostat=ioerr)
     *         nsecond_dyn, nsib, nsoil, max_ig_sfc, lrecl, vers
           if(ioerr.ne.0)
     * print *,'sib restart header ioerr, mytask ',ioerr,my_task
        endif

!-------------------------------------------------------------------------
! write out array data
!-------------------------------------------------------------------------
        do ksdm = 1,nsdm_sfc
          do j = 1,jjm_sfc
            do i = 1,iim_sfc
              if(hr_sib(i,j,ksdm)) then
                 write(4,rec=1+link_hr_sib(i,j,ksdm), iostat=ioerr )
     *              tcan(i,j,ksdm), tg(i,j,ksdm), deept(i,j,:,ksdm)
     *,             capac(i,j,:,ksdm), snow(i,j,:,ksdm), www(i,j,:,ksdm)
     *,             gs(i,j,ksdm), tkes(i,j,ksdm)
        if(ioerr.ne.0)
     * print *,'sib restart ioerr, mytask, rec '
     *,ioerr,my_task,link_hr_sib(i,j,ksdm)+1
              endif
            enddo
          enddo
        enddo

        close(4)
#endif

	end subroutine write_restart_sib
!======================================================================
! END OF WRITE_RESTART_SIB
!======================================================================

!======================================================================
! BEGINNING OF WRITE_RESTART_PBL
!======================================================================
      subroutine write_restart_pbl

!-- 6/2007: Write out TKE restart file (bag the parallel i/o if block)

      use fluxcoupler_fields_sfc
      implicit none
      
      real (kind=dbl_kind), dimension(:), allocatable :: tke_vec

      integer (kind=int_kind) :: i, j, lrecl, ioerr, n
      character*8, time_label
      integer (kind=int_kind) :: monthofyear

!-------------------------------------------------------------------------
!  label the restart with a time stamp 
!-------------------------------------------------------------------------
      if (n_restart > 0) then
        write(unit=time_label,fmt='(i7)')
     *                  1000000+nsecond_dyn/86400
        time_label(1:1) = 'd'
      else
        monthofyear = mod(nmonth-1,12)+1
        write(unit=time_label,fmt='(i7)')
     &        1000000+100*(nyear+nyear0)+monthofyear 
        time_label(1:1) = 'y'
      endif

      lrecl = kind(tke_sfc)

      if (my_task == 0) allocate(tke_vec(max_ig_sfc))
      call gather_fine(tke_sfc, work_glob)
      if (my_task == 0) call reduce_to_vector_global(work_glob,tke_vec)

!-------------------------------------------------------------------------
! write out scalar data
!-------------------------------------------------------------------------
      if (my_task == 0) then
        open(4,
     *     file=trim(datapath)//'restarts/model_restart_pbl.'//trim(time_label)
     *,      form='unformatted', access='DIRECT', recl=lrecl)
           write(4,rec=1,iostat=ioerr) max_ig_sfc, lrecl
           if (ioerr /= 0)
     *       print *,'pbl restart header ioerr, mytask ',ioerr,my_task

!-------------------------------------------------------------------------
! write out array data
!-------------------------------------------------------------------------
        do i = 1,max_ig_sfc
          write(4,rec=1+i,iostat=ioerr) tke_vec(i)
        enddo
        close(4)
      endif

      end subroutine write_restart_pbl
!======================================================================
! END OF WRITE_RESTART_PBL
!======================================================================

        end module sfc_restart
