c-------------------------------------------------------------------

      SUBROUTINE SIB
     *   ( nsib, len, nsoil            ! array and loop bounds
     *,    tau                         ! time independent variable
     *,    tc, tg, td, www, snow, capac, rst, tke   ! prognostic variables
     *,    thm, sh, ps, bps, ros, ts, psb    ! atmospheric forcing
     *,    cupr, lspr, radvbc, radnbc, radvdc, radndc ! atmospheric forcing
     *,    zb, spdm, dlwbot                           ! atmospheric forcing
     *,    z0d, zlt, z1, z2, cc1, cc2, dd, zdepth, poros ! surface parameters
     *,    phsat, bee, respcp, vmax0, green, tran, ref  ! surface parameters
     *,    gmudmu, trop, phc, trda, trdm, slti, shti    ! surface parameters
     *,    hltii, hhti, effcon, binter, gradm, atheta   ! surface parameters
     *,    btheta, aparc, wopt, zm, wsat, vcover, radfac! surface parameters
     *,    thermk, satco, slope, chil, ztdep, sandfrac  ! surface parameters
     *,    pi, grav, dt, cp, rgas, hltm             ! constants
     *,    delta, asnow, kapa, snomel, clai, cww, pr0   ! constants
     *,    ribc, vkrmn, pco2m, po2m, stefan, grav2  ! constants
     *,    gamfac, tice, snofac                     ! constants
     *,    fss, fws, drag, co2flx, cu, ct, ustar        ! output
     *,    ioffset
     *,    sibdrv, forcerestore, doSiBco2, fixday, dotkef  ! logical switches
     *,    ventmf, thvgm, louis, zwind, ztemp, respfactor
     *,    xgpp, prescribed_gw, roff, mrros, areas, zmelt
     *,    assimn, soilscalediag, respg 
     *,    eci, ect, egi, egs
     *,    exo, q3o, q3l, ta, sha, rha, rstfac_diag 
     *,    ra, rb, rd, chf, shf, radc31, radc32 )

      use kinds
      use eau_util

      IMPLICIT none

c-------------------------------------------------------------------
c     Subroutine  bldif conducts implicit time differencing for 
c          turbulence kinetic energy,ventilation mass flux calculations,
c          drag, and boundary layer processes, and calls subroutines 
c          calculating PBL and surface fluxes. 

c     REFERENCES: Sato, N., P. J. Sellers, D. A. Randall, E. K. Schneider, 
c          J. Shukla, J. L Kinter III, Y-T, Hou, and Albertazzi (1989) 
c          "Effects of implementing the simple biosphere model in a general
c          circulation model. J. Atmos. Sci., 46, 2757-2782.

c                 Sellers, P. J., D. A. Randall, C. J. Collatz, J. A. Berry,
c          C. B. Field, D. A. Dazlich, C. Zhang, G. Collelo (1996) A revise 
c          land-surface parameterization (SiB2) for atmospheric GCMs. Part 1:
c          Model formulation. (accepted by JCL)


c     MODIFICATIONS:
c	   - changed VQSAT call to VNQSAT.  kwitt 10/23
c          - added in the prognostic stomatal conductance in addinc. changan 
c          - moved sib diagnostics accumulation from dcontrol to bldif
c                dd 950202

c     SUBROUTINES CALLED:  VNQSAT, SNOW1, balan, VNTLAT
c          DELHF, DELEF, NETRAD, SIBSLV, endtem, updat2, addinc
c          inter2, balan, soilprop, soiltherm, begtem, rnload

c     FUNCTIONS CALLED:
c          none

C     Argument list variables
c       Intent: in
      integer (kind=int_kind),intent(in) ::
     *        nsib, len, nsoil    ! array and loop bounds
     *,       ioffset   ! subdomain offset
      logical (kind=log_kind), intent(in) ::
     *        sibdrv           ! run SiB with prescribe meteorology
     *,       forcerestore     ! do force-restore soil thermodynamics
     *,       doSiBco2            ! prognostic co2
     *,       fixday           ! perpetual day of year conditions
     *,       dotkef           ! use changan instead of deardorff flux 
                               !  parameterization
     *,       prescribed_gw    ! if true, soil wetness is not prognosed
     *,       louis            
      real (kind=dbl_kind), intent(in) ::
     *        tau    ! time independent variable (hr)
     *,       zwind, ztemp
      real (kind=dbl_kind), intent(in), dimension(len) ::
!  atmospheric forcing
     *     ps        ! surface pressure (hPa)
     *,    bps       ! (ps/1000)**kapa
     *,    ros       ! surface air density (kg/m^3)
     *,    ts        ! surface mixed layer air temperature
     *,    psb       ! boundary layer mass depth (hPa)
     *,    cupr      ! convective precipitation rate (mm/s)
     *,    lspr      ! stratiform precipitation rate (mm/s)
     *,    radvbc    ! surface incident visible direct beam (W/m^2)
     *,    radnbc    ! surface incident near IR direct beam (W/m^2)
     *,    radvdc    ! surface incident visible diffuse beam (W/m^2)
     *,    radndc    ! surface incident near IR diffuse beam (W/m^2)
     *,    zb        ! boundary layer thickness (m)
     *,    spdm      ! boundary layer wind speed (m/s) 
     *,    dlwbot    ! surface incident longwave radiation (W/m^2)          !  surface parameters
     *,    z0d        ! surface roughness length (m)
     *,    zlt       ! leaf area index
     *,    z1
     *,    z2
     *,    cc1
     *,    cc2
     *,    dd
     *,    poros     ! soil porosity
     *,    phsat
     *,    bee
     *,    respcp
     *,    vmax0
     *,    green
     *,    gmudmu
     *,    trop
     *,    phc
     *,    trda
     *,    trdm
     *,    slti
     *,    shti
     *,    hltii
     *,    hhti
     *,    effcon
     *,    binter
     *,    gradm
     *,    atheta
     *,    btheta
     *,    aparc
     *,    wopt
     *,    zm
     *,    wsat
     *,    vcover   ! vegetation cover fraction
     *,    thermk
     *,    satco
     *,    slope
     *,    chil
     *,    sandfrac     ! soil texture sand fraction
      real (kind=dbl_kind), intent(in), dimension(nsib,2,2) ::
     *     tran
     *,    ref
      real (kind=dbl_kind), intent(in), dimension(nsib,2,2,2) ::
     *     radfac
      real (kind=dbl_kind), intent(in), dimension(nsib,3) ::
     *     zdepth ! porosity * soil hydrology model layer depths (m)
      real (kind=dbl_kind), intent(in), dimension(nsib,nsoil) ::
     *     ztdep ! soil thermal model layer depths (m)
      real (kind=dbl_kind), intent(in), dimension(nsib,nsoil+1) ::
     *     respfactor
      real (kind=dbl_kind), intent(in) ::
     *     pi        ! 3.1415926....
     *,    grav      ! gravitational acceleration (m/s^2)
     *,    dt        ! time step (s)
     *,    cp        ! heat capacity of dry air (J/(kg K) )
     *,    rgas      ! gas constant for dry air (hPa m^3 / (kg K) )
     *,    hltm      ! latent heat of vaporization (J/kg)
     *,    delta
     *,    asnow
     *,    kapa
     *,    snomel
     *,    clai      ! leaf heat capacity
     *,    cww       ! water heat capacity
     *,    pr0
     *,    ribc
     *,    vkrmn     ! von karmann's constant
     *,    pco2m     ! CO2 partial pressure (Pa)
     *,    po2m      ! O2 partial pressure (Pa)
     *,    stefan    ! stefan-boltzman constant
     *,    grav2     ! grav / 100 Pa/hPa
     *,    gamfac
     *,    tice      ! freezing temperature (KP
     *,    snofac

c    Intent: in/out
      real (kind=dbl_kind), intent(inout), dimension(len) ::
     *     tc    ! canopy temperature (K)
     *,    tg    ! surface boundary temperature (K)
     *,    rst        ! stomatal resistance
     *,    tke        ! turbulent kinetic energy
     *,    thm        ! mixed layer potential temperature (K)
     *,    sh         ! mixed layer water vapor mixing ratio (kg/kg)
      real (kind=dbl_kind), intent(inout), dimension(nsib,2) ::
     *     snow    ! snow cover ( kg /m^2)
     *,    capac   ! liquid interception store (kg/m^2)
      real (kind=dbl_kind), intent(inout), dimension(nsib,3) ::
     *     www     ! soil wetness 
      real (kind=dbl_kind), intent(inout), dimension(nsib,nsoil) ::
     *     td  ! deep soil temperature (K)

c     Intent:out
      real (kind=dbl_kind), intent(inout), dimension(len) ::
     *     sha     ! canopy airspace water vapor mixing ratio (kg/kg)
      real (kind=dbl_kind), intent(out), dimension(len) ::
     *     fss    ! surface sensible heat flux (W/m^2)
     *,    fws    ! surface evaporation (kg/m^2/s)
     *,    co2flx ! surface CO2 flux
     *,    cu
     *,    ct
     *,    ustar  ! friction velocity
     *,    ventmf ! ventilation mass flux
     *,    thvgm
     *,    xgpp   ! gross primary productivity (micromoles/m**2/s)
     *,    mrros  ! surface runoff
     *,    areas  ! snow cover fraction
     *,    zmelt  ! snow melt
     *,    assimn ! net carbon assimilation (moles/m2/s)
     *,    respg  ! respiration
     *,    ta      ! canopy airspace temperature (K)
     *,    rha     ! canopy airspace relative humidity
     *,    ect     ! transpiration (J)
     *,    eci     ! canopy interception evaporation (J)
     *,    egs     ! soil evaporation (J)
     *,    egi     ! ground interception evaporation (J)
     *,    roff   ! total runoff (surface and subsurface)
     *,    q3l
     *,    q3o
     *,    exo
     *,    rb
     *,    rd
     *,    ra
     *,    chf     ! canopy heat flux (W/m^2)
     *,    shf     ! soil heat flux (W/m^2)
     *,    radc31     
     *,    radc32     

      real (kind=dbl_kind), intent(out), dimension(nsib,2) ::
     *     drag   ! surface drag coefficient
      real (kind=dbl_kind), intent(out), dimension(nsib,nsoil+1) ::
     *     soilscalediag   
      real (kind=dbl_kind), intent(out), dimension(nsib,4) ::
     *     rstfac_diag   
                       
C     Local variables: 
      real (kind=dbl_kind) ::
     *     dtinv
      real (kind=dbl_kind), dimension(len) ::
     *     psy     ! psychrometric 'constant'
     *,    tha     ! canopy airspace potential temperature (K)
     *,    ea      ! canopy airspace water vapor pressure (hPa)
     *,    em      ! mixed layer water vapor pressure (hPa)
     *,    d       ! dd corrected for snow covered canopy
     *,    rbc     ! cc1 corrected for snow covered canopy
     *,    rdc     ! cc2 corrected for snow covered canopy
     *,    etmass  ! evapotranspiration
     *,    totwb   ! total surface and soil water at beginning of timestep
     *,    hc     
     *,    hg
     *,    heaten
     *,    hflux
     *,    tgs     ! bare ground and snow surface mean temperature
     *,    czc     ! canopy heat capacity
     *,    btc
     *,    btg
     *,    rsoil
     *,    hr
     *,    wc
     *,    wg
     *,    csoil
     *,    cogr
     *,    cogs
     *,    rib     ! soil resistance
     *,    rc
     *,    ecmass
     *,    hrr
     *,    bintc  
     *,    aparkk
     *,    wsfws
     *,    wsfht
     *,    wsflt
     *,    wci
     *,    whs
     *,    omepot
     *,    assimpot
     *,    assimci
     *,    antemp
     *,    assimnp
     *,    wags
     *,    wegs
     *,    pfd
     *,    assim
     *,    drst    ! stomatal resistance increment 
     *,    ansqr
     *,    anwtc
     *,    hgdtg
     *,    hgdtc
     *,    hgdth
     *,    hcdtg
     *,    hcdtc
     *,    hcdth
     *,    aag
     *,    aac
     *,    aam
     *,    fc
     *,    fg
     *,    eg
     *,    ec
     *,    egdtg
     *,    egdqm
     *,    ecdtg
     *,    ecdtc
     *,    ecdqm
     *,    egdtc
     *,    deadtg
     *,    deadtc
     *,    deadqm
     *,    bbg
     *,    bbc
     *,    bbm
     *,    dtc     ! canopy temperature increment (K)
     *,    zmelt1
     *,    evt
     *,    eastar
     *,    roffo
     *,    zmelt2
      real (kind=dbl_kind), dimension(len) ::
     *     ggl
     *,    egmass
     *,    etc
     *,    etg
     *,    fac1
     *,    dth    ! mixed layer potential temperature increment (K)
     *,    dqm    ! mixed layer water vapor mixing ratio increment (kg/kg)
     *,    czh    ! surface layer heat capacity
     *,    ustaro  ! friction velocity (m/s) ( for oceanic z0)
     *,    cuo   ! ( for oceanic z0)
     *,    z0    ! surface roughness length corrected for canopy snow (m)
     *,    thgeff, tgeff
     *,    shgeff, canex
     *,    cuprt, lsprt ! copies of cupr and lspr
     *,    thmtem, shtem  ! copies of thm and sh
     *,    zzwind, zztemp
     *,    pstem  ! ps, but in Pa

      real (kind=dbl_kind), dimension(len,2) ::
     *    zmstscale
     *,    radc3
     *,    dtg   ! surface ground and snow temperature increments (K)
     *,    satcap(len,2)
      real (kind=dbl_kind), dimension(len,3) ::
     *    qqq
     *,    wwwtem  ! soil wetness copy
     *,    radt
      real (kind=dbl_kind), dimension(len,4) ::
     *    rstfac
      real (kind=dbl_kind), dimension(len,2,2) ::
     *     radn
      real (kind=dbl_kind), dimension(len,nsoil) ::
     *     slamda  ! soil thermal conductivities
     *,    shcap   ! soil heat capacities
     *,    dtd   ! deep soil temperature increments (K)
      real (kind=dbl_kind), dimension(len,nsoil+1) ::
     *     soilscale
     *,    soilq10

      integer (kind=int_kind) :: i
             
c-------------------------------------------------------------------

      dtinv = 1.0 / dt   ! inverse time step

c     some initialization, copy of soil wetness
      do i = 1,len
         cuprt(i) = cupr(i) * 0.001
         lsprt(i) = lspr(i) * 0.001
         zmelt(i) = 0.0
         roff(i) = 0.0
         wwwtem(i,1) = www(i,1)
         wwwtem(i,2) = www(i,2)
         wwwtem(i,3) = www(i,3)
         pstem(i) = ps(i) * 100.
      enddo                                                  

c     first guesses for ta and ea (see temrec 120)

      DO I=1,len
         TA(I) = TC(I)
         THA(I) = TA(I) / BPS(I)
      enddo

      call eau_sat(len, pstem, ta, qsats=sha)

      DO I=1,len
         EA(I) = SHA(I) * PS(I) / (0.622 + SHA(I))
         EM(I) = SH(I) * PS(I) / (0.622 + SH(I))
      enddo

c    distribute incident radiation between canopy and surface
      call rnload(len, nsib,
     *           radvbc,radvdc,radnbc,radndc,dlwbot,VCOVER,
     *           thermk,radfac,radn,radc3)

c    adjust aerodynamic quantities for snow covered canopy
c      CALL SNOW1(snow, d, z0, rbc, rdc,
c     *      areas, satcap, zlt, z1, z2, cc1, cc2,
c     *      z0d, dd, asnow, nsib, len )
      do i = 1,len
         CANEX(i)  = 1.-( SNOW(i,1)*5.-Z1(i))/
     *                           (Z2(i)-Z1(i))
         CANEX(i)  = MAX( 0.1_dbl_kind, CANEX(i) )
         CANEX(i)  = MIN( 1.0_dbl_kind, CANEX(i) )
         D(i)  = Z2(i) - ( Z2(i)-DD(i) ) * CANEX(i)
         Z0(i) = Z0D(i)/( Z2(i)-DD(i) ) * ( Z2(i)-D(i) )
         RBC(i)    = CC1(i)/CANEX(i)
         RDC(i)    = CC2(i)*CANEX(i)
         AREAS(i)    = MIN(1.0_dbl_kind, ASNOW*SNOW(i,2))
         SATCAP(i,1) = ZLT(i)*0.0001 * CANEX(i)
c
cc Collatz-Bounoua change satcap(2) to 0.0002
C      SATCAP(i,2) = 0.002
          satcap(i,2) = 0.0002           ! lahouari
      enddo

c    initialize energy and water budgets
      call balan(1,tau, zdepth, wwwtem, capac, cupr,
     *      lspr, roff, etmass, totwb, radt, chf, shf,
     *      dt, ect, eci, egs, egi, hc, hg, heaten,
     *      hflux, snow, thm, tc, tg, tgs, td,
     *      ps, kapa, nsib, len, ioffset, nsoil )

      call begtem(tc, tg, cp, hltm, ps, snomel
     ^,           zlt, clai, cww, wwwtem, poros, pi, psy
     ^,           phsat, bee, czc, czh, phc
     ^,           tgs, etc, etg, btc, btg, rstfac
     ^,           rsoil, hr,  wc, wg
     ^,           snow, capac, areas, satcap, csoil, tice, grav
     ^,           snofac, len, nsib, forcerestore )

C     CALCULATE RADT USING RADIATION FROM PHYSICS AND CURRENT
C     LOSSES FROM CANOPY AND GROUND

      call NETRAD(radc3, radt, stefan, fac1, vcover, thermk, tc,
     *      tg, tice, tgeff, areas, len )
      DO I=1,len
            THgeff(I) = Tgeff(I) / BPS(I)
        radc31(i) = radc3(i,1)
        radc32(i) = radc3(i,2)
      enddo
      call eau_sat(len, pstem, tgeff, qsats=shgeff)

C     GET RESISTANCES FOR SIB
      CALL VNTLAT(grav, tice,
     *            pr0, ribc, vkrmn, delta, dt, tc, tg, ts, ps, zlt,
     *            wwwtem, tgs, etc, etg,
     *            rstfac, rsoil, hr, wc, wg, areas,
     *            sh, z0, spdm, sha, zb, ros,
     *            cu, ra, thvgm, rib, ustar, ventmf, thm, tha, z2, d,
     *            fc, fg, rbc, rdc,
     *            respcp, cogr, cogs, rb, rd, bps, rst, rc, ecmass,
     *            ea, em, hrr, assimn, bintc, ta, pco2m, po2m, vmax0,
     *            green, tran, ref, gmudmu, trop, trda, trdm, slti,
     *            shti, hltii, hhti, radn, effcon, binter, gradm,
     *            atheta, btheta, aparkk, wsfws, wsfht, wsflt, wci,
     *            whs, omepot, assimpot, assimci, antemp, assimnp,
     *            wags, wegs, aparc, pfd, assim, td, wopt, zm, wsat,
     *            soilscale, zmstscale, drst,
     *            soilq10, ansqr,
     *            nsib, len, nsoil, forcerestore, dotkef,
     *            thgeff, shgeff, tke, ct, louis, zwind, ztemp,
     *            respg, respfactor )
      soilscalediag(1:len,:) = soilscale(1:len,:)

C   this call for ustar, cu for oceanic value of z0 
      if(louis) then
          do i = 1,len
             zzwind(i) = z2(i)-d(i)+ zwind
             zztemp(i) = z2(i)-d(i)+ ztemp
          enddo
         call VMFCALZO(PR0,RIBC,VKRMN,DELTA,GRAV
     *,                PS,tha          
     *,                SPDM,ROS,CUo,THVGM,RIB     
     *,                USTARo,zzwind,zztemp
     *,                len ) 
      else                                  
         CALL VMFCALo(PR0,RIBC,VKRMN,GRAV,SPDM   
     *,              ZB,ROS,CUo,THVGM,USTARo     
     *,              tha, len, thgeff, tke, dotkef )    
      endif                                 
      DO I=1,len
         DRAG(I,1) = ROS(I) * USTAR(I)**2/spdm(i)
         DRAG(I,2) = ROS(I) * USTARo(I)**2/spdm(i)
         ANWTC(i) = ANTEMP(i)* TC(i)
      enddo

         CALL DELHF(dt, CP, BPS, THM, tgs, TC, ROS, RA, RB, RD,
     *      HGDTG, HGDTC, HGDTH, HCDTG, HCDTC, HCDTH, AAG, AAC, AAM,
     *      hc, hg, len )

C     FC AND FG MUST BE PASSED FROM EA/TA LOOP IN BLDIF
C     THEY WILL NOT NECESSARILY BE 1., 1.

         CALL DELEF(dt, CP, PS, SH, ROS, WC, WG, FC, FG, HRR, RST,
     *      RA, RB, RD, RSOIL, ETC, ETG, EM, BTC, BTG, RC,
     *      EA, EC, EG, EGDTG, EGDQM, ECDTG, ECDTC, ECDQM, EGDTC,
     *      DEADTC, DEADTG, DEADQM, BBG, BBC, BBM, areas, cogr, cogs,
     *      psy, hr, len )

c     get soil thermal properties
         if(.not.forcerestore)
     *   call soilprop( td, tgs, slamda, shcap, wwwtem, poros, ztdep, 
     *                  asnow, snow(1,2), areas, tice, snomel,
     *                  sandfrac, nsib, len, nsoil )

c     check against new code, rn derivatives may be zero

         CALL SIBSLV(dt, GRAV2, CP, HLTM, stefan, tgs,
     *      TD(1,nsoil), slamda(1,nsoil), pi, areas,
     *      VENTMF, PSB, BPS, CZH, CZC, HGDTG, EGDTG, HGDTC, HGDTH,
     *      EGDQM, HCDTG, ECDTG, HCDTC, ECDTC, EGDTC, HCDTH, ECDQM, 
     *      AAG, AAC, AAM, BBG, BBC, BBM, EC, EG, RADT, FSS, FWS,
     *      csoil, hc, hg, dtg, dtc, dth, dqm, fac1, tc,
     *      len, sibdrv, forcerestore )

         do i = 1,len
            radt(i,2) = (1.-areas(i))*radt(i,2)+areas(i)*radt(i,3)
         enddo

C     FLXCOR CORRECTS THE FLUXES IN THE PRESENCE OF DEW
c     temre2 replaces flxcor
c     updat2 replace seep, snowm and rofff

         call endtem( etc, ea, etg, btc, btg, deadtg, deadtc, dtc, dtg,
     *      deadqm, dqm, wc, wg, rb, rd, dt, ros, psy, snow, capac,
     *      hltm, areas, asnow, snofac, csoil, hr, rst, fc, fg,
     *      rsoil, zdepth, wwwtem, czc, czh, hc, hg, ec, eg, eci, ect,
     *      egi, egs, chf, shf, heaten, tgs, td(1,nsoil), hcdtc, hcdtg,
     *      hcdth, hgdtc, hgdtg, hgdth, dth, tg, tice, cp, 
     *      slamda(1,nsoil), len, nsib, forcerestore )

         call updat2(snow ,capac , snofac, ect, eci, egi,
     *      egs, hltm, wwwtem, pi, czh, dtd, dtg, dtc, dt,
     *      roff, tc, td, tg, bee, poros, satco,
     *      slope, phsat, zdepth, ecmass, egmass, hflux,
     *      shf, tice, snomel, asnow, czc, csoil, chf,
     *      etmass, hc, hg, areas, q3l, q3o, exo,
     *      qqq, zmelt1, cww, len, nsib, nsoil,
     *      forcerestore )

         do i = 1, len
            EVT(i) = (55.56 *dtinv) * (ECMASS(i) + EGMASS(i))
            wegs(i) = wegs(i) * evt(i)
         enddo

c     get soil temperature increments
       if(.not.forcerestore)
     *   call soiltherm( td, dtd, tgs, dtg, slamda, shcap, ztdep, dt,
     *                  nsib, len, nsoil )

c    update prognostic variables, get total latent and sensible fluxes
         do i = 1,len
            thmtem(i) = thm(i)
            shtem(i) = sh(i)
         enddo
         call addinc( grav2, cp, dt, hc, hg, bps, ecmass,
     *      egmass, fss, fws, psb, td, thmtem, shtem,
     *      tc, tg, dtd, dtc, dtg,
     *      drst, rst, bintc, len, nsib, nsoil )

C        CALL INTERC

         call eau_sat(len, pstem, ta, ess=eastar)
         eastar = eastar * 0.01

c     inter2 replaces interc

         call inter2( cuprt, lsprt, snow, capac, wwwtem , pi,
     *      satcap, cww, tc, tg, clai, zlt, chil, roff,
     *      snomel, zdepth, ts, tice, asnow, csoil,
     *      satco, dt, vcover, roffo, zmelt2 ,len, nsib )

c     complete check of energy and water budgets
         call balan(2,tau, zdepth, wwwtem, capac, cupr,
     *         lspr, roff, etmass, totwb, radt, chf, shf,
     *         dt, ect, eci, egs, egi, hc, hg, heaten,
     *         hflux, snow, thmtem, tc, tg, tgs, td,
     *         ps, kapa, nsib, len, ioffset, nsoil )

c     for perpetual conditions, do not update soil moisture
         if( .not.(fixday.or.prescribed_gw) ) then
            do i = 1,len
               www(i,1) = wwwtem(i,1)
               www(i,2) = wwwtem(i,2)
               www(i,3) = wwwtem(i,3)
            enddo
         endif

         do i = 1,len
            xgpp(i) = assim(i) * 1.e6
         enddo
C     Calculate the surface flux of CO2 due to SiB (tracer T18)

c     The net flux to the atmosphere is given by the release of CO2
c     by soil respiration minus the uptake of CO2 by photosynthesis

c     ASSIMN is the net assimilation of CO2 by the plants (from SiB2)
c     respFactor*soilScale is the rate of release of CO2 by the soil

c     soilScale is a diagnostic of the instantaneous rate of 
c        soil respiration (derived by Jim Collatz, similar to TEM)
c     respFactor is the annual total accumulation of carbon in the
c        previous year at each grid cell (annual total ASSIMN) 
c        divided by the annual total of soilScale at the same grid pt.

c     The net flux is multiplied by 0.012 to convert from
c        moles/m2/sec to kg/m2/sec
         if(doSiBco2) then
            do i = 1,len
c               co2flx(i) = respFactor(i,1) * soilscale(i,1) - Assimn(i)
               co2flx(i) = 0.012 * (respg(i) - Assimn(i) )
            enddo
c            if(.not.forcerestore) then
c               ksoil=5
c               do l = 2,ksoil
c                  do i = 1, len
c                     co2flx(i) = co2flx(i) + 
c     *                    respFactor(i,l) * soilscale(i,l) 
c                  enddo
c               enddo
c            endif
c            co2flx = 0.012 * co2flx
         else
           do i = 1,len
            co2flx(i) = 0.0
           enddo
         endif

c    some quantities for diagnostic output
         do i = 1,len
            rha(i) = ea(i)/eastar(i)
            zmelt(i) = (zmelt1(i)+zmelt2(i)) * 1000. / dt

c           Calculate an overall leaf conductance, which is QP2(162) 
c           and is used as a weighting function in QP2(162 and 163)
            ggl(i) = 1. / (rst(i) * (rb(i) + rc(i)))

c   surface runoff
            mrros(i) = ( roff(i)-(qqq(i,3)*dt+exo(i)) ) * 1000.  / dt

            roff(i) = roff(i) * 1000./dt
            exo(i) = exo(i) * 1000./dt
            q3l(i) = q3l(i) * 1000./dt
            q3o(i) = q3o(i) * 1000./dt
            eci(i) = ecmass(i)*hltm - ect(i)
            egi(i) = egmass(i)*hltm - egs(i)
            eci(i) = eci(i) / (dt*hltm)
            ect(i) = ect(i) / (dt*hltm)
            egi(i) = egi(i) / (dt*hltm)
            egs(i) = egs(i) / (dt*hltm)
         enddo
      rstfac_diag(1:len,:) = rstfac(1:len,:)



      RETURN
      END
