!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
	
 	module sib_parameters

!-----------------------------------------------------------------
!	this module specifies parameters and options
!           for the SiB2 land surface parameterization
!       default values are provided below
!-----------------------------------------------------------------

	use kinds
        use domain_decomposition_sfc
        use physical_parameters

 	implicit none
 	save

        integer (kind=int_kind), parameter ::
     *     nsoil = 6
     *,    npt = 1

	logical (kind=log_kind), parameter ::
     *     forcerestore = .false.
     *,    bgcyc = .false.
     *,    prescribed_gw = .false.
     *,    doSiBco2 = .true.

        real (kind=dbl_kind), parameter ::
     *     snomel = 370518500.
     *,    clai = 4186.0*0.2
     *,    cww = 4186000.
     *,    asnow = 16.7
     *,    snofac = hltm / ( hltm + snomel*0.001)
     *,    pco2m = 35.
     *,    po2m = 20900.

!-------------------------------------------------------------------
!   VARIABLE DEFINITION
!-------------------------------------------------------------------
!
!     nsoil = number of soil levels
!     npt = number of plant types in biome for carbon cycle
!
!     forcerestore = if true, use Deardorff force-restore for deep soil
!                    thermodynamics
!     bgcyc = if true, do biogeocehmical cycle for carbon
!     prescribed_gw = if true, prescribe soil moisture as function of time
!     doSiBco2 = if true, compute the sib carbon flux
!
!     snomel = heat of fusion of snow (J/m3)
!     clai = canopy specific heat (J/kg/K)
!     cww =  Water specific heat (J/m3/K
!     asnow = fraction snow parameter
!     snofac = 
!     pco2m = co2 partial pressure (Pa)
!     po2m =  O2 partial pressure (Pa)
!
!-------------------------------------------------------------------

!   land domain definition
        integer (kind=int_kind) :: nsib=0
        logical (kind=log_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *          hr_sib=.false.
        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *          lon_sib=0, lat_sib=0
        integer (kind=int_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *          link_hr_sib=0

!   prognostic variables
        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *      tg = 0     !  ground temperature (K)
     *,     tcan = 0   !  canopy temperature (K)
     *,     gs = 0     !  stomatal conductance (m/s)
     *,     tkes = 0   !  turbulence kinetic energy

        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsoil,nsdm_sfc) ::
     *      deept = 0  !  deep soil temperature profile(K)

        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,3,nsdm_sfc) ::
     *      www = 0    !  soil moisture (-)

        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,2,nsdm_sfc) ::
     *      capac = 0  !  liquid water interception storage (m)
     *,     snow = 0   !  snow storage (m)

! boundary conditions - soil and plant physiological parameters

!    time invariant
        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *      chil = 0   !  
     *,     phc = 0    !  
     *,     z2 = 0     !  
     *,     rootd = 0  ! root zone depth (m) 
     *,     satco = 0  !  
     *,     bee = 0    !  
     *,     phsat = 0  !  
     *,     slope = 0  !  
     *,     sodep = 0  ! soil depth (m)
     *,     z1 = 0     !  
     *,     vcover = 0 ! horizontal vegetation coverage (-) 
     *,     poros = 0  ! soil porosity (-) 
     *,     vmax0 = 0  !  
     *,     trop = 0   !  
     *,     respcp = 0 !  
     *,     trda = 0   !  
     *,     trdm = 0   !  
     *,     slti = 0   !  
     *,     shti = 0   !  
     *,     hltii = 0  !  
     *,     effcon = 0 !  
     *,     binter = 0 !  
     *,     gradm = 0  !  
     *,     atheta = 0 !  
     *,     btheta = 0 !  
     *,     hhti = 0   !  
     *,     zm = 0     !  
     *,     wopt = 0   !  
     *,     wsat = 0   !  
     *,     sandfrac = 0   !  
     *,     zs_sib = 0   ! land surface geopotential (m2/s2) 

        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,2,nsdm_sfc) ::
     *      soref = 0  !  soil reflectance (-)

        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,2,2,nsdm_sfc) ::
     *      tran = 0   !  plant transmittance (-)
     *,     ref = 0    !  plant reflectance (-)

!    time varying - monthly
        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,12,nsdm_sfc) ::
     *      green_m = 0  !  green vegetation fraction (-)
     *,     zlt_m = 0    !  leaf area index (m^2/m^2)    
     *,     aparc_m = 0  !      
     *,     rbc_m = 0    !      
     *,     rdc_m = 0    !  
     *,     dd_m = 0     !  
     *,     z0d_m = 0    !  surface roughness length (m)
     *,     gmudmu_m = 0 !  

        real (kind=real_kind), dimension(iim_sfc,jjm_sfc,3,12,nsdm_sfc) ::
     *      www_m = 0    !  prescribed soil moisture (-)

!    time interpolated boundary conditions
        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *      green = 0  !  green vegetation fraction (-)
     *,     zlt = 0    !  leaf area index (m^2/m^2)    
     *,     aparc = 0  !      
     *,     rbc = 0    !      
     *,     rdc = 0    !  
     *,     dd = 0     !  
     *,     z0d = 0    !  surface roughness length (m)
     *,     gmudmu = 0 !  

!    fields calculated and used elsewhere
        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,3,nsdm_sfc) ::
     *      zdepth = 0  !  soil moisture layer capacities (m)

        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsoil,nsdm_sfc) ::
     *      ztdep = 0   !  soil thermal layer depths (m)

        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *      thermk = 0  !  

        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,2,2,nsdm_sfc) ::
     *      radn = 0    ! 

        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,2,nsdm_sfc) ::
     *      radc3 = 0   ! 

        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,2,2,2,nsdm_sfc) ::
     *      radfac = 0  !  
 
!    sib co2 type variables
        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsoil+1,nsdm_sfc) ::
     *      respFactor = 0  !  

!    time invariant boundary condition for carbon cycle
        integer (kind=int_kind), dimension(iim_sfc,jjm_sfc,npt,nsdm_sfc) ::
     *      msibbiome = 0   !  

 	end module sib_parameters

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
