      SUBROUTINE VMFCALZ(PR0,RIBC,VKRMN,DELTA,GRAV
     *,                SH,PS,z0           
     *,                SPDM,SHA,ROS,CU,ct,THVGM,RIB     
     *,                USTAR,VENTMF,THM, tha, zzwind, zztemp
     *,                cuni, cun, ctn, z1z0Urt, z1z0Trt, len ) 

      use kinds
                              
      IMPLICIT none

C*****************************************************************************                                                                       
C     VENTILATION MASS FLUX,Ustar, and transfer coefficients for momentum 
C     and heat fluxes, based on by Louis (1979, 1982), and revised by Holtslag
C     and Boville(1993), and by Beljaars and Holtslag (1991).              
C  
C     Rerences:
C       Beljars and Holtslag (1991): Flux parameterization over land surfaces
C              for atmospheric models. J. Appl. Meteo., 30, 327-341.
C       Holtslag and Boville (1993): Local versus nonlocal boundary-layer 
C              diffusion in a global climate model. J. of Climate, 6, 1825-
C              1842.
C       Louis, J. F., (1979):A parametric model of vertical eddy fluxes in
C              atmosphere. Boundary-Layer Meteo., 17, 187-202.
C       Louis, Tiedke, and Geleyn, (1982): A short history of the PBL
C              parameterization at ECMWF. Proc. ECMWF Workshop on Boundary-
C              Layer parameterization, ECMWF, 59-79.
C
C     General formulation:
C        surface_flux = transfer_coef.*U1*(mean_in_regerence - mean_at_sfc.) 
C     Transfer coefficients for mommentum and heat fluxes are:
C        CU = CUN*Fm, and
C        CT = CTN*Fh
C        where  CUN and CTN are nutral values of momentum and heat transfers,
C           and Fm and Fh are stability functions derived from surface
C           similarity relationships.     
C*****************************************************************************
                                                                           
      integer (kind=int_kind), intent(in) ::
     *     len
      REAL (kind=dbl_kind), intent(in) ::
     *     pr0, ribc, vkrmn, delta, grav                                                           
      REAL (kind=dbl_kind), intent(in), dimension(len) ::            
     *     z0, ROS, SH, PS      
     *,    THM, SHA, SPDM, tha 

      REAL (kind=dbl_kind), intent(out), dimension(len) ::            
     *     CU, THVGM, RIB  
     *,    USTAR, CT 
     *,    VENTMF, cuni

c     local variables
      REAL (kind=dbl_kind), dimension(len) :: 
     *       TEMV, wgm, THGM
     *,      zzwind, zztemp, zrib, cun, ctn
     *,      z1z0U, z1z0Urt, z1z0T, z1z0Trt
     *,      fmomn, fheat, CUI, CTI     
      REAL (kind=dbl_kind) ::            
     *      ribtemp,dm,dh
     *,     bunstablM, bunstablT, cunstablM, cunstablT, bstabl, cstabl
      integer (kind=int_kind) ::            
     *     i          
C      
C  condtants for surface flux functions, according to Holtslag and
c      Boville (1993, J. Climate)

      bunstablM = 10.       ! constants for unstable function
      bunstablT = 15
      cunstablM = 75.
      cunstablT = 75.
      bstabl = 8.           ! constants for stable function
      cstabl = 10.
C                                                                       
      DO 10 I=1,len                                                    
         zrib(i) = zzwind(i) **2 / zztemp(i)                                                    
         WGM(i)  = SHA(i) - SH(i)                                      
C                                                                       
C        SFC-AIR DEFICITS OF MOISTURE AND POTENTIAL TEMPERATURE         
C        WGM IS THE EFFECTIVE SFC-AIR TOTAL MIXING RATIO DIFFERENCE.    
C                                                                       
         THGM(i)  = THA(i)  - THM(i)                                   
         THVGM(i) = THGM(i) + THA(i) * DELTA * WGM(i)                                                                                       
   10 CONTINUE  

c   Ratio of reference height (zwind/ztemp) and roughness length:
      do i = 1, len                !for all grid points
         z1z0U(i) = zzwind(i)/ z0(i)
         z1z0Urt(i) = sqrt( z1z0U(i) )
         z1z0U(i) = log( z1z0U(i) )
         z1z0T(i) = zzwind(i)/ z0(i)
         z1z0Trt(i) = sqrt( z1z0T(i) )
         z1z0T(i) = log( z1z0T(i) )
      enddo

c   Neutral surface transfers for momentum CUN and for heat/moisture CTN:

      do i = 1, len       
         cun(i) = VKRMN*VKRMN / (z1z0U(i)*z1z0U(i) )   !neutral Cm & Ct
         ctn(i) = VKRMN*VKRMN / (z1z0T(i)*z1z0T(i) )
         cuni(i) = z1z0u(i) / vkrmn
      enddo                                                                                                               
C                                                                       
C   SURFACE TO AIR DIFFERENCE OF POTENTIAL TEMPERATURE.            
C   RIB IS THE BULK RICHARDSON NUMBER, between reference height and surface.           
C                                                                       
      DO I=1,len                                                    
         TEMV(i) = THA(i) * SPDM(i) * SPDM(i)                          
         temv(i) = max(0.000001_dbl_kind,temv(i))
         RIB(I) = -THVGM(I) * GRAV * zrib(i) / TEMV(i) 
      enddo                            
   
C   The stability functions for momentum and heat/moisture fluxes as
C   derived from the surface-similarity theory by Luis (1079, 1982), and
C   revised by Holtslag and Boville(1993), and by Beljaars and Holtslag 
C   (1991).
                                                                    
      DO 60 I=1,len 
         if(rib(i).ge.0.0) then                                           
C                                                                       
C        THE STABLE CASE. RIB IS USED WITH AN UPPER LIMIT              
C
            rib(i) = min( rib(i), 0.5_dbl_kind)                   
            fmomn(i) = (1. + cstabl * rib(i) * (1.+ bstabl * rib(i)))
            fmomn(i) = 1. / fmomn(i)
            fmomn(i) = max(0.0001_dbl_kind,fmomn(i))
            fheat(i) = fmomn(i)
                                         
         else                                  
C                                                                       
C        THE UNSTABLE CASE.    
C            
            ribtemp = abs(rib(i))
            ribtemp = sqrt( ribtemp )
            dm = 1. + cunstablM * cun(i) * z1z0Urt(i) * ribtemp
            dh = 1. + cunstablT * ctn(i) * z1z0Trt(i) * ribtemp
            fmomn(i) = 1. - (bunstablM * rib(i) ) / dm
            fheat(i) = 1. - (bunstablT * rib(i) ) / dh
                                                                                   
         END IF    
   60 CONTINUE 

c   surface-air transfer coefficients for momentum CU, for heat and 
c   moisture CT. The CUI and CTI are inversion of CU and CT respectively.

      do i = 1, len
         CU(i) = CUN(i) * fmomn(i) 
         CT(i) = CTN(i) * fheat(i)
         CUI(i) = 1. / CU(i)
         CTI(i) = 1. / CT(i)
      enddo

c   Ustar and ventlation mass flux: note that the ustar and ventlation 
c   are calculated differently from the Deardoff's methods due to their
c   differences in define the CU and CT.
                                                      
      DO 80 I=1,len 
         USTAR(i) = SPDM(i)*SPDM(i)*CU(i) 
         USTAR(i) = sqrt( USTAR(i) ) 
         VENTMF(i)= ROS(i)*CT(i)* SPDM(i)                                        
   80 CONTINUE                                                          
C                                                                       
C   Note there is no CHECK FOR VENTMF EXCEEDS TOWNSENDS(1962) FREE CONVECTION  
C   VALUE, like DEARDORFF EQ(40B), because the above CU and CT included
C   free convection conditions.                                                 
C  
                                                                                                          
      RETURN                                                                                                                                   
      END                                                               
