!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module pre_step_module

!-----------------------------------------------------------------------
! PURPOSE: call a series of subroutines which gets the model ready to
!	take a time step.
!-----------------------------------------------------------------------

        use kinds
        use physical_parameters
        use model_parameters
        use model_control
        use grid_metrics
        use operators
        use wrap_face_data
        use wrap_vertex_data
        use prognostics
        use timer
	use hstatc
        use compute_mass_flux

        implicit none
	private

	integer (kind = int_kind) :: i,j,k,itracer

#ifdef eau_ng
      integer (kind=int_kind):: ielq
#endif

	public :: pre_step

 
!**********************************************************************

        contains

!**********************************************************************


!======================================================================
! BEGINNING OF PRE_STEP
!======================================================================
        subroutine pre_step

        implicit none
        
        real(kind=dbl_kind), dimension(iim,jjm) :: zero2d = c0
        real(kind=dbl_kind), dimension(iim,jjm,nlm) :: zero3d = c0

!-----------------------------------------------------------------------
! PURPOSE: compute mass fluxes, vertical velocity, and averaged 
!	data to cell walls
!-----------------------------------------------------------------------

!------------------------------------------------------------------
! based on the time step, set the appropriate time stepping weights
!------------------------------------------------------------------
        call get_weights

!------------------------------------------------------------------
! average mass to cell corners and wrap (might want to move to post step)
! average other stuff too
!------------------------------------------------------------------
        do k=1,nlm
        do ksdm=1,nsdm

        call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  pthick(:,:,k,n3,ksdm),
     *                  pthick_c(:,:,:,k,ksdm))

        call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  th(:,:,k,n3,ksdm),
     *                  th_c(:,:,:,k,ksdm))

        call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  alphasig(:,:,k,ksdm),
     *                  alphasig_c(:,:,:,k,ksdm))

        call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  alpha(:,:,k,ksdm),
     *                  alpha_c(:,:,:,k,ksdm))

	do itracer=1,ntracer

        call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  tracer(:,:,k,itracer,n3,ksdm),
     *                  tracer_c(:,:,:,k,itracer,ksdm))

	enddo

#ifdef eau_ng
	do ielq = 1, nelqng
         call average_to_cell_corners(
     *                   ipad,
     *                   iim,
     *                   jjm,
     *                   ndata,
     *                   ncorners,
     *                   s_weights(:,:,:,:,ksdm),
     *                   elqng(:,:,k,ielq,n3,ksdm),
     *                   elqng_c(:,:,:,k,ielq,ksdm))
	enddo
#endif

	enddo
	enddo

	call wrap_vertices(1,nlm,pthick_c   (:,:,:,:,:))
        call wrap_vertices(1,nlm,th_c       (:,:,:,:,:))
        call wrap_vertices(1,nlm,alphasig_c (:,:,:,:,:))
        call wrap_vertices(1,nlm,alpha_c    (:,:,:,:,:))
	do itracer=1,ntracer
	call wrap_vertices(1,nlm,tracer_c   (:,:,:,:,itracer,:))
	enddo
	        
#ifdef eau_ng
      do ielq = 1, nelqng
         call wrap_vertices(1,nlm,elqng_c(:,:,:,:,ielq,:))
      enddo
#endif

!------------------------------------------------------------------
! begin loop over ksdm
!------------------------------------------------------------------
        do ksdm=1,nsdm

!-----------------------------------------------------------------------
! begin loop over vertical level k
!-----------------------------------------------------------------------
        call timer_start('mass flux')
        do k=1,nlm

!-----------------------------------------------------------------------
! compute mass flux at cell walls
!-----------------------------------------------------------------------

        call get_mass_flux(
     *            ksdm,
     *            pthick_c      (  :,:,:,k     ,ksdm),
     *            u             (:,:,:,:,k,n3  ,ksdm),
     *            mass_flux_c   (:,:,:,:,k,n3_f,ksdm),
     *            div_mass_flux (    :,:,k,n3_f,ksdm)  )

        enddo   ! do k=1,nlm
        call timer_stop('mass flux')

!-----------------------------------------------------------------------
! compute vertical velocity
!-----------------------------------------------------------------------
        call timer_start('dsigdt')
#ifdef no_pbl

        call get_dsigdt(
     *           div_mass_flux(:,:,:,n3_f,ksdm),
     *           dsigdt(:,:,:,ksdm) )

#else

#ifdef fluxcorrectedtransport
        call get_dsigdt(
     *           zero2d(:,:),
     *           zero2d(:,:),
     *           zero2d(:,:),
     *           div_mass_flux(:,:,:,n3_f,ksdm),
     *           dsigdtu(:,:,:,ksdm) )
        call get_dsigdt(
     *           entrainment(:,:,ksdm),
     *           cumulusmf(:,:,ksdm),
     *           pleak(:,:,ksdm),
     *           zero3d(:,:,:),
     *           dsigdte(:,:,:,ksdm) )
      dsigdt = dsigdtu + dsigdte
#else
        call get_dsigdt(
     *           entrainment(:,:,ksdm),
     *           cumulusmf(:,:,ksdm),
     *           pleak(:,:,ksdm),
     *           div_mass_flux(:,:,:,n3_f,ksdm),
     *           dsigdt(:,:,:,ksdm) )
#endif

#endif
        call timer_stop('dsigdt')


        enddo   ! do ksdm = 1,nsdm

!-----------------------------------------------------------------------
! wrap data that will get averaged to cell walls
!-----------------------------------------------------------------------
        call timer_start('wrap')
        call wrap_pre_step
        call timer_stop('wrap')

!-----------------------------------------------------------------------
! average to cell walls
!-----------------------------------------------------------------------
        call timer_start('averaging')
        do ksdm = 1,nsdm
        do k=1,nlm

        call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  dsigdt(:,:,k,ksdm),
     *                  dsigdt_c(:,:,:,k,ksdm))

        call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  invpthick(:,:,k,n3,ksdm),
     *                  invpthick_c(:,:,:,k,ksdm))

        enddo

#ifndef no_pbl
        call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),  
     *                  entrainment(:,:,ksdm),
     *                  entrainment_c(:,:,:,ksdm))

        call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  pleak(:,:,ksdm),  
     *                  pleak_c(:,:,:,ksdm))  

        call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  cumulusmf(:,:,ksdm),  
     *                  cumulusmf_c(:,:,:,ksdm))  

        call average_to_cell_corners(
     *                  ipad,
     *                  iim,
     *                  jjm,
     *                  ndata,
     *                  ncorners,
     *                  s_weights(:,:,:,:,ksdm),
     *                  drag(:,:,ksdm),  
     *                  drag_c(:,:,:,ksdm))  
#endif

        enddo

        call wrap_vertices(1,nlm,dsigdt_c   (:,:,:,:,:))
        call wrap_vertices(1,nlm,invpthick_c(:,:,:,:,:))
#ifndef no_pbl
        call wrap_vertices(1,1,entrainment_c(:,:,:,:))
        call wrap_vertices(1,1,pleak_c      (:,:,:,:))
	call wrap_vertices(1,1,cumulusmf_c  (:,:,:,:))
	call wrap_vertices(1,1,drag_c       (:,:,:,:))
#endif

        call timer_stop('averaging')


        end subroutine pre_step
!======================================================================
! END OF PRE_STEP
!======================================================================


!===========================================================================
! BEGINNING OF WRAP_PRE_STEP
!===========================================================================
        subroutine wrap_pre_step
        implicit none
#ifdef fluxcorrectedtransport
        integer (kind = int_kind), parameter :: nvar = 3*nlm+4
#else
        integer (kind = int_kind), parameter :: nvar = 1*nlm+4
#endif

#ifdef no_pbl

        call wrap ( max_level,
     *             iim,
     *             jjm,
     *             nlm,
     *             nsdm,
     *             dsigdt(1,1,1,1))
#else
        real (kind = dbl_kind) :: buffer(iim,jjm,nvar,nsdm) = 0

        i = 0
        j = 0

        do ksdm =1 ,nsdm
        do k=1,nlm
          j = i + k
          buffer(  1:2,    :,j,ksdm) = dsigdt(  1:2,    :,k,ksdm)
          buffer(iim-1,    :,j,ksdm) = dsigdt(iim-1,    :,k,ksdm)
          buffer(    :,  1:2,j,ksdm) = dsigdt(    :,  1:2,k,ksdm)
          buffer(    :,jjm-1,j,ksdm) = dsigdt(    :,jjm-1,k,ksdm)
        enddo
        enddo
        
#ifdef fluxcorrectedtransport        
        do ksdm =1 ,nsdm
        do k=1,nlm
          j = nlm + k
          buffer(  1:2,    :,j,ksdm) = dsigdtu(  1:2,    :,k,ksdm)
          buffer(iim-1,    :,j,ksdm) = dsigdtu(iim-1,    :,k,ksdm)
          buffer(    :,  1:2,j,ksdm) = dsigdtu(    :,  1:2,k,ksdm)
          buffer(    :,jjm-1,j,ksdm) = dsigdtu(    :,jjm-1,k,ksdm)
        enddo
        enddo
        do ksdm =1 ,nsdm
        do k=1,nlm
          j = 2*nlm + k
          buffer(  1:2,    :,j,ksdm) = dsigdte(  1:2,    :,k,ksdm)
          buffer(iim-1,    :,j,ksdm) = dsigdte(iim-1,    :,k,ksdm)
          buffer(    :,  1:2,j,ksdm) = dsigdte(    :,  1:2,k,ksdm)
          buffer(    :,jjm-1,j,ksdm) = dsigdte(    :,jjm-1,k,ksdm)
        enddo
        enddo
#endif
        j=j+1
        do ksdm =1 ,nsdm
          buffer(  1:2,    :,j,ksdm) = entrainment(  1:2,    :,ksdm)
          buffer(iim-1,    :,j,ksdm) = entrainment(iim-1,    :,ksdm)
          buffer(    :,  1:2,j,ksdm) = entrainment(    :,  1:2,ksdm)
          buffer(    :,jjm-1,j,ksdm) = entrainment(    :,jjm-1,ksdm)
        enddo

        j=j+1
        do ksdm =1 ,nsdm
          buffer(  1:2,    :,j,ksdm) = pleak(  1:2,    :,ksdm)
          buffer(iim-1,    :,j,ksdm) = pleak(iim-1,    :,ksdm)
          buffer(    :,  1:2,j,ksdm) = pleak(    :,  1:2,ksdm)
          buffer(    :,jjm-1,j,ksdm) = pleak(    :,jjm-1,ksdm)
        enddo

        j=j+1
        do ksdm =1 ,nsdm
          buffer(  1:2,    :,j,ksdm) = cumulusmf(  1:2,    :,ksdm)
          buffer(iim-1,    :,j,ksdm) = cumulusmf(iim-1,    :,ksdm)
          buffer(    :,  1:2,j,ksdm) = cumulusmf(    :,  1:2,ksdm)
          buffer(    :,jjm-1,j,ksdm) = cumulusmf(    :,jjm-1,ksdm)
        enddo

        j=j+1
        do ksdm =1 ,nsdm
          buffer(  1:2,    :,j,ksdm) = drag(  1:2,    :,ksdm)
          buffer(iim-1,    :,j,ksdm) = drag(iim-1,    :,ksdm)
          buffer(    :,  1:2,j,ksdm) = drag(    :,  1:2,ksdm)
          buffer(    :,jjm-1,j,ksdm) = drag(    :,jjm-1,ksdm)
        enddo

        if(j.ne.nvar) then
          write(6,*) ' stop 003'
          stop
        endif

        call wrap ( max_level,
     *             iim,
     *             jjm,
     *             nvar,
     *             nsdm,
     *             buffer(1,1,1,1))

        i = 0
        j = 0

        do ksdm = 1,nsdm
        do k=1,nlm
          j = i + k
          dsigdt(    1,    :,k,ksdm) = buffer(    1,    :,j,ksdm)
          dsigdt(iim  ,    :,k,ksdm) = buffer(iim  ,    :,j,ksdm)
          dsigdt(    :,    1,k,ksdm) = buffer(    :,    1,j,ksdm)
          dsigdt(    :,jjm  ,k,ksdm) = buffer(    :,jjm  ,j,ksdm)
        enddo
        enddo

#ifdef fluxcorrectedtransport
        do ksdm = 1,nsdm
        do k=1,nlm
          j = nlm + k
          dsigdtu(    1,    :,k,ksdm) = buffer(    1,    :,j,ksdm)
          dsigdtu(iim  ,    :,k,ksdm) = buffer(iim  ,    :,j,ksdm)
          dsigdtu(    :,    1,k,ksdm) = buffer(    :,    1,j,ksdm)
          dsigdtu(    :,jjm  ,k,ksdm) = buffer(    :,jjm  ,j,ksdm)
        enddo
        enddo
        do ksdm = 1,nsdm
        do k=1,nlm
          j = 2*nlm + k
          dsigdte(    1,    :,k,ksdm) = buffer(    1,    :,j,ksdm)
          dsigdte(iim  ,    :,k,ksdm) = buffer(iim  ,    :,j,ksdm)
          dsigdte(    :,    1,k,ksdm) = buffer(    :,    1,j,ksdm)
          dsigdte(    :,jjm  ,k,ksdm) = buffer(    :,jjm  ,j,ksdm)
        enddo
        enddo
#endif

        j=j+1
        do ksdm = 1,nsdm
          entrainment(    1,    :,ksdm) = buffer(    1,    :,j,ksdm)
          entrainment(iim  ,    :,ksdm) = buffer(iim  ,    :,j,ksdm)
          entrainment(    :,    1,ksdm) = buffer(    :,    1,j,ksdm)
          entrainment(    :,jjm  ,ksdm) = buffer(    :,jjm  ,j,ksdm)
        enddo

        j=j+1
        do ksdm = 1,nsdm
          pleak(    1,    :,ksdm) = buffer(    1,    :,j,ksdm)
          pleak(iim  ,    :,ksdm) = buffer(iim  ,    :,j,ksdm)
          pleak(    :,    1,ksdm) = buffer(    :,    1,j,ksdm)
          pleak(    :,jjm  ,ksdm) = buffer(    :,jjm  ,j,ksdm)
        enddo

        j=j+1
        do ksdm = 1,nsdm
          cumulusmf(    1,    :,ksdm) = buffer(    1,    :,j,ksdm)
          cumulusmf(iim  ,    :,ksdm) = buffer(iim  ,    :,j,ksdm)
          cumulusmf(    :,    1,ksdm) = buffer(    :,    1,j,ksdm)
          cumulusmf(    :,jjm  ,ksdm) = buffer(    :,jjm  ,j,ksdm)
        enddo


        j=j+1
        do ksdm = 1,nsdm
          drag(    1,    :,ksdm) = buffer(    1,    :,j,ksdm)
          drag(iim  ,    :,ksdm) = buffer(iim  ,    :,j,ksdm)
          drag(    :,    1,ksdm) = buffer(    :,    1,j,ksdm)
          drag(    :,jjm  ,ksdm) = buffer(    :,jjm  ,j,ksdm)
        enddo


        if(j.ne.nvar) then
          write(6,*) ' stop 004'
          stop
        endif
#endif

        end subroutine wrap_pre_step
!===========================================================================
! END OF WRAP_PRE_STEP
!===========================================================================

        end module pre_step_module

!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
