!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

	module prognostics

!-----------------------------------------------------------------
!
!   contains arrays related to the prognostic variables, and
!       those diagnostic variables derived from the prognostic
!       variables
!
!---- modifications:

!     added the arrays elqng (at ntprog) and elqng_f (at nttend) for
!     the eau_liqng parameterization of fractional cloudiness. nelqng
!     is equal to 6 and is defined in model_parameters.
!     Laura D. Fowler/mcbug (09-06-02).

!-----------------------------------------------------------------

	use kinds
	use physical_parameters
	use model_parameters
	use domain_decomposition
	use sigma_coordinate
	use tracer_names

	implicit none
	save

!-----------------------------------------------------------------
!   prognostics ntprog time levels
!-----------------------------------------------------------------
	real (kind=dbl_kind), dimension(iim,jjm,ntprog,nsdm) ::
     *                  ps
#ifndef no_pbl
     *,                 pb
#endif

	real (kind=dbl_kind), dimension(iim,jjm,nlm,ntprog,nsdm) ::
     *			eta,
     *			div,
     *                  th,
     *                  pthick,
     *			invpthick

        real (kind=dbl_kind) ::
     *                  u   (nspace, ncorners, iim, jjm, nlm, ntprog, nsdm)

	real (kind=dbl_kind), dimension(iim,jjm,nlm,ntracer,ntprog,nsdm) ::
     *			tracer

#ifdef eau_ng
      real (kind=dbl_kind), dimension(iim,jjm,nlm,nelqng,ntprog,nsdm)::
     *                  elqng

      real (kind=dbl_kind), dimension(iim,jjm,nlmp1,nelqngp1,ntprog,nsdm)::
     *                  elqngp1
#endif

!-----------------------------------------------------------------
!   prognostics 1 time level
!-----------------------------------------------------------------
#ifndef no_pbl
	real (kind=dbl_kind), dimension(iim,jjm,nlm,nlm,nsdm) ::
     *                  cke
#endif

!-----------------------------------------------------------------
!   time tendency of prognostics at nttend time levels
!-----------------------------------------------------------------
        real (kind=dbl_kind), dimension(iim,jjm,nttend,nsdm) ::
     *                  ps_f
#ifndef no_pbl
        real (kind=dbl_kind), dimension(iim,jjm,nttend,nsdm) ::
     *                  pb_f,
     *			pb_dsigdt_f
#endif

	real (kind=dbl_kind), dimension(iim,jjm,nlm,nttend,nsdm) ::
     *			eta_f,
     *			div_f,
     *                  th_f

        real (kind=dbl_kind) ::
     *                  u_f (nspace, ncorners, iim, jjm, nlm, nttend, nsdm)

        real (kind=dbl_kind), dimension(iim,jjm,nlm,ntracer,nttend,nsdm) ::
     *                  tracer_f

#ifdef eau_ng
        real (kind=dbl_kind), dimension(iim,jjm,nlm,nelqng,nttend,nsdm) ::
     *                  elqng_f
#endif

!-----------------------------------------------------------------
!   time tendency of fields related to mass flux
!-----------------------------------------------------------------
        real (kind=dbl_kind) ::
     *                  mass_flux_c (nspace,ncorners,iim,jjm,nlm,nttend,nsdm)

	real (kind=dbl_kind), dimension(iim,jjm,nlm,nttend,nsdm) ::
     *			div_mass_flux

!-----------------------------------------------------------------
!   data averaged to cell corners
!-----------------------------------------------------------------
        real (kind=dbl_kind), dimension(ncorners,iim,jjm,nlm,nsdm) ::
     *                  invpthick_c,
     *                  pthick_c

        real (kind=dbl_kind), dimension(ncorners,iim,jjm,nlm,nsdm) ::
     *                  th_c,
     *                  alphasig_c,
     *                  alpha_c

        real (kind=dbl_kind), dimension(ncorners,iim,jjm,nlm,ntracer,nsdm) ::
     *                  tracer_c

#ifdef eau_ng
        real (kind=dbl_kind), dimension(ncorners,iim,jjm,nlm,nelqng,nsdm) ::
     *                  elqng_c
#endif

        real (kind=dbl_kind), dimension(ncorners,iim,jjm,nlmp1,nsdm) ::
     *                  dsigdt_c

#ifndef no_pbl
        real (kind=dbl_kind), dimension(ncorners,iim,jjm,nsdm) ::
     *                  entrainment_c,
     *                  pleak_c,
     *                  psb_smth_c,
     *                  cumulusmf_c,
     *                  drag_c
#endif

!-----------------------------------------------------------------
!   variables at grid corners
!-----------------------------------------------------------------
        real (kind=dbl_kind) ::
     *        u_src     ( nspace, ncorners, iim, jjm, nlm, nsdm)

!-----------------------------------------------------------------
!   sources for the prognostic variables
!-----------------------------------------------------------------
#ifndef no_pbl
	real (kind=dbl_kind), dimension(iim,jjm,nsdm) ::
     *			entrainment,
     *                  cumulusmf,
     *                  pleak,
     *                  psb_smth,
     *                  psb_lapl
#endif

	real (kind=dbl_kind), dimension(iim,jjm,nlm,nsdm) ::
     *                  th_src,
     *                  th_res

	real (kind=dbl_kind), dimension(iim,jjm,nlm,ntracer,nsdm) ::
     *			tracer_src

#ifdef eau_ng
        real (kind=dbl_kind), dimension(iim,jjm,nlm,nelqng,nsdm) ::
     *                  elqng_src
#endif

#ifndef no_pbl
	real (kind=dbl_kind), dimension(iim,jjm,nlm,nlm,nsdm) ::
     *			cdwork
#endif

!-----------------------------------------------------------------
!   diagnostically determined variables at layer centers
!-----------------------------------------------------------------
	real (kind=dbl_kind), dimension(iim,jjm,nlm,nsdm) ::
     *			relative,
     *			ke,
     *			psi,
     *			chi,
     *                  phi,
     *                  bpg,
     *                  alphasig,
     *                  alpha,
     *                  plg,
     *                  qs,
     *                  ssl

!-----------------------------------------------------------------
!   diagnostically determined variables at layer edges
!-----------------------------------------------------------------
#ifndef no_pbl
#ifdef fluxcorrectedtransport
	real (kind=dbl_kind), dimension(iim,jjm,nlmp1,nsdm) ::
     *                  dsigdtu,
     *                  dsigdte
#endif
#endif
	real (kind=dbl_kind), dimension(iim,jjm,nlmp1,nsdm) ::
     *                  phil2,
     *                  thl2,
     *                  bpgl2,
     *                  dsigdt,
     *                  plgl2,
     *                  qsl2,
     *                  ssll2,
     *                  tl2,
     *			ril2

        real (kind=dbl_kind), dimension(iim,jjm,nlmp1,ntracer,nsdm) ::
     *                  tracerl2

#ifdef eau_ng
        real (kind=dbl_kind), dimension(iim,jjm,nlmp1,nelqng,nsdm)::
     *                  elqngl2
#endif

!-----------------------------------------------------------------
!   passive tracer variables
!-----------------------------------------------------------------

        real (kind=dbl_kind), dimension(iim,jjm,npassive,12,nsdm) ::
     *                  tracer_dat

        real (kind=dbl_kind), dimension(iim,jjm,npassive,nsdm) ::
     *                  tracer_flux

#ifndef no_pbl
!-----------------------------------------------------------------
!   surface stress related variables
!-----------------------------------------------------------------
	real (kind=dbl_kind), dimension(iim,jjm,nsdm) ::
     *                  drag,
     *                  spdm,
     *                  shear1
#endif

!-----------------------------------------------------------------
!   boundary conditions
!-----------------------------------------------------------------
	real (kind=dbl_kind), dimension(iim,jjm,nsdm) ::
     *			phis,
     *			f,
     *                  tpgvar

	real (kind=dbl_kind), dimension(ncorners,iim,jjm,nsdm) ::
     *			tpgvar_c



        real (kind=dbl_kind), dimension(iim,jjm,nlm,nsdm) :: 
     *             delt_rad     !temperature tendency           (K/s).


!-----------------------------------------------------------------
!
!   VARIABLE DEFINITION
!     syntax:  all variables which reside at layer edges end with 'l2'
!              all time tendencies end with '_f'
!
!       im            :  number of cells in 'x' direction (defined in model_paramaters.F)
!       jm            :  number of cells in 'y' direction (defined in model_paramater.F)
!       nlm           :  number of vertical levels (defined in sigma.F)
!       ntprog        :  number of time levels that prognostics are saved
!       nttend        :  number of time levels that time tendency of prognostics are saved
!       ntracer       :  number of tracer fields
!       ps            :  surface pressure (Pa)
!       pb            :  boundary layer top pressure (Pa) 
!       eta           :  absolute vorticity (1/s)
!       div           :  divergence (1/s)
!       th            :  potential temperature (K)
!       cke           :  cumulus kinetic energy
!       tracer        :  tracer fields (such as water vapor, CO2, cloud ice, ect)
!       ps_f          :  d/dt (ps)   (Pa/s)
!       pb_f          :  d/dt (pb)   (Pa/s)
!       eta_f         :  d/dt (eta)  (1/s2)
!       div_f         :  d/dt (div)  (1/s2)
!       th_f          :  d/dt (pthick*th) (Pa*K/s)
!       tracer_f      :  d/dt (pthick*tracer) (Pa * tracer units / s)
!       div_mass_flux     :  (divergence of mass_flux) * dsig  (Pa/s)
!       entrainment   :  PBL top entrainment (kg/m2/s)
!       cumulusmf     :  PBL top cumulus mass flux (kg/m2/s)
!       pleak         :  PBL top correction velocity (kg/m2/s)
!       th_src        :  summation of all potential temperature source (K/s)
!       tracer_src    :  summation of all sources for each passive tracers
!       cdwork        :  cloud work function (cke source)
!       th_res        :  potential temperature source used in Held/Suarez test case (K/s)
!       relative      :  relative vorticity (1/s) (defined as eta - f)
!       ke            :  kinetic energy (m2/s2)
!       psi           :  streamfunction (m2/s)
!       chi           :  velocity potential (m2/s)
!       phi           :  geopotential at layer center (m2/s2)
!       bpg           :  (pressure at layer center / reference pressure) ** kappa (special averaging)
!       alphasig      :  specific volume * sigma, layer center, used for pressure gradient calc
!       alpha         :  specific volume (m^3/kg)
!       pthick        :  pressure thickness (Pa), Arakawa's PI
!       invpthick     :  1/pthick (1/Pa)
!       plg           :  pressure at layer centers (Pa)
!	qs	      :  saturated mixing ratio (kg/kg)
!       ssl           :  dry static energy (J/kg)
!       thl2          :  potential temperature averaged to layer edges
!       tracerl2      :  tracer fields averaged to layer edges
!       bpgl2         :  bpg averaged to layer edges
!       plgl2         :  pressure at layer edges (Pa)
!	qsl2          :  saturated mixing ratio evaluated at layer edge (kg/kg)
!       ssll2         :  dry static energy computed at layer edge (J/kg)
!       tl2           :  temperature computed at layer edge (K)
!       dsigdt        :  vertical velocity using time level n tendencies, d/dt(pthick*sigma), (Pa/s)
!       phis          :  surface geopotential (m2/s2)
!       f             :  Coriolis force (1/s)
!       tpgvar        :  topographic variance (??)
!       shearl2       :  wind shear at layer edges, d/dz|V|, (1/s)
!       rhol2         :  density at layer edges (kg/m3)
!       ril2          :  Richardson # at layer edges (unitless)
!       diffl2        :  diffusion at layer edges
!       buoyl2        :  BV buoyancy frequency at layer edges (1/s)
!       drag          :  surface drag
!       spdm          :  PBL wind speed (m/s)
!       shear1        :  PBL top wind shear (1/s2)
!       tracer_dat    :  monthly passive tracer surface flux (kg/m2/s)
!       tracer_flux   :  time-interpolated tracer surface flux (kg/m2/s)
#ifdef eau_ng
!       nelqng        :  number of eau_liqng prognostic variables.
!       elqng         :  eau_liqng prognostic variables.
!       elqngl2       :  eau_liqng variables defined at layer edges.
!       elqng_f       :  d/dt (pthick*elqng) (Pa * elqng units / s)
!       elqng_src     :  summation of all sources for each elqng variable.
#endif
!----------------------------------------------------------------------------------------------

!*****************************************************************

	contains

!*****************************************************************


!======================================================================
! BEGINNING OF INIT_PROGNOSTICS
!======================================================================

	subroutine init_prognostics

	implicit none

        ps = 1.0e5
        eta = 1.0e-8
        div = 1.0e-8
        th = 300.0
        tracer = c1
        ps_f = c0
        eta_f = c0
        div_f = c0
        th_f = c0
        tracer_f = c0
	div_mass_flux = c0
        th_src = c0
        th_res = c0
        tracer_src = c0
        relative = c0
        ke = c0
        psi = c0
        chi = c0
        phi = c0
        bpg = c0
        alphasig = c0
        alpha = c0
        pthick = c0
        invpthick = c0

	mass_flux_c = c0
        pthick_c = c0
        invpthick_c = c0
        dsigdt_c = c0
	u = c0
	u_f = c0
	u_src = c0

        plg = c0
        qs = c0
        ssl = c0
        thl2 = c0
        bpgl2 = c0
        dsigdt = c0
        plgl2 = c0
        qsl2 = c0
        ssll2 = c0
        tl2 = c0
	ril2 = c0
        tracerl2 = c0
        phis = c0
        f = c0
        tpgvar = c0
	tpgvar_c = c0

#ifdef eau_ng
        elqng     = c0
        elqngl2   = c0
	  elqng_f   = c0
	  elqng_c   = c0
	  elqng_src = c0
#endif

#ifdef passive_tracers 
        tracer_dat  = c0
        tracer_flux = c0
#endif

#ifndef no_pbl
        cke = c0
        pb_f = c0
    	pb_dsigdt_f = c0
        entrainment = c0
        cumulusmf = c0
        pleak = c0
        cdwork = c0
        entrainment_c = c0
        pleak_c = c0
        cumulusmf_c = c0
        pb = 0.9*ps
        drag = c0
        spdm = c0
        shear1 = c0
        drag_c = c0
#endif

	end subroutine init_prognostics

!======================================================================
! END OF INIT_PROGNOSTICS
!======================================================================


	end module prognostics

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
