!-----------------------------------------------------------------------

      subroutine rad_cld
     +               (   nlen ,     len ,   nlm
     +,                   cld ,     clu , effrw
     +,                 effri ,  acldmx ,  acld
     +,               effrw_r , effri_r
     +               )
     
      use kinds

      implicit none

!-----------------------------------------------------------------------
! REFERENCES:
! rad_cld calculates the cloud fraction needed by the radiation codes.
! When the radiation codes handle fractional cloudiness, this subroutine
! will need to be removed or modified.
! Laura D. Fowler/slikrock (02-01-01).

! send comments to laura@atmos.colostate.edu

! MODIFICATIONS:
!     none.

! SUBROUTINES CALLED:
!     none.

! FUNCTIONS CALLED:
!     none.

! INCLUDED COMMON BLOCKS:
!     none.

! ARGUMENT LIST VARIABLES:
!     INPUT ARGUMENTS:
!     ----------------
      integer (kind=int_kind), intent(in)::
     &  nlen    !length of horizontal domain with ghost cells.
     &, len     !length of horizontal domain without ghost cells.
     &, nlm     !number of layers.

      real (kind=dbl_kind), intent(in), dimension(nlen,nlm)::
     &  clu     !convective cloud fraction                         (-).
     &, cld     !large-scale saturation cloud fraction             (-).
     &, effrw   !effective radius of water droplets               (mu).
     &, effri   !effective radius of ice crystals                 (mu).

!     OUTPUT ARGUMENTS:
!     -----------------
      real (kind=dbl_kind), intent(out), dimension(len)::
     &  acldmx  !maximum "radiative" cloud fraction                (-).

      real (kind=dbl_kind), intent(out), dimension(len,nlm+1)::
     &  acld    !"radiative" cloud fraction                        (-).
     &, effrw_r !effective radius of water droplets               (mu).
     &, effri_r !effective radius of ice crystals                 (mu).

! LOCAL VARIABLES:

      integer (kind=int_kind)::
     &  i, l, nlm_rad

!----------------------------------------------------------------------

      nlm_rad = nlm+1
      
!---- cloud fraction:

      acldmx(:) = 0.
      do i = 1, len

         acld(i,1)         = 0.
         acld(i,2:nlm_rad) = cld(i,1:nlm) + clu(i,1:nlm)

#ifndef overlap
         do l = 1, nlm_rad
            if(acld(i,l) .ge. 0.5) acld(i,l) = 1.0
            if(acld(i,l) .lt. 0.5) acld(i,l) = 0.0
         enddo
#endif

         do l = 1, nlm_rad
            acldmx(i) = max(acldmx(i),acld(i,l))
         enddo

      enddo

!---- effective radius for cloud water and cloud ice:

      do i = 1, len
         effrw_r(i,1) = effrw(i,1)
         effri_r(i,1) = effri(i,1)
         do l = 2, nlm_rad
            effrw_r(i,l) = effrw(i,l-1)
            effri_r(i,l) = effri(i,l-1)
         enddo
!        do l = 1, nlm_rad
!           effrw_r(i,l) = 10.0_dbl_kind
!           effri_r(i,l) = 40.0_dbl_kind
!        enddo
      enddo

      return
      end subroutine rad_cld
      
!----------------------------------------------------------------------
      
