!-----------------------------------------------------------------------

      subroutine rad_coszen
     +               (   len ,   cosd ,   sind
     +,               tofday , solcon , rsdist
     +,               rotper ,     pi ,    lat
     +,                  lon ,     s0 ,   cosz
     +,                 sfac
     +               )

      use kinds

      implicit none

!-----------------------------------------------------------------------
! REFERENCES:
! rad_coszen calculates the cosine of the solar zenith angle and the
! fraction of daytime. written using coszen.F.
! Laura D. Fowler/slikrock (02-01-01).

! MODIFICATIONS:
! * changed declarations to adapt the code from BUGS4 to BUGS5.
!   Laura D. Fowler/slikrock (02-01-00).

! SUBROUTINES CALLED:
!     none.

! FUNCTIONS CALLED:
!     none.

! INCLUDED COMMON BLOCKS:
!     none.

! ARGUMENT LIST VARIABLES:
!     INPUT ARGUMENTS:
!     ----------------
      integer (kind=int_kind) ::
     &  len    !length of horizontal domain.

      real (kind=dbl_kind), intent(in)::
     &  cosd   !
     &, sind   !
     &, pi     !
     &, tofday !time of day                                        (-).
     &, solcon !solar constant                                 (W/m^2).
     &, rsdist !square of ratio of orbital distance to mean           .
     &, rotper !number of hours in a day                           (-).
     
      real (kind=dbl_kind), dimension(len)::
     &  lat    !latitude grid                                      (-).
     &, lon    !longitude grid                                     (-).

!     OUTPUT ARGUMENTS:
!     -----------------
      real (kind=dbl_kind), intent(out)::
     &  s0     !solar constant                                 (W/m^2).

      real (kind=dbl_kind), intent(out), dimension(len)::
     &  cosz   !cosine of solar zenith angle                       (-).
     &, sfac   !fraction of daytime                                (-).   

! LOCAL VARIABLES:
      integer (kind=int_kind)::
     &  i

      real (kind=dbl_kind)::
     &  czcon, pi2
     
      real (kind=dbl_kind), dimension(len)::
     &  temp

!-----------------------------------------------------------------------

!     write(6,*) '--- enter rad_coszen:'

      s0  = solcon/rsdist
      pi2 = 2*pi

      do i = 1, len
         temp(i) = (tofday / rotper * pi2) + (lon(i)+pi)
         temp(i) = cos(temp(i))
         czcon   = sin(lat(i))*sind                                         
         cosz(i) = cosd*temp(i)
!        write(6,*) cosz(i)
         cosz(i) = czcon + cos(lat(i))*cosz(i)
         sfac(i) = 1.0_dbl_kind
      enddo

!---- note: slr needs to be modified to accomodate the difference in
!     s0 between the original radiation code and that in BUGS2.
!     sfac(1:len) = sfac(1:len) * s0/1339.945

!     write(6,*) '--- end rad_coszen'

      return
      end subroutine rad_coszen
      
!-----------------------------------------------------------------------      
