! CVS:  $Id: rad_gascon_ckd.F,v 1.1 2003/09/03 16:57:25 laura Exp $
! CVS:  $Name:  $ 

!-----------------------------------------------------------------------

      subroutine gascon_ckd
     +              (ncol , nlm, ib,   pp
     +,               ppl ,  dp, tt, rmix
     +,               tgm
     +              )

      use kinds

      implicit none
      
!-----------------------------------------------------------------------
! REFERENCES:
! Parameterized CKD_2.1 continuum absorption.
! adapted from the Fu-Liou 4-stream radiative transfer model original
! code by Fred Rose. use radparams_0898. 
! Phil Partain/graben (04/04/00).

! send comments to partain@atmos.colostate.edu.

! MODIFICATIONS:
! * changed declarations to adapt the code from BUGS4 to BUGS5.
!   Laura D. Fowler/slikrock (02-01-00).

! SUBROUTINES CALLED:
!     none.

! FUNCTIONS CALLED:
!     none.

! INCLUDED COMMONS:
!     none.

! ARGUMENT LIST VARIABLES:
!     INPUT ARGUMENTS:
!     ----------------
      integer (kind=int_kind), intent(in)::
     &  ncol  !Length of sub-domain.
     &, nlm   !Number of layers.
     &, ib    !Spectral interval.

      real (kind=dbl_kind), intent(in), dimension(ncol,nlm)::
     &  ppl   !Pressure                                          (hPa).
     &, dp    !Pressure thickness                                (hPa).
     &, tt    !Temperature                                         (K).
     &, rmix  !Water vapor mixing ratio                        (kg/kg).

!     OUTPUT ARGUMENTS:
!     -----------------
      real (kind=dbl_kind), intent(out), dimension(ncol,nlm)::
     &  tgm   !Water vapor absorption in continuum                 (-).

! LOCAL VARIABLES:

      integer (kind=int_kind)::
     &  i,l

      integer (kind=int_kind), dimension(18)::
     &  iflb
      data iflb /6*0,12,11,10,9,8,7,6,5,4,3,2,1/

      real (kind=dbl_kind):: 
     &  dz , amnt , patm,tv,parm_ckd

      real (kind=dbl_kind), dimension(ncol,nlm+1)::
     &  pp     
 
!-----------------------------------------------------------------------

      tgm(:,:) = 0. 
      if( iflb(ib) .eq. 0) return
 
      do i = 1, ncol
         do l = 1, nlm
            if(rmix(i,l).gt.0.0) then
               amnt     = 1.02*dp(i,l)*rmix(i,l)
               patm     = ppl(i,l) /1013.25
               tv       = tt(i,l)*(1+.61*rmix(i,l))
               dz       = 29.3*tv*log(pp(i,l+1)/pp(i,l))*0.001
               tgm(i,l) = parm_ckd(iflb(ib),amnt,patm,tt(i,l),dz)
            endif
         enddo
      enddo
 
      return
      end subroutine gascon_ckd

!-----------------------------------------------------------------------

      module cont_coef

      use kinds
      implicit none

!----------------------------------------------------------------------

      integer (kind=int_kind), parameter ::
     &  ncoef = 7
     &, nband = 12

      real (kind=dbl_kind), dimension(ncoef,nband)::
     &  aa0,aa1,aa2,aa3,aa4
 
      data aa0/ !! ORIGINAL !! pres atm
     x -1.00e+01, 0.000e+00, 0.000e+00, 0.000e+00,
     x 0.000e+00, 0.000e+00, 0.000e+00,
     x 3.457e+00, 9.977e-01,-6.202e-03, 1.822e+00,
     x 8.200e+00, 8.343e-03, 1.413e-02,
     x 2.327e+00, 1.011e+00,-7.050e-03, 1.368e+00,
     x 1.508e+01,-1.747e-03, 1.271e-01,
     x 6.139e+00, 1.015e+00,-1.505e-02, 5.499e-01,
     x 6.284e+00,-7.911e-03, 5.933e-01,
     x 9.234e+00, 1.003e+00,-2.254e-02, 1.204e-01,
     x 3.237e-01,-1.697e-03, 9.105e-01,
     x 9.426e+00, 9.999e-01,-2.438e-02, 2.410e-02,
     x-7.441e-01, 4.155e-05, 9.880e-01,
     x 8.408e+00, 1.000e+00,-2.319e-02, 2.111e-02,
     x-8.137e-01, 6.080e-06, 9.933e-01,
     x 7.285e+00, 1.001e+00,-2.049e-02, 5.285e-02,
     x-5.057e-01,-3.719e-04, 9.647e-01,
     x 1.096e+00, 1.008e+00,-6.852e-03, 1.021e+00,
     x 8.523e+00,-5.127e-03, 2.513e-01,
     x -1.00e+01, 0.000e+00, 0.000e+00, 0.000e+00,
     x 0.000e+00, 0.000e+00, 0.000e+00,
     x -1.00e+01, 0.000e+00, 0.000e+00, 0.000e+00,
     x 0.000e+00, 0.000e+00, 0.000e+00,
     x -1.00e+01, 0.000e+00, 0.000e+00, 0.000e+00,
     x 0.000e+00, 0.000e+00, 0.000e+00
     &/
      data aa1/ !ckdfu.fuliou.lin.out
     x 6.762e+00, 9.996e-01,-7.259e-03, 9.977e-01,
     x 1.603e+01, 1.709e-01,-1.394e-03,
     x 5.261e+00, 9.971e-01,-6.219e-03, 9.868e-01,
     x 1.561e+01, 1.275e-01, 2.789e-03,
     x 3.235e+00, 9.872e-01,-6.404e-03, 8.690e-01,
     x 2.107e+01, 1.222e-01, 6.908e-02,
     x 5.942e+00, 9.982e-01,-1.368e-02, 3.916e-01,
     x 9.076e+00, 2.730e-02, 5.256e-01,
     x 9.217e+00, 1.003e+00,-2.221e-02, 8.940e-02,
     x 6.127e-01,-6.341e-04, 8.978e-01,
     x 9.478e+00, 1.000e+00,-2.441e-02, 1.837e-02,
     x-7.174e-01,-8.151e-06, 9.865e-01,
     x 8.451e+00, 1.000e+00,-2.323e-02, 1.559e-02,
     x-8.067e-01, 5.546e-06, 9.932e-01,
     x 7.281e+00, 1.001e+00,-2.035e-02, 4.216e-02,
     x-4.622e-01,-4.440e-04, 9.615e-01,
     x 1.081e+00, 9.983e-01,-4.259e-03, 8.882e-01,
     x 9.883e+00, 1.679e-02, 6.666e-02,
     x 3.738e+00, 9.973e-01,-3.824e-03, 9.842e-01,
     x 8.171e+00, 5.865e-02, 4.520e-03,
     x 2.879e+00, 9.953e-01,-3.841e-03, 9.721e-01,
     x 7.980e+00, 5.344e-02, 1.106e-02,
     x 7.045e-01, 1.008e+00,-5.968e-03, 7.708e-01,
     x 7.788e+00,-5.452e-03, 1.683e-01
     &/
      data aa2/ !ckdfu.fuliou.log.out
     x 6.405e+00, 9.986e-01,-7.277e-03, 9.963e-01,
     x 1.304e+01, 1.588e-01,-3.569e-04,
     x 5.039e+00, 9.967e-01,-6.107e-03, 9.820e-01,
     x 1.819e+01, 1.188e-01, 4.073e-03,
     x 3.333e+00, 9.832e-01,-6.811e-03, 8.141e-01,
     x 2.107e+01, 1.416e-01, 1.102e-01,
     x 6.498e+00, 1.000e+00,-1.486e-02, 3.513e-01,
     x 7.720e+00, 2.083e-02, 5.794e-01,
     x 9.328e+00, 1.003e+00,-2.251e-02, 8.079e-02,
     x 4.519e-01,-5.421e-04, 9.084e-01,
     x 9.448e+00, 9.999e-01,-2.438e-02, 1.711e-02,
     x-7.349e-01, 5.084e-05, 9.879e-01,
     x 8.424e+00, 1.000e+00,-2.318e-02, 1.598e-02,
     x-8.060e-01,-2.452e-05, 9.931e-01,
     x 7.343e+00, 1.001e+00,-2.051e-02, 3.936e-02,
     x-5.032e-01,-3.829e-04, 9.655e-01,
     x 1.962e+00, 1.004e+00,-6.678e-03, 7.064e-01,
     x 9.220e+00, 5.062e-03, 2.399e-01,
     x 3.616e+00, 9.972e-01,-3.827e-03, 9.825e-01,
     x 8.823e+00, 5.525e-02, 5.027e-03,
     x 2.481e+00, 9.947e-01,-3.944e-03, 9.568e-01,
     x 9.687e+00, 5.039e-02, 1.942e-02,
     x 2.141e+00, 1.009e+00,-1.016e-02, 5.728e-01,
     x 5.100e+00,-5.114e-03, 3.688e-01
     &/
      data aa3/ !ckdfu.fuliou.lin.plnk.out
     x 6.899e+00, 9.989e-01,-7.230e-03, 9.979e-01,
     x 1.680e+01, 2.287e-01,-9.470e-04,
     x 5.300e+00, 9.971e-01,-6.528e-03, 9.864e-01,
     x 1.615e+01, 1.283e-01, 2.595e-03,
     x 3.351e+00, 9.867e-01,-6.864e-03, 8.681e-01,
     x 2.138e+01, 1.249e-01, 6.981e-02,
     x 5.981e+00, 9.983e-01,-1.384e-02, 3.944e-01,
     x 9.316e+00, 2.791e-02, 5.220e-01,
     x 9.253e+00, 1.003e+00,-2.230e-02, 9.128e-02,
     x 6.481e-01,-6.383e-04, 8.963e-01,
     x 9.555e+00, 1.000e+00,-2.460e-02, 1.921e-02,
     x-6.885e-01,-3.569e-05, 9.855e-01,
     x 8.495e+00, 1.000e+00,-2.333e-02, 1.484e-02,
     x-7.828e-01,-1.622e-06, 9.927e-01,
     x 7.328e+00, 1.001e+00,-2.046e-02, 3.898e-02,
     x-5.016e-01,-3.945e-04, 9.649e-01,
     x 8.158e-01, 9.974e-01,-3.582e-03, 8.775e-01,
     x 9.817e+00, 1.744e-02, 7.603e-02,
     x 3.482e+00, 9.971e-01,-3.205e-03, 9.827e-01,
     x 8.142e+00, 5.638e-02, 5.275e-03,
     x 3.262e+00, 9.969e-01,-4.671e-03, 9.760e-01,
     x 8.702e+00, 4.119e-02, 8.187e-03,
     x 1.006e+00, 1.008e+00,-6.560e-03, 8.004e-01,
     x 8.480e+00,-5.247e-03, 1.390e-01
     &/
      data aa4/  !ckdfu.fuliou.log.plnk.out
     x 6.883e+00, 1.007e+00,-7.231e-03, 1.006e+00,
     x 1.711e+01, 2.401e-01,-8.842e-04,
     x 5.128e+00, 1.006e+00,-6.531e-03, 9.905e-01,
     x 1.907e+01, 1.219e-01, 4.032e-03,
     x 3.529e+00, 9.903e-01,-7.494e-03, 8.184e-01,
     x 2.150e+01, 1.434e-01, 1.137e-01,
     x 6.623e+00, 1.008e+00,-1.525e-02, 3.557e-01,
     x 7.867e+00, 2.121e-02, 5.826e-01,
     x 9.438e+00, 1.010e+00,-2.281e-02, 8.279e-02,
     x 4.923e-01,-4.263e-04, 9.138e-01,
     x 9.574e+00, 1.005e+00,-2.472e-02, 1.761e-02,
     x-7.078e-01, 1.032e-04, 9.917e-01,
     x 8.530e+00, 1.007e+00,-2.347e-02, 1.612e-02,
     x-7.979e-01, 1.613e-05, 1.000e+00,
     x 7.429e+00, 1.006e+00,-2.076e-02, 3.668e-02,
     x-5.353e-01,-2.511e-04, 9.738e-01,
     x 1.838e+00, 1.013e+00,-6.037e-03, 6.632e-01,
     x 8.190e+00, 1.126e-04, 2.812e-01,
     x 3.284e+00, 9.994e-01,-3.042e-03, 9.830e-01,
     x 8.944e+00, 4.588e-02, 5.890e-03,
     x 2.977e+00, 9.987e-01,-5.050e-03, 9.663e-01,
     x 9.910e+00, 4.875e-02, 1.525e-02,
     x 2.177e+00, 1.010e+00,-1.035e-02, 6.566e-01,
     x 6.613e+00,-5.513e-03, 2.837e-01
     &/

      end module cont_coef

!-----------------------------------------------------------------------

      function parm_ckd
     +   (iband , amnt , patm , temp
     +,   dz
     +   )

      use kinds
      use cont_coef
      implicit none
      
!-----------------------------------------------------------------------
! REFERENCES:
! Parameterization of CKD_2.1 continuum over Fu-Liou Bands.
! Phil Partain/graben (04-04-00).

! Input:
! iband  =  integer (1-8) where
!              Band 1 ='  5:280cm-1'
!              Band 2 ='280:400cm-1'
!              Band 3 ='400:540cm-1'
!              Band 4 ='540:670cm-1'
!              Band 5 ='670:800cm-1'
!              Band 6 ='800:980cm-1'
!              Band 7 ='980:1100cm-1'
!              Band 8 ='1100:1250cm-1'
!              Band 9 ='1250:1400cm-1'
!              Band10 ='1400:1700cm-1'
!              Band11 ='1700:1900cm-1'
!              Band12 ='1900:2200cm-1'

! H20 Continuum absorbtion now for entire LW (5-2200cm-1)
!    iwtas    1:Linear                             /cont_tas/   INTEGER
!             2:log ,
!             3:linear w/Plank wgt   :: SUGGEST iwtas=3
!             4: Log w/plank wgt

! send comments to partain@atmos.colostate.edu.

! MODIFICATIONS:
! * changed declarations to adapt the code from BUGS4 to BUGS5.
!   Laura D. Fowler/slikrock (02-01-00).

! ARGUMENT LIST VARIABLES:
!     INPUT ARGUMENTS:
!     ----------------
      real (kind=dbl_kind), intent(in)::
     &  amnt     !Water vapor content                         (g/cm^2).
     &, patm     !Pressure                                       (atm).
     &, temp     !Temperature                                      (K).
     &, dz       !Path length                                     (km).

!     OUTPUT ARGUMENTS:
!     -----------------
!     clough_parm = parameterized CKD_2.1optical depth for band

      real (kind=dbl_kind)::
     &  parm_ckd !Transmission due to water vapor continuum        (-).

! LOCAL VARIABLES:
      
      integer (kind=int_kind)::
     &  iband
     
      integer (kind=int_kind), parameter::
     &  iwtas = 3
!    &,  ncoef = 7
!    &, nband = 12

      real (kind=dbl_kind)::
     &  patmx, ph2o, tau_log
     
      real (kind=dbl_kind), dimension(ncoef,nband)::
     &  aa

!     real (kind=dbl_kind), dimension(ncoef,nband)::
!    &  aa0 !ORIGINAL 280:1400
!    &, aa1 !lin no plank
!    &, aa2 !log no plank
!    &, aa3 !lin Plank wgt
!    &, aa4 !log Plank wgt

!-----------------------------------------------------------------------

      if(iwtas <0 .or. iwtas >4) stop 'iwtas'
      if ( iwtas == 0) aa= aa0
      if ( iwtas == 1) aa= aa1
      if ( iwtas == 2) aa= aa2
      if ( iwtas == 3) aa= aa3
      if ( iwtas == 4) aa= aa4
 
      ph2o = amnt *(8.314d+07 *temp )
     &     / (dz*1.0d+05*18.01534 *1.01325d+06)
 
      if (iwtas == 0) patmx = patm
      if (iwtas >  0) patmx = log(patm)
 
      tau_log     = aa(1,iband)             +
     $              aa(2,iband)* log(amnt)  +
     $              aa(3,iband)* temp       +
     $              aa(4,iband)* patmx       +
     $              aa(5,iband)* (ph2o)     +
     $              aa(6,iband)* amnt       +
     $              aa(7,iband)* log(ph2o)
 
      parm_ckd = exp(tau_log)
 
      return
      end
 
!-----------------------------------------------------------------------
