!------------------------------------------------------------------------------

subroutine gascon_robs(ncol,nlm,ib,ppl,dp,tt,rmix,tgm)

use kinds
use rad_params

implicit none

!------------------------------------------------------------------------------

!calculates the absorption by the e-continuum following Roberts.

!argument list variables:
!input arguments:
!----------------
      
integer (kind=int_kind):: &
   ncol,             &!length of sub-domain.
   nlm,              &!number of layers.
   ib                 !spectral interval. 
     
real (kind=dbl_kind), dimension(ncol,nlm):: &
   ppl,              &!layer pressure                                    (hPa).
   dp,               &!layer pressure thickness                          (hPa).
   tt,               &!layer temperature                                   (K).
   rmix               !layer water vapor mixing ratio                  (kg/kg).
     
!output arguments:
!-----------------

real (kind=dbl_kind), dimension(ncol,nlm):: &
   tgm                !
     
!local variables:

integer (kind=int_kind):: i, l

real (kind=dbl_kind):: ff,pe,s,w

real (kind=dbl_kind)::   &
   x = 4.18_dbl_kind,    &
   y = 5577.8_dbl_kind,  &
   z = 0.00787_dbl_kind, &
   r = 0.002_dbl_kind

real (kind=dbl_kind), dimension(18):: vv
data vv/10*0.,1175.,1040.,890.,735.,605.,470.,340.,0./

!------------------------------------------------------------------------------

!initializes output arrays:

tgm(:,:) = radc0

!computes the optical depth:

if(ib.gt.10 .and. ib.lt.18) then
   s = (x+y*exp(- z*vv(ib)))/1013.25
   do l = 1, nlm
	do i = 1, ncol
 	   pe = ppl(i,l)*rmix(i,l)/(0.622+0.378*rmix(i,l))
	   w  = exp(1800./tt(i,l)-6.08108)
	   ff = s*(pe+r*ppl(i,l))*w
	   tgm(i,l) = ff*rmix(i,l)*dp(i,l)*10/9.80616
      enddo
   enddo      
endif

end subroutine gascon_robs
	
!------------------------------------------------------------------------------	
