! CVS:  $Id: rad_gases.F,v 1.1 2003/09/03 17:09:53 laura Exp $
! CVS:  $Name:  $

!-----------------------------------------------------------------------

      subroutine gases
     +              ( ncol ,   nlm ,    ib ,    ig
     +,                 pp ,    dp ,    tt ,  rmix
     +,              o3mix , umco2 , umch4 , umn2o
     +,                 hk ,    tg ,   pkd ,   ip1
     +,                ip2
     +              )

      use kinds
      use ckd1
      use ckd2
      use ckd3
      use ckd4
      use ckd5
      use ckd6
      use ckd7
      use ckd8
      use ckd9
      use ckd10
      use ckd11
      use ckd12
      use ckd13
      use ckd14
      use ckd15
      use ckd16
      use ckd17
      use ckd18

      implicit none

!-----------------------------------------------------------------------
! REFERENCES:
! gases calculates the non-gray spectral absorption due to H2O, O3, CO2,
! CHA4, and N2O at long and short wavelengths. gases replaces the old
! gases subroutine written by G. Stephens.
! Laura D. Fowler (slikrock/08-23-96).

! send comments to laura@slikrock.atmos.colostate.edu and
! partain@atmos.colostate.edu.

! MODIFICATIONS:
! * changed declarations to adapt the code from BUGS4 to BUGS5.
!   Laura D. Fowler/slikrock (02-01-00).

! FUNCTIONS CALLED:
!     none.

! INCLUDED COMMONS:
!     none.

! ARGUMENT LIST VARIABLES:
!     INPUT ARGUMENTS:
!     ----------------
      integer (kind=int_kind), intent(in)::
     &  ncol  ! Length of sub-domain.
     &, nlm   ! Number of layers.
     &, ib    ! Index of spectral interval.
     &, ig    ! Index of k-distribution.

      integer (kind=int_kind), intent(in), dimension(ncol,nlm)::
     &  ip1   !Used in conjunction with pressure weigthing.
     &, ip2   !Used in conjunction with pressure weigthing.

      real (kind=dbl_kind), intent(in)::
     &  umco2 !Concentration in CO2                              (ppm).
     &, umch4 !Concentration in CH4                              (???).
     &, umn2o !Concentration in N2O                              (???).
     
      real (kind=dbl_kind), intent(in), dimension(ncol,nlm)::
     &  dp    !Layer Thickness                                   (hPa).
     &, tt    !Layer Temperature                                   (K).
     &, rmix  !Layer Water Vapor Mixing Ratio                  (kg/kg).
     &, o3mix !Layer Ozone Mixing Ratio                        (kg/kg).    
     &, pkd   !Used in conjunction with pressure weighting         (-).
     
      real (kind=dbl_kind), intent(in), dimension(ncol,nlm+1)::
     &  pp    !Level Pressure                                    (hPa).   
     
!     OUTPUT ARGUMENTS:
!     -----------------
      real (kind=dbl_kind), intent(out)::
     &  hk    !Weight                                              (-).
     
      real (kind=dbl_kind), intent(out), dimension(ncol,nlm):: 
     &  tg    !Transmission of gases                               (-).
    
! LOCAL VARIABLES:

      integer (kind=int_kind)::
     &  ii, l
     
      real (kind=dbl_kind)::
     &  fk     

      real (kind=dbl_kind), dimension(ncol,nlm)::
     &  fkg , fkga , fkgb , pq
     &, tg1 ,  tg2 ,  tg3
     
!-----------------------------------------------------------------------

	goto(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18 ) ib 
	stop

! In this band (50000-14500 cm^-1 ), we have considered the nongray
! gaseous absorption of O3.    
! 619.618 is the solar energy contained in that band (Wm^-2).
1     fk = fk1o3(ig)
      call qopo3s
     +        (ncol,nlm,fk,dp,o3mix,tg)
      hk = 619.618*hk1(ig)
      goto 20
      	
! In this band (14500-7700 cm^-1),we have considered the nongray
! gaseous absorption of H2O.  
! 484.295 is the solar energy contained in the band (Wm^-2).
2     call qks(ncol,nlm,c2h2o(1,1,ig),tt,fkg,pkd,ip1,ip2)
      call qoph2o(ncol,nlm,fkg,dp,rmix,tg)
      hk = 484.295 * hk2(ig)
      goto 20

! In this band (7700-5250 cm^-1), we have considered the nongray
! gaseous absorption of H2O. 
! 149.845 is the solar energy contained in the band (Wm^-2).
3     call qks(ncol,nlm,c3h2o(1,1,ig),tt,fkg,pkd,ip1,ip2)
      call qoph2o(ncol,nlm,fkg,dp,rmix,tg)
      hk = 149.845 * hk3(ig)
      goto 20

! In this band (5250-4000 cm^-1), we have considered the nongray
! gaseous absorption of H2O. 
! 48.7302 is the solar energy contained in the band (Wm^-2).	
4     call qks(ncol,nlm,c4h2o(1,1,ig),tt,fkg,pkd,ip1,ip2)
      call qoph2o(ncol,nlm,fkg,dp,rmix,tg)      
      hk = 48.7302 * hk4(ig)
      goto 20
	
! In this band (4000-2850 cm^-1), we have considered the nongray
! gaseous absorption of H2O. 
! 31.6576 is the solar energy contained in the band (Wm^-2).
5     call qks(ncol,nlm,c5h2o(1,1,ig),tt,fkg,pkd,ip1,ip2)
      call qoph2o(ncol,nlm,fkg,dp,rmix,tg)      
      hk = 31.6576 * hk5(ig)
      goto 20

! In this band (2850-2500 cm^-1), we have considered the nongray
! gaseous absorption of H2O. 
! 5.79927 is the solar energy contained in the band (Wm^-2).	
6     call qks(ncol,nlm,c6h2o(1,1,ig),tt,fkg,pkd,ip1,ip2)
      call qoph2o(ncol,nlm,fkg,dp,rmix,tg)      
      hk = 5.79927 * hk6(ig)
      goto 20

! In this band (2200-1900 cm^-1), we have considered the nongray
! gaseous absorption of H2O.
7     call qki(ncol,nlm,c7h2o(1,1,ig),tt,fkg,pkd,ip1,ip2)
      call qoph2o(ncol,nlm,fkg,dp,rmix,tg)
      hk = hk7(ig)
      goto 20
      
! In this band (1900-1700 cm^-1), we have considered the nongray
! gaseous absorption of H2O.
8     call qki(ncol,nlm,c8h2o(1,1,ig),tt,fkg,pkd,ip1,ip2)
      call qoph2o(ncol,nlm,fkg,dp,rmix,tg)      
      hk = hk8(ig)
      goto 20

! In this band (1700-1400 cm^-1), we have considered the nongray
! gaseous absorption of H2O.        
9     call qki(ncol,nlm,c9h2o(1,1,ig),tt,fkg,pkd,ip1,ip2)
      call qoph2o(ncol,nlm,fkg,dp,rmix,tg)      
      hk = hk9(ig)     
      goto 20

! In this band (1400-1250 cm^-1), we have considered the overlapping
! absorption of H2O, CH4, and N2O by approach one of Fu(1991).	
10    call qki(ncol,nlm,c10h2o(1,1,ig),tt,fkg,pkd,ip1,ip2)
      call qoph2o(ncol,nlm,fkg,dp,rmix,tg1)      
      call qki(ncol,nlm,c10ch4,tt,fkg,pkd,ip1,ip2)
      call qopch4(ncol,nlm,fkg,dp,tg2)
      call qki(ncol,nlm,c10n2o,tt,fkg,pkd,ip1,ip2)
      call qopn2o(ncol,nlm,fkg,dp,tg3)
      do l = 1, nlm
         do ii = 1, ncol
	    tg(ii,l) = tg1(ii,l)+tg2(ii,l)/1.6*umch4
     +               + tg3(ii,l)/0.28*umn2o
         enddo
      enddo
      hk = hk10(ig)     
      goto 20

! In this band (1250-1100 cm^-1), we have considered the overlapping
! absorption of H2O, CH4, and N2O by approach one of Fu(1991).      
11    call qki(ncol,nlm,c11h2o(1,1,ig),tt,fkg,pkd,ip1,ip2)
      call qoph2o(ncol,nlm,fkg,dp,rmix,tg1)        
      call qki(ncol,nlm,c11ch4,tt,fkg,pkd,ip1,ip2)
      call qopch4(ncol,nlm,fkg,dp,tg2)      
      call qki(ncol,nlm,c11n2o,tt,fkg,pkd,ip1,ip2)
      call qopn2o(ncol,nlm,fkg,dp,tg3)
      do l = 1, nlm
	 do ii = 1, ncol
	    tg(ii,l) = tg1(ii,l)+tg2(ii,l)/1.6*umch4
     +               + tg3(ii,l)/0.28*umn2o
         enddo
      enddo
      hk = hk11(ig)
      goto 20
	
! In this band (1100-980 cm^-1), we have considered the overlapping
! absorption of H2O and O3 by approach one of Fu(1991).	
12    call qkio3(ncol,nlm,c12o3(1,1,ig),tt,fkg,pkd,ip1,ip2)
      call qopo3i(ncol,nlm,fkg,dp,o3mix,tg1)
      call qki(ncol,nlm,c12h2o,tt,fkg,pkd,ip1,ip2)
      call qoph2o(ncol,nlm,fkg,dp,rmix,tg2)      
      do l = 1, nlm
         do ii = 1, ncol
            tg(ii,l) = tg1(ii,l)+tg2(ii,l)
         enddo 
      enddo
      hk = hk12(ig)
      goto 20

! In this band (980-800 cm^-1 ), we have considered the nongray
! gaseous absorption of H2O.	
13    call qki(ncol,nlm,c13h2o(1,1,ig),tt,fkg,pkd,ip1,ip2)
      call qoph2o(ncol,nlm,fkg,dp,rmix,tg)       
      hk = hk13(ig)
      goto 20

! In this band (800-670 cm^-1), we have considered the overlapping
! absorption of H2O and CO2 by approach two of Fu(1991).
14    do l = 1, nlm
         do ii =  1, ncol
            if(pp(ii,l) .ge. 63.1) then
               pq(ii,l) = rmix(ii,l)
            else
               pq(ii,l) = 0.0
       	    endif
         enddo
      enddo
      call qki(ncol,nlm,c14hca(1,1,ig),tt,fkga,pkd,ip1,ip2)
      call qki(ncol,nlm,c14hcb(1,1,ig),tt,fkgb,pkd,ip1,ip2)
      do l = 1, nlm
         do ii = 1, ncol
	    fkg(ii,l) = fkga(ii,l)/330.0*umco2+pq(ii,l)*fkgb(ii,l)
         enddo
      enddo
      call qophc(ncol,nlm,fkg,dp,tg)
      hk = hk14(ig)
      goto 20

! In this band (670-540 cm^-1), we have considered the overlapping
! absorption of H2O and CO2 by approach two of Fu(1991).
15    do l = 1, nlm
          do ii = 1, ncol
	     if(pp(ii,l) .ge. 63.1 ) then
                pq(ii,l) = rmix(ii,l)
             else
                pq(ii,l) = 0.0
       	     endif
          enddo
      enddo
      call qki(ncol,nlm,c15hca(1,1,ig),tt,fkga,pkd,ip1,ip2)
      call qki(ncol,nlm,c15hcb(1,1,ig),tt,fkgb,pkd,ip1,ip2)
      do l = 1, nlm
         do ii = 1, ncol
	    fkg(ii,l) = fkga(ii,l)/330.0*umco2+pq(ii,l)*fkgb(ii,l)
         enddo
      enddo
      call qophc(ncol,nlm,fkg,dp,tg)
      hk = hk15(ig)
      goto 20

! In this band (540-400 cm^-1 ), we have considered the nongray
! gaseous absorption of H2O.
16    call qki(ncol,nlm,c16h2o(1,1,ig),tt,fkg,pkd,ip1,ip2)
      call qoph2o(ncol,nlm,fkg,dp,rmix,tg)      
      hk = hk16(ig)
      goto 20

! In this band (400-280 cm^-1 ), we have considered the nongray
! gaseous absorption of H2O.	
17    call qki(ncol,nlm,c17h2o(1,1,ig),tt,fkg,pkd,ip1,ip2)
      call qoph2o(ncol,nlm,fkg,dp,rmix,tg)         
      hk = hk17(ig)
      goto 20

! In this band (280-000 cm^-1 ), we have considered the nongray
! gaseous absorption of H2O.
18    call qki(ncol,nlm,c18h2o(1,1,ig),tt,fkg,pkd,ip1,ip2)
      call qoph2o(ncol,nlm,fkg,dp,rmix,tg)       
      hk = hk18(ig)

20    continue      

      return
      end subroutine gases

!-----------------------------------------------------------------------
