!-----------------------------------------------------------------------

      module rad_params
   
      use kinds
      use physical_parameters
   
      implicit none
      save
   
!-----------------------------------------------------------------------
!rad_params contains parameters needed in the parameterization of long-
!and short-wave radiation transfer.
!Laura D. Fowler/slikrock (02-01-01).

!MODIFICATIONS NONE:
!     none.
!-----------------------------------------------------------------------

!---- logicals that governs how the radiation codes are run:
      logical (kind=log_kind), parameter::
     &  clizon = .true. !if true, uses a climatological distribution of
                        !ozone.
     &, ndiurn = .false.!if true, no diurnal cycle of radiation.

!---- equinox::
      integer (kind=int_kind)::
     &  eqnx   = 80     !day of year of vernal equinox.

!---- constant parameters:

      real (kind=dbl_kind), parameter::
     &  solcon = 1365._dbl_kind    !solar constant             (W/m^2).
     &, rotper = 24_dbl_kind       !number of hours in a day       (-).
     &, eccn   = 0.016715_dbl_kind !orbital eccentricity           (-).
     &, reccn  = (1./(1.-eccn*eccn)**3)
     &, decmax = 23.441_dbl_kind   !inclination of earth axis    (deg).
     &, h      = pi2/365._dbl_kind
     &, perhl  = 102.7_dbl_kind
     &, perhlr = perhl*(pi/180._dbl_kind)

!---- time varying variables:

      real (kind=dbl_kind) ::
     &  sind              !sin(solar declination angle)
     &, cosd              !cos(solar declination angle)
     &, rsdist            !square of ratio of orbital distance to mean.

!---- spectral intervals:

      integer (kind=int_kind), parameter::
     &  mb   = 18         !total number of spectral intervals.
     &, mbs  = 6          !number of long-wave spectral intervals.
     &, mbir = 12         !number of short-wave spectral intervals.

!---- initialization:

      real (kind=dbl_kind), parameter::
     &  radc0    = 0.0_dbl_kind
     &, radc1    = 1.0_dbl_kind
     
!---- needed for the properties of the sponge layer at the top of the model:

      end module rad_params

!-----------------------------------------------------------------------
