!-----------------------------------------------------------------------
subroutine rad_pbps                                           &
                     (ksdm, n_physics, n_pbp, nagrid, nlm,    &
                      swnetsfc, lwnetsfc, lwtoa, swnettoa,    &
                      swinc, lwdnsfc, swdnsfc, lwupsfc,       &
                      swtoa, swdnsfccl, lwdnsfccl, lwtoacl,   &
                      swtoacl, swnetsfccl,                    &
                      atl, atlcl, asl, aslcl, o3l)

  use kinds
  use pbp_module
  use rad_params
      
  implicit none
      
!-----------------------------------------------------------------------
! REFERENCES:
! rad_pbps calculates the pbp1s and pbp2s diagnostics of the long-wave and
! short-wave radiation codes.
! Mark D. Branson (09-15-05).

! send comments to mark@atmos.colostate.edu

! SUBROUTINES CALLED:
!     

! FUNCTIONS CALLED:
!     none.

! INCLUDED COMMON BLOCKS:
!     none

! ARGUMENT LIST VARIABLES:
! INPUT ARGUMENTS:
! ----------------
  integer (kind=int_kind) ::  &
        ksdm, &      !loop index.
        n_physics, & !nb of short-time steps per hour. 
        n_pbp, &     !number of time-steps between pbp outputs.
        nagrid, &    !length of total domain.
        nlm          !number of layers.

  real (kind=dbl_kind), intent(in), dimension(nagrid) :: &
        swnetsfc, &  !sfc net sw radiation                        (W/m^2).
        lwnetsfc, &  !sfc all-sky net lw radiation                (W/m^2).
        lwtoa, &     !toa all-sky outgoing lw radiation           (W/m^2).
        swnettoa, &  !toa net sw radiation                        (W/m^2).
        swinc, &     !toa incident solar radiation                (W/m^2).
        lwdnsfc, &   !sfc all-sky downward lw radiation           (W/m^2).
        swdnsfc, &   !sfc downward sw radiation                   (W/m^2).
        lwupsfc, &   !sfc upward lw radiation                     (W/m^2).
        swtoa, &     !toa all-sky reflected sw radiation          (W/m^2).
        swdnsfccl, & !sfc clear-sky downward sw radiation         (W/m^2).
        lwdnsfccl, & !sfc clear-sky downward lw radiation         (W/m^2).
        lwtoacl, &   !toa clear-sky outgoing lw radiation         (W/m^2).
        swtoacl, &   !toa clear-sky reflected sw radiation        (W/m^2).
        swnetsfccl   !sfc clear-sky net sw radiation              (W/m^2).

  real (kind=dbl_kind), intent(in), dimension(nagrid,nlm) :: &
        atl, &       !long-wave radiative heating rate              (K/s).
        asl, &       !short-wave radiative heating rate             (K/s).
        atlcl, &     !clear-sky lw radiative heating rate           (K/s).
        aslcl, &     !clear-sky sw radiative heating rate           (K/s).
        o3l          !ozone mixing ratio                          (kg/kg).

! LOCAL VARIABLES:

  real (kind=dbl_kind) :: &
        x_pbp

!-----------------------------------------------------------------------

  x_pbp = min(n_physics, n_pbp)
  
!-- 2d pbps
  call pbp1sum(swnetsfc, x_pbp, ipbp1_rss, ksdm, nagrid)     
  call pbp1sum(lwnetsfc, x_pbp, ipbp1_rls, ksdm, nagrid)     
  call pbp1sum(lwtoa, x_pbp, ipbp1_rlt, ksdm, nagrid)     
  call pbp1sum(swnettoa, x_pbp, ipbp1_rst, ksdm, nagrid)     
  call pbp1sum(swinc, x_pbp, ipbp1_rsdt, ksdm, nagrid)     
  call pbp1sum(lwdnsfc, x_pbp, ipbp1_rlds, ksdm, nagrid)     
  call pbp1sum(swdnsfc, x_pbp, ipbp1_rsds, ksdm, nagrid)     
  call pbp1sum(lwupsfc, x_pbp, ipbp1_rlus, ksdm, nagrid)     
  call pbp1sum(swtoa, x_pbp, ipbp1_rsut, ksdm, nagrid)     
  call pbp1sum(swdnsfccl, x_pbp, ipbp1_rsdscs, ksdm, nagrid)     
  call pbp1sum(lwdnsfccl, x_pbp, ipbp1_rldscs, ksdm, nagrid)     
  call pbp1sum(lwtoacl, x_pbp, ipbp1_rlutcs, ksdm, nagrid)     
  call pbp1sum(swtoacl, x_pbp, ipbp1_rsutcs, ksdm, nagrid)     

  call pbp1sum(swnetsfc, x_pbp, ipbp1_rsus, ksdm, nagrid)     
  call pbp1sum(swdnsfc,-x_pbp, ipbp1_rsus, ksdm, nagrid)     

  call pbp1sum(swnetsfccl, x_pbp, ipbp1_rsuscs, ksdm, nagrid)     
  call pbp1sum(swdnsfccl,-x_pbp, ipbp1_rsuscs, ksdm, nagrid)     

!-- 3d pbps
  call pbp2sum(o3l,    nagrid, x_pbp, ipbp2_o3,      ksdm, nagrid)
  call pbp2sum(asl,    nagrid, x_pbp, ipbp2_tntsw,   ksdm, nagrid)
  call pbp2sum(atl,    nagrid, x_pbp, ipbp2_tntlw,   ksdm, nagrid)
  call pbp2sum(aslcl,  nagrid, x_pbp, ipbp2_tntswcs, ksdm, nagrid)
  call pbp2sum(atlcl,  nagrid, x_pbp, ipbp2_tntlwcs, ksdm, nagrid)
  
end subroutine rad_pbps

!-----------------------------------------------------------------------
