! CVS:  $Id: rad_qkio3.F,v 1.1 2003/09/03 19:35:18 laura Exp $
! CVS:  $Name:  $

!-----------------------------------------------------------------------

      subroutine qkio3
     +               (ncol , nlm , coefki ,  tt
     +,                fkg , pkd ,    ip1 , ip2
     +               )
      
      use kinds

      implicit none
      
!-----------------------------------------------------------------------      
! REFERENCES:
! Cleaned up version of qkio3.f from G. Stephens. Computes the gaseous
! absorption coefficients in units of (cm-atm)^-1 for a given cumulative
! probability in nlm layers. coefki(3,19) are the coefficients used to
! calculate the absorption coefficient at the temperature t for the 19 
! pressures by
!   ln k = a + b * ( t - 250 ) + c * ( t - 250 ) ** 2
! and the absorption coefficient at conditions other than those nineteen
! pressures is interpolated linearly with pressure (Fu, 1991).
! Laura D. Fowler (slikrock. 08-18-97).

! send comments to laura@slikrock.atmos.colostate.edu and
! partain@atmos.colostate.edu.

! MODIFICATIONS:
! * changed declarations to adapt the code from BUGS4 to BUGS5.
!   Laura D. Fowler/slikrock (02-01-00).

! SUBROUTINES CALLED:
!     none.

! FUNCTIONS CALLED:
!     none.

! INCLUDED COMMONS:
!     none.

! ARGUMENT LIST VARIABLES:
!     INPUT ARGUMENTS:
!     ----------------
      integer (kind=int_kind), intent(in)::
     &  ncol   ! Length of sub-domain.
     &, nlm    ! Number of layers.

      integer (kind=int_kind), intent(in), dimension(ncol,nlm)::
     &  ip1    !Used in conjunction with pressure weigthing.
     &, ip2    !Used in conjunction with pressure weigthing.

      real (kind=dbl_kind), intent(in), dimension(3,19)::
     &  coefki !Pre-computed coefficients.
     
      real (kind=dbl_kind), intent(in), dimension(ncol,nlm)::
     &  tt     !Layer temperature   (K).
     &, pkd    !Scaled pressure   (hPa).
     
!     OUTPUT ARGUMENTS:
!     -----------------
      real (kind=dbl_kind), intent(out), dimension(ncol,nlm)::
     &  fkg    !Gaseous absorption coefficients (cm-atm)^-1.  

! LOCAL VARIABLES:

      integer (kind=int_kind)::
     &  i, l , i1, i2

      real (kind=dbl_kind)::
     &  x1,x2,y1
     
!-----------------------------------------------------------------------

      do l = 1, nlm
         do i = 1, ncol
            i1 = ip1(i,l)
            i2 =ip2(i,l)
            y1 = tt(i,l) - 250.0
            if (y1.lt.-70.) y1 = -70.
            if (y1.gt.75.) y1 = 70.
            x1 = exp(coefki(1,i1)+coefki(2,i1)*y1+coefki(3,i1)*y1**2)
            if(i1.eq.i2) then
               fkg(i,l) = x1 * pkd(i,l)
            else
               x2 = exp(coefki(1,i2)+coefki(2,i2)*y1+coefki(3,i2)*y1**2)
               fkg(i,l) = x1 + ( x2 - x1 ) * pkd(i,l)
            endif
         enddo
      enddo

      return
      end subroutine qkio3

!-----------------------------------------------------------------------
