! CVS:  $Id: rad_qks.F,v 1.1 2003/09/03 19:33:34 laura Exp $
! CVS:  $Name:  $

!-----------------------------------------------------------------------

      subroutine qks(ncol,nlm,coefks,tt,fkg,pkd,ip1,ip2)
      
      use kinds
      
      implicit none
      
!-----------------------------------------------------------------------      
! REFERENCES:
! Cleaned up version of qks.f from G. Stephens. Computes the gaseous
! absorption coefficients in units of (cm-atm)^-1 for a given cumulative
! probability in nlm layers. coefks(3,11) are the coefficients used to
! calculate the absorption coefficient at the temperature t for the 11 
! pressures by
!   ln k = a + b * ( t - 245 ) + c * ( t - 245 ) ** 2
! and the absorption coefficient at conditions other than those eleven
! pressures is interpolated linearly with pressure (Fu, 1991).
! Laura D. Fowler (slikrock. 08-18-97).

! send comments to laura@slikrock.atmos.colostate.edu and
! partain@atmos.colostate.edu.

! MODIFICATIONS:
! * changed declarations to adapt the code from BUGS4 to BUGS5.
!   Laura D. Fowler/slikrock (02-01-00).

! SUBROUTINES CALLED:
!     none.

! FUNCTIONS CALLED:
!     none.

! INCLUDED COMMONS:
!     none.

! ARGUMENT LIST VARIABLES:
!     INPUT ARGUMENTS:
!     ----------------
      integer (kind=int_kind), intent(in)::
     &  ncol   !Length of sub-domain.
     &, nlm    !Number of layers.

      integer (kind=int_kind), intent(in), dimension(ncol,nlm)::
     &  ip1    !Used in conjunction with pressure weigthing.
     &, ip2    !Used in conjunction with pressure weigthing.

      real (kind=dbl_kind), intent(in), dimension(3,11)::
     &  coefks !Pre-computed coefficients.
     
      real (kind=dbl_kind), intent(in), dimension(ncol,nlm)::
     &  tt     !Layer temperature (K).
     &, pkd    !Scaled pressure (hPa).
     
!     OUTPUT ARGUMENTS:
!     -----------------
      real (kind=dbl_kind), intent(out), dimension(ncol,nlm)::
     &  fkg    !Gaseous absorption coefficients (cm-atm)^-1.  

! LOCAL VARIABLES:

      integer (kind=int_kind)
     &  i, l, i1, i2

      real (kind=dbl_kind)
     &  x1,x2,y1
     
!-----------------------------------------------------------------------

      do l = 1, nlm
         do i = 1, ncol
            i1=ip1(i,l)
            i2=ip2(i,l)
            y1 = tt(i,l) - 245.0
            if (y1.lt.-65.) y1 = -65.
            if (y1.gt.75.) y1 = 75.
            x1 = exp(coefks(1,i1)+coefks(2,i1)*y1+coefks(3,i1)*y1**2)
            if(i1.eq.i2) then
               fkg(i,l) = x1 * pkd(i,l)
            else
               x2 = exp(coefks(1,i2)+coefks(2,i2)*y1+coefks(3,i2)*y1**2)
               fkg(i,l) = x1 + ( x2 - x1 ) * pkd(i,l)
            endif
         enddo
      enddo

      return
      end subroutine qks

!-----------------------------------------------------------------------
