!-----------------------------------------------------------------------

      subroutine rad_qps
     +               (      ksdm ,  n_physics ,    nagrid ,       nlm
     +,                    pcos1 ,      pcos2 ,     psin1 ,     psin2
     +,                 tauw_vis ,   taui_vis ,  taum_vis ,  taut_vis
     +,                 tauw_irw ,   taui_irw ,  taum_irw ,  taut_irw
     +,                     pl2g ,     tgeff4 ,     lwtoa ,   lwtoacl
     +,                  lwupsfc ,    lwdnsfc , lwdnsfccl ,  lwnetsfc
     +,               lwnetsfccl ,      lwtom ,   lwtomcl ,   lwpblfu
     +,                  lwpblfd ,      swinc ,     swtoa ,   swtoacl
     +,                 swnettoa , swnettoacl ,   swdnsfc , swdnsfccl
     +,                 swnetsfc , swnetsfccl ,     swtom ,   swtomcl
     +,                   radvdc ,     radndc ,    radvbc ,    radnbc
     +,                  radvdcc ,    radndcc ,   radvbcc ,   radnbcc
     +,                    scosz ,       sfac ,    alvdrc ,    alvdfc
     +,                   alndrc ,     alndfc ,       atl ,     atlcl
     +,                      asl ,      aslcl ,      lon0 ,      lat0
     +,                      o3l 
#ifdef rad_isccp
     +                   , isccp
#endif
     +                )

      use kinds
#ifndef scm
      use qp_module
      use gmeans_module
#endif
      use rad_params
      
      implicit none
      
!-----------------------------------------------------------------------
! REFERENCES:
! rad_qps calculates the qp2s and qp3s diagnostics of the long-wave and
! short-wave radiation codes.
! Laura D. Fowler/slikrock (02-01-01).

! send comments to laura@atmos.colostate.edu

! added global mean diagnostics
!       to this module.
!   2005-09-20 dazlich@atmos.colostate.edu

! SUBROUTINES CALLED:
!     

! FUNCTIONS CALLED:
!     none.

! INCLUDED COMMON BLOCKS:
!     none

! ARGUMENT LIST VARIABLES:
!     INPUT ARGUMENTS:
!     ----------------
      integer (kind=int_kind)::
     &  ksdm      !loop index.
     &, n_physics !nb of short-time steps per hour. 
     &, nagrid    !length of total domain.
     &, nlm       !number of layers.

      real (kind=dbl_kind), intent(in)::
     &  pcos1     !cosine of first diurnal harmonic                (-).
     &, psin1     !sine of second diurnal harmonic                 (-).
     &, pcos2     !cosine of first diurnal harmonic                (-).
     &, psin2     !sine of second diurnal harmonic                 (-).

      real (kind=dbl_kind), intent(in), dimension(nagrid)::
     &  tgeff4    !surface temperature^4                         (K^4).
     &, lwtoa     !toa all-sky outgoing lw radiation           (W/m^2).
     &, lwtoacl   !toa clear-sky outgoing lw radiation         (W/m^2).
     &, lwdnsfc   !sfc all-sky downward lw radiation           (W/m^2).
     &, lwdnsfccl !sfc clear-sky downward lw radiation         (W/m^2).
     &, lwnetsfc  !sfc all-sky net lw radiation                (W/m^2).
     &, lwnetsfccl!sfc clear-sky net lw radiation              (W/m^2).
     &, lwupsfc   !sfc upward lw radiation                     (W/m^2).
     &, lwtom     !tom all-sky net lw radiation                (W/m^2).
     &, lwtomcl   !tom clear-sky net lw radiation              (W/m^2).
     &, lwpblfu   !lw upward flux inside the PBL                (W/m2).
     &, lwpblfd   !lw downward flux inside the PBL              (W/m2).
     &, swinc     !toa incident solar radiation                (W/m^2).
     &, swtoa     !toa all-sky reflected sw radiation          (W/m^2).
     &, swtoacl   !toa clear-sky reflected sw radiation        (W/m^2).
     &, swnettoa  !toa net sw radiation                        (W/m^2).
     &, swnettoacl!toa clear-sky net sw radiation              (W/m^2).
     &, swdnsfc   !sfc downward sw radiation                   (W/m^2).
     &, swdnsfccl !sfc clear-sky net sw radiation              (W/m^2).
     &, swnetsfc  !sfc net sw radiation                        (W/m^2).
     &, swnetsfccl!sfc clear-sky net sw radiation              (W/m^2).
     &, swtom     !tom all-sky net sw radiation                (W/m^2).
     &, swtomcl   !tom clear-sky net sw radiation              (W/m^2).
     &, radvbc    !sfc visible all-sky direct net sw           (W/m^2).
     &, radvdc    !sfc visible all-sky diffuse net sw          (W/m^2).
     &, radnbc    !sfc near-ir all-sky direct net sw           (W/m^2).
     &, radndc    !sfc near-ir all-sky diffuse net sw          (W/m^2).
     &, radvbcc   !sfc visible clear-sky direct net sw         (W/m^2).
     &, radvdcc   !sfc visible clear-sky diffuse net sw        (W/m^2).
     &, radnbcc   !sfc near-ir clear-sky direct net sw         (W/m^2).
     &, radndcc   !sfc near-ir clear-sky diffuse net sw        (W/m^2).
     &, alvdrc    !sfc visible direct albedo                       (-).
     &, alvdfc    !sfc visible diffuse albedo                      (-).
     &, alndrc    !sfc near-ir direct albedo                       (-).
     &, alndfc    !sfc near-ir diffuse albedo                      (-).
     &, scosz     !cosine of solar zenith angle                    (-).
     &, sfac      !correction for length of day                    (-).

      real (kind=dbl_kind), intent(in), dimension(nagrid)::
     &  lon0      !longitude of grid cells                   (radians).
     &, lat0      !latitude of grid cells                    (radians).

      real (kind=dbl_kind), intent(in), dimension(nagrid,nlm)::
     &  atl       !long-wave radiative heating rate              (K/s).
     &, asl       !short-wave radiative heating rate             (K/s).
     &, atlcl     !clear-sky lw radiative heating rate           (K/s).
     &, aslcl     !clear-sky sw radiative heating rate           (K/s).
     &, tauw_vis  !optical depth of water clouds (sw:vis)          (-).
     &, taui_vis  !optical depth of ice clouds (sw:vis)            (-).
     &, taum_vis  !optical depth of mixed-phase clouds (sw:vis)    (-).
     &, taut_vis  !optical depth of total clouds (vis)             (-).
     &, tauw_irw  !optical depth of water clouds (lw:window)       (-).
     &, taui_irw  !optical depth of ice clouds (lw:window)         (-).
     &, taum_irw  !optical depth of mixed-phase clouds (lw:win)    (-).
     &, taut_irw  !optical depth of total clouds (lw:window)       (-).
     &, o3l       !ozone mixing ratio                          (kg/kg).

      real (kind=dbl_kind), intent(in), dimension(nagrid,nlm+1)::
     &  pl2g         !level pressure                             (hPa).

#ifdef rad_isccp
      real (kind=dbl_kind), dimension(nagrid,5,3)::
     &  isccp
#endif

! LOCAL VARIABLES:

      integer(kind=int_kind)::
     &  l
      real (kind=dbl_kind)::
     &  x_phys
      real (kind=dbl_kind), dimension(nagrid)::
     &  temp
      real (kind=dbl_kind), dimension(nagrid,nlm)::
     &  dplg, temp2

!-----------------------------------------------------------------------

      x_phys = radc1*n_physics
      
      do l = 1, nlm
         dplg(:,l) = pl2g(:,l+1)-pl2g(:,l)
      enddo

#ifndef scm
!---- lw qp2 diagnostics:

      call qp2sum(lwtoa,-1._dbl_kind,iqp2_rlut,ksdm,nagrid)

      call qp2sum(lwtoacl,-1._dbl_kind,iqp2_rlutcs,ksdm,nagrid)

      call qp2sum(lwupsfc,-1._dbl_kind,iqp2_rlus,ksdm,nagrid) 

      call qp2sum(lwdnsfc,1._dbl_kind,iqp2_rlds,ksdm,nagrid)

      call qp2sum(lwdnsfccl,1._dbl_kind,iqp2_rldscs,ksdm,nagrid)

      call qp2sum(lwnetsfc,-1._dbl_kind,iqp2_rls,ksdm,nagrid)

      call qp2sum(lwnetsfccl,-1._dbl_kind,iqp2_lwnetsfccl,ksdm,nagrid)

      call qp2sum(-lwtom,1._dbl_kind,iqp2_lwtom,ksdm,nagrid)
     
      call qp2sum(-lwtomcl,1._dbl_kind,iqp2_lwtomcl,ksdm,nagrid)
      
      call qp2sum(-lwpblfu,1._dbl_kind,iqp2_lwpblfu,ksdm,nagrid)
     
      call qp2sum(lwpblfd,1._dbl_kind,iqp2_lwpblfd,ksdm,nagrid)

      temp(:) = tgeff4(:)**0.25
      call qp2sum(temp,1._dbl_kind,iqp2_lwplanck,ksdm,nagrid)

!     call qp2sum(lwnetsfc,pcos1)

!     call qp2sum(lwnetsfc,psin1)

!     call qp2sum(lwnetsfc,pcos2)

!     call qp2sum(lwnetsfc,psin2)

!     call qp2sum(lwnetsfccl,pcos1)

!     call qp2sum(lwnetsfccl,psin1)

!     call qp2sum(lwnetsfccl,pcos2)

!     call qp2sum(lwnetsfccl,psin2)

!     call qp2sum(lwtoa,pcos1)
     
!     call qp2sum(lwtoa,psin1)

!     call qp2sum(lwtoa,pcos2)

!     call qp2sum(lwtoa,psin2)

!     call qp2sum(lwtoacl,pcos1)
     
!     call qp2sum(lwtoacl,psin1)

!     call qp2sum(lwtoacl,pcos2)

!     call qp2sum(lwtoacl,psin2)

      temp(:) = radc0
      do l = 1, nlm
         temp(:) = temp(:) + tauw_irw(:,l)
      enddo
      call qp2sum(temp,1._dbl_kind,iqp2_codtwirw,ksdm,nagrid)
      temp(:) = radc0
      do l = 1, nlm
         temp(:) = temp(:) + taui_irw(:,l)
      enddo
      call qp2sum(temp,1._dbl_kind,iqp2_codtiirw,ksdm,nagrid)
      temp(:) = radc0
      do l = 1, nlm
         temp(:) = temp(:) + taum_irw(:,l)
      enddo
      call qp2sum(temp,1._dbl_kind,iqp2_codtmirw,ksdm,nagrid)
      temp(:) = radc0
      do l = 1, nlm
         temp(:) = temp(:) + taut_irw(:,l)
      enddo
      call qp2sum(temp,1._dbl_kind,iqp2_codtirw,ksdm,nagrid)

!---- lw qp3 diagnostics:

      call qp3sum(atl(1,1),nagrid,x_phys,iqp3_tntlw,ksdm,nagrid)
      
      call qp3sum(atlcl(1,1),nagrid,x_phys,iqp3_tntlwcs,ksdm,nagrid)
      
      temp2(:,:) = tauw_irw(:,:)/dplg(:,:)
      call qp3sum(temp2(1,1),nagrid,x_phys,iqp3_codwirw,ksdm,nagrid)

      temp2(:,:) = taui_irw(:,:)/dplg(:,:)
      call qp3sum(temp2(1,1),nagrid,x_phys,iqp3_codiirw,ksdm,nagrid)
      
      temp2(:,:) = taum_irw(:,:)/dplg(:,:)
      call qp3sum(temp2(1,1),nagrid,x_phys,iqp3_codmirw,ksdm,nagrid)

      temp2(:,:) = taut_irw(:,:)/dplg(:,:)
      call qp3sum(temp2(1,1),nagrid,x_phys,iqp3_codtirw,ksdm,nagrid)

!---- lw high-frequency diagnostics:

      call hf2sum(lwtoa(1),-1._dbl_kind,ihf2_rlut,ksdm,nagrid)

!---- sw qp2 diagnostics:

      call qp2sum(swinc,1._dbl_kind,iqp2_rsdt,ksdm,nagrid)

      call qp2sum(swtoa,-1._dbl_kind,iqp2_rsut,ksdm,nagrid)

      call qp2sum(swtoacl,-1._dbl_kind,iqp2_rsutcs,ksdm,nagrid)

      call qp2sum(swnettoa,1._dbl_kind,iqp2_rst,ksdm,nagrid)

      call qp2sum(swnettoacl,1._dbl_kind,iqp2_swnettoacl,ksdm,nagrid)

      call qp2sum(swdnsfc,1._dbl_kind,iqp2_rsds,ksdm,nagrid)

      call qp2sum(swdnsfccl,1._dbl_kind,iqp2_rsdscs,ksdm,nagrid)

      call qp2sum(swnetsfc,1._dbl_kind,iqp2_rss,ksdm,nagrid)

      call qp2sum(swnetsfccl,1._dbl_kind,iqp2_swnetsfccl,ksdm,nagrid)

      call qp2sum(swtom,1._dbl_kind,iqp2_swtom,ksdm,nagrid)

      call qp2sum(swtomcl,1._dbl_kind,iqp2_swtomcl,ksdm,nagrid)
      
      call qp2sum(radvbc,1._dbl_kind,iqp2_swradvbc,ksdm,nagrid)

      call qp2sum(radnbc,1._dbl_kind,iqp2_swradnbc,ksdm,nagrid)

      call qp2sum(radvdc,1._dbl_kind,iqp2_swradvdc,ksdm,nagrid)

      call qp2sum(radndc,1._dbl_kind,iqp2_swradndc,ksdm,nagrid)

      call qp2sum(radvbcc,1._dbl_kind,iqp2_swradvbcc,ksdm,nagrid)

      call qp2sum(radnbcc,1._dbl_kind,iqp2_swradnbcc,ksdm,nagrid)

      call qp2sum(radvdcc,1._dbl_kind,iqp2_swradvdcc,ksdm,nagrid)

      call qp2sum(radndcc,1._dbl_kind,iqp2_swradndcc,ksdm,nagrid)

      call qp2sum(alvdrc,1._dbl_kind,iqp2_swalvdr,ksdm,nagrid)

      call qp2sum(alvdfc,1._dbl_kind,iqp2_swalvdf,ksdm,nagrid)
      
      call qp2sum(alndrc,1._dbl_kind,iqp2_swalndr,ksdm,nagrid)

      call qp2sum(alndfc,1._dbl_kind,iqp2_swalndf,ksdm,nagrid)
      
      call qp2sum(scosz,1._dbl_kind,iqp2_swcosz,ksdm,nagrid)

      call qp2sum(sfac,1._dbl_kind,iqp2_swsfac,ksdm,nagrid)

!     call qp2sum(lon0,1._dbl_kind,iqp2_radlon,ksdm,nagrid)
      
!     call qp2sum(lat0,1._dbl_kind,iqp2_radlat,ksdm,nagrid)

      temp(:) = radc0
      do l = 1, nlm
         temp(:) = temp(:) + tauw_vis(:,l)
      enddo
      call qp2sum(temp,1._dbl_kind,iqp2_codtwvis,ksdm,nagrid)
      temp(:) = radc0
      do l = 1, nlm
         temp(:) = temp(:) + taui_vis(:,l)
      enddo
      call qp2sum(temp,1._dbl_kind,iqp2_codtivis,ksdm,nagrid)
      temp(:) = radc0
      do l = 1, nlm
         temp(:) = temp(:) + taum_vis(:,l)
      enddo
      call qp2sum(temp,1._dbl_kind,iqp2_codtmvis,ksdm,nagrid)
      temp(:) = radc0
      do l = 1, nlm
         temp(:) = temp(:) + taut_vis(:,l)
      enddo
      call qp2sum(temp,1._dbl_kind,iqp2_codtvis,ksdm,nagrid)

!---- sw qp3 diagnostics:

      call qp3sum(asl(1,1),nagrid,x_phys,iqp3_tntsw,ksdm,nagrid)
      
      call qp3sum(aslcl(1,1),nagrid,x_phys,iqp3_tntswcs,ksdm,nagrid)

      call qp3sum(o3l(1,1),nagrid,x_phys,iqp3_o3,ksdm,nagrid)

      temp2(:,:) = tauw_vis(:,:)/dplg(:,:)
      call qp3sum(temp2(1,1),nagrid,x_phys,iqp3_codwvis,ksdm,nagrid)

      temp2(:,:) = taui_vis(:,:)/dplg(:,:)
      call qp3sum(temp2(1,1),nagrid,x_phys,iqp3_codivis,ksdm,nagrid)
      
      temp2(:,:) = taum_vis(:,:)/dplg(:,:)
      call qp3sum(temp2(1,1),nagrid,x_phys,iqp3_codmvis,ksdm,nagrid)

      temp2(:,:) = taut_vis(:,:)/dplg(:,:)
      call qp3sum(temp2(1,1),nagrid,x_phys,iqp3_codtvis,ksdm,nagrid)

!---- diagnostics of isccp cloud diagnostics:
#ifdef rad_isccp
      call qp2sum(isccp(:,1,1),1._dbl_kind,iqp2_fhcld1,ksdm,nagrid)
      call qp2sum(isccp(:,2,1),1._dbl_kind,iqp2_fhcld2,ksdm,nagrid)
      call qp2sum(isccp(:,3,1),1._dbl_kind,iqp2_fhcld3,ksdm,nagrid)
      call qp2sum(isccp(:,4,1),1._dbl_kind,iqp2_fhcld4,ksdm,nagrid)
      call qp2sum(isccp(:,5,1),1._dbl_kind,iqp2_fhcld5,ksdm,nagrid)

      call qp2sum(isccp(:,1,2),1._dbl_kind,iqp2_fmcld1,ksdm,nagrid)
      call qp2sum(isccp(:,2,2),1._dbl_kind,iqp2_fmcld2,ksdm,nagrid)
      call qp2sum(isccp(:,3,2),1._dbl_kind,iqp2_fmcld3,ksdm,nagrid)
      call qp2sum(isccp(:,4,2),1._dbl_kind,iqp2_fmcld4,ksdm,nagrid)
      call qp2sum(isccp(:,5,2),1._dbl_kind,iqp2_fmcld5,ksdm,nagrid)

      call qp2sum(isccp(:,1,3),1._dbl_kind,iqp2_flcld1,ksdm,nagrid)
      call qp2sum(isccp(:,2,3),1._dbl_kind,iqp2_flcld2,ksdm,nagrid)
      call qp2sum(isccp(:,3,3),1._dbl_kind,iqp2_flcld3,ksdm,nagrid)
      call qp2sum(isccp(:,4,3),1._dbl_kind,iqp2_flcld4,ksdm,nagrid)
      call qp2sum(isccp(:,5,3),1._dbl_kind,iqp2_flcld5,ksdm,nagrid)
#endif
#endif

      end subroutine rad_qps

!-----------------------------------------------------------------------
