!-----------------------------------------------------------------------

      subroutine rad_sfc
     +               (    nlen ,       len ,       mbs
     +,                   mbir ,        ts ,     alvdr
     +,                  alndr ,     alvdf ,     alndf
     +,                  sfc_t , sfc_asdir , sfc_asdif
     +,               sfc_emis
     +               )

      use kinds
      implicit none

!-----------------------------------------------------------------------

! references:
! rad_sfc sets the surface variables needed by the radiation codes, and
! in particular the surface albedo and surface emissivity.
! Laura D. Fowler/slikrock (02-01-01).

! send comments to laura@atmos.colostate.edu

! subroutines called:
!    none.

! argument list variables:
!     input arguments:
!     ----------------
      integer (kind=int_kind), intent(in)::
     &  nlen       !length of horizontal domain.
     &, len        !length of horizontal domain without ghost cells.
     &, mbs        !number of spectral intervals at short wavelengths.
     &, mbir       !number of spectral intervals at long wavelengths.

      real (kind=dbl_kind), intent(in), dimension(nlen)::
     &  alvdf      !diffuse surface albedo in visible spectrum     (-).
     &, alndf      !diffuse surface albedo in near-ir spectrum     (-).
     &, alvdr      !direct surface albedo in visible spectrum      (-).
     &, alndr      !direct surface albedo in near-ir spectrum      (-).
     &, ts         !surface temperature                            (K).

!     output arguments:
!     -----------------
      real (kind=dbl_kind), intent(out), dimension(len)::
     &  sfc_t      !surface temperature                            (K).

      real (kind=dbl_kind), dimension(len,mbs)::
     &  sfc_asdir  !spectral surface albedo (direct beam)          (-).
     &, sfc_asdif  !spectral surface albedo (diffuse beam)         (-).

      real (kind=dbl_kind), dimension(len,mbir)::
     &  sfc_emis   !spectral surface emissivity                    (-).

! local variables:

      integer (kind=int_kind)::
     &  i

!-----------------------------------------------------------------------

      do i = 1, len
      
         sfc_t(i)  = ts(i)**0.25_dbl_kind

         sfc_asdir(i,1)     = alvdr(i) ; sfc_asdir(i,2:mbs) = alndr(i)
         sfc_asdif(i,1)     = alvdf(i) ; sfc_asdif(i,2:mbs) = alndf(i)
      
         sfc_emis(i,1:mbir) = 1.0
     
      enddo

      end subroutine rad_sfc

!-----------------------------------------------------------------------
