!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module renormalization

!-----------------------------------------------------------------------
! PURPOSE: for divergence and relative vorticity to zero on each layer
!-----------------------------------------------------------------------

        use kinds
	use model_parameters
	use domain_decomposition
        use grid_metrics
        use prognostics
        use global_reductions

	implicit none

	private

	public ::
     *	  renormalize

!*********************************************************************

      contains

!*********************************************************************

	subroutine renormalize

	integer (kind=int_kind) :: k,size_2d
	real (kind=dbl_kind) :: div_avg, eta_avg,tot_area
	real (kind=dbl_kind), dimension(iim,jjm,nsdm) :: one = 1.0_dbl_kind
        real (kind=dbl_kind), dimension(iim,jjm,nsdm) :: area_mask

	size_2d = iim*jjm*nsdm
	area_mask = area * grid_center_mask

	tot_area = global_real_sum(
     *                  area_mask(:,:,:),
     *                  one(:,:,:),
     *                  size_2d)
	write(6,*) tot_area

	do k=1,nlm
          div_avg = global_real_sum(
     *                  div(:,:,k,n4,:),
     *                  area_mask(:,:,:),
     *                  size_2d) / tot_area
          eta_avg = global_real_sum(
     *			eta(:,:,k,n4,:),
     *                  area_mask(:,:,:),
     *                  size_2d) / tot_area

	  div(:,:,k,:,:) = div(:,:,k,:,:) - div_avg
	  eta(:,:,k,:,:) = eta(:,:,k,:,:) - eta_avg
	enddo

	if(my_task.eq.0) write(6,148) div_avg, eta_avg

 148	format( /,' div and eta have been renormalized to zero '/
     *		  '     drift in div(k=nlm)  : ',e20.10, /
     *		  '     drift in eta(k=nlm)  : ',e20.10, /)

#ifdef mpi
	call MPI_BARRIER (MPI_COMM_ATMOS,ierr)
#endif

        end subroutine renormalize

        end module renormalization
!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
