!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module domain_decomposition_sfc


!***********************************************************************
!
!	this module defines how the horizontal domain of the 
!	model surface is decomposed.
!
!	it also contains mpi specific information
!
!	this module is developed with analogy to domain_decomposition
!
!***********************************************************************

!-------------------------------------------------------------------------
!  VARIABLE DEFINITIONS
!
!	iim_sfc		= number of cells in i-direction
!	jjm_sfc		= number of cells in j-direction
!	npe_sfc		= number of processes for AGCM
!	nsdm_sfc	= number of subdomains each process owns
!-------------------------------------------------------------------------

      use kinds
      use domain_decomposition

      implicit none
      save

        integer (kind=int_kind), parameter ::
     *                  npe_sfc  = npe_atmos,
     *                  nsdm_sfc = nsdm,
     *                  im_sfc  = im,
     *                  jm_sfc  = jm,
     *                  iim_sfc  = iim,
     *                  jjm_sfc  = jjm,
     *                  iimjjm_sfc = iim_sfc * jjm_sfc,
     *                  nsfc_tot = iim_sfc * jjm_sfc * nsdm_sfc,
     *                  nsfc = nsfc_tot / npe_sfc + 1,
     *                  sfc_level = max_level,
     *                  max_ig_sfc = max_ig,
     *                  im5_sfc = im_sfc/5


	logical (kind=log_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) :: 
     *			grid_mask_sfc
        real (kind=dbl_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc) ::
     *			area_sfc

	integer (kind=int_kind), dimension(iim_sfc, jjm_sfc, nsdm_sfc) ::
     *     		grid_indx_sfc

        integer (kind=int_kind), parameter :: my_task_sfc = 0


	end module domain_decomposition_sfc

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
