!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

	module initialize

!-------------------------------------------------------------------------
!  PURPOSE:
!	1) initialize data to zero
!	2) read in data defining the grid structure
!	3) read in data to interpolate from geodesic grid to lat/lon
!	4) read in run-time variables
!	5) read in boundary conditions
!	6) read in or generate initial conditions
!	7) dump initial conditions to a data file

!---- modifications:
!     added initialization related to the eau_ng parameterization of frac
!     tional cloudiness.
!     Laura D. Fowler/mcbug (09-06-02).

!-------------------------------------------------------------------------

	use physical_parameters
	use model_parameters
	use domain_decomposition
	use model_control
	use prognostics
	use timer
	use hstatc
	use sigma_coordinate
	use fluxcoupler_fields
	use time_manager_module
        use scm_module
	use tracer_names
	use adjustments
	use post_step_module
#ifdef eau_ng
      use eau_names
#endif      

	implicit none
	save

!*************************************************************************

	contains

!*************************************************************************


!======================================================================
! BEGINNING OF INITIALIZE_MODEL
!======================================================================

	subroutine initialize_model

!-------------------------------------------------------------------------
! INITIALIZATION
!-------------------------------------------------------------------------
	call init_model_parameters
	call init_model_control
	call init_prognostics
	call init_adjustments
	call init_vertical_coordinate
	call init_tracer_names
	call init_time_manager
#ifdef eau_ng
      call init_eau_names
#endif

!-------------------------------------------------------------------------
! get runtime parameters
!-------------------------------------------------------------------------
        call input

!-------------------------------------------------------------------------
! read in boundary data (32 bit)
!-------------------------------------------------------------------------
        call read_atm_bc(3)

!-------------------------------------------------------------------------
! read in restart if desired, l_restart set in namelist file
!-------------------------------------------------------------------------
        call read_restart
	  tau_end = tau_end + tau
        write(6,*) my_task, tau_end, tau

!-------------------------------------------------------------------------
! get pointers based on step_count
!-------------------------------------------------------------------------
	call get_pointers

!---------------------------------------------------------------------------
! ALL INITIAL DATA HAS BEEN READ IN OR GENERATED
!---------------------------------------------------------------------------

#ifdef link_to_gcm_physics
!-----------------------------------------------------------------------
! Initialize and get data for the Single column model
!-----------------------------------------------------------------------
         call scm_init(tau)
!         grid_center_lon = lon_scm
!         grid_center_lat = lat_scm
#endif

!-----------------------------------------------------------------------
! DO HYDROSTATICS and derive diagnostic variables
!-----------------------------------------------------------------------
	call post_step

!-----------------------------------------------------------------------
! initialize fields that may be needed in the flux coupler
!-----------------------------------------------------------------------
	spdm_atm(:,:,:) = spdm(:,:,:)
	shear1_atm(:,:,:) = shear1(:,:,:)
        tpgvar_atm(:,:,:) = tpgvar(:,:,:)

	end subroutine initialize_model
!======================================================================
! END OF INITIALIZE_MODEL
!======================================================================

	end module initialize

!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
