!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

	module model_control

!-----------------------------------------------------------------
! 	this module hold variables which are specified at runtime
!	  or vary as the integration proceeds.does most of the i/o

!---- modifications:
!

!-----------------------------------------------------------------

	use kinds
	use physical_parameters
	use domain_decomposition
	use prognostics
	use time_manager_module

	implicit none
	save

	integer (kind=int_kind) :: &
    			icase, &
     			ksdm, &
     			ioerr

	character (len=40) :: simulation_tag

	integer (kind = int_kind), parameter :: interp_file = 47

!-----------------------------------------------------------------
!  VARIABLE DEFINITION
!-----------------------------------------------------------------
!
!  character strings
!       simulation_tag = name of test case
!
!  integer scalar
!       icase = case number
!
!-----------------------------------------------------------------


!****************************************************************

	contains

!****************************************************************


!======================================================================
! BEGINNING OF INIT_MODEL_CONTROL
!======================================================================
	subroutine init_model_control

	implicit none

	icase = 0
	simulation_tag = ''

	end subroutine init_model_control
!======================================================================
! END OF INIT_MODEL_CONTROL
!======================================================================
	

!======================================================================
! BEGINNING OF READ_RESTART
!======================================================================
	subroutine read_restart
	
	implicit none

	integer (kind=int_kind) :: i,j,nt,np,ibeg,iend
        integer (kind=int_kind) :: &
            max_ig_tem, nlm_tem, ntprog_tem, ntracer_tem, &
            nsecond_tem, step_count_tem
	logical (kind=log_kind), parameter :: psc_to_gsc = .false.

        real (kind=dbl_kind) :: xrec1, xrec2
        real (kind=dbl_kind), dimension(nlm) :: x1, x2

        open(4,file='./data/ic_atm',form='formatted', action='read', &
             status='old', iostat=ioerr )
        if (ioerr /= 0) then
          write(*,*) '*** read_restart:  ic_atm file error ',ioerr
          stop
        endif

!-------------------------------------------------------------------------
!  get the header record
!-------------------------------------------------------------------------
        read(4,*) nsecond_dyn,step_count,tau,max_ig_tem,nlm_tem, &
                  ntracer_tem,ntprog_tem

!-------------------------------------------------------------------------
!  get the data records
!-------------------------------------------------------------------------
        if (nlm_tem .ne. nlm) then 
          write(*,*) '>>> nlm_tem does not equal nlm!!!' 
          write(*,*) '>>> nlm_tem = ',nlm_tem,' nlm = ',nlm
          write(*,*) '>>> i.e., Number of layers in restart file does ', &
            'not match nlm in sigma_coordinate.F'
          write(*,*) '>>> EXITING!!!' 
          stop
        endif
        read(4,*) xrec1, xrec2
        ps(:,:,1,:) = xrec1 ; ps(:,:,2,:) = xrec2
        read(4,*) xrec1, xrec2
        pb(:,:,1,:) = xrec1 ; pb(:,:,2,:) = xrec2
        
        read(4,*) x1, x2
        eta(1,1,1:nlm,1,1) = x1(:) ; eta(1,1,1:nlm,2,1) = x2(:)
        read(4,*) x1, x2
        div(1,1,1:nlm,1,1) = x1(:) ; div(1,1,1:nlm,2,1) = x2(:)
        read(4,*) x1, x2
        th(1,1,1:nlm,1,1) = x1(:) ; th(1,1,1:nlm,2,1) = x2(:)

        do nt = 1,nwater
          read(4,*) x1, x2
          tracer(1,1,1:nlm,nt,1,1) = x1(:)
          tracer(1,1,1:nlm,nt,2,1) = x2(:)
        enddo
        close(4)
        write(6,*) nsecond_dyn,step_count,tau,max_ig_tem,nlm_tem, &
                   ntracer_tem,ntprog_tem
        !write(*,*) '>>theta = ',th(1,1,:,1,1)
        !write(*,*) '>>q = ',tracer(1,1,:,1,1,1)
        !pause
!-------------------------------------------------------------------------
! set step_count equal to zero, so the model begins with an Euler Forward
!-------------------------------------------------------------------------
        step_count = 0

!-------------------------------------------------------------------------
! reset pb if converting from a Phillips Sigma coordinate to Generalized
!-------------------------------------------------------------------------
	if (psc_to_gsc) pb = 0.9*ps


!-------------------------------------------------------------------------
! get pointers
!-------------------------------------------------------------------------
	call get_pointers

        end subroutine read_restart
!======================================================================
! END OF READ_RESTART
!======================================================================


!===========================================================================
! BEGINNING OF GET_POINTERS
!===========================================================================
        subroutine get_pointers

!----------------------------------------------------------------------------
! pointers hardwired for Adams Bashforth 3rd Order integration
!----------------------------------------------------------------------------
        n4   = MOD (step_count+1,2) + 1 ! {2,1}
        n3   = MOD (step_count+0,2) + 1 ! {1,2}
        n2   = 0
        n3_f = MOD (step_count+2,3) + 1 ! {3,1,2}
        n2_f = MOD (step_count+1,3) + 1 ! {2,3,1}
        n1_f = MOD (step_count+0,3) + 1 ! {1,2,3}

        end subroutine get_pointers
!===========================================================================
! END OF GET_POINTERS
!===========================================================================



!===========================================================================
! BEGINNING OF GET_WEIGHTS
!===========================================================================
        subroutine get_weights

        implicit none

        real (kind=dbl_kind), parameter :: &
            w3_ab3 = 1.91666666666666666666666666666666, &
            w2_ab3 =-1.33333333333333333333333333333333, &
            w1_ab3 = 0.41666666666666666666666666666666

        if (step_count.LE.1) then ! Euler forward time step
          w3 =  1.0
          w2 =  0.0
          w1 =  0.0
        endif
        if (step_count.EQ.2) then ! AB 2nd order time step
          w3 =  1.5
          w2 = -0.5
          w1 =  0.0
        endif
        if (step_count.GE.3) then ! AB 3rd order time step
          w3 = w3_ab3
          w2 = w2_ab3
          w1 = w1_ab3
        endif

        end subroutine get_weights
!===========================================================================
! END OF GET_WEIGHTS
!===========================================================================

!===========================================================================
! BEGINNING OF GET_WEIGHTS_SCM
!===========================================================================
        subroutine get_weights_scm

        implicit none

          w3 =  1.0
          w2 =  0.0
          w1 =  0.0

        end subroutine get_weights_scm
!===========================================================================
! END OF GET_WEIGHTS_SCM
!===========================================================================


!======================================================================
! BEGINNING OF READ_ATM_BC
!======================================================================
        subroutine read_atm_bc(iunit)
        
!-- read in surface geopotential, Coriolis parameter and 
!-- topographic variance

        implicit none

        integer (kind=int_kind) :: iunit, i, j, max_ig_tem, lrecl, wrecl, n
        character(char_len) :: filename='data/bc_atm'
        real (kind=real_kind) :: xval
        real (kind=real_kind), allocatable, dimension(:) :: xrec
        
!-----------------------------------------------------------------------
! open file, check error status
!-----------------------------------------------------------------------

          open(unit=iunit,file=filename,form='formatted', &
               status='old',iostat=ioerr)
          if (ioerr.ne.0) then
            write(6,*) ' read_atm_bc, error open ',ioerr
            stop
          endif

          read(iunit,*) xval  ; phis(:,:,:) = xval
          read(iunit,*) xval  ; f(:,:,:) = xval
          read(iunit,*) xval  ; tpgvar(:,:,:) = xval
           !write(*,*) 'tpgvar = ',tpgvar
           !read(*,*)
 
#ifdef gravity_wave_drag
!-------------------------------------------------------------------------
! average to cell walls
!-------------------------------------------------------------------------
        do ksdm = 1,nsdm
        call average_to_cell_corners(                &
                        ipad,                        &
                        iim,                         &
                        jjm,                         &
                        ndata,                       &
                        ncorners,                    &
                        s_weights(:,:,:,:,ksdm),     &
                        tpgvar(:,:,ksdm),            &
                        tpgvar_c(:,:,:,ksdm))
        enddo

#endif

!-----------------------------------------------------------------------
! close file
!-----------------------------------------------------------------------
          close(iunit)


!-------------------------------------------------------------------------
! Get tracer boundary conditions
!-------------------------------------------------------------------------

#ifdef passive_tracers
!-----------------------------------------------------------------------
! open file, check error status
!-----------------------------------------------------------------------

          open(unit=3,file='data/bc_co2',form='unformatted', &
               status='old',iostat=ioerr,access='DIRECT',recl=8)
          if (ioerr.ne.0) then
            write(6,*) ' bc_co2, error open ',ioerr
            tracer_dat = c0
          else

!-------------------------------------------------------------------------
!  get the header record
!-------------------------------------------------------------------------
            read(3,rec=1)max_ig_tem,lrecl

            close(3)

            open(unit=3,file='data/bc_co2',form='unformatted', &
                 status='old',iostat=ioerr,access='DIRECT',recl=lrecl)
            wrecl = lrecl/kind(xrec)
            allocate(xrec(wrecl))
            tracer_dat = c0

            do j = 1,jjm
              do i = 1,iim
                do ksdm = 1,nsdm
                   read(3,rec=1+1) xrec
                   do n = 1,min(npassive,wrecl/12)
                     tracer_dat(i,j,n,1:12,ksdm) = xrec(12*(n-1)+1:12*n)
                   enddo
                enddo
              enddo
            enddo

            deallocate(xrec)

!-----------------------------------------------------------------------
! close file
!-----------------------------------------------------------------------
          endif
          close(3)
#endif


        end subroutine read_atm_bc

!======================================================================
! END OF READ_ATM_BC
!======================================================================

!===========================================================================
! BEGINNING OF INPUT
!===========================================================================
      SUBROUTINE input
 
! shallow water version
! written by Ross Hiekes, CSU
 
! called from
!	dcontrol (currently main)
 
! load run time parameter settings
 
! variable definitions
 
!  character strings
!	test case = simulation_tag
 
!  integer scalar
!	icase = case number
 
!  real scalar
!	tau_end = time to end execution, if set equal to 0.0, tau_end=24.0*tau_end_day+tau_end_hour
!	tau_end_day = time in days to end execution
!	tau_end_hour = time in hours to end execution
  
!-----------------------------------------------------------------
! READ NAMELIST
! namelist is opened as unit 2

      if(my_task.eq.0) then
        OPEN (UNIT=2,FILE='data/namelist',FORM='FORMATTED',STATUS='OLD')
        READ (2,01) simulation_tag
        READ (2,*) icase
        READ (2,02) tau_end
        READ (2,02) tau_end_day
        READ (2,02) tau_end_hour
        READ (2,02) dt
        READ (2,*) n_pbp
#ifdef link_to_gcm_physics
        READ (2,*) n_physics
#endif

        CLOSE (UNIT=2)
   01   FORMAT (A40 )
   02   FORMAT (F7.2)
   03   FORMAT (I4  )
   04   FORMAT (L1  )

!----------------------------------------------------------------------
! compute time as the sum of days and hours if tau_end == 0.0
!----------------------------------------------------------------------
        IF (tau_end.EQ.0.0) tau_end = 24.*tau_end_day+tau_end_hour-0.001
 
! echo data to standard out
        PRINT *, ' input : simulation_tag = ',simulation_tag
        PRINT *, ' input : icase          = ',icase
        PRINT *, ' input : tau_end        = ',tau_end
        PRINT *, ' input : tau_end_day    = ',tau_end_day
        PRINT *, ' input : tau_end_hour   = ',tau_end_hour
        PRINT *, ' input : time step      = ',dt

	invdt = c1/dt
#ifdef link_to_gcm_physics
        dtphy = dt * n_physics
#endif

      endif 	! if(my_task.eq.0)

	end subroutine input
!===========================================================================
! END OF INPUT
!===========================================================================

!===========================================================================
! BEGINNING OF ECHO_DIRECTIVES
!===========================================================================
        subroutine echo_directives

	if(my_task.eq.0) then

        write(6,*) ' parameter settings for this run'
        write(6,*)

#if resolution == 02562
        write(6,*) '   resolution is 02562'
#endif

#if resolution == 10242
        write(6,*) '   resolution is 10242'
#endif

        write(6,*) '   time stepping is explicit'

#ifdef generalized_sigma
        write(6,*) '   generalized sigma coordinate is used'
#else
        write(6,*) '   phillips sigma coordinate is used'
#endif

#ifdef no_stratosphere
        write(6,*) '   there is no stratosphere zone'
#endif

#ifdef vertical_upwind_pbl
        write(6,*) '   the pbl tracers is vertically upwinded'
#endif

#ifdef vertical_upwind_pbl
        write(6,*) '   the pbl momentum is vertically upwinded'
#endif

#ifdef pbl_depth_limiter
        write(6,*) '   the pbl is depth limited'
#else
        write(6,*) '   the pbl is linearly restored'
#endif

#ifdef held_suarez
        write(6,*) '   Held-Saurez thermal forcing'
#endif

#ifdef held_suarez
        write(6,*) '   Held-Saurez momentum forcing'
#endif

#ifdef csu_momentum
        write(6,*) '   CSU momentum forcing'
#endif

#ifdef link_to_gcm_physics
        write(6,*) '   code is linked to BUGS gcm'
#endif

#ifdef del4
        write(6,*) '   del4 is turned on'
#endif

        write(6,*) '   dry convective adjustment is turned on'

#ifdef gravity_wave_drag
        write(6,*) '   gravity wave drag is turned on'
#endif

#ifdef passive_tracers
        write(6,*) '   transport of passive tracers is turned on'
#endif

        write(6,*) '   single column model is turned on'
	endif 	! if(my_task.eq.0)

	end subroutine echo_directives
!===========================================================================
! END ECHO_DIRECTIVES
!===========================================================================

	end module model_control

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
