!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

module sfc_init

!-----------------------------------------------------------------------
! PURPOSE:
! This file initializes the fields for the ocean,landice,and sib
! It reads in both initial conditions and boundary conditions
!-----------------------------------------------------------------------

  use kinds
  use model_parameters
  use pbl_parameters
  use domain_decomposition_sfc
  use fluxcoupler_fields_sfc
  use time_manager_module
  use scm_module, ONLY: lat_scm, lon_scm
  
  implicit none
      
  private

  real (kind=real_kind) :: lat_tem, lon_tem
  integer (kind=int_kind) :: ioerr
  real (kind=selected_real_kind(6)), dimension(:), allocatable ::  xrec4
  real (kind=selected_real_kind(13)), dimension(:), allocatable ::  xrec8

!-----------------------------------------------------------------------
! contains
!	init_sfc_variables
!	init_ocnflx
!	init_landice
!	init_sib
!-----------------------------------------------------------------------

  
  public :: init_sfc_variables, init_ocnflx, init_landice, init_sib,  &
            init_coupler

!***********************************************************************

  contains

!***********************************************************************


!======================================================================
! BEGIN SUBROUTINE INIT_SFC_VARIABLES
!======================================================================
  subroutine init_sfc_variables

    implicit none

!-----------------------------------------------------------------------
! initialize flux surface coupler fields
!-----------------------------------------------------------------------
    alvdr_sfc = c0
    alndf_sfc = c0
    alvdr_sfc = c0
    alndr_sfc = c0
    tgeff4_sfc = c0
    pc_sfc = c0
    cldpc_sfc = c0
    lwp_sfc = c0
    pblcw_sfc = c0
    pblci_sfc = c0
    radvdc_sfc = c0
    radndc_sfc = c0
    radvbc_sfc = c0
    radnbc_sfc = c0
    dlwbot_sfc = c0
    fluxd_sfc = c0
    dcupr_sfc = c0
    dlspr_sfc = c0
    dlsps_sfc = c0
    spdm_sfc = c0
    shear1_sfc = c0
    ps_sfc = c0
    psb_sfc = c0
    dpent_sfc = c0
    co2flx_sfc = c0

  end subroutine init_sfc_variables
!======================================================================
! END SUBROUTINE INIT_SFC_VARIABLES
!======================================================================


!======================================================================
! BEGIN SUBROUTINE INIT_OCNFLX
!======================================================================
  subroutine init_ocnflx

! This subroutine initializes the fields for the ocean surface flux
!    parameterization

!    use ocean_parameters
!    implicit none

!-----------------------------------------------------------------------
! local variables
!-----------------------------------------------------------------------
!    integer (kind=int_kind) :: i, j, ksdm, max_ig_tem, lrecl, nocean_tem, nsecond_tem

!-----------------------------------------------------------------------
!   ocean grid fields
!-----------------------------------------------------------------------
!    real (kind=real_kind) ::  area_tem, wght_tem
!    real (kind=real_kind), dimension(:), allocatable ::  xrec

!-----------------------------------------------------------------------
!    read in the gridmap file header
!-----------------------------------------------------------------------
!    open(unit=3,file=trim(datapath)//'data/gridmap_ocean',form='unformatted'
!     *,      access='DIRECT',recl=12,status='old',action='read'
!     *,      iostat=ioerr)
!        if (ioerr /= 0) then
!          write(*,*) '*** init_ocnflx: gridmap_ocean file error ',ioerr
!          stop
!        endif
!        read(3,rec=1)nocean,max_ig_tem,lrecl
!        close(3)

!       if(my_task_sfc.eq.0) then
!	write(6,*)
!	write(6,*) ' READING IN OCEAN COMPONENT DATA'
!	write(6,*) ' number of ocean points ',nocean,max_ig_tem
!       endif


!-----------------------------------------------------------------------
!    read in the gridmap file data
!-----------------------------------------------------------------------
!        lon_sea = c0
!        lat_sea = c0
!        open(unit=3,file=trim(datapath)//'data/gridmap_ocean',form='unformatted'
!     *,      access='DIRECT',recl=lrecl)
!        do ksdm = 1,nsdm_sfc
!          do j = 1,jjm_sfc
!            do i = 1,iim_sfc
!	      if(grid_mask_sfc(i,j,ksdm)) then
!                read(3,rec=1+grid_indx_sfc(i,j,ksdm))lon_tem,
!     *             lat_tem, area_tem
!     *,            wght_tem, link_hr_sea(i,j,ksdm), hr_sea(i,j,ksdm)
!	        if(hr_sea(i,j,ksdm)) then
!  		  region_flag = .true.
!                  lon_sea(i,j,ksdm) = lon_tem
!                  lat_sea(i,j,ksdm) = lat_tem
!                  area_sfc(i,j,ksdm) = area_tem
!	        else
!		  link_hr_sea(i,j,ksdm) = -10
!	        endif
!      endif
!            enddo
!          enddo
!        enddo

!        close(3)

!	if(maxval(lon_sea).gt.20.and.maxval(lat_sea).gt.20) then
!	  write(6,*) ' wrong units'
!	  write(6,*) maxval(lon_sea),maxval(lat_sea)
!	  stop
!	endif


!-----------------------------------------------------------------------
!    Read in the initial condition file
!      These fields are defined only at ocean points, and will be 
!      scattered to the global grid
!-----------------------------------------------------------------------
!        open(unit=4,file=trim(datapath)//'data/ic_ocean',form='unformatted'
!     *,      access='DIRECT', recl=16,status='old',action='read'
!     *,      iostat=ioerr)
!        if (ioerr /= 0) then
!          write(*,*) '*** init_ocnflx: ic_ocean file error ',ioerr
!          stop
!        endif
!        read(4,rec=1)nsecond_tem,nocean_tem,max_ig_tem,lrecl
!        close(4)

!        open(unit=4,file=trim(datapath)//'data/ic_ocean',form='unformatted'
!     *,      access='DIRECT', recl=lrecl)
!-----------------------------------------------------------------------
! read in data at ocean points
!-----------------------------------------------------------------------
!        if(lrecl.eq.24) then
!          if(my_task.eq.0)print *,'reading real*4 ic_ocean'
!          allocate(xrec4(6))
!          do ksdm = 1,nsdm_sfc
!            do j = 1,jjm_sfc
!              do i = 1,iim_sfc
!                if(hr_sea(i,j,ksdm)) then
!                   read(4,rec=1+link_hr_sea( i,j,ksdm) ) xrec4
!                   sst(i,j,ksdm)  = xrec4(1)
!                   ti(i,j,ksdm)   = xrec4(2)
!                   sn(i,j,ksdm)   = xrec4(3)
!                   iz(i,j,ksdm)   = xrec4(4)
!                   e2(i,j,ksdm)   = xrec4(5)
!                   tkeo(i,j,ksdm) = xrec4(6)
!	        endif
!              enddo
!            enddo
!          enddo
!          deallocate(xrec4)
!        elseif(lrecl.eq.48) then
!          if(my_task.eq.0)print *,'reading real*8 ic_ocean'
!          allocate(xrec8(6))
!          do ksdm = 1,nsdm_sfc
!            do j = 1,jjm_sfc
!              do i = 1,iim_sfc
!                if(hr_sea(i,j,ksdm)) then
!                   read(4,rec=1+link_hr_sea( i,j,ksdm) ) xrec8
!                   sst(i,j,ksdm)  = xrec8(1)
!                   ti(i,j,ksdm)   = xrec8(2)
!                   sn(i,j,ksdm)   = xrec8(3)
!                   iz(i,j,ksdm)   = xrec8(4)
!                   e2(i,j,ksdm)   = xrec8(5)
!                   tkeo(i,j,ksdm) = xrec8(6)
!	        endif
!              enddo
!            enddo
!          enddo
!          deallocate(xrec8)
!        endif

!-----------------------------------------------------------------------
! close file containing initial conditions
!-----------------------------------------------------------------------
!      close(4)
!      if(my_task_sfc.eq.0) write(6,*) ' ocean ic read in'

!-----------------------------------------------------------------------
! INITIAL CONDITIONS HAVE BEEN READ IN
!-----------------------------------------------------------------------


!-----------------------------------------------------------------------
!   read in the boundary conditions
!      These fields are defined only at ocean points, and will be 
!      scattered to the global grid
!-----------------------------------------------------------------------
!        open(unit=3,file=trim(datapath)//'data/bc_ocean',form='unformatted'
!     *,      access='DIRECT', recl=12,status='old',action='read'
!     *,      iostat=ioerr)
!        if (ioerr /= 0) then
!          write(*,*) '*** init_ocnflx: bc_ocean file error ',ioerr
!          stop
!        endif
!        read(3,rec=1)nocean_tem,max_ig_tem,lrecl
!        close(3)

!        open(unit=3,file=trim(datapath)//'data/bc_ocean',form='unformatted'
!     *,      access='DIRECT', recl = lrecl)

!        nmonth_bc = (lrecl/kind(xrec) - (2+12*3) ) / 2
!        allocate(xrec(2+12*3+2*nmonth_bc))
!        allocate(sstm(iim_sfc, jjm_sfc, nmonth_bc, nsdm_sfc))
!        allocate(izm(iim_sfc, jjm_sfc, nmonth_bc, nsdm_sfc))
!        sstm = 0
!        izm = 0
!        if(nmonth_bc.gt.12) read(3,rec=1)nocean_tem,max_ig_tem,lrecl,nyear0
!        do ksdm = 1,nsdm_sfc
!          do j = 1,jjm_sfc
!            do i = 1,iim_sfc
!              if(hr_sea(i,j,ksdm)) then
!                 read(3,rec=1+link_hr_sea( i,j,ksdm ) ) xrec
!                    zs_sea(i,j,ksdm) = xrec(1)
!                    hmax(i,j,ksdm) = xrec(2)
!                    sstm(i,j,:,ksdm) = xrec(3:2+nmonth_bc)
!                    izm(i,j,:,ksdm) = xrec(3+nmonth_bc:2+2*nmonth_bc)
!                    hmix(i,j,:,ksdm) = xrec(3+2*nmonth_bc:14+2*nmonth_bc)
!                    qfm(i,j,:,ksdm) = xrec(15+2*nmonth_bc:26+2*nmonth_bc)
!                    e2m(i,j,:,ksdm) = xrec(27+2*nmonth_bc:38+2*nmonth_bc)
!               endif
!            enddo
!          enddo
!        enddo
!        deallocate(xrec)

!-----------------------------------------------------------------------
! close boundary conditions file
!-----------------------------------------------------------------------
!       close(3)
!      if(my_task_sfc.eq.0) write(6,*) ' ocean bc read in'

!-----------------------------------------------------------------------
! BOUNDARY CONDITIONS HAVE BEEN READ IN
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! compute lat,lon, and surface height (gpm)
!-----------------------------------------------------------------------
!        where(hr_sea)
!              !zs_sfc(:,:,:)  = zs_sea(:,:,:)
!              lon_sfc(:,:,:) = lon_sea(:,:,:)
!              lat_sfc(:,:,:) = lat_sea(:,:,:)
!              frac_sfc(:,:,:) = 1.0
!        endwhere

      end subroutine init_ocnflx
!======================================================================
! END OF INIT_OCNFLX
!======================================================================


!======================================================================
! BEGINNING OF INIT_LANDICE
!======================================================================
      subroutine init_landice

! This subroutine initializes the fields for the land ice surface flux
!    parameterization

!      use landice_parameters
!      implicit none

!      real (kind=real_kind), dimension(iim_sfc,jjm_sfc,nsdm_sfc)
!     *,    intent(out) :: frac_sfc

!-----------------------------------------------------------------------
! local variables
!-----------------------------------------------------------------------
!      integer (kind=int_kind) :: i, j, ksdm
!     *,     max_ig_tem, lrecl, nice_tem, nsecond_tem

!-----------------------------------------------------------------------
!    read in the gridmap file header
!-----------------------------------------------------------------------
!        open(unit=3,file=trim(datapath)//'data/gridmap_landice',
!     *       form='unformatted'
!     *,      access='DIRECT',recl=12, status='old', action='read'
!     *,      iostat=ioerr)
!        if (ioerr /= 0) then
!          write(*,*) '*** init_landice: gridmap_landice file error '
!     *,     ioerr
!          stop
!        endif
!        read(3,rec=1)nlandice,max_ig_tem,lrecl
!        close(3)
!          write(*,*) 'init_landice: nlandice = ',nlandice,' max_ig_tem = ',max_ig_tem

!       if(my_task_sfc.eq.0) then
!	write(6,*)
!	write(6,*) ' READING IN ICE COMPONENT DATA'
!	write(6,*) ' number of ice points ',nlandice
!      endif


!-----------------------------------------------------------------------
!    read in the gridmap file data
!-----------------------------------------------------------------------
!        lon_ice = c0
!        lat_ice = c0
!        open(unit=3,file=trim(datapath)//'data/gridmap_landice',
!     *       form='unformatted'
!     *,      access='DIRECT',recl=lrecl)
!        do ksdm = 1,nsdm_sfc
!          do j = 1,jjm_sfc
!            do i = 1,iim_sfc
!	      if(grid_mask_sfc(i,j,ksdm)) then
!              read(3,rec=1+grid_indx_sfc(i,j,ksdm))lon_tem,
!     *             lat_tem, area_tem
!     *,            wght_tem, link_hr_ice(i,j,ksdm), hr_ice(i,j,ksdm)
!	      if(hr_ice(i,j,ksdm)) then
!	 	region_flag = .true.
!                lon_ice(i,j,ksdm) = lon_tem
!                lat_ice(i,j,ksdm) = lat_tem
!                  area_sfc(i,j,ksdm) = area_tem
!	      else
!	        link_hr_ice(i,j,ksdm) = -10
!	      endif
!	      endif
!            enddo
!          enddo
!        enddo

!        close(3)

!        if(maxval(lon_ice).gt.20.and.maxval(lat_ice).gt.20) then
!          write(6,*) ' wrong units'
!          stop
!        endif

!-----------------------------------------------------------------------
!     mask out wrapped ice points
!-----------------------------------------------------------------------
!      hr_ice(:,:,:) = hr_ice(:,:,:).and.grid_mask_sfc(:,:,:)

!-----------------------------------------------------------------------
!    Read in the initial condition file
!      These fields are defined only at ice points, and will be 
!      scattered to the global grid
!-----------------------------------------------------------------------
!        open(unit=4,file=trim(datapath)//'data/ic_landice',form='unformatted'
!     *,      access='DIRECT', recl=16,status='old',action='read'
!     *,      iostat=ioerr)
!        if (ioerr /= 0) then
!          write(*,*) '*** init_landice: ic_landice file error ',ioerr
!          stop
!        endif
!        read(4,rec=1)nsecond_tem,nice_tem,max_ig_tem,lrecl
!        close(4)

!        open(unit=4,file=trim(datapath)//'data/ic_landice',form='unformatted'
!     *,      access='DIRECT', recl=lrecl)
!-----------------------------------------------------------------------
! read in data at ice points
!-----------------------------------------------------------------------
!        allocate(xrec4(2))
!        allocate(xrec8(2))
!        read(4,rec=2)xrec4
!        read(4,rec=2)xrec8
!        if(xrec4(1).gt.1.e02 .and. xrec4(1).lt.3.e02) then
!          if(my_task.eq.0)print *,'reading real*4 ic_landice'
!          do ksdm = 1,nsdm_sfc
!            do j = 1,jjm_sfc
!              do i = 1,iim_sfc
!                if(hr_ice(i,j,ksdm)) then
!                   read(4,rec=1+link_hr_ice( i,j,ksdm) ) xrec4
!                   tgi(i,j,ksdm)   = xrec4(1)
!                   tkei(i,j,ksdm) = xrec4(2)
!	        endif
!              enddo
!            enddo
!          enddo
!        elseif(xrec8(1).gt.1.d02 .and. xrec8(1).lt.3.d02) then
!          if(my_task.eq.0)print *,'reading real*8 ic_landice'
!          do ksdm = 1,nsdm_sfc
!            do j = 1,jjm_sfc
!              do i = 1,iim_sfc
!                if(hr_ice(i,j,ksdm)) then
!                   read(4,rec=1+link_hr_ice( i,j,ksdm) ) xrec8
!                   tgi(i,j,ksdm)   = xrec8(1)
!                   tkei(i,j,ksdm) = xrec8(2)
!	        endif
!              enddo
!            enddo
!          enddo
!        else
!          if(my_task.eq.0)print *,'cant identify type of ic_landice - stopping'
!          stop
!        endif
!        deallocate(xrec8)
!        deallocate(xrec4)

!-----------------------------------------------------------------------
! close file containing initial conditions
!-----------------------------------------------------------------------
!      close(4)

!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!   read in the boundary conditions
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!    Read in the boundary condition file
!      These fields are defined only at ice points, and will be 
!      scattered to the global grid
!-----------------------------------------------------------------------
!        open(unit=3,file=trim(datapath)//'data/bc_landice',form='unformatted'
!     *,      access='DIRECT', recl=12,status='old',action='read'
!     *,      iostat=ioerr)
!        if (ioerr /= 0) then
!          write(*,*) '*** init_landice: bc_landice file error ',ioerr
!          stop
!        endif
!        read(3,rec=1)nice_tem,max_ig_tem,lrecl
!        close(3)

!        open(unit=3,file=trim(datapath)//'data/bc_landice',form='unformatted'
!     *,      access='DIRECT', recl=lrecl)
!-----------------------------------------------------------------------
! read in data at ice points
!-----------------------------------------------------------------------
!        do ksdm = 1,nsdm_sfc
!          do j = 1,jjm_sfc
!            do i = 1,iim_sfc
!              if(hr_ice(i,j,ksdm)) then
!                 read(3,rec=1+link_hr_ice( i,j,ksdm) )zs_tem
!                 zs_ice(i,j,ksdm) = zs_tem
!              endif
!            enddo
!          enddo
!        enddo

!-----------------------------------------------------------------------
! close file containing boundary conditions
!-----------------------------------------------------------------------
!      close(3)
!      if(my_task_sfc.eq.0) write(6,*) ' landice bc read in'

!-----------------------------------------------------------------------
! compute lat,lon, and surface height (gpm)
!-----------------------------------------------------------------------
!        where(hr_ice)
!              !zs_sfc(:,:,:) = zs_ice(:,:,:)
!              lon_sfc(:,:,:) = lon_ice(:,:,:)
!              lat_sfc(:,:,:) = lat_ice(:,:,:)
!              frac_sfc(:,:,:) = 1.0
!        endwhere

  end subroutine init_landice
!======================================================================
! END OF INIT_LANDICE
!======================================================================


!======================================================================
! BEGINNING OF INIT_SIB
!======================================================================
  subroutine init_sib

!-- This subroutine initializes the fields for the SIB surface flux
!-- parameterization

    use sib_parameters
    implicit none

!-----------------------------------------------------------------------
! local variables
!-----------------------------------------------------------------------
    logical (kind=log_kind) :: exans

    nsib = 1
    
!-----------------------------------------------------------------------
! Read in the initial condition file
!   These fields are defined only at sib points, and will be 
!   scattered to the global grid
!-----------------------------------------------------------------------
    inquire(file=trim(datapath)//'data/ic_sib',exist=exans)
    if (exans) then 
      open(unit=4,file=trim(datapath)//'data/ic_sib',form='formatted', &
           status='old',action='read',iostat=ioerr)
      read(4,*) tcan
      read(4,*) tg
      read(4,*) deept
      read(4,*) capac
      read(4,*) snow
      read(4,*) www
      read(4,*) gs
      read(4,*) tkes
      close(4)
    else
      write(*,*) 'ic_sib file not found'
      stop
    endif

!-----------------------------------------------------------------------
!   read in the boundary conditions
!      These fields are defined only at sib points, and will be 
!      scattered to the global grid
!-----------------------------------------------------------------------
    inquire(file=trim(datapath)//'data/bc_sib',exist=exans)
    if (exans) then 
      open(unit=3,file=trim(datapath)//'data/bc_sib',form='formatted', &
           status='old',action='read',iostat=ioerr)
      read(3,*) z2
      read(3,*) z1
      read(3,*) vcover
      read(3,*) chil
      read(3,*) sodep
      read(3,*) rootd
      read(3,*) phc
      read(3,*) tran(1,1,1,1,1)
      read(3,*) tran(1,1,2,1,1)
      read(3,*) tran(1,1,1,2,1)
      read(3,*) tran(1,1,2,2,1)
      read(3,*) ref(1,1,1,1,1)
      read(3,*) ref(1,1,2,1,1)
      read(3,*) ref(1,1,1,2,1)
      read(3,*) ref(1,1,2,2,1)
      read(3,*) vmax0
      read(3,*) effcon
      read(3,*) gradm
      read(3,*) binter 
      read(3,*) atheta
      read(3,*) btheta
      read(3,*) trda
      read(3,*) trdm
      read(3,*) trop
      read(3,*) respcp
      read(3,*) slti
      read(3,*) shti
      read(3,*) hltii
      read(3,*) hhti
      read(3,*) soref(1,1,1,1),soref(1,1,2,1)
      read(3,*) bee
      read(3,*) phsat
      read(3,*) satco
      read(3,*) poros
      read(3,*) slope
      read(3,*) wopt
      read(3,*) wsat
      read(3,*) zm
      read(3,*) zs_sib
      !-- time varying parameters
      read(3,*) aparc_m
      read(3,*) zlt_m
      read(3,*) green_m
      read(3,*) z0d_m
      read(3,*) dd_m
      read(3,*) rbc_m
      read(3,*) rdc_m
      read(3,*) gmudmu_m
      close(3)      
    else
      write(*,*) 'bc_sib file not found'
      stop
    endif

!-----------------------------------------------------------------------
! compute lat,lon, and surface height (gpm)
!-----------------------------------------------------------------------
    !lon_sfc(:,:,:) = lon_scm * 180. / pi
    !lat_sfc(:,:,:) = lat_scm * 180. / pi
    lon_sfc(:,:,:) = lon_scm
    lat_sfc(:,:,:) = lat_scm
    hr_sib = .true.
    !write(*,*) 'init_sib: lat_sfc = ',lat_sfc
    !write(*,*) 'init_sib: lon_sfc = ',lon_sfc
    !pause
    
  end subroutine init_sib
!======================================================================
! END OF INIT_SIB
!======================================================================

!======================================================================
! BEGINNING OF INIT_COUPLER
!======================================================================
  subroutine init_coupler

!-- June, 2007:  Read in PBL turbulent kinetic energy (TKE) and surface
!--  geopotential the way the coupled model does.

    integer (kind=int_kind) :: ioerr
    character(char_len) :: filename
    real (kind=real_kind) :: xrec
    real (kind=dbl_kind) :: xrec2
    logical (kind=log_kind) :: l_exist
  
!-- 1) sfc geopotential
!-----------------------------------------------------------------------
! open correct file based on grid size
!-----------------------------------------------------------------------
    filename = 'data/bc_atm'

    inquire(file=trim(filename),exist=l_exist)
    if (l_exist) then
      open(unit=3,file=trim(filename),form='formatted',  &
           status='old',iostat=ioerr)
      read(3,*) xrec
      zs_sfc(:,:,:) = xrec
      close(3)
    else
      write(*,*) 'bc_atm file not found'
      stop
    endif

!-- 2) PBL TKE
!-----------------------------------------------------------------------
! read in PBL turbulent kinetic energy
!-----------------------------------------------------------------------
!-- we've never provide a tke restart file for the scm
    tke_sfc = tkemin
      
  end subroutine init_coupler
!======================================================================
! END OF INIT_COUPLER
!======================================================================

end module sfc_init