!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

	module step_scm

!-----------------------------------------------------------------------
! PURPOSE: call a series of subroutine which each time step a single
!       prognostic variable.  Prognostic variables are held in prognostics.F
!       Includes material from pre_step.F to be used in scm
!-----------------------------------------------------------------------

	use kinds
        use physical_parameters
        use model_parameters
        use model_control
        use prognostics
        use timer
	use tracer_names
	use hstatc
        use scm_module

	implicit none
	save

        integer (kind = int_kind) :: i,j,k,nt

	real (kind = dbl_kind), dimension(iim,jjm) ::
     *			work1_2d,
     *			work2_2d,
     *                  work3_2d,
     *                  work4_2d,
     *                  work5_2d,
     *                  work6_2d,
     *                  work7_2d

	real (kind = dbl_kind), dimension(nwalls,iim,jjm) ::
     *			work_w

        real (kind = dbl_kind), dimension(iim,jjm,nlm) ::
     *			work1_3d

	real (kind = dbl_kind), dimension(iim,jjm,nlm+2,nsdm) ::
     *			work_wrap=c0
	
	logical ::  init_b2 = .true.

	real (kind = dbl_kind), dimension(nwalls,iim,jjm,nlm) ::
     *			work1_w,
     *			work2_w

	private ::
     *	  i, j, k, nt,
     *                  work1_2d,
     *                  work2_2d,
     *                  work3_2d,
     *                  work4_2d,
     *                  work5_2d,
     *                  work6_2d,
     *                  work7_2d,
     *                  work_w,
     *                  work1_3d,
     *                  work1_w,
     *			work2_w,
     *			work_wrap,
     *			init_b2

!----------------------------------------------------------------------------
! VARIABLE DESCRIPTION
!----------------------------------------------------------------------------
!       n{4,3,2}        : pointer of time level index {n+1,n,n-1} for prognostics
!       n{3,2,1}_f      : pointer of time level index {n, n-1, n-2} for tendencies
!       ntprog          : number of time level which prognostics must be saved
!       nttend          : number of time level which tendencies  must be saved
!	ntracer		: number of passive tracers to integrate in time
!       div             : divergence (im,jm,ntprog)
!	div_f		: time tendency of divergence (im,jm,nttend)
!	div_src		: rhs source to divergence field
!       eta             : absolute vorticity (im,jm,ntprog)
!       eta_f           : time tendency of eta (im,jm,nttend)
!	eta_src		: rhs source to vorticity field
!       tracer          : passive tracers advected by flow
!       tracer_f        : tendency of passive tracers
!       tracer_src      : rhs source to tracer field
!       psi             : steamfunction at step_count
!       chi             : velocity potential at step_count
!       w{3,2,1}        : weights for time integration
!       step_count      : counter index
!       hs              : surface height (m)
!	work		: local work arrays
!----------------------------------------------------------------------------

!----------------------------------------------------------------------------
! COMMENTS
!	all tendency terms included in the *_f arrays will be integrated
!	using Adams Bashforth 3rd Order integration. all tendency terms
!	included in the *_src arrays will be integrated Euler forward
!
! SCM NOTE: all integration is done euler forward in the SCM
!----------------------------------------------------------------------------

!**********************************************************************

	contains

!**********************************************************************


!======================================================================
! BEGINNING OF STEP_DT
!======================================================================
      subroutine step_dt_scm

	implicit none

        real (kind=dbl_kind), dimension(iim,jjm) :: div_mean

!------------------------------------------------------------------
! based on the time step, set the appropriate time stepping weights
!------------------------------------------------------------------
        call get_weights_scm

!--------------------------------------------------------------------------
! LOOP OVER EACH SUBDOMAIN
!--------------------------------------------------------------------------
	do ksdm = 1, nsdm

!-----------------------------------------------------------------------
!  compute mass fluxes, vertical velocity
!-----------------------------------------------------------------------

          div_mean(:,:) = c0

          do k = 1,nlm
              if ((scm_mode == 'rlx') .or. (scm_mode == 'RLX') .or. 
     *            (scm_mode == 'hrz') .or. (scm_mode == 'HRZ')) then
                div_mass_flux(:,:,k,n3_f,ksdm) = pthick(:,:,k,n3,ksdm)
     *                                         * div_terp(:,:,k,ksdm)
             else
                div_mass_flux(:,:,k,n3_f,ksdm) = c0
             endif
             div_mean(:,:) = div_mean(:,:) +
     *                   div_mass_flux(:,:,k,n3_f,ksdm)*dsig(k)
          enddo

!-----------------------------------------------------------------------
! adjust divergence profile so that mass is conserved
!-----------------------------------------------------------------------
          div_mean(:,:) = (div_mean(:,:) + ptend_scm(:,:,ksdm) )
     *               / (ps(:,:,n3,ksdm) - p_top)
          do k = 1,nlm
             div_mass_flux(:,:,k,n3_f,ksdm) = div_mass_flux(:,:,k,n3_f,ksdm)
     *              - div_mean(:,:) * pthick(:,:,k,n3,ksdm)
          enddo

!-----------------------------------------------------------------------
! compute vertical velocity
!-----------------------------------------------------------------------
          call get_dsigdt(
     *           entrainment(:,:,ksdm),
     *           cumulusmf(:,:,ksdm),
     *           pleak(:,:,ksdm),
     *           div_mass_flux(:,:,:,n3_f,ksdm),
     *           dsigdt(:,:,:,ksdm) )

!--------------------------------------------------------------------------
! UPDATE PROGNOSTICS 
!--------------------------------------------------------------------------

!--------------------------------------------------------------------------
! UPDATE SURFACE PRESSURE
!--------------------------------------------------------------------------
          ps(:,:,n4,ksdm) = ps(:,:,n3,ksdm) + dt * ptend_scm(:,:,ksdm)

!--------------------------------------------------------------------------
! UPDATE PBL TOP PRESSURE
!--------------------------------------------------------------------------
#ifdef generalized_sigma
          call step_pbl_top_pressure_scm
#endif

!-----------------------------------------------------------------------
! get pressure thickness at n4 (Arakawa's Pi)
!-----------------------------------------------------------------------
        call get_pthick(
     *          ps         (:,:,n4,ksdm),
     *          pb         (:,:,n4,ksdm),
     *          pthick     (:,:,:,n4,ksdm),
     *          invpthick  (:,:,:,n4,ksdm) )

!--------------------------------------------------------------------------
! UPDATE THETA
!--------------------------------------------------------------------------
          call step_theta_scm

!-----------------------------------------------------------------
! UPDATE TRACERS
!-----------------------------------------------------------------
          call step_tracer_scm
 

	enddo	! do ksdm = 1,nsdm
!--------------------------------------------------------------------------
! END SECOND LOOP OVER KSDM
!--------------------------------------------------------------------------


	end subroutine step_dt_scm
!===========================================================================
! END OF STEP_DT
!===========================================================================



!=======================================================================
! START STEP_PBL_TOP_PRESSURE
!=======================================================================
        subroutine step_pbl_top_pressure_scm

!-----------------------------------------------------------------------------
! PURPOSE : find tendencies of the pbl top pressure and integrate it in time.
!-----------------------------------------------------------------------------

#ifdef generalized_sigma

#ifdef pbl_depth_limiter
        real (kind=dbl_kind), parameter ::
     *                  pbl_min      = 1.0e3,
     *                  pbl_max      = 1.6e4
#else
        real (kind=dbl_kind), parameter ::
     *                  pbl_min      = 8.0e3,
     *                  pbl_max      = 8.0e3
#endif

        integer (kind=int_kind), dimension(iim,jjm) ::
     *                   min_mask=0,
     *                   max_mask=0

        real (kind=dbl_kind), dimension(iim,jjm) ::
     *                  pbl_depth = c0

!----------------------------------------------------------------------
! sum mass flux divergence from top down to pbl top
!----------------------------------------------------------------------
	pb_f(:,:,n3_f,ksdm) = c0
        do k=1,nstrat+ntrop
           pb_f(:,:,n3_f,ksdm) =    pb_f(:,:,n3_f,ksdm) 
     *                         - div_mass_flux(:,:,k,n3_f,ksdm)*dsig(k)
        enddo

!----------------------------------------------------------------------
! add in the part of the vertical velocity @ the pbl which is to
! be integrated AB3
!----------------------------------------------------------------------
        pb_dsigdt_f(:,:,n3_f,ksdm) = - dsigdt(:,:,nstrat+ntrop+1,ksdm)

!----------------------------------------------------------------------
! increment surface pressure via AB3 or Leap Frog
! this is a provisional value and will be modified if the pbl
! depth is too thin to too thick
!----------------------------------------------------------------------
        pb(:,:,n4,ksdm) = pb(:,:,n3,ksdm) 
     *    + dt*(w3*(pb_f(:,:,n3_f,ksdm) + pb_dsigdt_f(:,:,n3_f,ksdm)) +
     *          w2*(pb_f(:,:,n2_f,ksdm) + pb_dsigdt_f(:,:,n2_f,ksdm)) +
     *          w1*(pb_f(:,:,n1_f,ksdm) + pb_dsigdt_f(:,:,n1_f,ksdm))  )

!---------------------------------------------------------------------
! compute the depth of the pbl
!---------------------------------------------------------------------
        pbl_depth = ps(:,:,n4,ksdm) - pb(:,:,n4,ksdm)

!---------------------------------------------------------------------
! find mask
!---------------------------------------------------------------------
        min_mask = 0
        max_mask = 0
        where(pbl_depth.lt.pbl_min) min_mask = 1
        where(pbl_depth.gt.pbl_max) max_mask = 1
	
!---------------------------------------------------------------------
! find leak velocity where pbl is less than minimum
!---------------------------------------------------------------------
        where(min_mask(:,:).eq.1)
          pleak(:,:,ksdm) =
     *     + ((ps(:,:,n4,ksdm) - pbl_min) - pb(:,:,n3,ksdm))*invdt
     *     - (w3*(pb_f(:,:,n3_f,ksdm)                             ) +
     *        w2*(pb_f(:,:,n2_f,ksdm) + pb_dsigdt_f(:,:,n2_f,ksdm)) +
     *        w1*(pb_f(:,:,n1_f,ksdm) + pb_dsigdt_f(:,:,n1_f,ksdm))  )
     *     +  w3*grav*(entrainment(:,:,ksdm) - cumulusmf(:,:,ksdm))
	  pleak(:,:,ksdm) = -pleak(:,:,ksdm)/(grav*w3)
        endwhere

!---------------------------------------------------------------------
! find leak velocity where pbl is greater than maximum
!---------------------------------------------------------------------
        where(max_mask(:,:).eq.1)
          pleak(:,:,ksdm) =
     *     + ((ps(:,:,n4,ksdm) - pbl_max) - pb(:,:,n3,ksdm))*invdt
     *     - (w3*(pb_f(:,:,n3_f,ksdm)                             ) +
     *        w2*(pb_f(:,:,n2_f,ksdm) + pb_dsigdt_f(:,:,n2_f,ksdm)) +
     *        w1*(pb_f(:,:,n1_f,ksdm) + pb_dsigdt_f(:,:,n1_f,ksdm))  )
     *     +  w3*grav*(entrainment(:,:,ksdm) - cumulusmf(:,:,ksdm))
          pleak(:,:,ksdm) = -pleak(:,:,ksdm)/(grav*w3)
        endwhere

!----------------------------------------------------------------------
! compute new vertical velocity based on new leak
!----------------------------------------------------------------------
	call get_dsigdt(
     *           entrainment(:,:,ksdm),
     *           cumulusmf(:,:,ksdm),
     *           pleak(:,:,ksdm),
     *           div_mass_flux(:,:,:,n3_f,ksdm),
     *           dsigdt(:,:,:,ksdm) )
	
!----------------------------------------------------------------------
! compute new pb_dsigdt_f based on new leak
!----------------------------------------------------------------------
        pb_dsigdt_f(:,:,n3_f,ksdm) = - dsigdt(:,:,nstrat+ntrop+1,ksdm)

!----------------------------------------------------------------------
! compute new pbl top based on new leak
!----------------------------------------------------------------------
        pb(:,:,n4,ksdm) = pb(:,:,n3,ksdm)
     *       + dt*(w3*(pb_f(:,:,n3_f,ksdm) + pb_dsigdt_f(:,:,n3_f,ksdm)) +
     *             w2*(pb_f(:,:,n2_f,ksdm) + pb_dsigdt_f(:,:,n2_f,ksdm)) +
     *             w1*(pb_f(:,:,n1_f,ksdm) + pb_dsigdt_f(:,:,n1_f,ksdm))  )
#endif

        end subroutine step_pbl_top_pressure_scm
!=======================================================================
! END STEP_PBL_TOP_PRESSURE
!=======================================================================


!===========================================================================
!  START OF STEP_THETA
!===========================================================================
        subroutine step_theta_scm

!---------------------------------------------------------------------------
! PURPOSE
!  compute the time tendency of potential temperature and integrate
!  potential temperature in time
!---------------------------------------------------------------------------

        implicit none

!---------------------------------------------------------------------------
! initialize
!---------------------------------------------------------------------------

!---------------------------------------------------------------------------
! compute tendency due to vertical motion
!---------------------------------------------------------------------------
        call vert_flux_divergence(
     *          th          (:,:,:,n3,ksdm),
     *          thl2        (:,:,:,ksdm),
     *          dsigdt      (:,:,:,ksdm),
     *          entrainment (:,:,ksdm),
     *          cumulusmf   (:,:,ksdm),
     *          pleak       (:,:,ksdm),
     *          work1_3d    (:,:,:)  )

        th_f(:,:,:,n3_f,ksdm) =  - work1_3d(:,:,:)

!---------------------------------------------------------------------------
! begin loop over vertical level k
!---------------------------------------------------------------------------
	do k=1,nlm

!          if ((scm_mode == 'rev') .or. (scm_mode == 'REV')) then
          if ((scm_mode == 'rev') .or. (scm_mode == 'REV') .or.
     *        (scm_mode == 'rvx') .or. (scm_mode == 'rvx')) then
!---------------------------------------------------------------------------
! compute theta at new time level
!---------------------------------------------------------------------------

! prescribed forcing and physics added here
             th(:,:,k,n4,ksdm) = th(:,:,k,n3,ksdm)
     *                          + dt*th_src(:,:,k,ksdm)/pthick(:,:,k,n3,ksdm)

!-- fix to revealed forcing:  don't do the new two th assigments
! virtual vertical advection added here
             th(:,:,k,n4,ksdm) = th(:,:,k,n4,ksdm)*pthick(:,:,k,n3,ksdm) 
     *                          + dt*(w3*th_f(:,:,k,n3_f,ksdm) +
     *                                w2*th_f(:,:,k,n2_f,ksdm) +
     *                                w1*th_f(:,:,k,n1_f,ksdm)   )

!---------------------------------------------------------------------------
! divide by mass to get real theta
!---------------------------------------------------------------------------
	     th(:,:,k,n4,ksdm) = th(:,:,k,n4,ksdm) * invpthick(:,:,k,n4,ksdm)
          else
!             if (.not.scm_vtadvt) then
             if ((scm_mode == 'rlx') .or. (scm_mode == 'RLX') .or. 
     *           (scm_mode == 'hrz') .or. (scm_mode == 'HRZ')) then
	       th_f(:,:,k,n3_f,ksdm) = th_f(:,:,k,n3_f,ksdm) - 
     *                th(:,:,k,n3,ksdm) * div_mass_flux(:,:,k,n3_f,ksdm)
             endif

!---------------------------------------------------------------------------
! compute theta at new time level
!---------------------------------------------------------------------------
             th(:,:,k,n4,ksdm) = th(:,:,k,n3,ksdm)*pthick(:,:,k,n3,ksdm) 
     *                          + dt*(w3*th_f(:,:,k,n3_f,ksdm) +
     *                                w2*th_f(:,:,k,n2_f,ksdm) +
     *                                w1*th_f(:,:,k,n1_f,ksdm)   )
     *                          + dt*th_src(:,:,k,ksdm)

!---------------------------------------------------------------------------
! divide by mass to get real theta
!---------------------------------------------------------------------------
	     th(:,:,k,n4,ksdm) = th(:,:,k,n4,ksdm) * invpthick(:,:,k,n4,ksdm)
          endif

	enddo   ! do k = 1,nlm

	end subroutine step_theta_scm
!===========================================================================
!  END OF STEP_THETA
!===========================================================================


!===========================================================================
!  START OF STEP_TRACER
!===========================================================================
        subroutine step_tracer_scm

!---------------------------------------------------------------------------
! PURPOSE
!  compute the time tendency of tracers and integrate them in time
!---------------------------------------------------------------------------

        implicit none

!---------------------------------------------------------------------------
! loop over tracers
!---------------------------------------------------------------------------
	do nt = 1,ntracer

!---------------------------------------------------------------------------
! initialize
!---------------------------------------------------------------------------

!---------------------------------------------------------------------------
! compute tendency due to vertical motion
!---------------------------------------------------------------------------
        call vert_flux_divergence(
     *          tracer      (:,:,:,nt,n3,ksdm),
     *          tracerl2    (:,:,:,nt,ksdm),
     *          dsigdt      (:,:,:,ksdm),
     *          entrainment (:,:,ksdm),
     *          cumulusmf   (:,:,ksdm),
     *          pleak       (:,:,ksdm),
     *          work1_3d    (:,:,:)     )

        tracer_f(:,:,:,nt,n3_f,ksdm) =  - work1_3d(:,:,:)

!---------------------------------------------------------------------------
! begin loop over vertical level k
!---------------------------------------------------------------------------
        do k=1,nlm

!          if ((scm_mode == 'rev') .or. (scm_mode == 'REV')) then
          if ((scm_mode == 'rev') .or. (scm_mode == 'REV') .or.
     *        (scm_mode == 'rvx') .or. (scm_mode == 'rvx')) then
!---------------------------------------------------------------------------
! compute theta at new time level
!---------------------------------------------------------------------------

! prescribed forcing and physics added here
             tracer(:,:,k,nt,n4,ksdm) = tracer(:,:,k,nt,n3,ksdm)
     *              + dt*tracer_src(:,:,k,nt,ksdm)/pthick(:,:,k,n3,ksdm)

! virtual vertical advection added here
             tracer(:,:,k,nt,n4,ksdm) = 
     *                       tracer(:,:,k,nt,n4,ksdm)*pthick(:,:,k,n3,ksdm) 
     *                     + dt*(w3*tracer_f(:,:,k,nt,n3_f,ksdm) +
     *                           w2*tracer_f(:,:,k,nt,n2_f,ksdm) +
     *                           w1*tracer_f(:,:,k,nt,n1_f,ksdm)   )

!---------------------------------------------------------------------------
! divide by mass to get real theta
!---------------------------------------------------------------------------
             tracer(:,:,k,nt,n4,ksdm) =    tracer(:,:,k,nt,n4,ksdm) 
     *                                 * invpthick(:,:,k,n4,ksdm)
          else
!             if (.not.scm_vtadvt) then
             if ((scm_mode == 'rlx') .or. (scm_mode == 'RLX') .or. 
     *           (scm_mode == 'hrz') .or. (scm_mode == 'HRZ')) then
	        tracer_f(:,:,k,nt,n3_f,ksdm) = tracer_f(:,:,k,nt,n3_f,ksdm) - 
     *                tracer(:,:,k,nt,n3,ksdm) * div_mass_flux(:,:,k,n3_f,ksdm)
            if (nt <= nwater) qadvt_terp(:,:,k,ksdm) = qadvt_terp(:,:,k,ksdm) + 
     *                     (w3*tracer_f(:,:,k,nt,n3_f,ksdm) +
     *                      w2*tracer_f(:,:,k,nt,n2_f,ksdm) +
     *                      w1*tracer_f(:,:,k,nt,n1_f,ksdm))
     *                       *invpthick(:,:,k,n4,ksdm)
             endif

!---------------------------------------------------------------------------
! compute tracer at new time level
!---------------------------------------------------------------------------
             tracer(:,:,k,nt,n4,ksdm) = 
     *                       tracer(:,:,k,nt,n3,ksdm)*pthick(:,:,k,n3,ksdm) 
     *                     + dt*(w3*tracer_f(:,:,k,nt,n3_f,ksdm) +
     *                           w2*tracer_f(:,:,k,nt,n2_f,ksdm) +
     *                           w1*tracer_f(:,:,k,nt,n1_f,ksdm)   )
     *                     + dt*tracer_src(:,:,k,nt,ksdm)


!---------------------------------------------------------------------------
! divide by mass to get real tracer values (kg/kg)
!---------------------------------------------------------------------------
             tracer(:,:,k,nt,n4,ksdm) =    tracer(:,:,k,nt,n4,ksdm) 
     *                                 * invpthick(:,:,k,n4,ksdm)
          endif

	enddo   ! do k = 1,nlm
	enddo	! do nt=1,ntracer

        end subroutine step_tracer_scm
!===========================================================================
!  END OF STEP_TRACER
!===========================================================================

	end module step_scm

!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
