!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module time_manager_module

!-----------------------------------------------------------------
!       PURPOSE : keeps track of current time and logicals based
!	   on time
!-----------------------------------------------------------------

        use kinds
        use physical_parameters
	use model_parameters
	use domain_decomposition

	implicit none
        save

        logical (kind=log_kind) ::
     *                  l_restart,
     *                  l_halt,
     *                  l_output,
     *                  l_diagnostics,
     *                  l_timer,
     *			l_renorm,
     *                  l_physics,
     *                  l_qp,
     *                  l_pbp,
     *                  l_bc_update,
     *                  init_step = .true.,
     *                  amip_sampling = .true.

	integer (kind=int_kind) ::
     *                  n_restart,
     *                  n_output,
     *                  n_diagnostics,
     *                  n_timer,
     *                  n_renorm,
     *                  n_qp,
     *                  n_pbp,
     *                  n_physics

        real (kind=dbl_kind) ::
     *                  tau,
     *                  tofday,
     *                  tau_end,
     *                  tau_end_day,
     *                  tau_end_hour

        real (kind=dbl_kind) ::
     *                  restart_freq,
     *                  restart_count

        integer (kind=int_kind), save ::
     *                  nsecond_dyn,
     *			ndayofyear, 
     *                  step_count,
     *                  nmonth,
     *                  nyear,
     *                  nyear0 = 0,  ! reference year for nsecond_dyn=0
     *                  nhf_freq = 12! frequency snapshots are saved to hf files
                                     ! (multiples of 6 hours)

        real (kind=dbl_kind), dimension(:), allocatable, save::
     *                  pcos,
     *                  psin
     
	integer (kind=int_kind), dimension(0:12), save ::
     *   ndayend=(/0,31,59,90,120,151,181,212,243,273,304,334,365/)
      character*3, dimension(12) :: cmonth=(/'JAN','FEB','MAR','APR',
     *       'MAY','JUN','JUL','AUG','SEP','OCT','NOV','DEC'/)


!****************************************************************

        contains

!****************************************************************


!=================================================================
! BEGINNING OF INIT_TIME_MANAGER
!=================================================================
	subroutine init_time_manager

        l_restart =.false.
        l_halt  = .false.
        l_output = .false.
        l_diagnostics =.false.
        l_timer = .false.
	l_renorm = .false.
	l_physics = .false.
	l_bc_update = .false.
	l_qp = .false.
	l_pbp = .false.
        nsecond_dyn = 0
        nmonth = 1
        nyear = 0
        step_count = 0
        tau = c0
        tau_end = c0
        tau_end_day = c0
        tau_end_hour = c0
        restart_freq = 0
        restart_count = 0

	end subroutine init_time_manager
!======================================================================
! END OF INIT_TIME_MANAGER
!======================================================================


!======================================================================
! BEGINNING OF TIME_MANAGER
!======================================================================
        subroutine time_manager

        integer (kind=int_kind) :: itofday1, itofday2, month_of_year,
     *                             nmonthofyear, ndayofmonth
        logical (kind=log_kind) :: newmonth
        real (kind=dbl_kind) :: rtofday

!-------------------------------------------------------------------------
! reset logicals
!-------------------------------------------------------------------------
        l_halt = .false.
        l_restart = .false.
        l_output = .false.
        l_diagnostics = .false.
        l_timer = .false.
	l_renorm = .false.
	l_physics = .false.
	l_bc_update = .false.
	l_qp = .false.
	l_pbp = .false.
        newmonth = .false.

!-------------------------------------------------------------------------
! increment the step counter (integer) and time (integer  seconds)
!-------------------------------------------------------------------------
        step_count = step_count + 1
        nsecond_dyn = nsecond_dyn + dt

!-------------------------------------------------------------------------
! find out if this is a new month
!-------------------------------------------------------------------------
	if(.not.init_step) then
	   month_of_year = mod(nmonth-1,12) + 1
           if(nsecond_dyn.ge.86400*(365*nyear+ndayend(month_of_year)))then
              write(6,*)'this is a new month!'
	      nmonth = nmonth + 1
              newmonth = .true.
	      nyear = nsecond_dyn / (86400*365)
           endif
 	endif

!-------------------------------------------------------------------------
! find time in hours
!-------------------------------------------------------------------------
        tau = nsecond_dyn/3600.0
        tofday = mod(tau-dt/3600.0_dbl_kind, 24.0_dbl_kind)
        rtofday = mod(tau, 24.0_dbl_kind)
        ndayofyear = mod(nsecond_dyn / 86400,365) + 1
        nmonthofyear = mod(nmonth-1,12)+1
        ndayofmonth = ndayofyear - ndayend(nmonthofyear-1)
!	if(my_task.eq.0) write(6,*) step_count,nsecond_dyn,tau,
!     *                     rtofday, ndayofyear, nmonthofyear, nyear+1
	if(.not.init_step.and.my_task.eq.0) 
     *	           write(6,'(i7,i12," Date: ",f6.3,"Z",i3,a4,i5)') 
     *             step_count,nsecond_dyn,
     *             rtofday, ndayofmonth, cmonth(nmonthofyear), nyear+nyear0

!-------------------------------------------------------------------------
! based on tau or step_count, set logicals
!-------------------------------------------------------------------------
        if(tau.gt.tau_end+dt/3600.0_dbl_kind) l_halt=.true.
	if(n_restart.gt.0) then
          if(mod(step_count,n_restart     ).eq.0) l_restart = .true.
        elseif(n_restart.lt.0) then
          if(newmonth.and.mod(nmonth,n_restart ).eq.0) l_restart = .true.
        endif
	if(n_output.gt.0) then
          if(mod(step_count,n_output     ).eq.0) l_output = .true.
        elseif(n_output.lt.0) then
          if(newmonth.and.mod(nmonth,n_output ).eq.0) l_output = .true.
        endif
        if(n_diagnostics.ne.0.and.mod(step_count,n_diagnostics ).eq.0)
     *                                          l_diagnostics = .true.
	if(n_timer.gt.0) then
          if(mod(step_count,n_timer       ).eq.0) l_timer = .true.
        elseif(n_timer.lt.0) then
          if(newmonth.and.mod(nmonth,n_timer ).eq.0) l_timer = .true.
        endif
	if(n_renorm.gt.0) then
	  if(mod(step_count,n_renorm      ).eq.0) l_renorm = .true.
        elseif(n_renorm.lt.0) then
          if(newmonth.and.mod(nmonth,n_renorm ).eq.0) l_renorm = .true.
        endif
	if(n_qp.gt.0) then
	  if(mod(step_count,n_qp          ).eq.0) l_qp = .true.
          if(n_qp.lt.21600/dt) amip_sampling = .false.
        elseif(n_qp.lt.0) then
          if(newmonth.and.mod(nmonth,n_qp ).eq.0) l_qp = .true.
        endif
	if(n_pbp.ne.0.and.mod(step_count,n_pbp ).eq.0) l_pbp = .true.
#ifdef link_to_gcm_physics
	if(mod(step_count-1,n_physics     ).eq.0) l_physics = .true.
!  update the boundary conditions on a new day
        itofday1 = mod(nsecond_dyn-2*int(dt), 86400)
        itofday2 = mod(nsecond_dyn-int(dt), 86400)
	if(itofday2.lt.itofday1) l_bc_update = .true.
#endif

!  always do physics and boundary conditions on the first time step.
        if(init_step) then
          l_physics = .true.
          l_bc_update = .true.
          init_step = .false.
!  find nmonth initially
          nyear = nsecond_dyn/(86400*365)
	  nmonth = nyear*12
          month_of_year = 0
          do while (nsecond_dyn.ge.86400*(365*nyear+ndayend(month_of_year)))
            nmonth = nmonth + 1
            month_of_year = mod(nmonth-1,12)+1
          enddo
          tau = nsecond_dyn/3600.0
          tofday = mod(tau-dt/3600.0_dbl_kind, 24.0_dbl_kind)
          rtofday = mod(tau, 24.0_dbl_kind)
          ndayofyear = mod(nsecond_dyn / 86400,365) + 1
          nmonthofyear = mod(nmonth-1,12)+1
          ndayofmonth = ndayofyear - ndayend(nmonthofyear-1)
!	  if(my_task.eq.0) write(6,*) step_count,nsecond_dyn,tau,
!     *                     rtofday, ndayofyear, nmonthofyear, nyear+1
	  if(my_task.eq.0) write(6,'(i7,i12," Date: ",f6.3,"Z",i3,a4,i5)') 
     *               step_count,nsecond_dyn,
     *               rtofday, ndayofmonth, cmonth(nmonthofyear), nyear+nyear0
        endif

#ifdef mpi
        call MPI_BCAST(
     *       step_count,
     *       1,
     *       mpi_int_type,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       nsecond_dyn,
     *       1,
     *       mpi_int_type,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       tau,
     *       1,
     *       mpi_float_type,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       tofday,
     *       1,
     *       mpi_float_type,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       l_halt,
     *       1,
     *       MPI_LOGICAL,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       l_restart,
     *       1,
     *       MPI_LOGICAL,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       l_output,
     *       1,
     *       MPI_LOGICAL,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)
        call MPI_BCAST(
     *       l_diagnostics,
     *       1,
     *       MPI_LOGICAL,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       l_timer,
     *       1,
     *       MPI_LOGICAL,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       l_renorm,
     *       1,
     *       MPI_LOGICAL,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       l_qp,
     *       1,
     *       MPI_LOGICAL,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       l_pbp,
     *       1,
     *       MPI_LOGICAL,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

#ifdef link_to_gcm_physics
        call MPI_BCAST(
     *       l_physics,
     *       1,
     *       MPI_LOGICAL,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)

        call MPI_BCAST(
     *       l_bc_update,
     *       1,
     *       MPI_LOGICAL,
     *       0,
     *       MPI_COMM_ATMOS,
     *       ierr)
#endif

#endif

        end subroutine time_manager
!======================================================================
! END OF TIME_MANAGER
!======================================================================

	end module time_manager_module
!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
