!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module tracer_names

!-----------------------------------------------------------------------
! PURPOSE: name the tracers via an integer variable so each tracer
!    field can be extracted in an logical manner
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! NOTE : parameters set here may be obtained from another module in
!    the future
!-----------------------------------------------------------------------

	use kinds
	use domain_decomposition
	use model_parameters

        implicit none
        save

	logical (kind = log_kind), parameter ::
     *			doozone = .false.

	integer (kind = int_kind), parameter :: 
     *			nwater = 5,
     *			npassive = 2,
     *			nozone = 1,
     *			nmax = 100

	integer (kind = int_kind) ::
     *			iq,
     *			icw,
     *			irw,
     *			ici,
     *			iri,
     *			iozone

	integer (kind = int_kind), dimension(npassive) ::
     *			ipassive

	integer (kind = int_kind), dimension(nmax) :: fill_status

	character (char_len), dimension(nmax) :: tracer_name

!**********************************************************************

        contains

!**********************************************************************


!======================================================================
! BEGINNING OF INIT_TRACER_NAMES
!======================================================================
	subroutine init_tracer_names

	integer (kind = int_kind) :: nt, i

!-----------------------------------------------------------------
! initialize
!-----------------------------------------------------------------
	tracer_name(:) = trim(' NO NAME       : ')
	ipassive = -1

!-----------------------------------------------------------------
! water comes first
!-----------------------------------------------------------------
	iq  = 1
	icw = 2
	irw = 3
	ici = 4
	iri = 5
	
!-----------------------------------------------------------------
! then ozone if applicable
!-----------------------------------------------------------------
	nt = nwater
	if(doozone) then
	  nt = nt+1
	  iozone = nt
	else
	  iozone = -1
	endif

!-----------------------------------------------------------------
! now passive tracers
!-----------------------------------------------------------------
#ifdef passive_tracers
	do i = 1, npassive
	   nt = nt + 1
	   ipassive(i) = nt
	enddo
#else
	ipassive = -1
#endif

!-----------------------------------------------------------------
! put additional tracers here
!-----------------------------------------------------------------


!-----------------------------------------------------------------
! fill status holds information on how (or if) to do hole filling
!	fill_status = 0 , no hole filling, no negative value check
!	fill_status = 1,  fill holes to prevent negative values
!	fill_status = 2,  no hole filling, stop on a negative value
!-----------------------------------------------------------------
	fill_status = 1

!-----------------------------------------------------------------
! rudimentary error checking
! done on one process only
!-----------------------------------------------------------------
	if(my_task.eq.0) then

	if(nt.gt.nmax) then
	  write(6,*) ' adjust nmax in tracer_names.F'
	  write(6,*)  nt, nmax
	endif

	  write(6,*)
	  write(6,*) ' the number of tracer set in init_tracer_names : ',nt
	  write(6,*) ' the value of ntracer set in model_parameters.F ',ntracer
	  if(nt.ne.ntracer) stop
	  write(6,*)

	endif

!-----------------------------------------------------------------
! now define the names of the tracers
!-----------------------------------------------------------------
	tracer_name( iq) = trim(' WATER VAPOR   : ')
	tracer_name(icw) = trim(' CLOUD WATER   : ')
	tracer_name(irw) = trim(' RAIN WATER    : ')
	tracer_name(ici) = trim(' CLOUD ICE     : ')
	tracer_name(iri) = trim(' SNOW          : ')

	if(iozone.gt.0) then
	  tracer_name(iozone) = trim(' OZONE      : ')
	endif

	if(maxval(ipassive).gt.0) then
!	  name your tracers or read from file
!	  tracer_name(ipassive(1)) = trim(' TRACER 1 : ')
!         tracer_name(ipassive(2)) = trim(' TRACER 2 : ')
	endif

!-----------------------------------------------------------------
! dump names to the screen
!-----------------------------------------------------------------
	if(my_task.eq.0) then
	  write(6,*) ' A LISTING OF THE TRACERS to be INTEGRATED'
	  do i=1,nt
	    write(6,10) tracer_name(i),i
	  enddo
	endif
 10	format(a20,i5)
	write(6,*)

	end subroutine init_tracer_names
!======================================================================
! END OF INIT_TRACER_NAMES
!======================================================================

        end module tracer_names

!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
