!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

        module vertical_diffusion

!-----------------------------------------------------------------------
! PURPOSE: call a series of routines which set mixing coefficients
!    for purpose of computing the vertical diffusion in the PBL layers.
!-----------------------------------------------------------------------

        use kinds
        use prognostics
	use hstatc
        use model_control

        implicit none
	save

        real (kind=dbl_kind), dimension(nlm,iim,jjm,nsdm) :: 
     *  	         bmixh, cmixh, dmixh, emixh
        real (kind=dbl_kind), dimension(nlm,nwalls,iim,jjm,nsdm) :: 
     *  	         cmixm_w, dmixm_w, emixm_w

!**********************************************************************

        contains

!**********************************************************************

!===========================================================================
!  START OF DIFF_COEFF
!===========================================================================

	subroutine diff_coeff

	implicit none
!---------------------------------------------------------------------------
! compute the coefficients for the diffusion terms in the pbl, mdb 7-19-00
! this does the potential temperature diffusion
!---------------------------------------------------------------------------
 
!---------------------------------------------------------------------------
! VARIABLE DEFINITIONS
!---------------------------------------------------------------------------
!	plg	   =  pressure at layer centers (Pa)
!	alpha      =  specific volume (m3/kg)
!	km         =  mixing coefficients (m2/s)
!       invdsig    =  reciprocal of delta-sigma values
 
!---------------------------------------------------------------------------
! LOCAL VARIABLES
!---------------------------------------------------------------------------
	real (kind=dbl_kind), dimension(iim,jjm,nlmp1)  :: 
     *			dzm1, alphal2
        real (kind=dbl_kind), dimension(nlm)    :: 
     *			mcoeff
	real (kind=dbl_kind), dimension(nlmp1)  ::  km

        integer (kind = int_kind) :: i,j,k

        km(:) = c0
!        km(ntrop+nstrat+2:nlm) = 1000.
        km(ntrop+nstrat+2:nlm) = 100.
        
        alphal2(:,:,1) = c0 
        alphal2(:,:,nlmp1) = c0
        dzm1(:,:,1) = c0
        dzm1(:,:,nlmp1) = c0
 
	do ksdm = 1,nsdm
          do k = 2,nlm
            alphal2(:,:,k) = 0.5*(alpha(:,:,k-1,ksdm) 
     +                          + alpha(:,:,k,ksdm))
          enddo

          do k=nlm,2,-1
            dzm1(:,:,k) = (plg(:,:,k,ksdm) - plg(:,:,k-1,ksdm))*
     +                      alphal2(:,:,k)/grav
          enddo
 
          do k = 1,nlm
            mcoeff(k) = -grav*dt*invdsig(k)
          enddo
 
          cmixh(1,:,:,ksdm) = c0
          do k = 2,nlm
            do i = 1,iim
              do j = 1,jjm
                cmixh(k,i,j,ksdm) = mcoeff(k)*km(k)/
     +                        (alphal2(i,j,k)*dzm1(i,j,k))
              enddo
            enddo
          enddo

          emixh(nlm,:,:,ksdm) = c0
          do k = 1,nlm-1
            do i = 1,iim
              do j = 1,jjm
                emixh(k,i,j,ksdm) = mcoeff(k)*km(k+1)/
     +                       (alphal2(i,j,k+1)*dzm1(i,j,k+1))
              enddo    
            enddo
          enddo
          do k = 1,nlm
            do i = 1,iim
              do j = 1,jjm
                dmixh(k,i,j,ksdm) = pthick(i,j,k,n4,ksdm)-
     +            cmixh(k,i,j,ksdm)-emixh(k,i,j,ksdm)
!                bmixh(k,i,j,ksdm) = pthick(i,j,k,n3,ksdm)
!     +            *th(i,j,k,n3,ksdm)
              enddo
            enddo
          enddo

	enddo	! do ksdm = 1,nsdm

	end subroutine diff_coeff

!===========================================================================
!  END OF DIFF_COEFF
!===========================================================================

!===========================================================================
!  START OF DIFF_COEFF_MOM
!===========================================================================

	subroutine diff_coeff_mom

	implicit none
!---------------------------------------------------------------------------
! compute the coefficients for the diffusion terms in the pbl, mdb 7-19-00
! this does the diffusion of momentum
!---------------------------------------------------------------------------
 
!---------------------------------------------------------------------------
! VARIABLE DEFINITIONS
!---------------------------------------------------------------------------
!	plg	   =  pressure at layer centers (Pa)
!	alpha      =  specific volume (m3/kg)
!	km         =  mixing coefficients (m2/s)
!       invdsig    =  reciprocal of delta-sigma values
 
!---------------------------------------------------------------------------
! LOCAL VARIABLES
!---------------------------------------------------------------------------
        real (kind=dbl_kind), dimension(nlm,iim,jjm,nsdm) :: 
     *  	         cmixm, dmixm, emixm
	real (kind=dbl_kind), dimension(iim,jjm,nlmp1,nsdm)  :: 
     *			alphal2, dzm1
	real (kind=dbl_kind), dimension(iim,jjm,nlm,nsdm)  :: 
     *			dzm2
        real (kind=dbl_kind), dimension(nwalls,iim,jjm)    :: 
     *			ctmp_w, dtmp_w, etmp_w
	real (kind=dbl_kind), dimension(nlmp1)  ::  km

        integer (kind = int_kind) :: i,j,k,iwall

        km(:) = c0
!        km(ntrop+nstrat+2:nlm) = 1000.
        km(ntrop+nstrat+2:nlm) = 100.
        
        alphal2(:,:,1,:) = c0 
        alphal2(:,:,nlmp1,:) = c0
        dzm1(:,:,1,:) = c0
        dzm1(:,:,nlmp1,:) = c0
 
	do ksdm = 1,nsdm
          do k = 2,nlm
            alphal2(:,:,k,ksdm) = 0.5*(alpha(:,:,k-1,ksdm) 
     +                          + alpha(:,:,k,ksdm))
          enddo

          do k=nlm,2,-1
            dzm1(:,:,k,ksdm) = (plg(:,:,k,ksdm) - plg(:,:,k-1,ksdm))*
     +                      alphal2(:,:,k,ksdm)/grav
          enddo
 
          do k=nlm,1,-1
            dzm2(:,:,k,ksdm) = (plgl2(:,:,k+1,ksdm) - plgl2(:,:,k,ksdm))*
     +                      alpha(:,:,k,ksdm)/grav
          enddo
  
          cmixm(1,:,:,ksdm) = c0
          do k = 2,nlm
            do i = 1,iim
              do j = 1,jjm
                cmixm(k,i,j,ksdm) = -dt*km(k)/
     +                          (dzm2(i,j,k,ksdm)*dzm1(i,j,k,ksdm))
              enddo
            enddo
          enddo
!          write(*,*) 'cmixm(18,3,3,ksdm) = ',cmixm(18,3,3,ksdm),
!     +      dzm2(3,3,18),dzm1(3,3,18),dt

          emixm(nlm,:,:,ksdm) = c0
          do k = 1,nlm-1
            do i = 1,iim
              do j = 1,jjm
                emixm(k,i,j,ksdm) = -dt*km(k+1)/
     +                       (dzm2(i,j,k+1,ksdm)*dzm1(i,j,k+1,ksdm))
              enddo    
            enddo
          enddo
          do k = 1,nlm
            do i = 1,iim
              do j = 1,jjm
                dmixm(k,i,j,ksdm) = c1-cmixm(k,i,j,ksdm)
     +            -emixm(k,i,j,ksdm)
              enddo
            enddo
          enddo

! averaging matrix coefficients to cell walls
          do k = 1,nlm
!tdrmod
!           call average_to_cell_walls (cmixm(k,:,:,ksdm),ctmp_w)
!           call average_to_cell_walls (dmixm(k,:,:,ksdm),dtmp_w)
!           call average_to_cell_walls (emixm(k,:,:,ksdm),etmp_w)
!endtdrmod

!            if (k == 18) then
!              write(*,*) cmixm(k,3,3,ksdm),ctmp_w(:,3,3)
!              read(*,*)
!            endif

            do i = 1,iim
              do j = 1,jjm
                do iwall = 1,nwalls
                  cmixm_w(k,iwall,i,j,ksdm) = ctmp_w(iwall,i,j)
                  dmixm_w(k,iwall,i,j,ksdm) = dtmp_w(iwall,i,j)
                  emixm_w(k,iwall,i,j,ksdm) = etmp_w(iwall,i,j)
                enddo
              enddo
            enddo
          enddo
      
	enddo	! do ksdm = 1,nsdm

!        write(*,*) 'bottom of diff_coeff_mom:'
!        write(*,*) 'cmixm(:,1,1,1) = ',cmixm(:,1,1,1)
!        write(*,*) 'dmixm(:,1,1,1) = ',cmixm(:,1,1,1)
!        write(*,*) 'emixm(:,1,1,1) = ',cmixm(:,1,1,1)
!        write(*,*) 'cmixm_w(33:36,1,3,3,1) = ',cmixm_w(33:36,1,3,3,1)
!        write(*,*) 'dmixm_w(33:36,1,3,3,1) = ',dmixm_w(33:36,1,3,3,1)
!        write(*,*) 'emixm_w(33:36,1,3,3,1) = ',emixm_w(33:36,1,3,3,1)

!        write(*,*) 'dzm1(3,3,33:36,1) = ',dzm1(3,3,33:36,1)
!        write(*,*) 'dzm2(3,3,33:36,1) = ',dzm2(3,3,33:36,1)
!        write(*,*) 'plgl2 values:'
!        do k = 37,31,-1
!          write(*,*) plgl2(3,3,k,1)/100.,alphal2(3,3,k,1)
!        enddo
!        write(*,*) 'plg values:'
!        do k = 36,31,-1
!          write(*,*) plg(3,3,k,1)/100.,alpha(3,3,k,1),th(3,3,k,n4,1),
!     +      (th(3,3,k-1,n4,1)-th(3,3,k,n4,1))
!        enddo
!        write(*,*) 'layer thicknesses = ',
!     +   (plgl2(3,3,37,1)-plgl2(3,3,36,1))/100.,
!     +   (plgl2(3,3,36,1)-plgl2(3,3,35,1))/100.,
!     +   (plgl2(3,3,35,1)-plgl2(3,3,34,1))/100.,
!     +   (plgl2(3,3,34,1)-plgl2(3,3,33,1))/100.,
!     +   (plgl2(3,3,33,1)-plgl2(3,3,32,1))/100.,
!     +   (plgl2(3,3,32,1)-plgl2(3,3,31,1))/100.
!        read(*,*)
 
	end subroutine diff_coeff_mom

!===========================================================================
!  END OF DIFF_COEFF_MOM
!===========================================================================

	end module vertical_diffusion
!||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
