      MODULE wrap_vertex_data
! 19 SEPTEMBER 2001
      USE kinds
      USE model_parameters
      USE assign_owner

      IMPLICIT NONE
      SAVE

      INTEGER (KIND=int_kind) :: nghbr_list(12,2+10*(jm-2)**2),
     &                           cornSEND(15,6,5*isdm*jsdm),
     &                           cornRECV(12,6,5*isdm*jsdm),
     &                           cornPOLE(11,12)

      CONTAINS
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE initialize_wrap_vertices (l_error)
! 19 SEPTEMBER 2001
      IMPLICIT NONE
      LOGICAL (KIND=log_kind) :: l_error
      INTEGER (KIND=int_kind) :: count_tag

      l_error = .FALSE.

      IF (iim==3) THEN
         IF (my_task==0) THEN
            PRINT *,' initialize_wrap_vertices : with ',
     &          2+10*(jm-2)**2,' cells, and ',5*isdm*jsdm,' subdomains '
            PRINT *,'   each subdomain will have only one cell.  '
            PRINT *,'   wrap_vertices will not work at this resolution.'
         ENDIF
         l_error = .TRUE.
      ENDIF

      CALL set_nghbr_list ()
      CALL set_corner_connections ()
      count_tag = 0
      CALL set_tags (count_tag)
      CALL set_pole (count_tag)

      END SUBROUTINE initialize_wrap_vertices 
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE set_corner_connections ()
! 19 SEPTEMBER 2001
      IMPLICIT NONE

      LOGICAL (KIND=log_kind) :: l_receive_diag

      INTEGER (KIND=int_kind) :: 
     &         cell_global(8,im,jm),cell(8,iim,jjm,5*isdm*jsdm),
     &         cell_corner(3,2,iim,jjm,5*isdm*jsdm),
     &         msgEDGE(8,iim-2,4,5*isdm*jsdm),msgEDGE_count(5*isdm*jsdm)

      INTEGER (KIND=int_kind) :: 
     &    i_nghbr(4),i_start(4),di(4),j_nghbr(4),j_start(4),dj(4),
     &    i,ii,i1,i2,j,jj,j1,j2,cc,ca(3),cb(3),panel,isd,jsd,nsd,
     &    count,nsd_nghbr,edge,diag,nsd_src,find_cell

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! INITIALIZE
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      cornSEND(:,:,:) = 0
      cornRECV(:,:,:) = 0
      cornPOLE(:,:)   = 0

      cell_global(:,:,:)=0;cell(:,:,:,:)=0;cell_corner(:,:,:,:,:)=0
      msgEDGE(:,:,:,:) = 0; msgEDGE_count(:) = 0

      i_nghbr=(/iim,2,1,2/); i_start=(/iim-1,2,2,2/); di=(/0,1,0,1/)
      j_nghbr=(/2,jjm,2,1/); j_start=(/2,jjm-1,2,2/); dj=(/1,0,1,0/)
      
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! NUMBER THE CELL FACES OF cell_global IN THE CONVENTIONAL WAY
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      cell_global(1,1,jm-1) = 1; cell_global(1,im-1,1) = 2; count = 2
      DO j = 2,jm-1
         DO panel = 0,4
            DO i = 2,im5-1
               count = count + 1
               cell_global(1,im5*panel+i,j) = count
            ENDDO
         ENDDO
      ENDDO

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! SET THE SUBDOMAIN NUMBERS OF cell_global
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      cell_global(2,1,jm-1) = isdm*(  jsdm-1)+1 ! NORTH POLE
      cell_global(2,im-1,1) = isdm*(4*jsdm+1)   ! SOUTH POLE
      nsd = 0
      DO panel = 0,4
         DO jsd = 0,jsdm-1
            DO isd = 0,isdm-1
               nsd = nsd + 1
               DO jj = 2,jjm-1
                  DO ii = 2,iim-1
                     cell_global(2,im5*panel+(iim-2)*isd+ii,
     &                                       (jjm-2)*jsd+jj) = nsd
                  ENDDO
               ENDDO
            ENDDO
         ENDDO
      ENDDO

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! WRAP cell_global TO WRAP CELL CENTERS
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      CALL wrap_cell_global (cell_global)

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! SET TWO CORNERS FOR EACH CELL IN cell_global
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      DO panel = 0,4
         DO jsd = 0,jsdm-1
            DO isd = 0,isdm-1
               DO jj = 2,jjm-1
                  DO ii = 2,iim-1
                     i = im5*panel+(iim-2)*isd+ii; j = (jjm-2)*jsd+jj
                     cell_global(3:5,i,j) = (/cell_global(1,i,j),
     &                   cell_global(1,i-1,j  ),cell_global(1,i-1,j-1)/)
                     CALL sort (cell_global(3,i,j))
                     cell_global(6:8,i,j) = (/cell_global(1,i,j),
     &                   cell_global(1,i-1,j-1),cell_global(1,i  ,j-1)/)
                     CALL sort (cell_global(6,i,j))
                  ENDDO
               ENDDO
            ENDDO
         ENDDO
      ENDDO

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! WRAP cell_global TO WRAP CELL CORNERS
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      CALL wrap_cell_global (cell_global)

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! SET cell FOR EACH SUBDOMAIN
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      nsd = 0
      DO panel = 0,4
         DO jsd = 0,jsdm-1
            DO isd = 0,isdm-1
               nsd = nsd + 1
               DO jj = 1,jjm
                  DO ii = 1,iim
                     i = im5*panel+(iim-2)*isd+ii; j = (jjm-2)*jsd+jj
                     cell(:,ii,jj,nsd) = cell_global(:,i,j)
                  ENDDO
               ENDDO
            ENDDO
         ENDDO
      ENDDO

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! SET cell_corner FOR EACH SUBDOMAIN
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      DO nsd = 1,5*isdm*jsdm
         DO jj = 2,jjm
            DO ii = 2,iim
               cell_corner(:,1,ii,jj,nsd) = (/cell(1,ii,jj,nsd),
     &                     cell(1,ii-1,jj  ,nsd),cell(1,ii-1,jj-1,nsd)/)
               CALL sort (cell_corner(1,1,ii,jj,nsd))

               cell_corner(:,2,ii,jj,nsd) = (/cell(1,ii,jj,nsd),
     &                     cell(1,ii-1,jj-1,nsd),cell(1,ii  ,jj-1,nsd)/)
               CALL sort (cell_corner(1,2,ii,jj,nsd))
            ENDDO
         ENDDO
         DO ii = 1,iim-1
            IF (hexagonQ (cell(1,ii,1,nsd))) THEN
               CALL set_corner (cell(1,ii,1,nsd),cell(1,ii,2,nsd),1,
     &                                        cell_corner(1,1,ii,1,nsd))
            ENDIF
         ENDDO
         DO jj = 2,jjm-1
            IF (hexagonQ (cell(1,1,jj,nsd))) THEN
               CALL set_corner (cell(1,1,jj,nsd),cell(1,2,jj,nsd),3,
     &                                        cell_corner(1,1,1,jj,nsd))
            ENDIF
         ENDDO
      ENDDO

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! ELIMINATE CORNER OF EACH SUBDOMAIN IN cell WITH A (iim-1,jjm-1) PENTAGON
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      DO nsd = 1,5*isdm*jsdm
         IF (MOD (nsd-1,isdm*jsdm)+1==isdm*jsdm-jsdm) THEN
            cell(1:2,iim,jjm,nsd) = -999
         ENDIF
         IF (MOD (nsd-1,isdm*jsdm)+1==isdm*jsdm     ) THEN
            cell(1:2,iim,jjm,nsd) = -999
         ENDIF
      ENDDO

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! DETERMINE THE DESTINATION OF AN EDGE MESSAGE
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      DO nsd = 1,5*isdm*jsdm
         DO edge = 1,4
            nsd_nghbr = cell(2,i_nghbr(edge),j_nghbr(edge),nsd)
            cornSEND(1,edge,nsd) = nsd_nghbr
            msgEDGE_count(nsd_nghbr) = msgEDGE_count(nsd_nghbr)+1
            DO count = 1,iim-2
               i = i_start(edge)+di(edge)*(count-1)
               j = j_start(edge)+dj(edge)*(count-1)
               msgEDGE(:,count,msgEDGE_count(nsd_nghbr),nsd_nghbr) =
     &                                                   cell(:,i,j,nsd)
            ENDDO
         ENDDO
      ENDDO

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! DETERMINE HOW THE EDGE MESSAGES SHOULD BE STORED 
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      DO nsd = 1,5*isdm*jsdm
         DO edge = 1,4

! SUBDOMAIN SOURCE OF MESSAGE
            cornRECV(1,edge,nsd) = msgEDGE(2,1,edge,nsd)

! STARTING POSITION AND STRIDE
            DO jj = 1,jjm
               DO ii = 1,iim
                  IF (cell(1,ii,jj,nsd)==msgEDGE(1,1,edge,nsd)) THEN
                     i1=ii;j1=jj
                  ENDIF
                  IF (cell(1,ii,jj,nsd)==msgEDGE(1,2,edge,nsd)) THEN
                     i2=ii;j2=jj
                  ENDIF
               ENDDO
            ENDDO
            cornRECV( 3: 6,edge,nsd) = (/i1,j1,i2-i1,j2-j1/)

! OFFSET FOR CORNERS: IF THE SECOND CELL ON THE MESSAGE LIST IS A HEXAGON,
! THEN THERE EXISTS A PAIR OF MATCHING CORNERS FOR THAT CELL ON THE 
! cell_corner ARRAY.  IF THE SECOND CELL IS A PENTAGON THEN USE THE 
! FIRST CELL ON THE MESSAGE LIST TO FIND A MATCH. 

            IF (hexagonQ (msgEDGE(1,2,edge,nsd))) THEN
               find_cell = 2
            ELSE
               find_cell = 1
            ENDIF
            DO jj = 1,jjm
               DO ii = 1,iim
                  DO cc = 1,2
                     IF (equal_cornerQ(cell_corner(1,cc,ii,jj,nsd),
     &                              msgEDGE(3,find_cell,edge,nsd))) THEN
                        ca(:) = (/ii,jj,cc/)
                     ENDIF
                     IF (equal_cornerQ(cell_corner(1,cc,ii,jj,nsd),
     &                              msgEDGE(6,find_cell,edge,nsd))) THEN
                        cb(:) = (/ii,jj,cc/)
                     ENDIF
                  ENDDO
               ENDDO
            ENDDO
            IF (hexagonQ (msgEDGE(1,2,edge,nsd))) THEN
               cornRECV( 7: 9,edge,nsd) = ca(:)-(/i2,j2,0/)
               cornRECV(10:12,edge,nsd) = cb(:)-(/i2,j2,0/)
            ELSE
               cornRECV( 7: 9,edge,nsd) = ca(:)-(/i1,j1,0/)
               cornRECV(10:12,edge,nsd) = cb(:)-(/i1,j1,0/)
            ENDIF

         ENDDO
      ENDDO

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! DETERMINE HOW A DIAGONAL MESSAGE SHOULD BE STORED 
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      nsd = 0
      DO panel = 0,4
         DO jsd = 1,jsdm
            DO isd = 1,isdm
               nsd = nsd+1
               DO diag = 5,6
                  l_receive_diag = .FALSE.
                  IF (diag==5) THEN
                     IF (.NOT.(((isd==jsdm).OR.(isd==isdm)).AND.
     &                                                (jsd==jsdm))) THEN
                        l_receive_diag = .TRUE.; i2=iim; j2=jjm
                     ENDIF
                  ENDIF
                  IF (diag==6) THEN
                     IF ((2<=isd.AND.isd<=jsdm).AND.jsd==jsdm) THEN
                        l_receive_diag = .TRUE.; i2=1; j2=jjm
                     ENDIF
                     IF (isd==isdm.AND.(2<=jsd.AND.jsd<=jsdm)) THEN
                        l_receive_diag = .TRUE.; i2=iim; j2=1
                     ENDIF
                  ENDIF
                  IF (l_receive_diag) THEN
                     nsd_src = cell(2,i2,j2,nsd)

                     DO ii = 1,iim
                        DO jj = 1,jjm
                           IF (cell(1,ii,jj,nsd_src)==
     &                                           cell(1,i2,j2,nsd)) THEN
                              i1=ii; j1=jj
                           ENDIF
                        ENDDO
                     ENDDO
                     ca(:)=(/i2,j2,-1/); cb(:)=(/i2,j2,-1/)
                     DO ii = 1,iim
                        DO jj = 1,jjm
                           DO cc = 1,2
                              IF (equal_cornerQ (
     &                                      cell_corner(1,cc,ii,jj,nsd),
     &                                      cell(3,i1,j1,nsd_src))) THEN
                                 ca(:) = (/ii,jj,cc/)
                              ENDIF
                              IF (equal_cornerQ (
     &                                      cell_corner(1,cc,ii,jj,nsd),
     &                                      cell(6,i1,j1,nsd_src))) THEN
                                 cb(:) = (/ii,jj,cc/)
                              ENDIF
                           ENDDO
                        ENDDO
                     ENDDO
                     cornSEND( 1: 4,diag,nsd_src) = (/nsd    ,0,i1,j1/)
                     cornRECV( 1: 4,diag,nsd    ) = (/nsd_src,0,i2,j2/)
                     cornRECV( 7: 9,diag,nsd    ) = ca(:)-(/i2,j2,0/)
                     cornRECV(10:12,diag,nsd    ) = cb(:)-(/i2,j2,0/)
                  ENDIF
               ENDDO
            ENDDO
         ENDDO
      ENDDO

      END SUBROUTINE set_corner_connections
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE set_nghbr_list ()
! 06 NOVEMBER 2001
      IMPLICIT NONE
      INTEGER (KIND=int_kind) :: p,q,i,ii,j,panel,count,ij,o1,o2,n,
     &                   cell_type(im,jm),cell_number(im,jm),tmp_list(6)

      p = im-1; q = jm-1

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! MAKE A MASK FOR GHOST CELLS, HEXAGONS AND PENTAGONS
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      cell_type(:,:) = 0 ! GHOST CELL
      cell_type(1,q) = 5 ! NORTH POLE PENTAGON
      cell_type(p,1) = 5 ! SOUTH POLE PENTAGON
! INTERIOR HEXAGON
      DO panel = 0,4
         DO j = 2,jm
            DO i = 2,im5
               cell_type(im5*panel+i,j) = 6
            ENDDO
         ENDDO
! NORTH HEMISPHERE MID-LATITUDE PENTAGON
         cell_type(im5*panel+(  jm-1),jm-1) = 5
! SOUTH HEMISPHERE MID-LATITUDE PENTAGON
         cell_type(im5*panel+(2*jm-3),jm-1) = 5
      ENDDO

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! NUMBER THE CELLS IN cell_number IN THE CONVENTIONAL WAY
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      cell_number(1,q) = 1; cell_number(p,1) = 2; count = 2
      DO j = 2,jm-1
         DO panel = 0,4
            DO i = 2,im5-1
               count = count+1
               cell_number(im5*panel+i,j) = count
            ENDDO
         ENDDO
      ENDDO

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! WRAP cell_number
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      DO panel = 0,4
         o1 = im5*(MOD (panel+0,5)); o2 = im5*(MOD (panel+1,5))

         cell_number(o1+1    ,q) = cell_number(1,q) ! NORTH POLE
         cell_number(o1+im5-1,1) = cell_number(p,1) ! SOUTH POLE

         DO ij = 2,q ! OFFSET 1 : TOP LEFT
            cell_number(o1+ij,jm) = cell_number(o2+2,q+2-ij)
         ENDDO                                                                
         DO ij = q+1,im5-1 ! OFFSET 1 : TOP RIGHT
            cell_number(o1+ij,jm) = cell_number(o2-q+1+ij,2)
         ENDDO
         DO ij = 2,q ! OFFSET 1 : RIGHT EDGE
            cell_number(o1+im5,ij) = cell_number(o2+im5+1-ij,2)
         ENDDO

         DO ij = 1,q-1 ! OFFSET 2 : LEFT EDGE
            cell_number(o2+1,ij) = cell_number(o1+q+1-ij,q)
         ENDDO
         DO ij = 2,q ! OFFSET 2 : BOTTOM LEFT
            cell_number(o2+ij,1) = cell_number(o1+q-1+ij,q)
         ENDDO
         DO ij = q+1,im5-2 ! OFFSET 2 : BOTTOM RIGHT
            cell_number(o2+ij,1) = cell_number(o1+im5-1,im5-ij)
         ENDDO
      ENDDO

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! SET nghbr_list
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      nghbr_list(:,:) = 0
! NORTH POLE
      tmp_list = (/cell_number(0*im5+2,q),cell_number(1*im5+2,q),
     &             cell_number(2*im5+2,q),cell_number(3*im5+2,q),
     &             cell_number(4*im5+2,q),0/)
      DO n = 1,12
         nghbr_list(n,1) = tmp_list(1+MOD (n-1,5))
      ENDDO
! SOUTH POLE
      tmp_list = (/cell_number(5*im5-2,2),cell_number(4*im5-2,2),
     &             cell_number(3*im5-2,2),cell_number(2*im5-2,2),
     &             cell_number(1*im5-2,2),0/)
      DO n = 1,12
         nghbr_list(n,2) = tmp_list(1+MOD (n-1,5))
      ENDDO
! INTERIOR CELLS
      count = 2
      DO j = 2,jm-1
         DO panel = 0,4
            DO i = 2,im5-1
               count = count+1
               ii = panel*im5+i
               IF (cell_type(ii,j)==6) THEN
                  tmp_list = 
     & (/cell_number(ii-1,j),cell_number(ii-1,j-1),cell_number(ii,j-1),
     &   cell_number(ii+1,j),cell_number(ii+1,j+1),cell_number(ii,j+1)/)
                  DO n = 1,12
                     nghbr_list(n,count) = tmp_list(1+MOD (n-1,6))
                  ENDDO
               ENDIF
               IF (cell_type(ii,j)==5) THEN
                  tmp_list = 
     & (/cell_number(ii-1,j),cell_number(ii-1,j-1),cell_number(ii,j-1),
     &                       cell_number(ii+1,j),cell_number(ii,j+1),0/)
                  DO n = 1,12
                     nghbr_list(n,count) = tmp_list(1+MOD (n-1,5))
                  ENDDO
               ENDIF
            ENDDO
         ENDDO
      ENDDO

      END SUBROUTINE set_nghbr_list
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE wrap_cell_global (cell_global)
! 19 SEPTEMBER 2001
      IMPLICIT NONE
      INTEGER (KIND=int_kind) :: cell_global(8,im,jm)
      INTEGER (KIND=int_kind) :: ij,o1,o2,q,p,panel

      p = im-1; q = jm-1

      DO panel = 0,4
         o1 = im5*(MOD (panel+0,5)); o2 = im5*(MOD (panel+1,5))

         cell_global(:,o1+1    ,q) = cell_global(:,1,q) ! NORTH POLE
         cell_global(:,o1+im5-1,1) = cell_global(:,p,1) ! SOUTH POLE

         DO ij = 2,q ! OFFSET 1 : TOP LEFT
            cell_global(:,o1+ij,jm) = cell_global(:,o2+2,q+2-ij)
         ENDDO                                                                
         DO ij = q+1,im5-1 ! OFFSET 1 : TOP RIGHT
            cell_global(:,o1+ij,jm) = cell_global(:,o2-q+1+ij,2) 
         ENDDO
         DO ij = 2,q ! OFFSET 1 : RIGHT EDGE
            cell_global(:,o1+im5,ij) = cell_global(:,o2+im5+1-ij,2)  
         ENDDO

         DO ij = 1,q-1 ! OFFSET 2 : LEFT EDGE
            cell_global(:,o2+1,ij) = cell_global(:,o1+q+1-ij,q)                       
         ENDDO
         DO ij = 2,q ! OFFSET 2 : BOTTOM LEFT
            cell_global(:,o2+ij,1) = cell_global(:,o1+q-1+ij,q)                
         ENDDO
         DO ij = q+1,im5-2 ! OFFSET 2 : BOTTOM RIGHT
            cell_global(:,o2+ij,1) = cell_global(:,o1+im5-1,im5-ij)      
         ENDDO
      ENDDO

      END SUBROUTINE wrap_cell_global
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      LOGICAL FUNCTION equal_cornerQ (c1,c2)
! 19 SEPTEMBER 2001
      IMPLICIT NONE
      INTEGER (KIND=int_kind) :: c1(3),c2(3)

      IF ((c1(1)==c2(1)).AND.(c1(2)==c2(2)).AND.(c1(3)==c2(3))) THEN
         equal_cornerQ = .TRUE.
      ELSE
         equal_cornerQ = .FALSE.
      ENDIF

      END FUNCTION equal_cornerQ
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE sort (c)
! 19 SEPTEMBER 2001
      IMPLICIT NONE
      INTEGER (KIND=int_kind) :: c(3)
      IF ((c(1)<=c(2)).AND.(c(2)<=c(3))) c(:) = (/c(1),c(2),c(3)/)
      IF ((c(1)<=c(3)).AND.(c(3)<=c(2))) c(:) = (/c(1),c(3),c(2)/)
      IF ((c(2)<=c(1)).AND.(c(1)<=c(3))) c(:) = (/c(2),c(1),c(3)/)
      IF ((c(2)<=c(3)).AND.(c(3)<=c(1))) c(:) = (/c(2),c(3),c(1)/)
      IF ((c(3)<=c(1)).AND.(c(1)<=c(2))) c(:) = (/c(3),c(1),c(2)/)
      IF ((c(3)<=c(2)).AND.(c(2)<=c(1))) c(:) = (/c(3),c(2),c(1)/)
      END SUBROUTINE sort
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE set_corner (cell_number,cell_refcell,twist,cell_corner)
! 29 november 2001
      IMPLICIT NONE
      LOGICAL (KIND=log_kind) :: found
      INTEGER (KIND=int_kind) :: cell_number,cell_refcell,twist,
     &                           cell_corner(3,2)
      INTEGER (KIND=int_kind) :: n,n_refcell

      found = .FALSE.; n = 0
      DO WHILE ((.NOT.found).AND.(n<12))
         n=n+1
         IF (nghbr_list(n,cell_number)==cell_refcell) THEN
            found = .TRUE.
            n_refcell = n
         ENDIF
      ENDDO
 
      cell_corner(:,1) =
     &         (/cell_number,nghbr_list(n_refcell+twist  ,cell_number),
     &                       nghbr_list(n_refcell+twist+1,cell_number)/)
      cell_corner(:,2) =
     &         (/cell_number,nghbr_list(n_refcell+twist+1,cell_number),
     &                       nghbr_list(n_refcell+twist+2,cell_number)/)

      CALL sort (cell_corner(1,1))
      CALL sort (cell_corner(1,2))

      END SUBROUTINE set_corner
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      LOGICAL FUNCTION hexagonQ (cell_number)
! 30 november 2001
      INTEGER (KIND=int_kind) :: cell_number
      INTEGER (KIND=int_kind) :: cell_max,n

      cell_max = 2+10*(jm-2)**2

      hexagonQ = .TRUE.
      IF (cell_number==1) hexagonQ = .FALSE. ! NORTH POLE
      IF (cell_number==2) hexagonQ = .FALSE. ! SOUTH POLE
      DO n = 0,9
         IF (cell_max-n*(jm-2)==cell_number) hexagonQ = .FALSE.
      ENDDO

      END FUNCTION hexagonQ
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE set_tags (count_tag)
! 19 SEPTEMBER 2001
      IMPLICIT NONE

      INTEGER (KIND=int_kind) :: count_tag
      INTEGER (KIND=int_kind) :: 
     &               owner_tmp(5*isdm*jsdm),assign_tmp(npe_atmos,nsdm),
     &               task,nsd1,nsd1_owner,msg1,nsd_dstn,nsd2,msg2

      owner_tmp(:) = sbdmn_owner(max_level)%owner(:)
      DO task = 0,npe_atmos-1
         assign_tmp(task+1,:) = sbdmn_assign(task+1,max_level)%sd(:)
      ENDDO

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! GENERATE A UNIQUE TAG FOR EACH MESSAGE SENT
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      DO nsd1 = 1,5*isdm*jsdm
         nsd1_owner = owner_tmp(nsd1)
         DO msg1 = 1,6
            nsd_dstn = cornSEND(1,msg1,nsd1)
            IF (nsd_dstn/=0) THEN
               IF (nsd1_owner==owner_tmp(nsd_dstn)) THEN
                  DO nsd2 = 1,nsdm
                     IF (assign_tmp(nsd1_owner+1,nsd2)==nsd_dstn) THEN
                        cornSEND(15,msg1,nsd1) = nsd2
                     ENDIF
                  ENDDO
                  DO msg2 = 1,6
                     IF(cornRECV(1,msg2,nsd_dstn)==nsd1) THEN
                        cornSEND(5:14,msg1,nsd1) =
     &                                      cornRECV(3:12,msg2,nsd_dstn)
                     ENDIF
                  ENDDO
               ELSE
                  count_tag = count_tag+1
                  cornSEND(2,msg1,nsd1) = count_tag
                  DO msg2 = 1,6
                     IF (cornRECV(1,msg2,nsd_dstn)==nsd1) THEN
                        cornRECV(2,msg2,nsd_dstn) = count_tag
                     ENDIF
                  ENDDO
               ENDIF
            ENDIF
         ENDDO
      ENDDO

      END SUBROUTINE set_tags
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE set_pole (count_tag)
! 04 december 2001
      IMPLICIT NONE

      INTEGER (KIND=int_kind) :: count_tag
      INTEGER (KIND=int_kind) :: 
     &                owner_tmp(5*isdm*jsdm),assign_tmp(npe_atmos,nsdm),
     &                task,panel,nsd_pole(2),nsd_nghbr(5,2),count,npole,
     &                nsd_src,nsd_dstn,nsd1,nsd2,task_src,task_dstn

      owner_tmp(:) = sbdmn_owner(max_level)%owner(:)
      DO task = 0,npe_atmos-1
         assign_tmp(task+1,:) = sbdmn_assign(task+1,max_level)%sd(:)
      ENDDO

      nsd_pole(1) = isdm*(  jsdm-1)+1 ! SUBDOMAIN NUMBER OF NORTH POLE
      nsd_pole(2) = isdm*(4*jsdm+1)   ! SUBDOMAIN NUMBER OF SOUTH POLE

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! MESSAGES SENT TO SUBDOMAINS WHICH NEIGHBOR THE NORTH AND SOUTH POLE
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      DO panel = 0,4
         nsd_nghbr(panel+1,1) = isdm*jsdm*panel+isdm*(jsdm-1)+1
         nsd_nghbr(panel+1,2) = isdm*jsdm*panel+isdm   
      ENDDO

      count = 0

      DO npole = 1,2
         DO panel = 0,4
            nsd_src = nsd_nghbr(MOD (panel+2,5)+1,npole)
            task_src = owner_tmp(nsd_src)
            nsd1 = stack_position (task_src,nsd_src,assign_tmp)

            nsd_dstn = nsd_nghbr(panel+1,npole)
            task_dstn = owner_tmp(nsd_dstn)
            nsd2 = stack_position (task_dstn,nsd_dstn,assign_tmp)

            count = count+1
            cornPOLE(1:2,count) = (/nsd_src,nsd_dstn/)

            IF (task_src==task_dstn) THEN
               cornPOLE(3,count) = 0
            ELSE
               count_tag = count_tag+1
               cornPOLE(3,count) = count_tag
            ENDIF

            IF (npole==1) THEN
               cornPOLE(4:11,count) = (/2,jjm-1,1,nsd1,2,jjm,1,nsd2/)
            ENDIF
            IF (npole==2) THEN
               cornPOLE(4:11,count) = (/iim-1,2,2,nsd1,iim,2,2,nsd2/)
            ENDIF
         ENDDO
      ENDDO

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! ONE ADDITIONAL MESSAGE IS SENT TO BOTH THE NORTH AND SOUTH POLE 
! SO THAT THE PENTAGON CAN SEE ALL FIVE CORNERS
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      DO npole = 1,2
         nsd_dstn = nsd_pole(npole)
         task_dstn = owner_tmp(nsd_dstn)
         nsd2 = stack_position (task_dstn,nsd_dstn,assign_tmp)

         IF (npole==1) THEN
            nsd_src = 3*isdm*jsdm+isdm*(jsdm-1)+1
         ENDIF
         IF (npole==2) THEN
            nsd_src = 2*isdm*jsdm+isdm 
         ENDIF

         task_src = owner_tmp(nsd_src)
         nsd1 = stack_position (task_src,nsd_src,assign_tmp)

         count = count+1
         cornPOLE(1:2,count) = (/nsd_src,nsd_dstn/)

         IF (task_src==task_dstn) THEN
            cornPOLE(3,count) = 0
         ELSE
            count_tag = count_tag+1
            cornPOLE(3,count) = count_tag
         ENDIF

         IF (npole==1) THEN
            cornPOLE(4:11,count)=(/2,jjm-1,1,nsd1,1,jjm-1,1,nsd2/)
         ENDIF
         IF (npole==2) THEN
            cornPOLE(4:11,count)=(/iim-1,2,2,nsd1,iim-1,1,2,nsd2/)
         ENDIF 
      ENDDO

      END SUBROUTINE set_pole
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      INTEGER FUNCTION stack_position (task,nsd_target,assign_tmp)
! 04 DECEMBER 2001
      INTEGER (KIND=int_kind) :: task,nsd_target,
     &                                        assign_tmp(npe_atmos,nsdm)
      INTEGER (KIND=int_kind) :: nsd

      stack_position = -999
      DO nsd = 1,nsdm
         IF (nsd_target==assign_tmp(task+1,nsd)) THEN
            stack_position = nsd
         ENDIF
      ENDDO

      END FUNCTION stack_position
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      SUBROUTINE wrap_vertices (nspace,ntprog,x)
! 14 DECEMBER 2001
      IMPLICIT NONE
#ifdef mpi
      INCLUDE 'mpif.h'
#endif

      INTEGER (KIND=int_kind) :: nspace,ntprog
      INTEGER (KIND=int_kind) :: 
     &        owner_tmp(5*isdm*jsdm),assign_tmp(npe_atmos,nsdm),
     &        i ,j , i_start   , j_start   ,di    ,dj    , c(3),nsd ,
     &        i1,j1,i1_start(4),j1_start(4),di1(4),dj1(4),c1(3),nsd1,
     &        i2,j2,i2_start   ,j2_start   ,di2   ,dj2   ,c2(3),nsd2,
     &        sbdmn_number,edge,diag,mssg,size_edge,size_diag,size_pole,
     &        send_req(6*nsdm+12),count_send_req,
     &        recv_req(6*nsdm+12),count_recv_req,
     &        task_src,task_dest,mssg_tag,ca(3),cb(3),
     &        count,task,ierr

#ifdef mpi
      INTEGER (KIND=int_kind), ALLOCATABLE :: 
     &                                 send_status(:,:),recv_status(:,:)
#endif

      REAL (KIND=dbl_kind) :: x(nspace,2,iim,jjm,ntprog,nsdm)

      REAL (KIND=dbl_kind),ALLOCATABLE :: buffer_send_edge(:,:,:,:,:,:),
     &                                    buffer_send_diag(:,:,:,:,:),
     &                                    buffer_send_pole(:,:,:)
      REAL (KIND=dbl_kind),ALLOCATABLE :: buffer_recv_edge(:,:,:,:,:,:),
     &                                    buffer_recv_diag(:,:,:,:,:),
     &                                    buffer_recv_pole(:,:,:)

      ALLOCATE (buffer_send_edge(nspace,2,iim-2,ntprog,1:4,nsdm))
      ALLOCATE (buffer_send_diag(nspace,2,      ntprog,5:6,nsdm))
      ALLOCATE (buffer_send_pole(nspace,        ntprog,      12))

      ALLOCATE (buffer_recv_edge(nspace,2,iim-2,ntprog,1:4,nsdm))
      ALLOCATE (buffer_recv_diag(nspace,2,      ntprog,5:6,nsdm))
      ALLOCATE (buffer_recv_pole(nspace,        ntprog,      12))

      buffer_send_edge(:,:,:,:,:,:) = 0.0
      buffer_send_diag(:,:,:,:,:)   = 0.0
      buffer_send_pole(:,:,:)       = 0.0
      buffer_recv_edge(:,:,:,:,:,:) = 0.0
      buffer_recv_diag(:,:,:,:,:)   = 0.0
      buffer_recv_pole(:,:,:)       = 0.0

      send_req(:) = -999; recv_req(:) = -999
      
      i1_start=(/iim-1,2,2,2/); di1=(/0,1,0,1/)
      j1_start=(/2,jjm-1,2,2/); dj1=(/1,0,1,0/)

      size_edge = 2*nspace*(iim-2)*ntprog
      size_diag = 2*nspace*        ntprog
      size_pole =   nspace*        ntprog

      count_send_req = 0; count_recv_req = 0

      owner_tmp(:) = sbdmn_owner(max_level)%owner(:)
      DO task = 0,npe_atmos-1
         assign_tmp(task+1,:) = sbdmn_assign(task+1,max_level)%sd(:)
      ENDDO

#ifdef mpi
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! POST RECEIVES FOR EDGES, DIAGONALS AND POLES
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      DO nsd1 = 1,nsdm
         sbdmn_number = assign_tmp(my_task+1,nsd1)
! POST EDGE RECEIVES
         DO edge = 1,4
            IF (cornRECV(2,edge,sbdmn_number)/=0) THEN
               task_src = owner_tmp(cornRECV(1,edge,sbdmn_number))
               mssg_tag =           cornRECV(2,edge,sbdmn_number)
               count_recv_req = count_recv_req+1
               CALL MPI_IRECV (buffer_recv_edge(1,1,1,1,edge,nsd1),
     &                       size_edge,mpi_float_type,task_src,mssg_tag,
     &                     MPI_COMM_ATMOS,recv_req(count_recv_req),ierr)
            ENDIF
         ENDDO

! POST DIAGONAL RECEIVE
         DO diag = 5,6
            IF ((cornRECV(1,diag,sbdmn_number)/=0).AND. 
     &                          (cornRECV(2,diag,sbdmn_number)/=0)) THEN
               task_src = owner_tmp(cornRECV(1,diag,sbdmn_number))
               mssg_tag =           cornRECV(2,diag,sbdmn_number)
               count_recv_req = count_recv_req+1
               CALL MPI_IRECV (buffer_recv_diag(1,1,1,diag,nsd1),
     &                       size_diag,mpi_float_type,task_src,mssg_tag,
     &                     MPI_COMM_ATMOS,recv_req(count_recv_req),ierr)
            ENDIF
         ENDDO
      ENDDO ! LOOP OVER nsdm SUBDOMAINS

! POST RECEIVES FOR THE POLES
      DO mssg = 1,12
         IF ((my_task==
     &             owner_tmp(cornPOLE(2,mssg))).AND.
     &                                       (cornPOLE(3,mssg)/=0)) THEN
            task_src = owner_tmp(cornPOLE(1,mssg))
            mssg_tag =           cornPOLE(3,mssg)
            count_recv_req = count_recv_req+1
            CALL MPI_IRECV (buffer_recv_pole(1,1,mssg),
     &                       size_pole,mpi_float_type,task_src,mssg_tag,
     &                     MPI_COMM_ATMOS,recv_req(count_recv_req),ierr)
         ENDIF
      ENDDO
#endif

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! POST SENDS FOR EDGES AND DIAGONAL AND POLE
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      DO nsd1 = 1,nsdm
         sbdmn_number = assign_tmp(my_task+1,nsd1)
! POST SENDS FOR EDGES
         DO edge = 1,4
            IF (cornSEND(2,edge,sbdmn_number)==0) THEN
               i2_start = cornSEND( 5   ,edge,sbdmn_number)
               j2_start = cornSEND( 6   ,edge,sbdmn_number)
               di2      = cornSEND( 7   ,edge,sbdmn_number)
               dj2      = cornSEND( 8   ,edge,sbdmn_number)
               ca(:)    = cornSEND( 9:11,edge,sbdmn_number)
               cb(:)    = cornSEND(12:14,edge,sbdmn_number)
               nsd2     = cornSEND(15   ,edge,sbdmn_number)
               DO count = 1,iim-2
                 i1 = i1_start(edge)+di1(edge)*(count-1)
                 j1 = j1_start(edge)+dj1(edge)*(count-1)
                 i2 = i2_start      +di2      *(count-1)
                 j2 = j2_start      +dj2      *(count-1)
                 x(:,ca(3),i2+ca(1),j2+ca(2),:,nsd2)=x(:,1,i1,j1,:,nsd1)
                 x(:,cb(3),i2+cb(1),j2+cb(2),:,nsd2)=x(:,2,i1,j1,:,nsd1)
               ENDDO
            ELSE
               task_dest = owner_tmp(cornSEND(1,edge,sbdmn_number))
               mssg_tag  =           cornSEND(2,edge,sbdmn_number)
               DO count = 1,iim-2
                  i1 = i1_start(edge)+di1(edge)*(count-1)
                  j1 = j1_start(edge)+dj1(edge)*(count-1)
                  buffer_send_edge(:,:,count,:,edge,nsd1) =
     &                                               x(:,:,i1,j1,:,nsd1)
               ENDDO
#ifdef mpi
               count_send_req = count_send_req+1
               CALL MPI_ISEND (buffer_send_edge(1,1,1,1,edge,nsd1),
     &                      size_edge,mpi_float_type,task_dest,mssg_tag,
     &                     MPI_COMM_ATMOS,send_req(count_send_req),ierr)
#endif
            ENDIF

         ENDDO

! POST SENDS FOR DIAGONAL
         DO diag = 5,6
            IF (cornSEND(1,diag,sbdmn_number)/=0) THEN

               i1 = cornSEND(3,diag,sbdmn_number)
               j1 = cornSEND(4,diag,sbdmn_number)
               IF (cornSEND(2,diag,sbdmn_number)==0) THEN
                  i2    = cornSEND( 5   ,diag,sbdmn_number)
                  j2    = cornSEND( 6   ,diag,sbdmn_number)
                  ca(:) = cornSEND( 9:11,diag,sbdmn_number)
                  cb(:) = cornSEND(12:14,diag,sbdmn_number)
                  nsd2  = cornSEND(15   ,diag,sbdmn_number)
                  IF (ca(3)/=-1) THEN
                    x(:,ca(3),i2+ca(1),j2+ca(2),:,nsd2) =
     &                                               x(:,1,i1,j1,:,nsd1)
                  ENDIF
                  IF (cb(3)/=-1) THEN
                    x(:,cb(3),i2+cb(1),j2+cb(2),:,nsd2) =
     &                                               x(:,2,i1,j1,:,nsd1)
                  ENDIF
               ELSE
                  task_dest = owner_tmp(cornSEND(1,diag,sbdmn_number))
                  mssg_tag  =           cornSEND(2,diag,sbdmn_number)
                  buffer_send_diag(:,:,:,diag,nsd1)=x(:,:,i1,j1,:,nsd1)
#ifdef mpi
                  count_send_req = count_send_req+1
                  CALL MPI_ISEND (buffer_send_diag(1,1,1,diag,nsd1),
     &                      size_diag,mpi_float_type,task_dest,mssg_tag,
     &                     MPI_COMM_ATMOS,send_req(count_send_req),ierr)
#endif
               ENDIF
            ENDIF
         ENDDO
      ENDDO ! LOOP OVER nsdm SUBDOMAINS

! POST SENDS FOR THE POLES
      DO mssg = 1,12
         IF (my_task==owner_tmp(cornPOLE(1,mssg))) THEN
            c1(:) = cornPOLE( 4: 6,mssg)
            nsd1  = cornPOLE( 7   ,mssg)
            IF (cornPOLE(3,mssg)==0) THEN
               c2(:) = cornPOLE( 8:10,mssg)
               nsd2  = cornPOLE(11   ,mssg)
               x(:,c2(3),c2(1),c2(2),:,nsd2) = 
     &                                     x(:,c1(3),c1(1),c1(2),:,nsd1)
            ELSE
               task_dest = owner_tmp(cornPOLE(2,mssg))
               mssg_tag  =           cornPOLE(3,mssg)
               buffer_send_pole(:,:,mssg) = 
     &                                     x(:,c1(3),c1(1),c1(2),:,nsd1)
#ifdef mpi
               count_send_req = count_send_req+1
               CALL MPI_ISEND (buffer_send_pole(1,1,mssg),
     &                      size_pole,mpi_float_type,task_dest,mssg_tag,
     &                     MPI_COMM_ATMOS,send_req(count_send_req),ierr)
#endif
            ENDIF
         ENDIF
      ENDDO

#ifdef mpi
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! WAIT FOR MESSAGES TO BE SENT AND TO BE RECEIVED
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ALLOCATE (send_status(MPI_STATUS_SIZE,count_send_req))
      ALLOCATE (recv_status(MPI_STATUS_SIZE,count_recv_req))
      send_status(:,:) = 0
      recv_status(:,:) = 0

      CALL MPI_WAITALL (count_send_req,send_req,send_status,ierr)
      CALL MPI_WAITALL (count_recv_req,recv_req,recv_status,ierr)
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! UNPACK BUFFER FOR EDGE AND DIAGONAL AND POLE MESSAGES
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      DO nsd = 1,nsdm
         sbdmn_number = assign_tmp(my_task+1,nsd)
         DO edge = 1,4
            IF (cornRECV(2,edge,sbdmn_number)/=0) THEN
               i_start = cornRECV( 3   ,edge,sbdmn_number)
               j_start = cornRECV( 4   ,edge,sbdmn_number)
               di      = cornRECV( 5   ,edge,sbdmn_number)
               dj      = cornRECV( 6   ,edge,sbdmn_number)
               ca(:)   = cornRECV( 7: 9,edge,sbdmn_number)
               cb(:)   = cornRECV(10:12,edge,sbdmn_number)
               DO count = 1,iim-2
                  i = i_start + di*(count-1)
                  j = j_start + dj*(count-1)
                  x(:,ca(3),i+ca(1),j+ca(2),:,nsd) = 
     &                            buffer_recv_edge(:,1,count,:,edge,nsd)
                  x(:,cb(3),i+cb(1),j+cb(2),:,nsd) = 
     &                            buffer_recv_edge(:,2,count,:,edge,nsd)
               ENDDO
            ENDIF
         ENDDO
         
         DO diag = 5,6
            IF ((cornRECV(1,diag,sbdmn_number)/=0).AND.
     &                          (cornRECV(2,diag,sbdmn_number)/=0)) THEN
               i     = cornRECV( 3   ,diag,sbdmn_number)
               j     = cornRECV( 4   ,diag,sbdmn_number)
               ca(:) = cornRECV( 7: 9,diag,sbdmn_number)
               cb(:) = cornRECV(10:12,diag,sbdmn_number)
               IF (ca(3)/=-1) THEN
                  x(:,ca(3),i+ca(1),j+ca(2),:,nsd) = 
     &                                  buffer_recv_diag(:,1,:,diag,nsd)
               ENDIF
               IF (cb(3)/=-1) THEN
                  x(:,cb(3),i+cb(1),j+cb(2),:,nsd) = 
     &                                  buffer_recv_diag(:,2,:,diag,nsd)
               ENDIF
            ENDIF
         ENDDO
      ENDDO ! LOOP OVER nsdm SUBDOMAINS

! UNPACK BUFFER FOR POLE MESSAGES
      DO mssg = 1,12
         IF ((my_task==owner_tmp(cornPOLE(2,mssg))).AND.
     &                                       (cornPOLE(3,mssg)/=0)) THEN
              c(:) = cornPOLE( 8:10,mssg)
              nsd  = cornPOLE(11   ,mssg)
            x(:,c(3),c(1),c(2),:,nsd) = buffer_recv_pole(:,:,mssg)
         ENDIF
      ENDDO
#endif

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! IF (iim-1,jjm-1) IS A PENTAGON 
! THEN BOTH CORNERS AT (iim,jjm) ARE SET EQUAL TO EACH OTHER
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      DO nsd = 1,nsdm
         sbdmn_number = assign_tmp(my_task+1,nsd)
         IF (1+MOD (sbdmn_number-1,isdm*jsdm)==(jsdm-1)*isdm+jsdm) THEN
            x(:,2,iim,jjm,:,nsd) = x(:,1,iim,jjm,:,nsd)
         ENDIF
         IF (1+MOD (sbdmn_number-1,isdm*jsdm)==(jsdm-1)*isdm+isdm) THEN
            x(:,1,iim,jjm,:,nsd) = x(:,2,iim,jjm,:,nsd)
         ENDIF
      ENDDO

      DEALLOCATE (buffer_send_edge,buffer_send_diag,buffer_send_pole)
      DEALLOCATE (buffer_recv_edge,buffer_recv_diag,buffer_recv_pole)
#ifdef mpi
      DEALLOCATE (send_status,recv_status)
#endif

      END SUBROUTINE wrap_vertices
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc
      END MODULE wrap_vertex_data
!cccccccc1ccccccccc2ccccccccc3ccccccccc4ccccccccc5cccgtgccc6ccccccccc7cc

      



