C
C==================SUBROUTINE ADDINC====================================
C
      subroutine addinc(grav2,cp,dtt,hc,hg,bps,ecmass,
     *      egmass,fss,fws,psb,
     *      td,thm,sh,tc,tg,
     *      dtd,dtc,dtg,drst, rst,bintc, len, nsib, nsoil )

      use kinds

      implicit none
C
C=======================================================================
c
C        Add prognostic variable increments to prognostic variables
c           and diagnose evapotranspiration and sensible heat flux.
c
c        Modified for multitasking - introduced gather/scatter indices
c           - DD 951206                                  
C
C=======================================================================
c

c++++++++++++++++++++++++++++++OUTPUT+++++++++++++++++++++++++++++++++++
c
C       TC             CANOPY TEMPERATURE (K)
C       TG             GROUND SURFACE TEMPERATURE (K)
C       TD             DEEP SOIL TEMPERATURE (K)
C       THM            MIXED LAYER POTENTIAL TEMPERATURE (K)
C       QM (gsh)       MIXED LAYER MIXING RATIO (KG KG-1)
C       ETMASS (FWS)   EVAPOTRANSPIRATION (MM)
C       HFLUX (FSS)    SENSIBLE HEAT FLUX (W M-2)
C       rst (FSS)     STOMATAL RESISTANCE (S M-1)
C
c++++++++++++++++++++++++++DIAGNOSTICS++++++++++++++++++++++++++++++++++
C
C       DTH            MIXED LAYER POTENTIAL TEMPERATURE INCREMENT (K)
C       DQM            MIXED LAYER MIXING RATIO INCREMENT (KG KG-1)
c
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      integer (kind=int_kind), intent(in) ::
     *     len, nsib, nsoil
      real (kind=dbl_kind), intent(in) ::
     *      grav2, cp, dtt
      real (kind=dbl_kind), dimension(len), intent(in) ::
     *      hc,hg,bps,ecmass,
     *      egmass,psb,
     *      dtc,dtg,DRST,
     *      bintc
      real (kind=dbl_kind), dimension(len,nsoil), intent(in) ::
     *      dtd
      real (kind=dbl_kind), dimension(len), intent(out) ::
     *      fss,fws
      real (kind=dbl_kind), dimension(len), intent(inout) ::
     *      thm,sh,tc,tg,
     *      rst
      real (kind=dbl_kind), dimension(nsib,nsoil), intent(inout) ::
     *      td(nsib,nsoil)

c     local variables
      integer (kind=int_kind) :: i, n
      real (kind=dbl_kind) ::
     *     dqm, dth


         DO I=1,len                                               
           FSS(i)   = (HC(I) + HG(I)) / (DTt * BPS(i))            
           FWS(i)   = (ECMASS(I) + EGMASS(I)) / DTt                        
           DTH      = FSS(i) * Grav2 * DTt / (CP * PSB(i))               
           DQM      = FWS(i) * Grav2 * DTt / PSB(i)                      
           FSS(i)   = FSS(i) *BPS(i)                                    
           THM(i)   = THM(i) + DTH                                   
           SH(i)    = SH(i)  + DQM                                   
           TC(i)    = TC(i)  + DTC(I)                                   
           tg(i)    = tg(i)  + DTG(I)                                   
           rst(i) = rst(i) + drst(i)
c bintc(i)- smallest canopy stomatal conductance needs to be passed in here.
c ---- c.zhang, 2/3/93
           rst(i)=MIN( 1.0_dbl_kind/bintc(i), rst(i) )
           rst(i)=MAX( 10.0_dbl_kind, rst(i) )                                  
         enddo                                                       
      do n = 1,nsoil
         DO I=1,len
           TD(i,n)    = TD(i,n) + dtd(i,n)      
         enddo
      enddo                                               

      return
      end
C
C======================SUBROUTINE ADJUST================================
C
      SUBROUTINE ADJUST ( TS, SPECHC, CAPACP, SNOWWP, IVEG 
     *,     capac,snoww,tm,tf,snomel,www,zdepth
     *,     satcap,cw,nsib, len)

      use kinds

      implicit none

      integer (kind=int_kind), intent(in) ::
     *    len, nsib, iveg
      real (kind=dbl_kind), intent(in), dimension(len,2) ::
     *     satcap(len,2)
      real (kind=dbl_kind), intent(in) ::
     *     zdepth,
     *     spechc, capacp, snowwp, tm, tf, snomel,
     *     cw
      real (kind=dbl_kind), intent(inout) ::
     *     www, ts
      real (kind=dbl_kind), intent(inout), dimension(nsib,2) ::
     *     capac,snoww
C
C=======================================================================
C
C     TEMPERATURE CHANGE DUE TO ADDITION OF PRECIPITATION
C
C=======================================================================
C
c++++++++++++++++++++++++++++++OUTPUT+++++++++++++++++++++++++++++++++++
c
C       TC             CANOPY TEMPERATURE (K)
C       WWW(1)         GROUND WETNESS OF SURFACE LAYER 
C       CAPAC(2)       CANOPY/GROUND LIQUID INTERCEPTION STORE (M)
C       SNOWW(2)       CANOPY/GROUND SNOW INTERCEPTION STORE (M)
C
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

c     local variables
      real (kind=dbl_kind) ::
     *    freeze, diff, ccp, cct, tsd, tta, ttb, cca, ccb, ccc, xs
      FREEZE = 0.
      DIFF = ( CAPAC(1,IVEG)+SNOWW(1,IVEG) - CAPACP-SNOWWP )*CW
      CCP = SPECHC
      CCT = SPECHC + DIFF
C
      TSD = ( TS * CCP + TM * DIFF ) / CCT
C
      IF ( ( TS .GT. TF .AND. TM .le. TF ) .or. 
     *     ( TS .LE. TF .AND. TM .gt. TF ) )then
C
         TTA = TS
         TTB = TM
         CCA = CCP
         CCB = DIFF
         IF ( TSD .le. TF ) then
C
C----------------------------------------------------------------------
C    FREEZING OF WATER ON CANOPY OR GROUND
C----------------------------------------------------------------------
C
            CCC = CAPACP * SNOMEL
            IF ( TS .LT. TM ) CCC = DIFF * SNOMEL / CW
            TSD = ( TTA * CCA + TTB * CCB + CCC ) / CCT
C
            FREEZE = ( TF * CCT - ( TTA * CCA + TTB * CCB ) )
            FREEZE = (MIN ( CCC, FREEZE )) / SNOMEL
            IF(TSD .GT. TF)TSD = TF - 0.01
C
         else
C
C----------------------------------------------------------------------
C    MELTING OF SNOW ON CANOPY OR GROUND, WATER INFILTRATES.
C----------------------------------------------------------------------
C
            CCC = - SNOWW(1,IVEG) * SNOMEL
            IF ( TS .GT. TM ) CCC = - DIFF * SNOMEL / CW
C
            TSD = ( TTA * CCA + TTB * CCB + CCC ) / CCT
C
            FREEZE = ( TF * CCT - ( TTA * CCA + TTB * CCB ) )
            FREEZE = (MAX( CCC, FREEZE )) / SNOMEL
            IF(TSD .LE. TF)TSD = TF - 0.01
C
         endif
      endif
      SNOWW(1,IVEG) = SNOWW(1,IVEG) + FREEZE
      CAPAC(1,IVEG) = CAPAC(1,IVEG) - FREEZE
      snoww(1,IVEG) = max(snoww(1,IVEG),0.0_dbl_kind)
      capac(1,IVEG) = max(capac(1,IVEG),0.0_dbl_kind)
      
C
      XS = MAX( 0.0_dbl_kind, ( CAPAC(1,IVEG) - SATCAP(1,IVEG) ) )
      IF( SNOWW(1,IVEG) .GE. 0.0000001_dbl_kind ) XS = CAPAC(1,IVEG)
      WWW = WWW + XS / ZDEPTH 
      CAPAC(1,IVEG) = CAPAC(1,IVEG) - XS
      TS = TSD

C
      RETURN
      END
C                                                                      
C===================SUBROUTINE BALAN====================================
C                                                                      
      SUBROUTINE BALAN ( IPLACE, tau, zdepth,www
     ^,            capac,ppc
     ^,            ppl,roff,etmass,totwb,radt
     ^,            chf,shf,dtt,ect,eci,egs,egi
     ^,            hc,hg,heaten,hflux ,snoww,thm,tc,tg,tgs,td,ps,kapa
     ^,            nsib, len, ioffset, nsoil )

      use kinds
   
      implicit none

      integer (kind=int_kind), intent(in) ::
     *     len, nsib, iplace, nsoil, ioffset
      real (kind=dbl_kind), intent(in) ::
     *     kapa, tau, dtt
      real (kind=dbl_kind), dimension(len), intent(in) ::
     *     ppc, ppl
     *,    chf, shf
     *,    ect, eci, egs, egi, hc
     *,    hg, heaten, hflux, thm, tc
     *,    tg, tgs, ps  
      real (kind=dbl_kind), dimension(nsib,2), intent(in) ::
     *     capac, snoww
      real (kind=dbl_kind), dimension(nsib,3), intent(in) ::
     *     zdepth
      real (kind=dbl_kind), dimension(len,2), intent(in) ::
     *     radt
      real (kind=dbl_kind), dimension(len,3), intent(in) ::
     *     www
      real (kind=dbl_kind), dimension(nsib,nsoil), intent(in) ::
     *     td

      real (kind=dbl_kind), dimension(len), intent(inout) ::
     *     roff, etmass, totwb

c     local variables
      integer (kind=int_kind) ::
     *    i, j, igp, nerror, n
      integer (kind=int_kind), dimension(len) ::
     *    indxerr
      real (kind=dbl_kind) ::
     *    tm
      real (kind=dbl_kind), dimension(len) ::
     *     endwb, errorw, pcmeter, plmeter
     *,    emeter, cbal, gbal, errore
     *,    zlhs, zrhs                     
C                                                                      
C=======================================================================
C                                                                      
C     ENERGY AND WATER BALANCE CHECK.                                  
C                                                                      
C-----------------------------------------------------------------------
C                                                                      
      IF( IPLACE .EQ. 1 ) then                                    
C 
      do i = 1,len                                                                     
         ETMASS(i) = 0.                                                      
         ROFF(i)   = 0.                                                      
C                                                                      
         TOTWB(i) = WWW(i,1) * ZDEPTH(i,1)                               
     &            + WWW(i,2) * ZDEPTH(i,2)                               
     &            + WWW(i,3) * ZDEPTH(i,3)                               
     &            + CAPAC(i,1) + CAPAC(i,2) + snoww(i,1) + snoww(i,2) 
      enddo                           
C                                                                      
      else
C   
      nerror = 0     
      do i = 1,len                                                             
         ENDWB(i) = WWW(i,1) * ZDEPTH(i,1)                               
     &      + WWW(i,2) * ZDEPTH(i,2)                               
     &      + WWW(i,3) * ZDEPTH(i,3)                               
     &      + CAPAC(i,1) + CAPAC(i,2) + snoww(i,1) + snoww(i,2)                            
     &      - (PPL(i)+PPC(i))*0.001*dtt
     &      + ETMASS(i)*0.001 + ROFF(i)                    
         ERRORW(i)= TOTWB(i) - ENDWB(i)                                            
         pcmeter(i) = ppc(i) * 0.001*dtt
         plmeter(i) = ppl(i) * 0.001*dtt                                        
         EMETER(i)= ETMASS(i) * 0.001
         if(abs(errorw(i)).gt.1.e-6) then
            nerror = nerror + 1
            indxerr(nerror) = i
         endif
      enddo                                             
C                                                                      
      do j = 1,nerror  
         i = indxerr(j)                                                          
            igp = i+ioffset
            WRITE(6,900) tau,IGP, TOTWB(i), ENDWB(i), ERRORW(i),                      
     &                WWW(i,1), WWW(i,2), WWW(i,3),                    
     &                CAPAC(i,1), CAPAC(i,2),snoww(i,1),snoww(i,2),                        
     &                pcmeter(i),plmeter(i), EMETER(i), ROFF(i) 
      enddo                                             
C                                                                      
C-----------------------------------------------------------------------
C     
      nerror = 0                                                                 
      do i = 1,len                                                             
         CBAL(i) = RADT(i,1) - CHF(i) - 
     *                   (ECT(i)+HC(i)+ECI(i) )/DTT                          
         GBAL(i) = RADT(i,2) - SHF(i) - (EGS(i)+HG(i)+EGI(i)
     *               - HEATEN(i) )/DTT             
         ZLHS(i) = RADT(i,1)+RADT(i,2) - CHF(i) - SHF(i)                             
         ZRHS(i) = HFLUX(i) + (ECT(i) + ECI(i) + EGI(i) + EGS(i)
     *               + HEATEN(i) ) /DTT
C                                                                      
         ERRORE(i)= ZLHS(i) - ZRHS(i)                                              
         if(abs(errore(i)).gt.1.0_dbl_kind) then
            nerror = nerror + 1
            indxerr(nerror) = i
         endif
      enddo                                             
C                                                                      
      do j = 1,nerror  
         i = indxerr(j)                                                          
            tm = thm(i) * (0.001*ps(i))**kapa                                          
            igp = i+ioffset
            WRITE(6,910) tau,IGP, ZLHS(i), ZRHS(i),
     &        RADT(i,1), RADT(i,2),CHF(i), SHF(i),   
     &        HFLUX(i), ECT(i), ECI(i), EGI(i), EGS(i),
     &        tm,tc(i),tg(i),tgs(i)
            write(6,911)(td(i,n),n=nsoil,1,-1)
            write(6,912) HC(i), HG(i), HEATEN(i), CBAL(i), GBAL(i) 
            WRITE(6,901)  TOTWB(i), ENDWB(i), ERRORW(i),                      
     &                WWW(i,1), WWW(i,2), WWW(i,3),                    
     &                CAPAC(i,1), CAPAC(i,2),snoww(i,1),snoww(i,2),                        
     &                pcmeter(i),plmeter(i), EMETER(i), ROFF(i)
      enddo        
      endif  

900   FORMAT(//,10X,'** WARNING: WATER BALANCE VIOLATION **  ',//,     
     & /,1X,'TAU ', F10.2,' AT SIB POINT (I) = ',I4,                                               
     &/,1X,'BEGIN, END, DIFF  ', 2(F10.7,1X),g13.5,                           
     &/,1X,'WWW,1-3           ', 3(F10.8,1X),                           
     &/,1X,'CAPAC1-2,snow 1-2 ', 4(g13.5,1X),                           
     &/,1X,'PPc,PPl, ET, ROFF ', 4(g13.5,1X) )                          
910   FORMAT(//,10X,'** WARNING: ENERGY BALANCE VIOLATION **',//,      
     & /,1X,'TAU ', F10.2,' AT SIB POINT (I) = ',I4,                                               
     & /,1X,'RHS, LHS              ', 2G13.5,                          
     & /,1X,'RN1, RN2, CHF, SHF, H ', 5G13.5,                          
     & /,1X,'ECT, ECI, EGI, EGS    ', 4G13.5,                          
     & /,1X,'TM,  TC,  TG, TGS     ', 4G13.5,                          
     & /,1X,'HC        HG          ',  G12.5, 12X, G12.5,              
     & /,1X,'HEATEN, C-BAL, G-BAL  ', 3G13.5 )
911   FORMAT(1x,'TD ', 5G13.5)                          
912   FORMAT(1X,'HC        HG          ',  G12.5, 12X, G12.5,              
     & /,1X,'HEATEN, C-BAL, G-BAL  ', 3G13.5 )
901   FORMAT(10X,'WATER BALANCE'     
     &/,1X,'BEGIN, END, DIFF  ', 2(F10.7,1X),g13.5,                           
     &/,1X,'WWW,1-3           ', 3(F10.8,1X),                           
     &/,1X,'CAPAC1-2,snow 1-2 ', 4(g13.5,1X),                           
     &/,1X,'PPc,PPl, ET, ROFF ', 4(g13.5,1X) )                          
C                                                                      
      RETURN                                                           
      END                                                              
C
C================SUBROUTINE BEGTEM=======================================
C
      SUBROUTINE begtem(tc,tg,cpair,hlat,psur,snomel
     ^,               zlt,clai,cw,www,poros,pie, psy
     ^,               phsat,bee,ccx,cg, phc
     ^,               tgs,etc,etgs,getc,getgs,rstfac
     ^,               rsoil,hr,wc,wg,snoww,capac
     ^,               areas,satcap,csoil,tf,g
     ^,               snofac,len,nlen, forcerestore )

      use kinds

      implicit none

c========================================================================
C
C     Calculation of flux potentials and constants prior to heat 
C         flux calculations.  Corresponds to first half of TEMREC
C         in 1D model.
C
c======================================================================== 


c++++++++++++++++++++++++++++++OUTPUT+++++++++++++++++++++++++++++++++++
c
C       RSTFAC(2)      SOIL MOISTURE STRESS FACTOR 
C       RSOIL          SOIL SURFACE RESISTANCE (S M-1)
C       HR             SOIL SURFACE RELATIVE HUMIDITY
C       WC             CANOPY WETNESS FRACTION
C       WG             GROUND WETNESS FRACTION
C       CCX            CANOPY HEAT CAPACITY (J M-2 K-1)
C       CG             GROUND HEAT CAPACITY (J M-2 K-1)
C
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      integer (kind=int_kind), intent(in) ::
     *     len, nlen
      logical (kind=log_kind), intent(in) ::
     *     forcerestore
      real (kind=dbl_kind), intent(in), dimension(len) ::
     ^     tc,tg,psur,
     ^     zlt,poros,phsat,
     ^     bee,
     ^     areas,
     ^     phc 
      real (kind=dbl_kind), intent(in), dimension(len,2) ::
     *     satcap
      real (kind=dbl_kind), intent(in), dimension(len,3) ::
     *     www
      real (kind=dbl_kind), intent(in), dimension(nlen,2) ::
     ^     snoww,capac
      real (kind=dbl_kind), intent(out), dimension(len) ::
     ^     ccx,cg,tgs,etc,
     ^     etgs,getgs,rsoil,
     ^     hr,wc,wg,csoil,
     ^     getc, psy 
      real (kind=dbl_kind), intent(out), dimension(len,4) ::
     *     rstfac
 
      real (kind=dbl_kind), intent(in) ::
     *     cpair, snofac, snomel, cw, tf, g, clai, pie, hlat
c     local variables
      integer (kind=int_kind) :: i  
      real (kind=dbl_kind) ::
     *     e, ge, slamda, shcap, x, tsnow, rsnow, fac, psit, argg 
     *,    shcapf, shcapu
      real (kind=dbl_kind), dimension(len) ::
     *     phroot

C----------------------------------------------------------------------
C     E(X) IS VAPOUR PRESSURE IN MBARS AS A FUNCTION OF TEMPERATURE
C     GE(X) IS D E(X) / D ( TEMP )
C----------------------------------------------------------------------

      E(X) = EXP( 21.18123 - 5418. / X ) / .622
      GE(X) = EXP( 21.18123 - 5418. / X ) * 5418.
     $        / (X*X) / .622


C----------------------------------------------------------------------
C     CALCULATION OF CANOPY AND GROUND HEAT CAPACITIES.
C     N.B. THIS SPECIFICATION DOES NOT NECESSARILY CONSERVE ENERGY WHEN
C     DEALING WITH VERY LARGE SNOWPACKS.
C     HEAT CAPACITY OF THE SOIL, AS USED IN FORCE-RESTORE HEAT FLUX
C     DESCRIPTION. DEPENDENCE OF CSOIL ON POROSITY AND WETNESS.
C
C      CG          (cg)    : EQUATION (?) , CS-81
C----------------------------------------------------------------------
      if(forcerestore) then
        do i = 1,len
        SLAMDA = ( 1.5*(1.-POROS(i)) +
     &            1.3*WWW(i,1)*POROS(i) ) /
     &           ( 0.75 + 0.65*POROS(i) -
     &            0.4*WWW(i,1)*POROS(i) )
     &                    * 0.4186
        SHCAP  = ( 0.5*(1.-POROS(i)) + WWW(i,1)*  
     &                     POROS(i)) * 4.186E6
        CSOIL(i)  = SQRT( SLAMDA * SHCAP * 86400./PIE ) * 0.5
 
c950511 adjust for different heat capacity of snow          
c       CCX(i) = ZLT(i) * CLAI +
c    &             (0.5*SNOWw(i,1)+CAPAC(i,1))*CW
        CCX(i) = ZLT(i)*CLAI+
     &            (SNOWw(i,1)+CAPAC(i,1))*CW
c950511 adjust for different heat capacity of snow          
c       CG(i)  = CSOIL(i) + 
c    *       MIN ( 0.025_dbl_kind, (0.5 *SNOWw(i,2)+CAPAC(i,2))) * CW
        CG(i)  = CSOIL(i) + 
     *       MIN ( 0.05_dbl_kind, (SNOWw(i,2)+CAPAC(i,2))) * CW
        enddo
      else
        do i = 1,len
C
c--------------------
c   new calculation for ground heat capacity cg - no longer force-restore
c      now for the top 1cm of snow and soil, with phase change in the soil
c      incorporated into the heat capacity from +0.5C to -0.5C

        SHCAPu  = ( 0.5*(1.-POROS(i)) + WWW(i,1)*  
     &                     POROS(i)) * 4.186E6
        shcapf =  0.5 * (1. + poros(i) * (www(i,1)-1.0)) * 4.186E6
        if(tg(i).ge.tf+0.5) then
           csoil(i) = shcapu * 0.02
        else if(tg(i).le.tf-0.5) then
           csoil(i) = shcapf * 0.02
        else
           csoil(i) = (0.5*(shcapu+shcapf) + 
     &              snomel*poros(i)*www(i,1) ) * 0.02
        endif
 
        CCX(i) = ZLT(i)*CLAI+
     &            (0.5*SNOWw(i,1)+CAPAC(i,1))*CW
        CG(i)  = (1.-areas(i))*CSOIL(i) + 
     &                 cw * (capac(i,2) + 0.01 * areas(i))
C
        enddo
      endif
      do i = 1,len
c       HLAT(i)  = ( 3150.19 - 2.378 * TM(i) ) * 1000. !use constant passed in
        PSY(i)      = CPAIR / HLAT * PSUR(i) / .622
C
C----------------------------------------------------------------------
C      Calculation of ground surface temperature and wetness fractions
C        
C----------------------------------------------------------------------
C
        TSNOW = MIN ( TF-0.01, TG(i) )
        RSNOW = SNOWw(i,2) /
     &             (SNOWw(i,2)+CAPAC(i,2)+1.E-10)
C 
        TGS(i) = TSNOW*AREAS(i) + TG(i)*(1.-AREAS(i))
C
        ETC(i)   = E(TC(i))
        ETGS(i)  = E(TGS(i))
        GETC(i)  = GE(TC(i))
        GETGS(i) = GE(TGS(i))
C
        WC(i) = MIN( 1.0_dbl_kind,( CAPAC(i,1) +
     &                   SNOWw(i,1))/SATCAP(i,1) )
        WG(i) = MAX( 0.0_dbl_kind,  CAPAC(i,2)/SATCAP(i,2) )*0.25

C-----------------------------------------------------------------------
C     CALCULATION OF SOIL MOISTURE STRESS FACTOR.
C     AVERAGE SOIL MOISTURE POTENTIAL IN ROOT ZONE (LAYER-2) USED AS
C     SOURCE FOR TRANSPIRATION.
C
C      PHROOT      (PSI-R) : EQUATION (48) , SE-86
C      RSTFAC(2)  F(PSI-L) : MODIFICATION OF EQUATION (12), SE-89
C-----------------------------------------------------------------------
C
        PHROOT(i) = PHSAT(i) *
     &         MAX( 0.02_dbl_kind, WWW(i,2) ) ** ( - BEE(i) )
        PHROOT(i) = MAX ( PHROOT(i), -2.E3_dbl_kind )
        RSTFAC(i,2) = 1./( 1. + EXP( 0.02*( PHC(i)-PHROOT(i)) ))
        RSTFAC(i,2) = MAX( 0.0001_dbl_kind, RSTFAC(i,2) )
        RSTFAC(i,2) = MIN( 1.0_dbl_kind,     RSTFAC(i,2) )
C
C----------------------------------------------------------------------
C
C      RSOIL FUNCTION FROM FIT TO FIFE-87 DATA.  Soil surface layer
C         relative humidity.
C
C      RSOIL      (RSOIL) : HEISER 1992 (PERSONAL COMMUNICATION)
C      HR         (Fh)    : EQUATION (66) , SE-86
C----------------------------------------------------------------------
C
        FAC = MIN( WWW(i,1), 1.0_dbl_kind )
        FAC = MAX( FAC, 0.02_dbl_kind  )
c
c Collatz-Bounoua change rsoil to  FIFE rsoil formulation from eq(19) SE-92
c
ccbl     RSOIL(i) =  MAX (0.1_dbl_kind, 694. - FAC*1500.) + 23.6 
         rsoil(i) =  exp(8.206 - 4.255 * fac)      ! lahouari
 
C
        PSIT = PHSAT(i) * FAC ** (- BEE(i) )
        ARGG = MAX(-10.0_dbl_kind,(PSIT*G/ (461.5*TGS(i)) ))
        HR(i) = EXP(ARGG)
      enddo
      return
      end
C
C=====================SUBROUTINE CYCALC=================================
C

      SUBROUTINE CYCALC( APARKK, VM, ATHETA, BTHETA, par,
     &                   GAMMAS, RESPC, RRKK, OMSS, C3, C4,
     &                   PCO2I, ASSIMN, assim, len )

      use kinds

      implicit none
C
C=======================================================================
C
C     CALCULATION EQUIVALENT TO STEPS IN FIGURE 4 OF SE-92A
C     C4 CALCULATION BASED ON CO-92.
C 
C=======================================================================
C 

c++++++++++++++++++++++++++++++OUTPUT+++++++++++++++++++++++++++++++++++
c
C       PCO2I          CANOPY INTERNAL CO2 CONCENTRATION (MOL MOL-1)
C       GSH2O          CANOPY CONDUCTANCE (MOL M-2 S-1)
C       H2OS           CANOPY SURFACE H2O CONCENTRATION (MOL MOL-1)
C
c++++++++++++++++++++++++++DIAGNOSTICS++++++++++++++++++++++++++++++++++
C
C       OMC            RUBISCO LIMITED ASSIMILATION (MOL M-2 S-1)
C       OME            LIGHT LIMITED ASSIMILATION (MOL M-2 S-1)
C       OMS            SINK LIMITED ASSIMILATION (MOL M-2 S-1)
C       CO2S           CANOPY SURFACE CO2 CONCENTRATION (MOL MOL-1)
c
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      integer (kind=int_kind), intent(in) :: len
      real (kind=dbl_kind), intent(in), dimension(len) ::
     *     aparkk,vm,atheta,
     *     btheta,gammas,par,
     *     respc,rrkk,omss,
     *     c3,c4,pco2i
      real (kind=dbl_kind), intent(out), dimension(len) ::
     *     assimn,assim

c    local variables
      real (kind=dbl_kind) :: ome, omc, omp, oms, sqrtin
      integer (kind=int_kind) :: i
 
C-----------------------------------------------------------------------
C     CALCULATE ASSIMILATION RATE
C
C      OMC         (OMEGA-C): EQUATION (11) , SE-92A
C      OME         (OMEGA-E): EQUATION (12) , SE-92A
C      OMS         (OMEGA-S): EQUATION (13) , SE-92A
C      ASSIMN      (A-N)    : EQUATION (14,15), SE-92A
C-----------------------------------------------------------------------

      do i = 1,len
        OMC = VM(i) *(PCO2I(i)-GAMMAS(i))/(PCO2I(i) + RRKK(i))*C3(i)
     &	                  + VM(i) * C4(i)
        OME = PAR(i)*(PCO2I(i)-GAMMAS(i))/(PCO2I(i)+2.*GAMMAS(i))*C3(i) 
     &	                 + PAR(i) * C4(i)
        SQRTIN= MAX( 1.e-36_dbl_kind, ( (OME+OMC)**2 - 4.*ATHETA(i)*OME*OMC ) )
        OMP  = ( ( OME+OMC ) - SQRT( SQRTIN ) ) / ( 2.*ATHETA(i) )
        OMS  = OMSS(i) * C3(i) + OMSS(i)*PCO2I(i) * C4(i)
        SQRTIN= MAX( 1.e-36_dbl_kind, ( (OMP+OMS)**2 - 4.*BTHETA(i)*OMP*OMS ) )
        ASSIM(i) = ( ( OMS+OMP ) - SQRT( SQRTIN ) ) /
     &                       ( 2.*BTHETA(i) )
        ASSIMN(i)= ( ASSIM(i) - RESPC(i)) * APARKK(i)
 
      enddo
C
      RETURN
      END
C
C==================SUBROUTINE DELEF======================================
C
      SUBROUTINE DELEF(DTA,CP,ps,sh,ros,wc,wg,fc,fg,HRr   
     *,                rst,ra,rb,rd,rsoil,etc,etg,em,btc
     *,                btg,rc,ea,ec,egs,EGDTG,EGDQM,ECDTG,ECDTC 
     *,                ECDQM,EGDTC,DEADTC,DEADTG,DEADQM,BBG,BBC,BBM
     *,                areas,cogr,cogs, psy, hr
     *,                len )    
c========================================================================
C
C     Calculation of partial derivatives of canopy and ground latent
C        heat fluxes with respect to Tc, Tgs, Theta-m, and Qm.
C     Calculation of initial latent heat fluxes.
C
c======================================================================== 

c++++++++++++++++++++++++++++++OUTPUT+++++++++++++++++++++++++++++++++++
c
C       EC             ECT + ECI
C       EG             EGS + EGI
C       ECDTC          dEC/dTC
C       ECDTG          dEC/dTGS
C       ECDQM          dEC/dQM
C       EGDTC          dEG/dTC
C       EGDTG          dEG/dTGS
C       EGDQM          dEG/dQM
C       BBC            dE/dTC
C       BBG            dE/dTGS
C       BBM            dE/dQM
C
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      use kinds

      IMPLICIT none
C         
      integer (kind=int_kind), intent(in) :: len
      REAL (kind=dbl_kind), intent(in) :: dta, cp
      REAL (kind=dbl_kind), intent(in), dimension(len) ::
     *     wc        
     *,    wg        
     *,    fc,fg  
     *,    rst,rb          
     *,    rsoil,rd      
     *,    ra,em,etc,etg            
     *,    ps,btc 
     *,    btg,sh 
     *,    hr     
     *,    ros, psy
     *,    areas
      REAL (kind=dbl_kind), intent(out), dimension(len) ::
     *     EGDTG,EGDQM,ECDTG        
     *,    ECDTC,ECDQM,EGDTC        
     *,    BBG,BBC,BBM  
     *,    hrr,rc          
     *,    ea,ec,egs 
     *,    DEADTC,DEADTG     
     *,    DEADQM
     *,    cogr,cogs

!  local variables
      REAL (kind=dbl_kind), dimension(len) ::
     *    cpdpsy, cog1, cog2, coc, d2
      integer (kind=int_kind) :: i
C                                                                       
C     MODIFICATION FOR SOIL DRYNESS : HR=REL. HUMIDITY IN TOP LAYER     
C                                                                       
      DO I=1,len                                                   
         hrr(I) = HR(I)                                                 
         if(fg(i).lt.0.5_dbl_kind) hrr(i) = 1.0
      enddo                              

      DO I=1,len

        cpdpsy(i) = cp / psy(i)

C-----------------------------------------------------------------------
C                                                                       
C     CALCULATION OF SURFACE RESISTANCE COMPONENTS, SEE EQUATIONS (64,66)
C       OF SE-86                                   
C                                                                       
C-----------------------------------------------------------------------

        rc(I) = rst(i)      * fc(I) + 2.*rb(I)       
        COC(I) = (1.0  - wc(I))/rc(I)+wc(I) / (2.0 * rb(I))
        COGR(i) = (1.-wg(i))/(rsoil(i)*fg(i)+rd(i))                                     
        COGS(i) =  wg(i)/rd(i)                                                    
        COG1(i) = (COGS(i) + COGR(i)*hrr(i))
     ^             * (1.-AREAS(i)) + AREAS(i)/rd(i)                 
        COG2(i) = (COGS(i) + COGR(i)    ) 
     ^           * (1.-AREAS(i)) + AREAS(i)/rd(i)
        D2(I)  = 1.0 / ra(I) + COC(I) + COG2(I)                          
        ea(i) = (COC(I) * etc(I) + COG1(I) * etg(I)                    
     *         + em(i)  / ra(I)) / D2(i)                                

C-----------------------------------------------------------------------
C                                                                       
C     FLUXES EXPRESSED IN JOULES M-2                                    
C                                                                       
C      ec         (EC)    : EQUATION (64) , SE-86
C      egs         (EGS)   : EQUATION (66) , SE-86
C-----------------------------------------------------------------------

        ec(I) = (etc(I) - ea(i)) * COC(I) *
     *                  ros(i)  * dta * cpdpsy(i)    
        egs(I) = (etg(I) * COG1(I) - ea(i) * COG2(I)) *         
     *                  ros(i) * dta * cpdpsy(i)                                             

        DEADTC(I) = btc(I) * COC(I)  / D2(i)                             
        DEADTG(I) = btg(I) * COG1(I) / D2(i)                             
        DEADQM(I) = (0.622  + sh(i)) * (0.622 + sh(i))                  
        DEADQM(I) = 0.622   * ps(i)  / DEADQM(I)                         
     *            * 1.0     / (ra(I) * D2(i))                            

C-----------------------------------------------------------------------
C      ECDTC     (dEC/dTC) : EQUATION (14) , SA-89B
C      ECDTG     (dEC/dTGS): EQUATION (14) , SA-89B
C      ECDQM     (dEC/dQM) : EQUATION (14) , SA-89B
C      EGDTC     (dEG/dTC) : EQUATION (15) , SA-89B
C      EGDTG     (dEG/dTGS): EQUATION (15) , SA-89B
C      EGDQM     (dEG/dQM) : EQUATION (15) , SA-89B
C      BBC       (dE/dTC)  : EQUATION (13) , SA-89B
C      BBG       (dE/dTGS) : EQUATION (13) , SA-89B
C      BBM       (dE/dQM)  : EQUATION (13) , SA-89B
C-----------------------------------------------------------------------

        ECDTC(I) = (btc(I) - DEADTC(I)) * COC(I) * ros(i) *
     &                                          CPDPSY(i)   
        ECDTG(I) = -DEADTG(I) * COC(I) * ros(i) * CPDPSY(i)             
        ECDQM(I) = -DEADQM(I) * COC(I) * ros(i) * CPDPSY(i)          
C                                                                       
        EGDTG(I) = (btg(I) * COG1(I) - DEADTG(I) * COG2(I)) *  
     *            ros(i) * CPDPSY(i)                                                 
        EGDTC(I) = -DEADTC(I) * COG2(I) * ros(i) * CPDPSY(i)                
        EGDQM(I) = -DEADQM(I) * COG2(I) * ros(i) * CPDPSY(i) 
C                                                                       
        BBG(I) = (COG1(I) / D2(i))
     *          * btg(I) * 0.622 * ps(i)           
     *       / ((ps(i) - etg(I)) * (ps(i) - etg(I)))                
        BBC(I) = (COC(I)  / D2(i))
     *            * btc(I) * 0.622 * ps(i)           
     *       / ((ps(i) - etc(I)) * (ps(i) - etc(I)))                
        BBM(I) = 1.0   / (ra(I)  * D2(i))                             
C                                                                       
      enddo                                                          
C                                                                       
      RETURN                                                            
      END                                                               
C
C===================SUBROUTINE DELHF=====================================
C

      SUBROUTINE DELHF(DTA,CP,bps,thm,tgs,tc,ros,ra,rb,rd  
     *,                HGDTG,HGDTC,HGDTH,HCDTG,HCDTC,HCDTH,AAG,AAC,AAM
     *,                hc, hg, len ) 

c========================================================================
C
C     Calculation of partial derivatives of canopy and ground sensible
C        heat fluxes with respect to Tc, Tgs, and Theta-m.
C     Calculation of initial sensible heat fluxes.
C
c======================================================================== 


c++++++++++++++++++++++++++++++OUTPUT+++++++++++++++++++++++++++++++++++
c
C       HC             CANOPY SENSIBLE HEAT FLUX (J M-2)
C       HG             GROUND SENSIBLE HEAT FLUX (J M-2)
C       HCDTC          dHC/dTC 
C       HCDTG          dHC/dTGS
C       HCDTH          dHC/dTH
C       HGDTC          dHG/dTC
C       HGDTG          dHG/dTGS
C       HGDTH          dHG/dTH
C       AAC            dH/dTC
C       AAG            dH/dTGS
C       AAM            dH/dTH
C
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      use kinds

      IMPLICIT none
C
      integer (kind=int_kind), intent(in) :: len
      real (kind=dbl_kind), intent(in) :: dta, cp
      real (kind=dbl_kind), intent(in), dimension(len) ::
     *     bps,thm,tgs          
     *,    ra
     *,    rb,rd,tc 
     *,    ros
      real (kind=dbl_kind), intent(out), dimension(len) ::
     *     HGDTG          
     *,    HGDTC,HGDTH,HCDTG,HCDTC      
     *,    HCDTH,AAG,AAC,AAM
     *,    hc, hg 

c     local variables
      real (kind=dbl_kind) ::
     *    D1, ta, tha, d1i, rai, rbi,  rdi
      integer (kind=int_kind) :: i                                      

C-----------------------------------------------------------------------
C                                                                       
C     FLUXES EXPRESSED IN JOULES M-2                                    
C                                                                       
C      HC          (HC)    : EQUATION (63) , SE-86
C      HG          (HGS)   : EQUATION (65) , SE-86
C-----------------------------------------------------------------------

      DO I=1,len  
        rai = 1.0 / ra(I)                                              
        rbi = 1.0 / rb(I)                                              
        rdi = 1.0 / rd(I)                                              
        D1 = rai + rbi + rdi  
        d1i = 1. / d1              
C                                                                       
        tha = ((tgs(I) * rdi + tc(i) * rbi ) / bps(i)         
     *          + thm(i) * rai ) * d1i                              
        ta  = tha  * bps(i)                                      
C                                                                       
        HC(I)   = CP * ros(i) * (tc(i) - ta) * rbi * DTA         
        HG(I)   = CP * ros(i) * (tgs(I) - ta) * rdi * DTA         

C-----------------------------------------------------------------------
C      HCDTC     (dHC/dTC) : EQUATION (14) , SA-89B
C      HCDTG     (dHC/dTGS): EQUATION (14) , SA-89B
C      HCDTH     (dHC/dTH) : EQUATION (14) , SA-89B
C      HGDTC     (dHG/dTC) : EQUATION (15) , SA-89B
C      HGDTG     (dHG/dTGS): EQUATION (15) , SA-89B
C      HGDTH     (dHG/dTH) : EQUATION (15) , SA-89B
C      AAC       (dH/dTC)  : EQUATION (12) , SA-89B
C      AAG       (dH/dTGS) : EQUATION (12) , SA-89B
C      AAM       (dH/dTH)  : EQUATION (12) , SA-89B
C-----------------------------------------------------------------------

        HCDTC(I) = CP * ros(i) * rbi  * ( rai
     *           + rdi )  * d1i                                                  
        HCDTG(I) = -CP * ros(i) / (rb(I) * rd(I)) * d1i              
        HCDTH(I) = -CP * ros(i) / (rb(I) * ra(I)) * d1i *
     *                                   bps(i)    
C                                                                       
        HGDTG(I) = CP * ros(i) * rdi  * ( rai
     *           + rbi ) * d1i                                                  
        HGDTC(I) = -CP * ros(i) / (rd(I) * rb(I)) * d1i              
        HGDTH(I) = -CP * ros(i) / (rd(I) * ra(I)) * d1i *
     *                                   bps(i)    

        AAG(I) = rdi * d1i                                    
        AAC(I) = rbi * d1i                                    
        AAM(I) = rai * d1i * bps(i)                          
C                                                                       
      ENDDO
                                                          
      RETURN                                                            
      END                                                               
C
C====================SUBROUTINE ENDTEM==================================
C
      subroutine endtem(etc,ea,etgs, btc,BTG,deadtg
     ^,             deadtc,dtc,dtg,deadqm,dqm,wc
     ^,             wg,rb,rd,dtt,ros,psy,snoww,capac
     ^,             hlat,areas,asnow,snofac
     ^,             csoil,hr,rst,fc,fg
     ^,             rsoil,zdepth,www,ccx,cg
     ^,             hc,hg,ec,eg,eci,ect
     ^,             egi,egs,chf,shf,heaten
     ^,             tgs,td,hcdtc,hcdtg
     ^,             hcdth,hgdtc,hgdtg,hgdth,dth
     ^,             tg,tf,cp, slamda, len, nsib
     ^,             forcerestore )

      use kinds

      implicit none

c========================================================================
C
C     Calculation of heat and moisture fluxes after calculation of
C       increments (dtc,dtg,dth,dqm).  Corresponds to second half of
C       TEMREC in 1D model.
C
c======================================================================== 


c++++++++++++++++++++++++++++++OUTPUT+++++++++++++++++++++++++++++++++++
c
C       ECT            CANOPY TRANSPIRATION (J M-2)
C       ECI            CANOPY INTERCEPTION LOSS (J M-2)
C       EGS            GROUND EVAPORATION (J M-2)
C       EGI            GROUND INTERCEPTION LOSS (J M-2)
C       EC             ECT + ECI
C       EG             EGS + EGI
C       HC             CANOPY SENSIBLE HEAT FLUX (J M-2)
C       HG             GROUND SENSIBLE HEAT FLUX (J M-2)
C       CHF            CANOPY HEAT STORAGE FLUX (J M-2)
C       SHF            SOIL HEAT STORAGE FLUX (J M-2)
C       FC             CANOPY DEW INDICATOR
C       FG             GROUND DEW INDICATOR
C       HEATEN         HEAT LOSS TO SNOW MELT PROCESS (J M-2)
C
c++++++++++++++++++++++++++DIAGNOSTICS++++++++++++++++++++++++++++++++++
C
C       TSNOW          SNOW TEMPERATURE (K)
c
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      integer (kind=int_kind), intent(in) ::
     *     len, nsib
      logical (kind=log_kind), intent(in) ::
     *     forcerestore

      real (kind=dbl_kind), intent(in) ::
     *      asnow, tf, snofac, cp, hlat, dtt
      real (kind=dbl_kind), intent(in), dimension(len) ::
     ^     www,
     ^     etc,ea,etgs, btc,deadtg,
     ^     dtc,deadqm,dqm,wc,
     ^     wg,rb,rd,ros,areas,
     ^     csoil,hr,rst,fc,
     ^     fg,rsoil,ccx,cg,
     ^     tgs,td,
     ^     hcdtg,hcdtc,hcdth,hgdtc,
     ^     hgdtg,hgdth,dth,tg,
     ^     BTG,deadtc, psy, 
     ^     slamda
      real (kind=dbl_kind), intent(in), dimension(len,2) ::
     ^     dtg
      real (kind=dbl_kind), intent(in), dimension(len,3) ::
     ^     zdepth
      real (kind=dbl_kind), intent(in), dimension(nsib,2) ::
     ^     snoww,capac
      real (kind=dbl_kind), intent(out), dimension(len) ::
     ^     ec,eg,eci,ect,
     ^     egi,egs,chf,shf,
     ^     heaten
      real (kind=dbl_kind), intent(inout), dimension(len) ::
     ^     hc, hg
     
c     work array
      real (kind=dbl_kind), dimension(len) ::
     *     rcp, dtgs
      real (kind=dbl_kind) ::
     *     dti, ecpot, ecidif, egpot, egidif, egit, t1
     *,    t2, aven, darea, ecf, egf, dewc, dewg, egsmax, cogs2
     *,    cogs1, hrr, coct, arean, rsnow, tsnow, timcon
      integer (kind=int_kind) :: i

      timcon = 3.1415926 / 86400.0
      dti = 1. / dtt

C
C----------------------------------------------------------------------
C


      do i = 1,len

C-----------------------------------------------------------------------
C     CALCULATION OF EVAPORATIVE POTENTIALS (ECPOT, EGPOT) AND 
C        INTERCEPTION LOSSES; FLUXES IN J M**-2.  ECIDIF AND EGIDIF
C        HOLD THE EXCESS ENERGY IF ALL INTERCEPTED WATER IS EVAPORATED
C        DURING THE TIMESTEP.  THESE ENERGIES ARE LATER ADDED TO THE
C        SENSIBLE HEAT FLUXES.
C
C      ECI         (E-WC)  : EQUATION (59) , SE-86
C      EGI         (E-WG)  : EQUATION (60) , SE-86
C-----------------------------------------------------------------------

        dtgs(i) = (1.-areas(i))*dtg(i,1) + areas(i)*dtg(i,2)
        rcp(i) = ros(i) * cp   
        TSNOW = MIN ( TF-0.01, TG(i) )
        RSNOW = SNOWw(i,2) /
     &          (SNOWw(i,2)+CAPAC(i,2)+1.E-10)
c
        ECPOT = ( (ETC(i)-EA(i)) + (  btc(i)-DEADTC(i))*DTC(i)
     &             -DEADTG(i)*dtgs(i)-DEADQM(i)*DQM(i) )
        ECI(i) = ECPOT * WC(i) /(2.*RB(i)*psy(i)) * RCP(i) * DTT
        ECIdif=MAX(0.0_dbl_kind,(ECI(i)-(SNOWw(i,1)+CAPAC(i,1))
     &                 *1.E3*hlat))
        ECI(i)   =MIN(ECI(i),
     &         ( (SNOWw(i,1)+CAPAC(i,1))*1.E3*hlat))
C
        EGPOT = ( (ETGS(i)-EA(i)) + (BTG(i)-DEADTG(i))*dtgs(i)
     &               -DEADTC(i)*DTC(i)-DEADQM(i)*DQM(i) )
        EGI(i) = EGPOT/(RD(i)*psy(i))*RCP(i)*DTT *
     &            ( WG(i)*(1.-AREAS(i)) + AREAS(i) )
        EGIdif=
     &     MAX(0.0_dbl_kind,EGI(i)-(SNOWw(i,2)+CAPAC(i,2))*1.E3*hlat)
     &       *(1.-RSNOW)
        EGIT  =
     &      MIN(EGI(i), (SNOWw(i,2)+CAPAC(i,2))*1.E3*hlat )
     &       *(1.-RSNOW)
C
C----------------------------------------------------------------------
C     CALCULATION OF INTERCEPTION LOSS FROM GROUND-SNOW. IF SNOW PATCH
C     SHRINKS, ENERGY IS TAKEN FROM EGI TO WARM EXPOSED SOIL TO TGS.
C----------------------------------------------------------------------
C
        T1 = SNOWw(i,2) - 1./ASNOW
        T2 = MAX( 0.0_dbl_kind, T1 )
        AVEN = EGI(i) - T2*hlat*1.E3/SNOFAC
        IF ( (T1-T2)*EGI(i) .GT. 0. ) AVEN = EGI(i)
        DAREA = AVEN/( (TSNOW-TG(i))*CSOIL(i)
     &               - 1./ASNOW*hlat*1.E3/SNOFAC)
        AREAN = AREAS(i) + DAREA
        EGIdif = EGIdif - MIN( 0.0_dbl_kind, AREAN )
     &              *hlat*1.E3/(asnow*SNOFAC)*RSNOW
        DAREA = MAX( DAREA, -AREAS(i) )
        DAREA = MIN( 1.-AREAS(i), DAREA )
        HEATEN(i) = (TSNOW-TG(i))*CSOIL(i)*DAREA*RSNOW
        EGIT = EGIT + ( EGI(i) - HEATEN(i) - EGIdif )*RSNOW
        EGI(i) = EGIT
C
C----------------------------------------------------------------------
C     CALCULATION OF TRANSPIRATION AND SOIL EVAPORATION FLUXES FOR THE
C        END OF THE TIMESTEP. SEE FIGURE (2) OF SE-86.
C
C      ECT         (E-C)   : EQUATION (64) , SE-86
C      EGS         (E-S)   : EQUATION (66) , SE-86
C----------------------------------------------------------------------
C
        HRR = HR(i)
        IF ( FG(i) .LT. .5 ) HRR = 1.
C
        COCT = (1.-WC(i))/ ( RST(i)*FC(i) + 2.*RB(i) )
        COGS1 = (1.-AREAS(i))/(RD(i)+RSOIL(i)*FG(i))*(1.-WG(i))*HRR
        COGS2 = COGS1 / HRR
C
        ECT(i) = ECPOT * COCT * RCP(i)/psy(i) * DTT
        EGS(i) = (ETGS(i) + BTG(i)*dtgs(i) ) * COGS1
     &      - ( EA(i) + DEADTG(i)*dtgs(i) + DEADTC(i)*DTC(i)
     &      + DEADQM(i)*DQM(i) ) * COGS2
        EGS(i) = EGS(i) * RCP(i)/psy(i) * DTT
        EGSMAX = WWW(i) * ZDEPTH(i,1) * hlat * 1.e3 * 0.5
        EGIdif = EGIdif + MAX( 0.0_dbl_kind, EGS(i) - EGSMAX )
        EGS(i) = MIN ( EGS(i), EGSMAX )
C    
C----------------------------------------------------------------------
C     CALCULATION OF SENSIBLE HEAT FLUXES FOR THE END OF THE TIMESTEP.
C        SEE FIGURE (2) OF SE-86.  NOTE THAT INTERCEPTION LOSS EXCESS
C        ENERGIES (ECIDIF, EGIDIF) ARE ADDED.
C
C      HC          (HC)    : EQUATION (63) , SE-86
C      HG          (HGS)   : EQUATION (65) , SE-86
C----------------------------------------------------------------------
C
        HC(i) = HC(i) + (HCDTC(i)*DTC(i) + HCDTG(i)*dtgs(i)
     &                + HCDTH(i)*DTH(i))*DTT + ECIdif
        HG(i) = HG(i) + (HGDTC(i)*DTC(i) + HGDTG(i)*dtgs(i)
     &                + HGDTH(i)*DTH(i))*DTT + EGIdif
C
C----------------------------------------------------------------------
C     TEST OF DEW CONDITION. LATENT HEAT FLUXES SET TO ZERO IF SIGN
C     OF FLUX CHANGES OVER TIME STEP.EXCESS ENERGY DONATED TO SENSIBLE
C     HEAT FLUX.
C     CALCULATION OF TOTAL LATENT HEAT FLUXES,  SEE FIGURE (2), SE-86.
C
C      EC          (EC)    : EQUATION (63) , SE-86
C      EG          (EG)    : EQUATION (65) , SE-86
C----------------------------------------------------------------------
C
        ECF = SIGN( 1.0_dbl_kind, ECPOT )
        EGF = SIGN( 1.0_dbl_kind, EGPOT )
        DEWC = FC(i) * 2. - 1.
        DEWG = FG(i) * 2. - 1.
C
        IF(DEWC*ECF.le.0.0_dbl_kind) then
          HC(i) = HC(i) + ECI(i) + ECT(i)
          ECI(i) = 0.
          ECT(i) = 0.
	endif
        IF(DEWG*EGF.le.0.0_dbl_kind) then
          HG(i) = HG(i) + EGS(i) + EGI(i)
          EGS(i) = 0.
          EGI(i) = 0.
	endif
C
        EC(i) = ECI(i) + ECT(i)
        EG(i) = EGI(i) + EGS(i)

C----------------------------------------------------------------------
C     CALCULATION OF STORAGE HEAT FLUXES
C
C---------------------------------------------------------------------- 
C
        CHF(i) = CCX(i) * dti * DTC(i)
      enddo

      if(forcerestore) then
         do i = 1,len
           SHF(i) = dti * ( (1.-areas(i))*dtg(i,1) + 
     &                         areas(i)*dtg(i,2) ) * cg(i)
     &           + TIMCON*csoil(i)*2. *( TGS(i)+dtgs(i) - TD(i) )
         enddo
      else
        do i = 1,len
c  new soil thermodynamic model         
           SHF(i) = dti * ( (1.-areas(i))*dtg(i,1) + 
     &                         areas(i)*dtg(i,2) ) * cg(i)
     &           + slamda(i) *( TGS(i)+dtgs(i) - TD(i) )
         enddo
      endif
C

      RETURN
      END
C
C===================SUBROUTINE GAUSS=====================================
C
      SUBROUTINE GAUSS(len,WORK,N,NP1,X)   
    
      use kinds
                      
      IMPLICIT none
c========================================================================
C
C     SOLVE A LINEAR SYSTEM BY GAUSSIAN ELIMINATION.  DEVELOPED BY      
C     DR. CHIN-HOH MOENG.  A IS THE MATRIX OF COEFFICIENTS, WITH THE    
C     VECTOR OF CONSTANTS APPENDED AS AN EXTRA COLUMN.  X IS THE VECTOR 
C     CONTAINING THE RESULTS.  THE INPUT MATRIX IS NOT DESTROYED.       
C
c======================================================================== 
C          
      integer (kind=int_kind), intent(in) ::
     *       len, n, np1 
      real (kind=selected_real_kind(13)), intent(inout), dimension(len,n,np1) ::
     *       WORK
      real (kind=selected_real_kind(13)), intent(out), dimension(len,n) ::
     *       X

c     local variables
      real (kind=selected_real_kind(13)), dimension(len,2) :: TEMV
      integer (kind=int_kind) :: ii, j, i, k, kk, l                
C                                                                       
      DO 20 II=2,N                                                      
      DO 20 J=II,N                                                      
      DO 10 I=1,len                                                   
      TEMV(I,1) = WORK(I,J,II-1) / WORK(I,II-1,II-1)                    
   10 CONTINUE                                                          
      DO 20 K=1,NP1                                                     
      DO 20 I=1,len                                                   
      WORK(I,J,K) = WORK(I,J,K) - TEMV(I,1) * WORK(I,II-1,K)            
   20 CONTINUE                                                          
C                                                                       
      DO 45 K=N,2,-1                                                    
      DO 30 I=1,len                                                   
      TEMV(I,1) = WORK(I,K,NP1) / WORK(I,K,K)                           
   30 CONTINUE                                                          
      KK = K-1                                                          
      DO 45 L=KK,1,-1                                                   
      DO 40 I=1,len                                                   
      TEMV(I,2) = TEMV(I,1) * WORK(I,L,K)                               
   40 CONTINUE                                                          
      DO 45 I=1,len                                                   
      WORK(I,L,NP1) = WORK(I,L,NP1) - TEMV(I,2)                         
   45 CONTINUE                                                          
C                                                                       
      DO 50 II=1,N                                                      
      DO 50 I=1,len                                                   
      X(I,II) = WORK(I,II,NP1) / WORK(I,II,II)                          
   50 CONTINUE                                                          
C                                                                       
      RETURN                                                            
      END                                                               
C
C=====================SUBROUTINE INTER2=================================
C
      SUBROUTINE INTER2(ppc,ppl,snoww,capac,www
     ^,   pie,satcap,cw,tc,tg,clai,zlt,chil,roff
     ^,   snomel,zdepTH,tm,tf,asnow,csoil,satco,dtt,vcover,roffo
     ^,   zmelt, len, nsib)

      use kinds

      implicit none

      integer (kind=int_kind), intent(in) ::
     *     len, nsib
      real (kind=dbl_kind), intent(in) ::
     *     tf, asnow, clai, dtt, pie, cw, snomel
      real (kind=dbl_kind), intent(in), dimension(len) ::
     *     ZDEPTH,  
     *     zlt, chil,
     *     csoil, satco,
     *     vcover                                               
      real (kind=dbl_kind), intent(inout), dimension(len) ::
     *     ppl, ppc, tg, tm, roff, tc
      real (kind=dbl_kind), intent(in), dimension(len,2) ::
     *     satcap
      real (kind=dbl_kind), intent(out), dimension(len) ::
     *     roffo, zmelt                                               
      real (kind=dbl_kind), intent(inout), dimension(nsib,2) ::
     *     snoww, capac
      real (kind=dbl_kind), intent(inout), dimension(len,3) ::
     *     WWW

C=======================================================================
C
C     CALCULATION OF  INTERCEPTION AND DRAINAGE OF RAINFALL AND SNOW
C     INCORPORATING EFFECTS OF PATCHY SNOW COVER AND TEMPERATURE
C     ADJUSTMENTS.
C
C----------------------------------------------------------------------
C
C     (1) NON-UNIFORM PRECIPITATION
C         CONVECTIVE PPN. IS DESCRIBED BY AREA-INTENSITY
C         RELATIONSHIP :-
C
C                   F(X) = A*EXP(-B*X)+C
C
C         THROUGHFALL, INTERCEPTION AND INFILTRATION
C         EXCESS ARE FUNCTIONAL ON THIS RELATIONSHIP
C         AND PROPORTION OF LARGE-SCALE PPN.
C         REFERENCE: SA-89B, APPENDIX.
C
C     (2) REORGANISATION OF SNOWMELT AND RAIN-FREEZE PROCEDURES.
C               SUBROUTINE ADJUST
C
C     (3) ADDITIONAL CALCULATION FOR PARTIAL SNOW-COVER CASE.
C               SUBROUTINE PATCHS
C
C     (4) REORGANISATION OF OVERLAND FLOW.
C         REFERENCE: SA-89B, APPENDIX.
C
C     (5) MODIFIED CALCULATION OF SOIL HEAT CAPACITY AND
C         CONDUCTIVITY.
C
C=======================================================================
C      1D MODEL SUBROUTINE PATCHS INLINED.
C----------------------------------------------------------------------
C

c++++++++++++++++++++++++++++++OUTPUT+++++++++++++++++++++++++++++++++++
c
C       ROFF           RUNOFF (MM)
C       TC             CANOPY TEMPERATURE (K)
C       TG             GROUND SURFACE TEMPERATURE (K)
C       WWW(1)         GROUND WETNESS OF SURFACE LAYER 
C       CAPAC(2)       CANOPY/GROUND LIQUID INTERCEPTION STORE (M)
C       SNOWW(2)       CANOPY/GROUND SNOW INTERCEPTION STORE (M)
C
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

c     local variables
      real (kind=dbl_kind) :: 
     *      bp, totalp,
     *      realc, realg, xss, xsc, chiv, aa, bb,
     *      exrain, zload, xs, arg,
     *      equdep,
     *      dcap, tsd, ex, dareas, rhs, areas, snowhc
      real (kind=dbl_kind), dimension(len) :: 
     *      ap, cp, thru,
     *      fpi,
     *      tex, tti, snowwp,
     *      capacp, spechc, pinf, ts,
     *      p0
      real (kind=dbl_kind), dimension(2,2) :: 
     *      PCOEFS(2,2)
      integer (kind=int_kind) :: i, iveg

      DATA PCOEFS(1,1)/ 20. /, PCOEFS(1,2)/ .206E-8 /,
     &     PCOEFS(2,1)/ 0.0001 /, PCOEFS(2,2)/ 0.9999 /, BP /20. /
C
C-----------------------------------------------------------------------
C
C     PREC ( PI-X )   : EQUATION (C.3), SA-89B
C
C-----------------------------------------------------------------------
C
      do i = 1,len
         roffo(i) = 0.0
         zmelt(i) = 0.0
         AP(i) = PCOEFS(2,1)
         CP(i) = PCOEFS(2,2)
         TOTALP = (PPC(i) + PPL(i)) * dtt 
         IF( SNOWW(i,1) .GT. 0. .OR. SNOWW(i,2) .GT. 0. 
     &            .OR. TM(i) .LT. TF ) PPC(i) = 0.
         PPL(i) = TOTALP/dtt - PPC(i)
         IF(TOTALP.ge.1.E-8) then
            AP(i) = PPC(i)*dtt/TOTALP * PCOEFS(1,1) +
     &                PPL(i)*dtt/TOTALP * PCOEFS(2,1)
            CP(i) = PPC(i)*dtt/TOTALP * PCOEFS(1,2) +
     &                PPL(i)*dtt/TOTALP * PCOEFS(2,2)
         endif
C
         THRU(i) = 0.
         FPI(i)  = 0.

C
C
C----------------------------------------------------------------------
C     INPUT PRECIPITATION IS GIVEN IN MM, CONVERTED TO M TO GIVE P0.
C----------------------------------------------------------------------
C
C
         P0(i) = TOTALP
      enddo

      DO 1000 IVEG = 1, 2
         REALC = 2. - IVEG                                                
         REALG = IVEG - 1.                                                
C 
         do i = 1,len                                                                     
C
            XSC = MAX(0.0_dbl_kind, CAPAC(i,IVEG) - SATCAP(i,IVEG) )
            CAPAC(i,IVEG) = CAPAC(i,IVEG) - XSC
            XSS = MAX(0.0_dbl_kind, SNOWW(i,IVEG) - SATCAP(i,IVEG) ) * REALC
            SNOWW(i,IVEG) = SNOWW(i,IVEG) - XSS
            ROFF(i) = ROFF(i) + XSC + XSS

            CAPACP(i) = CAPAC(i,IVEG)                                             
            SNOWWP(i) = SNOWW(i,IVEG)
C
            SPECHC(i) = 
     &          MIN( 0.05_dbl_kind, ( CAPAC(i,IVEG) + SNOWW(i,IVEG) ) ) * CW
     &          + REALC * ZLT(i) * CLAI + REALG * CSOIL(i)
            TS(i) = TC(i) * REALC + TG(i) * REALG 
C
C----------------------------------------------------------------------
C    PROPORTIONAL SATURATED AREA (XS) AND LEAF DRAINAGE(TEX)
C
C     TTI ( D-D )     : EQUATION (C.4), SA-89B
C     XS  ( X-S )     : EQUATION (C.7), SA-89B
C     TEX ( D-C )     : EQUATION (C.8), SA-89B
C
C-----------------------------------------------------------------------
C
            CHIV = CHIL(i)
            IF ( ABS(CHIV) .LE. 0.01 ) CHIV = 0.01
            AA = 0.5 - 0.633 * CHIV - 0.33 * CHIV * CHIV
            BB = 0.877 * ( 1. - 2. * AA )
            EXRAIN = AA + BB
C
            ZLOAD = CAPAC(i,IVEG) + SNOWW(i,IVEG)
            FPI(i)=( 1.-EXP( - EXRAIN*ZLT(i)/VCOVER(i) ) )*
     &                  VCOVER(i)*REALC + REALG
            TTI(i) = P0(i) * ( 1.-FPI(i) )
            XS = 1.
            IF ( P0(i) .ge. 1.E-9 ) then                                    
               ARG =  ( SATCAP(i,IVEG)-ZLOAD )/
     &                   ( P0(i)*FPI(i)*AP(i) ) -CP(i)/AP(i)               
               IF ( ARG .ge. 1.E-9 ) then                                 
                  XS = -1./BP * LOG( ARG )                                      
                  XS = MIN( XS, 1.0_dbl_kind )                                            
                  XS = MAX( XS, 0.0_dbl_kind )                                           
               endif   
            endif   
            TEX(i) = P0(i)*FPI(i) * 
     &           ( AP(i)/BP*( 1.- EXP( -BP*XS )) + CP(i)*XS ) -     
     &           ( SATCAP(i,IVEG) - ZLOAD ) * XS                        
            TEX(i) = MAX( TEX(i), 0.0_dbl_kind ) 
         enddo                                     
C                                                              
C-------------------------------------------------------------
C    TOTAL THROUGHFALL (THRU) AND STORE AUGMENTATION         
C-----------------------------------------------------------
C                                                          
         IF ( IVEG .EQ. 1 ) then                        
C             
            do i = 1,len                                           
               thru(i) = TTI(i) + TEX(i)                                  
               PINF(i) = P0(i) - THRU(i)                                 
               IF( TM(i).GT.TF ) then
                  CAPAC(i,IVEG) = CAPAC(i,IVEG) + PINF(i) 
               else
                  SNOWW(i,IVEG) = SNOWW(i,IVEG) + PINF(i)               
               endif               
C                                                                    
            CALL ADJUST(Tc(i), SPECHC(i), CAPACP(i), SNOWWP(i),
     *          IVEG, capac(i,1), snoww(i,1), tm(i), tf,
     *          snomel, www(i,1), zdepth(i),
     *          satcap(i,1), cw, nsib, len )
C                                                                   
               P0(i) = THRU(i)                                                    
            enddo

         else
C 
            do i = 1,len                                                             
               IF ( TG(i) .GT. TF .AND. SNOWW(i,2) .GT. 0. ) THEN           
C                                                            
c========================================================================
c------------------PATCHS INLINED-------------------------------------
c========================================================================
C
C     CALCULATION OF EFFECT OF INTERCEPTED SNOW AND RAINFALL ON GROUND.
C     PATCHY SNOWCOVER SITUATION INVOLVES COMPLEX TREATMENT TO KEEP
C     ENERGY CONSERVED.
C
C=======================================================================

C                                                                      
C     MARGINAL SITUATION: SNOW EXISTS IN PATCHES AT TEMPERATURE TF    
C     WITH REMAINING AREA AT TEMPERATURE TG > TF.                    
C                                                                   
C------------------------------------------------------------------
C
                  PINF(i) = P0(i)                                                   
                  THRU(i) = 0.                                                  
                  SNOWHC = MIN( 0.05_dbl_kind, SNOWW(i,2) ) * CW                       
                  areas = MIN( 1.0_dbl_kind,(ASNOW*SNOWW(i,2)) )                       
                  IF( TM(i) .le. TF ) then                              
C                                                            
C-----------------------------------------------------------
C     SNOW FALLING ONTO AREA                               
C---------------------------------------------------------
C                                                        
                     RHS = TM(i)*PINF(i)*CW + TF*(SNOWHC +
     &                      CSOIL(i)*areas)      
     &                    + TG(i)*CSOIL(i)*(1.-areas)                        
                     DAREAS = MIN( ASNOW*PINF(i), ( 1.-areas ) )                       
                     EX = RHS - TF*PINF(i)*CW -
     &                    TF*(SNOWHC + CSOIL(i)*(areas + DAREAS))  
     &                    - TG(i)*CSOIL(i)*(1.-areas-DAREAS)                              
                     IF( (areas+DAREAS) .GE. 0.999 )
     &                                    TG(i) = TF - 0.01              
                     IF( EX .ge. 0. ) then                                 
C                                                               
C----------------------------------------------------------------------
C     EXCESS ENERGY IS POSITIVE, SOME SNOW MELTS AND INFILTRATES. 
C---------------------------------------------------------------------
C                                                                    
                        ZMELT(i) = EX/SNOMEL                                             
                        IF( ASNOW*(SNOWW(i,2) + PINF(i) - ZMELT(i))
     &                                                 .le. 1. ) then      
                           ZMELT(i) = 0.                                                  
                           IF( ASNOW*(SNOWW(i,2) + PINF(i)) .GE. 1. )                      
     &                           ZMELT(i) = ( ASNOW*(SNOWW(i,2) +
     &                           PINF(i)) - 1. ) / ASNOW      
                           ZMELT(i) = ( EX - ZMELT(i)*SNOMEL )/
     &                          ( SNOMEL + ASNOW*CSOIL(i)*
     &                           (TG(i)-TF) ) + ZMELT(i)                                                   
                        endif
                        SNOWW(i,2) =  SNOWW(i,2) + PINF(i) - ZMELT(i)                            
                        WWW(i,1) = WWW(i,1) + ZMELT(i)/ZDEPTH(i)
                     else                                                         
C
C----------------------------------------------------------------------
C     EXCESS ENERGY IS NEGATIVE, BARE GROUND COOLS TO TF, THEN WHOLE
C     AREA COOLS TOGETHER TO LOWER TEMPERATURE.
C----------------------------------------------------------------------
C
                        TSD = 0.
                        IF( (areas+DAREAS) .LE. 0.999 )
     &                       TSD = EX/(CSOIL(i)*( 1.-areas-DAREAS))
     &                          + TG(i)
                        IF( TSD .le. TF ) 
     &                     TSD = TF + ( EX - (TF-TG(i))*
     &                            CSOIL(i)*(1.-areas-DAREAS) )
     &                           /(SNOWHC+PINF(i)*CW+CSOIL(i))
                        TG(i) = TSD
                        SNOWW(i,2) = SNOWW(i,2) + PINF(i)
                     endif
                  else
C
C----------------------------------------------------------------------
C     RAIN FALLING ONTO AREA
C----------------------------------------------------------------------
C
C----------------------------------------------------------------------
C     RAIN FALLS ONTO SNOW-FREE SECTOR FIRST.
C----------------------------------------------------------------------
                     TSD = TF - 0.01
                     IF ( areas .LT. 0.999 ) TSD = 
     &                         ( TM(i)*PINF(i)*CW +
     &                           TG(i)*CSOIL(i) )
     &                           /  ( PINF(i)*CW + CSOIL(i) )
                     TG(i) = TSD
                     WWW(i,1)= WWW(i,1)+PINF(i)*(1.-areas)/
     &                       ZDEPTH(i)
C----------------------------------------------------------------------
C     RAIN FALLS ONTO SNOW-COVERED SECTOR NEXT.
C----------------------------------------------------------------------
                     EX = ( TM(i) - TF )*PINF(i)*CW*areas
                     DCAP = -EX / ( SNOMEL + ( TG(i)-TF )*
     &                       CSOIL(i)*ASNOW )
                     IF( (SNOWW(i,2) + DCAP) .ge. 0. ) then
                        WWW(i,1) = WWW(i,1)+(PINF(i)*areas-DCAP)/
     &                         ZDEPTH(i)
                        SNOWW(i,2) = SNOWW(i,2) + DCAP
                     else
                        TG(i) = ( EX - SNOMEL*SNOWW(i,2) -
     &                        ( TG(i)-TF )*CSOIL(i)*areas ) /
     &                          CSOIL(i) + TG(i)
                        WWW(i,1)=WWW(i,1)+(SNOWW(i,2)+PINF(i)*
     &                                areas)/zdepth(i)
                        CAPAC(i,2) = 0.
                        SNOWW(i,2) = 0.
                     endif
C
                  endif
C
C-----------------------------------------------------------------------
C---------------------END OF PATCHS ------------------------------------
C-----------------------------------------------------------------------

               else
C                                                                   
                  THRU(i) = TTI(i) + TEX(i)                                             
                  IF ( TG(i) .LE. TF .OR. TM(i) .LE. TF )
     &                 THRU(i) = 0.                 
                  PINF(i) = P0(i) - THRU(i)                                           
                  IF( TM(i) .gt. TF )then                                        
                     CAPAC(i,IVEG) = CAPAC(i,IVEG) + PINF(i)         
C                                                                      
C---------------------------------------------------------------------
C                                                                    
C    INSTANTANEOUS OVERLAND FLOW CONTRIBUTION ( ROFF )              
C                                                                  
C     ROFF( R-I )     : EQUATION (C.13), SA-89B                   
C                                                                
C---------------------------------------------------------------
C                                                              
                     EQUDEP = SATCO(i) * DTT                                    
C                                                            
                     XS = 1.                                               
                     IF ( THRU(i) .ge. 1.E-9 ) then                     
                        ARG = EQUDEP / ( THRU(i) * AP(i) ) -CP(i)/AP(i)                 
                        IF ( ARG .ge. 1.E-9 ) then                    
                           XS = -1./BP * LOG( ARG )                         
                           XS = MIN( XS, 1.0_dbl_kind )                               
                           XS = MAX( XS, 0.0_dbl_kind )                              
                        endif
                     endif
                     ROFFO(i) = THRU(i) * ( AP(i)/BP *
     &                     ( 1.-EXP( -BP*XS )) + CP(i)*XS ) -EQUDEP*XS
                     ROFFO(i) = MAX ( ROFFO(i), 0.0_dbl_kind )
                     ROFF(i) = ROFF(i) + ROFFO(i)
                     WWW(i,1) = WWW(i,1) +
     &                     (THRU(i) - ROFFO(i)) / ZDEPTH(i) 
                  else
                     SNOWW(i,IVEG) = SNOWW(i,IVEG) + PINF(i)        
                  endif
C
C----------------------------------------------------------------------
C
            CALL ADJUST ( Tg(i), SPECHC(i), CAPACP(i), SNOWWP(i),
     *          IVEG, capac(i,1), snoww(i,1), tm(i), tf,
     *          snomel, www(i,1), zdepth(i),
     *          satcap(i,1), cw, nsib, len  )
C
C----------------------------------------------------------------------
C
               endif
            enddo 
         endif   ! if(iveg.eq.1)

c     make either all capac or all snow

         do i = 1,len
            if(capac(i,iveg).gt.snoww(i,iveg)) then
               capac(i,iveg) = capac(i,iveg) + snoww(i,iveg)
               snoww(i,iveg) = 0.0
            else
               snoww(i,iveg) = snoww(i,iveg) + capac(i,iveg)
               capac(i,iveg) = 0.0
            endif
         enddo
C
 1000 continue
C
      RETURN
      END
C
C==================SUBROUTINE NETRAD====================================
C
      subroutine NETRAD(radc3, radt, stefan, fac1,
     *      vcover, thermk, tc, tg, tf, tgeff, areas, len )

      use kinds

      implicit none
C
C=======================================================================
c
C                                                                       
C        CALCULATE RADT USING RADIATION FROM PHYSICS AND CURRENT        
C        LOSSES FROM CANOPY AND GROUND                                  
C
C
C=======================================================================
c

c++++++++++++++++++++++++++++++OUTPUT+++++++++++++++++++++++++++++++++++
c
C       RADt (2)       SUM OF ABSORBED RADIATIVE FLUXES (W M-2) 
C
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      integer (kind=int_kind), intent(in) :: len
      real (kind=dbl_kind), intent(in) ::
     *      tf, stefan
      real (kind=dbl_kind), intent(in), dimension(len) ::
     *      
     *      vcover,thermk,tc,tg, 
     *      areas
      real (kind=dbl_kind), intent(out), dimension(len) ::
     *      tgeff, fac1
      real (kind=dbl_kind), intent(in), dimension(len,2) ::
     *      radc3
      real (kind=dbl_kind), intent(out), dimension(len,3) ::
     *      radt

c     local variables
      integer (kind=int_kind) :: i
      real (kind=dbl_kind) ::
     *       tc4, tg4, ts4, closs, gloss, sloss, tsnow, radtbar, zlwup
     *,      feedfac   !  feedback factor 
      data feedfac/1.0_dbl_kind/

        DO I=1,len
         tsnow = min(tf, tg(i))
         TC4 = TC(i)**4
         TG4 = TG(i)**4
         ts4 = tsnow**4
         FAC1(i) =  VCOVER(i) * ( 1.-THERMK(i) )
         CLOSS =  2. * FAC1(i) * STEFAN * TC4
         CLOSS =  CLOSS - FAC1(i) * STEFAN * 
     &                 ( (1.-areas(i))*TG4+areas(i)*ts4)
         GLOSS =  STEFAN * TG4 - FAC1(i) * STEFAN * TC4
         SLOSS =  STEFAN * Ts4 - FAC1(i) * STEFAN * TC4
         RADT(i,1) = RADC3(i,1) - closs                                         
         RADT(i,2) = RADC3(i,2) - gloss 
         RADT(i,3) = RADC3(i,2) - sloss 
         radtbar = areas(i)*radt(i,3) + (1.-areas(i))*radt(i,2)
         radt(i,2) = radtbar + (1.+feedfac)*(radt(i,2)-radtbar)
         radt(i,3) = radtbar + (1.+feedfac)*(radt(i,3)-radtbar)
         zlwup = fac1(i) * tc4 + 
     *           (1.-fac1(i)) * (areas(i)*ts4+(1.-areas(i))*tg4)
         TGEFF(i) =  ZLWUP ** 0.25
       enddo

      return
      end
C==================SUBROUTINE PHOSIB===================================
C
      SUBROUTINE PHOSIB(pco2m,po2m,vmax0,tf,psur,green
     ^,          tran,ref,gmudmu,zlt,tc,trop,trda
     ^,          trdm,slti,shti,hltii,hhti,radn,etc,etgs,wc
     ^,          ea,em,rb,ra,tm
     ^,          effcon,rstfac,binter,gradm,assimn
     ^,          rst,atheta,btheta,tgs,respcp
     ^,          aparkk,len,nsib,
     >     omepot,assimpot,assimci,antemp,assimnp,
     >     wsfws,wsfht,wsflt,wci,whs,
     >     wags,wegs,aparc,pfd,assim,td,www,
     >     wopt,zm,wsat,tg,
     >     drst,pdamp,qdamp,ecmass,dtt,bintc,tprcor,ansqr,
     >     nsoil, forcerestore, respg )

      use kinds

      implicit none

c
C
C=======================================================================
C
C     CALCULATION OF CANOPY CONDUCTANCE USING THE INTEGRATED   
C     MODEL RELATING ASSIMILATION AND STOMATAL CONDUCTANCE.
C     UNITS ARE CONVERTED FROM MKS TO BIOLOGICAL UNITS IN THIS ROUTINE.
C     BASE REFERENCE IS SE-92A
C
C                          UNITS
C                         -------
C
C      PCO2M, PCO2A, PCO2I, PO2M                : PASCALS
C      CO2A, CO2S, CO2I, H2OA, H2OS, H2OA       : MOL MOL-1
C      VMAX0, RESPN, ASSIM, GS, GB, GA, PFD     : MOL M-2 S-1
C      EFFCON                                   : MOL CO2 MOL QUANTA-1
C      GCAN, 1/RB, 1/RA, 1/RST                  : M S-1
C      EVAPKG                                   : KG M-2 S-1
C      Q                                        : KG KG-1
C
C                       CONVERSIONS
C                      -------------
C
C      1 MOL H2O           = 0.018 KG
C      1 MOL CO2           = 0.044 KG
C      H2O (MOL MOL-1)     = EA / PSUR ( MB MB-1 )
C      H2O (MOL MOL-1)     = Q*MM/(Q*MM + 1)
C      GS  (CO2)           = GS (H2O) * 1./1.6
C      GS  (MOL M-2 S-1 )  = GS (M S-1) * 44.6*TF/T*P/PO
C      PAR (MOL M-2 S-1 )  = PAR(W M-2) * 4.6*1.E-6
C      MM  (MOLAIR/MOLH2O) = 1.611
C
C
C                         OUTPUT
C                      -------------
C
C      ASSIMN              = CANOPY NET ASSIMILATION RATE
C      EA                  = CANOPY AIR SPACE VAPOR PRESSURE
C      1/RST               = CANOPY CONDUCTANCE
C      PCO2I               = INTERNAL CO2 CONCENTRATION
C      RESPC               = CANOPY RESPIRATION
C      RESPG               = GROUND RESPIRATION
C
C----------------------------------------------------------------------
C
C         RSTFAC(1) ( F(H-S) )               : EQUATION (17,18), SE-92A
C         RSTFAC(2) ( F(SOIL) )              : EQUATION (12 mod), SE-89
C         RSTFAC(3) ( F(TEMP) )              : EQUATION (5b)   , CO-92
C         RSTFAC(4) ( F(H-S)*F(SOIL)*F(TEMP))
C
C-----------------------------------------------------------------------
C

c++++++++++++++++++++++++++++++OUTPUT+++++++++++++++++++++++++++++++++++
c
C       ASSIMN         CARBON ASSIMILATION FLUX (MOL M-2 S-1) 
C       RST            CANOPY RESISTANCE (S M-1)
C       RSTFAC(4)      CANOPY RESISTANCE STRESS FACTORS 
C
c++++++++++++++++++++++++++DIAGNOSTICS++++++++++++++++++++++++++++++++++
C
C       RESPC          CANOPY RESPIRATION (MOL M-2 S-1)
C       RESPG          GROUND RESPIRATION (MOL M-2 S-1)
C       PCO2I          CANOPY INTERNAL CO2 CONCENTRATION (MOL MOL-1)
C       GSH2O          CANOPY CONDUCTANCE (MOL M-2 S-1)
C       H2OS           CANOPY SURFACE H2O CONCENTRATION (MOL MOL-1)
c
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

c     Modifications:
c       - gs (stomatal conductance reduced for freezing soils per Jim Collatz
c         dd 950221      
c
c      Modified for multitasking - introduced gather/scatter indices
c          - DD 951206

C
c     input arrays:
      integer (kind=int_kind), intent(in) ::
     *      len, nsib, nsoil
      logical (kind=log_kind), intent(in) ::
     *      forcerestore

      real (kind=dbl_kind), intent(in), dimension(len) ::
     *    vmax0,psur,green,gmudmu,
     *    zlt,tc,trop,trda,
     *    slti,shti,hltii,hhti,
     *    etgs,wc,em,ra,rb,
     *    tm,effcon,
     *    binter,gradm,atheta,btheta,
     *    tgs,respcp,
     *    ecmass,trdm,etc,
     *    aparc,tg,rst,ea,
     >    wopt,zm,wsat,respg
      real (kind=dbl_kind), intent(in), dimension(nsib,2,2) ::
     *    tran,ref
      real (kind=dbl_kind), intent(in), dimension(nsib,nsoil) ::
     *    td
      real (kind=dbl_kind), intent(in), dimension(len,2,2) ::
     *    radn
      real (kind=dbl_kind), intent(in), dimension(len,2) ::
     *    www
      real (kind=dbl_kind), intent(in) ::
     *    pdamp, qdamp, dtt, pco2m, tf, po2m
     
c     output arrays:
      real (kind=dbl_kind), intent(inout), dimension(len,4) ::
     *     rstfac

      real (kind=dbl_kind), intent(out), dimension(len) ::
     *     assimn,drst,
czz new diagostics 10/14/92 
c
c output arrays
     >     omepot,assimpot,assimci,
     >     assimnp,whs,antemp,
     >     wsfws,wsfht,wsflt,wci,
     >     wags,wegs,pfd,
     >     assim,
     >     tprcor,bintc,
     >     ansqr
     
c     work arrays:

      real (kind=dbl_kind),  dimension(len,6) ::
     *     PCO2Y, EYY,assimny,
     *     assimy
      real (kind=dbl_kind),  dimension(len) ::
     *     c3,c4,range,gammas,
     *     aparkk,gah2o,
     *     gbh2o,
     *     par,rrkk,
     *     omss,vm,gsh2o,pco2s,
     *     templ,temph,
     *     qt,co2s,scatp,scatg,
     *     park,respn,zkc,
     *     zko,spfy,
     *     soilfrz,
     *     pco2i, respc
      integer (kind=int_kind), dimension(len) ::
     *     icconv,igath

      real (kind=dbl_kind) ::
     *     cwsflt, cwsfht, cwsfws,
     *     ccoms, ccomc, ascitemp, dompdomc, omsci, ompci, omcci,
     *     omcpot, omppot, sqrtin, omspot, pco2ipot, ohtp2, sttp2,
     *     gsh2oinf, h2osrh, h2os, ecmole, h2oa, h2oi, dtti,
     *     pco2in, pco2a, soilfrztd, soilfrztg

      integer (kind=int_kind) ::
     *     i, ic1, ic, l
C
C----------------------------------------------------------------------
C
      do i = 1,len                !  LOOP OVER GRIDPOINT
c        RESPG(i) = 0.
C
C----------------------------------------------------------------------
C
        IF( EFFCON(i) .GT. 0.07 ) then
          C3(i) = 1.
        else
          c3(i) = 0.
        endif
        C4(i)     = 1. - C3(i)
C
C-----------------------------------------------------------------------
C
C     CALCULATION OF CANOPY PAR USE PARAMETER.
C
C      APARKK      (PI)     : EQUATION (31) , SE-92A
C-----------------------------------------------------------------------
C  
        SCATP(I) =     GREEN(i)   * 
     &           ( TRAN(i,1,1) + REF(i,1,1) )
     &           +( 1.-GREEN(i) ) *
     &           ( TRAN(i,1,2) + REF(i,1,2) )
        SCATG(i) = TRAN(i,1,1) + REF(i,1,1)
        PARK(i) = SQRT(1.-SCATP(i)) * GMUDMU(i)
c
c Collatz-Bounoua commented the calculation of  aparc
c replaced it with theone calculated in new_mapper.
c
cb        APARC(i) = 1. - EXP ( -PARK(i)*ZLT(i) )   ! lahouari
c
        APARKK(i)   = APARC(i) / PARK(i) * GREEN(i)
C-----------------------------------------------------------------------
C
C     Q-10 AND STRESS TEMPERATURE EFFECTS
C
C      QT          (QT)    : TABLE (2)     , SE-92A
C-----------------------------------------------------------------------
C
        qt(i) = 0.1*( TC(i) - TROP(i) )
        RESPN(i) = RESPCP(i) * VMAX0(i) * RSTFAC(i,2)
        RESPC(i) = RESPN(i) * 2.0**qt(i)
     *        /( 1. + EXP( TRDA(i)*(TC(i)-TRDM(i))))
        VM(i) = VMAX0(i) * 2.1**qt(i)
        TEMPL(i) = 1. + EXP(SLTI(i)*(HLTIi(i)-TC(i)))
        TEMPH(i) = 1. + EXP(SHTI(i)*(TC(i)-HHTI(i)))
        RSTFAC(i,3) = 1./( TEMPL(i)*TEMPH(i))
        VM(i)    = VM(i)/TEMPH(i) * RSTFAC(i,2)*C3(i)
     &      + VM(i) * RSTFAC(i,2)*RSTFAC(i,3) * C4(i)
C
C-----------------------------------------------------------------------
C
C     MICHAELIS-MENTEN CONSTANTS FOR CO2 AND O2, CO2/O2 SPECIFICITY,
C     COMPENSATION POINT       
C
C      ZKC          (KC)     : TABLE (2)     , SE-92A
C      ZKO          (KO)     : TABLE (2)     , SE-92A
C      SPFY         (S)      : TABLE (2)     , SE-92A
C      GAMMAS       (GAMMA-*): TABLE (2)     , SE-92A
C      OMSS         (OMEGA-S): EQUATION (13) , SE-92A
C      BINTC        (B*ZLT)  : EQUATION (35) , SE-92A
C-----------------------------------------------------------------------
C
        ZKC(i) = 30. * 2.1**qt(i)
        ZKO(i) = 30000. * 1.2**qt(i)
        SPFY(i) = 2600. * 0.57**qt(i)
        GAMMAS(i) = 0.5 * PO2M/SPFY(i) * C3(i)
        PFD(i)    = 4.6E-6 * GMUDMU(i)*
     *             ( RADN(i,1,1)+RADN(i,1,2) )
C
        TPRCOR(i) = TF*PSUR(i)*100./1.013 E5                                                                     
        GSH2O(i)  = 1.0/RST(i) * 44.6*TPRCOR(i)/TC(i)
        GBH2O(i)  = 0.5/RB(i) * 44.6*TPRCOR(i)/TC(i)
        GAH2O(i)  = 1.0/RA(i) * 44.6*TPRCOR(i)/TM(i)
C
        RRKK(i)   = ZKC(i)*( 1. + PO2M/ZKO(i) ) * C3(i)
     &               + VMAX0(i)/5.* ( 1.8**qt(i)) * C4(i)
        PAR(i)    = pfd(i)*EFFCON(i)*( 1.-SCATG(i) )
        soilfrztg = 1.+exp(-1.5 * (max(270.0_dbl_kind,tgs(i))-273.16) )
        soilfrztd = 1.+exp(-1.5 * (max(270.0_dbl_kind,td (i,nsoil))-273.16) )
        soilfrz(i) = max(1./soilfrztg, 1./soilfrztd)
        soilfrz(i) = max( soilfrz(i), 0.05_dbl_kind)
        BINTC(i)  = BINTER(i)*ZLT(i)*GREEN(i)*
     &                 RSTFAC(i,2) * soilfrz(i)
        OMSS(i)   = ( VMAX0(i)/2.0 ) * ( 1.8**qt(i) )
     &                  /TEMPL(i) * RSTFAC(i,2) * C3(i)
     &                  + RRKK(i) * RSTFAC(i,2) * C4(i)
C
C-----------------------------------------------------------------------
C
C     FIRST GUESS IS MIDWAY BETWEEN COMPENSATION POINT AND MAXIMUM
C     ASSIMILATION RATE.
C
C-----------------------------------------------------------------------
        RANGE(i)    = PCO2M * ( 1. - 1.6/GRADM(i) ) - GAMMAS(i)
	icconv(i) = 1
	
      enddo
      
C
      DO 1000 IC = 1, 6
        do i = 1,len        ! LOOP OVER GRIDPOINT
          PCO2Y(i,IC) = 0.
          EYY(i,IC) = 0.
	enddo
1000  CONTINUE
C

      DO 2000 IC = 1, 6
C
        CALL       SORTIN( EYY, PCO2Y, RANGE, GAMMAS, IC,len )
C
        CALL       CYCALC( APARKK, VM, ATHETA, BTHETA,par,
     &                     GAMMAS, RESPC, RRKK, OMSS, C3, C4,
     &                     PCO2Y(1,IC), assimny(1,ic), assimy(1,ic),
     &                     len  )
C
      do i = 1,len
        PCO2A = PCO2M - (1.4/MAX(0.446_dbl_kind,GAH2O(i)) *
     >             (ASSIMNy(i,ic) - RESPG(i))* PSUR(i)*100.)
        PCO2S(i) = PCO2A - (1.4/GBH2O(i) * ASSIMNy(i,ic)
     >            * PSUR(i)*100.)
        PCO2IN = PCO2S(i) - (1.6/GSH2O(i) * ASSIMNy(i,ic)
     >            * PSUR(i)*100.)
        EYY(i,IC) = PCO2Y(i,IC) - PCO2IN
      enddo
c
        if(ic.ge.2) then
        ic1 = ic-1
          do i = 1,len        ! LOOP OVER GRIDPOINT
            if(abs(eyy(i,ic1)).ge.0.1_dbl_kind)then
              icconv(i) = ic
              do l = 1,ic1
                if(eyy(i,ic).eq.eyy(i,l)) then
                   eyy(i,ic) = eyy(i,ic) + 1.e-5_dbl_kind
                   pco2y(i,ic) = pco2y(i,ic) + 1.e-5_dbl_kind
                endif
              enddo
            else
              eyy(i,ic) = eyy(i,ic1)
              pco2y(i,ic) = pco2y(i,ic1)
            endif
          enddo
        endif
	
C
2000  CONTINUE
C
      do i = 1,len        ! LOOP OVER GRIDPOINT
        icconv(i) = min(icconv(i),6)
        igath(i) = i+(icconv(i)-1)*len
      enddo
      
      do i = 1,len         ! LOOP OVER GRIDPOINT
        pco2i(i) = pco2y(i,icconv(i))
        assimn(i) = assimny(i,icconv(i))
	assim(i) = assimy(i,icconv(i))
      enddo
C
      dtti = 1./dtt
      do i = 1,len        ! LOOP OVER GRIDPOINT
cczzggrst5 - new code
        H2OI   = ETC(i)/PSUR(i)
        H2OA   =  EA(i)/PSUR(i)
        ECMOLE = 55.56 * ECMASS(i) * dtti  ! ecmass must be computed and passed in
        H2OS = H2OA + ECMOLE / GBH2O(i)
        H2OS  = MIN( H2OS, H2OI )
        H2OS  = MAX( H2OS, 1.e-7_dbl_kind)
        H2OSRH = H2OS/H2OI
C  need qdamp and pdamp calculated and passed to here!
c        CO2S(i) = MAX(PCO2S(I),PCO2M*0.5) / (PSUR(i)*100.)
        CO2S(i) = MAX(PCO2S(I),PCO2M*0.05) / (PSUR(i)*100.)
        GSH2OINF = (GRADM(i) * MAX(1.e-12_dbl_kind,ASSIMN(i))
     >            * H2OSRH * soilfrz(i) / CO2S(i)) + BINTC(i)
        DRST(i) = RST(i) * QDAMP * ((GSH2O(i)-GSH2OINF)/
     &            (PDAMP*GSH2O(i)+QDAMP*GSH2OINF))
c
        RSTFAC(i,1) = H2OS/H2OI
        RSTFAC(i,4) = RSTFAC(i,1)*RSTFAC(i,2)* RSTFAC(i,3)
      enddo
C
CZ CARNEGIE new diagnostics----start!!!(c.zhang&joe berry, 10/19/92)
c-----------------------------------------------------------------------
C  INPUTS: PSUR(i),CO2S,ASSIMN(i),GRADM(i),BINTC(i),VMAX0(i),RRKK(i),C3(i),
C    C4(i),PAR(i),ATHETA(i),BTHETA(i),APARKK(i),OMSS(i),RSTFAC(i,2),TEMPH,
C    TEMPL,RSTFAC(i,1),VM(i),ASSIM,GSH20(i),EFFCON(i),QT,GAMMAS(i),
C    PFD(i)
C

        sttp2 = 73.**0.2
        ohtp2 = 100.**0.2
        do i = 1,len
C-----------------------------------------------------------------------
C CALCULATION OF POTENTIAL ASSIMILATION
C-----------------------------------------------------------------------
C
C Make assimn a top leaf, not the canopy.
      ASSIMNp(i) = ASSIMN(i) / APARKK(i)
C
C Bottom stopped assim.
      ANTEMP(i) = MAX(0.0_dbl_kind,ASSIMNp(i))
C
C Potential intercellular co2.
      PCO2IPOT = PSUR(i)*100.*(co2s(i)-(1.6*ASSIMNp(i)/
     &       ((GRADM(i)*ANTEMP(i)/co2s(i))+BINTC(i))))
C
C Potential rubisco limitation.
      OMCPOT = VMAX0(i)*2.1**qt(i)*((PCO2IPOT-GAMMAS(i))/
     &       (PCO2IPOT+RRKK(i))*C3(i) + C4(i))
C
C Potential light limitation.
      OMEPOT(i) = PAR(i)*((PCO2IPOT-GAMMAS(i))/
     &           (PCO2IPOT+2.*GAMMAS(i))*C3(i) + C4(i))
C
C Quad 1.
      SQRTIN = MAX(1.e-36_dbl_kind,((OMEPOT(i)+OMCPOT)**2-
     &               4.*ATHETA(i)*OMEPOT(i)*OMCPOT))
C
C Quad 1. Intermediate  top leaf photosynthesis.
      OMPPOT = ((OMEPOT(i)+OMCPOT)-SQRT(SQRTIN))/(2.*ATHETA(i))
C
C Potential sink or pep limitation.
      OMSPOT = (VMAX0(i)/2.0)*(1.8**qt(i))*C3(i) 
     &                + RRKK(i)*PCO2IPOT*C4(i)
C
C Quad 2.
      SQRTIN=MAX(1.e-36_dbl_kind,((OMPPOT+OMSPOT)**2-4.*BTHETA(i)*
     &        OMPPOT*OMSPOT))
C
C Quad 2. Final Potential top leaf photosynthesis.
      ASSIMPOT(i) = ((OMSPOT+OMPPOT)-SQRT(SQRTIN))/(2.*BTHETA(i))
C
C-----------------------------------------------------------------------
C CALCULATION OF STRESS FACTOR LIMITED ASSIMILATION
C-----------------------------------------------------------------------
C
C Stressed rubisco limitation.
      OMCCI = VM(i)*((PCO2IPOT-GAMMAS(i))/(PCO2IPOT+RRKK(i))*C3(i) 
     &             + C4(i))
C
C Quad 1.
      SQRTIN = MAX(1.e-36_dbl_kind,(OMEPOT(i)+OMCCI)**2-
     &                4.*ATHETA(i)*OMEPOT(i)*OMCCI)
C
C Quad 1. Intermediate stress limited top leaf photosynthesis.
      OMPCI = ((OMEPOT(i)+OMCCI)-SQRT(SQRTIN))/(2.*ATHETA(i))
C
C Stressed sink or pep limitation.
      OMSCI = OMSS(i)*(C3(i) + PCO2IPOT*C4(i))
C
C Quad 2.
      SQRTIN = MAX(1.e-36_dbl_kind,(OMPCI+OMSCI)**2-4.*BTHETA(i)*OMPCI*OMSCI)
C 
C Quad 2. Final stress limited top leaf photosynthesis.
      ASSIMCI(i) = ((OMSCI+OMPCI)-SQRT(SQRTIN))/(2.*BTHETA(i))
C
C-----------------------------------------------------------------------
C CALCULATION OF CONTROL COEFFICIENTS
C-----------------------------------------------------------------------
C
C Intermediate.
      DOMPDOMC = (OMPCI-OMEPOT(i))/
     &          (2.*ATHETA(i)*OMPCI-OMCCI-OMEPOT(i))
C
C Bottom stopped final stress limited top leaf photosynthesis.
      ASCITEMP = MAX(ASSIMCI(i),1.e-12_dbl_kind)
C
C Rubisco control coefficient.
      CCOMC = (DOMPDOMC*(ASSIMCI(i)-OMSCI)/
     &      (2.*BTHETA(i)*ASSIMCI(i)-OMPCI-OMSCI))*OMCCI/ASCITEMP
C
C Sink or pep control coefficient.
      CCOMS = ((ASSIMCI(i)-OMPCI)/
     &       (2.*BTHETA(i)*ASSIMCI(i)-OMPCI-OMSCI))*OMSCI/ASCITEMP
C
C-----------------------------------------------------------------------
C  OUTPUT:  POTENTIAL ASSIMILATION RATES TO BE SUMMED
C-----------------------------------------------------------------------
C Canopy values (overwrites top leaf).
C 
      OMEPOT(i) = OMEPOT(i)*APARKK(i)
      ASSIMPOT(i) = ASSIMPOT(i)*APARKK(i)
      ASSIMCI(i) = ASSIMCI(i)*APARKK(i)
      ASSIM(i) = ASSIM(i)*APARKK(i)
      ANTEMP(i) = ANTEMP(i)*APARKK(i)
      ANSQR(i) = ANTEMP(i)*ANTEMP(i)
      ASSIMNp(i) = ASSIMNp(i)*APARKK(i)
C
C-----------------------------------------------------------------------
C OUTPUT:  WEIGHTED STRESS FACTORS AND OTHER DIAGNOSTIC OUTPUTS TO BE SUMMED
C-----------------------------------------------------------------------
C
C Water stress.
      WSFWS(i) = ASSIMPOT(i)*(1.-RSTFAC(i,2))*(CCOMC+CCOMS)
C
C High temperature stress.
      WSFHT(i) = ASSIMPOT(i)*(1.-1./TEMPH(i))*CCOMC
C
C Low temperature stress.
      WSFLT(i) = ASSIMPOT(i)*(1.-1./TEMPL(i))*(CCOMS*C3(i)+CCOMC*C4(i))
c
c  protection for wsfws, wsfht, and wsflt from <0 or >>xxx(2/24/93)
      cwsfws = (1.-RSTFAC(i,2))*(CCOMC+CCOMS)
      if(cwsfws.gt.1. .or. cwsfws.lt.0.) wsfws(i)=0.
      cwsfht = (1.-1./TEMPH(i))*CCOMC
      if(cwsfht.gt.1. .or. cwsfht.lt.0.) wsfht(i)=0.
      cwsflt = (1.-1./TEMPL(i))*(CCOMS*C3(i)+CCOMC*C4(i))
      if(cwsflt.gt.1. .or. cwsflt.lt.0.) wsflt(i)=0.

C
C Intermediate assimilation weighted Ci.
      WCI(i) = ANTEMP(i)*PCO2I(i)
C
C Intermediate assimilation weighted relative humidty stress factor.
      WHS(i) = ANTEMP(i)*RSTFAC(i,1)
C
C Intermediate assimilation weighted stomatal conductance.
      WAGS(i) = GSH2O(i)*ANTEMP(i)
C
C Intermediate evaporation weighted stomatal conductance.(Step 1.
C   Step 2 after subroutine updat2)
      WEGS(i) = GSH2O(i)

      enddo

CZZ carnegie new diagnostics end!!! 

      RETURN
      END

c
C=================SUBROUTINE RADA2======================================
C
      SUBROUTINE RADA2(snoww,zlt,z1,z2
     *,          asnow,tg,sunang,tf,ref,tran,chil
     *,          green,vcover,soref,radfac,salb,thermk
     *,          tgeff4,tc,len,nlen) 

      use kinds    

      implicit none                                            

C=======================================================================
C
C     CALCULATION OF ALBEDOS VIA TWO STREAM APPROXIMATION( DIRECT
C     AND DIFFUSE ) AND PARTITION OF RADIANT ENERGY
C
C-----------------------------------------------------------------------


c++++++++++++++++++++++++++++++OUTPUT+++++++++++++++++++++++++++++++++++
c
C       SALB(2,2)      SURFACE ALBEDOS 
C       TGEFF4         EFFECTIVE SURFACE RADIATIVE TEMPERATURE (K) 
C       RADFAC(2,2,2)  RADIATION ABSORPTION FACTORS 
C       THERMK         CANOPY GAP FRACTION FOR TIR RADIATION 
C
c++++++++++++++++++++++++++DIAGNOSTICS++++++++++++++++++++++++++++++++++
C
C       ALBEDO(2,2,2)  COMPONENT REFLECTANCES 
c
c
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
c     arg list declarations
c
      integer (kind=int_kind), intent(in) ::
     *      len, nlen
      real (kind=dbl_kind), intent(in) ::
     *     tf, asnow
      real (kind=dbl_kind), intent(in), dimension(nlen) ::
     ^     zlt,z1,z2,tg,sunang,
     ^     chil,green,vcover,
     ^     tc
      real (kind=dbl_kind), intent(in), dimension(nlen,2) ::
     *     snoww, soref
      real (kind=dbl_kind), intent(in), dimension(nlen,2,2) ::
     *     ref, tran
      real (kind=dbl_kind), intent(out), dimension(nlen) ::
     *     tgeff4, thermk
      real (kind=dbl_kind), intent(out), dimension(nlen,2,2) ::
     *     salb
      real (kind=dbl_kind), intent(out), dimension(nlen,2,2,2) ::
     *     radfac

c     local variables
      real (kind=dbl_kind), dimension(len) ::
     *     f, fmelt, zmew, canex, areas
      real (kind=dbl_kind), dimension(2) ::
     *     tranc1, tranc2, tranc3
      real (kind=dbl_kind), dimension(len,2) ::
     *     satcap
      real (kind=dbl_kind), dimension(len,2,2,2) ::
     *     albedo
      real (kind=dbl_kind) ::
     ^     facs, scov, scat, chiv, aa, bb,
     ^     fac2, fac1, zkat, tg4, tc4, tgs, hh6,
     ^     hh5, hh3, hh2, zmk, hh10, hh9, hh8, hh7, den, zp, f1, ge,
     ^     ek, epsi, power2, power1, zat, psi, hh4, hh1, fe, de, bot,
     ^     ce, be, betao, upscat, acss, extkb, proj, tran2, tran1,
     ^     reff1, reff2
      integer (kind=int_kind) ::iwave, i, irad
C
C----------------------------------------------------------------------
C
C
C     MODIFICATION FOR EFFECT OF SNOW ON UPPER STOREY ALBEDO
C         SNOW REFLECTANCE   = 0.80, 0.40 . MULTIPLY BY 0.6 IF MELTING
C         SNOW TRANSMITTANCE = 0.20, 0.54
C
C
C-----------------------------------------------------------------------
C
C
      do i = 1,len                      ! loop over gridpoint
c        this portion is snow1 inlined
          CANEX(i)  = 1.-( SNOWw(i,1)*5.-Z1(i))/
     *             (Z2(i)-Z1(i))
          CANEX(i)  = MAX( 0.1_dbl_kind, CANEX(i) )
          CANEX(i)  = MIN( 1.0_dbl_kind, CANEX(i) )
          AREAS(i)    = MIN(1.0_dbl_kind, ASNOW*SNOWw(i,2))
          SATCAP(i,1) = ZLT(i)*0.0001 * CANEX(i)
c
cc Collatz-Bounoua change satcap(2) to 0.0002
c
cbl          SATCAP(i,2) = 0.002
        SATCAP(i,2) = 0.0002             ! lahouari
c    end old snow1
        F(i) = max(0.01746_dbl_kind,SUNANG(i))
        FACS  = ( TG(i)-TF ) * 0.04
        FACS  = MAX( 0.0_dbl_kind , FACS)
        FACS  = MIN( 0.4_dbl_kind, FACS)
        FMELT(i) = 1. - FACS
      enddo
C
C-----------------------------------------------------------------------
C
      DO 1000 IWAVE = 1, 2
C
C----------------------------------------------------------------------
      do i = 1,len                      ! loop over gridpoint
        SCOV =  MIN( 0.5_dbl_kind, SNOWw(i,1)/SATCAP(i,1) )
        REFF1 = ( 1. - SCOV ) * REF(i,IWAVE,1) + SCOV * ( 1.2 -
     &          IWAVE * 0.4 ) * FMELT(i)
        REFF2 = ( 1. - SCOV ) * REF(i,IWAVE,2) + SCOV * ( 1.2 -
     &          IWAVE * 0.4 ) * FMELT(i)
        TRAN1 = TRAN(i,IWAVE,1) * ( 1. - SCOV )
     &          + SCOV * ( 1.- ( 1.2 - IWAVE * 0.4 ) * FMELT(i) )
     &          * TRAN(i,IWAVE,1)
        TRAN2 = TRAN(i,IWAVE,2) * ( 1. - SCOV )
     &          + SCOV * ( 1.- ( 1.2 - IWAVE * 0.4 ) * FMELT(i) ) * 0.9
     &          * TRAN(i,IWAVE,2)
C
C-----------------------------------------------------------------------
C
C     CALCULATE AVERAGE SCATTERING COEFFICIENT, LEAF PROJECTION AND
C     OTHER COEFFICIENTS FOR TWO-STREAM MODEL.
C
C      SCAT  (OMEGA)        : EQUATION (1,2) , SE-85
C      PROJ  (G(MU))        : EQUATION (13)  , SE-85
C      EXTKB (K, G(MU)/MU)  : EQUATION (1,2) , SE-85
C      ZMEW  (INT(MU/G(MU)) : EQUATION (1,2) , SE-85
C      ACSS  (A-S(MU))      : EQUATION (5)   , SE-85
C      EXTK  (K, VARIOUS)   : EQUATION (13)  , SE-85
C      UPSCAT(OMEGA*BETA)   : EQUATION (3)   , SE-85
C      BETAO (BETA-0)       : EQUATION (4)   , SE-85 
C
C-----------------------------------------------------------------------
C
        SCAT = GREEN(i)*( TRAN1 + REFF1 ) +( 1.-GREEN(i) ) *
     &       ( TRAN2 + REFF2)
        CHIV = CHIL(i)
C
        IF ( ABS(CHIV) .LE. 0.01 ) CHIV = 0.01
        AA = 0.5 - 0.633 * CHIV - 0.33 * CHIV * CHIV
        BB = 0.877 * ( 1. - 2. * AA )
C
        PROJ = AA + BB * F(i)
        EXTKB = ( AA + BB * F(i) ) / F(i)
        ZMEW(i) = 1. / BB * ( 1. - AA / BB 
     *               * LOG ( ( AA + BB ) / AA ) )
        ACSS = SCAT / 2. * PROJ / ( PROJ + F(i) * BB )
        ACSS = ACSS * ( 1. - F(i) * AA
     *          / ( PROJ + F(i) * BB ) * LOG ( ( PROJ
     *         +   F(i) * BB + F(i) * AA ) / ( F(i) * AA ) ) )
C
        UPSCAT = GREEN(i) * TRAN1 + ( 1.-GREEN(i) ) * TRAN2
        UPSCAT = 0.5 * ( SCAT + ( SCAT - 2. * UPSCAT ) *
     *           (( 1. - CHIV ) / 2. ) ** 2 )
        BETAO = ( 1. + ZMEW(i) * EXTKB )
     *           / ( SCAT * ZMEW(i) * EXTKB ) * ACSS
C
C-----------------------------------------------------------------------
C
C     Intermediate variables identified in appendix of SE-85.
C
C      BE          (B)     : APPENDIX      , SE-85
C      CE          (C)     : APPENDIX      , SE-85
C      BOT         (SIGMA) : APPENDIX      , SE-85
C      HH1         (H1)    : APPENDIX      , SE-85
C      HH2         (H2)    : APPENDIX      , SE-85
C      HH3         (H3)    : APPENDIX      , SE-85
C      HH4         (H4)    : APPENDIX      , SE-85
C      HH5         (H5)    : APPENDIX      , SE-85
C      HH6         (H6)    : APPENDIX      , SE-85
C      HH7         (H7)    : APPENDIX      , SE-85
C      HH8         (H8)    : APPENDIX      , SE-85
C      HH9         (H9)    : APPENDIX      , SE-85
C      HH10        (H10)   : APPENDIX      , SE-85
C      PSI         (H)     : APPENDIX      , SE-85
C      ZAT         (L-T)   : APPENDIX      , SE-85
C      EPSI        (S1)    : APPENDIX      , SE-85
C      EK          (S2)    : APPENDIX      , SE-85
C-----------------------------------------------------------------------
C
        BE = 1. - SCAT + UPSCAT
        CE = UPSCAT
        BOT = ( ZMEW(i) * EXTKB ) ** 2 + ( CE**2 - BE**2 )
        IF ( ABS(BOT) .le. 1.E-10) then
          SCAT = SCAT* 0.98
          BE = 1. - SCAT + UPSCAT
          BOT = ( ZMEW(i) * EXTKB ) ** 2 + ( CE**2 - BE**2 )
        end if
        DE = SCAT * ZMEW(i) * EXTKB * BETAO
        FE = SCAT * ZMEW(i) * EXTKB * ( 1. - BETAO )
        HH1 = -DE * BE + ZMEW(i) * DE * EXTKB - CE * FE
        HH4 = -BE * FE - ZMEW(i) * FE * EXTKB - CE * DE
C
        PSI = SQRT(BE**2 - CE**2)/ZMEW(i)
C
        ZAT = ZLT(i)/VCOVER(i)*CANEX(i)
C
        POWER1 = MIN( PSI*ZAT, 50.0_dbl_kind )
        POWER2 = MIN( EXTKB*ZAT, 50.0_dbl_kind )
        EPSI = EXP( - POWER1 )
        EK = EXP ( - POWER2 )
C
        ALBEDO(i,2,IWAVE,1) = SOREF(i,IWAVE)*(1.-AREAS(i))
     &                    + ( 1.2-IWAVE*0.4 )*FMELT(i) * AREAS(i)
        ALBEDO(i,2,IWAVE,2) = SOREF(i,IWAVE)*(1.-AREAS(i))
     &                    + ( 1.2-IWAVE*0.4 )*FMELT(i) * AREAS(i)
        GE = ALBEDO(i,2,IWAVE,1)/ALBEDO(i,2,IWAVE,2)
C
C-----------------------------------------------------------------------
C     CALCULATION OF DIFFUSE ALBEDOS
C
C      ALBEDO(1,IWAVE,2) ( I-UP ) : APPENDIX , SE-85
C-----------------------------------------------------------------------
C
        F1 = BE - CE / ALBEDO(i,2,IWAVE,2)
        ZP = ZMEW(i) * PSI
C
        DEN = ( BE + ZP ) * ( F1 - ZP ) / EPSI -
     &        ( BE - ZP ) * ( F1 + ZP ) * EPSI
        HH7 = CE * ( F1 - ZP ) / EPSI / DEN
        HH8 = -CE * ( F1 + ZP ) * EPSI / DEN
        F1 = BE - CE * ALBEDO(i,2,IWAVE,2)
        DEN = ( F1 + ZP ) / EPSI - ( F1 - ZP ) * EPSI
C
        HH9 = ( F1 + ZP ) / EPSI / DEN
        HH10 = - ( F1 - ZP ) * EPSI / DEN
        TRANC2(IWAVE) = HH9 * EPSI + HH10 / EPSI
C
        ALBEDO(i,1,IWAVE,2) =  HH7 + HH8
C
C-----------------------------------------------------------------------
C     CALCULATION OF DIRECT ALBEDOS AND CANOPY TRANSMITTANCES.
C
C      ALBEDO(1,IWAVE,1) ( I-UP )   : EQUATION(11)   , SE-85
C      TRANC(IWAVE)      ( I-DOWN ) : EQUATION(10)   , SE-85
C
C-----------------------------------------------------------------------
C
        F1 = BE - CE / ALBEDO(i,2,IWAVE,2)
        ZMK = ZMEW(i) * EXTKB
C
        DEN = ( BE + ZP ) * ( F1 - ZP ) / EPSI -
     &        ( BE - ZP ) * ( F1 + ZP ) * EPSI
        HH2 = ( DE - HH1 / BOT * ( BE + ZMK ) )
     &         * ( F1 - ZP ) / EPSI -
     &           ( BE - ZP ) * ( DE - CE*GE - HH1 / BOT
     &         * ( F1 + ZMK ) ) * EK
        HH2 = HH2 / DEN
        HH3 = ( BE + ZP ) * (DE - CE*GE - 
     &         HH1 / BOT * ( F1 + ZMK ))* EK -
     &         ( DE - HH1 / BOT * ( BE + ZMK ) ) * 
     &         ( F1 + ZP ) * EPSI
        HH3 = HH3 / DEN
        F1 = BE - CE * ALBEDO(i,2,IWAVE,2)
        DEN = ( F1 + ZP ) / EPSI - ( F1 - ZP ) * EPSI
        HH5 = - HH4 / BOT * ( F1 + ZP ) / EPSI -
     &        ( FE + CE*GE*ALBEDO(i,2,IWAVE,2) + 
     &          HH4 / BOT*( ZMK-F1 ) ) * EK
        HH5 = HH5 / DEN
        HH6 =   HH4 / BOT * ( F1 - ZP ) * EPSI +
     &        ( FE + CE*GE*ALBEDO(i,2,IWAVE,2) + 
     &          HH4 / BOT*( ZMK-F1 ) ) * EK
        HH6 = HH6 / DEN
        TRANC1(IWAVE) = EK
        TRANC3(IWAVE) = HH4 / BOT * EK + HH5 * EPSI + HH6 / EPSI
C
        ALBEDO(i,1,IWAVE,1) = HH1 / BOT + HH2 + HH3
C
C----------------------------------------------------------------------
C
C
C----------------------------------------------------------------------
C     CALCULATION OF TERMS WHICH MULTIPLY INCOMING SHORT WAVE FLUXES
C     TO GIVE ABSORPTION OF RADIATION BY CANOPY AND GROUND
c
C      RADFAC      (F(IL,IMU,IV)) : EQUATION (19,20) , SE-86
C----------------------------------------------------------------------
C
        RADFAC(i,2,IWAVE,1) = ( 1.-VCOVER(i) )
     &       * ( 1.-ALBEDO(i,2,IWAVE,1) ) + VCOVER(i)
     &       * ( TRANC1(IWAVE) * ( 1.-ALBEDO(i,2,IWAVE,1) )
     &       + TRANC3(IWAVE) * ( 1.-ALBEDO(i,2,IWAVE,2) ) )
C
        RADFAC(i,2,IWAVE,2) = ( 1.-VCOVER(i) )
     &       * ( 1.-ALBEDO(i,2,IWAVE,2) ) + VCOVER(i)
     &       *  TRANC2(IWAVE) * ( 1.-ALBEDO(i,2,IWAVE,2) )
C
        RADFAC(i,1,IWAVE,1) = VCOVER(i)
     &       * ( ( 1.-ALBEDO(i,1,IWAVE,1) )
     &       - TRANC1(IWAVE) * ( 1.-ALBEDO(i,2,IWAVE,1) )
     &       - TRANC3(IWAVE) * ( 1.-ALBEDO(i,2,IWAVE,2) ) )
C
        RADFAC(i,1,IWAVE,2) = VCOVER(i)
     &         * ( ( 1.-ALBEDO(i,1,IWAVE,2) )
     &         - TRANC2(IWAVE) * ( 1.-ALBEDO(i,2,IWAVE,2) ) )
      enddo
C
C----------------------------------------------------------------------
C     CALCULATION OF TOTAL SURFACE ALBEDOS ( SALB ) WITH WEIGHTING
C     FOR COVER FRACTIONS.
C----------------------------------------------------------------------
C
      DO 3000 IRAD = 1, 2
        do i = 1,len        !  loop over gridpoint
          SALB(i,IWAVE,IRAD) = ( 1.-VCOVER(i) ) 
     &                 * ALBEDO(i,2,IWAVE,IRAD) +
     &                   VCOVER(i) * ALBEDO(i,1,IWAVE,IRAD)
        enddo
3000  CONTINUE
C
C----------------------------------------------------------------------
C
1000  CONTINUE
C
C----------------------------------------------------------------------
C
C     CALCULATION OF LONG-WAVE FLUX TERMS FROM CANOPY AND GROUND
C
C----------------------------------------------------------------------
C
      do i = 1,len                  !  loop over gridpoint
        TGS = MIN(TF,TG(i))*AREAS(i)
     &               + TG(i)*(1.-AREAS(i))
        TC4 = TC(i)**4
        TG4 = TGS**4
C
        ZKAT = 1./ZMEW(i) * ZLT(i) / VCOVER(i)
        ZKAT = MIN( 50.0_dbl_kind , ZKAT )
        ZKAT = MAX( 1.0e -5_dbl_kind, ZKAT )
        THERMK(i) = EXP(-ZKAT)
C
        FAC1 =  VCOVER(i) * ( 1.-THERMK(i) )
        FAC2 =  1.
        TGEFF4(i) =  FAC1 * TC4 
     &                + (1. - FAC1 ) * FAC2 * TG4
      enddo

      RETURN
      END
c
C==================SUBROUTINE RBRD=======================================
C
       SUBROUTINE RBRD(tc,tm,rbc,zlt,tg,z2,u2
     ^,             rd,rb,ta,cogr,cogs
     ^,             cog1,cog2,g,rdc,ra,tgs,wg
     ^,             rsoil,fg,hr,areas,respcp, len )

      use kinds

      implicit none

c========================================================================
C
C      CALCULATION OF RB AND RD AS FUNCTIONS OF U2 AND TEMPERATURES
C
c======================================================================== 


c++++++++++++++++++++++++++++++OUTPUT+++++++++++++++++++++++++++++++++++
c
C       RB (GRB)       CANOPY TO CAS AERODYNAMIC RESISTANCE (S M-1)
C       RD (GRD)       GROUND TO CAS AERODYNAMIC RESISTANCE (S M-1)
C       TA (GTA)       CAS TEMPERATURE (K)
c
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      integer (kind=int_kind), intent(in) :: len
      real (kind=dbl_kind), intent(in) ::
     *     g
      real (kind=dbl_kind), intent(in), dimension(len) ::
     *     tc,tm,rbc,zlt,tg,
     ^     z2,u2,
     ^     rdc,ra,tgs,wg,
     ^     rsoil,fg,hr,areas,
     ^     respcp
      real (kind=dbl_kind), intent(out), dimension(len) ::
     *     rb, rd, ta, cogr, cogs, cog1, cog2

!  local variables
      integer (kind=int_kind) :: i
      real (kind=dbl_kind) :: fac, fih
      real (kind=dbl_kind), dimension(len) :: d1, temdif
C
C-----------------------------------------------------------------------
C      RB       (RB)       : EQUATION (A9), SE-86
C-----------------------------------------------------------------------
C
      do i = 1,len                      !  loop over gridpoint                                                                      
        TEMDIF(i) = MAX( 0.1_dbl_kind,  TC(i)-TM(i) )
        FAC = ZLT(i)/890.* (TEMDIF(i)*20.0)**0.25
        RB(i)  = 1.0/(SQRT(U2(i))/RBC(i)+FAC)                                     
C                                                                      
C-----------------------------------------------------------------------
C      RD       (RD)       : EQUATION (A15), SE-86
C-----------------------------------------------------------------------
C                                                                      
        TEMDIF(i) = MAX( 0.1_dbl_kind, TGs(i)-TM(i) )                                  
        FIH = SQRT( 1.+9.*G*TEMDIF(i)*Z2(i)/(TGS(i)*U2(i)*U2(i)) )         
        RD(i)  = RDC(i) / (U2(i) * FIH)                                             
C                                                                      
C-----------------------------------------------------------------------
C      CALCULATION OF TA
C-----------------------------------------------------------------------
C
        D1(i) = 1./RA(i) + 1./RB(i) + 1./RD(i)
        TA(i) = ( TGS(i)/RD(i) + TC(i)/RB(i) +
     ^              TM(i)/RA(i) ) / D1(i)
C                                                                      
C-----------------------------------------------------------------------
C      CALCULATION OF SURFACE RESISTANCE COMPONENTS, SEE EQUATION (66)
C        OF SE-86 WITH SNOW AREA TERM ADDED
C-----------------------------------------------------------------------
C
        COGR(i) = (1.-WG(i))/(RSOIL(i)*FG(i)+RD(i))
        COGS(i) =  WG(i)/RD(i)
        COG1(i) = (COGS(i)+COGR(i)*HR(i))*(1.-AREAS(i))+AREAS(i)/RD(i)
        COG2(i) = (COGS(i) + COGR(i) ) * (1.-AREAS(i))+AREAS(i)/RD(i)
      enddo
C
      RETURN
      END
c      call respsib(len, nsib, nsoil, wopt, zm, www, wsat, 
c     ^             tgs, td, forcerestore, respfactor, respg, soilscale, 
c     ^             zmstscale, soilq10)
      subroutine respsib(len, nsib, nsoil, wopt, zm, www, wsat, 
     ^            tg, td, forcerestore, respfactor, respg, soilscale, 
     ^            zmstscale, soilq10)

      use kinds

      implicit none
     
      integer (kind=int_kind), intent(in) ::
     *    len, nsib, nsoil
      logical (kind=log_kind), intent(in) ::
     *    forcerestore
      real (kind=dbl_kind), intent(in), dimension(len) ::
     *    wopt, zm, wsat, tg
      real (kind=dbl_kind), intent(in), dimension(len,3) ::
     *    www
      real (kind=dbl_kind), intent(in), dimension(nsib,nsoil) ::
     *    td
      real (kind=dbl_kind), intent(in), dimension(nsib,nsoil+1) ::
     *    respfactor
c output      
      real (kind=dbl_kind), dimension(len) ::
     *    respg
      real (kind=dbl_kind), dimension(len,2) ::
     *    zmstscale
      real (kind=dbl_kind), dimension(len,nsoil+1) ::
     *     soilscale, soilq10

c local arrays
      real (kind=dbl_kind) :: woptzm
      real (kind=dbl_kind), dimension(len,2) :: b
      integer (kind=int_kind) :: i, l

c    Calculates the rate of CO2 efflux from soils, according to the "R-star"
c    approach of Denning et al (1996), adapted for use with the Bonan
c    6-layer soil thermodynamics module

c    Changed soil Q10 value for soil respiration from 2.0 to 2.4
c    following Raich and Schelsinger (1992, Tellus 44B, 81-89),
c    Scott Denning, 9/14/95

      if(.not.forcerestore) then

      	do i = 1,len

          ! Moisture effect on soil respiration, shallow and root zone
      	  woptzm = wopt(i)**zm(i)
      	  b(i,1) = (((100.*www(i,1))**zm(i)-woptzm)/
     >	         (woptzm - 100.**zm(i)))**2
      	  b(i,2) = (((100.*www(i,2))**zm(i)-woptzm)/
     >	         (woptzm - 100.**zm(i)))**2
      	  b(i,1) = min(b(i,1),10.0_dbl_kind)
      	  b(i,2) = min(b(i,2),10.0_dbl_kind)
      	  zmstscale(i,1) = 0.8*wsat(i)**b(i,1) + 0.2
      	  zmstscale(i,2) = 0.8*wsat(i)**b(i,2) + 0.2
 
          ! Temperature effect is a simple Q10, with a reference T of 25 C

          ! Deepest soil layers do not respire (no carbon below root zone)
            do L = 1, 2
      	      soilscale(i,L) = 0.0
      	      soilq10(i,L) = 0.0
            enddo

          ! Layers 3 through nsoil (root zone) use WWW(2) and TD(3:nSoil)
      	    do l = 3, nsoil
              soilQ10(i,L) = exp(0.087547 * (td(i,L) - 298.15))
      	      soilscale(i,L) = soilQ10(i,L) * zmstscale(i,2)
      	    enddo

          ! Surface soil uses TG and water layer 1
      	  soilQ10(i,nsoil+1) = exp(0.087547 * (tg(i) - 298.15))
      	  soilscale(i,nsoil+1) = soilQ10(i,nsoil+1) * zmstscale(i,1)

      	  ! Dimensionalize soil resp flux to balance annual budget
      	  respg(i) = respfactor(i,1) * soilscale(i,1)
      	  
      	    do L = 2, nSoil+1
      	      respg(i) = respg(i) + respfactor(i,L) * soilscale(i,L)
      	    enddo
      	
      	enddo
      	
c      	print*,'wopt,zm,www1,www2,wsat'
c      	print*,wopt(6),zm(6),www(6,1),www(6,2),wsat(6)
c      	
c      	print*,'zmstscale1,zmstscale2,b1,b2'
c      	print*,zmstscale(6,1),zmstscale(6,2),b(6,1),b(6,2)
c      	
c      	print*,' soilscale(6,L),soilQ10(6,L),respfactor(6,L)'
c      	print*,'td6'
c      	  do L = 1, nSoil+1
c            print*,soilscale(6,L),soilQ10(6,L),respfactor(6,L)
c      	    print*,td(6,l)
c          enddo
c
c      	print*,'respg6'
c      	print*,respg(6)
c
c  	
c      	print*,'wopt,zm,www1,www2,wsat'
c      	print*,wopt(26),zm(26),www(26,1),www(26,2),wsat(26)
c      	
c      	print*,'zmstscale1,zmstscale2,b1,b2'
c      	print*,zmstscale(26,1),zmstscale(26,2),b(26,1),b(26,2)
c      	
c      	print*,' soilscale(26,L),soilQ10(26,L),respfactor(26,L)'
c      	print*,'td26'
c      	  do L = 1, nSoil+1
c            print*,soilscale(26,L),soilQ10(26,L),respfactor(26,L)
c      	    print*,td(26,l)
c          enddo
c
c      	print*,'respg26'
c      	print*,respg(26)


      else  ! (FORCERESTORE CASE ... only two soil T levels available)

        do i = 1,len
        
          ! Moisture effect from TEM
      	  woptzm = wopt(i)**zm(i)
      	  b(i,2) = (((100.*www(i,2))**zm(i)-woptzm)/
     >	       (woptzm - 100.**zm(i)))**2
      	  b(i,2) = min(b(i,2),10.0_dbl_kind)
      	  zmstscale(i,1) = 0.8*wsat(i)**b(i,2) +0.2

          ! Temperature effect is Q10 =2.4 from ref T of 25 C
      	  soilQ10(i,1) = exp(0.087547 * (td(i,nsoil) - 298.15))
      	  soilscale(i,1) = soilQ10(i,1) * zmstscale(i,1)
      	
      	  ! Dimensionalize soil resp flux to balance annual budget
      	  respg(i) = respfactor(i,1) * soilscale(i,1)
      	
      	enddo
      	

      endif
      
      

      return
      end

C
C==================SUBROUTINE RNLOAD====================================
C
      subroutine rnload(len, nsib,
     *           radvbc,radvdc,radnbc,radndc,dlwbot,VCOVER,
     *           thermk,radfac,radn,radc3)

      use kinds

      implicit none
C
C=======================================================================
c
c    calculation of absorption of radiation by surface.  Note that
c       output from this calculation (radc3) only accounts for the 
c       absorption of incident longwave and shortwave fluxes.  The
c       total net radiation calculation is performed in subroutine
c       netrad.
C
C=======================================================================
c

c++++++++++++++++++++++++++++++OUTPUT+++++++++++++++++++++++++++++++++++
c
C       RADN(2,3)      INCIDENT RADIATION FLUXES (W M-2)
C       RADC3(2)       SUM OF ABSORBED RADIATIVE FLUXES (W M-2) 
C
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      integer (kind=int_kind), intent(in) ::
     *      len, nsib
      real (kind=dbl_kind), intent(in), dimension(len) ::
     *      radvbc, radvdc, radnbc, radndc,
     *      dlwbot, VCOVER, thermk
      real (kind=dbl_kind), intent(in), dimension(nsib,2,2,2) ::
     *      radfac

      real (kind=dbl_kind), intent(out), dimension(len,2) ::
     *      radc3
      real (kind=dbl_kind), intent(out), dimension(len,2,2) ::
     *      radn

      integer (kind=int_kind) :: i, iveg, iwave, irad

C-----------------------------------------------------------------------
C     CALCULATION OF SOIL MOISTURE STRESS FACTOR.
C     AVERAGE SOIL MOISTURE POTENTIAL IN ROOT ZONE (LAYER-2) USED AS
C     SOURCE FOR TRANSPIRATION.
C
C      RADN        (F(IW,IMU,O)) : EQUATION (19-22) , SE-86
C      RADC3       (FC,FGS)      : EQUATION (21,22) , SE-86
C-----------------------------------------------------------------------

         do i = 1,len
            RADc3(i,1) = 0.
            RADc3(i,2) = 0.
            radn(i,1,1) = radvbc(i)
            radn(i,1,2) = radvdc(i)
            radn(i,2,1) = radnbc(i)
            radn(i,2,2) = radndc(i)
         enddo

          do iveg=1,2
             do iwave=1,2
                do irad=1,2
                  do i = 1,len
                     radc3(i,iveg) = radc3(i,iveg) +
     *                  radfac(i,iveg,iwave,irad) *
     *                  radn(i,iwave,irad)
                  enddo
                enddo
             enddo
          enddo

c     absorbed downwelling radiation only

         do i = 1,len
            RADc3(i,1) = RADc3(i,1) + dlwbot(i) *
     *          VCOVER(i) * (1.- THERMK(i))
            RADc3(i,2) = RADc3(i,2) + dlwbot(i) *
     *         (1.-VCOVER(i) * (1.-THERMK(i)))
         enddo

      return
      end
C
C======================SUBROUTINE SIBSLV=================================
C
      SUBROUTINE SIBSLV(DTA,GRAV2,CP,HLTM,stefan,tgs, td, slamda, pi
     *,                 areas
     *,                 VENTMF,PSB,BPS,cg,ccx,HGDTG,EGDTG,HGDTC,HGDTH  
     *,                 EGDQM,HCDTG,ECDTG,HCDTC,ECDTC,EGDTC,HCDTH,ECDQM 
     *,                 AAG,AAC,AAM,BBG,BBC,BBM,ec,egs,RADT,FSS,FWS
     *,                 csoil, hc, hg, dtg, dtc, dth, dqm, fac1, tc
     *,                 len, sibdrv, forcerestore  )   
c========================================================================
C
C     Calculation of time increments in Tc, Tgs, Theta-m and Qm using an
C        implicit backwards method.  See equations (10-15), SA-92B.                                     
C
c     Longwave feedbacks are now included
c======================================================================== 

c++++++++++++++++++++++++++++++OUTPUT+++++++++++++++++++++++++++++++++++
c
C       DTC            CANOPY TEMPERATURE INCREMENT (K)
C       DTG            GROUND SURFACE TEMPERATURE INCREMENT (K)
C       DTH            MIXED LAYER POTENTIAL TEMPERATURE INCREMENT (K)
C       DQM            MIXED LAYER MIXING RATIO INCREMENT (KG KG-1)
C       ETMASS (FWS)   EVAPOTRANSPIRATION (MM)
C       HFLUX (FSS)    SENSIBLE HEAT FLUX (W M-2)
c
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      use kinds

      IMPLICIT none
C           
      integer len
      logical sibdrv, forcerestore   

      real (kind=dbl_kind), intent(in) ::
     *     pi, dta, hltm, cp, grav2, stefan            
      real (kind=dbl_kind), intent(in), dimension(len) ::
     *     VENTMF,tgs,PSB,BPS,ec
     *,    cg,ccx,HGDTG,EGDTG          
     *,    HGDTC,HGDTH,EGDQM,HCDTG      
     *,    ECDTG,HCDTC,ECDTC,HCDTH      
     *,    ECDQM,EGDTC,AAG,AAC          
     *,    AAM,BBG,BBC,BBM  
     *,    egs,td
     *,    csoil, hc, hg, fac1, tc, areas
     *,    slamda  
      real (kind=dbl_kind), intent(in), dimension(len,3) ::
     *     radt

      real (kind=dbl_kind), intent(out), dimension(len) ::
     *     fss, fws, dth, dqm, dtc
      real (kind=dbl_kind), intent(out), dimension(len,2) ::
     *     dtg

c     local variables
      real (kind=dbl_kind) ::
     *     timcon, tmcn2, dtai, ddthl, cpdta
      real (kind=dbl_kind), dimension(len) ::
     *     TEMV
      real (kind=selected_real_kind(13)), dimension(len,5,6) ::
     *     AVEC            
      real (kind=selected_real_kind(13)), dimension(len,5) ::
     *     BVEC            
      integer (kind=int_kind) :: i        
C                                                                       
         TIMCON = PI  / 86400.0                                         
         TMCN2  = 2.0 * TIMCON     
      DTAI  = 1.0 / DTA                                                 
      DDTHL = 1.0 / (DTA * HLTM)                                        
      CPDTA = CP  * DTA                                                 
C               
      DO I=1,len                                                   
        TEMV(I) = GRAV2 * VENTMF(i)
C                                                                       
C     TG EQUATION                                                       
C                                                                       
c950511 cg changed to csoil          
c   soil heat conduction term changed          
        AVEC(I,1,3) = HGDTC(I) + EGDTC(I)
        AVEC(I,1,4) = HGDTH(I)                                            
        AVEC(I,1,5) = EGDQM(I)        
        avec(i,1,2) = 0.0                                    
C                                                                       
C     TS EQUATION                                                       
C                                                                       
c950511 cg changed to csoil          
c   soil heat conduction term changed          
        AVEC(I,2,3) = HGDTC(I) + EGDTC(I)
        AVEC(I,2,4) = HGDTH(I)                                            
        AVEC(I,2,5) = EGDQM(I)     
        avec(i,2,1) = 0.0                                       
C                                                                       
C     TC EQUATION                                                       
C                                                                       
        AVEC(I,3,1) = (1.-areas(i))*(HCDTG(I) + ECDTG(I))
        AVEC(I,3,2) = areas(i)*(HCDTG(I) + ECDTG(I))
        AVEC(I,3,3) = ccx(I)   * DTAI + HCDTC(I) + ECDTC(I)
        AVEC(I,3,4) = HCDTH(I)                                            
        AVEC(I,3,5) = ECDQM(I) 

C                                                                       
C     USE PROVISIONAL VALUES OF THE FLUXES FOR RHS OF THE SYSTEM        
C                                                                       
        FSS(i) = (HC(I)  + HG(I))  / (CPDTA * BPS(i))                     
        FWS(i) = (ec(I) + egs(I)) * DDTHL                                
C                                                             
        AVEC(I,3,6) = RADT(i,1) - HC(I) * DTAI - ec(I) * DTAI     
      enddo 

      if(forcerestore) then
        do i = 1,len
c950511 cg changed to csoil          
c   soil heat conduction term changed          
          AVEC(I,1,1) = cg(i) * DTAI + HGDTG(I) + EGDTG(I)               
     *            + TMCN2    * csoil(I)                                   
          AVEC(I,2,2) = cg(i) * DTAI + HGDTG(I) + EGDTG(I)               
     *            + TMCN2    * csoil(I)                                   
c950511 cg changed to csoil
c   soil heat conduction term changed          
          AVEC(I,1,6) = RADT(i,2) - HG(I) * DTAI - egs(I) * DTAI            
     *              - TMCN2 * (tgs(I) - td(i)) * csoil(I)                   
          AVEC(I,2,6) = RADT(i,3) - HG(I) * DTAI - egs(I) * DTAI            
     *              - TMCN2 * (tgs(I) - td(i)) * csoil(I)                   
        enddo
      else
        do i = 1,len
          AVEC(I,1,1) = cg(i)   * DTAI + HGDTG(I) + EGDTG(I)               
     *            + slamda(i)
          AVEC(I,2,2) = cg(i)   * DTAI + HGDTG(I) + EGDTG(I)               
     *            + slamda(i)
c950511 cg changed to csoil
c   soil heat conduction term changed          
          AVEC(I,1,6) = RADT(i,2) - HG(I) * DTAI - egs(I) * DTAI            
     *              - slamda(i) * (tgs(I) - td(i))                   
          AVEC(I,2,6) = RADT(i,3) - HG(I) * DTAI - egs(I) * DTAI            
     *              - slamda(i) * (tgs(I) - td(i))                   
        enddo
      endif                                                        

      if(sibdrv) then                                                         
         DO I=1,len                                                   
C                                                                       
C     THETA EQUATION                                                    
C                                                                       
            AVEC(I,4,1) = 0.0                                 
            AVEC(I,4,2) = 0.0                                 
            AVEC(I,4,3) = 0.0                                 
            AVEC(I,4,4) = 1.0         
            AVEC(I,4,5) =  0.0                                                
C                                                                       
C     SH EQUATION                                                       
C                                                                       
            AVEC(I,5,1) = 0.0                                   
            AVEC(I,5,2) = 0.0                                   
            AVEC(I,5,3) =  0.0                                                
            AVEC(I,5,4) =  0.0                                                
            AVEC(I,5,5) = 1.0 

            AVEC(I,4,6) = 0.0                                      
            AVEC(I,5,6) = 0.0                                      
         enddo 
      else
         DO I=1,len                                                   
C                                                                       
C     THETA EQUATION                                                    
C                                                                       
            AVEC(I,4,1) = -temv(I) * AAG(I) * (1.-areas(i))                                 
            AVEC(I,4,2) = -temv(I) * AAG(I) * areas(i)                               
            AVEC(I,4,3) = -TEMV(I) * AAC(I)                                 
            AVEC(I,4,4) = -TEMV(I) * (AAM(I) - 1.0) +
     *                        PSB(i) * DTAI         
            AVEC(I,4,5) =  0.0                                                
C                                                                       
C     SH EQUATION                                                       
C                                                                       
            AVEC(I,5,1) = -TEMV(I)*BBG(I) * (1.-areas(i))                       
            AVEC(I,5,2) = -TEMV(I)*BBG(I) * areas(i)                                  
            AVEC(I,5,3) = -TEMV(I)*BBC(I)                                   
            AVEC(I,5,4) =  0.0                                                
            AVEC(I,5,5) = -TEMV(I) * (BBM(I) - 1.0) +
     *                            PSB(i) * DTAI 

            AVEC(I,4,6) = GRAV2 * FSS(i)                                      
            AVEC(I,5,6) = GRAV2 * FWS(i)                                      
         enddo 
      endif                                                         
C                                                                       
C     SOLVE 5 X 5 MATRIX EQUATION                                       
C
      CALL GAUSS(len,AVEC,5,6,BVEC)                              
C
      DO I=1,len                                                   
        FSS(i) = FSS(i) * CP                                              
        FwS(i) = FwS(i) * hltm                                              
        DTG(I,1) = BVEC(I,1)                                                
        DTG(I,2) = BVEC(I,2)                                                
        DTC(I) = BVEC(I,3)                                                
        DTH(i) = BVEC(I,4)                                                
        DQM(i) = BVEC(I,5)
      enddo                                                          
C                                                                       
      RETURN                                                            
      END                                                               
C
C=========================SUBROUTINE SNOW1==============================
C
      SUBROUTINE SNOW1(snoww,d,z0,rbc,rdc
     *,          areas,satcap,zlt,z1,z2,cc1,cc2,z0d
     *,          dd,asnow, nsib, len )                                          
C
C=======================================================================
C
C     CALCULATION OF EFFECTS OF SNOW COVER ON SURFACE MORPHOLOGY AND
C     MAXIMUM WATER STORAGE VALUES. 
C=======================================================================
C

c++++++++++++++++++++++++++++++OUTPUT+++++++++++++++++++++++++++++++++++
c
C       Z0             ROUGHNESS LENGTH (M)
C       D              ZERO PLANE DISPLACEMENT (M)
C       RBC            RB COEFFICIENT (C1) (S M-1)**1/2
C       RDC            RD COEFFICIENT (C2)
C       SATCAP(2)      INTERCEPTION CAPACITIES (M)
C       CANEX          FRACTION OF EXPOSED CANOPY (SNOW-FREE)
C       AREAS          GROUND SNOW COVER FRACTION
C
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      use kinds

      implicit none

c
c      declare arglist arrays
      integer (kind=int_kind), intent(in) ::
     *  nsib, len
      real (kind=dbl_kind),intent(in) ::
     *  asnow
      real (kind=dbl_kind),intent(in), dimension(len) ::
     *  z1, z2, dd, z0d, zlt, cc1, cc2
      real (kind=dbl_kind),intent(in), dimension(nsib,2) ::
     *  snoww

      real (kind=dbl_kind),intent(out), dimension(len) ::
     *  d, z0, rbc, rdc, areas
      real (kind=dbl_kind),intent(out), dimension(len,2) ::
     *  satcap

!  local variables
      integer (kind=int_kind) :: i
      real (kind=dbl_kind), dimension(len) ::
     *  canex
c
C
C----------------------------------------------------------------------
C    ALTERATION OF AERODYNAMIC TRANSFER PROPERTIES IN CASE OF SNOW
C    ACCUMULATION. CALCULATION OF MAXIMUM WATER STORAGE VALUES.
C
C      CANEX       (FRACTION OF CANOPY NOT COVERED BY SNOW)  
C      D           (SNOW-MODIFIED VALUE OF DD, USED IN ALL CALCULATIONS)  
C      Z0          (SNOW-MODIFIED VALUE OF Z0D, USED IN ALL CALCULATIONS)  
C      RBC         (SNOW-MODIFIED VALUE OF CC1, USED IN ALL CALCULATIONS)  
C      RDC         (SNOW-MODIFIED VALUE OF CC2, USED IN ALL CALCULATIONS)  
C      AREAS       (FRACTION OF GROUND COVERED BY SNOW)  
C      SATCAP(1)   (S-C)   : EQUATION (56) , SE-86, PAGE 741 SE-89
C      SATCAP(2)   (S-G)   : 0.002, SURFACE INTERCEPTION STORE
C----------------------------------------------------------------------
C
      do i = 1,len
         CANEX(i)  = 1.-( SNOWW(i,1)*5.-Z1(i))/
     *                           (Z2(i)-Z1(i))
         CANEX(i)  = MAX( 0.1_dbl_kind, CANEX(i) )
         CANEX(i)  = MIN( 1.0_dbl_kind, CANEX(i) )
         D(i)  = Z2(i) - ( Z2(i)-DD(i) ) * CANEX(i)
         Z0(i) = Z0D(i)/( Z2(i)-DD(i) ) * ( Z2(i)-D(i) )
         RBC(i)    = CC1(i)/CANEX(i)
         RDC(i)    = CC2(i)*CANEX(i)
         AREAS(i)    = MIN(1.0_dbl_kind, ASNOW*SNOWW(i,2))
         SATCAP(i,1) = ZLT(i)*0.0001 * CANEX(i)
c
cc Collatz-Bounoua change satcap(2) to 0.0002
C      SATCAP(i,2) = 0.002
          satcap(i,2) = 0.0002           ! lahouari
      enddo
C
      RETURN
      END
      subroutine soilprop( td, tg, slamda, shcap, www, poros, ztdep,
     *                  asnow, snoww, areas, tf, snomel, sandfrac,
     *                  nsib, len, nsoil )

      use kinds

      implicit none

c     this subroutine calculates the soil thermal properties heat capacity
c         (shcap) and conductivity (slamda). Phase changes are incorporated
c         into the heat capacity over a range of -0.5C to 0.5C.
c     slamda(n) is the conductivity between layers n and n+1 / delta z
c     layer 1 is the deepest layer, layer nsoil is immediately below the 
c         surface
c     treatment based on Bonan

c     argument list variables
      integer (kind=int_kind), intent(in) ::
     *   len, nsib, nsoil
      real (kind=dbl_kind), intent(in) ::
     *       asnow, snomel, tf 
      real (kind=dbl_kind), intent(in), dimension(len) ::
     *       tg, sandfrac, poros, areas, snoww
      real (kind=dbl_kind), intent(in), dimension(len,3) ::
     *       www
      real (kind=dbl_kind), intent(in), dimension(nsib,nsoil) ::
     *       td, ztdep

      real (kind=dbl_kind), intent(out), dimension(len,nsoil) ::
     *       slamda, shcap

c     local variables
      real (kind=dbl_kind) :: 
     *       shcapu, shcapf, shsnow,
     *       kf, ku, ksnow, 
     *       shcap1, klamda1, klamda2
      real (kind=dbl_kind), dimension(len) :: 
     *       zsnow,
     *       ksoil 
      real (kind=dbl_kind), dimension(len,3) :: 
     *       kiw, kii
      real (kind=dbl_kind), dimension(len,nsoil) :: 
     *       klamda
      integer (kind=int_kind) ::
     *       i,n

c     snow density assumed 0.25 that of water to convert from Bonan's constants
c        for ksnow and max effective snow depth
      ksnow = 0.085   
      do i = 1,len
c         ksoil(i) = (8.8 * sandfrac(i) + 
c     *                    2.92 * (1.-sandfrac(i)) )
c     *                           **(1.-poros(i))
         ksoil(i) = 6.0**(1.-poros(i))
      enddo
      do n = 1,3
         do i = 1,len
            kiw(i,n) = 0.6**(www(i,n)*poros(i))
            kii(i,n) = 2.2**(www(i,n)*poros(i))
         enddo
      enddo
c     heat capacity calculation and conductivity calculation

c     soil layers 1 and 2 ( deepest layers, use www(3) )
      do n = 1,2
         do i = 1,len
            SHCAPu  = ( 0.5*(1.-POROS(i)) + WWW(i,3)*  
     &                     POROS(i)) * 4.186E6
            shcapf =  0.5 * (1. + poros(i) * 
     &                     (www(i,3)-1.0)) * 4.186E6
            ku = (ksoil(i)*kiw(i,3)-0.15)*www(i,3) + 0.15 
            kf = (ksoil(i)*kii(i,3)-0.15)*www(i,3) + 0.15
            if(td(i,n).ge.tf+0.5) then
               shcap(i,n) = shcapu * ztdep(i,n)
               klamda(i,n) = ku
            else if(td(i,n).le.tf-0.5) then
               shcap(i,n) = shcapf * ztdep(i,n)
               klamda(i,n) = kf
            else
               shcap(i,n) = (0.5*(shcapu+shcapf) + 
     &               poros(i)*www(i,3)*snomel) * ztdep(i,n)
               klamda(i,n) = kf + (ku-kf)*(td(i,n)+0.5-tf)
            endif
         enddo
      enddo

c     soil layers 3,4,5 and nsoil ( intermediate layers, use www(2) )
      do n = 3,nsoil
         do i = 1,len
            SHCAPu  = ( 0.5*(1.-POROS(i)) + WWW(i,2)*  
     &                     POROS(i)) * 4.186E6
            shcapf =  0.5 * (1. + poros(i) * 
     &                     (www(i,2)-1.0)) * 4.186E6
            ku = (ksoil(i)*kiw(i,2)-0.15)*www(i,2) + 0.15 
            kf = (ksoil(i)*kii(i,2)-0.15)*www(i,2) + 0.15
            if(td(i,n).ge.tf+0.5) then
               shcap(i,n) = shcapu * ztdep(i,n)
               klamda(i,n) = ku
            else if(td(i,n).le.tf-0.5) then
               shcap(i,n) = shcapf * ztdep(i,n)
               klamda(i,n) = kf
            else
               shcap(i,n) = (0.5*(shcapu+shcapf) + 
     &              poros(i)*www(i,2)*snomel) * ztdep(i,n)
               klamda(i,n) = kf + (ku-kf)*(td(i,n)+0.5-tf)
            endif
         enddo
      enddo

c     soil layer nsoil ( top layer, use www(1) )
c     if snow covered add additional heat capacity due to snow
      do i = 1,len
            SHCAPu  = ( 0.5*(1.-POROS(i)) + WWW(i,1)*  
     &                     POROS(i)) * 4.186E6
            shcapf =  0.5 * (1. + poros(i) * 
     &                     (www(i,1)-1.0)) * 4.186E6
            ku = (ksoil(i)*kiw(i,1)-0.15)*www(i,1) + 0.15 
            kf = (ksoil(i)*kii(i,1)-0.15)*www(i,1) + 0.15
            if(td(i,nsoil).ge.tf+0.5) then
               shcap1 = shcapu 
               klamda1 = ku
            else if(td(i,nsoil).le.tf-0.5) then
               shcap1 = shcapf
               klamda1 = kf
            else
               shcap1 = 0.5*(shcapu+shcapf) + 
     &                   poros(i)*www(i,1)*snomel
               klamda1 = kf + (ku-kf)*(td(i,nsoil)+0.5-tf)
            endif
            if(tg(i).ge.tf+0.5) then
               klamda2 = ku
            else if(tg(i).le.tf-0.5) then
               klamda2 = kf
            else
               klamda2 = kf + (ku-kf)*(tg(i)+0.5-tf)
            endif
         zsnow(i) = min( max(1./asnow,snoww(i)) - 0.02, 0.25_dbl_kind )
         shsnow = (0.5 * 4.186e6 * zsnow(i) + shcap1 * 0.02) /
     &                 (zsnow(i)+0.02)
         shcap(i,nsoil) =  shcap(i,nsoil) * (1.-areas(i)) + areas(i) * 
     &      shcap(i,nsoil)*shsnow*(ztdep(i,nsoil)+zsnow(i)+0.02) /
     &      (shsnow*ztdep(i,nsoil)+shcap(i,nsoil) *
     &      (zsnow(i)+0.02)) * (ztdep(i,nsoil)+zsnow(i)+0.02)
         klamda1 = ksnow*klamda1 * (0.02+zsnow(i)) / 
     &      (ksnow*0.02 + klamda1*zsnow(i))

c    soil conductivities / delta z

         slamda(i,nsoil) = (1.-areas(i)) * 2.*klamda(i,nsoil)*klamda2 /
     *       (klamda2*ztdep(i,nsoil)+0.02*klamda(i,nsoil)) +
     *                areas(i) * klamda(i,nsoil)*klamda1 /
     *       (klamda1*ztdep(i,nsoil)*0.05 +
     *        klamda(i,nsoil)*(zsnow(i)+0.02))

      enddo

      do n = 1,nsoil-1
         do i = 1,len
            slamda(i,n) = 2.0 * klamda(i,n)*klamda(i,n+1) / 
     *                   (klamda(i,n)*ztdep(i,n+1)+
     *                         klamda(i,n+1)*ztdep(i,n))
         enddo
      enddo

      do i = 1,len
      enddo

      return
      end
      subroutine soiltherm( td, dtd, tgs, dtg, slamda, shcap, ztdep,
     *                  dt, nsib, len, nsoil )

      use kinds

      implicit none

c     this subroutine calculates the temperature increments dtd for
c         the soil, based on the soil thermodynamic model of Bonan
c     layer 1 is the deepest layer, layer nsoil is immediately below the 
c         surface
c     the time step is crank-nicholson. a tridiagonal matrix system is solved

c     argument list variables
      integer (kind=int_kind), intent(in) ::
     *       len, nsib, nsoil
      real (kind=dbl_kind), intent(in) ::
     *       dt
      real (kind=dbl_kind), intent(in), dimension(len) ::
     *       tgs, dtg
      real (kind=dbl_kind), intent(in), dimension(len,nsoil) ::
     *       shcap, slamda
      real (kind=dbl_kind), intent(in), dimension(nsib,nsoil) ::
     *       ztdep, td

      real (kind=dbl_kind), intent(out), dimension(len,nsoil) ::
     *       dtd

c     local variables    a(n)*dtd(n-1)+b(n)*dtd(n)+c(n)*dtd(n+1) = r(n)
      real (kind=dbl_kind), dimension(len,nsoil) ::
     *      a     ! lower sub-diagonal
     *,     b     ! diagonal
     *,     c     ! upper sib-diagonal
     *,     r     ! right hand side
      real (kind=dbl_kind) ::
     *      lamtem, rtem , dti, fac
      integer (kind=int_kind) :: i, n

      dti = 1. / dt   ! inverse time step

C     construct matrix
      do n = 1,nsoil
         do i = 1,len
            b(i,n) = shcap(i,n) * dti
            r(i,n) = 0.0
         enddo
      enddo
      do n = 1,nsoil-1
CDIR$ IVDEP
         do i = 1,len
            lamtem = -0.5 * slamda(i,n)
            rtem = slamda(i,n) * (td(i,n+1) - td(i,n))
            a(i,n+1) = lamtem
            c(i,n) = lamtem
            b(i,n) = b(i,n) - c(i,n)
            b(i,n+1) = b(i,n+1) - a(i,n+1)
            r(i,n) = r(i,n) + rtem
            r(i,n+1) = r(i,n+1) - rtem
         enddo
      enddo
      do i = 1,len
         r(i,nsoil) = r(i,nsoil) + slamda(i,nsoil) * (tgs(i)+dtg(i)
     *              - td(i,nsoil))
      enddo

c     eliminate lower diagonal
      do n = 1,nsoil - 1
CDIR$ IVDEP
         do i = 1,len
            fac = a(i,n+1) / b(i,n)
            b(i,n+1) = b(i,n+1) - c(i,n) * fac
            r(i,n+1) = r(i,n+1) - r(i,n) * fac
         enddo
      enddo
c     back-substitution
      do i = 1,len
         dtd(i,nsoil) = r(i,nsoil) / b(i,nsoil)
      enddo
      do n = nsoil-1,1,-1
CDIR$ IVDEP
         do i = 1,len
            dtd(i,n) = (r(i,n) - c(i,n) * dtd(i,n+1)) / b(i,n)
         enddo
      enddo

      return
      end
C
C===================SUBROUTINE SORTIN===================================
C
      SUBROUTINE SORTIN( EYY, PCO2Y, RANGE, GAMMAS, IC,len )
C
C=======================================================================
C
C     ARRANGES SUCCESSIVE PCO2/ERROR PAIRS IN ORDER OF INCREASING PCO2.
C       ESTIMATES NEXT GUESS FOR PCO2 USING COMBINATION OF LINEAR AND
C       QUADRATIC FITS.
C
C=======================================================================

      use kinds
C
      implicit none
                               
      integer (kind=int_kind), intent(in) ::
     *     len, ic  
      real (kind=dbl_kind), intent(in), dimension(len) ::  
     *     gammas, range                 
      real (kind=dbl_kind), intent(inout), dimension(len,6) ::  
     *     EYY, PCO2Y

c     work arrays

      real (kind=dbl_kind), dimension(len) ::  
     *     eyyi1,eyyi2,eyyi3,eyyis,
     *     eyyisp,pco2yis,pco2yisp,
     *     pco2b,
     *     pco2yi1,pco2yi2,pco2yi3
      real (kind=dbl_kind) ::  
     *     aterm, bterm, cterm, pco2yq, cc1, cc2, bc1, bc2, ac1, ac2,
     *     pco2yl, a, b, pmin, emin, one, denom
      integer (kind=int_kind), dimension(len) ::
     *      is
      logical (kind=log_kind), dimension(len) ::
     *      bitx
      integer (kind=int_kind) ::
     *      i, ix, i1, i2, i3, isp, n, l, j

C
      one = 1.0
      IF( IC .eq. 1 ) then
        do i = 1,len
          PCO2Y(i,1) = GAMMAS(i) + 0.5*RANGE(i)
        enddo
      else IF( IC .eq. 2 ) then
        do i = 1,len
          PCO2Y(i,2) = GAMMAS(i)
     &                 + RANGE(i)*( 0.5 - 0.3*SIGN(one,EYY(i,1)) )
        enddo
      else IF( IC .eq. 3 ) then
        do i = 1,len
          PCO2Y(i,3) = PCO2Y(i,1)- (PCO2Y(i,1)-PCO2Y(i,2))
     &                 /(EYY(i,1)-EYY(i,2)+1.E-10)*EYY(i,1)
C
          PMIN = MIN( PCO2Y(i,1), PCO2Y(i,2) )
          EMIN = MIN(   EYY(i,1),   EYY(i,2) )
          IF ( EMIN .GT. 0. .AND. PCO2Y(i,3) .GT. PMIN )
     &                 PCO2Y(i,3) = GAMMAS(i)
        enddo
      else
C
        N = IC - 1
        do l = 1,len
          bitx(l) = abs(eyy(l,n)).gt.0.1_dbl_kind
          if(.not.bitx(l)) pco2y(l,ic) = pco2y(l,n)
        enddo
        do l = 1,len
          if(bitx(l)) then
            DO 1000 J = 2, N
              A = EYY(l,J)
              B = PCO2Y(l,J)
              DO 2000 I = J-1,1,-1
                IF(EYY(l,I) .le. A ) go to 100
                EYY(l,I+1) = EYY(l,I)
                PCO2Y(l,I+1) = PCO2Y(l,I)
2000          CONTINUE
              i = 0
 100          continue
              EYY(l,I+1) = A
              PCO2Y(l,I+1) = B
1000        CONTINUE
          endif
        enddo
C
C-----------------------------------------------------------------------
C
        do l = 1,len
          if(bitx(l)) then
            PCO2B(l) = 0.
            IS(l)    = 1
          endif
        enddo

        DO 3000 IX = 1, N
          do l = 1,len
            if(bitx(l)) then
              IF( EYY(l,IX) .LT. 0.0_dbl_kind )  then
                PCO2B(l) = PCO2Y(l,IX)
                IS(l) = IX
              endif
            endif
          enddo
3000    CONTINUE
        do l = 1,len
          if(bitx(l)) then
            I1 = IS(l)-1
            I1 = MAX(1, I1)
            I1 = MIN(N-2, I1)
            I2 = I1 + 1
            I3 = I1 + 2
            ISP   = IS(l) + 1
            ISP = MIN0( ISP, N )
            IS(l) = ISP - 1
            eyyisp(l) = eyy(l,isp)
            eyyis(l) = eyy(l,is(l))
            eyyi1(l) = eyy(l,i1)
            eyyi2(l) = eyy(l,i2)
            eyyi3(l) = eyy(l,i3)
            pco2yisp(l) = pco2y(l,isp)
            pco2yis(l) = pco2y(l,is(l))
            pco2yi1(l) = pco2y(l,i1)
            pco2yi2(l) = pco2y(l,i2)
            pco2yi3(l) = pco2y(l,i3)
          endif
        enddo
C
        do l = 1,len
          if(bitx(l)) then
            PCO2YL=PCO2Yis(l)
     &            - (PCO2Yis(l)-PCO2Yisp(l))
     &            /(EYYis(l)-EYYisp(l))*EYYis(l)
C
C   METHOD USING A QUADRATIC FIT
C
            AC1 = EYYi1(l)**2 - EYYi2(l)**2
            AC2 = EYYi2(l)**2 - EYYi3(l)**2
            BC1 = EYYi1(l) - EYYi2(l)
            BC2 = EYYi2(l) - EYYi3(l)
            CC1 = PCO2Yi1(l) - PCO2Yi2(l)
            CC2 = PCO2Yi2(l) - PCO2Yi3(l)
            denom = 1. / (bc1*ac2 - ac1*bc2)
            BTERM = (CC1*AC2-CC2*AC1) * denom
            ATERM = (cc2*bc1 - cc1*bc2) * denom
            CTERM = PCO2Yi2(l) - eyyi2(l) * (bterm + eyyi2(l)*aterm)
            PCO2YQ= MAX( cterm, PCO2B(l) )
            PCO2Y(l,IC) = ( PCO2YL+PCO2YQ)/2.
          endif
        enddo
C
      endif
      do i = 1,len
        pco2y(i,ic) = max(pco2y(i,ic),0.01_dbl_kind)
      enddo
C
      RETURN
      END
C
C=====================SUBROUTINE UPDAT2=================================
C
      SUBROUTINE UPDAT2(snoww,capac,snofac,ect,eci,egi
     ^,             egs,hlat,www,pi,cg,dtd,dtg,dtc
     ^,             dtt,roff,tc,td,tg,bee, poros
     ^,             satco,slope,phsat,zdepth,ecmass
     ^,             egmass,hflux,shf,tf,snomel,asnow
     ^,             ccx,csoil,chf,etmass,hc,hg,areas
     ^,             q3l,q3o,exo,qqq,zmelt,cw, len, nsib
     ^,             nsoil, forcerestore )

      use kinds

      implicit none

      integer (kind=int_kind), intent(in) ::
     *      len, nsib, nsoil
      logical (kind=log_kind), intent(in) ::
     *      forcerestore
      real (kind=dbl_kind), intent(in) ::
     *      hlat, pi, dtt, tf, snomel, asnow, cw, snofac
      real (kind=dbl_kind), intent(in), dimension(len) ::
     *      bee, satco, slope, phsat, poros, chf, shf, csoil,
     *      tc, tg, ccx, cg
      real (kind=dbl_kind), intent(in), dimension(nsib,3) ::
     *      zdepth
      real (kind=dbl_kind), intent(in), dimension(nsib,nsoil) ::
     *      td

      real (kind=dbl_kind), intent(inout), dimension(len) ::
     *     eci, ect, egi, egs, roff, hc, hg, dtc, areas
      real (kind=dbl_kind), intent(inout), dimension(len,2) ::
     *     dtg
      real (kind=dbl_kind), intent(inout), dimension(len,3) ::
     *     www
      real (kind=dbl_kind), intent(inout), dimension(nsib,2) ::
     *     capac, snoww

      real (kind=dbl_kind), intent(out), dimension(len) ::
     *     ecmass, egmass, q3l, q3o, exo, hflux, etmass, zmelt
      real (kind=dbl_kind), intent(out), dimension(len,3) ::
     *     qqq
      real (kind=dbl_kind), intent(out), dimension(len,nsoil) ::
     *     dtd

     
C=======================================================================
C
C     UPDATING OF ALL HYDROLOGICAL PROGNOSTIC VARIABLES.  SNOW AND
C        RUNOFF CALCULATIONS (SEE ALSO INTER2).  SUBROUTINES SNOW2 AND
C        RUN2 OF 1D MODEL ARE INLINED IN THIS CODE.
C
C=======================================================================
C

c++++++++++++++++++++++++++++++OUTPUT+++++++++++++++++++++++++++++++++++
c
C       DTC            CANOPY TEMPERATURE INCREMENT (K)
C       DTD            DEEP SOIL TEMPERATURE INCREMENT (K)
C       DTG            GROUND SURFACE TEMPERATURE INCREMENT (K)
C       WWW(3)         GROUND WETNESS 
C       CAPAC(2)       CANOPY/GROUND LIQUID INTERCEPTION STORE (M)
C       SNOWW(2)       CANOPY/GROUND SNOW INTERCEPTION STORE (M)
C       ROFF           RUNOFF (MM)
C       ETMASS (FWS)   EVAPOTRANSPIRATION (MM)
C       HFLUX (FSS)    SENSIBLE HEAT FLUX (W M-2)
C
c++++++++++++++++++++++++++DIAGNOSTICS++++++++++++++++++++++++++++++++++
C
C       ECMASS         CANOPY EVAPOTRANSPIRATION (MM)
C       EGMASS         GROUND EVAPOTRANSPIRATION (MM)
c
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      real (kind=dbl_kind) ::
     &     rsnow, facks,
     &     hlati, 
     &     realc, realg, excess, qmax, qmin, 
     &     rdenom, denom, dpdwdz, props, tgs, avkmax, avkmin, div,
     &     rsame, pmin, wmin, pmax, wmax, egsdif, ectdif, extrak,
     &     facl, dtsg2, dtsg3, cool, zmelt2, dtsg, heat,
     &     exmelt, exheat, safe, avheat, avmelt, avk, pows,
     &     avex, freeze
      real (kind=dbl_kind), dimension(len) ::
     &     cctt,
     &     ts, dts, flux, tsnow, snowhc,
     &     dpdw, cct,
     &     fluxef, dts2
      real (kind=dbl_kind), dimension(len,2) ::
     &     AAA , BBB  , CCC
      real (kind=dbl_kind), dimension(len,3) ::
     &     TEMW, TEMWP, TEMWPP
      integer (kind=int_kind) :: i, l, iveg, ksoil

      if(forcerestore) then
         ksoil = nsoil
      else
         ksoil = 3
      endif
C
C----------------------------------------------------------------------
C    INTERCEPTION LOSSES APPLIED TO SURFACE WATER STORES.                                              
C    EVAPORATION LOSSES ARE EXPRESSED IN J M-2 : WHEN DIVIDED BY
C    ( HLAT*1000.) LOSS IS IN M M-2. MASS TERMS ARE IN KG M-2 DT-1
C    INTERCEPTION AND DRAINAGE TREATED IN INTER2.
C
C      CAPAC/SNOWW(1) (M-C)   : EQUATION (3)  , SE-86
C      CAPAC/SNOWW(2) (M-G)   : EQUATION (4)  , SE-86
C----------------------------------------------------------------------
C 
      hlati = 1. / hlat
      do i = 1,len                                                                    
         RSNOW = SNOWW(i,1)/(SNOWW(i,1)+CAPAC(i,1)+1.E-10)
         FACKS = 1. + RSNOW * ( SNOFAC-1. )
         IF ( (ECT(i)+ECI(i)) .le. 0.) then
            ECI(i) = ECT(i)+ECI(i)
            ECT(i) = 0.
            FACKS = 1. / FACKS
         endif
         CAPAC(i,1) = CAPAC(i,1)-( 1.-RSNOW )*ECI(i)*FACKS*hlati*0.001
         SNOWW(i,1) = SNOWW(i,1)-     RSNOW  *ECI(i)*FACKS*hlati*0.001
         snoww(i,1) = max(snoww(i,1),0.0_dbl_kind)
         capac(i,1) = max(capac(i,1),0.0_dbl_kind)
         ECMASS(i) = ECI(i)*FACKS *hlati
      enddo
c      do i = 1,len
c         if(snoww(i,1).lt.0.0)
c     *      print *,'snoww1 after updat2 100 ',i,snoww(i,1)
c         if(capac(i,1).lt.0.0)
c     *      print *,'capac1 after updat2 100 ',i,capac(i,1)
c      enddo
C
      do i = 1,len
         RSNOW = SNOWW(i,2)/(SNOWW(i,2)+CAPAC(i,2)+1.e-10)
         FACKS = 1. + RSNOW * ( SNOFAC-1. )
         IF ( (EGS(i)+EGI(i)) .le. 0. ) then
            EGI(i) = EGS(i)+EGI(i)
            EGS(i)= 0.
            FACKS = 1. / FACKS
         endif
         CAPAC(i,2) = CAPAC(i,2)-( 1.-RSNOW )*EGI(i)*FACKS*hlati*0.001
         SNOWW(i,2) = SNOWW(i,2)-     RSNOW  *EGI(i)*FACKS*hlati*0.001
         snoww(i,2) = max(snoww(i,2),0.0_dbl_kind)
         capac(i,2) = max(capac(i,2),0.0_dbl_kind)
         EGMASS(i) = EGI(i)*FACKS *hlati
      enddo
c      do i = 1,len
c         if(snoww(i,2).lt.0.0)
c     *      print *,'snoww2 after updat2 200 ',i,snoww(i,2)
c         if(capac(i,2).lt.0.0)
c     *      print *,'capac2 after updat2 200 ',i,capac(i,2)
c      enddo
C
C----------------------------------------------------------------------
C    DUMPING OF SMALL CAPAC VALUES ONTO SOIL SURFACE STORE
C----------------------------------------------------------------------
C
      DO 1000 IVEG = 1, 2
         do i = 1,len
            IF ( (SNOWW(i,iveg)+CAPAC(i,IVEG)) .le. 0.00001 ) then
               WWW(i,1) = WWW(i,1) + (SNOWW(i,IVEG)+CAPAC(i,IVEG)) /
     *                  ZDEPTH(i,1) 
               CAPAC(i,IVEG) = 0.
               SNOWW(i,IVEG) = 0.
            endif
         enddo
1000  CONTINUE
c      do i = 1,len
c         if(www(i,1).le.0.0)print *,'www after updat2 1000 ',i,www(i,1)
c      enddo
C
C
C=======================================================================
c------------------SNOW2 INLINED-------------------------------------
C----------------------------------------------------------------------
C    SNOWMELT / REFREEZE CALCULATION                                  
C----------------------------------------------------------------------
C                                                                     
C     CALCULATION OF SNOWMELT AND MODIFICATION OF TEMPERATURES       
C                                                                   
C     MODIFICATION DEALS WITH SNOW PATCHES:                        
C          TS < TF, TSNOW = TS                                    
C          TS > TF, TSNOW = TF                                 
C                                                             
C=======================================================================
C                                                                      
      DO 1002 IVEG = 1, 2                                             
C
         REALC = (2 - IVEG)*1.
         REALG = (IVEG - 1)*1.
C
         do i = 1,len
            CCTT(i) = REALC*CCX (i) +  REALG*CG(i)
            CCT(i)  = REALC*CCX(i)  +  REALG*CSOIL(i)
            TS(i)   = REALC*TC(i)   +  REALG*TG(i)
            DTS(i)  = REALC*DTC(i)  +  REALG*DTG(i,1)
            DTS2(i)  = REALC*DTC(i)  +  REALG*DTG(i,2)
            FLUX(i) = REALC*CHF(i)  +  REALG*
     *         ( (1.-areas(i))*DTG(i,1)+
     *                areas(i)*dtg(i,2)  )*cg(i) /DTT
c  fluxef moved up here to conserve energy
            fluxef(i) = ( shf(i) - flux(i)) * realg
C
            TSNOW(i) = MIN ( TF-0.01, TS(i) )
c950511 adjust for different heat capacity of snow          
c           SNOWHC(i) = MIN( 0.05_dbl_kind, SNOWW(i,IVEG) )*CW * REALG * 0.5
            SNOWHC(i) = MIN( 0.05_dbl_kind, SNOWW(i,IVEG) ) * CW * REALG
            ZMELT(i) = 0.
         enddo
C
         do i = 1,len  ! this scalar loop needs vector optimization
      IF ( SNOWW(i,IVEG) .GT. 0. ) GO TO 102
      IF ( ( TS(i)+DTS(i)) .GT. TF ) GO TO 502
C-----------------------------------------------------------------------
C
C     NO SNOW  PRESENT, SIMPLE THERMAL BALANCE WITH POSSIBLE FREEZING.
C
C-----------------------------------------------------------------------
      FREEZE = MIN ( 0.0_dbl_kind, (FLUX(i)*DTT-( TF-0.01 - TS(i))*CCTT(i)))
      SNOWW(i,IVEG) = MIN( CAPAC(i,IVEG), - FREEZE/SNOMEL )
      ZMELT(i) = CAPAC(i,IVEG) - SNOWW(i,IVEG)
      CAPAC(i,IVEG) = 0.
      DTS(i) = DTS(i) + SNOWW(i,IVEG)*SNOMEL/CCTT(i)
      GO TO 502
C
C-----------------------------------------------------------------------
C
C     SNOW PRESENT                                                      
C                                                                      
C---------------------------------------------------------------------
102   CONTINUE                                                       
C                                                                   
      IF ( TS(i) .LT. TF .AND. (TS(i)+DTS(i)) .LT. TF ) GO TO 502           
      IF ( ts(i) .GT. TF ) GO TO 202                                 
C----------------------------------------------------------------
C                                                               
C     SNOW PRESENT : TS < TF,  TS+DTS > TF                     
C                                                             
C------------------------------------------------------------
      AVEX = FLUX(i) - ( TF-0.01 - TS(i) ) * CCTT(i)/DTT             
      AVMELT = ( AVEX/SNOMEL * (AREAS(i)*REALG + REALC ) )*DTT
      ZMELT(i) = MIN( AVMELT, SNOWW(i,IVEG) )  
      SNOWW(i,IVEG) = SNOWW(i,IVEG) - ZMELT(i)                   
      AVHEAT = AVEX*( 1.-AREAS(i) )*REALG +
     *     ( AVMELT-ZMELT(i) )*SNOMEL/DTT
      AREAS(i) = MIN( 0.999_dbl_kind, ASNOW*SNOWW(i,2) )                        
      SAFE = MAX( ( 1.-AREAS(i)*REALG ), 1.e-8_dbl_kind )                      
      DTS(i) = TF-0.01 - TS(i) + AVHEAT / ( CCTT(i)*SAFE )*DTT             
      GO TO 502                                                         
C----------------------------------------------------------------------
C                                                                     
C     SNOW PRESENT AND TS > TF : GROUND ONLY.                        
C                                                                   
C------------------------------------------------------------------
202   CONTINUE                                                    
C                                                                
      EXHEAT = CCT(i)*( 1.001-MAX(0.1_dbl_kind,AREAS(i))) * DTS(i)              
      EXMELT = FLUX(i)*DTT - EXHEAT                             
      HEAT = EXHEAT                                         
      DTSG = EXHEAT / ( CCT(i)*(1.001-AREAS(i) ))                
      IF ( (TS(i)+DTSG) .GT. TF ) GO TO 302                  
      HEAT = ( TF-0.01 - TS(i) ) * ( CCT(i)*(1.-AREAS(i)) )       
      DTSG = TF-0.01 - TS(i)                               
C                                                      
302   EXMELT = EXMELT + EXHEAT - HEAT                 
C                                                    
      IF( EXMELT .LT. 0. ) GO TO 402                
      ZMELT(i) = EXMELT/SNOMEL                                            
      IF( ASNOW*(SNOWW(i,IVEG)-ZMELT(i)) .LT. 1. )                         
     &           ZMELT(i) = MAX( 0.0_dbl_kind, SNOWW(i,IVEG) - 1./ASNOW )           
      SNOWW(i,IVEG) = SNOWW(i,IVEG) - ZMELT(i)                             
      EXMELT = EXMELT - ZMELT(i)*SNOMEL                               
      ZMELT2 = EXMELT/ ( CCT(i)*( TS(i)-TF )*ASNOW + SNOMEL )           
      ZMELT2 = MIN( ZMELT2, SNOWW(i,IVEG) )                        
      ZMELT(i) = ZMELT(i) + ZMELT2                                    
      SNOWW(i,IVEG) = SNOWW(i,IVEG) - ZMELT2                       
      EXMELT = EXMELT - ZMELT2*( CCT(i)*( TS(i)-TF )*ASNOW + SNOMEL )
      DTS(i)  = DTSG + EXMELT/CCT(i)                                
      GO TO 502                                              
C                                                           
402   COOL = MIN( 0.0_dbl_kind, TF-0.01 -(TS(i)+DTSG)) * CCT(i)*(1.-AREAS(i))        
      DTSG2 = MAX ( COOL, EXMELT ) / ( CCT(i)*( 1.001-AREAS(i) ) )       
      EXMELT = EXMELT - DTSG2*CCT(i)*(1.-AREAS(i))                      
      DTSG3 =EXMELT/CCTT(i)                                         
      DTS(i) = DTSG + DTSG2 + DTSG3                                       
C                                                                     
502   CONTINUE  
         enddo                                                       
c
         do i = 1,len
            WWW(i,1) = WWW(i,1) + ZMELT(i) /
     *                     ZDEPTH(i,1)  
C
            DTC(i) = DTC(i)*REALG + DTS(i)*REALC
            DTG(i,1) = DTG(i,1)*REALC + DTS(i)*REALG
         enddo
C
1002  CONTINUE
C
      if(forcerestore) then
         do i = 1,len
c960320 fluxef calculation moved up to conserve energy
c950511 changed cg to csoil          
            DTD(i,nsoil) = FLUXEF(i) /
     *                ( csoil(i) * 2.*SQRT( PI*365. ) ) * DTT
         enddo
      endif
      do i = 1,len
C
c------------------END SNOW2  -------------------------------------
C----------------------------------------------------------------------
C    EVAPOTRANSPIRATION LOSSES APPLIED TO SOIL MOISTURE STORE.
C    EXTRACTION OF TRANSPIRATION LOSS FROM ROOT ZONE, SOIL EVAPORATION.
C
C      ECT         (E-DC)  : EQUATION (5,6), SE-86
C      EGS         (E-S)   : EQUATION (5)  , SE-86
C----------------------------------------------------------------------
C
         FACL   = hlati*0.001/ZDEPTH(i,2)
         EXTRAK = ECT(i)*FACL
         EXTRAK = MIN( EXTRAK, WWW(i,2)*0.5 )
         ECTDIF = ECT(i) - EXTRAK/FACL
         ECT(i)    = EXTRAK/FACL
         HC(i)     = HC(i) + ECTDIF
         ECMASS(i) = ECMASS(i) + ECT(i)*hlati
         WWW(i,2) = WWW(i,2) - ECT(i)*FACL
C
         FACL   = 0.001*hlati/ZDEPTH(i,1)
         EXTRAK = EGS(i)*FACL
         EXTRAK = MIN( EXTRAK, WWW(i,1) *0.5 )
         EGSDIF = EGS(i) - EXTRAK/FACL
         EGS(i)    = EXTRAK/FACL
         HG(i)     = HG(i) + EGSDIF
         EGMASS(i) = EGMASS(i) + EGS(i)*hlati
         WWW(i,1) = WWW(i,1) - EGS(i)*FACL

C-----------------------------------------------------------------------
C    CALCULATION OF TOTAL MOISTURE AND SENSIBLE HEAT FLUXES FROM SURFACE.
C-----------------------------------------------------------------------

         ETMASS(i) = ECMASS(i) + EGMASS(i)
         HFLUX(i)  = (HC(i)+HG(i)) / DTT
      enddo
C
c========================================================================
c------------------RUN2 INLINED-------------------------------------
c========================================================================
C----------------------------------------------------------------------
C    CALCULATION OF INTERFLOW, INFILTRATION EXCESS AND LOSS TO
C    GROUNDWATER .  ALL LOSSES ARE ASSIGNED TO VARIABLE 'ROFF' .
C----------------------------------------------------------------------
C
C
      DO 1001 I = 1, 3
C
         do l = 1,len
            TEMW(l,I)   = MAX( 0.03_dbl_kind, WWW(l,I) )
            TEMWP(l,I)  = TEMW(l,I) ** ( -BEE(l) )
            TEMWPP(l,I) = MIN( 1.0_dbl_kind, TEMW(l,I))**( 2.*BEE(l)+ 3. )
         enddo
1001  CONTINUE
C
C-----------------------------------------------------------------------
C
C    CALCULATION OF GRAVITATIONALLY DRIVEN DRAINAGE FROM W(3) : TAKEN
C    AS AN INTEGRAL OF TIME VARYING CONDUCTIVITY.
C
C     qqq(3) (Q3) : EQUATION (62) , SE-86
C
C    QQQ(3) IS AUGMENTED BY A LINEAR LOSS TERM RECOMMENDED BY LISTON (1992)
C-----------------------------------------------------------------------
C
      do i = 1,len
         POWS = 2.*BEE(i)+2.
         qqq(i,3) = TEMW(i,3)**(-POWS) + SATCO(i)/
     &                (ZDEPTH(i,3) )*
     &                  SLOPE(i)*POWS*DTT
         qqq(i,3) = qqq(i,3) ** ( 1. / POWS )
         qqq(i,3) = - ( 1. / qqq(i,3) - WWW(i,3) ) *
     &                  ZDEPTH(i,3) / DTT
         qqq(i,3) = MAX( 0.0_dbl_kind, qqq(i,3) )
         qqq(i,3) = MIN( qqq(i,3), WWW(i,3)*
     &                        ZDEPTH(i,3)/DTT )
         q3o(i) = qqq(i,3) * dtt
C
c         Q3l(i) = 0.002*ZDEPTH(i,3)*0.5 / 86400.* 
c     *            max(0.0_dbl_kind,(www(i,3)-0.01)/0.99 )
         q3l(i) = 0.00
         qqq(i,3) = qqq(i,3) + q3l(i)
         q3l(i) = q3l(i) * dtt
C
C----------------------------------------------------------------------
C
C    CALCULATION OF INTER-LAYER EXCHANGES OF WATER DUE TO GRAVITATION
C    AND HYDRAULIC GRADIENT. THE VALUES OF W(X) + DW(X) ARE USED TO
C    CALCULATE THE POTENTIAL GRADIENTS BETWEEN LAYERS.
C    MODIFIED CALCULATION OF MEAN CONDUCTIVITIES FOLLOWS MILLY AND
C    EAGLESON (1982 ), REDUCES RECHARGE FLUX TO TOP LAYER.
C
C      DPDW           : ESTIMATED DERIVATIVE OF SOIL MOISTURE POTENTIAL
C                       WITH RESPECT TO SOIL WETNESS. ASSUMPTION OF
C                       GRAVITATIONAL DRAINAGE USED TO ESTIMATE LIKELY
C                       MINIMUM WETNESS OVER THE TIME STEP.
C
C      QQQ  (Q     )  : EQUATION (61) , SE-86
C             I,I+1
C            -
C      AVK  (K     )  : EQUATION (4.14) , ME-82
C             I,I+1
C
C----------------------------------------------------------------------
C
         WMAX = MAX( WWW(i,1), WWW(i,2), WWW(i,3), 0.05_dbl_kind )
         WMAX = MIN( WMAX, 1.0_dbl_kind )
         PMAX = WMAX**(-BEE(i))
         WMIN = (PMAX-2.*poros(i)/( PHSAT(i)*
     &    (ZDEPTH(i,1)+2.*ZDEPTH(i,2)+ZDEPTH(i,3))))
     &           **(-1./BEE(i))
         WMIN = MIN( WWW(i,1), WWW(i,2), WWW(i,3), WMIN )
         WMIN = MAX( WMIN, 0.02_dbl_kind )
         PMIN = WMIN**(-BEE(i))
         DPDW(i) = PHSAT(i)*( PMAX-PMIN )/( WMAX-WMIN )
      enddo
C
      DO 2001 I = 1, 2
C
         do l = 1,len
      RSAME = 0.
      AVK  = TEMWP(l,I)*TEMWPP(l,I) - TEMWP(l,I+1)*TEMWPP(l,I+1)
      DIV  = TEMWP(l,I+1) - TEMWP(l,I)
      IF ( ABS(DIV) .LT. 1.E-6 ) RSAME = 1.
      AVK = SATCO(l)*AVK /
     &            ( ( 1. + 3./BEE(l) ) * DIV + RSAME )
      AVKMIN = SATCO(l) * MIN( TEMWPP(l,I), TEMWPP(l,I+1) )
      AVKMAX = SATCO(l) * MAX( TEMWPP(l,I), TEMWPP(l,I+1) )*1.01
      AVK = MAX( AVK, AVKMIN )
      AVK = MIN( AVK, AVKMAX )
c
cc  Collatz-Bounoua change to effective hydraulic conductivity making 
c   it 10x harder for water to move up than down if the upper soil layer
c   is wetter than lower soil layer.
c----------------------------------------------------------------------
      if (www(l,i) .lt. www(l,i+1)) avk = 0.1 * avk     ! lahouari
C
C-----------------------------------------------------------------------
C     CONDUCTIVITIES AND BASE FLOW REDUCED WHEN TEMPERATURE DROPS BELOW
C        FREEZING.
C-----------------------------------------------------------------------

      TSNOW(l) = MIN ( TF-0.01, TG(l) )
      TGS = TSNOW(l)*AREAS(l) + TG(l)*(1.-AREAS(l))
      TS(l)    = TGS*(2-I) + TD(l,ksoil)*(I-1)
      PROPS = ( TS(l)-(TF-10.) ) / 10.
      PROPS = MAX( 0.05_dbl_kind, MIN( 1.0 _dbl_kind, PROPS ) )
      AVK  = AVK * PROPS
      qqq(l,3)  = qqq(l,3) * PROPS
      q3l(l) = q3l(l) * props
      q3o(l) = q3o(l) * props

C-----------------------------------------------------------------------
C     BACKWARD IMPLICIT CALCULATION OF FLOWS BETWEEN SOIL LAYERS.
C-----------------------------------------------------------------------

      DPDWDZ = DPDW(l)*2.*poros(l)/
     &           ( ZDEPTH(l,I) + ZDEPTH(l,I+1) )
      AAA(l,I) = 1. + AVK*DPDWDZ*
     &           ( 1./ZDEPTH(l,I)+1./ZDEPTH(l,I+1) )
     &            *DTT
      BBB(l,I) =-AVK *   DPDWDZ * 1./ZDEPTH(l,2)*DTT
      CCC(l,I) = AVK * ( DPDWDZ * ( WWW(l,I)-WWW(l,I+1) ) + 1. +
     &           (I-1)*DPDWDZ*qqq(l,3)*1./ZDEPTH(l,3)*
     &            DTT )
         enddo
2001  CONTINUE
C
      do i = 1,len
         DENOM  = ( AAA(i,1)*AAA(i,2) - BBB(i,1)*BBB(i,2) )
         RDENOM = 0.
         IF ( ABS(DENOM) .LT. 1.E-6 ) RDENOM = 1.
         RDENOM = ( 1.-RDENOM)/( DENOM + RDENOM )
         QQQ(i,1)  = ( AAA(i,2)*CCC(i,1) - BBB(i,1)*CCC(i,2) ) * RDENOM
         QQQ(i,2)  = ( AAA(i,1)*CCC(i,2) - BBB(i,2)*CCC(i,1) ) * RDENOM

C-----------------------------------------------------------------------
C     UPDATE WETNESS OF EACH SOIL MOISTURE LAYER DUE TO LAYER INTERFLOW
C        AND BASE FLOW.
C-----------------------------------------------------------------------

         WWW(i,3) = WWW(i,3) - qqq(i,3)*DTT/ZDEPTH(i,3)
         ROFF(i) = ROFF(i) + qqq(i,3) * DTT
      enddo

      DO 3001 I = 1, 2
C
         do l = 1,len
            QMAX   =  WWW(l,I)   *
     &          (ZDEPTH(l,I)  /DTT) * 0.5
            QMIN   = -WWW(l,I+1) *
     &          (ZDEPTH(l,I+1)/DTT) * 0.5
            QQQ(l,I) = MIN( QQQ(l,I),QMAX)
            QQQ(l,I) = MAX( QQQ(l,I),QMIN)
            WWW(l,I)   =   WWW(l,I)   -
     &             QQQ(l,I)/ZDEPTH(l,I) *DTT
            WWW(l,I+1) =   WWW(l,I+1) +
     &             QQQ(l,I)/ZDEPTH(l,I+1)*DTT
         enddo
3001  CONTINUE
C
      do i = 1,len
         exo(i) = 0.0
      enddo
      DO 4001 I = 1, 3
         do l = 1,len
            EXCESS = MAX(0.0_dbl_kind,(WWW(l,I) - 1.))
            WWW(l,I) = WWW(l,I) - EXCESS
            exo(l) = exo(l) + EXCESS * ZDEPTH(l,I)
c
c Collatz-Bounoua put excess water into runoff according to
c original sib2 offline code .
c
            roff(l) = roff(l) + EXCESS *
     &            ZDEPTH(l,I)    ! lahouari
         enddo
      
4001  CONTINUE

c     prevent negative values
c     DO 4002 I = 1, 2
c     deficit = max(0.0_dbl_kind,(1.e-12-www(i)))
c     if(deficit.gt.0.0_dbl_kind)
c    *    print *,'deficit ',deficit,' at layer ',i,' grid ',ng
c     WWW(I) = WWW(I) + deficit
c     www(i+1)   = www(i+1)   - deficit * zdepth(i) / zdepth(i+1)
c002  CONTINUE
c     if(www(3).le.1.e-12)print *,'www(3) after 4002 ',ng,www(3)
c     www(3) = max(www(3),1.e-12)
C
c------------------END RUN2  -------------------------------------

      RETURN
      END
      SUBROUTINE VNTLAT
     *           (grav, tice,
     *            pr0, ribc, vkrmn, delta, dtt, tc, gt, ts, ps, zlt,
     *            www, tgs, etc, etg,
     *            rstfac, rsoil, hr, wc, wg, areas,
     *            sh, z0, spdm, sha, zb, ros,
     *            cu, ra, thvgm, rib, ustar, ventmf, thm, tha, z2, d,
     *            fc, fg, rbc, rdc,
     *            respcp, cogr, cogs, rb, rd, bps, rst, rc, ecmass,
     *            ea, em, hrr, assimn, bintc, ta, pco2m, po2m, vmax0,
     *            green, tran, ref, gmudmu, trop, trda, trdm, slti,
     *            shti, hltii, hhti, radn, effcon, binter, gradm,
     *            atheta, btheta, aparkk, wsfws, wsfht, wsflt, wci,
     *            whs, omepot, assimpot, assimci, antemp, assimnp,
     *            wags, wegs, aparc, pfd, assim, td, wopt, zm, wsat,
     *            soilscale, zmstscale, drst,
     *            soilq10, ansqr, 
     *            nsib, len, nsoil, forcerestore, dotkef,
     *            thgeff, shgeff, tke, ct, louis, zwind, ztemp, 
     *            respg, respfactor )
     
c
c
c	- optimized subroutine phosib
c	  dd 92.06.10
c
c

      use kinds

      IMPLICIT none

C    argument list declarations  
      integer (kind=int_kind), intent(in) ::
     *     nsib, len, nsoil
      logical (kind=log_kind), intent(in) ::
     *     forcerestore, dotkef, louis
      real (kind=dbl_kind), intent(in) ::
     *     grav, tice, pr0, pco2m, po2m
     *,    ribc, vkrmn, delta, dtt, zwind, ztemp
      real (kind=dbl_kind), intent(in), dimension(len) ::
     *     tc, gt, ts, ps, zlt
     *,    tgs, etc, etg
     *,    rsoil, hr
     *,    wc, wg, areas
     *,    sh
     *,    z0, spdm, zb, ros
     *,    thm, z2, d
     *,    rbc, rdc
     *,    respcp
     *,    bps, rst
     *,    em
     *,    vmax0, green
     *,    gmudmu, trop, trda
     *,    trdm, slti, shti, hltii
     *,    hhti, effcon, binter
     *,    gradm, atheta, btheta
     *,    aparc
     *,    wopt, zm, wsat
     *,    tke
      real (kind=dbl_kind), intent(in), dimension(len,3) ::
     *     www
      real (kind=dbl_kind), intent(in), dimension(nsib,2,2) ::
     *     tran
     *,    ref
      real (kind=dbl_kind), intent(in), dimension(len,2,2) ::
     *     radn
      real (kind=dbl_kind), intent(in), dimension(nsib,5) ::
     *     td
      real (kind=dbl_kind), intent(in), dimension(nsib,nsoil+1) ::
     *     respfactor

      real (kind=dbl_kind), intent(inout), dimension(len) ::
     *     ea
      real (kind=dbl_kind), intent(out), dimension(len) ::
     *     sha
     *,    cu, ra, thvgm, rib, ustar
     *,    ventmf, tha
     *,    fc, fg
     *,    cogr, cogs, rb, rd
     *,    rc, ecmass
     *,    hrr, assimn, bintc, ta
     *,    aparkk, wsfws, wsfht, wsflt
     *,    wci, whs, omepot, assimpot
     *,    assimci, antemp, assimnp, wags
     *,    wegs, pfd, assim
     *,    drst, ansqr
     *,    respg
     *,    thgeff, shgeff, ct
      real (kind=dbl_kind), intent(out), dimension(len,4) ::
     *     rstfac(len,4)
      real (kind=dbl_kind), intent(out), dimension(len,nsoil+1) ::
     *     soilscale
     *,    soilq10
      real (kind=dbl_kind), intent(out), dimension(len,2) ::
     *     zmstscale

c    local variables
                                                                    
      integer (kind=int_kind) :: i, itmax, itno 
      LOGICAL (kind=log_kind) ::  FRSTVM 
      real (kind=dbl_kind) ::
     *    gcon, zln2, ghalf, dmin, pdamp, qdamp, d2, dttin 
      real (kind=dbl_kind), dimension(len) ::
     *     u2, cog1, cog2, vsib
     *,    thsib, tsav, esav, coc, tprcor 
     *,    cni, cuni, ctni, ctni3, cti
     *,    cui , temv, cun, ctn
     *,    z1z0Urt, z1z0Trt, zzwind, zztemp                     
C                                                        
      DATA  ITMAX/4/                                     
C                                                                       
      GCON   = GRAV / 461.5                                          
C                                                                       
cczzggrst   calculate damping factors
      zln2 = 6.9314718 e -1
      ghalf = 1.0257068 e 1
      dttin = 3.6 e 3 
      dmin = 6.0 e 1
      pdamp = exp (-1.0 * zln2*(dtt*dmin)/(dttin*ghalf))
      qdamp = 1.0 - pdamp
c      pdamp = 0.0
c      qdamp = 1.0
cczzggrst                                                  
C     FOR OCEAN POINTS, THIS IS THE ONLY CALL TO VMFCAL.  FOR VEGETATED 
C     POINTS, ITS JUST THE FIRST CALL BEFORE AN ITERATION.             
C                                                                       
      FRSTVM = .TRUE.                                                   
cxx
      if(louis) then
          do i = 1,len
             zzwind(i) = z2(i)-d(i)+ zwind
             zztemp(i) = z2(i)-d(i)+ ztemp
          enddo
          CALL VMFCALZ(PR0,RIBC,VKRMN,DELTA,GRAV
     *,                SH,PS,z0           
     *,                SPDM,SHA,ROS,CU,ct,THVGM,RIB     
     *,                USTAR,VENTMF,THM, tha, zzwind, zztemp
     *,                cuni, cun, ctn, z1z0Urt, z1z0Trt, len ) 
      else
         CALL VMFCAL(PR0,RIBC,VKRMN,DELTA,GRAV
     *,              SH,PS,Z0,SPDM   
     *,              SHA,ZB,ROS,CU,THVGM,RIB,USTAR     
     *,              VENTMF,THM, tha, cni, cuni, ctni
     *,              ctni3, cti, cui, len  
     *,              thgeff, shgeff, tke, ct, dotkef ) 
      endif                                    
C                                                                       
C     AERODYNAMIC RESISTANCE                                            
C                                                                       
         DO I=1,len                                              
            RA(I)    = ROS(i) / VENTMF(i)                              
            TEMV(I) = (Z2(i) - D(i)) / z0(i)                      
            U2(I)     = SPDM(i) / (CUNI(i) * VKRMN)
	    temv(i) = log(temv(i))                                           
            U2(I) = U2(I) * TEMV(I)                                
         enddo                                                    
      FRSTVM = .FALSE.                                                  
C                                                                       
      DO 100 I=1,len                                               
         FC(I)=1.0                                                     
         FG(I)=1.0                                                     
  100 CONTINUE                                                       
C                                                                       
      CALL RBRD(tc, ts, rbc, zlt, gt, z2, u2
     ^,         rd, rb, ta, cogr, cogs
     ^,         cog1, cog2, grav, rdc, ra, tgs, wg
     ^,         rsoil, fg, hr, areas, respcp
     ^,         len  )
C                                                                       
      DO 50 I=1,len                                                
         VSIB(I)  = 1.0/RB(I) + 1.0/RD(I)                             
         THSIB(I) = (GT(i)/RD(I) + TC(i)/RB(I)) /
     *              (BPS(i)*VSIB(I)) 
   50 CONTINUE                                                       
C                                                                       
C                                                                       
C        ITERATE FOR AIR TEMPERATURE AND VENTILATION MASS FLUX          
C                                                                       
C        ITERATE A FIXED NUMBER OF TIMES FOR VEGETATED LAND             
C                                                                       
      DO 2000 ITNO=1,ITMAX                                           
         DO 120 I=1,len                                            
            TSAV(I)   = TA(i)                                          
            ESAV(I)   = EA(i)                                          
            SHA(i)    = 0.622 e 0 * EA(i) /
     *                     ( PS(i) - EA(i))              
  120    CONTINUE                                                    
czgrst
c  grst is known
c                        
         if(louis) then
            CALL VMFCALZ2(PR0,RIBC,VKRMN,DELTA,GRAV
     *,                   SH,PS         
     *,                   SPDM,SHA,ROS,CU,ct,RA,THVGM,RIB     
     *,                   USTAR,VENTMF,THM, tha
     *,                   cun, ctn, zzwind, zztemp
     *,                   z1z0Urt, z1z0Trt, len )                               
         else                                              
            CALL VMFCAL2(PR0,RIBC,VKRMN,DELTA,GRAV
     *,                  SH,PS,Z0,SPDM   
     *,                  SHA,ZB,ROS,CU,RA,THVGM,RIB,USTAR     
     *,                  VENTMF,THM, tha, cni, cuni, ctni
     *,                  ctni3, cti, cui, len
     *,                  thgeff, shgeff, tke, ct, dotkef )   
         endif                                 
czgrst
C   insert a block - start
cxx   check grsoil, grst, eliminate second vegetation layer 
c                                                                      
         DO 140 I=1,len                                            
            THA(i) = (VSIB(I) * THSIB(I) + VENTMF(i) *         
     *              THM(i)) / (VSIB(I) + VENTMF(i))                              
            TA(i) = THA(i) * BPS(i)                                                                                       
C
cczgrst  grst known (taken from fluxor.F)
c
            RC(I) = RST(i)      * FC(I) + 2.*RB(I)       
            COC(I) = (1.0  - WC(I))/RC(I)+WC(I) / (2.0 * RB(I))
cxx   taken from new sib      
            COGR(i) = (1.-WG(i))/(RSOIL(i)*FG(i)+RD(i))                                     
            COGS(i) =  WG(i)/RD(i)                                                    
            COG1(i) = (COGS(i) + COGR(i)*HR(i))
     ^           * (1.-AREAS(i)) + AREAS(i)/RD(i)                 
            COG2(i) = (COGS(i) + COGR(i)    ) 
     ^           * (1.-AREAS(i)) + AREAS(i)/RD(i)
cxx   end of new code                 
            D2  = 1.0 / RA(I) + COC(I) + COG2(I)                          
            EA(i) = (COC(I) * ETC(I) + COG1(I) * ETG(I)                    
     *           + EM(i)  / RA(I)) / D2
c
  140    CONTINUE                                                                      
 2000 CONTINUE
ccgggrst
c  insertion for grst -- end
c
cczzggrst calculate ecmass -- canopy evapotranspiration
c
       do i=1,len
          ecmass(i) = (etc(i) - ea(i)) * coc(i) *
     *                  ros (i) * 0.622 e 0 /ps(i) * dtt
       enddo 
cpl include here a call to respsib
cpl pass it 

      call respsib(len, nsib, nsoil, wopt, zm, www, wsat, 
     ^            tgs, td, forcerestore, respfactor, respg, soilscale,
     ^            zmstscale, soilq10)
cczzggrst
c
c     calculation of canopy conductance and photosynthesis
c        (implicit calculation of ea and ta)
c
      call phosib(pco2m,po2m,vmax0,tice,ps,green
     ^,          tran,ref,gmudmu,zlt,tc,trop,trda
     ^,          trdm,slti,shti,hltii,hhti
     ^,          radn,etc,etg,wc
     ^,          ea,em,rb,ra,ts
     ^,          effcon,rstfac,binter,gradm,assimn
     ^,          rst,atheta
     ^,          btheta,tgs,respcp,aparkk, len, nsib,
     >     omepot,assimpot,assimci,antemp,assimnp,
     >     wsfws,wsfht,wsflt,wci,whs,
     >     wags,wegs,aparc,pfd,assim, td,www,
     >     wopt,zm,wsat,gt,
     >     drst, pdamp,qdamp,ecmass,dtt,bintc,tprcor,ansqr,
     >     nsoil, forcerestore, respg )

C
cczzggrst block moved up (cxx-140-2000 loop)
c

      do i = 1,len
         bintc(i) = bintc(i) * tc(i) / ( 44.6 * tprcor(i) )
	 if(ea(i).gt.etc(i)) fc(i) = 0.0
	 if(ea(i).gt.etg(i)) fg(i) = 0.0
         HRR(I) = HR(I)                                                
         IF (FG(I) .LT. 0.5) HRR(I) = 1.0                             
         RC(I) = RST(i)      * FC(I) + RB(I) + RB(I) * FC(I)    
         COC(I) = (1.0 - WC(I)) / RC(I) + WC(I)
     *                   / (2.0   * RB(I))   
         SHA(i) = EA(i)*0.622/(PS(i)-EA(i))                         
C                                                                       
      enddo                                                       
C                                                                       
      RETURN                                                            
      END                                                               
